@@ -25,7 +25,6 @@
 # exception statement from all source files in the program, then also delete
 # it in the license file.
 #
-# pylint: disable=too-many-lines
 """
 IDL Parser.
 
@@ -91,7 +90,6 @@ def _generic_parser(
         syntax_node,  # type: Any
         mapping_rules  # type: Dict[str, _RuleDesc]
 ):  # type: (...) -> None
-    # pylint: disable=too-many-branches
     field_name_set = set()  # type: Set[str]
 
     for [first_node, second_node] in node.value:
@@ -134,6 +132,8 @@ def _generic_parser(
                 if ctxt.is_mapping_node(second_node, first_name):
                     syntax_node.__dict__[first_name] = rule_desc.mapping_parser_func(
                         ctxt, second_node)
+            elif rule_desc.node_type == "required_bool_scalar":
+                syntax_node.__dict__[first_name] = ctxt.get_required_bool(second_node)
             else:
                 raise errors.IDLError(
                     "Unknown node_type '%s' for parser rule" % (rule_desc.node_type))
@@ -149,7 +149,7 @@ def _generic_parser(
 
         # A bool is never "None" like other types, it simply defaults to "false".
         # It means "if bool is None" will always return false and there is no support for required
-        # 'bool' at this time.
+        # 'bool' at this time. Use the node type 'required_bool_scalar' if this behavior is not desired.
         if not rule_desc.node_type == 'bool_scalar':
             if syntax_node.__dict__[name] is None:
                 ctxt.add_missing_required_field_error(node, syntax_node_name, name)
@@ -254,6 +254,7 @@ def _parse_type(ctxt, spec, name, node):
             "serializer": _RuleDesc('scalar'),
             "deserializer": _RuleDesc('scalar'),
             "deserialize_with_tenant": _RuleDesc('bool_scalar'),
+            "internal_only": _RuleDesc('bool_scalar'),
             "default": _RuleDesc('scalar'),
         })
 
@@ -304,6 +305,8 @@ def _parse_condition(ctxt, node):
             "preprocessor": _RuleDesc("scalar"),
             "constexpr": _RuleDesc("scalar"),
             "expr": _RuleDesc("scalar"),
+            "feature_flag": _RuleDesc("scalar"),
+            "min_fcv": _RuleDesc("scalar"),
         })
 
     return condition
@@ -331,6 +334,23 @@ def _parse_field_type(ctxt, node):
         return variant
     else:
         assert node.id == "scalar"
+        if node.value.startswith('array<variant<'):
+            variant_types = syntax.parse_array_variant_types(node.value)
+            variant = syntax.FieldTypeVariant(ctxt.file_name, node.start_mark.line,
+                                              node.start_mark.column)
+            if variant_types is None:
+                location = common.SourceLocation(ctxt.file_name, node.start_mark.line,
+                                                 node.start_mark.column)
+                ctxt.add_bad_array_variant_types_error(location, node.value)
+            else:
+                for variant_type in variant_types:
+                    single = syntax.FieldTypeSingle(ctxt.file_name, node.start_mark.line,
+                                                    node.start_mark.column)
+                    single.type_name = variant_type
+                    variant.variant.append(single)
+
+            return syntax.FieldTypeArray(variant)
+
         single = syntax.FieldTypeSingle(ctxt.file_name, node.start_mark.line,
                                         node.start_mark.column)
 
@@ -382,6 +402,12 @@ def _parse_field(ctxt, name, node):
                 _RuleDesc("scalar"),
             "always_serialize":
                 _RuleDesc("bool_scalar"),
+            "forward_to_shards":
+                _RuleDesc('bool_scalar'),
+            "forward_from_shards":
+                _RuleDesc('bool_scalar'),
+            "query_shape":
+                _RuleDesc('scalar'),
         })
 
     return field
@@ -552,6 +578,9 @@ def _parse_struct(ctxt, spec, name, node):
             "non_const_getter": _RuleDesc('bool_scalar'),
             "cpp_validator_func": _RuleDesc('scalar'),
             "is_command_reply": _RuleDesc('bool_scalar'),
+            "is_generic_cmd_list": _RuleDesc('scalar'),
+            "unsafe_dangerous_disable_extra_field_duplicate_checks": _RuleDesc("bool_scalar"),
+            "query_shape_component": _RuleDesc('bool_scalar'),
         })
 
     # PyLint has difficulty with some iterables: https://github.com/PyCQA/pylint/issues/3105
@@ -562,6 +591,11 @@ def _parse_struct(ctxt, spec, name, node):
         return
 
     spec.symbols.add_struct(ctxt, struct)
+    if struct.is_generic_cmd_list:
+        if struct.is_generic_cmd_list == "arg":
+            spec.symbols.add_generic_argument_list(struct)
+        elif struct.is_generic_cmd_list == "reply":
+            spec.symbols.add_generic_reply_field_list(struct)
 
 
 def _parse_generic_argument_list(ctxt, spec, name, node):
@@ -570,8 +604,7 @@ def _parse_generic_argument_list(ctxt, spec, name, node):
     if not ctxt.is_mapping_node(node, "generic_argument_list"):
         return
 
-    field_list = syntax.GenericArgumentList(ctxt.file_name, node.start_mark.line,
-                                            node.start_mark.column)
+    field_list = syntax.Struct(ctxt.file_name, node.start_mark.line, node.start_mark.column)
     field_list.name = name
 
     _generic_parser(
@@ -584,7 +617,7 @@ def _parse_generic_argument_list(ctxt, spec, name, node):
                 _RuleDesc('mapping', mapping_parser_func=_parse_generic_argument_list_entries),
         })
 
-    spec.symbols.add_generic_argument_list(ctxt, field_list)
+    spec.symbols.add_generic_argument_list(field_list)
 
 
 def _parse_generic_reply_field_list(ctxt, spec, name, node):
@@ -593,8 +626,7 @@ def _parse_generic_reply_field_list(ctxt, spec, name, node):
     if not ctxt.is_mapping_node(node, "generic_reply_field_list"):
         return
 
-    field_list = syntax.GenericReplyFieldList(ctxt.file_name, node.start_mark.line,
-                                              node.start_mark.column)
+    field_list = syntax.Struct(ctxt.file_name, node.start_mark.line, node.start_mark.column)
     field_list.name = name
 
     _generic_parser(
@@ -607,7 +639,7 @@ def _parse_generic_reply_field_list(ctxt, spec, name, node):
                 _RuleDesc('mapping', mapping_parser_func=_parse_generic_reply_field_list_entries),
         })
 
-    spec.symbols.add_generic_reply_field_list(ctxt, field_list)
+    spec.symbols.add_generic_reply_field_list(field_list)
 
 
 def _parse_field_list_entry(ctxt, name, node, is_generic_argument_field_list):
@@ -810,8 +842,6 @@ def _parse_command(ctxt, spec, name, node):
     # type: (errors.ParserContext, syntax.IDLSpec, str, Union[yaml.nodes.MappingNode, yaml.nodes.ScalarNode, yaml.nodes.SequenceNode]) -> None
     """Parse a command section in the IDL file."""
 
-    # pylint: disable=too-many-branches
-
     if not ctxt.is_mapping_node(node, "command"):
         return
 
@@ -1034,7 +1064,6 @@ def parse_file(stream, error_file_name):
     stream: is a io.Stream.
     error_file_name: just a file name for error messages to use.
     """
-    # pylint: disable=too-many-branches
 
     # This will raise an exception if the YAML parse fails
     root_node = yaml.compose(stream)
@@ -1128,7 +1157,6 @@ def parse(stream, input_file_name, resolver):
     stream: is a io.Stream.
     input_file_name: a file name for error messages to use, and to help resolve imported files.
     """
-    # pylint: disable=too-many-locals
 
     root_doc = parse_file(stream, input_file_name)
 
