@@ -19,6 +19,8 @@ from buildscripts.resmokelib import utils
 from buildscripts.resmokelib.utils import globstar
 from buildscripts.resmokelib.utils import jscomment
 
+ENTERPRISE_TEST_DIR = os.path.normpath("src/mongo/db/modules/enterprise/jstests")
+
 ########################
 #  Test file explorer  #
 ########################
@@ -215,6 +217,13 @@ class _TestList(object):
                          format(path)))
                 self._filtered.discard(path)
 
+    def filter_enterprise_tests(self):
+        """Exclude tests that start with the enterprise module directory from the test list."""
+        self._filtered = {
+            test
+            for test in self._filtered if not os.path.normpath(test).startswith(ENTERPRISE_TEST_DIR)
+        }
+
     def match_tag_expression(self, tag_expression, get_tags):
         """Filter the test list to only include tests that match the tag expression.
 
@@ -339,10 +348,9 @@ def _make_expression_list(configs):
 class _SelectorConfig(object):
     """Base object to represent the configuration for test selection."""
 
-    def __init__(  # pylint: disable=too-many-arguments
-            self, root=None, roots=None, include_files=None, exclude_files=None, include_tags=None,
-            exclude_tags=None, include_with_any_tags=None, exclude_with_any_tags=None,
-            tag_file=None):
+    def __init__(self, root=None, roots=None, include_files=None, exclude_files=None,
+                 include_tags=None, exclude_tags=None, include_with_any_tags=None,
+                 exclude_with_any_tags=None, tag_file=None):
         """Initialize the _SelectorConfig from the configuration elements.
 
         Args:
@@ -439,8 +447,10 @@ class _Selector(object):
         # 1. Find the root files.
         if selector_config.roots is not None:
             roots = selector_config.roots
-        else:
+        elif selector_config.root is not None:
             roots = self._test_file_explorer.read_root_file(selector_config.root)
+        else:
+            roots = []
 
         # 2. Create a _TestList.
         test_list = _TestList(self._test_file_explorer, roots, self._tests_are_files)
@@ -453,6 +463,9 @@ class _Selector(object):
         # 5. Apply the include files last with force=True to take precedence over the tags.
         if self._tests_are_files and selector_config.include_files:
             test_list.include_files(selector_config.include_files, force=True)
+        # 6: Apply the enterprise tests filter
+        if self.get_enterprise_tests_status() == "off":
+            test_list.filter_enterprise_tests()
 
         return self.sort_tests(*test_list.get_tests())
 
@@ -468,13 +481,18 @@ class _Selector(object):
         """Retrieve the tags associated with the give test file."""
         return []
 
+    @staticmethod
+    def get_enterprise_tests_status() -> str:
+        """Get the status of enterprise tests from the configuration."""
+        return config.ENABLE_ENTERPRISE_TESTS
+
 
 class _JSTestSelectorConfig(_SelectorConfig):
     """_SelectorConfig subclass for JavaScript tests."""
 
-    def __init__(  # pylint: disable=too-many-arguments
-            self, roots=None, include_files=None, exclude_files=None, include_with_any_tags=None,
-            exclude_with_any_tags=None, include_tags=None, exclude_tags=None, tag_file=None):
+    def __init__(self, roots=None, include_files=None, exclude_files=None,
+                 include_with_any_tags=None, exclude_with_any_tags=None, include_tags=None,
+                 exclude_tags=None, tag_file=None):
         _SelectorConfig.__init__(
             self, roots=roots, include_files=include_files, exclude_files=exclude_files,
             include_with_any_tags=include_with_any_tags,
