@@ -1,9 +1,11 @@
+# pylint: disable=invalid-name
 """Extension to the unittest.TestResult.
 
 This is used to support additional test status and timing information for the report.json file.
 """
 
 import copy
+import os
 import threading
 import time
 import unittest
@@ -14,7 +16,7 @@ from buildscripts.resmokelib.testing.symbolizer_service import ResmokeSymbolizer
 
 
 # pylint: disable=attribute-defined-outside-init
-class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attributes
+class TestReport(unittest.TestResult):
     """Record test status and timing information."""
 
     def __init__(self, job_logger, suite_options, job_num=None):
@@ -103,12 +105,13 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
 
         return combined_report
 
-    def startTest(self, test):  # pylint: disable=invalid-name
+    def startTest(self, test):
         """Call before 'test' is run."""
 
         unittest.TestResult.startTest(self, test)
 
         test_info = TestInfo(test.id(), test.test_name, test.dynamic)
+        test_info.group_id = f"job{self.job_num}"
 
         basename = test.basename()
         command = test.as_command()
@@ -127,6 +130,15 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
                                                                       test.basename(), command,
                                                                       test.logger, self.job_num,
                                                                       test.id(), self.job_logger)
+
+        # Set up logging handlers to capture exceptions.
+        test_info.exception_extractors = logging.loggers.configure_exception_capture(test_logger)
+
+        test_info.log_info = {
+            "log_name": logging.loggers.get_evergreen_log_name(self.job_num, test.id()),
+            "logs_to_merge": [logging.loggers.get_evergreen_log_name(self.job_num)],
+            "rendering_type": "resmoke", "version": 0
+        }
         test_info.url_endpoint = url_endpoint
         if self.logging_prefix is not None:
             test_logger.info(self.logging_prefix)
@@ -136,42 +148,40 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         test.override_logger(test_logger)
         test_info.start_time = time.time()
 
-    def stopTest(self, test):  # pylint: disable=invalid-name
+    def stopTest(self, test):
         """Call after 'test' has run."""
 
-        # check if there are stacktrace files, if so, invoke the symbolizer here.
-        # If there are no stacktrace files for this job, we do not need to invoke the symbolizer at all.
-        # Take a lock to download the debug symbols if it hasn't already been downloaded.
-        # log symbolized output to test.logger.info()
+        try:
+            # check if there are stacktrace files, if so, invoke the symbolizer here.
+            # log symbolized output to test.logger.info()
+            symbolizer = ResmokeSymbolizer()
+            symbolizer.symbolize_test_logs(test)
 
-        symbolizer = ResmokeSymbolizer()
-        symbolizer.symbolize_test_logs(test)
-        # symbolization completed
+            unittest.TestResult.stopTest(self, test)
 
-        unittest.TestResult.stopTest(self, test)
+            with self._lock:
+                test_info = self.find_test_info(test)
+                test_info.end_time = time.time()
+                test_status = "no failures detected" if test_info.status == "pass" else "failed"
 
-        failed_test_status = "failed"
+            time_taken = test_info.end_time - test_info.start_time
+            self.job_logger.info("%s ran in %0.2f seconds: %s.", test.basename(), time_taken,
+                                 test_status)
 
-        with self._lock:
-            test_info = self.find_test_info(test)
-            test_info.end_time = time.time()
-            test_status = "no failures detected" if test_info.status == "pass" else failed_test_status
+        finally:
+            # This is a failsafe. In the event that 'stopTest' fails,
+            # any rogue logger handlers will be removed from this test.
+            # If not cleaned up, these will trigger 'setup failures' --
+            # indicated by exiting with LoggerRuntimeConfigError.EXIT_CODE.
+            for handler in test.logger.handlers:
+                # We ignore the cancellation token returned by close_later() since we always want the
+                # logs to eventually get flushed.
+                logging.flush.close_later(handler)
 
-        time_taken = test_info.end_time - test_info.start_time
-        self.job_logger.info("%s ran in %0.2f seconds: %s.", test.basename(), time_taken,
-                             test_status)
+            # Restore the original logger for the test.
+            test.reset_logger()
 
-        # Asynchronously closes the buildlogger test handler to avoid having too many threads open
-        # on 32-bit systems.
-        for handler in test.logger.handlers:
-            # We ignore the cancellation token returned by close_later() since we always want the
-            # logs to eventually get flushed.
-            logging.flush.close_later(handler, test_failed_flag=(test_status == failed_test_status))
-
-        # Restore the original logger for the test.
-        test.reset_logger()
-
-    def addError(self, test, err):  # pylint: disable=invalid-name
+    def addError(self, test, err):
         """Call when a non-failureException was raised during the execution of 'test'."""
 
         unittest.TestResult.addError(self, test, err)
@@ -184,8 +194,9 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
             test_info.status = "error"
             test_info.evergreen_status = "fail"
             test_info.return_code = test.return_code
+            test_info.error = self._exc_info_to_string(err, test).split('\n')
 
-    def setError(self, test):  # pylint: disable=invalid-name
+    def setError(self, test, err):
         """Change the outcome of an existing test to an error."""
         self.job_logger.info("setError(%s)", test)
 
@@ -198,6 +209,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
             test_info.status = "error"
             test_info.evergreen_status = "fail"
             test_info.return_code = 2
+            test_info.error = self._exc_info_to_string(err, test).split('\n')
 
         # Recompute number of success, failures, and errors.
         self.num_succeeded = len(self.get_successful())
@@ -205,7 +217,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         self.num_errored = len(self.get_errored())
         self.num_interrupted = len(self.get_interrupted())
 
-    def addFailure(self, test, err):  # pylint: disable=invalid-name
+    def addFailure(self, test, err):
         """Call when a failureException was raised during the execution of 'test'."""
 
         unittest.TestResult.addFailure(self, test, err)
@@ -215,15 +227,10 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
 
             test_info = self.find_test_info(test)
             test_info.status = "fail"
-            if test_info.dynamic:
-                # Dynamic tests are used for data consistency checks, so the failures are never
-                # silenced.
-                test_info.evergreen_status = "fail"
-            else:
-                test_info.evergreen_status = self.suite_options.report_failure_status
+            test_info.evergreen_status = "fail"
             test_info.return_code = test.return_code
 
-    def setFailure(self, test, return_code=1):  # pylint: disable=invalid-name
+    def setFailure(self, test, return_code=1):
         """Change the outcome of an existing test to a failure."""
 
         with self._lock:
@@ -232,12 +239,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
                 raise ValueError("stopTest was not called on %s" % (test.basename()))
 
             test_info.status = "fail"
-            if test_info.dynamic:
-                # Dynamic tests are used for data consistency checks, so the failures are never
-                # silenced.
-                test_info.evergreen_status = "fail"
-            else:
-                test_info.evergreen_status = self.suite_options.report_failure_status
+            test_info.evergreen_status = "fail"
             test_info.return_code = return_code
 
         # Recompute number of success, failures, and errors.
@@ -246,7 +248,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         self.num_errored = len(self.get_errored())
         self.num_interrupted = len(self.get_interrupted())
 
-    def addSuccess(self, test):  # pylint: disable=invalid-name
+    def addSuccess(self, test):
         """Call when 'test' executed successfully."""
 
         unittest.TestResult.addSuccess(self, test)
@@ -259,7 +261,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
             test_info.evergreen_status = "pass"
             test_info.return_code = test.return_code
 
-    def wasSuccessful(self):  # pylint: disable=invalid-name
+    def wasSuccessful(self):
         """Return true if all tests executed successfully."""
 
         with self._lock:
@@ -300,19 +302,18 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
             for test_info in self.test_infos:
                 result = {
                     "test_file": test_info.test_file,
+                    "group_id": test_info.group_id,
                     "status": test_info.evergreen_status,
                     "exit_code": test_info.return_code,
                     "start": test_info.start_time,
                     "end": test_info.end_time,
                     "elapsed": test_info.end_time - test_info.start_time,
+                    "log_info": test_info.log_info,
                 }
 
                 if test_info.display_test_name is not None:
                     result["display_test_name"] = test_info.display_test_name
 
-                if test_info.group_id is not None:
-                    result["group_id"] = test_info.group_id
-
                 if test_info.url_endpoint is not None:
                     result["url"] = test_info.url_endpoint
                     result["url_raw"] = test_info.url_endpoint + "?raw=1"
@@ -342,6 +343,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
             test_info = TestInfo(test_file, test_file, is_dynamic)
             test_info.display_test_name = result.get("display_test_name")
             test_info.group_id = result.get("group_id")
+            test_info.log_info = result.get("log_info")
             test_info.url_endpoint = result.get("url")
             test_info.status = result["status"]
             test_info.evergreen_status = test_info.status
@@ -387,7 +389,7 @@ class TestReport(unittest.TestResult):  # pylint: disable=too-many-instance-attr
         raise ValueError("Details for %s not found in the report" % (test.basename()))
 
 
-class TestInfo(object):  # pylint: disable=too-many-instance-attributes
+class TestInfo(object):
     """Holder for the test status and timing information."""
 
     def __init__(self, test_id, test_file, dynamic):
@@ -395,16 +397,19 @@ class TestInfo(object):  # pylint: disable=too-many-instance-attributes
 
         self.test_id = test_id
         self.test_file = test_file
+        self.group_id = None
         self.display_test_name = None
         self.dynamic = dynamic
 
-        self.group_id = None
         self.start_time = None
         self.end_time = None
         self.status = None
         self.evergreen_status = None
         self.return_code = None
+        self.log_info = None
         self.url_endpoint = None
+        self.exception_extractors = []
+        self.error = None
 
 
 def test_order(test_name):
