@@ -4,35 +4,29 @@ from __future__ import absolute_import
 
 import collections
 import datetime
-import json
+from io import StringIO
 import os
 import sys
 import subprocess
 import unittest
 
-from math import ceil
 from mock import Mock, patch, MagicMock
-
-import requests
-
-from shrub.v2 import ShrubProject, BuildVariant
+import yaml
 
 import buildscripts.burn_in_tests as under_test
-from buildscripts.ciconfig.evergreen import parse_evergreen_file
-import buildscripts.util.teststats as teststats_utils
+from buildscripts.ciconfig.evergreen import parse_evergreen_file, VariantTask
 import buildscripts.resmokelib.parser as _parser
-import buildscripts.resmokelib.config as _config
 _parser.set_run_options()
 
-# pylint: disable=missing-docstring,protected-access,too-many-lines,no-self-use
+# pylint: disable=protected-access
 
 
 def create_tests_by_task_mock(n_tasks, n_tests):
     return {
-        f"task_{i}_gen": {
-            "display_task_name": f"task_{i}", "resmoke_args": f"--suites=suite_{i}",
-            "tests": [f"jstests/tests_{j}" for j in range(n_tests)]
-        }
+        f"task_{i}_gen": under_test.TaskInfo(display_task_name=f"task_{i}", resmoke_args="", tests=[
+            f"jstests/tests_{j}" for j in range(n_tests)
+        ], require_multiversion_setup=False, distro=f"distro_{i}", suite=f"suite_{i}",
+                                             build_variant="dummy_variant")
         for i in range(n_tasks)
     }
 
@@ -75,75 +69,6 @@ def get_evergreen_config(config_file_path):
     return parse_evergreen_file(config_file_path)
 
 
-class TestAcceptance(unittest.TestCase):
-    def tearDown(self):
-        _parser.set_run_options()
-
-    @patch(ns("write_file"))
-    def test_no_tests_run_if_none_changed(self, write_json_mock):
-        """
-        Given a git repository with no changes,
-        When burn_in_tests is run,
-        Then no tests are discovered to run.
-        """
-        variant = "build_variant"
-        repos = [mock_changed_git_files([])]
-        repeat_config = under_test.RepeatConfig()
-        gen_config = under_test.GenerateConfig(
-            variant,
-            "project",
-        )  # yapf: disable
-
-        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, MagicMock(),
-                           repos, None, None, 'install_dir/bin')
-
-        write_json_mock.assert_called_once()
-        written_config = json.loads(write_json_mock.call_args[0][1])
-        display_task = written_config["buildvariants"][0]["display_tasks"][0]
-        self.assertEqual(1, len(display_task["execution_tasks"]))
-        self.assertEqual(under_test.BURN_IN_TESTS_GEN_TASK, display_task["execution_tasks"][0])
-
-    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    @patch(ns("write_file"))
-    def test_tests_generated_if_a_file_changed(self, write_json_mock):
-        """
-        Given a git repository with changes,
-        When burn_in_tests is run,
-        Then tests are discovered to run.
-        """
-        # Note: this test is using actual tests and suites. So changes to those suites could
-        # introduce failures and require this test to be updated.
-        # You can see the test file it is using below. This test is used in the 'auth' and
-        # 'auth_audit' test suites. It needs to be in at least one of those for the test to pass.
-        _config.NAMED_SUITES = None
-        variant = "enterprise-rhel-62-64-bit"
-        repos = [mock_changed_git_files(["jstests/auth/auth1.js"])]
-        repeat_config = under_test.RepeatConfig()
-        gen_config = under_test.GenerateConfig(
-            variant,
-            "project",
-        )  # yapf: disable
-        evg_config = get_evergreen_config("etc/evergreen.yml")
-
-        under_test.burn_in(repeat_config, gen_config, "", "testfile.json", False, evg_config, repos,
-                           None, None, 'install_dir/bin')
-
-        write_json_mock.assert_called_once()
-        written_config = json.loads(write_json_mock.call_args[0][1])
-        n_tasks = len(written_config["tasks"])
-        # Ensure we are generating at least one task for the test.
-        self.assertGreaterEqual(n_tasks, 1)
-
-        written_build_variant = written_config["buildvariants"][0]
-        self.assertEqual(variant, written_build_variant["name"])
-        self.assertEqual(n_tasks, len(written_build_variant["tasks"]))
-
-        display_task = written_build_variant["display_tasks"][0]
-        # The display task should contain all the generated tasks as well as 1 extra task for
-        # the burn_in_test_gen task.
-        self.assertEqual(n_tasks + 1, len(display_task["execution_tasks"]))
-
-
 class TestRepeatConfig(unittest.TestCase):
     def test_validate_no_args(self):
         repeat_config = under_test.RepeatConfig()
@@ -228,139 +153,6 @@ class TestRepeatConfig(unittest.TestCase):
         self.assertEqual(repeat_options.strip(), "--repeatSuites=2")
 
 
-class TestGenerateConfig(unittest.TestCase):
-    def test_run_build_variant_with_no_run_build_variant(self):
-        gen_config = under_test.GenerateConfig("build_variant", "project")
-
-        self.assertEqual(gen_config.build_variant, gen_config.run_build_variant)
-
-    def test_run_build_variant_with_run_build_variant(self):
-        gen_config = under_test.GenerateConfig("build_variant", "project", "run_build_variant")
-
-        self.assertNotEqual(gen_config.build_variant, gen_config.run_build_variant)
-        self.assertEqual(gen_config.run_build_variant, "run_build_variant")
-
-    def test_validate_non_existing_build_variant(self):
-        evg_conf_mock = MagicMock()
-        evg_conf_mock.get_variant.return_value = None
-
-        gen_config = under_test.GenerateConfig("build_variant", "project", "run_build_variant")
-
-        with self.assertRaises(ValueError):
-            gen_config.validate(evg_conf_mock)
-
-    def test_validate_existing_build_variant(self):
-        evg_conf_mock = MagicMock()
-
-        gen_config = under_test.GenerateConfig("build_variant", "project", "run_build_variant")
-        gen_config.validate(evg_conf_mock)
-
-    def test_validate_non_existing_run_build_variant(self):
-        evg_conf_mock = MagicMock()
-
-        gen_config = under_test.GenerateConfig("build_variant", "project")
-        gen_config.validate(evg_conf_mock)
-
-
-class TestParseAvgTestRuntime(unittest.TestCase):
-    def test__parse_avg_test_runtime(self):
-        task_avg_test_runtime_stats = [
-            teststats_utils.TestRuntime(test_name="dir/test1.js", runtime=30.2),
-            teststats_utils.TestRuntime(test_name="dir/test2.js", runtime=455.1)
-        ]
-        result = under_test._parse_avg_test_runtime("dir/test2.js", task_avg_test_runtime_stats)
-        self.assertEqual(result, 455.1)
-
-
-class TestCalculateTimeout(unittest.TestCase):
-    def test__calculate_timeout(self):
-        avg_test_runtime = 455.1
-        expected_result = ceil(avg_test_runtime * under_test.AVG_TEST_TIME_MULTIPLIER)
-        self.assertEqual(expected_result, under_test._calculate_timeout(avg_test_runtime))
-
-    def test__calculate_timeout_avg_is_less_than_min(self):
-        avg_test_runtime = 10
-        self.assertEqual(under_test.MIN_AVG_TEST_TIME_SEC,
-                         under_test._calculate_timeout(avg_test_runtime))
-
-
-class TestCalculateExecTimeout(unittest.TestCase):
-    def test__calculate_exec_timeout(self):
-        repeat_config = under_test.RepeatConfig(repeat_tests_secs=600)
-        avg_test_runtime = 455.1
-
-        exec_timeout = under_test._calculate_exec_timeout(repeat_config, avg_test_runtime)
-
-        self.assertEqual(1771, exec_timeout)
-
-    def test_average_timeout_greater_than_execution_time(self):
-        repeat_config = under_test.RepeatConfig(repeat_tests_secs=600, repeat_tests_min=2)
-        avg_test_runtime = 750
-
-        exec_timeout = under_test._calculate_exec_timeout(repeat_config, avg_test_runtime)
-
-        # The timeout needs to be greater than the number of the test * the minimum number of runs.
-        minimum_expected_timeout = avg_test_runtime * repeat_config.repeat_tests_min
-
-        self.assertGreater(exec_timeout, minimum_expected_timeout)
-
-
-class TestGenerateTimeouts(unittest.TestCase):
-    def test__generate_timeouts(self):
-        repeat_config = under_test.RepeatConfig(repeat_tests_secs=600)
-        runtime_stats = [teststats_utils.TestRuntime(test_name="dir/test2.js", runtime=455.1)]
-        test_name = "dir/test2.js"
-
-        timeout_info = under_test._generate_timeouts(repeat_config, test_name, runtime_stats)
-
-        self.assertEqual(timeout_info.exec_timeout, 1771)
-        self.assertEqual(timeout_info.timeout, 1366)
-
-    def test__generate_timeouts_no_results(self):
-        repeat_config = under_test.RepeatConfig(repeat_tests_secs=600)
-        runtime_stats = []
-        test_name = "dir/new_test.js"
-
-        timeout_info = under_test._generate_timeouts(repeat_config, test_name, runtime_stats)
-
-        self.assertIsNone(timeout_info.cmd)
-
-    def test__generate_timeouts_avg_runtime_is_zero(self):
-        repeat_config = under_test.RepeatConfig(repeat_tests_secs=600)
-        runtime_stats = [
-            teststats_utils.TestRuntime(test_name="dir/test_with_zero_runtime.js", runtime=0)
-        ]
-        test_name = "dir/test_with_zero_runtime.js"
-
-        timeout_info = under_test._generate_timeouts(repeat_config, test_name, runtime_stats)
-
-        self.assertIsNone(timeout_info.cmd)
-
-
-class TestGetTaskRuntimeHistory(unittest.TestCase):
-    @patch(ns("HistoricTaskData.get_stats_from_s3"))
-    def test__get_task_runtime_history(self, get_stats_from_s3_mock):
-        test_stats = [
-            teststats_utils.HistoricalTestInformation(
-                test_name="dir/test2.js",
-                num_pass=1,
-                num_fail=0,
-                avg_duration_pass=10.1,
-            )
-        ]
-        get_stats_from_s3_mock.return_value = test_stats
-
-        result = under_test._get_task_runtime_history("project1", "task1", "variant1")
-        self.assertEqual(result, [("dir/test2.js", 10.1)])
-
-    @patch(ns("HistoricTaskData.get_stats_from_s3"))
-    def test__get_task_runtime_history_when_s3_has_no_data(self, get_stats_from_s3_mock):  # pylint: disable=invalid-name
-        get_stats_from_s3_mock.return_value = []
-
-        result = under_test._get_task_runtime_history("project1", "task1", "variant1")
-        self.assertEqual(result, [])
-
-
 class TestGetTaskName(unittest.TestCase):
     def test__get_task_name(self):
         name = "mytask"
@@ -375,36 +167,6 @@ class TestGetTaskName(unittest.TestCase):
         self.assertEqual(task_name, under_test._get_task_name(task))
 
 
-class TestSetResmokeArgs(unittest.TestCase):
-    def test__set_resmoke_args(self):
-        resmoke_args = "--suites=suite1 test1.js"
-        task = Mock()
-        task.combined_resmoke_args = resmoke_args
-        task.is_generate_resmoke_task = False
-        self.assertEqual(resmoke_args, under_test._set_resmoke_args(task))
-
-    def test__set_resmoke_args_gen_resmoke_task(self):
-        resmoke_args = "--suites=suite1 test1.js"
-        new_suite = "suite2"
-        new_resmoke_args = "--suites={} test1.js".format(new_suite)
-        task = Mock()
-        task.combined_resmoke_args = resmoke_args
-        task.is_generate_resmoke_task = True
-        task.get_vars_suite_name = lambda cmd_vars: cmd_vars["suite"]
-        task.generate_resmoke_tasks_command = {"vars": {"suite": new_suite}}
-        self.assertEqual(new_resmoke_args, under_test._set_resmoke_args(task))
-
-    def test__set_resmoke_args_gen_resmoke_task_no_suite(self):
-        suite = "suite1"
-        resmoke_args = "--suites={} test1.js".format(suite)
-        task = Mock()
-        task.combined_resmoke_args = resmoke_args
-        task.is_generate_resmoke_task = True
-        task.get_vars_suite_name = lambda cmd_vars: cmd_vars["task"]
-        task.generate_resmoke_tasks_command = {"vars": {"task": suite}}
-        self.assertEqual(resmoke_args, under_test._set_resmoke_args(task))
-
-
 class TestSetResmokeCmd(unittest.TestCase):
     def test__set_resmoke_cmd_no_opts_no_args(self):
         repeat_config = under_test.RepeatConfig()
@@ -418,102 +180,20 @@ class TestSetResmokeCmd(unittest.TestCase):
         resmoke_args = ["arg1", "arg2"]
 
         resmoke_cmd = under_test._set_resmoke_cmd(repeat_config, resmoke_args)
+        expected_resmoke_cmd = [sys.executable, 'buildscripts/resmoke.py', 'run'
+                                ] + resmoke_args + ['--repeatSuites=2']
 
-        self.assertListEqual(resmoke_args + ['--repeatSuites=2'], resmoke_cmd)
+        self.assertListEqual(expected_resmoke_cmd, resmoke_cmd)
 
     def test__set_resmoke_cmd(self):
         repeat_config = under_test.RepeatConfig(repeat_tests_num=3)
         resmoke_args = ["arg1", "arg2"]
 
         resmoke_cmd = under_test._set_resmoke_cmd(repeat_config, resmoke_args)
+        expected_resmoke_cmd = [sys.executable, 'buildscripts/resmoke.py', 'run'
+                                ] + resmoke_args + ['--repeatSuites=3']
 
-        self.assertListEqual(resmoke_args + ['--repeatSuites=3'], resmoke_cmd)
-
-
-TESTS_BY_TASK = {
-    "task1": {
-        "resmoke_args": "--suites=suite1",
-        "tests": ["jstests/test1.js", "jstests/test2.js"]},
-    "task2": {
-        "resmoke_args": "--suites=suite1",
-        "tests": ["jstests/test1.js", "jstests/test3.js"]},
-    "task3": {
-        "resmoke_args": "--suites=suite3",
-        "tests": ["jstests/test4.js", "jstests/test5.js"]},
-    "task4": {
-        "resmoke_args": "--suites=suite4", "tests": []},
-}  # yapf: disable
-
-
-class TestCreateGenerateTasksConfig(unittest.TestCase):
-    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    def test_no_tasks_given(self):
-        build_variant = BuildVariant("build variant")
-        gen_config = MagicMock(run_build_variant="variant")
-        repeat_config = MagicMock()
-
-        under_test.create_generate_tasks_config(build_variant, {}, gen_config, repeat_config, None)
-
-        evg_config_dict = build_variant.as_dict()
-        self.assertEqual(0, len(evg_config_dict["tasks"]))
-
-    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    def test_one_task_one_test(self):
-        n_tasks = 1
-        n_tests = 1
-        resmoke_options = "options for resmoke"
-        build_variant = BuildVariant("build variant")
-        gen_config = MagicMock(run_build_variant="variant", distro=None)
-        repeat_config = MagicMock()
-        repeat_config.generate_resmoke_options.return_value = resmoke_options
-        tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
-
-        under_test.create_generate_tasks_config(build_variant, tests_by_task, gen_config,
-                                                repeat_config, None)
-
-        shrub_config = ShrubProject.empty().add_build_variant(build_variant)
-        evg_config_dict = shrub_config.as_dict()
-        tasks = evg_config_dict["tasks"]
-        self.assertEqual(n_tasks * n_tests, len(tasks))
-        cmd = tasks[0]["commands"]
-        self.assertIn(resmoke_options, cmd[1]["vars"]["resmoke_args"])
-        self.assertIn("--suites=suite_0", cmd[1]["vars"]["resmoke_args"])
-        self.assertIn("tests_0", cmd[1]["vars"]["resmoke_args"])
-
-    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    def test_n_task_m_test(self):
-        n_tasks = 3
-        n_tests = 5
-        build_variant = BuildVariant("build variant")
-        gen_config = MagicMock(run_build_variant="variant", distro=None)
-        repeat_config = MagicMock()
-        tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
-
-        under_test.create_generate_tasks_config(build_variant, tests_by_task, gen_config,
-                                                repeat_config, None)
-
-        evg_config_dict = build_variant.as_dict()
-        self.assertEqual(n_tasks * n_tests, len(evg_config_dict["tasks"]))
-
-
-class TestCreateGenerateTasksFile(unittest.TestCase):
-    @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    @patch(ns("sys.exit"))
-    @patch(ns("create_generate_tasks_config"))
-    @patch(ns("validate_task_generation_limit"))
-    def test_cap_on_task_generate(self, validate_mock, _, exit_mock):
-        evg_api = MagicMock()
-        gen_config = MagicMock(use_multiversion=False)
-        repeat_config = MagicMock()
-        tests_by_task = MagicMock()
-
-        validate_mock.return_value = False
-
-        exit_mock.side_effect = ValueError("exiting")
-        with self.assertRaises(ValueError):
-            under_test.create_generate_tasks_file(tests_by_task, gen_config, repeat_config, evg_api)
-
-        exit_mock.assert_called_once()
+        self.assertListEqual(expected_resmoke_cmd, resmoke_cmd)
 
 
 class RunTests(unittest.TestCase):
@@ -607,22 +287,21 @@ class CreateExecutorList(unittest.TestCase):
         mock_get_named_suites.return_value = ["dbtest"]
 
         under_test.create_executor_list([], [])
-        self.assertEqual(mock_suite_class.call_count, 0)
+        self.assertEqual(mock_suite_class.call_count, 1)
 
 
 def create_variant_task_mock(task_name, suite_name, distro="distro"):
     variant_task = MagicMock()
     variant_task.name = task_name
     variant_task.generated_task_name = task_name
-    variant_task.resmoke_suite = suite_name
-    variant_task.get_vars_suite_name.return_value = suite_name
-    variant_task.combined_resmoke_args = f"--suites={suite_name}"
-    variant_task.multiversion_path = None
+    variant_task.get_suite_name.return_value = suite_name
+    variant_task.resmoke_args = f"--suites={suite_name}"
+    variant_task.require_multiversion_setup.return_value = False
     variant_task.run_on = [distro]
     return variant_task
 
 
-class TestGatherTaskInfo(unittest.TestCase):
+class TestTaskInfo(unittest.TestCase):
     def test_non_generated_task(self):
         suite_name = "suite_1"
         distro_name = "distro_1"
@@ -637,13 +316,13 @@ class TestGatherTaskInfo(unittest.TestCase):
             "suite 3": [f"test{i}.js" for i in range(2)],
         }
 
-        task_info = under_test._gather_task_info(task_mock, tests_by_suite, evg_conf_mock, variant)
+        task_info = under_test.TaskInfo.from_task(task_mock, tests_by_suite, evg_conf_mock, variant)
 
-        self.assertIn(suite_name, task_info["resmoke_args"])
+        self.assertIn(suite_name, task_info.resmoke_args)
         for test in test_list:
-            self.assertIn(test, task_info["tests"])
-        self.assertIsNone(task_info["use_multiversion"])
-        self.assertEqual(distro_name, task_info["distro"])
+            self.assertIn(test, task_info.tests)
+        self.assertFalse(task_info.require_multiversion_setup)
+        self.assertEqual(distro_name, task_info.distro)
 
     def test_generated_task_no_large_on_task(self):
         suite_name = "suite_1"
@@ -661,13 +340,13 @@ class TestGatherTaskInfo(unittest.TestCase):
             "suite 3": [f"test{i}.js" for i in range(2)],
         }
 
-        task_info = under_test._gather_task_info(task_mock, tests_by_suite, evg_conf_mock, variant)
+        task_info = under_test.TaskInfo.from_task(task_mock, tests_by_suite, evg_conf_mock, variant)
 
-        self.assertIn(suite_name, task_info["resmoke_args"])
+        self.assertIn(suite_name, task_info.resmoke_args)
         for test in test_list:
-            self.assertIn(test, task_info["tests"])
-        self.assertIsNone(task_info["use_multiversion"])
-        self.assertEqual(distro_name, task_info["distro"])
+            self.assertIn(test, task_info.tests)
+        self.assertFalse(task_info.require_multiversion_setup)
+        self.assertEqual(distro_name, task_info.distro)
 
     def test_generated_task_no_large_on_build_variant(self):
         suite_name = "suite_1"
@@ -685,13 +364,13 @@ class TestGatherTaskInfo(unittest.TestCase):
             "suite 3": [f"test{i}.js" for i in range(2)],
         }
 
-        task_info = under_test._gather_task_info(task_mock, tests_by_suite, evg_conf_mock, variant)
+        task_info = under_test.TaskInfo.from_task(task_mock, tests_by_suite, evg_conf_mock, variant)
 
-        self.assertIn(suite_name, task_info["resmoke_args"])
+        self.assertIn(suite_name, task_info.resmoke_args)
         for test in test_list:
-            self.assertIn(test, task_info["tests"])
-        self.assertIsNone(task_info["use_multiversion"])
-        self.assertEqual(distro_name, task_info["distro"])
+            self.assertIn(test, task_info.tests)
+        self.assertFalse(task_info.require_multiversion_setup)
+        self.assertEqual(distro_name, task_info.distro)
 
     def test_generated_task_large_distro(self):
         suite_name = "suite_1"
@@ -715,13 +394,13 @@ class TestGatherTaskInfo(unittest.TestCase):
             "suite 3": [f"test{i}.js" for i in range(2)],
         }
 
-        task_info = under_test._gather_task_info(task_mock, tests_by_suite, evg_conf_mock, variant)
+        task_info = under_test.TaskInfo.from_task(task_mock, tests_by_suite, evg_conf_mock, variant)
 
-        self.assertIn(suite_name, task_info["resmoke_args"])
+        self.assertIn(suite_name, task_info.resmoke_args)
         for test in test_list:
-            self.assertIn(test, task_info["tests"])
-        self.assertIsNone(task_info["use_multiversion"])
-        self.assertEqual(large_distro_name, task_info["distro"])
+            self.assertIn(test, task_info.tests)
+        self.assertFalse(task_info.require_multiversion_setup)
+        self.assertEqual(large_distro_name, task_info.distro)
 
 
 class TestCreateTaskList(unittest.TestCase):
@@ -765,11 +444,11 @@ class TestCreateTaskList(unittest.TestCase):
 
         self.assertIn("task 1", task_list)
         task_info = task_list["task 1"]
-        self.assertIn("suite_1", task_info["resmoke_args"])
+        self.assertIn("suite_1", task_info.resmoke_args)
         for i in range(3):
-            self.assertIn(f"test{i}.js", task_info["tests"])
-        self.assertIsNone(task_info["use_multiversion"])
-        self.assertEqual("distro 1", task_info["distro"])
+            self.assertIn(f"test{i}.js", task_info.tests)
+        self.assertFalse(task_info.require_multiversion_setup)
+        self.assertEqual("distro 1", task_info.distro)
 
     def test_create_task_list_with_excludes(self):
         variant = "variant name"
@@ -813,7 +492,17 @@ class TestCreateTaskList(unittest.TestCase):
             under_test.create_task_list(evg_conf_mock, variant, suite_dict, [])
 
 
-class TestFindChangedTests(unittest.TestCase):
+class TestCreateTestsByTask(unittest.TestCase):
+    def test_build_variant_not_in_evg_project_config(self):
+        variant = "novariant"
+        evg_conf_mock = MagicMock()
+        evg_conf_mock.get_variant.return_value = None
+
+        with self.assertRaises(ValueError):
+            under_test.create_tests_by_task(variant, evg_conf_mock, set(), "install-dir/bin")
+
+
+class TestLocalFileChangeDetector(unittest.TestCase):
     @patch(ns("find_changed_files_in_repos"))
     @patch(ns("os.path.isfile"))
     def test_non_js_files_filtered(self, is_file_mock, changed_files_mock):
@@ -826,7 +515,8 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = True
 
-        found_tests = under_test.find_changed_tests([repo_mock])
+        file_change_detector = under_test.LocalFileChangeDetector(None)
+        found_tests = file_change_detector.find_changed_tests([repo_mock])
 
         self.assertIn(file_list[0], found_tests)
         self.assertIn(file_list[2], found_tests)
@@ -844,7 +534,8 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = False
 
-        found_tests = under_test.find_changed_tests([repo_mock])
+        file_change_detector = under_test.LocalFileChangeDetector(None)
+        found_tests = file_change_detector.find_changed_tests([repo_mock])
 
         self.assertEqual(0, len(found_tests))
 
@@ -860,9 +551,26 @@ class TestFindChangedTests(unittest.TestCase):
         changed_files_mock.return_value = set(file_list)
         is_file_mock.return_value = True
 
-        found_tests = under_test.find_changed_tests([repo_mock])
+        file_change_detector = under_test.LocalFileChangeDetector(None)
+        found_tests = file_change_detector.find_changed_tests([repo_mock])
 
         self.assertIn(file_list[0], found_tests)
         self.assertIn(file_list[2], found_tests)
         self.assertNotIn(file_list[1], found_tests)
         self.assertEqual(2, len(found_tests))
+
+
+class TestYamlBurnInExecutor(unittest.TestCase):
+    @patch('sys.stdout', new_callable=StringIO)
+    def test_found_tasks_should_be_reported_as_yaml(self, stdout):
+        n_tasks = 5
+        n_tests = 3
+        tests_by_task = create_tests_by_task_mock(n_tasks, n_tests)
+
+        yaml_executor = under_test.YamlBurnInExecutor()
+        yaml_executor.execute(tests_by_task)
+
+        yaml_raw = stdout.getvalue()
+        results = yaml.safe_load(yaml_raw)
+        self.assertEqual(n_tasks, len(results["discovered_tasks"]))
+        self.assertEqual(n_tests, len(results["discovered_tasks"][0]["test_list"]))
