@@ -11,7 +11,7 @@
  * @tags: [
  *  requires_fcv_60,
  *  uses_transactions,
- *  assumes_unsharded_collection
+ *  assumes_unsharded_collection,
  * ]
  */
 load('jstests/concurrency/fsm_libs/extend_workload.js');
@@ -197,8 +197,11 @@ var $config = extendWorkload($config, function($config, $super) {
     $config.data.isAcceptableAggregateCmdError = function isAcceptableAggregateCmdError(res) {
         // The aggregate command is expected to involve running getMore commands which are not
         // retryable after network errors.
-        return TestData.runningWithShardStepdowns && res &&
-            (res.code == ErrorCodes.QueryPlanKilled);
+        // The linearizable read has a 15s timeout, then a LinearizableReadConcernError will be
+        // thrown, so we retry on this error in test
+        return res &&
+            (res.code == ErrorCodes.LinearizableReadConcernError ||
+             (TestData.runningWithShardStepdowns && res.code == ErrorCodes.QueryPlanKilled));
     };
 
     $config.data.getRandomDocument = function getRandomDocument(db, collName) {
