@@ -2,11 +2,15 @@
  * Tests index usage on meta and time fields for timeseries collections.
  *
  * @tags: [
- *   does_not_support_stepdowns,
- *   does_not_support_transactions,
- *   requires_pipeline_optimization,
  *   # Explain of a resolved view must be executed by mongos.
  *   directly_against_shardsvrs_incompatible,
+ *   # Refusing to run a test that issues an aggregation command with explain because it may return
+ *   # incomplete results if interrupted by a stepdown.
+ *   does_not_support_stepdowns,
+ *   # Tests that optimization produces expected query plans.
+ *   requires_pipeline_optimization,
+ *   # We need a timeseries collection.
+ *   requires_timeseries,
  * ]
  */
 (function() {
@@ -54,8 +58,9 @@ const generateTest = (useHint) => {
 
         /**
          * Creates the index specified by the spec and options, then explains the query to ensure
-         * that the created index is used. Runs the query and verifies that the expected number of
-         * documents are matched. Finally, deletes the created index.
+         * that the created index is used or was considered by multi-planner.
+         * Runs the query and verifies that the expected number of documents are matched.
+         * Finally, deletes the created index.
          */
         const testQueryUsesIndex = function(
             filter, numMatches, indexSpec, indexOpts = {}, queryOpts = {}) {
@@ -71,9 +76,23 @@ const generateTest = (useHint) => {
             assert.eq(numMatches, query.itcount());
 
             const explain = query.explain();
-            const ixscan = getAggPlanStage(explain, "IXSCAN");
-            assert.neq(null, ixscan, tojson(explain));
-            assert.eq("testIndexName", ixscan.indexName, tojson(ixscan));
+            if (useHint) {
+                const ixscan = getAggPlanStage(explain, "IXSCAN");
+                assert.neq(null, ixscan, tojson(explain));
+                assert.eq("testIndexName", ixscan.indexName, tojson(ixscan));
+            } else {
+                let ixscan = getAggPlanStage(explain, "IXSCAN");
+                // If ixscan is not present, check rejected plans
+                if (ixscan === null) {
+                    const rejectedPlans =
+                        getRejectedPlans(getAggPlanStage(explain, "$cursor")["$cursor"]);
+                    assert.eq(1, rejectedPlans.length);
+                    const ixscans = getPlanStages(getRejectedPlan(rejectedPlans[0]), "IXSCAN");
+                    assert.eq(1, ixscans.length);
+                    ixscan = ixscans[0];
+                }
+                assert.eq("testIndexName", ixscan.indexName, tojson(ixscan));
+            }
             assert.commandWorked(coll.dropIndex("testIndexName"));
         };
 
@@ -384,8 +403,7 @@ const generateTest = (useHint) => {
                            {},
                            collation);
 
-        /*********************************** Tests $expr predicates
-         * *********************************/
+        /*********************************** Tests $expr predicates *******************************/
         resetCollections();
         assert.commandWorked(insert(coll, [
             {_id: 0, [timeFieldName]: ISODate('1990-01-01 00:00:00.000Z'), [metaFieldName]: 2},
