@@ -15,6 +15,22 @@ function getWinningPlan(queryPlanner) {
                                                                 : queryPlanner.winningPlan;
 }
 
+/**
+ * Returns the winning plan from the corresponding sub-node of classic/SBE explain output. Takes
+ * into account that the plan may or may not have agg stages.
+ */
+function getWinningPlanFromExplain(explain) {
+    if (explain.hasOwnProperty("shards")) {
+        for (const shardName in explain.shards) {
+            let queryPlanner = getQueryPlanner(explain.shards[shardName]);
+            return getWinningPlan(queryPlanner);
+        }
+    }
+
+    let queryPlanner = getQueryPlanner(explain);
+    return getWinningPlan(queryPlanner);
+}
+
 /**
  * Returns an element of explain output which represents a rejected candidate plan.
  */
@@ -185,7 +201,8 @@ function hasRejectedPlans(root) {
  * Returns an array of execution stages from the given replset or sharded explain output.
  */
 function getExecutionStages(root) {
-    if (root.executionStats.executionStages.hasOwnProperty("shards")) {
+    if (root.hasOwnProperty("executionStats") &&
+        root.executionStats.executionStages.hasOwnProperty("shards")) {
         const executionStages = [];
         for (let shard of root.executionStats.executionStages.shards) {
             executionStages.push(Object.assign(
@@ -194,6 +211,13 @@ function getExecutionStages(root) {
         }
         return executionStages;
     }
+    if (root.hasOwnProperty("shards")) {
+        const executionStages = [];
+        for (const shard in root.shards) {
+            executionStages.push(root.shards[shard].executionStats.executionStages);
+        }
+        return executionStages;
+    }
     return [root.executionStats.executionStages];
 }
 
@@ -328,6 +352,9 @@ function aggPlanHasStage(root, stage) {
 /**
  * Given the root stage of explain's BSON representation of a query plan ('root'),
  * returns true if the plan has a stage called 'stage'.
+ *
+ * Expects that the stage appears once or zero times per node. If the stage appears more than once
+ * on one node's query plan, an error will be thrown.
  */
 function planHasStage(db, root, stage) {
     const matchingStages = getPlanStages(root, stage);
@@ -521,26 +548,54 @@ function assertStagesForExplainOfCommand({coll, cmdObj, expectedStages, stagesNo
 }
 
 /**
- * Get the "planCacheKey" from the explain result.
+ * Utility to obtain a value from 'explainRes' using 'getValueCallback'.
+ */
+function getFieldValueFromExplain(explainRes, getValueCallback) {
+    assert(explainRes.hasOwnProperty("queryPlanner"), explainRes);
+    const plannerOutput = explainRes.queryPlanner;
+    const fieldValue = getValueCallback(plannerOutput);
+    assert.eq(typeof fieldValue, "string");
+    return fieldValue;
+}
+
+/**
+ * Get the 'planCacheKey' from 'explainRes'.
  */
 function getPlanCacheKeyFromExplain(explainRes, db) {
-    const hash = FixtureHelpers.isMongos(db) &&
-            explainRes.queryPlanner.hasOwnProperty("winningPlan") &&
-            explainRes.queryPlanner.winningPlan.hasOwnProperty("shards")
-        ? explainRes.queryPlanner.winningPlan.shards[0].planCacheKey
-        : explainRes.queryPlanner.planCacheKey;
-    assert.eq(typeof hash, "string");
+    return getFieldValueFromExplain(explainRes, function(plannerOutput) {
+        return FixtureHelpers.isMongos(db) && plannerOutput.hasOwnProperty("winningPlan") &&
+                plannerOutput.winningPlan.hasOwnProperty("shards")
+            ? plannerOutput.winningPlan.shards[0].planCacheKey
+            : plannerOutput.planCacheKey;
+    });
+}
 
-    return hash;
+/**
+ * Get the 'queryHash' from 'explainRes'.
+ */
+function getQueryHashFromExplain(explainRes, db) {
+    return getFieldValueFromExplain(explainRes, function(plannerOutput) {
+        return FixtureHelpers.isMongos(db) ? plannerOutput.winningPlan.shards[0].queryHash
+                                           : plannerOutput.queryHash;
+    });
 }
 
 /**
  * Helper to run a explain on the given query shape and get the "planCacheKey" from the explain
  * result.
  */
-function getPlanCacheKeyFromShape({query = {}, projection = {}, sort = {}, collection, db}) {
-    const explainRes =
-        assert.commandWorked(collection.explain().find(query, projection).sort(sort).finish());
+function getPlanCacheKeyFromShape({
+    query = {},
+    projection = {},
+    sort = {},
+    collation = {
+        locale: "simple"
+    },
+    collection,
+    db
+}) {
+    const explainRes = assert.commandWorked(
+        collection.explain().find(query, projection).collation(collation).sort(sort).finish());
 
     return getPlanCacheKeyFromExplain(explainRes, db);
 }
@@ -567,3 +622,115 @@ function flattenQueryPlanTree(winningPlan) {
     stages.reverse();
     return stages;
 }
+
+/**
+ * Assert that a command plan has no FETCH stage or if the stage is present, it has no filter.
+ */
+function assertNoFetchFilter({coll, cmdObj}) {
+    const plan = assert.commandWorked(coll.runCommand({explain: cmdObj}));
+    const winningPlan = getWinningPlan(plan.queryPlanner);
+    const fetch = getPlanStage(winningPlan, "FETCH");
+    assert((fetch === null || !fetch.hasOwnProperty("filter")),
+           "Unexpected fetch: " + tojson(fetch));
+    return winningPlan;
+}
+
+/**
+ * Assert that a find plan has a FETCH stage with expected filter and returns a specified number of
+ * results.
+ */
+function assertFetchFilter({coll, predicate, expectedFilter, nReturned}) {
+    const exp = coll.find(predicate).explain("executionStats");
+    const plan = getWinningPlan(exp.queryPlanner);
+    const fetch = getPlanStage(plan, "FETCH");
+    assert(fetch !== null, "Missing FETCH stage " + plan);
+    assert(fetch.hasOwnProperty("filter"),
+           "Expected filter in the fetch stage, got " + tojson(fetch));
+    assert.eq(expectedFilter,
+              fetch.filter,
+              "Expected filter " + tojson(expectedFilter) + " got " + tojson(fetch.filter));
+
+    if (nReturned !== null) {
+        assert.eq(exp.executionStats.nReturned,
+                  nReturned,
+                  "Expected " + nReturned + " documents, got " + exp.executionStats.nReturned);
+    }
+}
+
+/**
+ * Recursively checks if a javascript object contains a nested property key and returns the
+ value.
+ * Note, this only recurses into other objects, array elements are ignored.
+ *
+ * This helper function can be used for any optimizer.
+ */
+function getNestedProperty(object, key) {
+    if (typeof object !== "object") {
+        return null;
+    }
+
+    for (const k in object) {
+        if (k == key) {
+            return object[k];
+        }
+
+        const result = getNestedProperty(object[k], key);
+        if (result) {
+            return result;
+        }
+    }
+    return null;
+}
+
+/**
+ * Returns the output from a single shard if 'explain' was obtained from an unsharded collection;
+ * returns 'explain' as is otherwise.
+ *
+ * This helper function can be used for any optimizer.
+ */
+function getSingleNodeExplain(explain) {
+    if ("shards" in explain) {
+        const shards = explain.shards;
+        const shardNames = Object.keys(shards);
+        // There should only be one shard given that this function assumes that 'explain' was
+        // obtained from an unsharded collection.
+        assert.eq(shardNames.length, 1, explain);
+        return shards[shardNames[0]];
+    }
+    return explain;
+}
+
+/**
+ * Utility to return the 'queryPlanner' section of 'explain'. The input is the root of the
+ explain
+ * output.
+ *
+ * This helper function can be used for any optimizer.
+ */
+function getQueryPlanner(explain) {
+    explain = getSingleNodeExplain(explain);
+    if ("queryPlanner" in explain) {
+        const qp = explain.queryPlanner;
+        // Sharded case.
+        if ("winningPlan" in qp && "shards" in qp.winningPlan) {
+            return qp.winningPlan.shards[0];
+        }
+        return qp;
+    }
+    assert(explain.hasOwnProperty("stages"), explain);
+    const stage = explain.stages[0];
+    assert(stage.hasOwnProperty("$cursor"), explain);
+    const cursorStage = stage.$cursor;
+    assert(cursorStage.hasOwnProperty("queryPlanner"), explain);
+    return cursorStage.queryPlanner;
+}
+
+/**
+ * Recognizes the query engine used by the query (sbe/classic).
+ *
+ * This helper function can be used for any optimizer.
+ */
+function getEngine(explain) {
+    const queryPlanner = {...getQueryPlanner(explain)};
+    return getNestedProperty(queryPlanner, "slotBasedPlan") ? "sbe" : "classic";
+}
