@@ -1,11 +1,17 @@
-// @tags: [does_not_support_stepdowns, requires_profiling, assumes_read_preference_unchanged]
-//
-//  Confirms that profiled find queries and corresponding logs have matching queryHashes.
+/**
+ * Confirms that profiled find queries and corresponding logs have matching queryHashes.
+ * @tags: [
+ *  does_not_support_stepdowns,
+ *  requires_profiling,
+ *  assumes_read_preference_unchanged,
+ *  # TODO SERVER-67607: support query hash in slow query log lines.
+ *  cqf_incompatible,
+ * ]
+ */
 (function() {
 "use strict";
 
 // For getLatestProfilerEntry().
-load("jstests/libs/logv2_helpers.js");
 load("jstests/libs/profiler.js");
 load("jstests/libs/sbe_util.js");
 
@@ -31,12 +37,8 @@ assert.commandWorked(testDB.setLogLevel(0, "query"));
 // Returns true if the logLine command components correspond to the profile entry. This is
 // sufficient for the purpose of testing query hashes.
 function logMatchesEntry(logLine, profileEntry) {
-    if ((!isJsonLogNoConn() ? logLine.indexOf("command: find { find: \"test\"") >= 0
-                            : logLine.indexOf('command":{"find":"test"') >= 0) &&
-        logLine.indexOf(profileEntry["command"]["comment"]) >= 0) {
-        return true;
-    }
-    return false;
+    return logLine.indexOf('command":{"find":"test"') >= 0 &&
+        logLine.indexOf(profileEntry["command"]["comment"]) >= 0;
 }
 
 // Fetch the log line that corresponds to the profile entry. If there is no such line, return
@@ -56,7 +58,7 @@ function retrieveLogLine(log, profileEntry) {
 
 // Run the find command, retrieve the corresponding profile object and log line, then ensure
 // that both the profile object and log line have matching stable query hashes (if any).
-function runTestsAndGetHashes(db, {comment, test, hasPlanCacheKey}) {
+function runTestsAndGetHashes(db, {comment, test}) {
     assert.commandWorked(db.adminCommand({clearLog: "global"}));
     assert.doesNotThrow(() => test(db, comment));
     const log = assert.commandWorked(db.adminCommand({getLog: "global"})).log;
@@ -65,17 +67,13 @@ function runTestsAndGetHashes(db, {comment, test, hasPlanCacheKey}) {
     const logLine = retrieveLogLine(log, profileEntry);
     assert.neq(logLine, null);
 
-    // Confirm that the query hashes either exist or don't exist in both log and profile
-    // entries. If the queryHash and planCacheKey exist, ensure that the hashes from the
-    // profile entry match the log line.
-    assert(profileEntry.hasOwnProperty("queryHash"));
-    assert.eq(hasPlanCacheKey, profileEntry.hasOwnProperty("planCacheKey"));
-    assert(logLine.indexOf(profileEntry["queryHash"]) >= 0);
-    assert.eq(hasPlanCacheKey, (logLine.indexOf(profileEntry["planCacheKey"]) >= 0));
-    if (hasPlanCacheKey) {
-        return {queryHash: profileEntry["queryHash"], planCacheKey: profileEntry["planCacheKey"]};
-    }
-    return null;
+    assert(profileEntry.hasOwnProperty("queryHash"), profileEntry);
+    assert(profileEntry.hasOwnProperty("planCacheKey"), profileEntry);
+    assert(logLine.indexOf(profileEntry["queryHash"]) >= 0,
+           `entry=${tojson(profileEntry)}, logLine=${tojson(logLine)}`);
+    assert(logLine.indexOf(profileEntry["planCacheKey"]) >= 0,
+           `entry=${tojson(profileEntry)}, logLine=${tojson(logLine)}`);
+    return {queryHash: profileEntry["queryHash"], planCacheKey: profileEntry["planCacheKey"]};
 }
 
 // Add data and indices.
@@ -108,7 +106,6 @@ const testList = [
         test: function(db, comment) {
             assert.eq(200, db.test.find().comment(comment).itcount());
         },
-        hasPlanCacheKey: checkSBEEnabled(testDB, ["featureFlagSbeFull"])
     },
     {
         comment: "Test1 find query",
@@ -117,7 +114,6 @@ const testList = [
                       db.test.find(queryB, projectionB).sort(sortC).comment(comment).itcount(),
                       'unexpected document count');
         },
-        hasPlanCacheKey: true
     },
     {
         comment: "Test2 find query",
@@ -126,7 +122,6 @@ const testList = [
                       db.test.find(queryA, projectionB).sort(sortC).comment(comment).itcount(),
                       'unexpected document count');
         },
-        hasPlanCacheKey: true
     }
 ];
 
@@ -147,8 +142,6 @@ const testInactiveCreationLog = {
             db.test.find({b: {$lt: 12}, a: {$eq: 500}}).sort({a: -1}).comment(comment).itcount(),
             'unexpected document count');
     },
-    hasPlanCacheKey: true
-
 };
 const onCreationHashes = runTestsAndGetHashes(testDB, testInactiveCreationLog);
 const log = assert.commandWorked(testDB.adminCommand({getLog: "global"})).log;
