@@ -4,8 +4,33 @@
 
 "use strict";
 
+import {
+    makeX509OptionsForTest,
+    makeMigrationCertificatesForTest,
+    createTenantMigrationRecipientRoleIfNotExist,
+    createTenantMigrationDonorRoleIfNotExist,
+    runTenantMigrationCommand,
+    isMigrationCompleted,
+    checkTenantDBHashes,
+    getExternalKeys,
+    isShardMergeEnabled,
+    isNamespaceForTenant,
+    getTenantMigrationAccessBlocker,
+} from "jstests/replsets/libs/tenant_migration_util.js";
 load("jstests/aggregation/extras/utils.js");
-load("jstests/replsets/libs/tenant_migration_util.js");
+
+function loadDummyData() {
+    const numDocs = 20;
+    const testData = [];
+    for (let i = 0; i < numDocs; ++i) {
+        testData.push({_id: i, x: i});
+    }
+    return testData;
+}
+
+function buildErrorMsg(migrationId, expectedState, expectedAccessState, configDoc, recipientMtab) {
+    return tojson({migrationId, expectedState, expectedAccessState, configDoc, recipientMtab});
+}
 
 /**
  * This fixture allows the user to optionally pass in a custom ReplSetTest for the donor and
@@ -13,138 +38,160 @@ load("jstests/replsets/libs/tenant_migration_util.js");
  *
  * If the caller does not provide their own replica set, a two node replset will be initialized
  * instead, with all nodes running the latest version.
- *
- * @param {string} [name] the name of the replica sets
- * @param {boolean} [enableRecipientTesting] whether recipient would actually migrate tenant data
- * @param {Object} [donorRst] the ReplSetTest instance to adopt for the donor
- * @param {Object} [recipientRst] the ReplSetTest instance to adopt for the recipient
- * @param {Object} [sharedOptions] an object that can contain 'nodes' <number>, the number of nodes
- *     each RST will contain, and 'setParameter' <object>, an object with various server parameters.
- * @param {boolean} [allowDonorReadAfterMigration] whether donor would allow reads after a committed
- *     migration.
- * @param {boolean} [initiateRstWithHighElectionTimeout] whether donor and recipient replica sets
- *     should be initiated with high election timeout.
- * @param {boolean} [quickGarbageCollection] whether to set a low garbageCollectionDelayMS.
- * @param {string} [insertDataForTenant] create dummy data in <tenantId>_test database.
  */
-function TenantMigrationTest({
-    name = "TenantMigrationTest",
-    enableRecipientTesting = true,
-    donorRst,
-    recipientRst,
-    sharedOptions = {},
-    // Default this to true so it is easier for data consistency checks.
-    allowStaleReadsOnDonor = true,
-    initiateRstWithHighElectionTimeout = true,
-    quickGarbageCollection = false,
-    insertDataForTenant,
-    optimizeMigrations = true,
-}) {
-    const donorPassedIn = (donorRst !== undefined);
-    const recipientPassedIn = (recipientRst !== undefined);
-
-    const migrationX509Options = TenantMigrationUtil.makeX509OptionsForTest();
-    const migrationCertificates = TenantMigrationUtil.makeMigrationCertificatesForTest();
-
-    const nodes = sharedOptions.nodes || 2;
-    const setParameterOpts = sharedOptions.setParameter || {};
-    if (optimizeMigrations) {
-        // A tenant migration recipient's `OplogFetcher` uses aggregation which does not support
-        // tailable awaitdata cursors. For aggregation commands `OplogFetcher` will default to half
-        // the election timeout (e.g: 5 seconds) between getMores. That wait is largely unnecessary.
-        setParameterOpts["failpoint.setSmallOplogGetMoreMaxTimeMS"] = tojson({"mode": "alwaysOn"});
-    }
-    if (quickGarbageCollection) {
-        setParameterOpts.tenantMigrationGarbageCollectionDelayMS = 0;
-        setParameterOpts.ttlMonitorSleepSecs = 1;
+export class TenantMigrationTest {
+    /**
+     * Takes in the response to the donarStartMigration command and asserts the command
+     * works and the state is 'committed'.
+     */
+    static assertCommitted(stateRes) {
+        assert.commandWorked(stateRes);
+        assert.eq(stateRes.state, TenantMigrationTest.DonorState.kCommitted, tojson(stateRes));
+        return stateRes;
     }
 
-    donorRst = donorPassedIn ? donorRst : performSetUp(true /* isDonor */);
-    recipientRst = recipientPassedIn ? recipientRst : performSetUp(false /* isDonor */);
-
-    donorRst.asCluster(donorRst.nodes, () => {
-        donorRst.getPrimary();
-        donorRst.awaitReplication();
-        TenantMigrationUtil.createTenantMigrationRecipientRoleIfNotExist(donorRst);
-    });
-
-    recipientRst.asCluster(recipientRst.nodes, () => {
-        recipientRst.getPrimary();
-        recipientRst.awaitReplication();
-        TenantMigrationUtil.createTenantMigrationDonorRoleIfNotExist(recipientRst);
-    });
-
-    function loadDummyData() {
-        const numDocs = 20;
-        const testData = [];
-        for (let i = 0; i < numDocs; ++i) {
-            testData.push({_id: i, x: i});
+    /**
+     * Takes in the response to the donarStartMigration command and asserts the command
+     * works and the state is 'aborted', with optional errorCode.
+     */
+    static assertAborted(stateRes, errorCode) {
+        assert.commandWorked(stateRes);
+        assert.eq(stateRes.state, TenantMigrationTest.DonorState.kAborted, tojson(stateRes));
+        if (errorCode !== undefined) {
+            assert.eq(stateRes.abortReason.code, errorCode, tojson(stateRes));
+        }
+        return stateRes;
+    }
+
+    /**
+     * Make a new TenantMigrationTest
+     *
+     * @param {string} [name] the name of the replica sets
+     * @param {boolean} [enableRecipientTesting] whether recipient would actually migrate tenant
+     *     data
+     * @param {Object} [donorRst] the ReplSetTest instance to adopt for the donor
+     * @param {Object} [recipientRst] the ReplSetTest instance to adopt for the recipient
+     * @param {Object} [sharedOptions] an object that can contain 'nodes' <number>, the number of
+     *     nodes each RST will contain, and 'setParameter' <object>, an object with various server
+     *     parameters.
+     * @param {boolean} [allowDonorReadAfterMigration] whether donor would allow reads after a
+     *     committed migration.
+     * @param {boolean} [initiateRstWithHighElectionTimeout] whether donor and recipient replica
+     *     sets should be initiated with high election timeout.
+     * @param {boolean} [quickGarbageCollection] whether to set a low garbageCollectionDelayMS.
+     * @param {string} [insertDataForTenant] create dummy data in <tenantId>_test database.
+     */
+    constructor({
+        name = "TenantMigrationTest",
+        enableRecipientTesting = true,
+        donorRst,
+        recipientRst,
+        sharedOptions = {},
+        // Default this to true so it is easier for data consistency checks.
+        allowStaleReadsOnDonor = true,
+        initiateRstWithHighElectionTimeout = true,
+        quickGarbageCollection = false,
+        insertDataForTenant,
+        optimizeMigrations = true,
+    }) {
+        this._donorPassedIn = (donorRst !== undefined);
+        this._recipientPassedIn = (recipientRst !== undefined);
+        const migrationX509Options = makeX509OptionsForTest();
+        const nodes = sharedOptions.nodes || 2;
+        const setParameterOpts = sharedOptions.setParameter || {};
+        if (optimizeMigrations) {
+            // A tenant migration recipient's `OplogFetcher` uses aggregation which does not support
+            // tailable awaitdata cursors. For aggregation commands `OplogFetcher` will default to
+            // half the election timeout (e.g: 5 seconds) between getMores. That wait is largely
+            // unnecessary.
+            setParameterOpts["failpoint.setSmallOplogGetMoreMaxTimeMS"] =
+                tojson({"mode": "alwaysOn"});
+        }
+        if (quickGarbageCollection) {
+            setParameterOpts.tenantMigrationGarbageCollectionDelayMS = 0;
+            setParameterOpts.ttlMonitorSleepSecs = 1;
+        }
+
+        /**
+         * Creates a ReplSetTest instance. The repl set will have 2 nodes if not otherwise
+         * specified.
+         */
+        function performSetUp(isDonor) {
+            if (TestData.logComponentVerbosity) {
+                setParameterOpts["logComponentVerbosity"] =
+                    tojsononeline(TestData.logComponentVerbosity);
+            }
+
+            if (!(isDonor || enableRecipientTesting)) {
+                setParameterOpts["failpoint.returnResponseOkForRecipientSyncDataCmd"] =
+                    tojson({mode: 'alwaysOn'});
+            }
+
+            if (allowStaleReadsOnDonor) {
+                setParameterOpts["failpoint.tenantMigrationDonorAllowsNonTimestampedReads"] =
+                    tojson({mode: 'alwaysOn'});
+            }
+
+            let nodeOptions = isDonor ? migrationX509Options.donor : migrationX509Options.recipient;
+            nodeOptions["setParameter"] = setParameterOpts;
+
+            const rstName = `${name}_${(isDonor ? "donor" : "recipient")}`;
+            const rst = new ReplSetTest({name: rstName, nodes, serverless: true, nodeOptions});
+            rst.startSet();
+            if (initiateRstWithHighElectionTimeout) {
+                rst.initiateWithHighElectionTimeout();
+            } else {
+                rst.initiate();
+            }
+
+            return rst;
+        }
+
+        this._donorRst = this._donorPassedIn ? donorRst : performSetUp(true /* isDonor */);
+        this._recipientRst =
+            this._recipientPassedIn ? recipientRst : performSetUp(false /* isDonor */);
+
+        this._donorRst.asCluster(this._donorRst.nodes, () => {
+            this._donorRst.getPrimary();
+            this._donorRst.awaitReplication();
+            createTenantMigrationRecipientRoleIfNotExist(this._donorRst);
+        });
+
+        this._recipientRst.asCluster(this._recipientRst.nodes, () => {
+            this._recipientRst.getPrimary();
+            this._recipientRst.awaitReplication();
+            createTenantMigrationDonorRoleIfNotExist(this._recipientRst);
+        });
+
+        // Shard Merge installs TenantRecipientAccessBlockers only for tenants with data, so most
+        // tests require some data.
+        if (insertDataForTenant !== undefined) {
+            this.insertDonorDB(`${insertDataForTenant}_test`, "test");
         }
-        return testData;
     }
 
     /**
      * Inserts documents into the specified collection on the donor primary.
      */
-    this.insertDonorDB = function(dbName, collName, data = loadDummyData()) {
+    insertDonorDB(dbName, collName, data = loadDummyData()) {
         jsTestLog(`Inserting data into collection ${collName} of DB ${dbName} on the donor`);
-        const primary = donorRst.getPrimary();
+        const primary = this._donorRst.getPrimary();
         const db = primary.getDB(dbName);
         const res = assert.commandWorked(
             db.runCommand({insert: collName, documents: data, writeConcern: {w: 'majority'}}));
         jsTestLog(`Inserted with w: majority, opTime ${tojson(res.operationTime)}`);
-    };
+    }
 
     /**
      * Inserts documents into the specified collection on the recipient primary.
      */
-    this.insertRecipientDB = function(dbName, collName, data = loadDummyData()) {
+    insertRecipientDB(dbName, collName, data = loadDummyData()) {
         jsTestLog(`Inserting data into collection ${collName} of DB ${dbName} on the recipient`);
-        const primary = recipientRst.getPrimary();
+        const primary = this._recipientRst.getPrimary();
         const db = primary.getDB(dbName);
         const res = assert.commandWorked(
             db.runCommand({insert: collName, documents: data, writeConcern: {w: 'majority'}}));
         jsTestLog(`Inserted with w: majority, opTime ${tojson(res.operationTime)}`);
-    };
-
-    // Shard Merge installs TenantRecipientAccessBlockers only for tenants with data, so most tests
-    // require some data.
-    if (insertDataForTenant !== undefined) {
-        this.insertDonorDB(`${insertDataForTenant}_test`, "test");
-    }
-
-    /**
-     * Creates a ReplSetTest instance. The repl set will have 2 nodes if not otherwise specified.
-     */
-    function performSetUp(isDonor) {
-        if (TestData.logComponentVerbosity) {
-            setParameterOpts["logComponentVerbosity"] =
-                tojsononeline(TestData.logComponentVerbosity);
-        }
-
-        if (!(isDonor || enableRecipientTesting)) {
-            setParameterOpts["failpoint.returnResponseOkForRecipientSyncDataCmd"] =
-                tojson({mode: 'alwaysOn'});
-        }
-
-        if (allowStaleReadsOnDonor) {
-            setParameterOpts["failpoint.tenantMigrationDonorAllowsNonTimestampedReads"] =
-                tojson({mode: 'alwaysOn'});
-        }
-
-        let nodeOptions = isDonor ? migrationX509Options.donor : migrationX509Options.recipient;
-        nodeOptions["setParameter"] = setParameterOpts;
-
-        const rstName = `${name}_${(isDonor ? "donor" : "recipient")}`;
-        const rst = new ReplSetTest({name: rstName, nodes, nodeOptions});
-        rst.startSet();
-        if (initiateRstWithHighElectionTimeout) {
-            rst.initiateWithHighElectionTimeout();
-        } else {
-            rst.initiate();
-        }
-
-        return rst;
     }
 
     /**
@@ -155,12 +202,8 @@ function TenantMigrationTest({
      * returns the command response containing the migration state on the donor after the
      * migration has completed.
      */
-    this.runMigration = function(migrationOpts, opts = {}) {
-        const {
-            retryOnRetryableErrors = false,
-            automaticForgetMigration = true,
-            enableDonorStartMigrationFsync = false
-        } = opts;
+    runMigration(migrationOpts, opts = {}) {
+        const {retryOnRetryableErrors = false, automaticForgetMigration = true} = opts;
 
         const startRes = this.startMigration(migrationOpts, opts);
         if (!startRes.ok) {
@@ -178,21 +221,20 @@ function TenantMigrationTest({
         }
 
         return completeRes;
-    };
+    }
 
     /**
      * Starts a tenant migration by running the 'donorStartMigration' command once.
      *
      * Returns the result of the 'donorStartMigration' command.
      */
-    this.startMigration = function(
-        migrationOpts,
-        {retryOnRetryableErrors = false, enableDonorStartMigrationFsync = false} = {}) {
+    startMigration(migrationOpts,
+                   {retryOnRetryableErrors = false, enableDonorStartMigrationFsync = false} = {}) {
         return this.runDonorStartMigration(migrationOpts, {
             retryOnRetryableErrors,
             enableDonorStartMigrationFsync,
         });
-    };
+    }
 
     /**
      * Waits for a migration to complete by continuously polling the donor primary with
@@ -201,8 +243,9 @@ function TenantMigrationTest({
      *
      * Returns the result of the last 'donorStartMigration' command executed.
      */
-    this.waitForMigrationToComplete = function(
-        migrationOpts, retryOnRetryableErrors = false, forgetMigration = false) {
+    waitForMigrationToComplete(migrationOpts,
+                               retryOnRetryableErrors = false,
+                               forgetMigration = false) {
         // Assert that the migration has already been started.
         assert(this.getDonorPrimary().getCollection(TenantMigrationTest.kConfigDonorsNS).findOne({
             _id: UUID(migrationOpts.migrationIdString)
@@ -216,7 +259,7 @@ function TenantMigrationTest({
 
         this.forgetMigration(migrationOpts.migrationIdString, retryOnRetryableErrors);
         return donorStartReply;
-    };
+    }
 
     /**
      * Executes the 'donorStartMigration' command on the donor primary.
@@ -228,15 +271,23 @@ function TenantMigrationTest({
      * If 'retryOnRetryableErrors' is set, this function will retry if the command fails
      * with a NotPrimary or network error.
      */
-    this.runDonorStartMigration = function({
+    runDonorStartMigration({
         migrationIdString,
         tenantId,
-        recipientConnectionString = recipientRst.getURL(),
+        protocol,
+        tenantIds,
+        recipientConnectionString = this._recipientRst.getURL(),
         readPreference = {mode: "primary"},
-        donorCertificateForRecipient = migrationCertificates.donorCertificateForRecipient,
-        recipientCertificateForDonor = migrationCertificates.recipientCertificateForDonor,
+        donorCertificateForRecipient,
+        recipientCertificateForDonor,
     },
-                                           opts = {}) {
+                           opts = {}) {
+        const migrationCertificates = makeMigrationCertificatesForTest();
+        donorCertificateForRecipient =
+            donorCertificateForRecipient || migrationCertificates.donorCertificateForRecipient;
+        recipientCertificateForDonor =
+            recipientCertificateForDonor || migrationCertificates.recipientCertificateForDonor;
+
         const {
             waitForMigrationToComplete = false,
             retryOnRetryableErrors = false,
@@ -247,28 +298,30 @@ function TenantMigrationTest({
             donorStartMigration: 1,
             migrationId: UUID(migrationIdString),
             tenantId,
+            tenantIds,
             recipientConnectionString,
             readPreference,
             donorCertificateForRecipient,
             recipientCertificateForDonor,
+            protocol
         };
 
-        const stateRes = TenantMigrationUtil.runTenantMigrationCommand(cmdObj, this.getDonorRst(), {
+        const stateRes = runTenantMigrationCommand(cmdObj, this.getDonorRst(), {
             enableDonorStartMigrationFsync,
             retryOnRetryableErrors,
             shouldStopFunc: stateRes =>
-                (!waitForMigrationToComplete || TenantMigrationUtil.isMigrationCompleted(stateRes))
+                (!waitForMigrationToComplete || isMigrationCompleted(stateRes))
         });
 
         // If the migration has been successfully committed, check the db hashes for the tenantId
         // between the donor and recipient.
         if (stateRes.state === TenantMigrationTest.State.kCommitted) {
-            TenantMigrationUtil.checkTenantDBHashes(
-                this.getDonorRst(), this.getRecipientRst(), tenantId);
+            checkTenantDBHashes(
+                {donorRst: this.getDonorRst(), recipientRst: this.getRecipientRst(), tenantId});
         }
 
         return stateRes;
-    };
+    }
 
     /**
      * Runs the donorForgetMigration command with the given migrationId and returns the response.
@@ -276,10 +329,9 @@ function TenantMigrationTest({
      * If 'retryOnRetryableErrors' is set, this function will retry if the command fails with a
      * NotPrimary or network error.
      */
-    this.forgetMigration = function(migrationIdString, retryOnRetryableErrors = false) {
+    forgetMigration(migrationIdString, retryOnRetryableErrors = false) {
         const cmdObj = {donorForgetMigration: 1, migrationId: UUID(migrationIdString)};
-        const res = TenantMigrationUtil.runTenantMigrationCommand(
-            cmdObj, this.getDonorRst(), {retryOnRetryableErrors});
+        const res = runTenantMigrationCommand(cmdObj, this.getDonorRst(), {retryOnRetryableErrors});
 
         // If the command succeeded, we expect that the migration is marked garbage collectable on
         // the donor and the recipient. Check the state docs for expireAt, check that the oplog
@@ -297,20 +349,26 @@ function TenantMigrationTest({
                     _id: UUID(migrationIdString)
                 });
 
+            const shardMergeRecipientStateDoc =
+                recipientPrimary.getCollection(TenantMigrationTest.kConfigShardMergeRecipientsNS)
+                    .findOne({_id: UUID(migrationIdString)});
+
             if (donorStateDoc) {
                 assert(donorStateDoc.expireAt);
             }
             if (recipientStateDoc) {
                 assert(recipientStateDoc.expireAt);
             }
+            if (shardMergeRecipientStateDoc) {
+                assert(shardMergeRecipientStateDoc.expireAt);
+            }
 
             const configDBCollections = recipientPrimary.getDB('config').getCollectionNames();
             assert(!configDBCollections.includes('repl.migration.oplog_' + migrationIdString),
                    configDBCollections);
 
             this.getDonorRst().asCluster(donorPrimary, () => {
-                const donorKeys =
-                    TenantMigrationUtil.getExternalKeys(donorPrimary, UUID(migrationIdString));
+                const donorKeys = getExternalKeys(donorPrimary, UUID(migrationIdString));
                 if (donorKeys.length) {
                     donorKeys.forEach(key => {
                         assert(key.hasOwnProperty("ttlExpiresAt"), tojson(key));
@@ -319,8 +377,7 @@ function TenantMigrationTest({
             });
 
             this.getRecipientRst().asCluster(recipientPrimary, () => {
-                const recipientKeys =
-                    TenantMigrationUtil.getExternalKeys(recipientPrimary, UUID(migrationIdString));
+                const recipientKeys = getExternalKeys(recipientPrimary, UUID(migrationIdString));
                 if (recipientKeys.length) {
                     recipientKeys.forEach(key => {
                         assert(key.hasOwnProperty("ttlExpiresAt"), tojson(key));
@@ -330,29 +387,27 @@ function TenantMigrationTest({
         }
 
         return res;
-    };
+    }
 
     /**
      * Runs the donorAbortMigration command with the given migration options and returns the
      * response.
      */
-    this.tryAbortMigration = function(migrationOpts, retryOnRetryableErrors = false) {
+    tryAbortMigration(migrationOpts, retryOnRetryableErrors = false) {
         const cmdObj = {
             donorAbortMigration: 1,
             migrationId: UUID(migrationOpts.migrationIdString),
         };
-        return TenantMigrationUtil.runTenantMigrationCommand(
-            cmdObj, this.getDonorRst(), {retryOnRetryableErrors});
-    };
+        return runTenantMigrationCommand(cmdObj, this.getDonorRst(), {retryOnRetryableErrors});
+    }
 
     /**
      * Asserts that durable and in-memory state for the migration 'migrationId' and 'tenantId' is
      * eventually deleted from the given nodes.
      */
-    this.waitForMigrationGarbageCollection = function(
-        migrationId, tenantId, donorNodes, recipientNodes) {
-        donorNodes = donorNodes || donorRst.nodes;
-        recipientNodes = recipientNodes || recipientRst.nodes;
+    waitForMigrationGarbageCollection(migrationId, tenantId, donorNodes, recipientNodes) {
+        donorNodes = donorNodes || this._donorRst.nodes;
+        recipientNodes = recipientNodes || this._recipientRst.nodes;
 
         if (typeof migrationId === "string") {
             migrationId = UUID(migrationId);
@@ -374,6 +429,10 @@ function TenantMigrationTest({
                 node.getCollection(TenantMigrationTest.kConfigRecipientsNS);
             assert.soon(() => 0 === configRecipientsColl.count({_id: migrationId}), tojson(node));
 
+            const configShardMergeRecipientsColl =
+                node.getCollection(TenantMigrationTest.kConfigShardMergeRecipientsNS);
+            assert.soon(() => 0 === configRecipientsColl.count({_id: migrationId}), tojson(node));
+
             let mtab;
             assert.soon(() => {
                 mtab =
@@ -381,34 +440,34 @@ function TenantMigrationTest({
                 return !mtab;
             }, tojson(mtab));
         });
-    };
+    }
 
     /**
      * Asserts that the migration 'migrationId' and 'tenantId' eventually goes to the
      * expected state on all the given donor nodes.
      */
-    this.waitForDonorNodesToReachState = function(nodes, migrationId, tenantId, expectedState) {
+    waitForDonorNodesToReachState(nodes, migrationId, tenantId, expectedState) {
         nodes.forEach(node => {
             assert.soon(
                 () => this.isDonorNodeInExpectedState(node, migrationId, tenantId, expectedState));
         });
-    };
+    }
 
     /**
      * Asserts that the migration 'migrationId' and 'tenantId' is in the expected state on all the
      * given donor nodes.
      */
-    this.assertDonorNodesInExpectedState = function(nodes, migrationId, tenantId, expectedState) {
+    assertDonorNodesInExpectedState(nodes, migrationId, tenantId, expectedState) {
         nodes.forEach(node => {
             assert(this.isDonorNodeInExpectedState(node, migrationId, tenantId, expectedState));
         });
-    };
+    }
 
     /**
      * Returns true if the durable and in-memory state for the migration 'migrationId' and
      * 'tenantId' is in the expected state, and false otherwise.
      */
-    this.isDonorNodeInExpectedState = function(node, migrationId, tenantId, expectedState) {
+    isDonorNodeInExpectedState(node, migrationId, tenantId, expectedState) {
         const configDonorsColl =
             this.getDonorPrimary().getCollection(TenantMigrationTest.kConfigDonorsNS);
         const configDoc = configDonorsColl.findOne({_id: migrationId});
@@ -421,18 +480,13 @@ function TenantMigrationTest({
             : TenantMigrationTest.DonorAccessState.kAborted;
         const mtab = this.getTenantMigrationAccessBlocker({donorNode: node, tenantId});
         return (mtab.donor.state === expectedAccessState);
-    };
-
-    function buildErrorMsg(
-        migrationId, expectedState, expectedAccessState, configDoc, recipientMtab) {
-        return tojson({migrationId, expectedState, expectedAccessState, configDoc, recipientMtab});
     }
 
     /**
      * Asserts that the migration 'migrationId' and 'tenantId' eventually goes to the expected state
      * on all the given recipient nodes.
      */
-    this.waitForRecipientNodesToReachState = function(
+    waitForRecipientNodesToReachState(
         nodes, migrationId, tenantId, expectedState, expectedAccessState) {
         nodes.forEach(node => {
             let result = {};
@@ -451,13 +505,13 @@ function TenantMigrationTest({
                                       result.recipientMtab);
                 });
         });
-    };
+    }
 
     /**
      * Asserts that the migration 'migrationId' and 'tenantId' is in the expected state on all the
      * given recipient nodes.
      */
-    this.assertRecipientNodesInExpectedState = function({
+    assertRecipientNodesInExpectedState({
         nodes,
         migrationId,
         tenantId,
@@ -476,13 +530,13 @@ function TenantMigrationTest({
                                   result.recipientMtab);
             });
         });
-    };
+    }
 
     /**
      * Returns true if the durable and in-memory state for the migration 'migrationId' and
      * 'tenantId' is in the expected state, and false otherwise.
      */
-    this.isRecipientNodeInExpectedState = function({
+    isRecipientNodeInExpectedState({
         node,
         migrationId,
         tenantId,
@@ -491,7 +545,13 @@ function TenantMigrationTest({
     }) {
         const configRecipientsColl =
             this.getRecipientPrimary().getCollection("config.tenantMigrationRecipients");
-        const configDoc = configRecipientsColl.findOne({_id: migrationId});
+        let configDoc = configRecipientsColl.findOne({_id: migrationId});
+        if (!configDoc) {
+            configDoc = this.getRecipientPrimary()
+                            .getCollection(TenantMigrationTest.kConfigShardMergeRecipientsNS)
+                            .findOne({_id: migrationId});
+        }
+
         const mtab = this.getTenantMigrationAccessBlocker({recipientNode: node, tenantId});
 
         let checkStates = () => {
@@ -502,17 +562,17 @@ function TenantMigrationTest({
         };
 
         return {value: checkStates(), configDoc: configDoc, recipientMtab: mtab.recipient};
-    };
+    }
 
     /**
      * Verifies that the documents on the recipient primary are correct.
      */
-    this.verifyRecipientDB = function(
+    verifyRecipientDB(
         tenantId, dbName, collName, migrationCommitted = true, data = loadDummyData()) {
         // We should migrate all data regardless of tenant id for shard merge.
         const shouldMigrate = migrationCommitted &&
-            (TenantMigrationUtil.isShardMergeEnabled(this.getRecipientPrimary().getDB("admin")) ||
-             TenantMigrationUtil.isNamespaceForTenant(tenantId, dbName));
+            (isShardMergeEnabled(this.getRecipientPrimary().getDB("admin")) ||
+             isNamespaceForTenant(tenantId, dbName));
 
         jsTestLog(`Verifying that data in collection ${collName} of DB ${dbName} was ${
             (shouldMigrate ? "" : "not")} migrated to the recipient`);
@@ -540,107 +600,77 @@ function TenantMigrationTest({
         const docsReturned = findRes.sort({_id: 1}).toArray();
         assert(arrayEq(docsReturned, data),
                () => (`${tojson(docsReturned)} is not equal to ${tojson(data)}`));
-    };
+    }
 
     /**
      * Crafts a tenant database name.
      */
-    this.tenantDB = function(tenantId, dbName) {
+    tenantDB(tenantId, dbName) {
         return `${tenantId}_${dbName}`;
-    };
-
-    /**
-     * Crafts a database name that does not belong to the tenant.
-     */
-    this.nonTenantDB = function(tenantId, dbName) {
-        return `non_${tenantId}_${dbName}`;
-    };
+    }
 
     /**
      * Returns the TenantMigrationAccessBlocker serverStatus output for the migration or shard merge
      * for the given node.
      */
-    this.getTenantMigrationAccessBlocker = function(obj) {
-        return TenantMigrationUtil.getTenantMigrationAccessBlocker(obj);
-    };
+    getTenantMigrationAccessBlocker(obj) {
+        return getTenantMigrationAccessBlocker(obj);
+    }
 
     /**
      * Returns the TenantMigrationStats on the node.
      */
-    this.getTenantMigrationStats = function(node) {
+    getTenantMigrationStats(node) {
         return assert.commandWorked(node.adminCommand({serverStatus: 1})).tenantMigrations;
-    };
+    }
 
     /**
      * Returns the donor ReplSetTest.
      */
-    this.getDonorRst = function() {
-        return donorRst;
-    };
+    getDonorRst() {
+        return this._donorRst;
+    }
 
     /**
      * Returns the recipient ReplSetTest.
      */
-    this.getRecipientRst = function() {
-        return recipientRst;
-    };
+    getRecipientRst() {
+        return this._recipientRst;
+    }
 
     /**
      * Returns the donor's primary.
      */
-    this.getDonorPrimary = function() {
+    getDonorPrimary() {
         return this.getDonorRst().getPrimary();
-    };
+    }
 
     /**
      * Returns the recipient's primary.
      */
-    this.getRecipientPrimary = function() {
+    getRecipientPrimary() {
         return this.getRecipientRst().getPrimary();
-    };
+    }
 
     /**
      * Returns the recipient's connection string.
      */
-    this.getRecipientConnString = function() {
+    getRecipientConnString() {
         return this.getRecipientRst().getURL();
-    };
+    }
 
     /**
      * Shuts down the donor and recipient sets, only if they were not passed in as parameters.
      * If they were passed in, the test that initialized them should be responsible for shutting
      * them down.
      */
-    this.stop = function() {
-        if (!donorPassedIn)
-            donorRst.stopSet();
-        if (!recipientPassedIn)
-            recipientRst.stopSet();
-    };
-}
-
-/**
- * Takes in the response to the donarStartMigration command and asserts the command
- * works and the state is 'committed'.
- */
-TenantMigrationTest.assertCommitted = function(stateRes) {
-    assert.commandWorked(stateRes);
-    assert.eq(stateRes.state, TenantMigrationTest.DonorState.kCommitted, tojson(stateRes));
-    return stateRes;
-};
-
-/**
- * Takes in the response to the donarStartMigration command and asserts the command
- * works and the state is 'aborted', with optional errorCode.
- */
-TenantMigrationTest.assertAborted = function(stateRes, errorCode) {
-    assert.commandWorked(stateRes);
-    assert.eq(stateRes.state, TenantMigrationTest.DonorState.kAborted, tojson(stateRes));
-    if (errorCode !== undefined) {
-        assert.eq(stateRes.abortReason.code, errorCode, tojson(stateRes));
+    stop() {
+        if (!this._donorPassedIn)
+            this._donorRst.stopSet();
+        if (!this._recipientPassedIn)
+            this._recipientRst.stopSet();
     }
-    return stateRes;
-};
+}
 
 TenantMigrationTest.DonorState = {
     kCommitted: "committed",
@@ -656,6 +686,16 @@ TenantMigrationTest.RecipientState = {
     kConsistent: "consistent",
     kDone: "done",
     kLearnedFilenames: "learned filenames",
+    kCommitted: "committed",
+    kAborted: "aborted",
+};
+
+TenantMigrationTest.ShardMergeRecipientState = {
+    kStarted: "started",
+    kLearnedFilenames: "learned filenames",
+    kConsistent: "consistent",
+    kCommitted: "committed",
+    kAborted: "aborted",
 };
 
 TenantMigrationTest.RecipientStateEnum =
@@ -681,3 +721,4 @@ TenantMigrationTest.RecipientAccessState = {
 
 TenantMigrationTest.kConfigDonorsNS = "config.tenantMigrationDonors";
 TenantMigrationTest.kConfigRecipientsNS = "config.tenantMigrationRecipients";
+TenantMigrationTest.kConfigShardMergeRecipientsNS = "config.shardMergeRecipients";
