@@ -10,11 +10,15 @@
  *   serverless,
  * ]
  */
-(function() {
-'use strict';
+
+import {TenantMigrationTest} from "jstests/replsets/libs/tenant_migration_test.js";
+import {
+    getCertificateAndPrivateKey,
+    isShardMergeEnabled,
+    makeX509OptionsForTest,
+} from "jstests/replsets/libs/tenant_migration_util.js";
 
 load("jstests/libs/uuid_util.js");
-load("jstests/replsets/libs/tenant_migration_test.js");
 
 function getRecipientSyncDataMetrics(recipientPrimary) {
     return recipientPrimary.adminCommand({serverStatus: 1}).metrics.commands.recipientSyncData;
@@ -47,17 +51,7 @@ function assertNoCertificateOrPrivateKey(string) {
     assert(!string.includes("PRIVATE KEY"), "found private key");
 }
 
-const kTenantIdPrefix = "testTenantId";
-let tenantCounter = 0;
-
-/**
- * Returns a tenantId that will not match any existing prefix.
- */
-function generateUniqueTenantId() {
-    return kTenantIdPrefix + tenantCounter++;
-}
-
-const {donor: donorNodeOptions} = TenantMigrationUtil.makeX509OptionsForTest();
+const {donor: donorNodeOptions} = makeX509OptionsForTest();
 donorNodeOptions.setParameter = donorNodeOptions.setParameter || {};
 Object.assign(donorNodeOptions.setParameter, {
     tenantMigrationGarbageCollectionDelayMS: 1 * 1000,
@@ -66,6 +60,7 @@ Object.assign(donorNodeOptions.setParameter, {
 const donorRst = new ReplSetTest({
     nodes: 1,
     name: 'donorRst',
+    serverless: true,
     nodeOptions: donorNodeOptions,
 });
 
@@ -86,7 +81,7 @@ function teardown() {
 // Test that a retry of a donorStartMigration command joins the existing migration that has
 // completed but has not been garbage-collected.
 (() => {
-    const tenantId = `${generateUniqueTenantId()}RetryAfterMigrationCompletes`;
+    const tenantId = ObjectId().str;
     const migrationId = UUID();
     const migrationOpts = {
         migrationIdString: extractUUIDFromObject(migrationId),
@@ -110,7 +105,7 @@ function teardown() {
 
 // Test that a retry of a donorStartMigration command joins the ongoing migration.
 (() => {
-    const tenantId = `${generateUniqueTenantId()}RetryBeforeMigrationCompletes`;
+    const tenantId = ObjectId().str;
     const migrationId = UUID();
     const migrationOpts = {
         migrationIdString: extractUUIDFromObject(migrationId),
@@ -259,7 +254,7 @@ function testConcurrentConflictingMigrations({
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: generateUniqueTenantId() + "DiffMigrationId",
+            tenantId: ObjectId().str,
         };
         const migrationOpts1 = Object.extend({}, migrationOpts0, true);
         migrationOpts1.migrationIdString = extractUUIDFromObject(UUID());
@@ -280,7 +275,7 @@ function testConcurrentConflictingMigrations({
 
 // Test different tenantIds.
 (() => {
-    if (TenantMigrationUtil.isShardMergeEnabled(donorPrimary.getDB("admin"))) {
+    if (isShardMergeEnabled(donorPrimary.getDB("admin"))) {
         jsTestLog(
             "Skip: featureFlagShardMerge is enabled and this test tests migrations with different tenant ids.");
         return;
@@ -289,10 +284,10 @@ function testConcurrentConflictingMigrations({
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: generateUniqueTenantId() + "DiffTenantId",
+            tenantId: ObjectId().str,
         };
         const migrationOpts1 = Object.extend({}, migrationOpts0, true);
-        migrationOpts1.tenantId = generateUniqueTenantId() + "DiffTenantId";
+        migrationOpts1.tenantId = ObjectId().str;
         return {
             tenantMigrationTest0: tenantMigrationTest,
             migrationOpts0,
@@ -313,7 +308,7 @@ function testConcurrentConflictingMigrations({
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: `${generateUniqueTenantId()}DiffRecipientConnString`,
+            tenantId: ObjectId().str,
         };
         // The recipient connection string will be populated by the TenantMigrationTest fixture, so
         // no need to set it here.
@@ -335,10 +330,15 @@ function testConcurrentConflictingMigrations({
 
 // Test different cloning read preference.
 (() => {
+    if (isShardMergeEnabled(donorPrimary.getDB("admin"))) {
+        jsTestLog(
+            "Skip: featureFlagShardMerge is enabled and this test tests migration's secondary read preference.");
+        return;
+    }
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: generateUniqueTenantId() + "DiffReadPref",
+            tenantId: ObjectId().str,
         };
         const migrationOpts1 = Object.extend({}, migrationOpts0, true);
         migrationOpts1.readPreference = {mode: "secondary"};
@@ -356,20 +356,20 @@ function testConcurrentConflictingMigrations({
 })();
 
 const kDonorCertificateAndPrivateKey =
-    TenantMigrationUtil.getCertificateAndPrivateKey("jstests/libs/tenant_migration_donor.pem");
-const kExpiredDonorCertificateAndPrivateKey = TenantMigrationUtil.getCertificateAndPrivateKey(
-    "jstests/libs/tenant_migration_donor_expired.pem");
+    getCertificateAndPrivateKey("jstests/libs/tenant_migration_donor.pem");
+const kExpiredDonorCertificateAndPrivateKey =
+    getCertificateAndPrivateKey("jstests/libs/tenant_migration_donor_expired.pem");
 const kRecipientCertificateAndPrivateKey =
-    TenantMigrationUtil.getCertificateAndPrivateKey("jstests/libs/tenant_migration_recipient.pem");
-const kExpiredRecipientCertificateAndPrivateKey = TenantMigrationUtil.getCertificateAndPrivateKey(
-    "jstests/libs/tenant_migration_recipient_expired.pem");
+    getCertificateAndPrivateKey("jstests/libs/tenant_migration_recipient.pem");
+const kExpiredRecipientCertificateAndPrivateKey =
+    getCertificateAndPrivateKey("jstests/libs/tenant_migration_recipient_expired.pem");
 
 // Test different donor certificates.
 (() => {
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: generateUniqueTenantId() + "DiffDonorCertificate",
+            tenantId: ObjectId().str,
             donorCertificateForRecipient: kDonorCertificateAndPrivateKey,
             recipientCertificateForDonor: kRecipientCertificateAndPrivateKey
         };
@@ -393,7 +393,7 @@ const kExpiredRecipientCertificateAndPrivateKey = TenantMigrationUtil.getCertifi
     const makeTestParams = () => {
         const migrationOpts0 = {
             migrationIdString: extractUUIDFromObject(UUID()),
-            tenantId: generateUniqueTenantId() + "DiffRecipientCertificate",
+            tenantId: ObjectId().str,
             donorCertificateForRecipient: kDonorCertificateAndPrivateKey,
             recipientCertificateForDonor: kRecipientCertificateAndPrivateKey
         };
@@ -413,4 +413,3 @@ const kExpiredRecipientCertificateAndPrivateKey = TenantMigrationUtil.getCertifi
 })();
 
 teardown();
-})();
