@@ -1,10 +1,24 @@
-function removeShard(st, shardName, timeout) {
+function removeShard(shardingTestOrConn, shardName, timeout) {
     if (timeout == undefined) {
         timeout = 10 * 60 * 1000;  // 10 minutes
     }
 
+    var s;
+    if (shardingTestOrConn instanceof ShardingTest) {
+        s = shardingTestOrConn.s;
+    } else {
+        s = shardingTestOrConn;
+    }
+
     assert.soon(function() {
-        let res = assert.commandWorked(st.s.adminCommand({removeShard: shardName}));
+        let res;
+        if (TestData.configShard && shardName == "config") {
+            // Need to use transitionToDedicatedConfigServer if trying
+            // to remove config server as a shard
+            res = s.adminCommand({transitionToDedicatedConfigServer: shardName});
+        } else {
+            res = s.adminCommand({removeShard: shardName});
+        }
         if (!res.ok && res.code === ErrorCodes.ShardNotFound) {
             // If the config server primary steps down right after removing the config.shards doc
             // for the shard but before responding with "state": "completed", the mongos would retry
@@ -14,6 +28,7 @@ function removeShard(st, shardName, timeout) {
             // to fail with ShardNotFound.
             return true;
         }
+        assert.commandWorked(res);
         return res.state == 'completed';
     }, "failed to remove shard " + shardName + " within " + timeout + "ms", timeout);
 }
