@@ -5,6 +5,9 @@
  * sure that the shard will be able to eventually reach the valid state on config.cache.
  */
 
+// Cannot run the filtering metadata check on tests that run refineCollectionShardKey.
+TestData.skipCheckShardFilteringMetadata = true;
+
 (function() {
 'use strict';
 
