@@ -3,7 +3,6 @@
  *
  * @tags: [
  *   requires_fcv_51,
- *   requires_find_command,
  * ]
  */
 
@@ -23,7 +22,8 @@ const dbName = 'testDB';
 const collName = 'coll';
 const timeField = "time";
 const metaField = "tag";
-const dateTime = ISODate("2021-07-12T16:00:00Z");
+const dateTime1 = ISODate("2021-07-12T16:00:00Z");
+const dateTime2 = ISODate("2021-07-13T16:00:00Z");
 
 //
 // Checks for feature flags.
@@ -35,13 +35,6 @@ if (!TimeseriesTest.shardedtimeseriesCollectionsEnabled(st.shard0)) {
     return;
 }
 
-if (!TimeseriesTest.timeseriesUpdatesAndDeletesEnabled(st.shard0)) {
-    jsTestLog(
-        "Skipping test because the updates and deletes on time-series collection feature flag is disabled");
-    st.stop();
-    return;
-}
-
 const testDB = mongos.getDB(dbName);
 testDB.dropDatabase();
 assert.commandWorked(mongos.adminCommand({enableSharding: dbName}));
@@ -66,29 +59,29 @@ if (!TimeseriesTest.shardedTimeseriesUpdatesAndDeletesEnabled(st.shard0)) {
 
 const doc1 = {
     _id: 1,
-    [timeField]: dateTime,
+    [timeField]: dateTime1,
     [metaField]: {a: "A", b: "B"}
 };
 const doc2 = {
     _id: 2,
-    [timeField]: dateTime,
+    [timeField]: dateTime2,
     [metaField]: {c: "C", d: 2},
     f: [{"k": "K", "v": "V"}]
 };
 const doc3 = {
     _id: 3,
-    [timeField]: dateTime,
+    [timeField]: dateTime1,
     f: "F"
 };
 const doc4 = {
     _id: 4,
-    [timeField]: dateTime,
+    [timeField]: dateTime1,
     [metaField]: {a: "A", b: "B"},
     f: "F"
 };
 const doc5 = {
     _id: 5,
-    [timeField]: dateTime,
+    [timeField]: dateTime1,
     [metaField]: {a: "A", b: "B", c: "C"}
 };
 
@@ -407,7 +400,7 @@ function testCaseBatchUpdates({testUpdate}) {
         ],
         resultDocList: [{
             _id: 2,
-            [timeField]: dateTime,
+            [timeField]: dateTime1,
             [metaField]: 3,
             f: [{"k": "K", "v": "V"}],
         }],
@@ -455,7 +448,7 @@ function testCaseBatchUpdates({testUpdate}) {
         ],
         resultDocList: [{
             _id: 2,
-            [timeField]: dateTime,
+            [timeField]: dateTime1,
             [metaField]: {c: "C", d: 8},
             f: [{"k": "K", "v": "V"}],
         }],
@@ -506,7 +499,7 @@ function testCaseBatchUpdates({testUpdate}) {
         ],
         resultDocList: [{
             _id: 2,
-            [timeField]: dateTime,
+            [timeField]: dateTime2,
             [metaField]: {c: "C", d: 15},
             f: [{"k": "K", "v": "V"}],
         }],
@@ -526,7 +519,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$rename: {[metaField + ".a"]: metaField + ".z"}},
             multi: true,
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: {z: "A", b: "B"}}, doc2],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: {z: "A", b: "B"}}, doc2],
         n: 1,
         pathToMetaFieldBeingUpdated: "a",
     });
@@ -539,7 +532,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: {c: "C"}}},
             multi: true,
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: {c: "C"}}],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: {c: "C"}}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -554,7 +547,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
         }],
         resultDocList: [
             doc1,
-            {_id: 2, [timeField]: dateTime, [metaField]: {c: 1, d: 2}, f: [{"k": "K", "v": "V"}]},
+            {_id: 2, [timeField]: dateTime2, [metaField]: {c: 1, d: 2}, f: [{"k": "K", "v": "V"}]},
             doc4,
             doc5
         ],
@@ -572,10 +565,15 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             multi: true,
         }],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: {b: "B"}},
-            {_id: 2, [timeField]: dateTime, [metaField]: {c: "C", d: 2}, f: [{"k": "K", "v": "V"}]},
-            {_id: 4, [timeField]: dateTime, [metaField]: {b: "B"}, f: "F"},
-            {_id: 5, [timeField]: dateTime, [metaField]: {b: "B", c: "C"}}
+            {_id: 1, [timeField]: dateTime1, [metaField]: {b: "B"}},
+            {
+                _id: 2,
+                [timeField]: dateTime1,
+                [metaField]: {c: "C", d: 2},
+                f: [{"k": "K", "v": "V"}]
+            },
+            {_id: 4, [timeField]: dateTime1, [metaField]: {b: "B"}, f: "F"},
+            {_id: 5, [timeField]: dateTime1, [metaField]: {b: "B", c: "C"}}
         ],
         ordered: false,
         n: 3,
@@ -590,7 +588,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: {c: "C"}}},
             multi: true,
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: {c: "C"}}],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: {c: "C"}}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -603,7 +601,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: {c: "C"}}},
             multi: true,
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: {c: "C"}}],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: {c: "C"}}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -616,9 +614,12 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$inc: {[metaField + ".d"]: 10}},
             multi: true,
         }],
-        resultDocList: [
-            {_id: 2, [timeField]: dateTime, [metaField]: {c: "C", d: 12}, f: [{"k": "K", "v": "V"}]}
-        ],
+        resultDocList: [{
+            _id: 2,
+            [timeField]: dateTime2,
+            [metaField]: {c: "C", d: 12},
+            f: [{"k": "K", "v": "V"}]
+        }],
         n: 1,
         pathToMetaFieldBeingUpdated: "d",
     });
@@ -632,8 +633,8 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             multi: true,
         }],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: {z: "Z"}},
-            {_id: 2, [timeField]: dateTime, [metaField]: {z: "Z"}, f: [{"k": "K", "v": "V"}]}
+            {_id: 1, [timeField]: dateTime1, [metaField]: {z: "Z"}},
+            {_id: 2, [timeField]: dateTime2, [metaField]: {z: "Z"}, f: [{"k": "K", "v": "V"}]}
         ],
         n: 2,
         pathToMetaFieldBeingUpdated: "",
@@ -644,7 +645,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
         initialDocList: [doc1],
         updates:
             [{q: {[metaField]: {a: "A", b: "B"}}, u: {$unset: {[metaField]: ""}}, multi: true}],
-        resultDocList: [{_id: 1, [timeField]: dateTime}],
+        resultDocList: [{_id: 1, [timeField]: dateTime1}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -660,9 +661,9 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             },
         ],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: {a: "A", b: "B", c: "C"}},
-            {_id: 4, [timeField]: dateTime, [metaField]: {a: "A", b: "B", c: "C"}, f: "F"},
-            {_id: 5, [timeField]: dateTime, [metaField]: {a: "A", b: "B", c: "C"}}
+            {_id: 1, [timeField]: dateTime1, [metaField]: {a: "A", b: "B", c: "C"}},
+            {_id: 4, [timeField]: dateTime1, [metaField]: {a: "A", b: "B", c: "C"}, f: "F"},
+            {_id: 5, [timeField]: dateTime1, [metaField]: {a: "A", b: "B", c: "C"}}
         ],
         n: 3,
         nModified: 2,
@@ -678,8 +679,8 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             multi: true
         }],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: "a"},
-            {_id: 2, [timeField]: dateTime, [metaField]: "a", f: [{"k": "K", "v": "V"}]},
+            {_id: 1, [timeField]: dateTime1, [metaField]: "a"},
+            {_id: 2, [timeField]: dateTime2, [metaField]: "a", f: [{"k": "K", "v": "V"}]},
             doc3
         ],
         n: 2,
@@ -694,7 +695,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: "a"}},
             multi: true
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: "a"}, doc2, doc3],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: "a"}, doc2, doc3],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -709,7 +710,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
     },
                expectFailedUpdate([doc1, doc2, doc3]));
 
-    const nestedMetaObj = {_id: 6, [timeField]: dateTime, [metaField]: {[metaField]: "A", a: 1}};
+    const nestedMetaObj = {_id: 6, [timeField]: dateTime1, [metaField]: {[metaField]: "A", a: 1}};
 
     // Query for documents using $jsonSchema with the metaField required and a required subfield of
     // the metaField with the same name as the metaField.
@@ -725,7 +726,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: "a"}},
             multi: true
         }],
-        resultDocList: [doc1, {_id: 6, [timeField]: dateTime, [metaField]: "a", a: 1}],
+        resultDocList: [doc1, {_id: 6, [timeField]: dateTime1, [metaField]: "a", a: 1}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -759,7 +760,7 @@ function testCaseValidMetaFieldUpdates({testUpdate}) {
             u: {$set: {[metaField]: "a"}},
             multi: true
         }],
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: "a"}, doc2, doc3],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: "a"}, doc2, doc3],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -801,9 +802,9 @@ function testCaseUpdateWithLetDoc({testUpdate}) {
         }],
         letDoc: {oldVal: "A"},
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: "aaa"},
-            {_id: 4, [timeField]: dateTime, [metaField]: "aaa", f: "F"},
-            {_id: 5, [timeField]: dateTime, [metaField]: "aaa"}
+            {_id: 1, [timeField]: dateTime1, [metaField]: "aaa"},
+            {_id: 4, [timeField]: dateTime1, [metaField]: "aaa", f: "F"},
+            {_id: 5, [timeField]: dateTime1, [metaField]: "aaa"}
         ],
         n: 3,
         pathToMetaFieldBeingUpdated: "",
@@ -820,7 +821,7 @@ function testCaseUpdateWithLetDoc({testUpdate}) {
             multi: true,
         }],
         letDoc: {myVar: "aaa"},
-        resultDocList: [{_id: 1, [timeField]: dateTime, [metaField]: "$$myVar"}],
+        resultDocList: [{_id: 1, [timeField]: dateTime1, [metaField]: "$$myVar"}],
         n: 1,
         pathToMetaFieldBeingUpdated: "",
     });
@@ -842,9 +843,9 @@ function testCaseUpdateWithLetDoc({testUpdate}) {
         ],
         letDoc: {val1: "A", val2: "aaa"},
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: "bbb"},
-            {_id: 4, [timeField]: dateTime, [metaField]: "bbb", f: "F"},
-            {_id: 5, [timeField]: dateTime, [metaField]: "bbb"}
+            {_id: 1, [timeField]: dateTime1, [metaField]: "bbb"},
+            {_id: 4, [timeField]: dateTime1, [metaField]: "bbb", f: "F"},
+            {_id: 5, [timeField]: dateTime1, [metaField]: "bbb"}
         ],
         n: 6,
         pathToMetaFieldBeingUpdated: "",
@@ -852,9 +853,9 @@ function testCaseUpdateWithLetDoc({testUpdate}) {
 }
 
 function testCaseCollationUpdates({testUpdate}) {
-    const collationDoc1 = {_id: 1, [timeField]: dateTime, [metaField]: "café"};
-    const collationDoc2 = {_id: 2, [timeField]: dateTime, [metaField]: "cafe"};
-    const collationDoc3 = {_id: 3, [timeField]: dateTime, [metaField]: "cafE"};
+    const collationDoc1 = {_id: 1, [timeField]: dateTime1, [metaField]: "café"};
+    const collationDoc2 = {_id: 2, [timeField]: dateTime1, [metaField]: "cafe"};
+    const collationDoc3 = {_id: 3, [timeField]: dateTime1, [metaField]: "cafE"};
     const initialDocList = [collationDoc1, collationDoc2, collationDoc3];
 
     // Query on the metaField and modify the metaField using collation with strength level 1.
@@ -867,9 +868,9 @@ function testCaseCollationUpdates({testUpdate}) {
             collation: {locale: "fr", strength: 1},
         }],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: "Updated"},
-            {_id: 2, [timeField]: dateTime, [metaField]: "Updated"},
-            {_id: 3, [timeField]: dateTime, [metaField]: "Updated"}
+            {_id: 1, [timeField]: dateTime1, [metaField]: "Updated"},
+            {_id: 2, [timeField]: dateTime1, [metaField]: "Updated"},
+            {_id: 3, [timeField]: dateTime1, [metaField]: "Updated"}
         ],
         n: 3,
         pathToMetaFieldBeingUpdated: "",
@@ -887,7 +888,7 @@ function testCaseCollationUpdates({testUpdate}) {
         }],
         resultDocList: [
             collationDoc1,
-            {_id: 2, [timeField]: dateTime, [metaField]: "Updated"},
+            {_id: 2, [timeField]: dateTime1, [metaField]: "Updated"},
             collationDoc3,
         ],
         n: 1,
@@ -897,9 +898,9 @@ function testCaseCollationUpdates({testUpdate}) {
 
 function testCaseNullUpdates({testUpdate}) {
     // Assumes shard key is meta.a.
-    const nullDoc = {_id: 1, [timeField]: dateTime, [metaField]: {a: null, b: 1}};
-    const missingDoc1 = {_id: 2, [timeField]: dateTime, [metaField]: {b: 1}};
-    const missingDoc2 = {_id: 3, [timeField]: dateTime, [metaField]: "foo"};
+    const nullDoc = {_id: 1, [timeField]: dateTime1, [metaField]: {a: null, b: 1}};
+    const missingDoc1 = {_id: 2, [timeField]: dateTime1, [metaField]: {b: 1}};
+    const missingDoc2 = {_id: 3, [timeField]: dateTime1, [metaField]: "foo"};
     const initialDocList = [nullDoc, missingDoc1, missingDoc2];
 
     // Query on the metaField and modify the metaField using collation with strength level 1.
@@ -911,9 +912,9 @@ function testCaseNullUpdates({testUpdate}) {
             multi: true,
         }],
         resultDocList: [
-            {_id: 1, [timeField]: dateTime, [metaField]: "Updated"},
-            {_id: 2, [timeField]: dateTime, [metaField]: "Updated"},
-            {_id: 3, [timeField]: dateTime, [metaField]: "Updated"},
+            {_id: 1, [timeField]: dateTime1, [metaField]: "Updated"},
+            {_id: 2, [timeField]: dateTime1, [metaField]: "Updated"},
+            {_id: 3, [timeField]: dateTime1, [metaField]: "Updated"},
         ],
         n: 3,
     });
