@@ -35,7 +35,11 @@
 #include "mongo/base/data_view.h"
 #include "mongo/bson/bson_depth.h"
 #include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonelementvalue.h"
 #include "mongo/bson/util/bsoncolumn.h"
+#include "mongo/bson/util/bsoncolumn_util.h"
+#include "mongo/crypto/encryption_fields_util.h"
+#include "mongo/crypto/fle_field_schema_gen.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/str_escape.h"
 
@@ -93,6 +97,10 @@ public:
     void checkDuplicateFieldName() {}
 
     void popLevel() {}
+
+    BSONValidateMode validateMode() {
+        return BSONValidateMode::kDefault;
+    }
 };
 
 class ExtendedValidator {
@@ -100,8 +108,8 @@ public:
     void checkNonConformantElem(const char* ptr, uint32_t offsetToValue, uint8_t type) {
         // Checks the field name before the element, if inside array.
         checkArrIndex(ptr);
-        // Increments the pointer to the actual element.
-        ptr += offsetToValue;
+        // Increments the pointer to the actual element value.
+        BSONElementValue bsonElemVal(ptr + offsetToValue);
         switch (type) {
             case BSONType::Undefined:
             case BSONType::DBRef:
@@ -116,15 +124,20 @@ public:
                 addIndexLevel(false /* isArr */);
                 break;
             case BSONType::RegEx: {
-                // Skips regular expression cstring.
-                const char* options = ptr + strlen(ptr) + 1;
-                _checkRegexOptions(options);
+                _checkRegexOptions(bsonElemVal);
                 break;
             }
             case BSONType::BinData: {
-                uint8_t subtype =
-                    ConstDataView(ptr + sizeof(uint32_t)).read<LittleEndian<uint8_t>>();
+                auto binData = bsonElemVal.BinData();
+                auto subtype = binData.type;
                 switch (subtype) {
+                    case BinDataType::BinDataGeneral:
+                    case BinDataType::Function:
+                    case BinDataType::Sensitive:
+                    case BinDataType::bdtCustom:
+                        break;
+                    case BinDataType::Column:
+                        break;
                     case BinDataType::ByteArrayDeprecated:
                     case BinDataType::bdtUUID:
                         uasserted(
@@ -132,24 +145,29 @@ public:
                             fmt::format("Use of deprecated BSON binary data subtype {}", subtype));
                         break;
                     case BinDataType::newUUID: {
-                        constexpr uint32_t UUIDLength = 16;
-                        uint32_t l = ConstDataView(ptr).read<LittleEndian<uint32_t>>();
-                        uassert(
-                            ErrorCodes::NonConformantBSON,
-                            fmt::format("BSON UUID length should be {} bytes. Found {} instead.",
-                                        UUIDLength,
-                                        l),
-                            l == UUIDLength);
+                        constexpr int32_t UUIDLength = 16;
+                        auto l = binData.length;
+                        uassert(ErrorCodes::NonConformantBSON,
+                                fmt::format(
+                                    "BSON UUID length should be 16 bytes. Found {} instead.", l),
+                                l == UUIDLength);
                         break;
                     }
                     case BinDataType::MD5Type: {
-                        constexpr uint32_t md5Length = 16;
-                        auto md5Size = ConstDataView(ptr).read<LittleEndian<uint32_t>>();
+                        constexpr int32_t md5Length = 16;
+                        auto l = binData.length;
                         uassert(NonConformantBSON,
-                                fmt::format("MD5 must be 16 bytes, got {} instead.", md5Size),
-                                md5Size == md5Length);
+                                fmt::format("MD5 must be 16 bytes, got {} instead.", l),
+                                l == md5Length);
                         break;
                     }
+                    case BinDataType::Encrypt: {
+                        _checkEncryptedBSONValue(binData);
+                        break;
+                    }
+                    default:
+                        uasserted(ErrorCodes::NonConformantBSON,
+                                  fmt::format("Unknown BSON Binary Data Type {}", subtype));
                 }
                 break;
             }
@@ -164,6 +182,10 @@ public:
         }
     }
 
+    BSONValidateMode validateMode() {
+        return BSONValidateMode::kExtended;
+    }
+
 private:
     struct Level {
         DecimalCounter<uint32_t> counter;  // Counter used to check whether indexes are sequential.
@@ -190,15 +212,16 @@ private:
         auto actualIndex = StringData(ptr + sizeof(char));
         uassert(NonConformantBSON,
                 fmt::format("Indices of BSON Array are invalid. Expected {}, but got {}.",
-                            indexCount.back().counter,
+                            (StringData)indexCount.back().counter,
                             actualIndex),
                 indexCount.back().counter == actualIndex);
         ++indexCount.back().counter;
     }
 
-    void _checkRegexOptions(const char* options) {
+    void _checkRegexOptions(const BSONElementValue& regex) {
         // Checks that the options are in ascending alphabetical order and that they're all valid.
         std::string validRegexOptions("ilmsux");
+        auto options = regex.RegexFlags();
         for (const auto& option : std::string(options)) {
             uassert(
                 NonConformantBSON,
@@ -213,6 +236,57 @@ private:
         }
     }
 
+    void _checkEncryptedBSONValue(const BSONBinData& binData) {
+        constexpr uint32_t UUIDLength = 16;
+        constexpr int32_t minLength = sizeof(uint8_t) + UUIDLength + sizeof(uint8_t);
+
+        auto len = binData.length;
+        // Make sure we can read the subtype byte of the Encrypted BSON Value.
+        uassert(ErrorCodes::NonConformantBSON,
+                fmt::format("Invalid Encrypted BSON Value length {}", len),
+                len);
+
+        // Skip the size bytes and BinData subtype byte to the actual encrypted data.
+        auto data = static_cast<const char*>(binData.data);
+        uint8_t encryptedBinDataTypeByte = ConstDataView(data).read<LittleEndian<uint8_t>>();
+        auto encryptedBinDataType = static_cast<EncryptedBinDataType>(encryptedBinDataTypeByte);
+        // Only subtype 1, 2, 6, 7, and 9 can exist in MongoDB collections.
+        switch (encryptedBinDataType) {
+            case EncryptedBinDataType::kDeterministic:
+            case EncryptedBinDataType::kRandom: {
+                uassert(ErrorCodes::NonConformantBSON,
+                        fmt::format("Invalid Encrypted BSON Value length {}", len),
+                        len > minLength);
+                break;
+            }
+            case EncryptedBinDataType::kFLE2UnindexedEncryptedValue:
+            case EncryptedBinDataType::kFLE2EqualityIndexedValue:
+            case EncryptedBinDataType::kFLE2RangeIndexedValue:
+            case EncryptedBinDataType::kFLE2EqualityIndexedValueV2:
+            case EncryptedBinDataType::kFLE2RangeIndexedValueV2:
+            case EncryptedBinDataType::kFLE2UnindexedEncryptedValueV2: {
+                uassert(ErrorCodes::NonConformantBSON,
+                        fmt::format("Invalid Encrypted BSON Value length {}", len),
+                        len >= minLength);
+                int8_t originalBsonTypeByte = ConstDataView(data + sizeof(uint8_t) + UUIDLength)
+                                                  .read<LittleEndian<uint8_t>>();
+                auto originalBsonType = static_cast<BSONType>(originalBsonTypeByte);
+                uassert(ErrorCodes::NonConformantBSON,
+                        fmt::format(
+                            "BSON type '{}' is not supported for Encrypted BSON Value subtype {}",
+                            typeName(originalBsonType),
+                            encryptedBinDataType),
+                        isFLE2SupportedType(encryptedBinDataType, originalBsonType));
+                break;
+            }
+            default: {
+                uasserted(ErrorCodes::NonConformantBSON,
+                          fmt::format("Unsupported Encrypted BSON Value type {} in the collection",
+                                      encryptedBinDataType));
+            }
+        }
+    }
+
 protected:
     // Behaves like a stack, used to validate array index count.
     std::vector<Level> indexCount;
@@ -221,22 +295,21 @@ protected:
 class FullValidator : private ExtendedValidator {
 public:
     void checkNonConformantElem(const char* ptr, uint32_t offsetToValue, uint8_t type) {
-        registerFieldName(ptr + 1);
+        registerFieldName(ptr + 1 /* fieldName */, offsetToValue - 1 /* length */);
         ExtendedValidator::checkNonConformantElem(ptr, offsetToValue, type);
-        // Check the field name is UTF-8 encoded.
-        checkUTF8Char(ptr + 1);
+        // Increments the pointer to the actual element value.
+        BSONElementValue bsonElemVal(ptr + offsetToValue);
         switch (type) {
             case BSONType::Array: {
-                objFrames.push_back({std::vector<std::string>(), false});
+                objFrames.push_back({std::vector<StringData>(), false});
                 break;
             }
             case BSONType::Object: {
-                objFrames.push_back({std::vector<std::string>(), true});
+                objFrames.push_back({std::vector<StringData>(), true});
                 break;
             };
             case BSONType::BinData: {
-                uint8_t subtype = ConstDataView(ptr + offsetToValue + sizeof(uint32_t))
-                                      .read<LittleEndian<uint8_t>>();
+                auto subtype = bsonElemVal.BinData().type;
                 switch (subtype) {
                     case BinDataType::Column: {
                         // Check for exceptions when decompressing.
@@ -247,13 +320,16 @@ public:
                             uasserted(NonConformantBSON,
                                       "Exception ocurred while decompressing a BSON column.");
                         }
+                        break;
                     }
+                    default:
+                        break;
                 }
                 break;
             }
             case BSONType::String: {
                 // Increment pointer to actual value and then four more to skip size.
-                checkUTF8Char(ptr + offsetToValue + 4);
+                checkUTF8Char(bsonElemVal.String());
             }
         }
     }
@@ -280,42 +356,55 @@ public:
         checkDuplicateFieldName();
     }
 
+    BSONValidateMode validateMode() {
+        return BSONValidateMode::kFull;
+    }
+
 private:
     // A given frame is an object if and only if frame.second == true.
-    std::vector<std::pair<std::vector<std::string>, bool>> objFrames = {
-        {std::vector<std::string>(), true}};
+    std::vector<std::pair<std::vector<StringData>, bool>> objFrames = {
+        {std::vector<StringData>(), true}};
 
-    void registerFieldName(std::string str) {
+    void registerFieldName(const char* ptr, uint32_t length) {
+        // Check the field name is UTF-8 encoded.
+        StringData fieldName(ptr, length);
+        checkUTF8Char(fieldName);
         if (objFrames.back().second) {
-            objFrames.back().first.emplace_back(str);
+            objFrames.back().first.emplace_back(fieldName);
         };
     }
 
-private:
-    void checkUTF8Char(const char* ptr) {
-        try {
-            str::checkInvalidUTF8(ptr);
-        } catch (const ExceptionFor<ErrorCodes::BadValue>&) {
-            uasserted(NonConformantBSON, "Found string that doesn't follow UTF-8 encoding.");
-        }
+    void checkUTF8Char(StringData str) {
+        uassert(NonConformantBSON,
+                "Found string that doesn't follow UTF-8 encoding.",
+                str::validUTF8(str));
     }
 };
 
+template <bool precise>
+Status _doValidateColumn(const char* originalBuffer,
+                         uint64_t maxLength,
+                         BSONValidateMode mode,
+                         ValidationVersion validationVersion);
+
 template <bool precise, typename BSONValidator>
 class ValidateBuffer {
 public:
-    ValidateBuffer(const char* data, uint64_t maxLength, BSONValidator validator)
-        : _data(data), _maxLength(maxLength), _validator(validator) {
+    ValidateBuffer(const char* data,
+                   uint64_t maxLength,
+                   BSONValidator validator,
+                   ValidationVersion validationVersion)
+        : _data(data),
+          _maxLength(maxLength),
+          _validator(validator),
+          _validationVersion(validationVersion) {
         if constexpr (precise)
             _frames.resize(BSONDepth::getMaxAllowableDepth() + 1);
     }
 
     Status validate() noexcept {
         try {
-            _currFrame = _frames.begin();
-            _currElem = nullptr;
-            auto maxFrames = BSONDepth::getMaxAllowableDepth() + 1;  // A flat BSON has one frame.
-            uassert(InvalidBSON, "Cannot enforce max nesting depth", _frames.size() <= maxFrames);
+            setupValidation();
             uassert(InvalidBSON, "BSON data has to be at least 5 bytes", _maxLength >= 5);
 
             // Read the length as signed integer, to ensure we limit it to < 2GB.
@@ -336,9 +425,53 @@ public:
         return Status::OK();
     }
 
+    /* Assumes the root level is a single literal element (which may contain nested objects).
+     * Only validates up to the termination of that first literal, more data is permitted to
+     * remain in the buffer after that and is not validated. Throws exception on invalid data.
+     * Confirm field names for literals in BSONColumn have empty field names.
+     */
+    int validateAndMeasureElem() {
+        setupValidation();
+        uassert(InvalidBSON,
+                "BSON literal is not followed by fieldname",
+                _maxLength > 1);  // must at least have a 0-terminator after control
+        // Confirm fieldName is just a null terminator
+        uassert(NonConformantBSON,
+                "BSON literal content does not have an empty fieldname",
+                _maxLength > 1 && _data[1] == 0);
+
+        // Handle one element without using iterative loop, and without expecting
+        // multiple instances or an EOO.  Only resume with the iterative loop if
+        // we have nested objects
+        _currElem = _data;
+        const char* ptr = _validateElem<false>(Cursor{_data + 2, _data + _maxLength}, *_data);
+        _validator.checkNonConformantElem(_data, 2, *_data);
+
+        if (_firstFrameUpdated) {
+            // We know that type was kObject/kArray/kCodeWScope
+            // Size is fieldname, type, and a stored int
+            int64_t size =
+                static_cast<int64_t>(ConstDataView(_data + 2).read<LittleEndian<int32_t>>()) + 2;
+            uassert(InvalidBSON,
+                    "BSON literal content exceeds buffer size",
+                    (size_t)size <= _maxLength);
+            _validateIterative(Cursor{ptr, _data + size});
+            return size;
+        } else {
+            return ptr - _data;
+        }
+    }
+
 private:
     struct Empty {};
 
+    void inline setupValidation() {
+        _currFrame = _frames.begin();
+        _currElem = nullptr;
+        auto maxFrames = BSONDepth::getMaxAllowableDepth() + 1;  // A flat BSON has one frame.
+        uassert(InvalidBSON, "Cannot enforce max nesting depth", _frames.size() <= maxFrames);
+    }
+
     /**
      * Extra information for each nesting level in the precise validation mode.
      */
@@ -354,6 +487,8 @@ private:
         typename std::conditional<precise, std::vector<Frame>, std::array<Frame, 32>>::type;
 
     struct Cursor {
+        /* Also requires remaining buf after the skip (both BSONColumn and BSONObj guarantee this
+           by having at minimum a trailing EOO) */
         void skip(size_t len) {
             uassert(InvalidBSON, "BSON size is larger than buffer size", (ptr += len) < end);
         }
@@ -388,9 +523,16 @@ private:
         uassert(ErrorCodes::Overflow,
                 "BSONObj exceeds maximum nested object depth",
                 ++_currFrame != _frames.end());
+        return _updateFrame(cursor);
+    }
+
+    const char* _updateFrame(Cursor cursor) {
         auto obj = cursor.ptr;
         auto len = cursor.template read<int32_t>();
-        uassert(ErrorCodes::InvalidBSON, "Nested BSON object has to be at least 5 bytes", len >= 5);
+        uassert(
+            ErrorCodes::InvalidBSON,
+            fmt::format("Nested BSON object has to be at least 5 bytes (decoded length: {})", len),
+            len >= 5);
         _currFrame->end = obj + len;
 
         if constexpr (precise) {
@@ -408,12 +550,23 @@ private:
         return true;
     }
 
-    static const char* _validateSpecial(Cursor cursor, uint8_t type) {
+    const char* _validateSpecial(Cursor cursor, uint8_t type) {
         switch (type) {
-            case BSONType::BinData:
-                cursor.skip(cursor.template read<uint32_t>());  // Like String, but...
-                cursor.skip(1);  // ...add extra skip for the subtype byte to avoid overflow.
+            case BSONType::BinData: {
+                auto count = cursor.template read<uint32_t>();
+                auto subtype = cursor.template read<uint8_t>();
+                const char* columnStart = cursor.ptr;
+                cursor.skip(count);
+                if (subtype == BinDataType::Column && _validationVersion >= V2_Column) {
+                    /* do not pass down cursor; we want to reset the nesting depth */
+                    uassert(NonConformantBSON,
+                            "Invalid BSON column",
+                            _doValidateColumn<precise>(
+                                columnStart, count, _validator.validateMode(), _validationVersion)
+                                .isOK());
+                }
                 break;
+            }
             case BSONType::Bool:
                 if (auto value = cursor.template read<uint8_t>())  // If not 0, must be 1.
                     uassert(InvalidBSON, "BSON bool is neither false nor true", value == 1);
@@ -437,10 +590,15 @@ private:
         return cursor.ptr;
     }
 
+    template <bool nestedFrame>
     const char* _pushCodeWithScope(Cursor cursor) {
-        cursor.ptr = _pushFrame(cursor);  // Push a dummy frame to check the CodeWScope size.
-        cursor.skipString();              // Now skip the BSON UTF8 string containing the code.
-        _currElem = cursor.ptr - 1;       // Use the terminating NUL as a dummy scope element.
+        // Push a dummy frame to check the CodeWScope size.
+        if constexpr (nestedFrame)
+            cursor.ptr = _pushFrame(cursor);
+        else
+            cursor.ptr = _updateFrame(cursor);
+        cursor.skipString();         // Now skip the BSON UTF8 string containing the code.
+        _currElem = cursor.ptr - 1;  // Use the terminating NUL as a dummy scope element.
         return _pushFrame(cursor);
     }
 
@@ -454,21 +612,30 @@ private:
         }
     }
 
+    template <bool nestedFrame>
     const char* _validateElem(Cursor cursor, uint8_t type) {
         if (MONGO_unlikely(type > JSTypeMax))
             return _validateSpecial(cursor, type);
 
         auto style = kTypeInfoTable[type];
-        if (MONGO_likely(style <= kSkip16))
+        if (MONGO_likely(style <= kSkip16)) {
             cursor.skip(style * 4);
-        else if (MONGO_likely(style == kString))
+        } else if (MONGO_likely(style == kString)) {
             cursor.skipString();
-        else if (MONGO_likely(style == kObjectOrArray))
-            cursor.ptr = _pushFrame(cursor);
-        else if (MONGO_unlikely(precise && type == CodeWScope))
-            cursor.ptr = _pushCodeWithScope(cursor);
-        else
+        } else if (MONGO_likely(style == kObjectOrArray)) {
+            if constexpr (nestedFrame) {
+                cursor.ptr = _pushFrame(cursor);
+            } else {
+                cursor.ptr = _updateFrame(cursor);
+                _firstFrameUpdated = true;
+            }
+        } else if (MONGO_unlikely(precise && type == CodeWScope)) {
+            cursor.ptr = _pushCodeWithScope<nestedFrame>(cursor);
+            if constexpr (!nestedFrame)
+                _firstFrameUpdated = true;
+        } else {
             cursor.ptr = _validateSpecial(cursor, type);
+        }
 
         return cursor.ptr;
     }
@@ -481,19 +648,21 @@ private:
             while (size_t len = cursor.strlen()) {
                 uint8_t type = *cursor.ptr;
                 _currElem = cursor.ptr;
+                // In case _currElem is moved (for instance when the type is CodeWScope).
+                auto elemStart = cursor.ptr;
                 cursor.ptr += len + 1;
-                cursor.ptr = _validateElem(cursor, type);
+                cursor.ptr = _validateElem<true>(cursor, type);
 
                 // Check if the data is compliant to other BSON specifications if the element is
                 // structurally correct.
-                _validator.checkNonConformantElem(_currElem, len + 1, type);
+                _validator.checkNonConformantElem(elemStart, len + 1, type);
 
                 if constexpr (precise) {
                     // See if the _id field was just validated. If so, set the global scope element.
                     if (_currFrame == _frames.begin() && StringData(_currElem + 1) == "_id"_sd)
                         _currFrame->elem = BSONElement(_currElem);  // This is fully validated now.
                 }
-                dassert(cursor.ptr <= cursor.end);
+                dassert(cursor.ptr < cursor.end);
             }
 
             // Got the EOO byte: skip it and compare its location with the expected frame end.
@@ -532,37 +701,168 @@ private:
     typename Frames::iterator _currFrame;  // Frame currently being validated.
     Frames _frames;  // Has end pointers to check and the containing element for precise mode.
     BSONValidator _validator;
+    ValidationVersion _validationVersion;
+    bool _firstFrameUpdated = false;  // Has the first frame received nested while measuring an elem
 };
 
 template <typename BSONValidator>
-Status _doValidate(const char* originalBuffer, uint64_t maxLength, BSONValidator validator) {
+Status _doValidate(const char* originalBuffer,
+                   uint64_t maxLength,
+                   BSONValidator validator,
+                   ValidationVersion validationVersion) {
     // First try validating using the fast but less precise version. That version will return
     // a not-OK status for objects with CodeWScope or nesting exceeding 32 levels. These cases and
     // actual failures will rerun the precise version that gives a detailed error context.
-    if (MONGO_likely((ValidateBuffer<false, BSONValidator>(originalBuffer, maxLength, validator)
+    if (MONGO_likely((ValidateBuffer<false, BSONValidator>(
+                          originalBuffer, maxLength, validator, validationVersion)
                           .validate()
                           .isOK())))
         return Status::OK();
 
-    return ValidateBuffer<true, BSONValidator>(originalBuffer, maxLength, validator).validate();
+    return ValidateBuffer<true, BSONValidator>(
+               originalBuffer, maxLength, validator, validationVersion)
+        .validate();
 }
+
+template <bool precise>
+class ColumnValidator {
+public:
+    static Status doValidateBSONColumn(const char* originalBuffer,
+                                       int maxLength,
+                                       BSONValidateMode mode,
+                                       ValidationVersion validationVersion) noexcept {
+        // run control pointer through to end of buffer
+        // run over literal data as directed by lengths from control
+        // check formatting of Simple8B blocks
+        // scan reference objects of interleaved mode starts
+        // confirm EOO terminations of interleaved modes
+        // content of interleaved objects does not need to be checked differently from
+        //      standard Simple8B block and literal decodings
+        // confirm we end at end of buffer
+        const char* ptr = originalBuffer;
+        const char* end = originalBuffer + maxLength;
+        bool interleavedMode = false;
+
+        try {
+            // Check this beforehand to ensure we cannot overflow the buffer with any strlen
+            uassert(NonConformantBSON,
+                    "BSON column is missing EOO termination",
+                    ptr < end && *(end - 1) == EOO);
+
+            while (ptr < end) {
+                uint8_t control = *ptr;
+                if (control == EOO) {
+                    ptr++;
+                    if (interleavedMode) {
+                        interleavedMode = false;
+                    } else {
+                        // should be the last control of the sequence
+                        uassert(NonConformantBSON,
+                                "BSONColumn EOO does not fully consume buffer",
+                                ptr == end);
+                        return Status::OK();
+                    }
+                } else if (bsoncolumn::isUncompressedLiteralControlByte(control)) {
+                    int size;
+                    if (MONGO_likely(mode == BSONValidateMode::kDefault))
+                        size = ValidateBuffer<precise, DefaultValidator>(
+                                   ptr, end - ptr, DefaultValidator(), validationVersion)
+                                   .validateAndMeasureElem();
+                    else if (mode == BSONValidateMode::kExtended)
+                        size = ValidateBuffer<precise, ExtendedValidator>(
+                                   ptr, end - ptr, ExtendedValidator(), validationVersion)
+                                   .validateAndMeasureElem();
+                    else if (mode == BSONValidateMode::kFull)
+                        size = ValidateBuffer<precise, FullValidator>(
+                                   ptr, end - ptr, FullValidator(), validationVersion)
+                                   .validateAndMeasureElem();
+                    else
+                        MONGO_UNREACHABLE;
+
+                    ptr += size;
+                } else if (bsoncolumn::isInterleavedStartControlByte(control)) {
+                    // interleaved objects begin with a reference object, and then a series
+                    // of diff blocks for followup objects, ending with an EOO. Nesting interleaved
+                    // mode is not allowed.
+                    uassert(NonConformantBSON, "Nested interleaved mode", !interleavedMode);
+                    ptr++;
+                    uassert(NonConformantBSON,
+                            "Invalid reference object for interleaved mode",
+                            validateBSON(ptr, end - ptr, mode).isOK());
+                    // we now know due to validateBSON that it is safe to interpret *ptr
+                    BSONObj reference(ptr);
+                    ptr += reference.objsize();
+                    interleavedMode = true;
+                } else {
+                    // Simple8b block sequence, just check for memory overflow of block count
+                    uint8_t numBlocks = bsoncolumn::numSimple8bBlocksForControlByte(control);
+                    int size = sizeof(uint64_t) * numBlocks;
+                    uassert(NonConformantBSON,
+                            "BSONColumn blocks exceed buffer size",
+                            ptr + size + 1 <= end);
+                    ptr += 1 + size;
+                }
+            }
+        } catch (const ExceptionForCat<ErrorCategory::ValidationError>& e) {
+            return Status(e.code(), str::stream() << e.what());
+        }
+
+        // We should not get here for a valid object, the final EOO should have returned OK
+        return Status(NonConformantBSON, "Missing terminating EOO");
+    }
+};
+
+template <bool precise>
+Status _doValidateColumn(const char* originalBuffer,
+                         uint64_t maxLength,
+                         BSONValidateMode mode,
+                         ValidationVersion validationVersion) {
+    if constexpr (precise) {
+        // First try validating using the fast but less precise version. That version will return
+        // a not-OK status for objects with CodeWScope or nesting exceeding 32 levels. These cases
+        // and actual failures will rerun the precise version that gives a detailed error context.
+        if (MONGO_likely(ColumnValidator<false>::doValidateBSONColumn(
+                             originalBuffer, maxLength, mode, validationVersion)
+                             .isOK()))
+            return Status::OK();
+
+        return ColumnValidator<true>::doValidateBSONColumn(
+            originalBuffer, maxLength, mode, validationVersion);
+    } else {
+        return ColumnValidator<false>::doValidateBSONColumn(
+            originalBuffer, maxLength, mode, validationVersion);
+    }
+}
+
 }  // namespace
 
 Status validateBSON(const char* originalBuffer,
                     uint64_t maxLength,
-                    BSONValidateMode mode) noexcept {
+                    BSONValidateMode mode,
+                    ValidationVersion validationVersion) noexcept {
     if (MONGO_likely(mode == BSONValidateMode::kDefault))
-        return _doValidate(originalBuffer, maxLength, DefaultValidator());
+        return _doValidate(originalBuffer, maxLength, DefaultValidator(), validationVersion);
     else if (mode == BSONValidateMode::kExtended)
-        return _doValidate(originalBuffer, maxLength, ExtendedValidator());
+        return _doValidate(originalBuffer, maxLength, ExtendedValidator(), validationVersion);
     else if (mode == BSONValidateMode::kFull)
-        return ValidateBuffer<true, FullValidator>(originalBuffer, maxLength, FullValidator())
+        return ValidateBuffer<true, FullValidator>(
+                   originalBuffer, maxLength, FullValidator(), validationVersion)
             .validate();
     else
         MONGO_UNREACHABLE;
 }
 
-Status validateBSON(const BSONObj& obj, BSONValidateMode mode) {
-    return validateBSON(obj.objdata(), obj.objsize(), mode);
+Status validateBSON(const BSONObj& obj,
+                    BSONValidateMode mode,
+                    ValidationVersion validationVersion) {
+    return validateBSON(obj.objdata(), obj.objsize(), mode, validationVersion);
 }
+
+Status validateBSONColumn(const char* originalBuffer,
+                          int maxLength,
+                          BSONValidateMode mode,
+                          ValidationVersion validationVersion) noexcept {
+    return _doValidateColumn<true>(originalBuffer, maxLength, mode, validationVersion);
+}
+
 }  // namespace mongo
