@@ -45,6 +45,7 @@
 #include "mongo/logv2/log.h"
 #include "mongo/platform/strnlen.h"
 #include "mongo/util/base64.h"
+#include "mongo/util/decimal_counter.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/scopeguard.h"
@@ -447,6 +448,7 @@ int BSONElement::compareElements(const BSONElement& l,
     */
 std::vector<BSONElement> BSONElement::Array() const {
     chk(mongo::Array);
+
     std::vector<BSONElement> v;
     BSONObjIterator i(Obj());
     while (i.more()) {
@@ -456,7 +458,7 @@ std::vector<BSONElement> BSONElement::Array() const {
         unsigned u;
         Status status = NumberParser{}(f, &u);
         if (status.isOK()) {
-            verify(u < 1000000);
+            MONGO_verify(u < 1000000);
             if (u >= v.size())
                 v.resize(u + 1);
             v[u] = e;
@@ -467,6 +469,23 @@ std::vector<BSONElement> BSONElement::Array() const {
     return v;
 }
 
+std::vector<BSONElement> BSONElement::ArrayVerifyIndexes() const {
+    chk(mongo::Array);
+
+    std::vector<BSONElement> v;
+    DecimalCounter<std::uint32_t> counter(0);
+    for (auto element : Obj()) {
+        auto fieldName = element.fieldNameStringData();
+        uassert(ErrorCodes::BadValue,
+                fmt::format(
+                    "Invalid array index field name: \"{}\", expected \"{}\"", fieldName, counter),
+                fieldName == counter);
+        counter++;
+        v.push_back(element);
+    }
+    return v;
+}
+
 int BSONElement::woCompare(const BSONElement& elem,
                            ComparisonRulesSet rules,
                            const StringData::ComparatorInterface* comparator) const {
@@ -612,12 +631,12 @@ BSONObj BSONElement::embeddedObjectUserCheck() const {
 }
 
 BSONObj BSONElement::embeddedObject() const {
-    verify(isABSONObj());
+    MONGO_verify(isABSONObj());
     return BSONObj(value(), BSONObj::LargeSizeTrait{});
 }
 
 BSONObj BSONElement::codeWScopeObject() const {
-    verify(type() == CodeWScope);
+    MONGO_verify(type() == CodeWScope);
     int strSizeWNull = ConstDataView(value() + 4).read<LittleEndian<int>>();
     return BSONObj(value() + 4 + 4 + strSizeWNull);
 }
@@ -686,7 +705,7 @@ MONGO_COMPILER_NOINLINE void msgAssertedBadType [[noreturn]] (const char* data)
 }
 }  // namespace
 
-int BSONElement::computeSize(int8_t type, const char* elem, int fieldNameSize) {
+int BSONElement::computeSize(int8_t type, const char* elem, int fieldNameSize, int bufSize) {
     enum SizeStyle : uint8_t {
         kFixed,         // Total size is a fixed amount + key length.
         kIntPlusFixed,  // Like Fixed, but also add in the int32 immediately following the key.
@@ -758,10 +777,23 @@ int BSONElement::computeSize(int8_t type, const char* elem, int fieldNameSize) {
 
     // RegEx is two c-strings back-to-back.
     const char* p = elem + fieldNameSize + 1;
-    size_t len1 = strlen(p);
-    p = p + len1 + 1;
-    size_t len2 = strlen(p);
-    return (len1 + 1 + len2 + 1) + fieldNameSize + 1;
+    if (bufSize == 0) {
+        size_t len1 = strlen(p);
+        p = p + len1 + 1;
+        size_t len2 = strlen(p);
+        return (len1 + 1 + len2 + 1) + fieldNameSize + 1;
+    } else {
+        int searchSize = bufSize - fieldNameSize - 1;
+        int len1 = strnlen(p, searchSize);
+        if (len1 == searchSize)
+            return -1;
+        p = p + len1 + 1;
+        searchSize -= len1 + 1;
+        int len2 = strnlen(p, searchSize);
+        if (len2 == searchSize)
+            return -1;
+        return (len1 + 1 + len2 + 1) + fieldNameSize + 1;
+    }
 }
 
 std::string BSONElement::toString(bool includeFieldName, bool full) const {
