@@ -75,10 +75,16 @@ const int BSONObjMaxUserSize = 16 * 1024 * 1024;
 */
 const int BSONObjMaxInternalSize = BSONObjMaxUserSize + (16 * 1024);
 
-const int BufferMaxSize = 64 * 1024 * 1024;
+/**
+ * Maximum size of a builder buffer and for BSONObj with BsonLargeSizeTrait. Limiting it to 27 bits
+ * because SharedBuffer::Holder might bit pack information. Setting it to 125 MB to have some
+ * wiggle room before size crosses 27 bits.
+ */
+const int BufferMaxSize = 125 * 1024 * 1024;
+static_assert(BufferMaxSize < (1 << 27));
 
 /**
- * This is the maximum size size of a buffer needed for storing a BSON object in a response message.
+ * This is the maximum size of a buffer needed for storing a BSON object in a response message.
  */
 const int kOpMsgReplyBSONBufferMaxSize = BSONObjMaxUserSize + (64 * 1024);
 
@@ -325,7 +331,7 @@ public:
         @return point to region that was skipped.  pointer may change later (on realloc), so for
         immediate use only
     */
-    char* skip(int n) {
+    char* skip(size_t n) {
         return grow(n);
     }
 
@@ -394,7 +400,7 @@ public:
     }
     void appendBuf(const void* src, size_t len) {
         if (len)
-            memcpy(grow((int)len), src, len);
+            memcpy(grow(len), src, len);
     }
 
     template <class T>
@@ -403,7 +409,7 @@ public:
     }
 
     void appendStr(StringData str, bool includeEndingNull = true) {
-        const int len = str.size() + (includeEndingNull ? 1 : 0);
+        const size_t len = str.size() + (includeEndingNull ? 1 : 0);
         str.copyTo(grow(len), includeEndingNull);
     }
 
@@ -423,8 +429,8 @@ public:
     }
 
     /* returns the pre-grow write position */
-    char* grow(int by) {
-        if (MONGO_likely(by <= _end - _nextByte)) {
+    char* grow(size_t by) {
+        if (MONGO_likely(by <= static_cast<size_t>(_end - _nextByte))) {
             char* oldNextByte = _nextByte;
             _nextByte += by;
             return oldNextByte;
@@ -506,7 +512,8 @@ protected:
         // Going beyond the maximum buffer size is not likely.
         if (MONGO_unlikely(minSize > BufferMaxSize)) {
             std::stringstream ss;
-            ss << "BufBuilder attempted to grow() to " << minSize << " bytes, past the 64MB limit.";
+            ss << "BufBuilder attempted to grow() to " << minSize << " bytes, past the "
+               << (BufferMaxSize / (1024 * 1024)) << "MB limit.";
             msgasserted(13548, ss.str().c_str());
         }
 
@@ -534,9 +541,9 @@ protected:
         } else if (MONGO_unlikely(reallocSize < 64)) {
             // The minimum allocation is 64 bytes.
             reallocSize = 64;
-        } else if (MONGO_unlikely(minSize > BufferMaxSize)) {
-            // If adding 'kBuffHolderSize' to 'minSize' pushes it beyond 'BufferMaxSize', then we'll
-            // allocate enough memory according to the 'BufferMaxSize'.
+        } else if (MONGO_unlikely(reallocSize + BufferAllocator::kBuffHolderSize > BufferMaxSize)) {
+            // If adding 'kBuffHolderSize' to 'reallocSize' pushes it beyond 'BufferMaxSize', then
+            // we'll allocate enough memory according to the 'BufferMaxSize'.
             reallocSize = BufferMaxSize + BufferAllocator::kBuffHolderSize;
         }
 
@@ -737,8 +744,8 @@ public:
         const int maxSize = 32;
         char* start = _buf.grow(maxSize);
         int z = snprintf(start, maxSize, "%.16g", x);
-        verify(z >= 0);
-        verify(z < maxSize);
+        MONGO_verify(z >= 0);
+        MONGO_verify(z < maxSize);
         _buf.setlen(prev + z);
         if (strchr(start, '.') == nullptr && strchr(start, 'E') == nullptr &&
             strchr(start, 'N') == nullptr) {
@@ -805,8 +812,8 @@ private:
     StringBuilderImpl& SBNUM(T val, int maxSize, const char* macro) {
         int prev = _buf.len();
         int z = snprintf(_buf.grow(maxSize), maxSize, macro, (val));
-        verify(z >= 0);
-        verify(z < maxSize);
+        MONGO_verify(z >= 0);
+        MONGO_verify(z < maxSize);
         _buf.setlen(prev + z);
         return *this;
     }
