@@ -39,8 +39,11 @@
 #include "mongo/client/sasl_client_authenticate.h"
 #include "mongo/config.h"
 #include "mongo/db/auth/sasl_command_constants.h"
+#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/commands/test_commands_enabled.h"
+#include "mongo/db/connection_health_metrics_parameter_gen.h"
 #include "mongo/db/dbmessage.h"
+#include "mongo/db/server_feature_flags_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/egress_tag_closer_manager.h"
@@ -55,11 +58,21 @@
 #include "mongo/util/net/ssl_peer_info.h"
 #include "mongo/util/version.h"
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 namespace mongo {
 MONGO_FAIL_POINT_DEFINE(pauseBeforeMarkKeepOpen);
+MONGO_FAIL_POINT_DEFINE(alwaysLogConnAcquisitionToWireTime)
+
+namespace {
+bool connHealthMetricsEnabled() {
+    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+    return gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe();
+}
+CounterMetric totalTimeForEgressConnectionAcquiredToWireMicros(
+    "network.totalTimeForEgressConnectionAcquiredToWireMicros", connHealthMetricsEnabled);
+}  // namespace
 
 Future<AsyncDBClient::Handle> AsyncDBClient::connect(
     const HostAndPort& peer,
@@ -73,22 +86,22 @@ Future<AsyncDBClient::Handle> AsyncDBClient::connect(
     return tl
         ->asyncConnect(
             peer, sslMode, std::move(reactor), timeout, connectionMetrics, transientSSLContext)
-        .then([peer, context](transport::SessionHandle session) {
+        .then([peer, context](std::shared_ptr<transport::Session> session) {
             return std::make_shared<AsyncDBClient>(peer, std::move(session), context);
         });
 }
 
-BSONObj AsyncDBClient::_buildIsMasterRequest(const std::string& appName,
-                                             executor::NetworkConnectionHook* hook) {
+BSONObj AsyncDBClient::_buildHelloRequest(const std::string& appName,
+                                          executor::NetworkConnectionHook* hook) {
     BSONObjBuilder bob;
 
-    bob.append("isMaster", 1);
+    bob.append("hello", 1);
 
     const auto versionString = VersionInfoInterface::instance().version();
     ClientMetadata::serialize(appName, versionString, &bob);
 
     if (getTestCommandsEnabled()) {
-        // Only include the host:port of this process in the isMaster command request if test
+        // Only include the host:port of this process in the "hello" command request if test
         // commands are enabled. mongobridge uses this field to identify the process opening a
         // connection to it.
         StringBuilder sb;
@@ -103,16 +116,16 @@ BSONObj AsyncDBClient::_buildIsMasterRequest(const std::string& appName,
     }
 
     if (hook) {
-        return hook->augmentIsMasterRequest(remote(), bob.obj());
+        return hook->augmentHelloRequest(remote(), bob.obj());
     } else {
         return bob.obj();
     }
 }
 
-void AsyncDBClient::_parseIsMasterResponse(BSONObj request,
-                                           const std::unique_ptr<rpc::ReplyInterface>& response) {
+void AsyncDBClient::_parseHelloResponse(BSONObj request,
+                                        const std::unique_ptr<rpc::ReplyInterface>& response) {
     uassert(50786,
-            "Expected OP_MSG response to isMaster",
+            "Expected OP_MSG response to 'hello'",
             response->getProtocol() == rpc::Protocol::kOpMsg);
     auto wireSpec = WireSpec::instance().get();
     auto responseBody = response->getCommandReply();
@@ -166,9 +179,9 @@ Future<void> AsyncDBClient::authenticate(const BSONObj& params) {
     // We will only have a valid clientName if SSL is enabled.
     std::string clientName;
 #ifdef MONGO_CONFIG_SSL
-    auto sslConfiguration = _session->getSSLConfiguration();
-    if (sslConfiguration) {
-        clientName = sslConfiguration->clientSubjectName.toString();
+    auto& sslManager = _session->getSSLManager();
+    if (sslManager) {
+        clientName = sslManager->getSSLConfiguration().clientSubjectName.toString();
     }
 #endif
 
@@ -185,9 +198,9 @@ Future<void> AsyncDBClient::authenticateInternal(
     // We will only have a valid clientName if SSL is enabled.
     std::string clientName;
 #ifdef MONGO_CONFIG_SSL
-    auto sslConfiguration = _session->getSSLConfiguration();
-    if (sslConfiguration) {
-        clientName = sslConfiguration->clientSubjectName.toString();
+    auto& sslManager = _session->getSSLManager();
+    if (sslManager) {
+        clientName = sslManager->getSSLConfiguration().clientSubjectName.toString();
     }
 #endif
 
@@ -211,7 +224,7 @@ Future<bool> AsyncDBClient::completeSpeculativeAuth(std::shared_ptr<SaslClientSe
 
     if (speculativeAuthType == auth::SpeculativeAuthType::kNone) {
         return Status(ErrorCodes::BadValue,
-                      str::stream() << "Received unexpected isMaster."
+                      str::stream() << "Received unexpected hello."
                                     << auth::kSpeculativeAuthenticate << " reply");
     }
 
@@ -237,7 +250,7 @@ Future<bool> AsyncDBClient::completeSpeculativeAuth(std::shared_ptr<SaslClientSe
 
 Future<void> AsyncDBClient::initWireVersion(const std::string& appName,
                                             executor::NetworkConnectionHook* const hook) {
-    auto requestObj = _buildIsMasterRequest(appName, hook);
+    auto requestObj = _buildHelloRequest(appName, hook);
     auto opMsgRequest = OpMsgRequest::fromDBAndBody("admin", requestObj);
 
     auto msgId = nextMessageId();
@@ -245,7 +258,7 @@ Future<void> AsyncDBClient::initWireVersion(const std::string& appName,
         .then([msgId, this]() { return _waitForResponse(msgId); })
         .then([this, requestObj, hook, timer = Timer{}](Message response) {
             auto cmdReply = rpc::makeReply(&response);
-            _parseIsMasterResponse(requestObj, cmdReply);
+            _parseHelloResponse(requestObj, cmdReply);
             if (hook) {
                 executor::RemoteCommandResponse cmdResp(*cmdReply, timer.elapsed());
                 uassertStatusOK(hook->validateHost(_peer, requestObj, std::move(cmdResp)));
@@ -263,7 +276,7 @@ Future<void> AsyncDBClient::_call(Message request, int32_t msgId, const BatonHan
     request.header().setId(msgId);
     request.header().setResponseToMsgId(0);
 #ifdef MONGO_CONFIG_SSL
-    if (!SSLPeerInfo::forSession(_session).isTLS) {
+    if (!SSLPeerInfo::forSession(_session).isTLS()) {
         OpMsg::appendChecksum(&request);
     }
 #else
@@ -288,18 +301,42 @@ Future<Message> AsyncDBClient::_waitForResponse(boost::optional<int32_t> msgId,
         });
 }
 
-Future<rpc::UniqueReply> AsyncDBClient::runCommand(OpMsgRequest request,
-                                                   const BatonHandle& baton,
-                                                   bool fireAndForget) {
+Future<rpc::UniqueReply> AsyncDBClient::runCommand(
+    OpMsgRequest request,
+    const BatonHandle& baton,
+    bool fireAndForget,
+    boost::optional<std::shared_ptr<Timer>> fromConnAcquiredTimer) {
     auto requestMsg = request.serialize();
     if (fireAndForget) {
         OpMsg::setFlag(&requestMsg, OpMsg::kMoreToCome);
     }
     auto msgId = nextMessageId();
     auto future = _call(std::move(requestMsg), msgId, baton);
+    auto logMetrics = [this, fromConnAcquiredTimer] {
+        if (fromConnAcquiredTimer) {
+            const auto timeElapsedMicros =
+                durationCount<Microseconds>(fromConnAcquiredTimer.get()->elapsed());
+            totalTimeForEgressConnectionAcquiredToWireMicros.increment(timeElapsedMicros);
+
+            if ((!gEnableDetailedConnectionHealthMetricLogLines || timeElapsedMicros < 1000) &&
+                !MONGO_unlikely(alwaysLogConnAcquisitionToWireTime.shouldFail())) {
+                return;
+            }
+
+            // Log slow acquisition times at info level but rate limit it to prevent spamming
+            // users.
+            static auto& logSeverity = *new logv2::SeveritySuppressor{
+                Seconds{1}, logv2::LogSeverity::Info(), logv2::LogSeverity::Debug(2)};
+            LOGV2_DEBUG(6496702,
+                        logSeverity().toInt(),
+                        "Acquired connection for remote operation and completed writing to wire",
+                        "durationMicros"_attr = timeElapsedMicros);
+        }
+    };
 
     if (fireAndForget) {
-        return std::move(future).then([msgId, this]() -> Future<rpc::UniqueReply> {
+        return std::move(future).then([msgId, logMetrics, this]() -> Future<rpc::UniqueReply> {
+            logMetrics();
             // Return a mock status OK response since we do not expect a real response.
             OpMsgBuilder builder;
             builder.setBody(BSON("ok" << 1));
@@ -311,19 +348,25 @@ Future<rpc::UniqueReply> AsyncDBClient::runCommand(OpMsgRequest request,
     }
 
     return std::move(future)
-        .then([msgId, baton, this]() { return _waitForResponse(msgId, baton); })
+        .then([msgId, logMetrics, baton, this]() {
+            logMetrics();
+            return _waitForResponse(msgId, baton);
+        })
         .then([this](Message response) -> Future<rpc::UniqueReply> {
             return rpc::UniqueReply(response, rpc::makeReply(&response));
         });
 }
 
 Future<executor::RemoteCommandResponse> AsyncDBClient::runCommandRequest(
-    executor::RemoteCommandRequest request, const BatonHandle& baton) {
+    executor::RemoteCommandRequest request,
+    const BatonHandle& baton,
+    boost::optional<std::shared_ptr<Timer>> fromConnAcquiredTimer) {
     auto startTimer = Timer();
     auto opMsgRequest = OpMsgRequest::fromDBAndBody(
         std::move(request.dbname), std::move(request.cmdObj), std::move(request.metadata));
     opMsgRequest.validatedTenancyScope = request.validatedTenancyScope;
-    return runCommand(std::move(opMsgRequest), baton, request.options.fireAndForget)
+    return runCommand(
+               std::move(opMsgRequest), baton, request.options.fireAndForget, fromConnAcquiredTimer)
         .then([this, startTimer = std::move(startTimer)](rpc::UniqueReply response) {
             return executor::RemoteCommandResponse(*response, startTimer.elapsed());
         });
