@@ -32,6 +32,7 @@
 #include "mongo/client/cyrus_sasl_client_session.h"
 
 #include "mongo/base/init.h"
+#include "mongo/client/authenticate.h"
 #include "mongo/client/native_sasl_client_session.h"
 #include "mongo/util/allocator.h"
 #include "mongo/util/assert_util.h"
@@ -47,7 +48,8 @@ void saslSetError(sasl_conn_t* conn, const std::string& msg) {
 }
 
 SaslClientSession* createCyrusSaslClientSession(const std::string& mech) {
-    if ((mech == "SCRAM-SHA-1") || (mech == "SCRAM-SHA-256") || mech == "MONGODB-AWS") {
+    if ((mech == auth::kMechanismScramSha1) || (mech == auth::kMechanismScramSha256) ||
+        (mech == auth::kMechanismMongoAWS) || (mech == auth::kMechanismMongoOIDC)) {
         return new NativeSaslClientSession();
     }
     return new CyrusSaslClientSession();
@@ -116,10 +118,32 @@ MONGO_INITIALIZER(CyrusSaslAllocatorsAndMutexes)(InitializerContext*) {
     sasl_set_mutex(saslMutexAlloc, saslMutexLock, saslMutexUnlock, saslMutexFree);
 }
 
-int saslClientLogSwallow(void* context, int priority, const char* message) throw() {
+int saslClientLogSwallow(void* context, int priority, const char* message) noexcept {
     return SASL_OK;  // do nothing
 }
 
+/**
+ * Implements the Cyrus SASL default_verifyfile_cb interface registered in the
+ * Cyrus SASL library to verify, and then accept or reject, the loading of
+ * plugin libraries from the target directory.
+ *
+ * On Windows environments, disable loading of plugin files.
+ */
+int saslClientVerifyPluginFile(void*, const char*, sasl_verify_type_t type) {
+
+    if (type != SASL_VRFY_PLUGIN) {
+        return SASL_OK;
+    }
+
+#ifdef _WIN32
+    return SASL_CONTINUE;  // A non-SASL_OK response indicates to Cyrus SASL that it
+                           // should not load a file. This effectively disables
+                           // loading plugins from path on Windows.
+#else
+    return SASL_OK;
+#endif
+}
+
 /**
  * Initializes the client half of the SASL library, but is effectively a no-op if the client
  * application has already done it.
@@ -136,6 +160,7 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(CyrusSaslClientContext,
 (InitializerContext* context) {
     static sasl_callback_t saslClientGlobalCallbacks[] = {
         {SASL_CB_LOG, SaslCallbackFn(saslClientLogSwallow), nullptr /* context */},
+        {SASL_CB_VERIFYFILE, SaslCallbackFn(saslClientVerifyPluginFile), nullptr /*context*/},
         {SASL_CB_LIST_END}};
 
     // If the client application has previously called sasl_client_init(), the callbacks passed
@@ -159,7 +184,7 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(CyrusSaslClientContext,
  * Note that in Mongo, the authentication and authorization ids (authid and authzid) are always
  * the same.  These correspond to SASL_CB_AUTHNAME and SASL_CB_USER.
  */
-int saslClientGetSimple(void* context, int id, const char** result, unsigned* resultLen) throw() {
+int saslClientGetSimple(void* context, int id, const char** result, unsigned* resultLen) noexcept {
     try {
         CyrusSaslClientSession* session = static_cast<CyrusSaslClientSession*>(context);
         if (!session || !result)
@@ -194,7 +219,7 @@ int saslClientGetSimple(void* context, int id, const char** result, unsigned* re
 int saslClientGetPassword(sasl_conn_t* conn,
                           void* context,
                           int id,
-                          sasl_secret_t** outSecret) throw() {
+                          sasl_secret_t** outSecret) noexcept {
     try {
         CyrusSaslClientSession* session = static_cast<CyrusSaslClientSession*>(context);
         if (!session || !outSecret)
