@@ -33,7 +33,6 @@
 #include <utility>
 
 #include "mongo/bson/util/builder.h"
-#include "mongo/client/client_deprecated.h"
 #include "mongo/client/connpool.h"
 #include "mongo/client/dbclient_cursor.h"
 #include "mongo/client/global_conn_pool.h"
@@ -212,7 +211,7 @@ bool _isSecondaryCommand(StringData commandName, const BSONObj& commandArgs) {
 }
 
 // Internal implementation of isSecondaryQuery, takes previously-parsed read preference
-bool _isSecondaryQuery(const string& ns,
+bool _isSecondaryQuery(StringData ns,
                        const BSONObj& filter,
                        const ReadPreferenceSetting& readPref) {
     // If the read pref is primary only, this is not a secondary query
@@ -276,6 +275,7 @@ DBClientConnection* DBClientReplicaSet::checkPrimary() {
 
     resetPrimary();
 
+    newConn->setShouldThrowOnStaleConfigError(_shouldThrowOnStaleConfigError);
     _primaryHost = h;
     _primary.reset(newConn);
     _primary->setParentReplSetName(_setName);
@@ -471,40 +471,40 @@ void DBClientReplicaSet::logout(const string& dbname, BSONObj& info) {
             _lastSecondaryOkConn->logout(dbname, dummy);
         } catch (const DBException&) {
             // Make sure we can't use this connection again.
-            verify(_lastSecondaryOkConn->isFailed());
+            MONGO_verify(_lastSecondaryOkConn->isFailed());
         }
     }
 }
 
 // ------------- simple functions -----------------
 
-void DBClientReplicaSet::insert(const string& ns,
+void DBClientReplicaSet::insert(const NamespaceString& nss,
                                 BSONObj obj,
                                 bool ordered,
                                 boost::optional<BSONObj> writeConcernObj) {
-    checkPrimary()->insert(ns, obj, ordered, writeConcernObj);
+    checkPrimary()->insert(nss, obj, ordered, writeConcernObj);
 }
 
-void DBClientReplicaSet::insert(const string& ns,
+void DBClientReplicaSet::insert(const NamespaceString& nss,
                                 const vector<BSONObj>& v,
                                 bool ordered,
                                 boost::optional<BSONObj> writeConcernObj) {
-    checkPrimary()->insert(ns, v, ordered, writeConcernObj);
+    checkPrimary()->insert(nss, v, ordered, writeConcernObj);
 }
 
-void DBClientReplicaSet::remove(const string& ns,
+void DBClientReplicaSet::remove(const NamespaceString& nss,
                                 const BSONObj& filter,
                                 bool removeMany,
                                 boost::optional<BSONObj> writeConcernObj) {
-    checkPrimary()->remove(ns, filter, removeMany, writeConcernObj);
+    checkPrimary()->remove(nss, filter, removeMany, writeConcernObj);
 }
 
 std::unique_ptr<DBClientCursor> DBClientReplicaSet::find(FindCommandRequest findRequest,
                                                          const ReadPreferenceSetting& readPref,
                                                          ExhaustMode exhaustMode) {
-    invariant(findRequest.getNamespaceOrUUID().nss());
-    const std::string nss = findRequest.getNamespaceOrUUID().nss()->ns();
-    if (_isSecondaryQuery(nss, findRequest.toBSON(BSONObj{}), readPref)) {
+    invariant(findRequest.getNamespaceOrUUID().isNamespaceString());
+    if (_isSecondaryQuery(
+            findRequest.getNamespaceOrUUID().nss().ns(), findRequest.toBSON(BSONObj{}), readPref)) {
         LOGV2_DEBUG(5951202,
                     3,
                     "dbclient_rs query using secondary or tagged node selection",
@@ -559,7 +559,7 @@ void DBClientReplicaSet::killCursor(const NamespaceString& ns, long long cursorI
     // since we don't know which server it belongs to
     // can't assume primary because of secondary ok
     // and can have a cursor survive a primary change
-    verify(0);
+    MONGO_verify(0);
 }
 
 void DBClientReplicaSet::isNotPrimary() {
@@ -674,6 +674,7 @@ DBClientConnection* DBClientReplicaSet::selectNodeUsingTags(
             str::stream() << "Failed to connect to " << _lastSecondaryOkHost.toString(),
             newConn != nullptr);
 
+    newConn->setShouldThrowOnStaleConfigError(_shouldThrowOnStaleConfigError);
     _lastSecondaryOkConn = std::shared_ptr<DBClientConnection>(newConn, std::move(dtor));
     _lastSecondaryOkConn->setParentReplSetName(_setName);
     _lastSecondaryOkConn->setRequestMetadataWriter(getRequestMetadataWriter());
@@ -715,7 +716,7 @@ void DBClientReplicaSet::say(Message& toSend, bool isRetry, string* actualServer
 }
 
 Status DBClientReplicaSet::recv(Message& m, int lastRequestId) {
-    verify(_lastClient);
+    MONGO_verify(_lastClient);
 
     try {
         return _lastClient->recv(m, lastRequestId);
