@@ -85,13 +85,13 @@ public:
         for (auto& response : bsonResponses) {
             const auto pair = response.Array();
             const auto address = HostAndPort(pair[0].String());
-            const auto bsonIsMaster = pair[1].Obj();
+            const auto bsonHello = pair[1].Obj();
 
-            if (bsonIsMaster.nFields() == 0) {
-                _isMasterResponses.push_back(HelloOutcome(address, BSONObj(), "network error"));
+            if (bsonHello.nFields() == 0) {
+                _helloResponses.push_back(HelloOutcome(address, BSONObj(), "network error"));
             } else {
-                _isMasterResponses.push_back(
-                    HelloOutcome(address, bsonIsMaster, duration_cast<HelloRTT>(kLatency)));
+                _helloResponses.push_back(
+                    HelloOutcome(address, bsonHello, duration_cast<HelloRTT>(kLatency)));
             }
         }
         _topologyOutcome = phase["outcome"].Obj();
@@ -112,7 +112,7 @@ public:
     PhaseResult execute(TopologyManager& topology) const {
         PhaseResult testResult{{}, _phaseNum};
 
-        for (auto response : _isMasterResponses) {
+        for (const auto& response : _helloResponses) {
             auto descriptionStr =
                 (response.getResponse()) ? response.getResponse()->toString() : "[ Network Error ]";
             LOGV2(20202,
@@ -183,91 +183,98 @@ private:
 
         std::string fieldName = expectedField.fieldName();
         if (fieldName == "type") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() {
-                                      auto status = parseServerType(expectedField.String());
-                                      if (!status.isOK()) {
-                                          auto errorDescription = std::make_pair(
-                                              serverDescriptionFieldName(serverDescription, "type"),
-                                              status.getStatus().toString());
-                                          result->errorDescriptions.push_back(errorDescription);
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() {
+                    auto status = parseServerType(expectedField.String());
+                    if (!status.isOK()) {
+                        auto errorDescription =
+                            std::make_pair(serverDescriptionFieldName(serverDescription, "type"),
+                                           status.getStatus().toString());
+                        result->errorDescriptions.push_back(errorDescription);
 
-                                          // return the actual value since we already have reported
-                                          // an error about the parsed server type from the json
-                                          // file.
-                                          return serverDescription->getType();
-                                      }
-                                      return status.getValue();
-                                  },
-                                  serverDescription->getType());
+                        // return the actual value since we already have reported
+                        // an error about the parsed server type from the json
+                        // file.
+                        return serverDescription->getType();
+                    }
+                    return status.getValue();
+                },
+                serverDescription->getType());
 
         } else if (fieldName == "setName") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() {
-                                      boost::optional<std::string> result;
-                                      if (expectedField.type() != BSONType::jstNULL) {
-                                          result = expectedField.String();
-                                      }
-                                      return result;
-                                  },
-                                  serverDescription->getSetName());
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() {
+                    boost::optional<std::string> result;
+                    if (expectedField.type() != BSONType::jstNULL) {
+                        result = expectedField.String();
+                    }
+                    return result;
+                },
+                serverDescription->getSetName());
 
         } else if (fieldName == "setVersion") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() {
-                                      boost::optional<int> result;
-                                      if (expectedField.type() != BSONType::jstNULL) {
-                                          result = expectedField.numberInt();
-                                      }
-                                      return result;
-                                  },
-                                  serverDescription->getElectionIdSetVersionPair().setVersion);
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() {
+                    boost::optional<int> result;
+                    if (expectedField.type() != BSONType::jstNULL) {
+                        result = expectedField.numberInt();
+                    }
+                    return result;
+                },
+                serverDescription->getElectionIdSetVersionPair().setVersion);
 
         } else if (fieldName == "electionId") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() {
-                                      boost::optional<OID> result;
-                                      if (expectedField.type() != BSONType::jstNULL) {
-                                          result = expectedField.OID();
-                                      }
-                                      return result;
-                                  },
-                                  serverDescription->getElectionIdSetVersionPair().electionId);
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() {
+                    boost::optional<OID> result;
+                    if (expectedField.type() != BSONType::jstNULL) {
+                        result = expectedField.OID();
+                    }
+                    return result;
+                },
+                serverDescription->getElectionIdSetVersionPair().electionId);
 
         } else if (fieldName == "logicalSessionTimeoutMinutes") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() {
-                                      boost::optional<int> result;
-                                      if (expectedField.type() != BSONType::jstNULL) {
-                                          result = expectedField.numberInt();
-                                      }
-                                      return result;
-                                  },
-                                  serverDescription->getLogicalSessionTimeoutMinutes());
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() {
+                    boost::optional<int> result;
+                    if (expectedField.type() != BSONType::jstNULL) {
+                        result = expectedField.numberInt();
+                    }
+                    return result;
+                },
+                serverDescription->getLogicalSessionTimeoutMinutes());
 
         } else if (fieldName == "minWireVersion") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() { return expectedField.numberInt(); },
-                                  serverDescription->getMinWireVersion());
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() { return expectedField.numberInt(); },
+                serverDescription->getMinWireVersion());
 
         } else if (fieldName == "maxWireVersion") {
-            doValidateServerField(result,
-                                  serverDescription,
-                                  fieldName,
-                                  [&]() { return expectedField.numberInt(); },
-                                  serverDescription->getMaxWireVersion());
+            doValidateServerField(
+                result,
+                serverDescription,
+                fieldName,
+                [&]() { return expectedField.numberInt(); },
+                serverDescription->getMaxWireVersion());
 
         } else {
             MONGO_UNREACHABLE;
@@ -339,18 +346,18 @@ private:
 
         {
             constexpr auto fieldName = "setName";
-            doValidateTopologyDescriptionField(result,
-                                               fieldName,
-                                               [&]() {
-                                                   boost::optional<std::string> ret;
-                                                   auto bsonField =
-                                                       bsonTopologyDescription[fieldName];
-                                                   if (!bsonField.isNull()) {
-                                                       ret = bsonField.String();
-                                                   }
-                                                   return ret;
-                                               },
-                                               topologyDescription->getSetName());
+            doValidateTopologyDescriptionField(
+                result,
+                fieldName,
+                [&]() {
+                    boost::optional<std::string> ret;
+                    auto bsonField = bsonTopologyDescription[fieldName];
+                    if (!bsonField.isNull()) {
+                        ret = bsonField.String();
+                    }
+                    return ret;
+                },
+                topologyDescription->getSetName());
         }
 
         {
@@ -422,7 +429,7 @@ private:
 
     MongoURI _testUri;
     int _phaseNum;
-    std::vector<HelloOutcome> _isMasterResponses;
+    std::vector<HelloOutcome> _helloResponses;
     BSONObj _topologyOutcome;
 };
 
@@ -542,7 +549,7 @@ public:
     std::vector<JsonTestCase::TestCaseResult> runTests() {
         std::vector<JsonTestCase::TestCaseResult> results;
         const auto testFiles = getTestFiles();
-        for (auto jsonTest : testFiles) {
+        for (const auto& jsonTest : testFiles) {
             auto testCase = JsonTestCase(jsonTest);
             try {
                 LOGV2(20208, "### Executing Test Case ###", "test"_attr = testCase.Name());
