@@ -30,7 +30,6 @@
 #pragma once
 
 #include <boost/optional.hpp>
-#include <memory>
 
 #include "mongo/base/status.h"
 #include "mongo/bson/oid.h"
@@ -106,14 +105,6 @@ public:
     static constexpr StringData V1_USER_NAME_FIELD_NAME = "user"_sd;
     static constexpr StringData V1_USER_SOURCE_FIELD_NAME = "userSource"_sd;
 
-    static const NamespaceString adminCommandNamespace;
-    static const NamespaceString rolesCollectionNamespace;
-    static const NamespaceString usersBackupCollectionNamespace;
-    static const NamespaceString usersCollectionNamespace;
-    static const NamespaceString versionCollectionNamespace;
-    static const NamespaceString defaultTempUsersCollectionNamespace;  // for mongorestore
-    static const NamespaceString defaultTempRolesCollectionNamespace;  // for mongorestore
-
     /**
      * Status to be returned when authentication fails. Being consistent about our returned Status
      * prevents information leakage.
@@ -186,6 +177,11 @@ public:
      */
     virtual bool isAuthEnabled() const = 0;
 
+    /**
+     * Returns whether a schema version document exists.
+     */
+    virtual Status hasValidAuthSchemaVersionDocumentForInitialSync(OperationContext* opCtx) = 0;
+
     /**
      * Returns via the output parameter "version" the version number of the authorization
      * system.  Returns Status::OK() if it was able to successfully fetch the current
@@ -222,6 +218,12 @@ public:
                                       const UserName& userName,
                                       BSONObj* result) = 0;
 
+    /**
+     * Returns true if there exists at least one user document in the system. If `tenantId` is set,
+     * only looks for users associated with `tenantId`.
+     */
+    virtual bool hasUser(OperationContext* opCtx, const boost::optional<TenantId>& tenantId) = 0;
+
     /**
      * Delegates method call to the underlying AuthzManagerExternalState.
      */
@@ -305,14 +307,14 @@ public:
                                             std::vector<BSONObj>* result) = 0;
 
     /**
-     * Returns a Status or UserHandle for the given userName. If the user cache already has a
+     * Returns a Status or UserHandle for the given userRequest. If the user cache already has a
      * user object for this user, it returns a handle from the cache, otherwise it reads the
-     * user document from disk or LDAP - this may block for a long time.
+     * user document from the AuthzManagerExternalState - this may block for a long time.
      *
      * The returned user may be invalid by the time the caller gets access to it.
      */
     virtual StatusWith<UserHandle> acquireUser(OperationContext* opCtx,
-                                               const UserName& userName) = 0;
+                                               const UserRequest& userRequest) = 0;
 
     /**
      * Validate the ID associated with a known user while refreshing session cache.
@@ -356,13 +358,6 @@ public:
      */
     virtual void invalidateUserCache(OperationContext* opCtx) = 0;
 
-    /**
-     * Sets the list of users that should be pinned in memory.
-     *
-     * This will start the PinnedUserTracker thread if it hasn't been started already.
-     */
-    virtual void updatePinnedUsersList(std::vector<UserName> names) = 0;
-
     /**
      * Hook called by replication code to let the AuthorizationManager observe changes
      * to relevant collections.
