@@ -31,6 +31,7 @@
 
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/privilege.h"
+#include "mongo/db/auth/user.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
@@ -67,12 +68,16 @@ public:
 
     OID getCacheGeneration() override;
 
+    Status hasValidAuthSchemaVersionDocumentForInitialSync(OperationContext* opCtx) override;
+
     bool hasAnyPrivilegeDocuments(OperationContext* opCtx) override;
 
     Status getUserDescription(OperationContext* opCtx,
                               const UserName& userName,
                               BSONObj* result) override;
 
+    bool hasUser(OperationContext* opCtx, const boost::optional<TenantId>& tenantId) override;
+
     Status rolesExist(OperationContext* opCtx, const std::vector<RoleName>& roleNames) override;
 
     StatusWith<ResolvedRoleData> resolveRoles(OperationContext* opCtx,
@@ -97,7 +102,8 @@ public:
                                     bool showBuiltinRoles,
                                     std::vector<BSONObj>* result) override;
 
-    StatusWith<UserHandle> acquireUser(OperationContext* opCtx, const UserName& userName) override;
+    StatusWith<UserHandle> acquireUser(OperationContext* opCtx,
+                                       const UserRequest& userRequest) override;
     StatusWith<UserHandle> reacquireUser(OperationContext* opCtx, const UserHandle& user) override;
 
     /**
@@ -123,8 +129,6 @@ public:
      */
     void invalidateUserCache(OperationContext* opCtx) override;
 
-    void updatePinnedUsersList(std::vector<UserName> names) override;
-
     void logOp(OperationContext* opCtx,
                StringData opstr,
                const NamespaceString& nss,
@@ -136,8 +140,6 @@ public:
 private:
     void _updateCacheGeneration();
 
-    void _pinnedUsersThreadRoutine() noexcept;
-
     std::unique_ptr<AuthzManagerExternalState> _externalState;
 
     // True if AuthSchema startup checks should be applied in this AuthorizationManager. Changes to
@@ -210,11 +212,6 @@ private:
     // Thread pool on which to perform the blocking activities that load the user credentials from
     // storage
     ThreadPool _threadPool;
-
-    Mutex _pinnedUsersMutex = MONGO_MAKE_LATCH("AuthorizationManagerImpl::_pinnedUsersMutex");
-    stdx::condition_variable _pinnedUsersCond;
-    std::once_flag _pinnedThreadTrackerStarted;
-    boost::optional<std::vector<UserName>> _usersToPin;
 };
 
 extern int authorizationManagerCacheSize;
