@@ -32,7 +32,6 @@
 #include <memory>
 
 #include "mongo/base/status.h"
-#include "mongo/bson/mutable/document.h"
 #include "mongo/config.h"
 #include "mongo/crypto/mechanism_scram.h"
 #include "mongo/crypto/sha1_block.h"
@@ -53,7 +52,7 @@
 #include "mongo/transport/session.h"
 #include "mongo/transport/transport_layer_mock.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/map_util.h"
+#include "mongo/util/net/ssl_peer_info.h"
 
 #define ASSERT_NULL(EXPR) ASSERT_FALSE(EXPR)
 #define ASSERT_NON_NULL(EXPR) ASSERT_TRUE(EXPR)
@@ -68,7 +67,7 @@ SSLX509Name buildX509Name() {
         {{{kOID_CommonName.toString(), 19 /* Printable String */, "mongodb.com"}}}));
 }
 
-void setX509PeerInfo(const transport::SessionHandle& session, SSLPeerInfo info) {
+void setX509PeerInfo(const std::shared_ptr<transport::Session>& session, SSLPeerInfo info) {
     auto& sslPeerInfo = SSLPeerInfo::forSession(session);
     sslPeerInfo = info;
 }
@@ -83,7 +82,6 @@ public:
         auto localAuthzManager = std::make_unique<AuthorizationManagerImpl>(
             getServiceContext(), std::move(localExternalState));
         authzManager = localAuthzManager.get();
-        externalState->setAuthorizationManager(authzManager);
         authzManager->setAuthEnabled(true);
         AuthorizationManager::set(getServiceContext(), std::move(localAuthzManager));
 
@@ -107,7 +105,7 @@ public:
     }
 
     transport::TransportLayerMock transportLayer;
-    transport::SessionHandle session = transportLayer.createSession();
+    std::shared_ptr<transport::Session> session = transportLayer.createSession();
     AuthorizationManager* authzManager;
     AuthzManagerExternalStateMock* externalState;
     BSONObj credentials;
@@ -142,7 +140,7 @@ TEST_F(AuthorizationManagerTest, testAcquireV2User) {
                                                                              << "admin"))),
                                                      BSONObj()));
 
-    auto swu = authzManager->acquireUser(opCtx.get(), UserName("v2read", "test"));
+    auto swu = authzManager->acquireUser(opCtx.get(), {{"v2read", "test"}, boost::none});
     ASSERT_OK(swu.getStatus());
     auto v2read = std::move(swu.getValue());
     ASSERT_EQUALS(UserName("v2read", "test"), v2read->getName());
@@ -155,7 +153,7 @@ TEST_F(AuthorizationManagerTest, testAcquireV2User) {
     ASSERT(testDBPrivilege.getActions().contains(ActionType::find));
     // Make sure user's refCount is 0 at the end of the test to avoid an assertion failure
 
-    swu = authzManager->acquireUser(opCtx.get(), UserName("v2cluster", "admin"));
+    swu = authzManager->acquireUser(opCtx.get(), {{"v2cluster", "admin"}, boost::none});
     ASSERT_OK(swu.getStatus());
     auto v2cluster = std::move(swu.getValue());
     ASSERT_EQUALS(UserName("v2cluster", "admin"), v2cluster->getName());
@@ -171,24 +169,19 @@ TEST_F(AuthorizationManagerTest, testAcquireV2User) {
 
 #ifdef MONGO_CONFIG_SSL
 TEST_F(AuthorizationManagerTest, testLocalX509Authorization) {
-    setX509PeerInfo(session,
-                    SSLPeerInfo(buildX509Name(),
-                                boost::none,
-                                {RoleName("read", "test"), RoleName("readWrite", "test")}));
+    std::set<RoleName> roles({{"read", "test"}, {"readWrite", "test"}});
+    UserRequest request(UserName("CN=mongodb.com", "$external"), roles);
 
-    auto swu = authzManager->acquireUser(opCtx.get(), UserName("CN=mongodb.com", "$external"));
+    auto swu = authzManager->acquireUser(opCtx.get(), request);
     ASSERT_OK(swu.getStatus());
     auto x509User = std::move(swu.getValue());
     ASSERT(x509User.isValid());
 
-    stdx::unordered_set<RoleName> expectedRoles{RoleName("read", "test"),
-                                                RoleName("readWrite", "test")};
-    RoleNameIterator roles = x509User->getRoles();
-    stdx::unordered_set<RoleName> acquiredRoles;
-    while (roles.more()) {
-        acquiredRoles.insert(roles.next());
+    std::set<RoleName> gotRoles;
+    for (auto it = x509User->getRoles(); it.more();) {
+        gotRoles.emplace(it.next());
     }
-    ASSERT_TRUE(expectedRoles == acquiredRoles);
+    ASSERT_TRUE(roles == gotRoles);
 
     const User::ResourcePrivilegeMap& privileges = x509User->getPrivileges();
     ASSERT_FALSE(privileges.empty());
@@ -204,59 +197,20 @@ TEST_F(AuthorizationManagerTest, testLocalX509AuthorizationInvalidUser) {
                                 {RoleName("read", "test"), RoleName("write", "test")}));
 
     ASSERT_NOT_OK(
-        authzManager->acquireUser(opCtx.get(), UserName("CN=10gen.com", "$external")).getStatus());
+        authzManager->acquireUser(opCtx.get(), {{"CN=10gen.com", "$external"}, boost::none})
+            .getStatus());
 }
 
 TEST_F(AuthorizationManagerTest, testLocalX509AuthenticationNoAuthorization) {
     setX509PeerInfo(session, {});
 
-    ASSERT_NOT_OK(authzManager->acquireUser(opCtx.get(), UserName("CN=mongodb.com", "$external"))
-                      .getStatus());
+    ASSERT_NOT_OK(
+        authzManager->acquireUser(opCtx.get(), {{"CN=mongodb.com", "$external"}, boost::none})
+            .getStatus());
 }
 
 #endif
 
-/**
- * An implementation of AuthzManagerExternalStateMock that overrides the getUserDescription method
- * to return the user document unmodified from how it was inserted.  When using this insert user
- * documents in the format that would be returned from a usersInfo command run with
- * showPrivilges:true, rather than the format that would normally be stored in a system.users
- * collection.  The main difference between using this mock and the normal
- * AuthzManagerExternalStateMock is that with this one you should specify the 'inheritedPrivileges'
- * field in any user documents added.
- */
-class AuthzManagerExternalStateMockWithExplicitUserPrivileges
-    : public AuthzManagerExternalStateMock {
-public:
-    /**
-     * This version of getUserDescription just loads the user doc directly as it was inserted into
-     * the mock's user document catalog, without performing any role resolution.  This way the tests
-     * can control exactly what privileges are returned for the user.
-     */
-    Status getUserDescription(OperationContext* opCtx,
-                              const UserRequest& user,
-                              BSONObj* result) override {
-        return _getUserDocument(opCtx, user.name, result);
-    }
-
-private:
-    Status _getUserDocument(OperationContext* opCtx, const UserName& userName, BSONObj* userDoc) {
-        Status status =
-            findOne(opCtx,
-                    AuthorizationManager::usersCollectionNamespace,
-                    BSON(AuthorizationManager::USER_NAME_FIELD_NAME
-                         << userName.getUser() << AuthorizationManager::USER_DB_FIELD_NAME
-                         << userName.getDB()),
-                    userDoc);
-        if (status == ErrorCodes::NoMatchingDocument) {
-            status = Status(ErrorCodes::UserNotFound,
-                            str::stream() << "Could not find user \"" << userName.getUser()
-                                          << "\" for db \"" << userName.getDB() << "\"");
-        }
-        return status;
-    }
-};
-
 // Tests SERVER-21535, unrecognized actions should be ignored rather than causing errors.
 TEST_F(AuthorizationManagerTest, testAcquireV2UserWithUnrecognizedActions) {
     ASSERT_OK(externalState->insertPrivilegeDocument(
@@ -283,7 +237,7 @@ TEST_F(AuthorizationManagerTest, testAcquireV2UserWithUnrecognizedActions) {
                                                          << "insert")))),
         BSONObj()));
 
-    auto swu = authzManager->acquireUser(opCtx.get(), UserName("myUser", "test"));
+    auto swu = authzManager->acquireUser(opCtx.get(), {{"myUser", "test"}, boost::none});
     ASSERT_OK(swu.getStatus());
     auto myUser = std::move(swu.getValue());
     ASSERT_EQUALS(UserName("myUser", "test"), myUser->getName());
@@ -301,129 +255,134 @@ TEST_F(AuthorizationManagerTest, testAcquireV2UserWithUnrecognizedActions) {
     ASSERT(actions.empty());
 }
 
-// These tests ensure that the AuthorizationManager registers a
-// Change on the RecoveryUnit, when an Op is reported that could
-// modify role data. This Change is might recompute
-// the RoleGraph when executed.
-class AuthorizationManagerLogOpTest : public AuthorizationManagerTest {
-public:
-    class MockRecoveryUnit : public RecoveryUnitNoop {
-    public:
-        MockRecoveryUnit(size_t* registeredChanges) : _registeredChanges(registeredChanges) {}
+TEST_F(AuthorizationManagerTest, testRefreshExternalV2User) {
+    constexpr auto kUserFieldName = "user"_sd;
+    constexpr auto kDbFieldName = "db"_sd;
+    constexpr auto kRoleFieldName = "role"_sd;
 
-        void registerChange(std::unique_ptr<Change> change) final {
-            // RecoveryUnitNoop takes ownership of the Change
-            RecoveryUnitNoop::registerChange(std::move(change));
-            ++(*_registeredChanges);
-        }
+    // Insert one user on db test and two users on db $external.
+    BSONObj externalCredentials = BSON("external" << true);
+    std::vector<BSONObj> userDocs{BSON("_id"
+                                       << "admin.v2read"
+                                       << "user"
+                                       << "v2read"
+                                       << "db"
+                                       << "test"
+                                       << "credentials" << credentials << "roles"
+                                       << BSON_ARRAY(BSON("role"
+                                                          << "read"
+                                                          << "db"
+                                                          << "test"))),
+                                  BSON("_id"
+                                       << "admin.v2externalOne"
+                                       << "user"
+                                       << "v2externalOne"
+                                       << "db"
+                                       << "$external"
+                                       << "credentials" << externalCredentials << "roles"
+                                       << BSON_ARRAY(BSON("role"
+                                                          << "read"
+                                                          << "db"
+                                                          << "test"))),
+                                  BSON("_id"
+                                       << "admin.v2externalTwo"
+                                       << "user"
+                                       << "v2externalTwo"
+                                       << "db"
+                                       << "$external"
+                                       << "credentials" << externalCredentials << "roles"
+                                       << BSON_ARRAY(BSON("role"
+                                                          << "read"
+                                                          << "db"
+                                                          << "test")))};
 
-    private:
-        size_t* _registeredChanges;
-    };
+    std::vector<BSONObj> initialRoles{BSON("role"
+                                           << "read"
+                                           << "db"
+                                           << "test")};
+    std::vector<BSONObj> updatedRoles{BSON("role"
+                                           << "readWrite"
+                                           << "db"
+                                           << "test")};
 
-    void setUp() override {
-        opCtx->setRecoveryUnit(std::unique_ptr<RecoveryUnit>(recoveryUnit),
-                               WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
-        AuthorizationManagerTest::setUp();
+    for (const auto& userDoc : userDocs) {
+        ASSERT_OK(externalState->insertPrivilegeDocument(opCtx.get(), userDoc, BSONObj()));
     }
 
-    size_t registeredChanges = 0;
-    MockRecoveryUnit* recoveryUnit = new MockRecoveryUnit(&registeredChanges);
-};
+    // Acquire these users to force the AuthorizationManager to load these users into the user
+    // cache. Store the users into a vector so that they are checked out.
+    std::vector<UserHandle> checkedOutUsers;
+    checkedOutUsers.reserve(userDocs.size());
+    for (const auto& userDoc : userDocs) {
+        const UserName userName(userDoc.getStringField(kUserFieldName),
+                                userDoc.getStringField(kDbFieldName));
+        auto swUser = authzManager->acquireUser(opCtx.get(), {userName, boost::none});
+        ASSERT_OK(swUser.getStatus());
+        auto user = std::move(swUser.getValue());
+        ASSERT_EQUALS(userName, user->getName());
+        ASSERT(user.isValid());
 
-TEST_F(AuthorizationManagerLogOpTest, testDropDatabaseAddsRecoveryUnits) {
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("dropDatabase"
-                             << "1"),
-                        nullptr);
-    ASSERT_EQ(size_t(1), registeredChanges);
-}
+        RoleNameIterator cachedUserRoles = user->getRoles();
+        for (const auto& userDocRole : initialRoles) {
+            ASSERT_EQUALS(cachedUserRoles.next(),
+                          RoleName(userDocRole.getStringField(kRoleFieldName),
+                                   userDocRole.getStringField(kDbFieldName)));
+        }
+        ASSERT_FALSE(cachedUserRoles.more());
+        checkedOutUsers.push_back(std::move(user));
+    }
 
-TEST_F(AuthorizationManagerLogOpTest, testDropAuthCollectionAddsRecoveryUnits) {
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("drop"
-                             << "system.users"),
-                        nullptr);
-    ASSERT_EQ(size_t(1), registeredChanges);
+    // Update each of the users added into the external state so that they gain the readWrite role.
+    for (const auto& userDoc : userDocs) {
+        BSONObj updateQuery = BSON("user" << userDoc.getStringField(kUserFieldName));
+        ASSERT_OK(
+            externalState->updateOne(opCtx.get(),
+                                     NamespaceString::kAdminUsersNamespace,
+                                     updateQuery,
+                                     BSON("$set" << BSON("roles" << BSON_ARRAY(updatedRoles[0]))),
+                                     true,
+                                     BSONObj()));
+    }
 
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("drop"
-                             << "system.roles"),
-                        nullptr);
-    ASSERT_EQ(size_t(2), registeredChanges);
+    // Refresh all external entries in the authorization manager's cache.
+    ASSERT_OK(authzManager->refreshExternalUsers(opCtx.get()));
 
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("drop"
-                             << "system.version"),
-                        nullptr);
-    ASSERT_EQ(size_t(3), registeredChanges);
+    // Assert that all checked-out $external users are now marked invalid.
+    for (const auto& checkedOutUser : checkedOutUsers) {
+        if (checkedOutUser->getName().getDB() == DatabaseName::kExternal.db()) {
+            ASSERT(!checkedOutUser.isValid());
+        } else {
+            ASSERT(checkedOutUser.isValid());
+        }
+    }
 
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("drop"
-                             << "system.profile"),
-                        nullptr);
-    ASSERT_EQ(size_t(3), registeredChanges);
-}
+    // Retrieve all users from the cache and verify that only the external ones contain the newly
+    // added role.
+    for (const auto& userDoc : userDocs) {
+        const UserName userName(userDoc.getStringField(kUserFieldName),
+                                userDoc.getStringField(kDbFieldName));
+        auto swUser = authzManager->acquireUser(opCtx.get(), {userName, boost::none});
+        ASSERT_OK(swUser.getStatus());
+        auto user = std::move(swUser.getValue());
+        ASSERT_EQUALS(userName, user->getName());
+        ASSERT(user.isValid());
 
-TEST_F(AuthorizationManagerLogOpTest, testCreateAnyCollectionAddsNoRecoveryUnits) {
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("create"
-                             << "system.users"),
-                        nullptr);
-
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("create"
-                             << "system.profile"),
-                        nullptr);
-
-    authzManager->logOp(opCtx.get(),
-                        "c",
-                        {"admin", "$cmd"},
-                        BSON("create"
-                             << "system.other"),
-                        nullptr);
-
-    ASSERT_EQ(size_t(0), registeredChanges);
-}
-
-TEST_F(AuthorizationManagerLogOpTest, testRawInsertAddsRecoveryUnits) {
-    authzManager->logOp(opCtx.get(),
-                        "i",
-                        {"admin", "system.profile"},
-                        BSON("_id"
-                             << "admin.user"),
-                        nullptr);
-    ASSERT_EQ(size_t(0), registeredChanges);
-
-    authzManager->logOp(opCtx.get(),
-                        "i",
-                        {"admin", "system.users"},
-                        BSON("_id"
-                             << "admin.user"),
-                        nullptr);
-    ASSERT_EQ(size_t(1), registeredChanges);
-
-    authzManager->logOp(opCtx.get(),
-                        "i",
-                        {"admin", "system.roles"},
-                        BSON("_id"
-                             << "admin.user"),
-                        nullptr);
-    ASSERT_EQ(size_t(2), registeredChanges);
+        RoleNameIterator cachedUserRolesIt = user->getRoles();
+        if (userDoc.getStringField(kDbFieldName) == DatabaseName::kExternal.db()) {
+            for (const auto& userDocRole : updatedRoles) {
+                ASSERT_EQUALS(cachedUserRolesIt.next(),
+                              RoleName(userDocRole.getStringField(kRoleFieldName),
+                                       userDocRole.getStringField(kDbFieldName)));
+            }
+        } else {
+            for (const auto& userDocRole : initialRoles) {
+                ASSERT_EQUALS(cachedUserRolesIt.next(),
+                              RoleName(userDocRole.getStringField(kRoleFieldName),
+                                       userDocRole.getStringField(kDbFieldName)));
+            }
+        }
+        ASSERT_FALSE(cachedUserRolesIt.more());
+    }
 }
 
 }  // namespace
