@@ -101,12 +101,13 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
         return status;
     }
 
-    opCtx->recoveryUnit()->onCommit([writableCollection](auto commitTime) {
-        // Ban reading from this collection on snapshots before now.
-        if (commitTime) {
-            writableCollection->setMinimumVisibleSnapshot(commitTime.value());
-        }
-    });
+    opCtx->recoveryUnit()->onCommit(
+        [writableCollection](OperationContext*, boost::optional<Timestamp> commitTime) {
+            // Ban reading from this collection on snapshots before now.
+            if (commitTime) {
+                writableCollection->setMinimumVisibleSnapshot(commitTime.value());
+            }
+        });
 
     const auto service = opCtx->getServiceContext();
     service->getOpObserver()->onEmptyCapped(opCtx, collection->ns(), collection->uuid());
@@ -122,8 +123,8 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
                              const NamespaceString& toNss,
                              long long size,
                              bool temp) {
-    CollectionPtr fromCollection =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, fromNss);
+    CollectionPtr fromCollection(
+        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, fromNss));
     if (!fromCollection) {
         uassert(ErrorCodes::CommandNotSupportedOnView,
                 str::stream() << "cloneCollectionAsCapped not supported for views: " << fromNss,
@@ -164,8 +165,8 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
         uassertStatusOK(createCollection(opCtx, toNss.dbName(), cmd.done()));
     }
 
-    CollectionPtr toCollection =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, toNss);
+    CollectionPtr toCollection(
+        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, toNss));
     invariant(toCollection);  // we created above
 
     // how much data to ignore because it won't fit anyway
@@ -245,12 +246,12 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
             }
 
             uassertStatusOK(collection_internal::insertDocument(
-                opCtx, toCollection, InsertStatement(objToClone), nullptr /* OpDebug */, true));
+                opCtx, toCollection, std::move(insertStmt), nullptr /* OpDebug */, true));
             wunit.commit();
 
             // Go to the next document
             retries = 0;
-        } catch (const WriteConflictException&) {
+        } catch (const StorageUnavailableException&) {
             CurOp::get(opCtx)->debug().additiveMetrics.incrementWriteConflicts(1);
             retries++;  // logAndBackoff expects this to be 1 on first call.
             logWriteConflictAndBackoff(retries, "cloneCollectionAsCapped", fromNss.ns());
@@ -271,7 +272,8 @@ void convertToCapped(OperationContext* opCtx, const NamespaceString& ns, long lo
     StringData shortSource = ns.coll();
 
     AutoGetCollection coll(opCtx, ns, MODE_X);
-    CollectionShardingState::get(opCtx, ns)->checkShardVersionOrThrow(opCtx);
+    CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, ns)->checkShardVersionOrThrow(
+        opCtx);
 
     bool userInitiatedWritesAndNotPrimary = opCtx->writesAreReplicated() &&
         !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, ns);
@@ -281,8 +283,9 @@ void convertToCapped(OperationContext* opCtx, const NamespaceString& ns, long lo
             !userInitiatedWritesAndNotPrimary);
 
     Database* const db = coll.getDb();
-    uassert(
-        ErrorCodes::NamespaceNotFound, str::stream() << "database " << dbname << " not found", db);
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "database " << dbname.toStringForErrorMsg() << " not found",
+            db);
 
     if (coll) {
         IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(coll->uuid());
