@@ -36,6 +36,13 @@ namespace mongo {
 
 class CollectionImpl final : public Collection {
 public:
+    // Uses the collator factory to convert the BSON representation of a collator to a
+    // CollatorInterface. Returns null if the BSONObj is empty. We expect the stored collation to be
+    // valid, since it gets validated on collection create.
+    static std::unique_ptr<CollatorInterface> parseCollation(OperationContext* opCtx,
+                                                             const NamespaceString& nss,
+                                                             BSONObj collationSpec);
+
     // TODO SERVER-56999: We should just need one API to create Collections
     explicit CollectionImpl(OperationContext* opCtx,
                             const NamespaceString& nss,
@@ -73,6 +80,10 @@ public:
     SharedCollectionDecorations* getSharedDecorations() const final;
 
     void init(OperationContext* opCtx) final;
+    Status initFromExisting(OperationContext* opCtx,
+                            const std::shared_ptr<const Collection>& collection,
+                            const DurableCatalogEntry& catalogEntry,
+                            boost::optional<Timestamp> readTimestamp) final;
     bool isInitialized() const final;
     bool isCommitted() const final;
     void setCommitted(bool val) final;
@@ -104,9 +115,11 @@ public:
     }
 
     std::shared_ptr<Ident> getSharedIdent() const final {
-        // Use shared_ptr's aliasing constructor so we can keep all shared state in a single
-        // reference counted object
-        return {_shared, _shared->_recordStore.get()};
+        return _shared->_recordStore->getSharedIdent();
+    }
+
+    void setIdent(std::shared_ptr<Ident> newIdent) final {
+        _shared->_recordStore->setIdent(std::move(newIdent));
     }
 
     BSONObj getValidatorDoc() const final {
@@ -137,81 +150,8 @@ public:
     std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
                                                     bool forward = true) const final;
 
-    /**
-     * Deletes the document with the given RecordId from the collection. For a description of
-     * the parameters, see the overloaded function below.
-     */
-    void deleteDocument(
-        OperationContext* opCtx,
-        StmtId stmtId,
-        const RecordId& loc,
-        OpDebug* opDebug,
-        bool fromMigrate = false,
-        bool noWarn = false,
-        Collection::StoreDeletedDoc storeDeletedDoc = Collection::StoreDeletedDoc::Off,
-        CheckRecordId checkRecordId = CheckRecordId::Off) const final;
-
-    /**
-     * Deletes the document from the collection.
-
-     * 'doc' the document to be deleted.
-     * 'stmtId' the statement id for this delete operation. Pass in kUninitializedStmtId if not
-     * applicable.
-     * 'fromMigrate' indicates whether the delete was induced by a chunk migration, and
-     * so should be ignored by the user as an internal maintenance operation and not a
-     * real delete.
-     * 'loc' key to uniquely identify a record in a collection.
-     * 'opDebug' Optional argument. When not null, will be used to record operation statistics.
-     * 'noWarn' if unindexing the record causes an error, if noWarn is true the error
-     * will not be logged.
-     * 'storeDeletedDoc' whether to store the document deleted in the oplog.
-     * 'checkRecordId' whether to confirm the recordId matches the record we are removing when
-     * unindexing.
-     */
-    void deleteDocument(
-        OperationContext* opCtx,
-        Snapshotted<BSONObj> doc,
-        StmtId stmtId,
-        const RecordId& loc,
-        OpDebug* opDebug,
-        bool fromMigrate = false,
-        bool noWarn = false,
-        Collection::StoreDeletedDoc storeDeletedDoc = Collection::StoreDeletedDoc::Off,
-        CheckRecordId checkRecordId = CheckRecordId::Off) const final;
-
-    /**
-     * Updates the document @ oldLocation with newDoc.
-     *
-     * If the document fits in the old space, it is put there; if not, it is moved.
-     * Sets 'args.updatedDoc' to the updated version of the document with damages applied, on
-     * success.
-     * 'opDebug' Optional argument. When not null, will be used to record operation statistics.
-     * @return the post update location of the doc (may or may not be the same as oldLocation)
-     */
-    RecordId updateDocument(OperationContext* opCtx,
-                            const RecordId& oldLocation,
-                            const Snapshotted<BSONObj>& oldDoc,
-                            const BSONObj& newDoc,
-                            bool indexesAffected,
-                            OpDebug* opDebug,
-                            CollectionUpdateArgs* args) const final;
-
     bool updateWithDamagesSupported() const final;
 
-    /**
-     * Not allowed to modify indexes.
-     * Illegal to call if updateWithDamagesSupported() returns false.
-     * Sets 'args.updatedDoc' to the updated version of the document with damages applied, on
-     * success.
-     * @return the contents of the updated record.
-     */
-    StatusWith<RecordData> updateDocumentWithDamages(OperationContext* opCtx,
-                                                     const RecordId& loc,
-                                                     const Snapshotted<RecordData>& oldRec,
-                                                     const char* damageSource,
-                                                     const mutablebson::DamageVector& damages,
-                                                     CollectionUpdateArgs* args) const final;
-
     // -----------
 
     /**
@@ -262,9 +202,6 @@ public:
      */
     Status checkValidatorAPIVersionCompatability(OperationContext* opCtx) const final;
 
-    bool getRecordPreImages() const final;
-    void setRecordPreImages(OperationContext* opCtx, bool val) final;
-
     bool isChangeStreamPreAndPostImagesEnabled() const final;
     void setChangeStreamPreAndPostImages(OperationContext* opCtx,
                                          ChangeStreamPreAndPostImagesOptions val) final;
@@ -275,7 +212,8 @@ public:
     void setTimeseriesBucketsMayHaveMixedSchemaData(OperationContext* opCtx,
                                                     boost::optional<bool> setting) final;
 
-    bool doesTimeseriesBucketsDocContainMixedSchemaData(const BSONObj& bucketsDoc) const final;
+    StatusWith<bool> doesTimeseriesBucketsDocContainMixedSchemaData(
+        const BSONObj& bucketsDoc) const final;
 
     bool getRequiresTimeseriesExtendedRangeSupport() const final;
     void setRequiresTimeseriesExtendedRangeSupport(OperationContext* opCtx) const final;
@@ -301,6 +239,14 @@ public:
     long long getCappedMaxDocs() const final;
     long long getCappedMaxSize() const final;
 
+    bool usesCappedSnapshots() const final;
+    std::vector<RecordId> reserveCappedRecordIds(OperationContext* opCtx, size_t nIds) const final;
+    void registerCappedInserts(OperationContext* opCtx,
+                               const RecordId& minRecord,
+                               const RecordId& maxRecord) const final;
+    CappedVisibilityObserver* getCappedVisibilityObserver() const final;
+    CappedVisibilitySnapshot takeCappedVisibilitySnapshot() const final;
+
     long long numRecords(OperationContext* opCtx) const final;
 
     long long dataSize(OperationContext* opCtx) const final;
@@ -361,13 +307,6 @@ public:
     StatusWith<std::vector<BSONObj>> addCollationDefaultsToIndexSpecsForCreate(
         OperationContext* opCtx, const std::vector<BSONObj>& indexSpecs) const final;
 
-    std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> makePlanExecutor(
-        OperationContext* opCtx,
-        const CollectionPtr& yieldableCollection,
-        PlanYieldPolicy::YieldPolicy yieldPolicy,
-        ScanDirection scanDirection,
-        const boost::optional<RecordId>& resumeAfterRecordId) const final;
-
     void indexBuildSuccess(OperationContext* opCtx, IndexCatalogEntry* index) final;
 
     void onDeregisterFromCatalog(OperationContext* opCtx) final;
@@ -432,6 +371,8 @@ public:
     void replaceMetadata(OperationContext* opCtx,
                          std::shared_ptr<BSONCollectionCatalogEntry::MetaData> md) final;
 
+    bool isMetadataEqual(const BSONObj& otherMetadata) const final;
+
     bool needsCappedLock() const final;
 
     bool isCappedAndNeedsDelete(OperationContext* opCtx) const final;
@@ -444,6 +385,16 @@ private:
     template <typename Func>
     void _writeMetadata(OperationContext* opCtx, Func func);
 
+    /**
+     * Helper for init() and initFromExisting() to initialize shared state.
+     */
+    void _initShared(OperationContext* opCtx, const CollectionOptions& options);
+
+    /**
+     * Helper for init() and initFromExisting() to initialize common state.
+     */
+    void _initCommon(OperationContext* opCtx);
+
     /**
      * Holder of shared state between CollectionImpl clones
      */
@@ -473,6 +424,15 @@ private:
 
         AtomicWord<bool> _committed{true};
 
+        // Tracks in-progress capped inserts to inform visibility for forward scans so that no
+        // uncommitted records are skipped.
+        CappedVisibilityObserver _cappedObserver;
+
+        // This mutex synchronizes allocating and registering RecordIds for uncommited writes on
+        // capped collections that accept concurrent writes (i.e. usesCappedSnapshots()).
+        mutable Mutex _registerCappedIdsMutex =
+            MONGO_MAKE_LATCH("CollectionImpl::_registerCappedIdsMutex");
+
         // Time-series collections are allowed to contain measurements with arbitrary dates;
         // however, many of our query optimizations only work properly with dates that can be stored
         // as an offset in seconds from the Unix epoch within 31 bits (roughly 1970-2038). When this
