@@ -35,6 +35,7 @@
 
 #include "mongo/db/json.h"
 #include "mongo/idl/server_parameter_test_util.h"
+#include "mongo/platform/decimal128.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
@@ -99,9 +100,27 @@ TEST(CollectionOptions, ErrorBadMax) {
         CollectionOptions::parse(BSON("capped" << true << "max" << (1LL << 31))).getStatus());
 }
 
+TEST(CollectionOptions, CappedSizeNotRoundUpForAlignment) {
+    serverGlobalParams.mutableFCV.setVersion(
+        multiversion::FeatureCompatibilityVersion::kVersion_6_2);
+    const long long kUnalignedCappedSize = 1000;
+    const long long kAlignedCappedSize = 1000;
+
+    // Check size rounds up to multiple of alignment.
+    auto options = assertGet(
+        CollectionOptions::parse((BSON("capped" << true << "size" << kUnalignedCappedSize))));
+
+    ASSERT_EQUALS(options.capped, true);
+    ASSERT_EQUALS(options.cappedSize, kAlignedCappedSize);
+    ASSERT_EQUALS(options.cappedMaxDocs, 0);
+}
+
 TEST(CollectionOptions, CappedSizeRoundsUpForAlignment) {
+    serverGlobalParams.mutableFCV.setVersion(
+        multiversion::FeatureCompatibilityVersion::kVersion_6_0);
     const long long kUnalignedCappedSize = 1000;
     const long long kAlignedCappedSize = 1024;
+
     // Check size rounds up to multiple of alignment.
     auto options = assertGet(
         CollectionOptions::parse((BSON("capped" << true << "size" << kUnalignedCappedSize))));
@@ -631,15 +650,35 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
     // TODO: SERVER-67760 remove feature flag
     RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
 
-    std::vector<std::string> typesAllowedIndexed({
-        "int",
-        "long",
-        "double",
-        "decimal",
-    });
+    ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : 1, max : 2}
+                }
+            ]
+        }
+    }})"))
+                  .getStatus());
 
+    ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "long",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : {$numberLong: "1"}, max : {$numberLong: "2"}}
+                }
+            ]
+        }
+    }})"))
+                  .getStatus());
 
-    for (const auto& type : typesAllowedIndexed) {
+    for (const auto& type : std::vector<std::string>{"double", "decimal"}) {
         ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
         encryptedFields: {
             "fields": [
@@ -647,7 +686,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": ")" << type << R"(",
-                    "queries": {"queryType": "range", "sparsity" : 1, min : 1, max : 2}
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1}
                 }
             ]
         }
@@ -655,6 +694,38 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                       .getStatus());
     }
 
+    ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": ")"
+                                                              << "double"
+                                                              << R"(",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 0.000, max: 1.000, precision: 3}
+                }
+            ]
+        }
+    }})"))
+                  .getStatus());
+
+    ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": ")"
+                                                              << "decimal"
+                                                              << R"(",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: NumberDecimal("0.000"), max: NumberDecimal("1.000"), precision: 3}
+                }
+            ]
+        }
+    }})"))
+                  .getStatus());
+
     // Validate date works
     ASSERT_OK(CollectionOptions::parse(fromjson(str::stream() << R"({
         encryptedFields: {
@@ -663,7 +734,7 @@ TEST(FLECollectionOptions, Range_AllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "date",
-                    "queries": {"queryType": "range", "sparsity" : 1, min : {"$date": {"$numberLong": "12344"}}, max : {"$date": {"$numberLong": "12345"}}}
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min : {"$date": {"$numberLong": "12344"}}, max : {"$date": {"$numberLong": "12345"}}}
                 }
             ]
         }
@@ -702,7 +773,7 @@ TEST(FLECollectionOptions, Range_DisAllowedTypes) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": ")" << type << R"(",
-                    "queries": {"queryType": "range"}
+                    "queries": {"queryType": "rangePreview"}
                 }
             ]
         }
@@ -721,7 +792,7 @@ TEST(FLECollectionOptions, Range_MissingFields) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "range"}
+                    "queries": {"queryType": "rangePreview"}
                 }
             ]
         }
@@ -734,7 +805,7 @@ TEST(FLECollectionOptions, Range_MissingFields) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "range", sparsity: 1}
+                    "queries": {"queryType": "rangePreview", sparsity: 1}
                 }
             ]
         }
@@ -747,7 +818,7 @@ TEST(FLECollectionOptions, Range_MissingFields) {
                     "path": "firstName",
                     "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
                     "bsonType": "int",
-                    "queries": {"queryType": "range", sparsity: 1, min : 1}
+                    "queries": {"queryType": "rangePreview", sparsity: 1, min : 1}
                 }
             ]
         }
@@ -803,24 +874,114 @@ TEST(FLECollectionOptions, Range_MinMax) {
     // TODO: SERVER-67760 remove feature flag
     RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
 
-    std::vector<std::pair<std::string, int>> types{
-        {"int", 6775208},
-        {"long", 6775209},
-        {"double", 6775210},
-        {"decimal", 6775211},
-    };
 
-    for (auto const& tc : types) {
+    {
         auto doc = BSON("encryptedFields"
-                        << BSON("fields" << BSON_ARRAY(BSON(
-                                    "path"
-                                    << "firstName"
-                                    << "keyId" << UUID::gen() << "bsonType" << tc.first << "queries"
-                                    << BSON("queryType"
-                                            << "range"
-                                            << "sparsity" << 1 << "min" << 2 << "max" << 1)))));
+                        << BSON("fields" << BSON_ARRAY(BSON("path"
+                                                            << "firstName"
+                                                            << "keyId" << UUID::gen() << "bsonType"
+                                                            << "int"
+                                                            << "queries"
+                                                            << BSON("queryType"
+                                                                    << "rangePreview"
+                                                                    << "sparsity" << 1 << "min" << 2
+                                                                    << "max" << 1)))));
 
-        ASSERT_STATUS_CODE(tc.second, CollectionOptions::parse(doc));
+        ASSERT_STATUS_CODE(6720005, CollectionOptions::parse(doc));
+    }
+
+    {
+        auto doc = BSON("encryptedFields"
+                        << BSON("fields" << BSON_ARRAY(BSON("path"
+                                                            << "firstName"
+                                                            << "keyId" << UUID::gen() << "bsonType"
+                                                            << "long"
+                                                            << "queries"
+                                                            << BSON("queryType"
+                                                                    << "rangePreview"
+                                                                    << "sparsity" << 1 << "min"
+                                                                    << 2LL << "max" << 1LL)))));
+
+        ASSERT_STATUS_CODE(6720005, CollectionOptions::parse(doc));
+    }
+
+    {
+        auto doc =
+            BSON("encryptedFields" << BSON(
+                     "fields" << BSON_ARRAY(BSON("path"
+                                                 << "firstName"
+                                                 << "keyId" << UUID::gen() << "bsonType"
+                                                 << "double"
+                                                 << "queries"
+                                                 << BSON("queryType"
+                                                         << "rangePreview"
+                                                         << "sparsity" << 1 << "min" << 2.0)))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
+
+        doc = BSON("encryptedFields" << BSON(
+                       "fields" << BSON_ARRAY(BSON("path"
+                                                   << "firstName"
+                                                   << "keyId" << UUID::gen() << "bsonType"
+                                                   << "double"
+                                                   << "queries"
+                                                   << BSON("queryType"
+                                                           << "rangePreview"
+                                                           << "sparsity" << 1 << "max" << 2.0)))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
+
+        doc = BSON("encryptedFields"
+                   << BSON("fields"
+                           << BSON_ARRAY(BSON("path"
+                                              << "firstName"
+                                              << "keyId" << UUID::gen() << "bsonType"
+                                              << "double"
+                                              << "queries"
+                                              << BSON("queryType"
+                                                      << "rangePreview"
+                                                      << "sparsity" << 1 << "precision" << 2)))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
+
+        doc = BSON("encryptedFields"
+                   << BSON("fields" << BSON_ARRAY(
+                               BSON("path"
+                                    << "firstName"
+                                    << "keyId" << UUID::gen() << "bsonType"
+                                    << "decimal"
+                                    << "queries"
+                                    << BSON("queryType"
+                                            << "rangePreview"
+                                            << "sparsity" << 1 << "min" << Decimal128(2.0))))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
+
+        doc = BSON("encryptedFields"
+                   << BSON("fields" << BSON_ARRAY(
+                               BSON("path"
+                                    << "firstName"
+                                    << "keyId" << UUID::gen() << "bsonType"
+                                    << "decimal"
+                                    << "queries"
+                                    << BSON("queryType"
+                                            << "rangePreview"
+                                            << "sparsity" << 1 << "max" << Decimal128(2.0))))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
+
+        doc = BSON("encryptedFields"
+                   << BSON("fields"
+                           << BSON_ARRAY(BSON("path"
+                                              << "firstName"
+                                              << "keyId" << UUID::gen() << "bsonType"
+                                              << "decimal"
+                                              << "queries"
+                                              << BSON("queryType"
+                                                      << "rangePreview"
+                                                      << "sparsity" << 1 << "precision" << 2)))));
+
+        ASSERT_STATUS_CODE(6967100, CollectionOptions::parse(doc));
     }
 
 
@@ -834,11 +995,65 @@ TEST(FLECollectionOptions, Range_MinMax) {
                                                         << "date"
                                                         << "queries"
                                                         << BSON("queryType"
-                                                                << "range"
+                                                                << "rangePreview"
                                                                 << "sparsity" << 1 << "min" << end
                                                                 << "max" << start)))));
 
-    ASSERT_STATUS_CODE(6775213, CollectionOptions::parse(doc));
+    ASSERT_STATUS_CODE(6720005, CollectionOptions::parse(doc));
+}
+
+TEST(FLECollectionOptions, Range_BoundTypeMismatch) {
+    RAIIServerParameterControllerForTest featureFlagController("featureFlagFLE2Range", true);
+    ASSERT_STATUS_CODE(7018200, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: {"$numberLong": "12344"}, max: {"$numberLong": "123440"}}
+                }
+            ]
+        }
+    }})")));
+
+    ASSERT_STATUS_CODE(7018200, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "long",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 1, max: 2}
+                }
+            ]
+        }
+    }})")));
+
+    ASSERT_STATUS_CODE(7018201, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "long",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: {$numberLong: "1"}, max: 2}
+                }
+            ]
+        }
+    }})")));
+    ASSERT_STATUS_CODE(7018201, CollectionOptions::parse(fromjson(str::stream() << R"({
+        encryptedFields: {
+            "fields": [
+                {
+                    "path": "firstName",
+                    "keyId": { '$uuid': '5f34e99a-b214-451f-b6f6-d3d28e933d15' },
+                    "bsonType": "int",
+                    "queries": {"queryType": "rangePreview", "sparsity" : 1, min: 1, max: {"$numberLong": "123440"}}
+                }
+            ]
+        }
+    }})")));
 }
 
 }  // namespace mongo
