@@ -58,25 +58,26 @@ namespace {
 
 MONGO_FAIL_POINT_DEFINE(hangDropCollectionBeforeLockAcquisition);
 MONGO_FAIL_POINT_DEFINE(hangDuringDropCollection);
+MONGO_FAIL_POINT_DEFINE(allowSystemViewsDrop);
 
-Status _checkNssAndReplState(OperationContext* opCtx,
-                             const CollectionPtr& coll,
-                             const NamespaceString& nss,
-                             const boost::optional<UUID>& expectedUUID = boost::none) {
+/**
+ * Checks that the collection has the 'expectedUUID' if given.
+ * Checks that writes are allowed to 'coll' -- e.g. whether this server is PRIMARY.
+ */
+Status _checkUUIDAndReplState(OperationContext* opCtx,
+                              const CollectionPtr& coll,
+                              const NamespaceString& nss,
+                              const boost::optional<UUID>& expectedUUID = boost::none) {
     try {
         checkCollectionUUIDMismatch(opCtx, nss, coll, expectedUUID);
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
 
-    if (!coll) {
-        return Status(ErrorCodes::NamespaceNotFound, "ns not found");
-    }
-
     if (opCtx->writesAreReplicated() &&
-        !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, coll->ns())) {
+        !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss)) {
         return Status(ErrorCodes::NotWritablePrimary,
-                      str::stream() << "Not primary while dropping collection " << coll->ns());
+                      str::stream() << "Not primary while dropping collection " << nss);
     }
 
     return Status::OK();
@@ -125,15 +126,11 @@ Status _dropView(OperationContext* opCtx,
                  const NamespaceString& collectionName,
                  const boost::optional<UUID>& expectedUUID,
                  DropReply* reply) {
-    if (!db) {
-        Status status = Status(ErrorCodes::NamespaceNotFound, "ns not found");
-        audit::logDropView(opCtx->getClient(), collectionName, "", {}, status.code());
-        return status;
-    }
+    invariant(db);
 
     // Views don't have UUIDs so if the expectedUUID is specified, we will always throw.
     try {
-        checkCollectionUUIDMismatch(opCtx, collectionName, nullptr, expectedUUID);
+        checkCollectionUUIDMismatch(opCtx, collectionName, CollectionPtr(), expectedUUID);
     } catch (const DBException& ex) {
         return ex.toStatus();
     }
@@ -141,9 +138,9 @@ Status _dropView(OperationContext* opCtx,
     auto view =
         CollectionCatalog::get(opCtx)->lookupViewWithoutValidatingDurable(opCtx, collectionName);
     if (!view) {
-        Status status = Status(ErrorCodes::NamespaceNotFound, "ns not found");
-        audit::logDropView(opCtx->getClient(), collectionName, "", {}, status.code());
-        return status;
+        audit::logDropView(
+            opCtx->getClient(), collectionName, "", {}, ErrorCodes::NamespaceNotFound);
+        return Status::OK();
     }
 
     // Validates the view or throws an "invalid view" error.
@@ -198,18 +195,22 @@ Status _abortIndexBuildsAndDrop(OperationContext* opCtx,
     // We only need to hold an intent lock to send abort signals to the active index builder on this
     // collection.
     boost::optional<AutoGetDb> optionalAutoDb(std::move(autoDb));
-    boost::optional<Lock::CollectionLock> collLock;
+    boost::optional<CollectionNamespaceOrUUIDLock> collLock;
     collLock.emplace(opCtx, startingNss, MODE_IX);
 
     // Abandon the snapshot as the index catalog will compare the in-memory state to the disk state,
     // which may have changed when we released the collection lock temporarily.
     opCtx->recoveryUnit()->abandonSnapshot();
 
-    CollectionPtr coll =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, startingNss);
-    Status status = _checkNssAndReplState(opCtx, coll, startingNss, expectedUUID);
+    CollectionPtr coll(
+        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, startingNss));
+
+    // Even if the collection doesn't exist, UUID mismatches must return an error.
+    Status status = _checkUUIDAndReplState(opCtx, coll, startingNss, expectedUUID);
     if (!status.isOK()) {
         return status;
+    } else if (!coll) {
+        return Status::OK();
     }
 
     warnEncryptedCollectionsIfNeeded(opCtx, coll);
@@ -233,8 +234,8 @@ Status _abortIndexBuildsAndDrop(OperationContext* opCtx,
     if (dropIfUUIDNotMatching && collectionUUID == *dropIfUUIDNotMatching) {
         return Status::OK();
     }
-    const NamespaceStringOrUUID dbAndUUID{coll->ns().db().toString(), coll->uuid()};
-    const int numIndexes = coll->getIndexCatalog()->numIndexesTotal(opCtx);
+    const NamespaceStringOrUUID dbAndUUID{coll->ns().dbName(), coll->uuid()};
+    const int numIndexes = coll->getIndexCatalog()->numIndexesTotal();
 
     while (true) {
         // Save a copy of the namespace before yielding our locks.
@@ -254,17 +255,22 @@ Status _abortIndexBuildsAndDrop(OperationContext* opCtx,
                                                          << collectionUUID << ") is being dropped");
 
         // Take an exclusive lock to finish the collection drop.
-        optionalAutoDb.emplace(opCtx, startingNss.db(), MODE_IX);
+        optionalAutoDb.emplace(opCtx, startingNss.dbName(), MODE_IX);
         collLock.emplace(opCtx, dbAndUUID, MODE_X);
 
         // Abandon the snapshot as the index catalog will compare the in-memory state to the
         // disk state, which may have changed when we released the collection lock temporarily.
         opCtx->recoveryUnit()->abandonSnapshot();
 
-        coll = CollectionCatalog::get(opCtx)->lookupCollectionByUUID(opCtx, collectionUUID);
-        status = _checkNssAndReplState(opCtx, coll, startingNss, expectedUUID);
+        coll = CollectionPtr(
+            CollectionCatalog::get(opCtx)->lookupCollectionByUUID(opCtx, collectionUUID));
+
+        // Even if the collection doesn't exist, UUID mismatches must return an error.
+        status = _checkUUIDAndReplState(opCtx, coll, startingNss, expectedUUID);
         if (!status.isOK()) {
             return status;
+        } else if (!coll) {
+            return Status::OK();
         }
 
         // Check if any new index builds were started while releasing the collection lock
@@ -284,9 +290,10 @@ Status _abortIndexBuildsAndDrop(OperationContext* opCtx,
 
     // Serialize the drop with refreshes to prevent dropping a collection and creating the same
     // nss as a view while refreshing.
-    CollectionShardingState::get(opCtx, resolvedNss)->checkShardVersionOrThrow(opCtx);
+    CollectionShardingState::assertCollectionLockedAndAcquire(opCtx, resolvedNss)
+        ->checkShardVersionOrThrow(opCtx);
 
-    invariant(coll->getIndexCatalog()->numIndexesInProgress(opCtx) == 0);
+    invariant(coll->getIndexCatalog()->numIndexesInProgress() == 0);
 
     status = dropFn(optionalAutoDb->getDb(), resolvedNss);
     if (!status.isOK()) {
@@ -308,11 +315,15 @@ Status _dropCollectionForApplyOps(OperationContext* opCtx,
                                   DropCollectionSystemCollectionMode systemCollectionMode,
                                   DropReply* reply) {
     Lock::CollectionLock collLock(opCtx, collectionName, MODE_X);
-    const CollectionPtr& coll =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, collectionName);
-    Status status = _checkNssAndReplState(opCtx, coll, collectionName);
+    CollectionPtr coll(
+        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, collectionName));
+
+    // Even if the collection doesn't exist, UUID mismatches must return an error.
+    Status status = _checkUUIDAndReplState(opCtx, coll, collectionName);
     if (!status.isOK()) {
         return status;
+    } else if (!coll) {
+        return Status::OK();
     }
 
     if (MONGO_unlikely(hangDuringDropCollection.shouldFail())) {
@@ -331,7 +342,7 @@ Status _dropCollectionForApplyOps(OperationContext* opCtx,
 
     WriteUnitOfWork wunit(opCtx);
 
-    int numIndexes = coll->getIndexCatalog()->numIndexesTotal(opCtx);
+    int numIndexes = coll->getIndexCatalog()->numIndexesTotal();
     IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(coll->uuid());
     status =
         systemCollectionMode == DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops
@@ -354,6 +365,7 @@ Status _dropCollection(OperationContext* opCtx,
                        const boost::optional<UUID>& expectedUUID,
                        DropReply* reply,
                        DropCollectionSystemCollectionMode systemCollectionMode,
+                       bool fromMigrate,
                        boost::optional<UUID> dropIfUUIDNotMatching = boost::none) {
 
     try {
@@ -362,12 +374,12 @@ Status _dropCollection(OperationContext* opCtx,
             auto db = autoDb.getDb();
             if (!db) {
                 return expectedUUID
-                    ? Status{CollectionUUIDMismatchInfo(collectionName.db().toString(),
+                    ? Status{CollectionUUIDMismatchInfo(collectionName.dbName(),
                                                         *expectedUUID,
                                                         collectionName.coll().toString(),
                                                         boost::none),
                              "Database does not exist"}
-                    : Status(ErrorCodes::NamespaceNotFound, "ns not found");
+                    : Status::OK();
             }
 
             if (CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, collectionName)) {
@@ -376,13 +388,14 @@ Status _dropCollection(OperationContext* opCtx,
                     std::move(autoDb),
                     collectionName,
                     expectedUUID,
-                    [opCtx, systemCollectionMode](Database* db, const NamespaceString& resolvedNs) {
+                    [opCtx, systemCollectionMode, fromMigrate](Database* db,
+                                                               const NamespaceString& resolvedNs) {
                         WriteUnitOfWork wuow(opCtx);
 
                         auto status = systemCollectionMode ==
                                 DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops
-                            ? db->dropCollection(opCtx, resolvedNs)
-                            : db->dropCollectionEvenIfSystem(opCtx, resolvedNs);
+                            ? db->dropCollection(opCtx, resolvedNs, {}, fromMigrate)
+                            : db->dropCollectionEvenIfSystem(opCtx, resolvedNs, {}, fromMigrate);
                         if (!status.isOK()) {
                             return status;
                         }
@@ -395,14 +408,18 @@ Status _dropCollection(OperationContext* opCtx,
                     dropIfUUIDNotMatching);
             }
 
-            auto dropTimeseries = [opCtx, &expectedUUID, &autoDb, &collectionName, &reply](
-                                      const NamespaceString& bucketNs, bool dropView) {
+            auto dropTimeseries = [opCtx,
+                                   &expectedUUID,
+                                   &autoDb,
+                                   &collectionName,
+                                   &reply,
+                                   fromMigrate](const NamespaceString& bucketNs, bool dropView) {
                 return _abortIndexBuildsAndDrop(
                     opCtx,
                     std::move(autoDb),
                     bucketNs,
                     expectedUUID,
-                    [opCtx, dropView, &expectedUUID, &collectionName, &reply](
+                    [opCtx, dropView, &expectedUUID, &collectionName, &reply, fromMigrate](
                         Database* db, const NamespaceString& bucketsNs) {
                         // Disallow checking the expectedUUID when dropping time-series collections.
                         uassert(ErrorCodes::InvalidOptions,
@@ -426,11 +443,13 @@ Status _dropCollection(OperationContext* opCtx,
 
                         // Drop the buckets collection in its own writeConflictRetry so that if
                         // it throws a WCE, only the buckets collection drop is retried.
-                        writeConflictRetry(opCtx, "drop", bucketsNs.ns(), [opCtx, db, &bucketsNs] {
-                            WriteUnitOfWork wuow(opCtx);
-                            db->dropCollectionEvenIfSystem(opCtx, bucketsNs).ignore();
-                            wuow.commit();
-                        });
+                        writeConflictRetry(
+                            opCtx, "drop", bucketsNs.ns(), [opCtx, db, &bucketsNs, fromMigrate] {
+                                WriteUnitOfWork wuow(opCtx);
+                                db->dropCollectionEvenIfSystem(opCtx, bucketsNs, {}, fromMigrate)
+                                    .ignore();
+                                wuow.commit();
+                            });
 
                         return Status::OK();
                     },
@@ -448,15 +467,19 @@ Status _dropCollection(OperationContext* opCtx,
                     return dropTimeseries(bucketsNs, false);
                 }
 
+                // There is no collection or view at the namespace. Check whether a UUID was given
+                // and error if so because the caller expects the collection to exist. If no UUID
+                // was given, then it is OK to return success.
                 try {
-                    checkCollectionUUIDMismatch(opCtx, collectionName, nullptr, expectedUUID);
+                    checkCollectionUUIDMismatch(
+                        opCtx, collectionName, CollectionPtr(), expectedUUID);
                 } catch (const DBException& ex) {
                     return ex.toStatus();
                 }
 
-                Status status = Status(ErrorCodes::NamespaceNotFound, "ns not found");
-                audit::logDropView(opCtx->getClient(), collectionName, "", {}, status.code());
-                return status;
+                audit::logDropView(
+                    opCtx->getClient(), collectionName, "", {}, ErrorCodes::NamespaceNotFound);
+                return Status::OK();
             }
             if (view->timeseries() &&
                 CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, view->viewOn())) {
@@ -469,9 +492,15 @@ Status _dropCollection(OperationContext* opCtx,
             return _dropView(opCtx, db, collectionName, expectedUUID, reply);
         });
     } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
-        // The shell requires that NamespaceNotFound error codes return the "ns not found"
-        // string.
-        return Status(ErrorCodes::NamespaceNotFound, "ns not found");
+        // Any unhandled namespace not found errors should be converted into success. Unless the
+        // caller specified a UUID and expects the collection to exist.
+        try {
+            checkCollectionUUIDMismatch(opCtx, collectionName, CollectionPtr(), expectedUUID);
+        } catch (const DBException& ex) {
+            return ex.toStatus();
+        }
+
+        return Status::OK();
     }
 }
 }  // namespace
@@ -480,7 +509,8 @@ Status dropCollection(OperationContext* opCtx,
                       const NamespaceString& nss,
                       const boost::optional<UUID>& expectedUUID,
                       DropReply* reply,
-                      DropCollectionSystemCollectionMode systemCollectionMode) {
+                      DropCollectionSystemCollectionMode systemCollectionMode,
+                      bool fromMigrate) {
     if (!serverGlobalParams.quiet.load()) {
         LOGV2(518070, "CMD: drop", logAttrs(nss));
     }
@@ -495,14 +525,16 @@ Status dropCollection(OperationContext* opCtx,
     const auto collectionName =
         nss.isTimeseriesBucketsCollection() ? nss.getTimeseriesViewNamespace() : nss;
 
-    return _dropCollection(opCtx, collectionName, expectedUUID, reply, systemCollectionMode);
+    return _dropCollection(
+        opCtx, collectionName, expectedUUID, reply, systemCollectionMode, fromMigrate);
 }
 
 Status dropCollection(OperationContext* opCtx,
                       const NamespaceString& nss,
                       DropReply* reply,
-                      DropCollectionSystemCollectionMode systemCollectionMode) {
-    return dropCollection(opCtx, nss, boost::none, reply, systemCollectionMode);
+                      DropCollectionSystemCollectionMode systemCollectionMode,
+                      bool fromMigrate) {
+    return dropCollection(opCtx, nss, boost::none, reply, systemCollectionMode, fromMigrate);
 }
 
 Status dropCollectionIfUUIDNotMatching(OperationContext* opCtx,
@@ -524,6 +556,7 @@ Status dropCollectionIfUUIDNotMatching(OperationContext* opCtx,
                                boost::none,
                                &repl,
                                DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops,
+                               false /*fromMigrate*/,
                                expectedUUID);
     }
 
@@ -546,11 +579,11 @@ Status dropCollectionForApplyOps(OperationContext* opCtx,
         AutoGetDb autoDb(opCtx, collectionName.dbName(), MODE_IX);
         Database* db = autoDb.getDb();
         if (!db) {
-            return Status(ErrorCodes::NamespaceNotFound, "ns not found");
+            return Status::OK();
         }
 
-        const CollectionPtr& coll =
-            CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, collectionName);
+        CollectionPtr coll(
+            CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, collectionName));
 
         DropReply unusedReply;
         if (!coll) {
@@ -567,7 +600,7 @@ void checkForIdIndexesAndDropPendingCollections(OperationContext* opCtx,
                                                 const DatabaseName& dbName) {
     invariant(opCtx->lockState()->isDbLockedForMode(dbName, MODE_IX));
 
-    if (dbName.db() == NamespaceString::kLocalDb) {
+    if (dbName == DatabaseName::kLocal) {
         // Collections in the local database are not replicated, so we do not need an _id index on
         // any collection. For the same reason, it is not possible for the local database to contain
         // any drop-pending collections (drops are effective immediately).
@@ -582,7 +615,7 @@ void checkForIdIndexesAndDropPendingCollections(OperationContext* opCtx,
             LOGV2(20321,
                   "Found drop-pending namespace {namespace} with drop optime {dropOpTime}",
                   "Found drop-pending namespace",
-                  "namespace"_attr = nss,
+                  logAttrs(nss),
                   "dropOpTime"_attr = dropOpTime);
             repl::DropPendingCollectionReaper::get(opCtx)->addDropPendingNamespace(
                 opCtx, dropOpTime, nss);
@@ -591,7 +624,7 @@ void checkForIdIndexesAndDropPendingCollections(OperationContext* opCtx,
         if (nss.isSystem())
             continue;
 
-        CollectionPtr coll = catalog->lookupCollectionByNamespace(opCtx, nss);
+        CollectionPtr coll(catalog->lookupCollectionByNamespace(opCtx, nss));
         if (!coll)
             continue;
 
@@ -608,7 +641,7 @@ void checkForIdIndexesAndDropPendingCollections(OperationContext* opCtx,
             "Collection lacks a unique index on _id. This index is "
             "needed for replication to function properly. To fix this, you need to create a unique "
             "index on _id. See http://dochub.mongodb.org/core/build-replica-set-indexes",
-            "namespace"_attr = nss);
+            logAttrs(nss));
     }
 }
 
@@ -618,33 +651,68 @@ void clearTempCollections(OperationContext* opCtx, const DatabaseName& dbName) {
     auto db = DatabaseHolder::get(opCtx)->getDb(opCtx, dbName);
     invariant(db);
 
-    CollectionCatalog::CollectionInfoFn callback = [&](const CollectionPtr& collection) {
+    CollectionCatalog::CollectionInfoFn callback = [&](const Collection* collection) {
         try {
             WriteUnitOfWork wuow(opCtx);
-            Status status = db->dropCollection(opCtx, collection->ns(), {});
+            Status status = db->dropCollection(opCtx, collection->ns());
             if (!status.isOK()) {
                 LOGV2_WARNING(20327,
                               "could not drop temp collection '{namespace}': {error}",
                               "could not drop temp collection",
-                              "namespace"_attr = collection->ns(),
+                              logAttrs(collection->ns()),
                               "error"_attr = redact(status));
             }
             wuow.commit();
-        } catch (const WriteConflictException&) {
+        } catch (const StorageUnavailableException&) {
             LOGV2_WARNING(
                 20328,
                 "could not drop temp collection '{namespace}' due to WriteConflictException",
                 "could not drop temp collection due to WriteConflictException",
-                "namespace"_attr = collection->ns());
+                logAttrs(collection->ns()));
             opCtx->recoveryUnit()->abandonSnapshot();
         }
         return true;
     };
 
     catalog::forEachCollectionFromDb(
-        opCtx, dbName, MODE_X, callback, [&](const CollectionPtr& collection) {
+        opCtx, dbName, MODE_X, callback, [&](const Collection* collection) {
             return collection->getCollectionOptions().temp;
         });
 }
 
+Status isDroppableCollection(OperationContext* opCtx, const NamespaceString& nss) {
+    if (!nss.isSystem()) {
+        return Status::OK();
+    }
+
+    auto isDroppableSystemCollection = [](const auto& nss) {
+        return nss.isHealthlog() || nss == NamespaceString::kLogicalSessionsNamespace ||
+            nss == NamespaceString::kKeysCollectionNamespace ||
+            nss.isTemporaryReshardingCollection() || nss.isTimeseriesBucketsCollection() ||
+            nss.isChangeStreamPreImagesCollection() ||
+            nss == NamespaceString::kConfigsvrRestoreNamespace || nss.isChangeCollection() ||
+            nss.isSystemDotJavascript() || nss.isSystemStatsCollection();
+    };
+
+    if (nss.isSystemDotProfile()) {
+        if (CollectionCatalog::get(opCtx)->getDatabaseProfileLevel(nss.dbName()) != 0)
+            return Status(ErrorCodes::IllegalOperation,
+                          "turn off profiling before dropping system.profile collection");
+    } else if (nss.isSystemDotViews()) {
+        if (!MONGO_unlikely(allowSystemViewsDrop.shouldFail())) {
+            const auto viewStats =
+                CollectionCatalog::get(opCtx)->getViewStatsForDatabase(opCtx, nss.dbName());
+            uassert(ErrorCodes::CommandFailed,
+                    "cannot drop collection {} when time-series collections are present"_format(
+                        nss.toStringForErrorMsg()),
+                    viewStats && viewStats->userTimeseries == 0);
+        }
+    } else if (!isDroppableSystemCollection(nss)) {
+        return Status(ErrorCodes::IllegalOperation,
+                      "cannot drop system collection {}"_format(nss.toStringForErrorMsg()));
+    }
+
+    return Status::OK();
+}
+
 }  // namespace mongo
