@@ -41,13 +41,16 @@
 #include "mongo/base/status_with.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/index/column_key_generator.h"
+#include "mongo/db/index/columns_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/wildcard_key_generator.h"
+#include "mongo/db/index/wildcard_validation.h"
 #include "mongo/db/index_names.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
@@ -61,7 +64,7 @@
 namespace mongo {
 namespace index_key_validate {
 
-std::function<void(std::set<StringData>&)> filterAllowedIndexFieldNames;
+std::function<void(std::map<StringData, std::set<IndexType>>&)> filterAllowedIndexFieldNames;
 
 using IndexVersion = IndexDescriptor::IndexVersion;
 
@@ -71,9 +74,9 @@ namespace {
 // specification.
 MONGO_FAIL_POINT_DEFINE(skipIndexCreateFieldNameValidation);
 
-// When the skipTTLIndexNaNExpireAfterSecondsValidation failpoint is enabled, validation for
-// TTL index 'expireAfterSeconds' will be disabled.
-MONGO_FAIL_POINT_DEFINE(skipTTLIndexNaNExpireAfterSecondsValidation);
+// When the skipTTLIndexInvalidExpireAfterSecondsValidationForCreateIndex failpoint is enabled,
+// validation for TTL index 'expireAfterSeconds' will be disabled in certain codepaths.
+MONGO_FAIL_POINT_DEFINE(skipTTLIndexInvalidExpireAfterSecondsValidationForCreateIndex);
 
 static const std::set<StringData> allowedIdIndexFieldNames = {
     IndexDescriptor::kCollationFieldName,
@@ -111,17 +114,21 @@ Status isIndexVersionAllowedForCreation(IndexVersion indexVersion, const BSONObj
 BSONObj buildRepairedIndexSpec(
     const NamespaceString& ns,
     const BSONObj& indexSpec,
-    const std::set<StringData>& allowedFieldNames,
+    const std::map<StringData, std::set<IndexType>>& allowedFieldNames,
     std::function<void(const BSONElement&, BSONObjBuilder*)> indexSpecHandleFn) {
+    const auto key = indexSpec.getObjectField(IndexDescriptor::kKeyPatternFieldName);
+    const auto indexName = IndexNames::nameToType(IndexNames::findPluginName(key));
     BSONObjBuilder builder;
     for (const auto& indexSpecElem : indexSpec) {
         StringData fieldName = indexSpecElem.fieldNameStringData();
-        if (allowedFieldNames.count(fieldName)) {
+        auto it = allowedFieldNames.find(fieldName);
+        if (it != allowedFieldNames.end() &&
+            (it->second.empty() || it->second.count(indexName) != 0)) {
             indexSpecHandleFn(indexSpecElem, &builder);
         } else {
             LOGV2_WARNING(23878,
                           "Removing unknown field from index spec",
-                          "namespace"_attr = redact(ns.toString()),
+                          "namespace"_attr = redact(toStringForLogging(ns)),
                           "fieldName"_attr = redact(fieldName),
                           "indexSpec"_attr = redact(indexSpec));
         }
@@ -130,7 +137,9 @@ BSONObj buildRepairedIndexSpec(
 }
 }  // namespace
 
-Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion indexVersion) {
+Status validateKeyPattern(const BSONObj& key,
+                          IndexDescriptor::IndexVersion indexVersion,
+                          bool checkFCV) {
     const ErrorCodes::Error code = ErrorCodes::CannotCreateIndex;
 
     if (key.objsize() > 2048)
@@ -149,6 +158,23 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
             return Status(code, str::stream() << "Unknown index plugin '" << pluginName << '\'');
     }
 
+    // (Ignore FCV check): This is intentional because we want clusters which have wildcard indexes
+    // still be able to use the feature even if the FCV is downgraded.
+    auto compoundWildcardIndexesAllowed =
+        feature_flags::gFeatureFlagCompoundWildcardIndexes.isEnabledAndIgnoreFCVUnsafe();
+    const auto fcvSnapshot = serverGlobalParams.featureCompatibility.acquireFCVSnapshot();
+    if (fcvSnapshot.isVersionInitialized() && checkFCV) {
+        compoundWildcardIndexesAllowed =
+            feature_flags::gFeatureFlagCompoundWildcardIndexes.isEnabled(fcvSnapshot);
+    }
+
+    if (pluginName == IndexNames::WILDCARD && compoundWildcardIndexesAllowed) {
+        auto status = validateWildcardIndex(key);
+        if (!status.isOK()) {
+            return status;
+        }
+    }
+
     BSONObjIterator it(key);
     while (it.more()) {
         BSONElement keyElement = it.next();
@@ -171,7 +197,8 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
                         return {code, "Values in the index key pattern cannot be NaN."};
                     } else if (value == 0.0) {
                         return {code, "Values in the index key pattern cannot be 0."};
-                    } else if (value < 0.0 && pluginName == IndexNames::WILDCARD) {
+                    } else if (value < 0.0 && pluginName == IndexNames::WILDCARD &&
+                               !compoundWildcardIndexesAllowed) {
                         return {code,
                                 "A numeric value in a $** index key pattern must be positive."};
                     }
@@ -197,14 +224,17 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
                                         << "' index must be a non-zero number, not a string.");
         }
 
-        if (pluginName == IndexNames::WILDCARD || pluginName == IndexNames::COLUMN) {
-            StringData fieldName(keyElement.fieldNameStringData());
+        StringData fieldName(keyElement.fieldNameStringData());
+
+        // TODO SERVER-68303: Remove the CompoundWildcardIndexes feature flag.
+        if ((pluginName == IndexNames::WILDCARD && !compoundWildcardIndexesAllowed) ||
+            pluginName == IndexNames::COLUMN) {
             if (key.nFields() != 1) {
-                // Some special index types do not support compound indexes.
+                // Columnstore and wildcard indexes do not support compound indexes.
                 return Status(code,
                               str::stream() << pluginName << " indexes do not allow compounding");
-            } else if ((fieldName != "$**") && !fieldName.endsWith(".$**")) {
-                // Invalid key names for wildcard or columnstore are not supported.
+            } else if (!WildcardNames::isWildcardFieldName(fieldName)) {
+                // Invalid key names for columnstore are not supported.
                 return Status(code,
                               str::stream() << "Invalid key name for " << pluginName << " indexes");
             }
@@ -272,7 +302,7 @@ BSONObj removeUnknownFields(const NamespaceString& ns, const BSONObj& indexSpec)
 
 BSONObj repairIndexSpec(const NamespaceString& ns,
                         const BSONObj& indexSpec,
-                        const std::set<StringData>& allowedFieldNames) {
+                        const std::map<StringData, std::set<IndexType>>& allowedFieldNames) {
     auto fixIndexSpecFn = [&indexSpec, &ns](const BSONElement& indexSpecElem,
                                             BSONObjBuilder* builder) {
         StringData fieldName = indexSpecElem.fieldNameStringData();
@@ -285,15 +315,17 @@ BSONObj repairIndexSpec(const NamespaceString& ns,
             !indexSpecElem.isNumber() && !indexSpecElem.isBoolean() && indexSpecElem.trueValue()) {
             LOGV2_WARNING(6444400,
                           "Fixing boolean field from index spec",
-                          "namespace"_attr = redact(ns.toString()),
+                          "namespace"_attr = redact(toStringForLogging(ns)),
                           "fieldName"_attr = redact(fieldName),
                           "indexSpec"_attr = redact(indexSpec));
             builder->appendBool(fieldName, true);
         } else if (IndexDescriptor::kExpireAfterSecondsFieldName == fieldName &&
-                   !(indexSpecElem.isNumber() && !indexSpecElem.isNaN())) {
+                   !validateExpireAfterSeconds(indexSpecElem,
+                                               ValidateExpireAfterSecondsMode::kSecondaryTTLIndex)
+                        .isOK()) {
             LOGV2_WARNING(6835900,
                           "Fixing expire field from TTL index spec",
-                          "namespace"_attr = redact(ns.toString()),
+                          "namespace"_attr = redact(toStringForLogging(ns)),
                           "fieldName"_attr = redact(fieldName),
                           "indexSpec"_attr = redact(indexSpec));
             builder->appendNumber(fieldName,
@@ -306,11 +338,13 @@ BSONObj repairIndexSpec(const NamespaceString& ns,
     return buildRepairedIndexSpec(ns, indexSpec, allowedFieldNames, fixIndexSpecFn);
 }
 
-StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& indexSpec) {
+StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx,
+                                      const BSONObj& indexSpec,
+                                      bool checkFCV) {
     bool hasKeyPatternField = false;
     bool hasIndexNameField = false;
     bool hasNamespaceField = false;
-    bool isTTLIndexWithNaNExpireAfterSeconds = false;
+    bool isTTLIndexWithInvalidExpireAfterSeconds = false;
     bool hasVersionField = false;
     bool hasCollationField = false;
     bool hasWeightsField = false;
@@ -351,7 +385,7 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
             }
 
             indexType = IndexNames::findPluginName(keyPattern);
-            if (apiStrict && indexType == IndexNames::TEXT) {
+            if (apiStrict && (indexType == IndexNames::TEXT || indexType == IndexNames::COLUMN)) {
                 return {ErrorCodes::APIStrictError,
                         str::stream()
                             << indexType << " indexes cannot be created with apiStrict: true"};
@@ -360,7 +394,7 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
             // Here we always validate the key pattern according to the most recent rules, in order
             // to enforce that all new indexes have well-formed key patterns.
             Status keyPatternValidateStatus =
-                validateKeyPattern(keyPattern, IndexDescriptor::kLatestIndexVersion);
+                validateKeyPattern(keyPattern, IndexDescriptor::kLatestIndexVersion, checkFCV);
             if (!keyPatternValidateStatus.isOK()) {
                 return keyPatternValidateStatus;
             }
@@ -371,6 +405,13 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                             str::stream()
                                 << "Values in the index key pattern cannot be empty strings"};
                 }
+                if (indexType == IndexNames::WILDCARD &&
+                    keyElement.fieldNameStringData() == "$**" && keyPattern.nFields() > 1 &&
+                    !indexSpec.hasField(IndexDescriptor::kWildcardProjectionFieldName)) {
+                    return {ErrorCodes::CannotCreateIndex,
+                            "Compound wildcard indexes on all fields must also specify "
+                            "'wildcardProjection' option"};
+                }
             }
 
             hasKeyPatternField = true;
@@ -511,8 +552,21 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                                       << "' field can't be an empty object"};
             }
             try {
-                // We use createProjectionExecutor to parse and validate the path projection spec.
                 if (isWildcard) {
+                    // (Ignore FCV check): This is intentional because we want clusters which have
+                    // wildcard indexes still be able to use the feature even if the FCV is
+                    // downgraded.
+                    if (key.nFields() > 1 &&
+                        feature_flags::gFeatureFlagCompoundWildcardIndexes
+                            .isEnabledAndIgnoreFCVUnsafe()) {
+                        auto validationStatus =
+                            validateWildcardProjection(key, indexSpecElem.embeddedObject());
+                        if (!validationStatus.isOK()) {
+                            return validationStatus;
+                        }
+                    }
+                    // We use createProjectionExecutor to parse and validate the path projection
+                    // spec. call here
                     WildcardKeyGenerator::createProjectionExecutor(key,
                                                                    indexSpecElem.embeddedObject());
                 } else {
@@ -563,14 +617,40 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                     IndexDescriptor::k2dIndexMinFieldName == indexSpecElemFieldName ||
                     IndexDescriptor::k2dIndexMaxFieldName == indexSpecElemFieldName ||
                     IndexDescriptor::k2dsphereCoarsestIndexedLevel == indexSpecElemFieldName ||
-                    IndexDescriptor::k2dsphereFinestIndexedLevel == indexSpecElemFieldName) &&
+                    IndexDescriptor::k2dsphereFinestIndexedLevel == indexSpecElemFieldName ||
+                    IndexDescriptor::kBucketSizeFieldName == indexSpecElemFieldName) &&
                    !indexSpecElem.isNumber()) {
             return {ErrorCodes::TypeMismatch,
                     str::stream() << "The field '" << indexSpecElemFieldName
                                   << "' must be a number, but got "
                                   << typeName(indexSpecElem.type())};
-        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == indexSpecElemFieldName) {
-            isTTLIndexWithNaNExpireAfterSeconds = indexSpecElem.isNaN();
+        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == indexSpecElemFieldName &&
+                   !validateExpireAfterSeconds(indexSpecElem,
+                                               ValidateExpireAfterSecondsMode::kSecondaryTTLIndex)
+                        .isOK() &&
+                   !skipTTLIndexInvalidExpireAfterSecondsValidationForCreateIndex.shouldFail()) {
+            isTTLIndexWithInvalidExpireAfterSeconds = true;
+        } else if (IndexDescriptor::kColumnStoreCompressorFieldName == indexSpecElemFieldName) {
+            if (IndexNames::findPluginName(indexSpec.getObjectField(
+                    IndexDescriptor::kKeyPatternFieldName)) != IndexNames::COLUMN) {
+                return {ErrorCodes::InvalidIndexSpecificationOption,
+                        str::stream()
+                            << "The field '" << indexSpecElemFieldName << "' is only allowed in '"
+                            << IndexDescriptor::kColumnStoreCompressorFieldName << "' indexes"};
+            }
+            if (indexSpecElem.type() != BSONType::String) {
+                return {ErrorCodes::TypeMismatch,
+                        str::stream()
+                            << "The field '" << IndexDescriptor::kColumnStoreCompressorFieldName
+                            << "' must be a string, but got " << typeName(indexSpecElem.type())};
+            }
+            if (!ColumnStoreAccessMethod::supportsBlockCompressor(
+                    indexSpecElem.valueStringData())) {
+                return {ErrorCodes::InvalidIndexSpecificationOption,
+                        str::stream() << "Unsupported value for "
+                                      << IndexDescriptor::kColumnStoreCompressorFieldName << ": "
+                                      << indexSpecElem.valueStringData()};
+            }
         } else {
             // We can assume field name is valid at this point. Validation of fieldname is handled
             // prior to this in validateIndexSpecFieldNames().
@@ -642,10 +722,9 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
         modifiedSpec = modifiedSpec.removeField(IndexDescriptor::kNamespaceFieldName);
     }
 
-    if (isTTLIndexWithNaNExpireAfterSeconds &&
-        !skipTTLIndexNaNExpireAfterSecondsValidation.shouldFail()) {
+    if (isTTLIndexWithInvalidExpireAfterSeconds) {
         // We create a new index specification with the 'expireAfterSeconds' field set as
-        // kExpireAfterSecondsForInactiveTTLIndex if the current value is NaN. A similar
+        // kExpireAfterSecondsForInactiveTTLIndex if the current value is invalid. A similar
         // treatment is done in repairIndexSpec(). This rewrites the 'expireAfterSeconds'
         // value to be compliant with the 'safeInt' IDL type for the listIndexes response.
         BSONObjBuilder builder;
@@ -665,7 +744,7 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
 
     if (hasOriginalSpecField) {
         StatusWith<BSONObj> modifiedOriginalSpec = validateIndexSpec(
-            opCtx, indexSpec.getObjectField(IndexDescriptor::kOriginalSpecFieldName));
+            opCtx, indexSpec.getObjectField(IndexDescriptor::kOriginalSpecFieldName), checkFCV);
         if (!modifiedOriginalSpec.isOK()) {
             return modifiedOriginalSpec.getStatus();
         }
@@ -856,6 +935,38 @@ Status validateExpireAfterSeconds(std::int64_t expireAfterSeconds,
     return Status::OK();
 }
 
+Status validateExpireAfterSeconds(BSONElement expireAfterSeconds,
+                                  ValidateExpireAfterSecondsMode mode) {
+    if (!expireAfterSeconds.isNumber()) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+                              << "' option must be numeric, but received a type of '"
+                              << typeName(expireAfterSeconds.type())};
+    }
+
+    if (expireAfterSeconds.isNaN()) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+                              << "' option must not be NaN"};
+    }
+
+    // Clustered indexes allow 64-bit integers for expireAfterSeconds, but secondary indexes only
+    // allow 32-bit integers, so we check the range here for secondary indexes.
+    if (mode == ValidateExpireAfterSecondsMode::kSecondaryTTLIndex &&
+        expireAfterSeconds.safeNumberInt() != expireAfterSeconds.safeNumberLong()) {
+        return {ErrorCodes::CannotCreateIndex,
+                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
+                              << "' must be within the range of a 32-bit integer"};
+    }
+
+    if (auto status = validateExpireAfterSeconds(expireAfterSeconds.safeNumberLong(), mode);
+        !status.isOK()) {
+        return {ErrorCodes::CannotCreateIndex, str::stream() << status.reason()};
+    }
+
+    return Status::OK();
+}
+
 bool isIndexTTL(const BSONObj& indexSpec) {
     return indexSpec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName);
 }
@@ -865,22 +976,11 @@ Status validateIndexSpecTTL(const BSONObj& indexSpec) {
         return Status::OK();
     }
 
-    const BSONElement expireAfterSecondsElt =
-        indexSpec[IndexDescriptor::kExpireAfterSecondsFieldName];
-    if (!expireAfterSecondsElt.isNumber()) {
-        return {ErrorCodes::CannotCreateIndex,
-                str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
-                              << "' option must be numeric, but received a type of '"
-                              << typeName(expireAfterSecondsElt.type())
-                              << "'. Index spec: " << indexSpec};
-    }
-
     if (auto status =
-            validateExpireAfterSeconds(expireAfterSecondsElt.safeNumberLong(),
+            validateExpireAfterSeconds(indexSpec[IndexDescriptor::kExpireAfterSecondsFieldName],
                                        ValidateExpireAfterSecondsMode::kSecondaryTTLIndex);
         !status.isOK()) {
-        return {ErrorCodes::CannotCreateIndex,
-                str::stream() << status.reason() << ". Index spec: " << indexSpec};
+        return status.withContext(str::stream() << ". Index spec: " << indexSpec);
     }
 
     const BSONObj key = indexSpec["key"].Obj();
@@ -900,6 +1000,40 @@ bool isIndexAllowedInAPIVersion1(const IndexDescriptor& indexDesc) {
         !indexDesc.isSparse();
 }
 
+BSONObj parseAndValidateIndexSpecs(OperationContext* opCtx,
+                                   const BSONObj& indexSpecObj,
+                                   bool checkFCV) {
+    constexpr auto k_id_ = "_id_"_sd;
+    constexpr auto kStar = "*"_sd;
+
+    BSONObj parsedIndexSpec = indexSpecObj;
+
+    auto indexSpecStatus = index_key_validate::validateIndexSpec(opCtx, parsedIndexSpec, checkFCV);
+    uassertStatusOK(indexSpecStatus.getStatus().withContext(
+        str::stream() << "Error in specification " << parsedIndexSpec.toString()));
+
+    auto indexSpec = indexSpecStatus.getValue();
+    if (IndexDescriptor::isIdIndexPattern(indexSpec[IndexDescriptor::kKeyPatternFieldName].Obj())) {
+        uassertStatusOK(index_key_validate::validateIdIndexSpec(indexSpec));
+    } else {
+        uassert(ErrorCodes::BadValue,
+                str::stream() << "The index name '_id_' is reserved for the _id index, "
+                                 "which must have key pattern {_id: 1}, found "
+                              << indexSpec[IndexDescriptor::kKeyPatternFieldName],
+                indexSpec[IndexDescriptor::kIndexNameFieldName].String() != k_id_);
+
+        // An index named '*' cannot be dropped on its own, because a dropIndex oplog
+        // entry with a '*' as an index name means "drop all indexes in this
+        // collection".  We disallow creation of such indexes to avoid this conflict.
+        uassert(ErrorCodes::BadValue,
+                "The index name '*' is not valid.",
+                indexSpec[IndexDescriptor::kIndexNameFieldName].String() != kStar);
+    }
+
+    return indexSpec;
+}
+
+
 GlobalInitializerRegisterer filterAllowedIndexFieldNamesInitializer(
     "FilterAllowedIndexFieldNames", [](InitializerContext* service) {
         if (filterAllowedIndexFieldNames)
