@@ -45,6 +45,7 @@
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/query/explain_options.h"
 #include "mongo/db/read_concern_support_result.h"
 #include "mongo/db/repl/read_concern_args.h"
@@ -106,18 +107,21 @@ public:
     }
 
     /**
-     * A behavior to perform after CommandInvocation::run()
+     * A behavior to perform after CommandInvocation::run(). Note that the response argument is not
+     * const, because the ReplyBuilderInterface does not expose any const methods to inspect the
+     * response body. However, onAfterRun must not mutate the response body.
      */
     virtual void onAfterRun(OperationContext* opCtx,
                             const OpMsgRequest& request,
-                            CommandInvocation* invocation) = 0;
+                            CommandInvocation* invocation,
+                            rpc::ReplyBuilderInterface* response) = 0;
 
     /**
      * A behavior to perform after CommandInvocation::asyncRun(). Defaults to `onAfterRun(...)`.
      */
     virtual void onAfterAsyncRun(std::shared_ptr<RequestExecutionContext> rec,
                                  CommandInvocation* invocation) {
-        onAfterRun(rec->getOpCtx(), rec->getRequest(), invocation);
+        onAfterRun(rec->getOpCtx(), rec->getRequest(), invocation, rec->getReplyBuilder());
     }
 };
 
@@ -135,9 +139,6 @@ struct CommandHelpers {
     // The first field is interpreted as a collection name.
     static NamespaceString parseNsCollectionRequired(const DatabaseName& dbName,
                                                      const BSONObj& cmdObj);
-    // TODO SERVER-68421: Remove this method once all call sites have been updated to pass
-    // DatabaseName.
-    static NamespaceString parseNsCollectionRequired(StringData dbname, const BSONObj& cmdObj);
 
     static NamespaceStringOrUUID parseNsOrUUID(const DatabaseName& dbName, const BSONObj& cmdObj);
 
@@ -164,7 +165,7 @@ struct CommandHelpers {
      * pattern or a database resource pattern, depending on whether parseNs returns a fully qualifed
      * collection name or just a database name.
      */
-    static ResourcePattern resourcePatternForNamespace(const std::string& ns);
+    static ResourcePattern resourcePatternForNamespace(StringData ns);
 
     static Command* findCommand(StringData name);
 
@@ -361,6 +362,13 @@ struct CommandHelpers {
      */
     static void handleMarkKillOnClientDisconnect(OperationContext* opCtx,
                                                  bool shouldMarkKill = true);
+
+    /**
+     * Provides diagnostics if the reply builder contains an internal-only error, and will cause
+     * deferred-fatality when testing diagnostics is enabled.
+     */
+    static void checkForInternalError(rpc::ReplyBuilderInterface* replyBuilder,
+                                      bool isInternalClient);
 };
 
 /**
@@ -370,6 +378,7 @@ class Command {
 public:
     using CommandMap = StringMap<Command*>;
     enum class AllowedOnSecondary { kAlways, kNever, kOptIn };
+    enum class HandshakeRole { kNone, kHello, kAuth };
 
     /**
      * Constructs a new command and causes it to be registered with the global commands list. It is
@@ -424,6 +433,13 @@ public:
         return false;
     }
 
+    /**
+     * Returns the role this command has in the connection handshake.
+     */
+    virtual HandshakeRole handshakeRole() const {
+        return HandshakeRole::kNone;
+    }
+
     /*
      * Returns the list of API versions that include this command.
      */
@@ -471,10 +487,11 @@ public:
     }
 
     /**
-     * Override and return true if the readConcernCounters in serverStatus should not be incremented
-     * on behalf of this command.
+     * Override and return true if the readConcernCounters and readPreferenceCounters in
+     * serverStatus should be incremented on behalf of this command. This should be true for
+     * read operations.
      */
-    virtual bool shouldAffectReadConcernCounter() const {
+    virtual bool shouldAffectReadOptionCounters() const {
         return false;
     }
 
@@ -643,6 +660,14 @@ public:
         return false;
     }
 
+    /**
+     * Override to true if this command should be allowed on a direct shard connection regardless
+     * of the directShardOperations ActionType.
+     */
+    virtual bool shouldSkipDirectConnectionChecks() const {
+        return false;
+    }
+
 private:
     // The full name of the command
     const std::string _name;
@@ -856,9 +881,8 @@ public:
         return CommandHelpers::parseNsFromCommand(dbName, cmdObj);
     }
 
-    ResourcePattern parseResourcePattern(const std::string& dbname, const BSONObj& cmdObj) const {
-        return CommandHelpers::resourcePatternForNamespace(
-            parseNs({boost::none, dbname}, cmdObj).ns());
+    ResourcePattern parseResourcePattern(const DatabaseName& dbName, const BSONObj& cmdObj) const {
+        return CommandHelpers::resourcePatternForNamespace(parseNs(dbName, cmdObj).ns());
     }
 
     //
@@ -905,11 +929,13 @@ public:
 
     /**
      * Checks if the client associated with the given OperationContext is authorized to run this
-     * command. Default implementation defers to checkAuthForCommand.
+     * command.
+     * Command imlpementations MUST provide a method here, even if no authz checks are required.
+     * Such commands should return Status::OK(), with a comment stating "No auth required".
      */
     virtual Status checkAuthForOperation(OperationContext* opCtx,
-                                         const std::string& dbname,
-                                         const BSONObj& cmdObj) const;
+                                         const DatabaseName& dbName,
+                                         const BSONObj& cmdObj) const = 0;
 
     /**
      * supportsWriteConcern returns true if this command should be parsed for a writeConcern
@@ -972,32 +998,6 @@ public:
 private:
     std::unique_ptr<CommandInvocation> parse(OperationContext* opCtx,
                                              const OpMsgRequest& request) final;
-
-    //
-    // Deprecated virtual methods.
-    //
-
-    /**
-     * Checks if the given client is authorized to run this command on database "dbname"
-     * with the invocation described by "cmdObj".
-     *
-     * NOTE: Implement checkAuthForOperation that takes an OperationContext* instead.
-     */
-    virtual Status checkAuthForCommand(Client* client,
-                                       const std::string& dbname,
-                                       const BSONObj& cmdObj) const;
-
-    /**
-     * Appends to "*out" the privileges required to run this command on database "dbname" with
-     * the invocation described by "cmdObj".  New commands shouldn't implement this, they should
-     * implement checkAuthForOperation (which takes an OperationContext*), instead.
-     */
-    virtual void addRequiredPrivileges(const std::string& dbname,
-                                       const BSONObj& cmdObj,
-                                       std::vector<Privilege>* out) const {
-        // The default implementation of addRequiredPrivileges should never be hit.
-        fassertFailed(16940);
-    }
 };
 
 /**
@@ -1157,10 +1157,10 @@ private:
     static RequestType _parseRequest(OperationContext* opCtx,
                                      const DatabaseName& dbName,
                                      const BSONObj& cmdObj) {
-        // TODO SERVER-67155 pass tenantId to the BSONObj parse function
         return RequestType::parse(
             IDLParserContext(RequestType::kCommandName,
-                             APIParameters::get(opCtx).getAPIStrict().value_or(false)),
+                             APIParameters::get(opCtx).getAPIStrict().value_or(false),
+                             dbName.tenantId()),
             cmdObj);
     }
 
@@ -1404,7 +1404,7 @@ CommandRegistry* globalCommandRegistry();
  */
 #define MONGO_REGISTER_FEATURE_FLAGGED_COMMAND(CmdType, featureFlag)        \
     MONGO_INITIALIZER(RegisterTestCommand_##CmdType)(InitializerContext*) { \
-        if (featureFlag.isEnabledAndIgnoreFCV()) {                          \
+        if (featureFlag.isEnabledAndIgnoreFCVUnsafeAtStartup()) {           \
             new CmdType();                                                  \
         }                                                                   \
     }
