@@ -61,6 +61,22 @@ namespace mongo {
 
 namespace {
 
+constexpr char SKIP_TEMP_COLLECTION[] = "skipTempCollections";
+
+std::shared_ptr<const CollectionCatalog> getConsistentCatalogAndSnapshot(OperationContext* opCtx) {
+    // Loop until we get a consistent catalog and snapshot. This is only used for the lock-free
+    // implementation of dbHash which skips acquiring database and collection locks.
+    while (true) {
+        const auto catalogBeforeSnapshot = CollectionCatalog::get(opCtx);
+        opCtx->recoveryUnit()->preallocateSnapshot();
+        const auto catalogAfterSnapshot = CollectionCatalog::get(opCtx);
+        if (catalogBeforeSnapshot == catalogAfterSnapshot) {
+            return catalogBeforeSnapshot;
+        }
+        opCtx->recoveryUnit()->abandonSnapshot();
+    }
+}
+
 class DBHashCmd : public BasicCommand {
 public:
     DBHashCmd() : BasicCommand("dbHash", "dbhash") {}
@@ -106,12 +122,16 @@ public:
                 kDefaultReadConcernNotPermitted};
     }
 
-    virtual void addRequiredPrivileges(const std::string& dbname,
-                                       const BSONObj& cmdObj,
-                                       std::vector<Privilege>* out) const {
-        ActionSet actions;
-        actions.addAction(ActionType::dbHash);
-        out->push_back(Privilege(ResourcePattern::forDatabaseName(dbname), actions));
+    Status checkAuthForOperation(OperationContext* opCtx,
+                                 const DatabaseName& dbName,
+                                 const BSONObj& cmdObj) const override {
+        auto* as = AuthorizationSession::get(opCtx->getClient());
+        if (!as->isAuthorizedForActionsOnResource(ResourcePattern::forDatabaseName(dbName.db()),
+                                                  ActionType::dbHash)) {
+            return {ErrorCodes::Unauthorized, "unauthorized"};
+        }
+
+        return Status::OK();
     }
 
     bool run(OperationContext* opCtx,
@@ -132,12 +152,16 @@ public:
             }
         }
 
-        // For empty databasename on first command field, the following code depends on the "."
-        // on ns to find the invalid empty db name instead of checking empty db name directly.
-        const std::string ns = parseNs(dbName, cmdObj).ns();
+        const bool skipTempCollections =
+            cmdObj.hasField(SKIP_TEMP_COLLECTION) && cmdObj[SKIP_TEMP_COLLECTION].trueValue();
+        if (skipTempCollections) {
+            LOGV2(6859700, "Skipping hash computation for temporary collections");
+        }
+
         uassert(ErrorCodes::InvalidNamespace,
-                str::stream() << "Invalid db name: " << ns,
-                NamespaceString::validDBName(ns, NamespaceString::DollarInDbNameBehavior::Allow));
+                "Cannot pass empty string for 'dbHash' field",
+                !(cmdObj.firstElement().type() == mongo::String &&
+                  cmdObj.firstElement().valueStringData().empty()));
 
         if (auto elem = cmdObj["$_internalReadAtClusterTime"]) {
             uassert(ErrorCodes::InvalidOptions,
@@ -204,26 +228,40 @@ public:
             opCtx->recoveryUnit()->setPrepareConflictBehavior(PrepareConflictBehavior::kEnforce);
         }
 
-        // We lock the entire database in S-mode in order to ensure that the contents will not
-        // change for the snapshot.
-        auto lockMode = LockMode::MODE_S;
-        boost::optional<ShouldNotConflictWithSecondaryBatchApplicationBlock> shouldNotConflictBlock;
-        if (opCtx->recoveryUnit()->getTimestampReadSource() ==
-            RecoveryUnit::ReadSource::kProvided) {
-            // However, if we are performing a read at a timestamp, then we only need to lock the
-            // database in intent mode and then collection in intent mode as well to ensure that
-            // none of the collections get dropped.
-            lockMode = LockMode::MODE_IS;
+        const bool isPointInTimeRead =
+            opCtx->recoveryUnit()->getTimestampReadSource() == RecoveryUnit::ReadSource::kProvided;
 
-            // Additionally, if we are performing a read at a timestamp, then we allow oplog
-            // application to proceed concurrently with the dbHash command. This is done
-            // to ensure a prepare conflict is able to eventually be resolved by processing a
-            // later commitTransaction or abortTransaction oplog entry.
+        boost::optional<ShouldNotConflictWithSecondaryBatchApplicationBlock> shouldNotConflictBlock;
+        if (isPointInTimeRead) {
+            // If we are performing a read at a timestamp, then we allow oplog application to
+            // proceed concurrently with the dbHash command. This is done to ensure a prepare
+            // conflict is able to eventually be resolved by processing a later commitTransaction or
+            // abortTransaction oplog entry.
             shouldNotConflictBlock.emplace(opCtx->lockState());
         }
 
-        AutoGetDb autoDb(opCtx, dbName, lockMode);
-        Database* db = autoDb.getDb();
+        // We take the global lock here as dbHash runs lock-free with point-in-time catalog lookups.
+        Lock::GlobalLock globalLock(opCtx, MODE_IS);
+
+        // The CollectionCatalog to use for lock-free reads with point-in-time catalog lookups.
+        std::shared_ptr<const CollectionCatalog> catalog;
+        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
+            // Make sure we get a CollectionCatalog in sync with our snapshot.
+            catalog = getConsistentCatalogAndSnapshot(opCtx);
+        }
+
+        boost::optional<AutoGetDb> autoDb;
+        if (isPointInTimeRead) {
+            // We only need to lock the database in intent mode and then collection in intent
+            // mode as well to ensure that none of the collections get dropped.
+            // TODO:SERVER-75848 Make this lock-free
+            autoDb.emplace(opCtx, dbName, MODE_IS);
+        } else {
+            // We lock the entire database in S-mode in order to ensure that the contents will not
+            // change for the snapshot when not reading at a timestamp.
+            autoDb.emplace(opCtx, dbName, MODE_S);
+        }
 
         result.append("host", prettyHostName());
 
@@ -234,61 +272,117 @@ public:
         std::map<std::string, UUID> collectionToUUIDMap;
         std::set<std::string> cappedCollectionSet;
 
-        catalog::forEachCollectionFromDb(
-            opCtx, dbName, MODE_IS, [&](const CollectionPtr& collection) {
-                auto collNss = collection->ns();
+        auto checkAndHashCollection = [&](const Collection* collection) -> bool {
+            auto collNss = collection->ns();
 
-                uassert(ErrorCodes::BadValue,
-                        str::stream() << "weird fullCollectionName [" << collNss.toString() << "]",
-                        collNss.size() - 1 > dbName.db().size());
-
-                if (repl::ReplicationCoordinator::isOplogDisabledForNS(collNss)) {
-                    return true;
-                }
-
-                if (collNss.coll().startsWith("tmp.mr.")) {
-                    // We skip any incremental map reduce collections as they also aren't
-                    // replicated.
-                    return true;
-                }
-
-                if (desiredCollections.size() > 0 &&
-                    desiredCollections.count(collNss.coll().toString()) == 0)
-                    return true;
-
-                // Don't include 'drop pending' collections.
-                if (collNss.isDropPendingNamespace())
-                    return true;
-
-                if (collection->isCapped()) {
-                    cappedCollectionSet.insert(collNss.coll().toString());
-                }
-
-                collectionToUUIDMap.emplace(collNss.coll().toString(), collection->uuid());
-
-                // Compute the hash for this collection.
-                std::string hash = _hashCollection(opCtx, db, collNss);
-
-                collectionToHashMap[collNss.coll().toString()] = hash;
+            uassert(ErrorCodes::BadValue,
+                    str::stream() << "weird fullCollectionName [" << collNss.toString() << "]",
+                    collNss.size() - 1 > dbName.db().size());
 
+            if (repl::ReplicationCoordinator::isOplogDisabledForNS(collNss)) {
                 return true;
-            });
+            }
+
+            if (collNss.coll().startsWith("tmp.mr.")) {
+                // We skip any incremental map reduce collections as they also aren't
+                // replicated.
+                return true;
+            }
+
+            if (skipTempCollections && collection->isTemporary()) {
+                return true;
+            }
+
+            if (desiredCollections.size() > 0 &&
+                desiredCollections.count(collNss.coll().toString()) == 0)
+                return true;
+
+            // Don't include 'drop pending' collections.
+            if (collNss.isDropPendingNamespace())
+                return true;
+
+            if (collection->isCapped()) {
+                cappedCollectionSet.insert(collNss.coll().toString());
+            }
+
+            collectionToUUIDMap.emplace(collNss.coll().toString(), collection->uuid());
+
+            // Compute the hash for this collection.
+            std::string hash = _hashCollection(opCtx, CollectionPtr(collection));
+
+            collectionToHashMap[collNss.coll().toString()] = hash;
+
+            return true;
+        };
+
+        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
+            for (auto&& coll : catalog->range(dbName)) {
+                UUID uuid = coll->uuid();
+
+                // The namespace must be found as the UUID is fetched from the same
+                // CollectionCatalog instance.
+                boost::optional<NamespaceString> nss = catalog->lookupNSSByUUID(opCtx, uuid);
+                invariant(nss);
+
+                // TODO:SERVER-75848 Make this lock-free
+                Lock::CollectionLock clk(opCtx, *nss, MODE_IS);
+
+                const Collection* collection = nullptr;
+                if (nss->isGlobalIndex()) {
+                    // TODO SERVER-74209: Reading earlier than the minimum valid snapshot is not
+                    // supported for global indexes. It appears that the primary and secondaries
+                    // apply operations differently resulting in hash mismatches. This requires
+                    // further investigation. In the meantime, global indexes use the behaviour
+                    // prior to point-in-time lookups.
+                    collection = coll;
+
+                    if (auto readTimestamp =
+                            opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx)) {
+                        auto minSnapshot = coll->getMinimumValidSnapshot();
+                        uassert(ErrorCodes::SnapshotUnavailable,
+                                str::stream()
+                                    << "Unable to read from a snapshot due to pending collection"
+                                       " catalog changes; please retry the operation. Snapshot"
+                                       " timestamp is "
+                                    << readTimestamp->toString()
+                                    << ". Collection minimum timestamp is "
+                                    << minSnapshot->toString(),
+                                !minSnapshot || *readTimestamp >= *minSnapshot);
+                    }
+                } else {
+                    collection = catalog->establishConsistentCollection(
+                        opCtx,
+                        {dbName, uuid},
+                        opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx));
+
+                    if (!collection) {
+                        // The collection did not exist at the read timestamp with the given UUID.
+                        continue;
+                    }
+                }
+
+                (void)checkAndHashCollection(collection);
+            }
+        } else {
+            catalog::forEachCollectionFromDb(opCtx, dbName, MODE_IS, checkAndHashCollection);
+        }
 
         BSONObjBuilder bb(result.subobjStart("collections"));
         BSONArrayBuilder cappedCollections;
         BSONObjBuilder collectionsByUUID;
 
-        for (auto elem : cappedCollectionSet) {
+        for (const auto& elem : cappedCollectionSet) {
             cappedCollections.append(elem);
         }
 
-        for (auto entry : collectionToUUIDMap) {
+        for (const auto& entry : collectionToUUIDMap) {
             auto collName = entry.first;
             auto uuid = entry.second;
             uuid.appendToBuilder(&collectionsByUUID, collName);
         }
 
-        for (auto entry : collectionToHashMap) {
+        for (const auto& entry : collectionToHashMap) {
             auto collName = entry.first;
             auto hash = entry.second;
             bb.append(collName, hash);
@@ -311,12 +405,7 @@ public:
     }
 
 private:
-    std::string _hashCollection(OperationContext* opCtx, Database* db, const NamespaceString& nss) {
-
-        CollectionPtr collection =
-            CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
-        invariant(collection);
-
+    std::string _hashCollection(OperationContext* opCtx, const CollectionPtr& collection) {
         boost::optional<Lock::CollectionLock> collLock;
         if (opCtx->recoveryUnit()->getTimestampReadSource() ==
             RecoveryUnit::ReadSource::kProvided) {
@@ -324,7 +413,7 @@ private:
             // intent mode. We need to also acquire the collection lock in intent mode to ensure
             // reading from the consistent snapshot doesn't overlap with any catalog operations on
             // the collection.
-            invariant(opCtx->lockState()->isCollectionLockedForMode(nss, MODE_IS));
+            invariant(opCtx->lockState()->isCollectionLockedForMode(collection->ns(), MODE_IS));
 
             auto minSnapshot = collection->getMinimumVisibleSnapshot();
             auto mySnapshot = opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx);
@@ -338,7 +427,7 @@ private:
                                   << minSnapshot->toString(),
                     !minSnapshot || *mySnapshot >= *minSnapshot);
         } else {
-            invariant(opCtx->lockState()->isDbLockedForMode(db->name(), MODE_S));
+            invariant(opCtx->lockState()->isDbLockedForMode(collection->ns().dbName(), MODE_S));
         }
 
         auto desc = collection->getIndexCatalog()->findIdIndex(opCtx);
@@ -358,7 +447,7 @@ private:
             exec = InternalPlanner::collectionScan(
                 opCtx, &collection, PlanYieldPolicy::YieldPolicy::NO_YIELD);
         } else {
-            LOGV2(20455, "Can't find _id index for namespace", "namespace"_attr = nss);
+            LOGV2(20455, "Can't find _id index for namespace", logAttrs(collection->ns()));
             return "no _id _index";
         }
 
@@ -366,16 +455,14 @@ private:
         md5_init(&st);
 
         try {
-            long long n = 0;
             BSONObj c;
-            verify(nullptr != exec.get());
+            MONGO_verify(nullptr != exec.get());
             while (exec->getNext(&c, nullptr) == PlanExecutor::ADVANCED) {
                 md5_append(&st, (const md5_byte_t*)c.objdata(), c.objsize());
-                n++;
             }
         } catch (DBException& exception) {
             LOGV2_WARNING(
-                20456, "Error while hashing, db possibly dropped", "namespace"_attr = nss);
+                20456, "Error while hashing, db possibly dropped", logAttrs(collection->ns()));
             exception.addContext("Plan executor error while running dbHash command");
             throw;
         }
