@@ -57,8 +57,11 @@
 #include "mongo/db/query/query_planner_common.h"
 #include "mongo/db/query/view_response_formatter.h"
 #include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/s/query_analysis_writer.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/logv2/log.h"
+#include "mongo/s/query_analysis_sampler_util.h"
+#include "mongo/util/database_name_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -102,7 +105,7 @@ public:
         return ReadConcernSupportResult::allSupportedAndDefaultPermitted();
     }
 
-    bool shouldAffectReadConcernCounter() const override {
+    bool shouldAffectReadOptionCounters() const override {
         return true;
     }
 
@@ -119,7 +122,7 @@ public:
     }
 
     Status checkAuthForOperation(OperationContext* opCtx,
-                                 const std::string& dbname,
+                                 const DatabaseName& dbname,
                                  const BSONObj& cmdObj) const override {
         AuthorizationSession* authSession = AuthorizationSession::get(opCtx->getClient());
 
@@ -128,11 +131,10 @@ public:
         }
 
         const auto hasTerm = false;
-        return auth::checkAuthForFind(
-            authSession,
-            CollectionCatalog::get(opCtx)->resolveNamespaceStringOrUUID(
-                opCtx, CommandHelpers::parseNsOrUUID({boost::none, dbname}, cmdObj)),
-            hasTerm);
+        return auth::checkAuthForFind(authSession,
+                                      CollectionCatalog::get(opCtx)->resolveNamespaceStringOrUUID(
+                                          opCtx, CommandHelpers::parseNsOrUUID(dbname, cmdObj)),
+                                      hasTerm);
     }
 
     bool allowedInTransactions() const final {
@@ -147,14 +149,16 @@ public:
                    const OpMsgRequest& request,
                    ExplainOptions::Verbosity verbosity,
                    rpc::ReplyBuilderInterface* result) const override {
-        const DatabaseName dbName(request.getValidatedTenantId(), request.getDatabase());
+        const DatabaseName dbName =
+            DatabaseNameUtil::deserialize(request.getValidatedTenantId(), request.getDatabase());
         const BSONObj& cmdObj = request.body;
         // Acquire locks. The RAII object is optional, because in the case of a view, the locks
         // need to be released.
         boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> ctx;
-        ctx.emplace(opCtx,
-                    CommandHelpers::parseNsCollectionRequired(dbName, cmdObj),
-                    AutoGetCollectionViewMode::kViewsPermitted);
+        ctx.emplace(
+            opCtx,
+            CommandHelpers::parseNsCollectionRequired(dbName, cmdObj),
+            AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
         const auto nss = ctx->getNss();
 
         const ExtensionsCallbackReal extensionsCallback(opCtx, &nss);
@@ -173,7 +177,9 @@ public:
             }
 
             auto viewAggCmd =
-                OpMsgRequest::fromDBAndBody(nss.db(), viewAggregation.getValue()).body;
+                OpMsgRequestBuilder::createWithValidatedTenancyScope(
+                    nss.dbName(), request.validatedTenancyScope, viewAggregation.getValue())
+                    .body;
             auto viewAggRequest = aggregation_request_helper::parseFromBSON(
                 opCtx,
                 nss,
@@ -184,7 +190,7 @@ public:
             // An empty PrivilegeVector is acceptable because these privileges are only checked on
             // getMore and explain will not open a cursor.
             return runAggregate(
-                opCtx, nss, viewAggRequest, viewAggregation.getValue(), PrivilegeVector(), result);
+                opCtx, viewAggRequest, viewAggregation.getValue(), PrivilegeVector(), result);
         }
 
         const auto& collection = ctx->getCollection();
@@ -206,9 +212,10 @@ public:
         // Acquire locks and resolve possible UUID. The RAII object is optional, because in the case
         // of a view, the locks need to be released.
         boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> ctx;
-        ctx.emplace(opCtx,
-                    CommandHelpers::parseNsOrUUID(dbName, cmdObj),
-                    AutoGetCollectionViewMode::kViewsPermitted);
+        ctx.emplace(
+            opCtx,
+            CommandHelpers::parseNsOrUUID(dbName, cmdObj),
+            AutoGetCollection::Options{}.viewMode(auto_get_collection::ViewMode::kViewsPermitted));
         const auto& nss = ctx->getNss();
 
         if (!ctx->getView()) {
@@ -239,6 +246,16 @@ public:
         if (parsedDistinct.isMirrored()) {
             const auto& invocation = CommandInvocation::get(opCtx);
             invocation->markMirrored();
+        } else if (auto sampleId = analyze_shard_key::getOrGenerateSampleId(
+                       opCtx,
+                       nss,
+                       analyze_shard_key::SampledCommandNameEnum::kDistinct,
+                       parsedDistinct)) {
+            auto cq = parsedDistinct.getQuery();
+            analyze_shard_key::QueryAnalysisWriter::get(opCtx)
+                ->addDistinctQuery(
+                    *sampleId, nss, cq->getQueryObj(), cq->getFindCommandRequest().getCollation())
+                .getAsync([](auto) {});
         }
 
         if (ctx->getView()) {
@@ -248,10 +265,17 @@ public:
             auto viewAggregation = parsedDistinct.asAggregationCommand();
             uassertStatusOK(viewAggregation.getStatus());
 
-            BSONObj aggResult = CommandHelpers::runCommandDirectly(
-                opCtx,
-                OpMsgRequest::fromDBAndBody(dbName.db(), std::move(viewAggregation.getValue())));
-            uassertStatusOK(ViewResponseFormatter(aggResult).appendAsDistinctResponse(&result));
+            using VTS = auth::ValidatedTenancyScope;
+            boost::optional<VTS> vts = boost::none;
+            if (dbName.tenantId()) {
+                vts = VTS(dbName.tenantId().value(), VTS::TrustedForInnerOpMsgRequestTag{});
+            }
+            auto aggRequest = OpMsgRequestBuilder::createWithValidatedTenancyScope(
+                dbName, vts, std::move(viewAggregation.getValue()));
+
+            BSONObj aggResult = CommandHelpers::runCommandDirectly(opCtx, aggRequest);
+            uassertStatusOK(ViewResponseFormatter(aggResult).appendAsDistinctResponse(
+                &result, dbName.tenantId()));
             return true;
         }
 
@@ -314,8 +338,6 @@ public:
             auto&& [stats, _] =
                 explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
             LOGV2_WARNING(23797,
-                          "Plan executor error during distinct command: {error}, "
-                          "stats: {stats}, cmd: {cmd}",
                           "Plan executor error during distinct command",
                           "error"_attr = exception.toStatus(),
                           "stats"_attr = redact(stats),
@@ -336,7 +358,7 @@ public:
         }
         curOp->debug().setPlanSummaryMetrics(stats);
 
-        if (curOp->shouldDBProfile(opCtx)) {
+        if (curOp->shouldDBProfile()) {
             auto&& [stats, _] =
                 explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
             curOp->debug().execStats = std::move(stats);
@@ -366,6 +388,7 @@ public:
             keyBob.append("query", 1);
             keyBob.append("collation", 1);
             keyBob.append("shardVersion", 1);
+            keyBob.append("databaseVersion", 1);
             return keyBob.obj();
         }();
 
