@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/auth/authorization_checks.h"
@@ -35,8 +34,8 @@
 #include "mongo/db/catalog/collection_uuid_mismatch.h"
 #include "mongo/db/client.h"
 #include "mongo/db/clientcursor.h"
+#include "mongo/db/collection_type.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/commands/cqf/cqf_command_utils.h"
 #include "mongo/db/commands/run_aggregate.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/cursor_manager.h"
@@ -46,6 +45,7 @@
 #include "mongo/db/fle_crud.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
+#include "mongo/db/pipeline/query_request_conversion.h"
 #include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/cursor_response.h"
@@ -54,7 +54,12 @@
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_shape/query_shape.h"
+#include "mongo/db/query/query_stats/find_key.h"
+#include "mongo/db/query/query_stats/key.h"
+#include "mongo/db/query/query_stats/query_stats.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/s/query_analysis_writer.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/stats/resource_consumption_metrics.h"
@@ -64,6 +69,9 @@
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
+#include "mongo/s/query_analysis_sampler_util.h"
+#include "mongo/util/assert_util.h"
+#include "mongo/util/database_name_util.h"
 #include "mongo/util/fail_point.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
@@ -76,75 +84,29 @@ MONGO_FAIL_POINT_DEFINE(allowExternalReadsForReverseOplogScanRule);
 
 const auto kTermField = "term"_sd;
 
-// Parses the command object to a FindCommandRequest. If the client request did not specify any
-// runtime constants, make them available to the query here.
-std::unique_ptr<FindCommandRequest> parseCmdObjectToFindCommandRequest(OperationContext* opCtx,
-                                                                       NamespaceString nss,
-                                                                       BSONObj cmdObj) {
-    auto findCommand = query_request_helper::makeFromFindCommand(
-        std::move(cmdObj),
-        std::move(nss),
-        APIParameters::get(opCtx).getAPIStrict().value_or(false));
-
-    // Rewrite any FLE find payloads that exist in the query if this is a FLE 2 query.
-    if (shouldDoFLERewrite(findCommand)) {
-        invariant(findCommand->getNamespaceOrUUID().nss());
-        processFLEFindD(opCtx, findCommand->getNamespaceOrUUID().nss().value(), findCommand.get());
-    }
-
-    if (findCommand->getMirrored().value_or(false)) {
-        const auto& invocation = CommandInvocation::get(opCtx);
-        invocation->markMirrored();
-    }
-
-    return findCommand;
-}
 
 boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
     OperationContext* opCtx,
     const FindCommandRequest& findCommand,
+    const CollectionPtr& collPtr,
     boost::optional<ExplainOptions::Verbosity> verbosity) {
     std::unique_ptr<CollatorInterface> collator;
     if (!findCommand.getCollation().isEmpty()) {
         collator = uassertStatusOK(CollatorFactoryInterface::get(opCtx->getServiceContext())
                                        ->makeFromBSON(findCommand.getCollation()));
+    } else if (collPtr && collPtr->getDefaultCollator()) {
+        // The 'collPtr' will be null for views, but we don't need to worry about views here. The
+        // views will get rewritten into aggregate command and will regenerate the
+        // ExpressionContext.
+        collator = collPtr->getDefaultCollator()->clone();
     }
-
-    // Although both 'find' and 'aggregate' commands have an ExpressionContext, some of the data
-    // members in the ExpressionContext are used exclusively by the aggregation subsystem. This
-    // includes the following fields which here we simply initialize to some meaningless default
-    // value:
-    //  - explain
-    //  - fromMongos
-    //  - needsMerge
-    //  - bypassDocumentValidation
-    //  - mongoProcessInterface
-    //  - resolvedNamespaces
-    //  - uuid
-    //
-    // As we change the code to make the find and agg systems more tightly coupled, it would make
-    // sense to start initializing these fields for find operations as well.
-    auto expCtx = make_intrusive<ExpressionContext>(
-        opCtx,
-        verbosity,
-        false,  // fromMongos
-        false,  // needsMerge
-        findCommand.getAllowDiskUse().value_or(allowDiskUseByDefault.load()),
-        false,  // bypassDocumentValidation
-        false,  // isMapReduceCommand
-        findCommand.getNamespaceOrUUID().nss().value_or(NamespaceString()),
-        findCommand.getLegacyRuntimeConstants(),
-        std::move(collator),
-        nullptr,  // mongoProcessInterface
-        StringMap<ExpressionContext::ResolvedNamespace>{},
-        boost::none,                             // uuid
-        findCommand.getLet(),                    // let
-        CurOp::get(opCtx)->dbProfileLevel() > 0  // mayDbProfile
-    );
-    if (opCtx->readOnly()) {
-        // Disallow disk use if in read-only mode.
-        expCtx->allowDiskUse = false;
-    }
+    auto expCtx =
+        make_intrusive<ExpressionContext>(opCtx,
+                                          findCommand,
+                                          std::move(collator),
+                                          CurOp::get(opCtx)->dbProfileLevel() > 0,  // mayDbProfile
+                                          verbosity,
+                                          allowDiskUseByDefault.load());
     expCtx->tempDir = storageGlobalParams.dbpath + "/_tmp";
     expCtx->startExpressionCounters();
 
@@ -158,9 +120,51 @@ void beginQueryOp(OperationContext* opCtx, const NamespaceString& nss, const BSO
     auto curOp = CurOp::get(opCtx);
     stdx::lock_guard<Client> lk(*opCtx->getClient());
     curOp->setOpDescription_inlock(queryObj);
-    curOp->setNS_inlock(nss.ns());
+    curOp->setNS_inlock(nss);
 }
 
+/**
+ * Parses the grammar elements like 'filter', 'sort', and 'projection' from the raw
+ * 'FindCommandRequest', and tracks internal state like begining the operation's timer and recording
+ * query shape stats (if enabled).
+ */
+std::unique_ptr<CanonicalQuery> parseQueryAndBeginOperation(
+    OperationContext* opCtx,
+    const AutoGetCollectionForReadCommandMaybeLockFree& ctx,
+    const NamespaceString& nss,
+    BSONObj requestBody,
+    std::unique_ptr<FindCommandRequest> findCommand,
+    const CollectionPtr& collection) {
+    // Fill out curop information.
+    beginQueryOp(opCtx, nss, requestBody);
+    // Finish the parsing step by using the FindCommandRequest to create a CanonicalQuery.
+    const ExtensionsCallbackReal extensionsCallback(opCtx, &nss);
+
+    auto expCtx =
+        makeExpressionContext(opCtx, *findCommand, collection, boost::none /* verbosity */);
+
+    auto parsedRequest = uassertStatusOK(
+        parsed_find_command::parse(expCtx,
+                                   std::move(findCommand),
+                                   extensionsCallback,
+                                   MatchExpressionParser::kAllowAllSpecialFeatures));
+
+    // After parsing to detect if $$USER_ROLES is referenced in the query, set the value of
+    // $$USER_ROLES for the find command.
+    expCtx->setUserRoles();
+    // Register query stats collection. Exclude queries against collections with encrypted fields.
+    // It is important to do this before canonicalizing and optimizing the query, each of which
+    // would alter the query shape.
+    if (!(collection && collection.get()->getCollectionOptions().encryptedFieldConfig)) {
+        query_stats::registerRequest(opCtx, nss, [&]() {
+            return std::make_unique<query_stats::FindKey>(
+                expCtx, *parsedRequest, ctx.getCollectionType());
+        });
+    }
+
+    return uassertStatusOK(
+        CanonicalQuery::canonicalize(std::move(expCtx), std::move(parsedRequest)));
+}
 /**
  * A command for running .find() queries.
  */
@@ -217,7 +221,7 @@ public:
         return false;
     }
 
-    bool shouldAffectReadConcernCounter() const override {
+    bool shouldAffectReadOptionCounters() const override {
         return true;
     }
 
@@ -234,7 +238,9 @@ public:
         Invocation(const FindCmd* definition, const OpMsgRequest& request)
             : CommandInvocation(definition),
               _request(request),
-              _dbName(_request.getValidatedTenantId(), _request.getDatabase()) {
+              _dbName(DatabaseNameUtil::deserialize(_request.getValidatedTenantId(),
+                                                    _request.getDatabase())),
+              _ns(CommandHelpers::parseNsFromCommand(_dbName, _request.body)) {
             invariant(_request.body.isOwned());
         }
 
@@ -264,8 +270,7 @@ public:
         }
 
         NamespaceString ns() const override {
-            // TODO get the ns from the parsed QueryRequest.
-            return NamespaceString(CommandHelpers::parseNsFromCommand(_dbName, _request.body));
+            return _ns;
         }
 
         void doCheckAuthorization(OperationContext* opCtx) const final {
@@ -291,15 +296,34 @@ public:
             boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> ctx;
             ctx.emplace(opCtx,
                         CommandHelpers::parseNsCollectionRequired(_dbName, _request.body),
-                        AutoGetCollectionViewMode::kViewsPermitted);
+                        AutoGetCollection::Options{}.viewMode(
+                            auto_get_collection::ViewMode::kViewsPermitted));
             const auto nss = ctx->getNss();
 
+            // Going forward this operation must never ignore interrupt signals while waiting for
+            // lock acquisition. This InterruptibleLockGuard will ensure that waiting for lock
+            // re-acquisition after yielding will not ignore interrupt signals. This is necessary to
+            // avoid deadlocking with replication rollback, which at the storage layer waits for all
+            // cursors to be closed under the global MODE_X lock, after having sent interrupt
+            // signals to read operations. This operation must never hold open storage cursors while
+            // ignoring interrupt.
+            InterruptibleLockGuard interruptibleLockAcquisition(opCtx->lockState());
+
             // Parse the command BSON to a FindCommandRequest.
-            auto findCommand = parseCmdObjectToFindCommandRequest(opCtx, nss, _request.body);
+            auto findCommand = _parseCmdObjectToFindCommandRequest(opCtx, nss, _request.body);
 
             // Finish the parsing step by using the FindCommandRequest to create a CanonicalQuery.
             const ExtensionsCallbackReal extensionsCallback(opCtx, &nss);
-            auto expCtx = makeExpressionContext(opCtx, *findCommand, verbosity);
+
+            // The collection may be NULL. If so, getExecutor() should handle it by returning an
+            // execution tree with an EOFStage.
+            const auto& collection = ctx->getCollection();
+            if (!ctx->getView()) {
+                const bool isClusteredCollection = collection && collection->isClustered();
+                uassertStatusOK(query_request_helper::validateResumeAfter(
+                    findCommand->getResumeAfter(), isClusteredCollection));
+            }
+            auto expCtx = makeExpressionContext(opCtx, *findCommand, collection, verbosity);
             const bool isExplain = true;
             auto cq = uassertStatusOK(
                 CanonicalQuery::canonicalize(opCtx,
@@ -309,35 +333,27 @@ public:
                                              extensionsCallback,
                                              MatchExpressionParser::kAllowAllSpecialFeatures));
 
-            // If we are running a query against a view, or if we are trying to test the new
-            // optimizer, redirect this query through the aggregation system.
-            if (ctx->getView() || isEligibleForBonsai(*cq, opCtx, ctx->getCollection())) {
+            // After parsing to detect if $$USER_ROLES is referenced in the query, set the value of
+            // $$USER_ROLES for the find command.
+            cq->getExpCtx()->setUserRoles();
+
+            // If we are running a query against a view redirect this query through the aggregation
+            // system.
+            if (ctx->getView()) {
                 // Relinquish locks. The aggregation command will re-acquire them.
                 ctx.reset();
 
                 // Convert the find command into an aggregation using $match (and other stages, as
                 // necessary), if possible.
                 const auto& findCommand = cq->getFindCommandRequest();
-                auto viewAggregationCommand =
-                    uassertStatusOK(query_request_helper::asAggregationCommand(findCommand));
-
-                auto viewAggCmd =
-                    OpMsgRequest::fromDBAndBody(_dbName.db(), viewAggregationCommand).body;
-
-                // Create the agg request equivalent of the find operation, with the explain
-                // verbosity included.
-                auto aggRequest = aggregation_request_helper::parseFromBSON(
-                    opCtx,
-                    nss,
-                    viewAggCmd,
-                    verbosity,
-                    APIParameters::get(opCtx).getAPIStrict().value_or(false));
+                auto aggRequest = query_request_conversion::asAggregateCommandRequest(findCommand);
+                aggRequest.setExplain(verbosity);
 
                 try {
                     // An empty PrivilegeVector is acceptable because these privileges are only
                     // checked on getMore and explain will not open a cursor.
-                    uassertStatusOK(runAggregate(
-                        opCtx, nss, aggRequest, viewAggCmd, PrivilegeVector(), result));
+                    uassertStatusOK(
+                        runAggregate(opCtx, aggRequest, _request.body, PrivilegeVector(), result));
                 } catch (DBException& error) {
                     if (error.code() == ErrorCodes::InvalidPipelineOperator) {
                         uasserted(ErrorCodes::InvalidPipelineOperator,
@@ -349,10 +365,6 @@ public:
                 return;
             }
 
-            // The collection may be NULL. If so, getExecutor() should handle it by returning an
-            // execution tree with an EOFStage.
-            const auto& collection = ctx->getCollection();
-
             // Get the execution plan for the query.
             bool permitYield = true;
             auto exec =
@@ -386,11 +398,9 @@ public:
 
             // Parse the command BSON to a FindCommandRequest. Pass in the parsedNss in case cmdObj
             // does not have a UUID.
-            auto parsedNss = ns();
-            const bool isExplain = false;
-            const bool isOplogNss = (parsedNss == NamespaceString::kRsOplogNamespace);
-            auto findCommand =
-                parseCmdObjectToFindCommandRequest(opCtx, std::move(parsedNss), cmdObj);
+            const bool isOplogNss = (_ns == NamespaceString::kRsOplogNamespace);
+            auto findCommand = _parseCmdObjectToFindCommandRequest(opCtx, _ns, cmdObj);
+            CurOp::get(opCtx)->beginQueryPlanningTimer();
 
             // Only allow speculative majority for internal commands that specify the correct flag.
             uassert(ErrorCodes::ReadConcernMajorityNotEnabled,
@@ -398,10 +408,11 @@ public:
                     !(repl::ReadConcernArgs::get(opCtx).isSpeculativeMajority() &&
                       !findCommand->getAllowSpeculativeMajorityRead()));
 
+            const bool isFindByUUID = findCommand->getNamespaceOrUUID().isUUID();
             uassert(ErrorCodes::InvalidOptions,
                     "When using the find command by UUID, the collectionUUID parameter cannot also "
                     "be specified",
-                    !findCommand->getNamespaceOrUUID().uuid() || !findCommand->getCollectionUUID());
+                    !isFindByUUID || !findCommand->getCollectionUUID());
 
             auto replCoord = repl::ReplicationCoordinator::get(opCtx);
             const auto txnParticipant = TransactionParticipant::get(opCtx);
@@ -424,11 +435,11 @@ public:
             // The presence of a term in the request indicates that this is an internal replication
             // oplog read request.
             if (term && isOplogNss) {
-                // We do not want to take tickets for internal (replication) oplog reads. Stalling
-                // on ticket acquisition can cause complicated deadlocks. Primaries may depend on
-                // data reaching secondaries in order to proceed; and secondaries may get stalled
-                // replicating because of an inability to acquire a read ticket.
-                opCtx->lockState()->skipAcquireTicket();
+                // We do not want to wait to take tickets for internal (replication) oplog reads.
+                // Stalling on ticket acquisition can cause complicated deadlocks. Primaries may
+                // depend on data reaching secondaries in order to proceed; and secondaries may get
+                // stalled replicating because of an inability to acquire a read ticket.
+                opCtx->lockState()->setAdmissionPriority(AdmissionContext::Priority::kImmediate);
             }
 
             // If this read represents a reverse oplog scan, we want to bypass oplog visibility
@@ -467,31 +478,58 @@ public:
             boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> ctx;
             ctx.emplace(opCtx,
                         CommandHelpers::parseNsOrUUID(_dbName, _request.body),
-                        AutoGetCollectionViewMode::kViewsPermitted);
+                        AutoGetCollection::Options{}
+                            .viewMode(auto_get_collection::ViewMode::kViewsPermitted)
+                            .expectedUUID(findCommand->getCollectionUUID()));
             const auto& nss = ctx->getNss();
 
-            uassert(ErrorCodes::NamespaceNotFound,
-                    str::stream() << "UUID " << findCommand->getNamespaceOrUUID().uuid().value()
-                                  << " specified in query request not found",
-                    ctx || !findCommand->getNamespaceOrUUID().uuid());
-
-            checkCollectionUUIDMismatch(
-                opCtx, nss, ctx->getCollection(), findCommand->getCollectionUUID());
-
-            // Set the namespace if a collection was found, as opposed to nothing or a view.
-            if (ctx) {
-                query_request_helper::refreshNSS(ctx->getNss(), findCommand.get());
+            if (!findCommand->getMirrored()) {
+                if (auto sampleId = analyze_shard_key::getOrGenerateSampleId(
+                        opCtx,
+                        ns(),
+                        analyze_shard_key::SampledCommandNameEnum::kFind,
+                        *findCommand)) {
+                    analyze_shard_key::QueryAnalysisWriter::get(opCtx)
+                        ->addFindQuery(*sampleId,
+                                       nss,
+                                       findCommand->getFilter(),
+                                       findCommand->getCollation(),
+                                       findCommand->getLet())
+                        .getAsync([](auto) {});
+                }
             }
 
-            // Tailing a replicated capped clustered collection requires majority read concern.
-            const auto coll = ctx->getCollection().get();
-            if (coll) {
+            // Going forward this operation must never ignore interrupt signals while waiting for
+            // lock acquisition. This InterruptibleLockGuard will ensure that waiting for lock
+            // re-acquisition after yielding will not ignore interrupt signals. This is necessary to
+            // avoid deadlocking with replication rollback, which at the storage layer waits for all
+            // cursors to be closed under the global MODE_X lock, after having sent interrupt
+            // signals to read operations. This operation must never hold open storage cursors while
+            // ignoring interrupt.
+            InterruptibleLockGuard interruptibleLockAcquisition(opCtx->lockState());
+
+            const auto& collection = ctx->getCollection();
+
+            uassert(ErrorCodes::NamespaceNotFound,
+                    str::stream() << "UUID " << findCommand->getNamespaceOrUUID().uuid()
+                                  << " specified in query request not found",
+                    collection || !isFindByUUID);
+
+            bool isClusteredCollection = false;
+            if (collection) {
+                if (isFindByUUID) {
+                    // Replace the UUID in the find command with the fully qualified namespace of
+                    // the looked up Collection.
+                    findCommand->setNss(ctx->getNss());
+                }
+
+                // Tailing a replicated capped clustered collection requires majority read concern.
                 const bool isTailable = findCommand->getTailable();
                 const bool isMajorityReadConcern = repl::ReadConcernArgs::get(opCtx).getLevel() ==
                     repl::ReadConcernLevel::kMajorityReadConcern;
-                const bool isClusteredCollection = coll->isClustered();
-                const bool isCapped = coll->isCapped();
-                const bool isReplicated = coll->ns().isReplicated();
+                isClusteredCollection = collection->isClustered();
+                const bool isCapped = collection->isCapped();
+                const bool isReplicated = collection->ns().isReplicated();
                 if (isClusteredCollection && isCapped && isReplicated && isTailable) {
                     uassert(ErrorCodes::Error(6049203),
                             "A tailable cursor on a capped clustered collection requires majority "
@@ -500,42 +538,39 @@ public:
                 }
             }
 
-            // Fill out curop information.
-            beginQueryOp(opCtx, nss, _request.body);
+            // Views use the aggregation system and the $_resumeAfter parameter is not allowed. A
+            // more descriptive error will be raised later, but we want to validate this parameter
+            // before beginning the operation.
+            if (!ctx->getView()) {
+                uassertStatusOK(query_request_helper::validateResumeAfter(
+                    findCommand->getResumeAfter(), isClusteredCollection));
+            }
 
-            // Finish the parsing step by using the FindCommandRequest to create a CanonicalQuery.
-            const ExtensionsCallbackReal extensionsCallback(opCtx, &nss);
-            auto expCtx = makeExpressionContext(opCtx, *findCommand, boost::none /* verbosity */);
-            auto cq = uassertStatusOK(
-                CanonicalQuery::canonicalize(opCtx,
-                                             std::move(findCommand),
-                                             isExplain,
-                                             std::move(expCtx),
-                                             extensionsCallback,
-                                             MatchExpressionParser::kAllowAllSpecialFeatures));
+            auto cq = parseQueryAndBeginOperation(
+                opCtx, *ctx, nss, _request.body, std::move(findCommand), collection);
 
-            // If we are running a query against a view, or if we are trying to test the new
-            // optimizer, redirect this query through the aggregation system.
-            if (ctx->getView() || isEligibleForBonsai(*cq, opCtx, ctx->getCollection())) {
+            // If we are running a query against a view redirect this query through the aggregation
+            // system.
+            if (ctx->getView()) {
                 // Relinquish locks. The aggregation command will re-acquire them.
                 ctx.reset();
 
                 // Convert the find command into an aggregation using $match (and other stages, as
                 // necessary), if possible.
                 const auto& findCommand = cq->getFindCommandRequest();
-                auto viewAggregationCommand =
-                    uassertStatusOK(query_request_helper::asAggregationCommand(findCommand));
+                auto aggRequest = query_request_conversion::asAggregateCommandRequest(findCommand);
 
-                BSONObj aggResult = CommandHelpers::runCommandDirectly(
-                    opCtx,
-                    OpMsgRequest::fromDBAndBody(_dbName.db(), std::move(viewAggregationCommand)));
-                auto status = getStatusFromCommandResult(aggResult);
+                auto privileges = uassertStatusOK(
+                    auth::getPrivilegesForAggregate(AuthorizationSession::get(opCtx->getClient()),
+                                                    aggRequest.getNamespace(),
+                                                    aggRequest,
+                                                    false));
+                auto status = runAggregate(opCtx, aggRequest, _request.body, privileges, result);
                 if (status.code() == ErrorCodes::InvalidPipelineOperator) {
                     uasserted(ErrorCodes::InvalidPipelineOperator,
                               str::stream() << "Unsupported in view pipeline: " << status.reason());
                 }
                 uassertStatusOK(status);
-                result->getBodyBuilder().appendElements(aggResult);
                 return;
             }
 
@@ -547,8 +582,6 @@ public:
             uassertStatusOK(replCoord->checkCanServeReadsFor(
                 opCtx, nss, ReadPreferenceSetting::get(opCtx).canRunOnSecondary()));
 
-            const auto& collection = ctx->getCollection();
-
             if (cq->getFindCommandRequest().getReadOnce()) {
                 // The readOnce option causes any storage-layer cursors created during plan
                 // execution to assume read data will not be needed again and need not be cached.
@@ -566,9 +599,7 @@ public:
 
             // If the executor supports it, find operations will maintain the storage engine state
             // across commands.
-            if (gMaintainValidCursorsAcrossReadCommands && !opCtx->inMultiDocumentTransaction() &&
-                repl::ReadConcernArgs::get(opCtx).getLevel() !=
-                    repl::ReadConcernLevel::kSnapshotReadConcern) {
+            if (gMaintainValidCursorsAcrossReadCommands && !opCtx->inMultiDocumentTransaction()) {
                 exec->enableSaveRecoveryUnitAcrossCommandsIfSupported();
             }
 
@@ -582,10 +613,9 @@ public:
                 // there is no ClientCursor id, and then return.
                 const long long numResults = 0;
                 const CursorId cursorId = 0;
-                endQueryOp(opCtx, collection, *exec, numResults, cursorId);
+                endQueryOp(opCtx, collection, *exec, numResults, boost::none, cmdObj);
                 auto bodyBuilder = result->getBodyBuilder();
-                appendCursorResponseObject(
-                    cursorId, nss.ns(), BSONArray(), boost::none, &bodyBuilder);
+                appendCursorResponseObject(cursorId, nss, BSONArray(), boost::none, &bodyBuilder);
                 return;
             }
 
@@ -633,8 +663,6 @@ public:
                 auto&& [stats, _] =
                     explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
                 LOGV2_WARNING(23798,
-                              "Plan executor error during find command: {error}, "
-                              "stats: {stats}, cmd: {cmd}",
                               "Plan executor error during find command",
                               "error"_attr = exception.toStatus(),
                               "stats"_attr = redact(stats),
@@ -683,36 +711,38 @@ public:
                     pinnedCursor.getCursor()->setLeftoverMaxTimeMicros(
                         opCtx->getRemainingMaxTimeMicros());
                 }
-                pinnedCursor.getCursor()->setNReturnedSoFar(numResults);
-                pinnedCursor.getCursor()->incNBatches();
 
                 // Fill out curop based on the results.
-                endQueryOp(opCtx, collection, *cursorExec, numResults, cursorId);
+                endQueryOp(opCtx, collection, *cursorExec, numResults, pinnedCursor, cmdObj);
 
                 if (stashResourcesForGetMore) {
                     // Collect storage stats now before we stash the recovery unit. These stats are
                     // normally collected in the service entry point layer just before a command
-                    // ends, but they must be collected before stashing the
-                    // RecoveryUnit. Otherwise, the service entry point layer will collect the
-                    // stats from the new RecoveryUnit, which wasn't actually used for the query.
+                    // ends, but they must be collected before stashing the RecoveryUnit. Otherwise,
+                    // the service entry point layer will collect the stats from the new
+                    // RecoveryUnit, which wasn't actually used for the query.
                     //
                     // The stats collected here will not get overwritten, as the service entry
                     // point layer will only set these stats when they're not empty.
                     CurOp::get(opCtx)->debug().storageStats =
-                        opCtx->recoveryUnit()->getOperationStatistics();
+                        opCtx->recoveryUnit()->computeOperationStatisticsSinceLastCall();
                 }
             } else {
-                endQueryOp(opCtx, collection, *exec, numResults, cursorId);
+                endQueryOp(opCtx, collection, *exec, numResults, boost::none, cmdObj);
+                // We want to destroy executor as soon as possible to release any resources locks it
+                // may hold.
+                exec.reset();
             }
 
             // Generate the response object to send to the client.
-            firstBatch.done(cursorId, nss.ns());
+            firstBatch.done(cursorId, nss);
 
             // Increment this metric once we have generated a response and we know it will return
             // documents.
             auto& metricsCollector = ResourceConsumption::MetricsCollector::get(opCtx);
             metricsCollector.incrementDocUnitsReturned(nss.ns(), docUnitsReturned);
-            query_request_helper::validateCursorResponse(result->getBodyBuilder().asTempObj());
+            query_request_helper::validateCursorResponse(result->getBodyBuilder().asTempObj(),
+                                                         nss.tenantId());
         }
 
         void appendMirrorableRequest(BSONObjBuilder* bob) const override {
@@ -729,6 +759,8 @@ public:
                 keyBob.append("min", 1);
                 keyBob.append("max", 1);
                 keyBob.append("shardVersion", 1);
+                keyBob.append("databaseVersion", 1);
+                keyBob.append("encryptionInformation", 1);
                 return keyBob.obj();
             }();
 
@@ -742,6 +774,38 @@ public:
     private:
         const OpMsgRequest _request;
         const DatabaseName _dbName;
+        const NamespaceString _ns;
+
+        // Parses the command object to a FindCommandRequest. If the client request did not specify
+        // any runtime constants, make them available to the query here.
+        std::unique_ptr<FindCommandRequest> _parseCmdObjectToFindCommandRequest(
+            OperationContext* opCtx, NamespaceString nss, BSONObj cmdObj) {
+            auto findCommand = query_request_helper::makeFromFindCommand(
+                std::move(cmdObj),
+                std::move(nss),
+                APIParameters::get(opCtx).getAPIStrict().value_or(false));
+
+            // Rewrite any FLE find payloads that exist in the query if this is a FLE 2 query.
+            if (shouldDoFLERewrite(findCommand)) {
+                invariant(findCommand->getNamespaceOrUUID().isNamespaceString());
+                LOGV2_DEBUG(
+                    7964101, 2, "Processing Queryable Encryption command", "cmd"_attr = cmdObj);
+
+                if (!findCommand->getEncryptionInformation()->getCrudProcessed().value_or(false)) {
+                    processFLEFindD(
+                        opCtx, findCommand->getNamespaceOrUUID().nss(), findCommand.get());
+                }
+                stdx::lock_guard<Client> lk(*opCtx->getClient());
+                CurOp::get(opCtx)->setShouldOmitDiagnosticInformation_inlock(lk, true);
+            }
+
+            if (findCommand->getMirrored().value_or(false)) {
+                const auto& invocation = CommandInvocation::get(opCtx);
+                invocation->markMirrored();
+            }
+
+            return findCommand;
+        }
     };
 
 } findCmd;
