@@ -28,7 +28,6 @@
  */
 
 #include <fmt/format.h>
-#include <memory>
 #include <string>
 
 #include "mongo/bson/util/bson_extract.h"
@@ -37,6 +36,7 @@
 #include "mongo/db/auth/authorization_checks.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/external_data_source_scope_guard.h"
 #include "mongo/db/client.h"
 #include "mongo/db/clientcursor.h"
 #include "mongo/db/commands.h"
@@ -322,7 +322,8 @@ public:
         Invocation(Command* cmd, const OpMsgRequest& request)
             : CommandInvocation(cmd),
               _cmd(GetMoreCommandRequest::parse(IDLParserContext{"getMore"}, request)) {
-            NamespaceString nss(_cmd.getDbName(), _cmd.getCollection());
+            NamespaceString nss(NamespaceStringUtil::parseNamespaceFromRequest(
+                _cmd.getDbName(), _cmd.getCollection()));
             uassert(ErrorCodes::InvalidNamespace,
                     str::stream() << "Invalid namespace for getMore: " << nss.ns(),
                     nss.isValid());
@@ -347,7 +348,8 @@ public:
         }
 
         NamespaceString ns() const override {
-            return NamespaceString(_cmd.getDbName(), _cmd.getCollection());
+            return NamespaceStringUtil::parseNamespaceFromRequest(_cmd.getDbName(),
+                                                                  _cmd.getCollection());
         }
 
         void doCheckAuthorization(OperationContext* opCtx) const override {
@@ -431,7 +433,6 @@ public:
                 auto&& [stats, _] =
                     explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
                 LOGV2_WARNING(20478,
-                              "getMore command executor error: {error}, stats: {stats}, cmd: {cmd}",
                               "getMore command executor error",
                               "error"_attr = exception.toStatus(),
                               "stats"_attr = redact(stats),
@@ -456,6 +457,11 @@ public:
                                           rpc::ReplyBuilderInterface* reply,
                                           ClientCursorPin& cursorPin,
                                           CurOp* curOp) {
+            // Get a reference to the shared_ptr so that we drop the virtual collections (via the
+            // destructor) after deleting our cursors and releasing our read locks.
+            std::shared_ptr<ExternalDataSourceScopeGuard> extDataSourceScopeGuard =
+                ExternalDataSourceScopeGuard::get(cursorPin.getCursor());
+
             // Cursors come in one of two flavors:
             //
             // - Cursors which read from a single collection, such as those generated via the
@@ -476,7 +482,8 @@ public:
             // the stats twice.
             boost::optional<AutoGetCollectionForReadMaybeLockFree> readLock;
             boost::optional<AutoStatsTracker> statsTracker;
-            NamespaceString nss(_cmd.getDbName(), _cmd.getCollection());
+            NamespaceString nss(NamespaceStringUtil::parseNamespaceFromRequest(
+                _cmd.getDbName(), _cmd.getCollection()));
 
             const bool disableAwaitDataFailpointActive =
                 MONGO_unlikely(disableAwaitDataForGetMoreCmd.shouldFail());
@@ -485,6 +492,10 @@ public:
             setUpOperationContextStateForGetMore(
                 opCtx, *cursorPin.getCursor(), _cmd, disableAwaitDataFailpointActive);
 
+            // Update opCtx of the decorated ExternalDataSourceScopeGuard object so that it can drop
+            // virtual collections in the new 'opCtx'.
+            ExternalDataSourceScopeGuard::updateOperationContext(cursorPin.getCursor(), opCtx);
+
             // On early return, typically due to a failed assertion, delete the cursor.
             ScopeGuard cursorDeleter([&] { cursorPin.deleteUnderlying(); });
 
@@ -517,9 +528,8 @@ public:
                 // cursor's executor when constructing 'readLock'.
                 readLock.emplace(opCtx,
                                  cursorPin->getExecutor()->nss(),
-                                 AutoGetCollectionViewMode::kViewsForbidden,
-                                 Date_t::max(),
-                                 cursorPin->getExecutor()->getSecondaryNamespaces());
+                                 AutoGetCollection::Options{}.secondaryNssOrUUIDs(
+                                     cursorPin->getExecutor()->getSecondaryNamespaces()));
 
                 statsTracker.emplace(
                     opCtx,
@@ -533,34 +543,12 @@ public:
                     opCtx, nss, true));
             }
 
-            // If the 'waitAfterPinningCursorBeforeGetMoreBatch' fail point is enabled, set the
-            // 'msg' field of this operation's CurOp to signal that we've hit this point and then
-            // repeatedly release and re-acquire the collection readLock at regular intervals until
-            // the failpoint is released. This is done in order to avoid deadlocks caused by the
-            // pinned-cursor failpoints in this file (see SERVER-21997).
-            std::function<void()> dropAndReacquireReadLockIfLocked =
-                [&readLock, opCtx, &cursorPin]() {
-                    if (!readLock) {
-                        // This function is a no-op if 'readLock' is not held in the first place.
-                        return;
-                    }
-
-                    // Make sure an interrupted operation does not prevent us from reacquiring the
-                    // lock.
-                    UninterruptibleLockGuard noInterrupt(opCtx->lockState());
-                    readLock.reset();
-                    readLock.emplace(opCtx,
-                                     cursorPin->getExecutor()->nss(),
-                                     AutoGetCollectionViewMode::kViewsForbidden,
-                                     Date_t::max(),
-                                     cursorPin->getExecutor()->getSecondaryNamespaces());
-                };
             if (MONGO_unlikely(waitAfterPinningCursorBeforeGetMoreBatch.shouldFail())) {
                 CurOpFailpointHelpers::waitWhileFailPointEnabled(
                     &waitAfterPinningCursorBeforeGetMoreBatch,
                     opCtx,
                     "waitAfterPinningCursorBeforeGetMoreBatch",
-                    dropAndReacquireReadLockIfLocked,
+                    []() {}, /*empty function*/
                     nss);
             }
 
@@ -590,6 +578,10 @@ public:
                     curOp->setOriginatingCommand_inlock(originatingCommand);
                 }
 
+                curOp->debug().queryFramework = exec->getQueryFramework();
+                curOp->setShouldOmitDiagnosticInformation_inlock(
+                    lk, cursorPin->shouldOmitDiagnosticInformation());
+
                 // Update the genericCursor stored in curOp with the new cursor stats.
                 curOp->setGenericCursor_inlock(cursorPin->toGenericCursor());
             }
@@ -617,7 +609,6 @@ public:
                 options.atClusterTime = repl::ReadConcernArgs::get(opCtx).getArgsAtClusterTime();
             }
             CursorResponseBuilder nextBatch(reply, options);
-            BSONObj obj;
             std::uint64_t numResults = 0;
             ResourceConsumption::DocumentUnitCounter docUnitsReturned;
 
@@ -634,32 +625,17 @@ public:
                     // Use the commit point of the last batch for exhaust cursors.
                     lastKnownCommittedOpTime = cursorPin->getLastKnownCommittedOpTime();
                 }
-                if (lastKnownCommittedOpTime) {
-                    clientsLastKnownCommittedOpTime(opCtx) = lastKnownCommittedOpTime.value();
-                }
+                clientsLastKnownCommittedOpTime(opCtx) = lastKnownCommittedOpTime;
 
                 awaitDataState(opCtx).shouldWaitForInserts = true;
             }
 
-            // We're about to begin running the PlanExecutor in order to fill the getMore batch. If
-            // the 'waitWithPinnedCursorDuringGetMoreBatch' failpoint is active, set the 'msg' field
-            // of this operation's CurOp to signal that we've hit this point and then spin until the
-            // failpoint is released.
-            std::function<void()> saveAndRestoreStateWithReadLockReacquisition =
-                [exec, dropAndReacquireReadLockIfLocked, &readLock]() {
-                    exec->saveState();
-                    dropAndReacquireReadLockIfLocked();
-                    exec->restoreState(readLock ? &readLock->getCollection() : nullptr);
-                };
-
             waitWithPinnedCursorDuringGetMoreBatch.execute([&](const BSONObj& data) {
                 CurOpFailpointHelpers::waitWhileFailPointEnabled(
                     &waitWithPinnedCursorDuringGetMoreBatch,
                     opCtx,
                     "waitWithPinnedCursorDuringGetMoreBatch",
-                    data["shouldNotdropLock"].booleanSafe()
-                        ? []() {} /*empty function*/
-                        : saveAndRestoreStateWithReadLockReacquisition,
+                    []() {}, /*empty function*/
                     nss);
             });
 
@@ -684,7 +660,7 @@ public:
             // generate the stats eagerly for all operations due to cost.
             if (cursorPin->getExecutor()->lockPolicy() !=
                     PlanExecutor::LockPolicy::kLocksInternally &&
-                curOp->shouldDBProfile(opCtx)) {
+                curOp->shouldDBProfile()) {
                 auto&& explainer = exec->getPlanExplainer();
                 auto&& [stats, _] =
                     explainer.getWinningPlanStats(ExplainOptions::Verbosity::kExecStats);
@@ -699,27 +675,33 @@ public:
 
                 cursorPin->setLeftoverMaxTimeMicros(opCtx->getRemainingMaxTimeMicros());
 
-                if (opCtx->isExhaust() && !clientsLastKnownCommittedOpTime(opCtx).isNull()) {
-                    // Set the commit point of the latest batch.
+                if (opCtx->isExhaust() && clientsLastKnownCommittedOpTime(opCtx)) {
+                    // Update the cursor's lastKnownCommittedOpTime to the current
+                    // lastCommittedOpTime. The lastCommittedOpTime now may be staler than the
+                    // actual lastCommittedOpTime returned in the metadata of this latest batch (see
+                    // appendReplyMetadata).  As a result, we may sometimes return more empty
+                    // batches than we need to. But it is fine to be conservative in this.
                     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-                    cursorPin->setLastKnownCommittedOpTime(replCoord->getLastCommittedOpTime());
+                    auto myLastCommittedOpTime = replCoord->getLastCommittedOpTime();
+                    auto clientsLastKnownCommittedOpTime = cursorPin->getLastKnownCommittedOpTime();
+                    if (!clientsLastKnownCommittedOpTime.has_value() ||
+                        clientsLastKnownCommittedOpTime.value() < myLastCommittedOpTime) {
+                        cursorPin->setLastKnownCommittedOpTime(myLastCommittedOpTime);
+                    }
                 }
             } else {
                 curOp->debug().cursorExhausted = true;
             }
 
-            nextBatch.done(respondWithId, nss.ns());
+            nextBatch.done(respondWithId, nss);
 
             // Increment this metric once we have generated a response and we know it will return
             // documents.
             auto& metricsCollector = ResourceConsumption::MetricsCollector::get(opCtx);
             metricsCollector.incrementDocUnitsReturned(curOp->getNS(), docUnitsReturned);
-            cursorPin->incNReturnedSoFar(numResults);
-            cursorPin->incNBatches();
-
-            // Ensure log and profiler include the number of results returned in this getMore's
-            // response batch.
-            curOp->debug().nreturned = numResults;
+            curOp->debug().additiveMetrics.nBatches = 1;
+            curOp->setEndOfOpMetrics(numResults);
+            collectQueryStatsMongod(opCtx, cursorPin);
 
             if (respondWithId) {
                 cursorDeleter.dismiss();
@@ -753,11 +735,11 @@ public:
                 // internal clients (see checkAuthForGetMore).
                 curOp->debug().isReplOplogGetMore = true;
 
-                // We do not want to take tickets for internal (replication) oplog reads. Stalling
-                // on ticket acquisition can cause complicated deadlocks. Primaries may depend on
-                // data reaching secondaries in order to proceed; and secondaries may get stalled
-                // replicating because of an inability to acquire a read ticket.
-                opCtx->lockState()->skipAcquireTicket();
+                // We do not want to wait to take tickets for internal (replication) oplog reads.
+                // Stalling on ticket acquisition can cause complicated deadlocks. Primaries may
+                // depend on data reaching secondaries in order to proceed; and secondaries may get
+                // stalled replicating because of an inability to acquire a read ticket.
+                opCtx->lockState()->setAdmissionPriority(AdmissionContext::Priority::kImmediate);
             }
 
             // Perform validation checks which don't cause the cursor to be deleted on failure.
@@ -807,14 +789,15 @@ public:
             }
 
             if (getTestCommandsEnabled()) {
-                validateResult(reply);
+                validateResult(reply, nss.tenantId());
             }
         }
 
-        void validateResult(rpc::ReplyBuilderInterface* reply) {
+        void validateResult(rpc::ReplyBuilderInterface* reply, boost::optional<TenantId> tenantId) {
             auto ret = reply->getBodyBuilder().asTempObj();
-            CursorGetMoreReply::parse(IDLParserContext{"CursorGetMoreReply"},
-                                      ret.removeField("ok"));
+            CursorGetMoreReply::parse(
+                IDLParserContext{"CursorGetMoreReply", false /* apiStrict */, tenantId},
+                ret.removeField("ok"));
         }
 
         const GetMoreCommandRequest _cmd;
