@@ -169,9 +169,7 @@ BSONObj buildViewBson(const ViewDefinition& view, bool nameOnly) {
     return b.obj();
 }
 
-BSONObj buildTimeseriesBson(OperationContext* opCtx,
-                            const CollectionPtr& collection,
-                            bool nameOnly) {
+BSONObj buildTimeseriesBson(const Collection* collection, bool nameOnly) {
     invariant(collection);
 
     BSONObjBuilder builder;
@@ -209,7 +207,7 @@ BSONObj buildTimeseriesBson(StringData collName, bool nameOnly) {
  * Return an object describing the collection. Takes a collection lock if nameOnly is false.
  */
 BSONObj buildCollectionBson(OperationContext* opCtx,
-                            const CollectionPtr& collection,
+                            const Collection* collection,
                             bool includePendingDrops,
                             bool nameOnly) {
     if (!collection) {
@@ -340,19 +338,22 @@ public:
                 AutoGetDbForReadMaybeLockFree lockFreeReadBlock(opCtx, dbName);
                 auto catalog = CollectionCatalog::get(opCtx);
 
-                CurOpFailpointHelpers::waitWhileFailPointEnabled(&hangBeforeListCollections,
-                                                                 opCtx,
-                                                                 "hangBeforeListCollections",
-                                                                 []() {},
-                                                                 cursorNss);
+                CurOpFailpointHelpers::waitWhileFailPointEnabled(
+                    &hangBeforeListCollections,
+                    opCtx,
+                    "hangBeforeListCollections",
+                    []() {},
+                    cursorNss);
 
                 auto ws = std::make_unique<WorkingSet>();
                 auto root = std::make_unique<QueuedDataStage>(expCtx.get(), ws.get());
+                auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx);
 
                 if (DatabaseHolder::get(opCtx)->dbExists(opCtx, dbName)) {
                     if (auto collNames = _getExactNameMatches(matcher.get())) {
                         for (auto&& collName : *collNames) {
-                            auto nss = NamespaceString(dbName, collName);
+                            auto nss =
+                                NamespaceStringUtil::parseNamespaceFromRequest(dbName, collName);
 
                             // Only validate on a per-collection basis if the user requested
                             // a list of authorized collections
@@ -362,28 +363,22 @@ public:
                                 continue;
                             }
 
-                            // In case lock-free reads are disabled, we must be able to take a
-                            // collection lock.
-                            boost::optional<Lock::CollectionLock> clk;
-                            if (!opCtx->isLockFreeReadsOp()) {
-                                clk.emplace(opCtx, nss, MODE_IS);
-                            }
-
                             auto collBson = [&] {
-                                if (auto collection =
-                                        CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(
-                                            opCtx, nss)) {
+                                const Collection* collection =
+                                    catalog->establishConsistentCollection(
+                                        opCtx, nss, readTimestamp);
+                                if (collection != nullptr) {
                                     return buildCollectionBson(
                                         opCtx, collection, includePendingDrops, nameOnly);
                                 }
 
-                                auto view = catalog->lookupViewWithoutValidatingDurable(opCtx, nss);
+                                std::shared_ptr<const ViewDefinition> view =
+                                    catalog->lookupView(opCtx, nss);
                                 if (view && view->timeseries()) {
-                                    if (auto bucketsCollection = CollectionCatalog::get(opCtx)
-                                                                     ->lookupCollectionByNamespace(
-                                                                         opCtx, view->viewOn())) {
-                                        return buildTimeseriesBson(
-                                            opCtx, bucketsCollection, nameOnly);
+                                    if (auto bucketsCollection =
+                                            catalog->establishConsistentCollection(
+                                                opCtx, view->viewOn(), readTimestamp)) {
+                                        return buildTimeseriesBson(bucketsCollection, nameOnly);
                                     } else {
                                         // The buckets collection does not exist, so the time-series
                                         // view will be appended when we iterate through the view
@@ -400,23 +395,24 @@ public:
                             }
                         }
                     } else {
-                        auto perCollectionWork = [&](const CollectionPtr& collection) {
+                        auto perCollectionWork = [&](const Collection* collection) {
                             if (collection && collection->getTimeseriesOptions() &&
-                                !collection->ns().isDropPendingNamespace() &&
-                                catalog->lookupViewWithoutValidatingDurable(
-                                    opCtx, collection->ns().getTimeseriesViewNamespace()) &&
-                                (!authorizedCollections ||
-                                 as->isAuthorizedForAnyActionOnResource(
-                                     ResourcePattern::forExactNamespace(
-                                         collection->ns().getTimeseriesViewNamespace())))) {
-                                // The time-series view for this buckets namespace exists, so add it
-                                // here while we have the collection options.
-                                _addWorkingSetMember(
-                                    opCtx,
-                                    buildTimeseriesBson(opCtx, collection, nameOnly),
-                                    matcher.get(),
-                                    ws.get(),
-                                    root.get());
+                                !collection->ns().isDropPendingNamespace()) {
+                                auto viewNss = collection->ns().getTimeseriesViewNamespace();
+                                auto view =
+                                    catalog->lookupViewWithoutValidatingDurable(opCtx, viewNss);
+                                if (view && view->timeseries() &&
+                                    (!authorizedCollections ||
+                                     as->isAuthorizedForAnyActionOnResource(
+                                         ResourcePattern::forExactNamespace(viewNss)))) {
+                                    // The time-series view for this buckets namespace exists, so
+                                    // add it here while we have the collection options.
+                                    _addWorkingSetMember(opCtx,
+                                                         buildTimeseriesBson(collection, nameOnly),
+                                                         matcher.get(),
+                                                         ws.get(),
+                                                         root.get());
+                                }
                             }
 
                             if (authorizedCollections &&
@@ -436,14 +432,11 @@ public:
                         };
 
                         // If we are lock-free we can just iterate over our collection catalog
-                        // without
-                        // needing to yield as we don't take any locks.
+                        // without needing to yield as we don't take any locks.
                         if (opCtx->isLockFreeReadsOp()) {
                             auto collectionCatalog = CollectionCatalog::get(opCtx);
-                            for (auto it = collectionCatalog->begin(opCtx, dbName);
-                                 it != collectionCatalog->end(opCtx);
-                                 ++it) {
-                                perCollectionWork(*it);
+                            for (auto&& coll : collectionCatalog->range(dbName)) {
+                                perCollectionWork(coll);
                             }
                         } else {
                             mongo::catalog::forEachCollectionFromDb(
@@ -466,9 +459,7 @@ public:
                             }
 
                             if (view.timeseries()) {
-                                if (!CollectionCatalog::get(opCtx)
-                                         ->lookupCollectionByNamespaceForRead(opCtx,
-                                                                              view.viewOn())) {
+                                if (!catalog->lookupCollectionByNamespace(opCtx, view.viewOn())) {
                                     // There is no buckets collection backing this time-series view,
                                     // which means that it was not already added along with the
                                     // buckets collection above.
@@ -506,7 +497,7 @@ public:
                     batchSize = *listCollRequest.getCursor()->getBatchSize();
                 }
 
-                size_t bytesBuffered = 0;
+                FindCommon::BSONArrayResponseSizeTracker responseSizeTracker;
                 for (long long objCount = 0; objCount < batchSize; objCount++) {
                     BSONObj nextDoc;
                     PlanExecutor::ExecState state = exec->getNext(&nextDoc, nullptr);
@@ -517,14 +508,18 @@ public:
 
                     // If we can't fit this result inside the current batch, then we stash it for
                     // later.
-                    if (!FindCommon::haveSpaceForNext(nextDoc, objCount, bytesBuffered)) {
+                    if (!responseSizeTracker.haveSpaceForNext(nextDoc)) {
                         exec->stashResult(nextDoc);
                         break;
                     }
 
                     try {
                         firstBatch.push_back(ListCollectionsReplyItem::parse(
-                            IDLParserContext("ListCollectionsReplyItem"), nextDoc));
+                            IDLParserContext("ListCollectionsReplyItem",
+                                             false /* apiStrict*/,
+                                             cursorNss.tenantId(),
+                                             SerializationContext::stateCommandReply()),
+                            nextDoc));
                     } catch (const DBException& exc) {
                         LOGV2_ERROR(
                             5254300,
@@ -533,7 +528,7 @@ public:
                             "error"_attr = exc);
                         fassertFailed(5254301);
                     }
-                    bytesBuffered += nextDoc.objsize();
+                    responseSizeTracker.add(nextDoc);
                 }
                 if (exec->isEOF()) {
                     return createListCollectionsCursorReply(
