@@ -35,8 +35,8 @@
 #include "mongo/db/concurrency/lock_manager_defs.h"
 #include "mongo/db/concurrency/locker.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/storage/ticketholder_manager.h"
 #include "mongo/platform/atomic_word.h"
-#include "mongo/util/concurrency/admission_context.h"
 #include "mongo/util/concurrency/spin_lock.h"
 #include "mongo/util/concurrency/ticketholder.h"
 
@@ -141,7 +141,10 @@ public:
     virtual bool unlockGlobal();
 
     virtual LockResult lockRSTLBegin(OperationContext* opCtx, LockMode mode);
-    virtual void lockRSTLComplete(OperationContext* opCtx, LockMode mode, Date_t deadline);
+    virtual void lockRSTLComplete(OperationContext* opCtx,
+                                  LockMode mode,
+                                  Date_t deadline,
+                                  const LockTimeoutCallback& onTimeout);
 
     virtual bool unlockRSTLforPrepare();
 
@@ -191,14 +194,11 @@ public:
     virtual boost::optional<LockerInfo> getLockerInfo(
         boost::optional<SingleThreadedLockStats> lockStatsBase) const final;
 
-    virtual bool saveLockStateAndUnlock(LockSnapshot* stateOut);
+    virtual void saveLockStateAndUnlock(LockSnapshot* stateOut);
 
     virtual void restoreLockState(OperationContext* opCtx, const LockSnapshot& stateToRestore);
-    virtual void restoreLockState(const LockSnapshot& stateToRestore) {
-        restoreLockState(nullptr, stateToRestore);
-    }
 
-    bool releaseWriteUnitOfWorkAndUnlock(LockSnapshot* stateOut) override;
+    void releaseWriteUnitOfWorkAndUnlock(LockSnapshot* stateOut) override;
     void restoreWriteUnitOfWorkAndLock(OperationContext* opCtx,
                                        const LockSnapshot& stateToRestore) override;
 
@@ -237,7 +237,7 @@ public:
                              ResourceId resId,
                              LockMode mode,
                              Date_t deadline) {
-        _lockComplete(opCtx, resId, mode, deadline);
+        _lockComplete(opCtx, resId, mode, deadline, nullptr);
     }
 
 private:
@@ -280,14 +280,15 @@ private:
      * @param mode Mode which was passed to an earlier _lockBegin call. Must match.
      * @param deadline The absolute time point when this lock acquisition will time out, if not yet
      * granted.
+     * @param onTimeout Callback which will run if the lock acquisition is about to time out.
      *
      * Throws an exception if it is interrupted.
      */
-    void _lockComplete(OperationContext* opCtx, ResourceId resId, LockMode mode, Date_t deadline);
-
-    void _lockComplete(ResourceId resId, LockMode mode, Date_t deadline) {
-        _lockComplete(nullptr, resId, mode, deadline);
-    }
+    void _lockComplete(OperationContext* opCtx,
+                       ResourceId resId,
+                       LockMode mode,
+                       Date_t deadline,
+                       const LockTimeoutCallback& onTimeout);
 
     /**
      * The main functionality of the unlock method, except accepts iterator in order to avoid
@@ -377,11 +378,8 @@ private:
     // A structure for accumulating time spent getting flow control tickets.
     FlowControlTicketholder::CurOp _flowControlStats;
 
-    // Keeps state and statistics related to admission control.
-    AdmissionContext _admCtx;
-
     // The global ticketholders of the service context.
-    TicketHolder* _ticketHolder;
+    TicketHolderManager* _ticketHolderManager;
 
     // This will only be valid when holding a ticket.
     boost::optional<Ticket> _ticket;
@@ -417,41 +415,13 @@ public:
     virtual bool isRSTLLocked() const;
 
     bool isGlobalLockedRecursively() override;
+    bool canSaveLockState() override;
 
     virtual bool hasLockPending() const {
         return getWaitingResource().isValid();
     }
 };
 
-/**
- * RAII-style class to opt out of the ticket acquisition mechanism when acquiring a global lock.
- *
- * Operations that acquire the global lock but do not use any storage engine resources are eligible
- * to skip ticket acquisition. Otherwise, a ticket acquisition is required to prevent throughput
- * from suffering under high load.
- */
-class SkipTicketAcquisitionForLock {
-public:
-    SkipTicketAcquisitionForLock(const SkipTicketAcquisitionForLock&) = delete;
-    SkipTicketAcquisitionForLock& operator=(const SkipTicketAcquisitionForLock&) = delete;
-    explicit SkipTicketAcquisitionForLock(OperationContext* opCtx)
-        : _opCtx(opCtx), _shouldAcquireTicket(_opCtx->lockState()->shouldAcquireTicket()) {
-        if (_shouldAcquireTicket) {
-            _opCtx->lockState()->skipAcquireTicket();
-        }
-    }
-
-    ~SkipTicketAcquisitionForLock() {
-        if (_shouldAcquireTicket) {
-            _opCtx->lockState()->setAcquireTicket();
-        }
-    }
-
-private:
-    OperationContext* _opCtx;
-    const bool _shouldAcquireTicket;
-};
-
 /**
  * Retrieves the global lock manager instance.
  * Legacy global lock manager accessor for internal lock implementation * and debugger scripts
