@@ -36,18 +36,19 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/auth/user_acquisition_stats.h"
 #include "mongo/db/catalog/collection_catalog.h"
-#include "mongo/db/clientcursor.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/cursor_id.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/profile_filter.h"
-#include "mongo/db/server_options.h"
+#include "mongo/db/query/query_stats/key.h"
 #include "mongo/db/stats/resource_consumption_metrics.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/logv2/attribute_storage.h"
 #include "mongo/logv2/log_component.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/progress_meter.h"
+#include "mongo/util/system_tick_source.h"
 #include "mongo/util/tick_source.h"
 #include "mongo/util/time_support.h"
 
@@ -66,8 +67,10 @@ struct PlanSummaryStats;
 class OpDebug {
 public:
     /**
-     * Holds counters for execution statistics that are meaningful both for multi-statement
-     * transactions and for individual operations outside of a transaction.
+     * Holds counters for execution statistics that can be accumulated by one or more operations.
+     * They're accumulated as we go for a single operation, but are also extracted and stored
+     * externally if they need to be accumulated across multiple operations (which have multiple
+     * CurOps), including for cursors and multi-statement transactions.
      */
     class AdditiveMetrics {
     public:
@@ -119,6 +122,16 @@ public:
          */
         void incrementKeysDeleted(long long n);
 
+        /**
+         * Increments nreturned by n.
+         */
+        void incrementNreturned(long long n);
+
+        /**
+         * Increments nBatches by 1.
+         */
+        void incrementNBatches();
+
         /**
          * Increments ninserted by n.
          */
@@ -134,6 +147,11 @@ public:
          */
         void incrementPrepareReadConflicts(long long n);
 
+        /**
+         * Increments executionTime by n.
+         */
+        void incrementExecutionTime(Microseconds n);
+
         /**
          * Generates a string showing all non-empty fields. For every non-empty field field1,
          * field2, ..., with corresponding values value1, value2, ..., we will output a string in
@@ -149,6 +167,10 @@ public:
 
         // Number of records that match the query.
         boost::optional<long long> nMatched;
+        // Number of records returned so far.
+        boost::optional<long long> nreturned;
+        // Number of batches returned so far.
+        boost::optional<long long> nBatches;
         // Number of records written (no no-ops).
         boost::optional<long long> nModified;
         boost::optional<long long> ninserted;
@@ -169,6 +191,9 @@ public:
         AtomicWord<long long> prepareReadConflicts{0};
         AtomicWord<long long> writeConflicts{0};
         AtomicWord<long long> temporarilyUnavailableErrors{0};
+
+        // Amount of time spent executing a query.
+        boost::optional<Microseconds> executionTime;
     };
 
     OpDebug() = default;
@@ -178,6 +203,8 @@ public:
                 const ResourceConsumption::OperationMetrics* operationMetrics,
                 logv2::DynamicAttributes* pAttrs) const;
 
+    void reportStorageStats(logv2::DynamicAttributes* pAttrs) const;
+
     /**
      * Appends information about the current operation to "builder"
      *
@@ -239,15 +266,22 @@ public:
     boost::optional<long long> mongotCursorId{boost::none};
     boost::optional<long long> msWaitingForMongot{boost::none};
     long long mongotBatchNum = 0;
+    BSONObj mongotCountVal = BSONObj();
 
     bool hasSortStage{false};  // true if the query plan involves an in-memory sort
 
-    bool usedDisk{false};  // true if the given query used disk
+    bool usedDisk{false};         // true if the given query used disk
+    long long sortSpills{0};      // The total number of spills to disk from sort stages
+    long long sortSpillBytes{0};  // The total number of bytes spilled from sort stages.
+    // The spilled storage size after compression might be different from the bytes spilled.
+    size_t sortTotalDataSizeBytes{0};  // The amount of data we've sorted in bytes
+    long long keysSorted{0};           // The number of keys that we've sorted.
 
     // True if the plan came from the multi-planner (not from the plan cache and not a query with a
     // single solution).
     bool fromMultiPlanner{false};
 
+    bool fromPlanCache{false};
     // True if a replan was triggered during the execution of this operation.
     boost::optional<std::string> replanReason;
 
@@ -262,17 +296,53 @@ public:
     // The hash of the query's "stable" key. This represents the query's shape.
     boost::optional<uint32_t> queryHash;
 
-    // Has a value if this operation is a query. True if the execution tree for the find part of the
-    // query was built exclusively using the classic query engine, false if any part was built using
-    // SBE.
-    boost::optional<bool> classicEngineUsed;
+    /* The QueryStatsInfo struct was created to bundle all the queryStats related fields of CurOp &
+     * OpDebug together (SERVER-83280).
+     *
+     * ClusterClientCursorImpl and ClientCursor also contain _queryStatsKey and _queryStatsKeyHash
+     * members but NOT a wasRateLimited member. Variable names & accesses would be more consistent
+     * across the code if ClusterClientCursorImpl and ClientCursor each also had a QueryStatsInfo
+     * struct, but we considered and rejected two different potential implementations of this:
+     *  - Option 1:
+     *    Declare a QueryStatsInfo struct in each .h file. Every struct would have key and keyHash
+     *    fields, and a wasRateLimited field would be added only to CurOp. But, it seemed confusing
+     *    to have slightly different structs with the same name declared three different times.
+     *  - Option 2:
+     *    Create a query_stats_info.h that declares QueryStatsInfo--identical to the version defined
+     *    in this file. CurOp/OpDebug, ClientCursor, and ClusterClientCursorImpl would then all
+     *    have their own QueryStatsInfo instances, potentially as a unique_ptr or boost::optional. A
+     *    benefit to this would be the ability to to just move the entire QueryStatsInfo struct from
+     *    Op to the Cursor, instead of copying it over field by field (the current method). But:
+     *      - The current code moves ownership of the key, but copies the keyHash. So, for workflows
+     *        that require multiple cursors, like sharding, one cursor would own the key, but all
+     *        cursors would have copies of the keyHash. The problem with trying to move around the
+     *        struct in its entirety is that access to the *entire* struct would be lost on the
+     *        move, meaning there's no way to retain the keyHash (that doesn't largely nullify the
+     *        benefits of having the struct).
+     *      - It seemed odd to have ClientCursor and ClusterClientCursorImpl using the struct but
+     *        never needing the wasRateLimited field.
+     */
 
-    // Has a value if this operation is an aggregation query. True if `DocumentSources` were
-    // involved in the execution tree for this query, false if they were not.
-    boost::optional<bool> documentSourceUsed;
+    // Note that the only case when the three fields of the below struct are null, none, and false
+    // is if the query stats feature flag is turned off.
+    struct QueryStatsInfo {
+        // Uniquely identifies one query stats entry.
+        // nullptr if `wasRateLimited` is true.
+        std::unique_ptr<query_stats::Key> key;
+        // A cached value of `absl::HashOf(key)`.
+        // Always populated if `key` is non-null. boost::none if `wasRateLimited` is true.
+        boost::optional<std::size_t> keyHash;
+        // True if the request was rate limited and stats should not be collected.
+        bool wasRateLimited = false;
+        // True if the request was a change stream request.
+        // TODO SERVER-89058 will make it true for all tailable cursors.
+        bool willNeverExhaust = false;
+    };
 
-    // Indicates whether this operation used the common query framework (CQF).
-    bool cqfUsed{false};
+    QueryStatsInfo queryStatsInfo;
+
+    // The query framework that this operation used. Will be unknown for non query operations.
+    PlanExecutor::QueryFramework queryFramework{PlanExecutor::QueryFramework::kUnknown};
 
     // Tracks the amount of indexed loop joins in a pushed down lookup stage.
     int indexedLoopJoin{0};
@@ -286,12 +356,21 @@ public:
     // Tracks the amount of spills by hash lookup in a pushed down lookup stage.
     int hashLookupSpillToDisk{0};
 
+    // Tracks the number of spilled bytes by hash lookup in a pushed down lookup stage. The spilled
+    // storage size after compression might be different from the bytes spilled.
+    long long hashLookupSpillToDiskBytes{0};
+
     // Details of any error (whether from an exception or a command returning failure).
     Status errInfo = Status::OK();
 
-    // response info
-    Microseconds executionTime{0};
-    long long nreturned{-1};
+    // Amount of time spent planning the query. Begins after parsing and ends
+    // after optimizations.
+    Microseconds planningTime{0};
+
+    // Amount of CPU time used by this thread. Will remain -1 if this platform does not support
+    // this feature.
+    Nanoseconds cpuTime{-1};
+
     int responseLength{-1};
 
     // Shard targeting info.
@@ -308,13 +387,25 @@ public:
     // refreshes.
     Milliseconds catalogCacheCollectionLookupMillis{0};
 
+    // Total time spent looking up index entries in the local cache, including eventual refreshes.
+    Milliseconds catalogCacheIndexLookupMillis{0};
+
     // Stores the duration of time spent waiting for the shard to refresh the database and wait for
     // the database critical section.
     Milliseconds databaseVersionRefreshMillis{0};
 
     // Stores the duration of time spent waiting for the shard to refresh the collection and wait
     // for the collection critical section.
-    Milliseconds shardVersionRefreshMillis{0};
+    Milliseconds placementVersionRefreshMillis{0};
+
+    // Stores the duration of time spent waiting for the specified user write concern to
+    // be fulfilled.
+    Milliseconds waitForWriteConcernDurationMillis{0};
+
+    // Stores the total time an operation spends with an uncommitted oplog slot held open. Indicator
+    // that an operation is holding back replication by causing oplog holes to remain open for
+    // unusual amounts of time.
+    Microseconds totalOplogSlotDurationMicros{0};
 
     // Stores the amount of the data processed by the throttle cursors in MB/sec.
     boost::optional<float> dataThroughputLastSecond;
@@ -323,11 +414,13 @@ public:
     // Used to track the amount of time spent waiting for a response from remote operations.
     boost::optional<Microseconds> remoteOpWaitTime;
 
-    // Stores additive metrics.
+    // Stores the current operation's count of these metrics. If they are needed to be accumulated
+    // elsewhere, they should be extracted by another aggregator (like the ClientCursor) to ensure
+    // these only ever reflect just this CurOp's consumption.
     AdditiveMetrics additiveMetrics;
 
     // Stores storage statistics.
-    std::shared_ptr<StorageStats> storageStats;
+    std::unique_ptr<StorageStats> storageStats;
 
     bool waitingForFlowControl{false};
 
@@ -386,6 +479,8 @@ public:
                                          bool backtraceMode,
                                          BSONObjBuilder* infoBuilder);
 
+    static bool currentOpBelongsToTenant(Client* client, TenantId tenantId);
+
     /**
      * Serializes the fields of a GenericCursor which do not appear elsewhere in the currentOp
      * output. If 'maxQuerySize' is given, truncates the cursor's originatingCommand but preserves
@@ -395,9 +490,19 @@ public:
                                                      boost::optional<size_t> maxQuerySize);
 
     /**
-     * Constructs a nested CurOp at the top of the given "opCtx"'s CurOp stack.
+     * Pushes this CurOp to the top of the given "opCtx"'s CurOp stack.
      */
-    explicit CurOp(OperationContext* opCtx);
+    void push(OperationContext* opCtx);
+
+    CurOp() = default;
+
+    /**
+     * This allows the caller to set the command on the CurOp without using setCommand_inlock and
+     * having to acquire the Client lock or having to leave a comment indicating why the
+     * client lock isn't necessary.
+     */
+    explicit CurOp(const Command* command) : _command{command} {}
+
     ~CurOp();
 
     /**
@@ -406,20 +511,26 @@ public:
      * is set early in the request processing backend and does not typically need to be called
      * thereafter. Locks the client as needed to apply the specified settings.
      */
-    void setGenericOpRequestDetails(OperationContext* opCtx,
-                                    const NamespaceString& nss,
+    void setGenericOpRequestDetails(NamespaceString nss,
                                     const Command* command,
                                     BSONObj cmdObj,
                                     NetworkOp op);
 
+    /**
+     * Sets metrics collected at the end of an operation onto curOp's OpDebug instance. Note that
+     * this is used in tandem with OpDebug::setPlanSummaryMetrics so should not repeat any metrics
+     * collected there.
+     */
+    void setEndOfOpMetrics(long long nreturned);
+
     /**
      * Marks the operation end time, records the length of the client response if a valid response
      * exists, and then - subject to the current values of slowMs and sampleRate - logs this CurOp
      * to file under the given LogComponent. Returns 'true' if, in addition to being logged, this
      * operation should also be profiled.
      */
-    bool completeAndLogOperation(OperationContext* opCtx,
-                                 logv2::LogComponent logComponent,
+    bool completeAndLogOperation(logv2::LogComponent logComponent,
+                                 std::shared_ptr<const ProfileFilter> filter,
                                  boost::optional<size_t> responseLength = boost::none,
                                  boost::optional<long long> slowMsOverride = boost::none,
                                  bool forceLog = false);
@@ -444,7 +555,8 @@ public:
         return _originatingCommand;
     }
 
-    void enter_inlock(const char* ns, int dbProfileLevel);
+    void enter_inlock(NamespaceString nss, int dbProfileLevel);
+    void enter_inlock(const DatabaseName& dbName, int dbProfileLevel);
 
     /**
      * Sets the type of the current network operation.
@@ -480,9 +592,7 @@ public:
     /**
      * Gets the name of the namespace on which the current operation operates.
      */
-    std::string getNS() const {
-        return _ns;
-    }
+    std::string getNS() const;
 
     /**
      * Returns a const pointer to the UserAcquisitionStats for the current operation.
@@ -502,8 +612,8 @@ public:
     /**
      * Gets the name of the namespace on which the current operation operates.
      */
-    NamespaceString getNSS() const {
-        return NamespaceString{_ns};
+    const NamespaceString& getNSS() const {
+        return _nss;
     }
 
     /**
@@ -513,7 +623,7 @@ public:
      *
      * When a custom filter is set, we conservatively assume it would match this operation.
      */
-    bool shouldDBProfile(OperationContext* opCtx) {
+    bool shouldDBProfile() {
         // Profile level 2 should override any sample rate or slowms settings.
         if (_dbprofile >= 2)
             return true;
@@ -521,10 +631,10 @@ public:
         if (_dbprofile <= 0)
             return false;
 
-        if (CollectionCatalog::get(opCtx)->getDatabaseProfileSettings(getNSS().db()).filter)
+        if (CollectionCatalog::get(opCtx())->getDatabaseProfileSettings(getNSS().dbName()).filter)
             return true;
 
-        return elapsedTimeExcludingPauses() >= Milliseconds{serverGlobalParams.slowMS};
+        return elapsedTimeExcludingPauses() >= Milliseconds{serverGlobalParams.slowMS.load()};
     }
 
     /**
@@ -568,7 +678,7 @@ public:
     //
 
     void ensureStarted() {
-        static_cast<void>(startTime());
+        (void)startTime();
     }
     bool isStarted() const {
         return _start.load() != 0;
@@ -612,12 +722,12 @@ public:
      * This method is separate from startRemoteOpWait because operation types that do record
      * remoteOpWait, such as a getMore of a sharded aggregation, should always include the
      * remoteOpWait field even if its value is zero. An operation should call
-     * enableRecordRemoteOpWait() to declare that it wants to report remoteOpWait, and call
+     * ensureRecordRemoteOpWait() to declare that it wants to report remoteOpWait, and call
      * startRemoteOpWaitTimer()/stopRemoteOpWaitTimer() to measure the time.
      *
      * This timer uses the same clock source as elapsedTimeTotal().
      */
-    void enableRecordRemoteOpWait() {
+    void ensureRecordRemoteOpWait() {
         if (!_debug.remoteOpWaitTime) {
             _debug.remoteOpWaitTime.emplace(0);
         }
@@ -626,10 +736,15 @@ public:
     /**
      * Starts the remoteOpWait timer.
      *
-     * Does nothing if enableRecordRemoteOpWait() was not called.
+     * Does nothing if ensureRecordRemoteOpWait() was not called or the current operation was not
+     * marked as started.
      */
     void startRemoteOpWaitTimer() {
-        invariant(isStarted());
+        // There are some commands that send remote operations but do not mark the current operation
+        // as started. We do not record remote op wait time for those commands.
+        if (!isStarted()) {
+            return;
+        }
         invariant(!isDone());
         invariant(!isPaused());
         invariant(!_remoteOpStartTime);
@@ -641,10 +756,15 @@ public:
     /**
      * Stops the remoteOpWait timer.
      *
-     * Does nothing if enableRecordRemoteOpWait() was not called.
+     * Does nothing if ensureRecordRemoteOpWait() was not called or the current operation was not
+     * marked as started.
      */
     void stopRemoteOpWaitTimer() {
-        invariant(isStarted());
+        // There are some commands that send remote operations but do not mark the current operation
+        // as started. We do not record remote op wait time for those commands.
+        if (!isStarted()) {
+            return;
+        }
         invariant(!isDone());
         invariant(!isPaused());
         if (_debug.remoteOpWaitTime) {
@@ -696,14 +816,75 @@ public:
 
         return computeElapsedTimeTotal(start, _end.load()) - _totalPausedDuration;
     }
+    /**
+    * The planningTimeMicros metric, reported in the system profiler and in queryStats, is measured
+    * using the Curop instance's _tickSource. Currently, _tickSource is only paused in places where
+    logical work is being done. If this were to change, and _tickSource
+    were to be paused during query planning for reasons unrelated to the work of
+    planning/optimization, it would break the planning time measurement below.
+    *
+    */
+    void beginQueryPlanningTimer() {
+        // This is an inner executor/cursor, the metrics for which don't get tracked by
+        // OpDebug::planningTime.
+        if (_queryPlanningStart.load() != 0) {
+            return;
+        }
+        _queryPlanningStart = _tickSource->getTicks();
+    }
+
+    void stopQueryPlanningTimer() {
+        // The planningTime metric is defined as being done once PrepareExecutionHelper::prepare()
+        // is hit, which calls this function to stop the timer. As certain queries like $lookup
+        // require inner cursors/executors that will follow this same codepath, it is important to
+        // make sure the metric exclusively captures the time associated with the outermost cursor.
+        // This is done by making sure planningTime has not already been set and that start has been
+        // marked (as inner executors are prepared outside of the codepath that begins the planning
+        // timer).
+        auto start = _queryPlanningStart.load();
+        if (debug().planningTime == Microseconds{0} && start != 0) {
+            _queryPlanningEnd = _tickSource->getTicks();
+            debug().planningTime = computeElapsedTimeTotal(start, _queryPlanningEnd.load());
+        }
+    }
+
+    /**
+     * Starts the waitForWriteConcern timer.
+     *
+     * The timer must be ended before it can be started again.
+     */
+    void beginWaitForWriteConcernTimer() {
+        invariant(_waitForWriteConcernStart.load() == 0);
+        _waitForWriteConcernStart = _tickSource->getTicks();
+        _waitForWriteConcernEnd = 0;
+    }
+
+    /**
+     * Stops the waitForWriteConcern timer.
+     *
+     * Does nothing if the timer has not been started.
+     */
+    void stopWaitForWriteConcernTimer() {
+        auto start = _waitForWriteConcernStart.load();
+        if (start != 0) {
+            _waitForWriteConcernEnd = _tickSource->getTicks();
+            debug().waitForWriteConcernDurationMillis += duration_cast<Milliseconds>(
+                computeElapsedTimeTotal(start, _waitForWriteConcernEnd.load()));
+            _waitForWriteConcernStart = 0;
+        }
+    }
+
+    /**
+     * If the platform supports the CPU timer, and we haven't collected this operation's CPU time
+     * already, then calculates this operation's CPU time and stores it on the 'OpDebug'.
+     */
+    void calculateCpuTime();
 
     /**
      * 'opDescription' must be either an owned BSONObj or guaranteed to outlive the OperationContext
      * it is associated with.
      */
-    void setOpDescription_inlock(const BSONObj& opDescription) {
-        _opDescription = opDescription;
-    }
+    void setOpDescription_inlock(const BSONObj& opDescription);
 
     /**
      * Sets the original command object.
@@ -732,7 +913,7 @@ public:
      * If called from a thread other than the one executing the operation associated with this
      * CurOp, it is necessary to lock the associated Client object before executing this method.
      */
-    void reportState(OperationContext* opCtx, BSONObjBuilder* builder, bool truncateOps = false);
+    void reportState(BSONObjBuilder* builder, bool truncateOps = false);
 
     /**
      * Sets the message for FailPoints used.
@@ -749,9 +930,8 @@ public:
     /**
      * Sets the message and the progress meter for this CurOp.
      *
-     * While it is necessary to hold the lock while this method executes, the
-     * "hit" and "finished" methods of ProgressMeter may be called safely from
-     * the thread executing the operation without locking the Client.
+     * Accessors and modifiers of ProgressMeter associated with the CurOp must follow the same
+     * locking scheme as CurOp. It is necessary to hold the lock while this method executes.
      */
     ProgressMeter& setProgress_inlock(StringData name,
                                       unsigned long long progressMeterTotal = 0,
@@ -797,7 +977,8 @@ public:
      * generally the Context should set this up
      * but sometimes you want to do it ahead of time
      */
-    void setNS_inlock(StringData ns);
+    void setNS_inlock(NamespaceString nss);
+    void setNS_inlock(const DatabaseName& dbName);
 
     StringData getPlanSummary() const {
         return _planSummary;
@@ -821,28 +1002,25 @@ public:
         _tickSource = tickSource;
     }
 
-    /**
-     * Merge match counters from the current operation into the global map and stop counting.
-     */
-    void stopMatchExprCounter();
-
-    /**
-     * Increment the counter for the match expression with given name in the current operation.
-     */
-    void incrementMatchExprCounter(StringData name);
+    void setShouldOmitDiagnosticInformation_inlock(WithLock, bool shouldOmitDiagnosticInfo) {
+        _shouldOmitDiagnosticInformation = shouldOmitDiagnosticInfo;
+    }
+    bool getShouldOmitDiagnosticInformation() const {
+        return _shouldOmitDiagnosticInformation;
+    }
 
 private:
     class CurOpStack;
 
+    /**
+     * Gets the OperationContext associated with this CurOp.
+     * This must only be called after the CurOp has been pushed to an OperationContext's CurOpStack.
+     */
+    OperationContext* opCtx();
+
     TickSource::Tick startTime();
     Microseconds computeElapsedTimeTotal(TickSource::Tick startTime,
                                          TickSource::Tick endTime) const;
-
-    /**
-     * Adds 'this' to the stack of active CurOp objects.
-     */
-    void _finishInit(OperationContext* opCtx, CurOpStack* stack);
-
     /**
      * Handles failpoints that check whether a command has completed or not.
      * Used for testing purposes instead of the getLog command.
@@ -851,10 +1029,14 @@ private:
 
     static const OperationContext::Decoration<CurOpStack> _curopStack;
 
-    CurOp(OperationContext*, CurOpStack*);
+    // The stack containing this CurOp instance.
+    // This is set when this instance is pushed to the stack.
+    CurOpStack* _stack{nullptr};
 
-    CurOpStack* _stack;
+    // The CurOp beneath this CurOp instance in its stack, if any.
+    // This is set when this instance is pushed to a non-empty stack.
     CurOp* _parent{nullptr};
+
     const Command* _command{nullptr};
 
     // The time at which this CurOp instance was marked as started.
@@ -863,6 +1045,10 @@ private:
     // The time at which this CurOp instance was marked as done or 0 if the CurOp is not yet done.
     std::atomic<TickSource::Tick> _end{0};  // NOLINT
 
+    // This CPU timer tracks the CPU time spent for this operation. Will be nullptr on unsupported
+    // platforms.
+    std::unique_ptr<OperationCPUTimer> _cpuTimer;
+
     // The time at which this CurOp instance had its timer paused, or 0 if the timer is not
     // currently paused.
     TickSource::Tick _lastPauseTime{0};
@@ -883,7 +1069,7 @@ private:
 
     bool _isCommand{false};
     int _dbprofile{0};  // 0=off, 1=slow, 2=all
-    std::string _ns;
+    NamespaceString _nss;
     BSONObj _opDescription;
     BSONObj _originatingCommand;  // Used by getMore to display original command.
     OpDebug _debug;
@@ -895,12 +1081,24 @@ private:
     boost::optional<GenericCursor> _genericCursor;
 
     std::string _planSummary;
-    boost::optional<SingleThreadedLockStats>
-        _lockStatsBase;  // This is the snapshot of lock stats taken when curOp is constructed.
+
+    // The snapshot of lock stats taken when this CurOp instance is pushed to a
+    // CurOpStack.
+    boost::optional<SingleThreadedLockStats> _lockStatsBase;
 
     UserAcquisitionStats _userAcquisitionStats;
 
-    TickSource* _tickSource = nullptr;
+    TickSource* _tickSource = globalSystemTickSource();
+    // These values are used to calculate the amount of time spent planning a query.
+    std::atomic<TickSource::Tick> _queryPlanningStart{0};  // NOLINT
+    std::atomic<TickSource::Tick> _queryPlanningEnd{0};    // NOLINT
+
+    // These values are used to calculate the amount of time spent waiting for write concern.
+    std::atomic<TickSource::Tick> _waitForWriteConcernStart{0};  // NOLINT
+    std::atomic<TickSource::Tick> _waitForWriteConcernEnd{0};    // NOLINT
+
+    // Flag to decide if diagnostic information should be omitted.
+    bool _shouldOmitDiagnosticInformation{false};
 };
 
 }  // namespace mongo
