@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/dbdirectclient.h"
 
 #include <boost/core/swap.hpp>
@@ -125,7 +122,8 @@ namespace {
 DbResponse loopbackBuildResponse(OperationContext* const opCtx, Message& toSend) {
     DirectClientScope directClientScope(opCtx);
 
-    CurOp curOp(opCtx);
+    CurOp curOp;
+    curOp.push(opCtx);
 
     toSend.header().setId(nextMessageId());
     toSend.header().setResponseToMsgId(0);
@@ -155,25 +153,41 @@ std::unique_ptr<DBClientCursor> DBDirectClient::find(FindCommandRequest findRequ
 
 write_ops::FindAndModifyCommandReply DBDirectClient::findAndModify(
     const write_ops::FindAndModifyCommandRequest& findAndModify) {
-    auto response = runCommand(findAndModify.serialize({}));
+    auto request = findAndModify.serialize({});
+    if (const auto& tenant = findAndModify.getDbName().tenantId()) {
+        request.setDollarTenant(tenant.value());
+    }
+    auto response = runCommand(std::move(request));
     return FindAndModifyOp::parseResponse(response->getCommandReply());
 }
 
 write_ops::InsertCommandReply DBDirectClient::insert(
     const write_ops::InsertCommandRequest& insert) {
-    auto response = runCommand(insert.serialize({}));
+    auto request = insert.serialize({});
+    if (const auto& tenant = insert.getDbName().tenantId()) {
+        request.setDollarTenant(tenant.value());
+    }
+    auto response = runCommand(request);
     return InsertOp::parseResponse(response->getCommandReply());
 }
 
 write_ops::UpdateCommandReply DBDirectClient::update(
     const write_ops::UpdateCommandRequest& update) {
-    auto response = runCommand(update.serialize({}));
+    auto request = update.serialize({});
+    if (const auto& tenant = update.getDbName().tenantId()) {
+        request.setDollarTenant(tenant.value());
+    }
+    auto response = runCommand(request);
     return UpdateOp::parseResponse(response->getCommandReply());
 }
 
 write_ops::DeleteCommandReply DBDirectClient::remove(
     const write_ops::DeleteCommandRequest& remove) {
-    auto response = runCommand(remove.serialize({}));
+    auto request = remove.serialize({});
+    if (const auto& tenant = remove.getDbName().tenantId()) {
+        request.setDollarTenant(tenant.value());
+    }
+    auto response = runCommand(request);
     return DeleteOp::parseResponse(response->getCommandReply());
 }
 
@@ -184,16 +198,16 @@ long long DBDirectClient::count(const NamespaceStringOrUUID nsOrUuid,
                                 int skip,
                                 boost::optional<BSONObj> readConcernObj) {
     invariant(!readConcernObj, "passing readConcern to DBDirectClient functions is not supported");
-    DirectClientScope directClientScope(_opCtx);
     BSONObj cmdObj = _countCmd(nsOrUuid, query, options, limit, skip, boost::none);
+    auto request = OpMsgRequestBuilder::create(nsOrUuid.dbName(), cmdObj);
 
-    auto dbName = (nsOrUuid.uuid() ? nsOrUuid.dbname() : (*nsOrUuid.nss()).db().toString());
-
-    auto result = CommandHelpers::runCommandDirectly(
-        _opCtx, OpMsgRequest::fromDBAndBody(dbName, std::move(cmdObj)));
+    // Calls runCommand instead of runCommandDirectly to ensure the tenant inforamtion of this
+    // command gets validated and is used for parsing the command request.
+    auto response = runCommand(request);
+    auto& result = response->getCommandReply();
 
     uassertStatusOK(getStatusFromCommandResult(result));
-    return static_cast<unsigned long long>(result["n"].numberLong());
+    return result["n"].numberLong();
 }
 
 }  // namespace mongo
