@@ -57,12 +57,12 @@ public:
      * before actually doing any unpacking.
      */
     BucketUnpacker makeBucketUnpacker(std::set<std::string> fields,
-                                      BucketUnpacker::Behavior behavior,
+                                      BucketSpec::Behavior behavior,
                                       BSONObj bucket,
                                       boost::optional<std::string> metaFieldName = boost::none) {
-        auto spec = BucketSpec{kUserDefinedTimeName.toString(), metaFieldName, std::move(fields)};
-
-        BucketUnpacker unpacker{std::move(spec), behavior};
+        auto spec =
+            BucketSpec{kUserDefinedTimeName.toString(), metaFieldName, std::move(fields), behavior};
+        BucketUnpacker unpacker{std::move(spec)};
         unpacker.reset(std::move(bucket));
         return unpacker;
     }
@@ -72,12 +72,13 @@ public:
      * the given 'bucket'. Asserts that 'reset()' throws the given 'errorCode'.
      */
     void assertUnpackerThrowsCode(std::set<std::string> fields,
-                                  BucketUnpacker::Behavior behavior,
+                                  BucketSpec::Behavior behavior,
                                   BSONObj bucket,
                                   boost::optional<std::string> metaFieldName,
                                   int errorCode) {
-        auto spec = BucketSpec{kUserDefinedTimeName.toString(), metaFieldName, std::move(fields)};
-        BucketUnpacker unpacker{std::move(spec), behavior};
+        auto spec =
+            BucketSpec{kUserDefinedTimeName.toString(), metaFieldName, std::move(fields), behavior};
+        BucketUnpacker unpacker{std::move(spec)};
         ASSERT_THROWS_CODE(unpacker.reset(std::move(bucket)), AssertionException, errorCode);
     }
 
@@ -153,14 +154,11 @@ public:
                 int num = std::distance(col.begin(), col.end()) - 1;
                 ASSERT(num >= 0);
 
-                BSONColumnBuilder builder(fieldName);
+                BSONColumnBuilder builder;
                 auto it = col.begin();
                 for (int i = 0; i < num; ++i) {
                     auto elem = *it;
-                    if (!elem.eoo())
-                        builder.append(elem);
-                    else
-                        builder.skip();
+                    builder.append(elem);
                     ++it;
                 }
 
@@ -181,7 +179,7 @@ TEST_F(BucketUnpackerTest, UnpackBasicIncludeAllMeasurementFields) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(bucket),
                                        kUserDefinedMetaName.toString());
 
@@ -205,7 +203,7 @@ TEST_F(BucketUnpackerTest, ExcludeASingleField) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
 
@@ -220,8 +218,7 @@ TEST_F(BucketUnpackerTest, ExcludeASingleField) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, EmptyIncludeGetsEmptyMeasurements) {
@@ -234,7 +231,7 @@ TEST_F(BucketUnpackerTest, EmptyIncludeGetsEmptyMeasurements) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kInclude,
+                                           BucketSpec::Behavior::kInclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
 
@@ -247,8 +244,7 @@ TEST_F(BucketUnpackerTest, EmptyIncludeGetsEmptyMeasurements) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, EmptyExcludeMaterializesAllFields) {
@@ -261,7 +257,7 @@ TEST_F(BucketUnpackerTest, EmptyExcludeMaterializesAllFields) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -276,8 +272,7 @@ TEST_F(BucketUnpackerTest, EmptyExcludeMaterializesAllFields) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, SparseColumnsWhereOneColumnIsExhaustedBeforeTheOther) {
@@ -290,7 +285,7 @@ TEST_F(BucketUnpackerTest, SparseColumnsWhereOneColumnIsExhaustedBeforeTheOther)
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -303,8 +298,7 @@ TEST_F(BucketUnpackerTest, SparseColumnsWhereOneColumnIsExhaustedBeforeTheOther)
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, UnpackBasicIncludeWithDollarPrefix) {
@@ -318,7 +312,7 @@ TEST_F(BucketUnpackerTest, UnpackBasicIncludeWithDollarPrefix) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kInclude,
+                                           BucketSpec::Behavior::kInclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -333,8 +327,7 @@ TEST_F(BucketUnpackerTest, UnpackBasicIncludeWithDollarPrefix) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, BucketsWithMetadataOnly) {
@@ -346,7 +339,7 @@ TEST_F(BucketUnpackerTest, BucketsWithMetadataOnly) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -359,8 +352,7 @@ TEST_F(BucketUnpackerTest, BucketsWithMetadataOnly) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, UnorderedRowKeysDoesntAffectMaterialization) {
@@ -373,7 +365,7 @@ TEST_F(BucketUnpackerTest, UnorderedRowKeysDoesntAffectMaterialization) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -403,7 +395,7 @@ TEST_F(BucketUnpackerTest, MissingMetaFieldDoesntMaterializeMetadata) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -418,8 +410,7 @@ TEST_F(BucketUnpackerTest, MissingMetaFieldDoesntMaterializeMetadata) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, MissingMetaFieldDoesntMaterializeMetadataUnorderedKeys) {
@@ -431,7 +422,7 @@ TEST_F(BucketUnpackerTest, MissingMetaFieldDoesntMaterializeMetadataUnorderedKey
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -459,7 +450,7 @@ TEST_F(BucketUnpackerTest, ExcludedMetaFieldDoesntMaterializeMetadataWhenBucketH
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -474,8 +465,7 @@ TEST_F(BucketUnpackerTest, ExcludedMetaFieldDoesntMaterializeMetadataWhenBucketH
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, UnpackerResetThrowsOnUndefinedMeta) {
@@ -487,15 +477,14 @@ TEST_F(BucketUnpackerTest, UnpackerResetThrowsOnUndefinedMeta) {
 
     auto test = [&](BSONObj bucket) {
         assertUnpackerThrowsCode(fields,
-                                 BucketUnpacker::Behavior::kExclude,
+                                 BucketSpec::Behavior::kExclude,
                                  std::move(bucket),
                                  kUserDefinedMetaName.toString(),
                                  5369600);
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, UnpackerResetThrowsOnUnexpectedMeta) {
@@ -508,15 +497,14 @@ TEST_F(BucketUnpackerTest, UnpackerResetThrowsOnUnexpectedMeta) {
 
     auto test = [&](BSONObj bucket) {
         assertUnpackerThrowsCode(fields,
-                                 BucketUnpacker::Behavior::kExclude,
+                                 BucketSpec::Behavior::kExclude,
                                  std::move(bucket),
                                  boost::none /* no metaField provided */,
                                  5369601);
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, NullMetaInBucketMaterializesAsNull) {
@@ -528,7 +516,7 @@ TEST_F(BucketUnpackerTest, NullMetaInBucketMaterializesAsNull) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -543,8 +531,7 @@ TEST_F(BucketUnpackerTest, NullMetaInBucketMaterializesAsNull) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, GetNextHandlesMissingMetaInBucket) {
@@ -561,7 +548,7 @@ TEST_F(BucketUnpackerTest, GetNextHandlesMissingMetaInBucket) {
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(fields,
-                                           BucketUnpacker::Behavior::kExclude,
+                                           BucketSpec::Behavior::kExclude,
                                            std::move(bucket),
                                            kUserDefinedMetaName.toString());
         ASSERT_TRUE(unpacker.hasNext());
@@ -576,22 +563,22 @@ TEST_F(BucketUnpackerTest, GetNextHandlesMissingMetaInBucket) {
     };
 
     test(bucket);
-    test(*timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-              .compressedBucket);
+    test(*timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket);
 }
 
 TEST_F(BucketUnpackerTest, EmptyDataRegionInBucketIsTolerated) {
     std::set<std::string> fields{};
 
-    auto bucket = Document{{"_id", 1},
-                           {"control", Document{{"version", 1}}},
-                           {"meta", Document{{"m1", 999}, {"m2", 9999}}},
-                           {"data", Document{}}}
-                      .toBson();
+    auto bucket = Document{
+        {"_id", 1},
+        {"control", Document{{"version", 1}}},
+        {"meta", Document{{"m1", 999}, {"m2", 9999}}},
+        {"data",
+         Document{}}}.toBson();
 
     auto test = [&](BSONObj bucket) {
         auto unpacker = makeBucketUnpacker(
-            fields, BucketUnpacker::Behavior::kExclude, bucket, kUserDefinedMetaName.toString());
+            fields, BucketSpec::Behavior::kExclude, bucket, kUserDefinedMetaName.toString());
         ASSERT_FALSE(unpacker.hasNext());
     };
 
@@ -603,7 +590,7 @@ TEST_F(BucketUnpackerTest, UnpackerResetThrowsOnEmptyBucket) {
 
     auto bucket = Document{};
     assertUnpackerThrowsCode(std::move(fields),
-                             BucketUnpacker::Behavior::kExclude,
+                             BucketSpec::Behavior::kExclude,
                              bucket.toBson(),
                              kUserDefinedMetaName.toString(),
                              5346510);
@@ -621,51 +608,121 @@ TEST_F(BucketUnpackerTest, EraseMetaFromFieldSetAndDetermineIncludeMeta) {
     }
 })");
     auto unpacker = makeBucketUnpacker(empFields,
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(bucket),
                                        kUserDefinedMetaName.toString());
 
     // Tests a spec with 'metaField' in include list.
     std::set<std::string> fields{kUserDefinedMetaName.toString()};
 
-    auto specWithMetaInclude = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(fields)};
+    auto specWithMetaInclude = BucketSpec{kUserDefinedTimeName.toString(),
+                                          kUserDefinedMetaName.toString(),
+                                          std::move(fields),
+                                          BucketSpec::Behavior::kInclude};
     // This calls eraseMetaFromFieldSetAndDetermineIncludeMeta.
-    unpacker.setBucketSpecAndBehavior(std::move(specWithMetaInclude),
-                                      BucketUnpacker::Behavior::kInclude);
+    unpacker.setBucketSpec(std::move(specWithMetaInclude));
     ASSERT_TRUE(unpacker.includeMetaField());
     ASSERT_EQ(unpacker.bucketSpec().fieldSet().count(kUserDefinedMetaName.toString()), 0);
 
     std::set<std::string> fieldsNoMetaInclude{"foo"};
     auto specWithFooInclude = BucketSpec{kUserDefinedTimeName.toString(),
                                          kUserDefinedMetaName.toString(),
-                                         std::move(fieldsNoMetaInclude)};
+                                         std::move(fieldsNoMetaInclude),
+                                         BucketSpec::Behavior::kInclude};
 
     std::set<std::string> fieldsNoMetaExclude{"foo"};
     auto specWithFooExclude = BucketSpec{kUserDefinedTimeName.toString(),
                                          kUserDefinedMetaName.toString(),
-                                         std::move(fieldsNoMetaExclude)};
+                                         std::move(fieldsNoMetaExclude),
+                                         BucketSpec::Behavior::kExclude};
 
-    unpacker.setBucketSpecAndBehavior(std::move(specWithFooExclude),
-                                      BucketUnpacker::Behavior::kExclude);
+    unpacker.setBucketSpec(std::move(specWithFooExclude));
     ASSERT_TRUE(unpacker.includeMetaField());
-    unpacker.setBucketSpecAndBehavior(std::move(specWithFooInclude),
-                                      BucketUnpacker::Behavior::kInclude);
+    unpacker.setBucketSpec(std::move(specWithFooInclude));
     ASSERT_FALSE(unpacker.includeMetaField());
 
     // Tests a spec with 'metaField' not in exclude list.
     std::set<std::string> excludeFields{};
-    auto specMetaExclude = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(excludeFields)};
+    auto specMetaExclude = BucketSpec{kUserDefinedTimeName.toString(),
+                                      kUserDefinedMetaName.toString(),
+                                      std::move(excludeFields),
+                                      BucketSpec::Behavior::kExclude};
+
     auto specMetaInclude = specMetaExclude;
-    unpacker.setBucketSpecAndBehavior(std::move(specMetaExclude),
-                                      BucketUnpacker::Behavior::kExclude);
+    specMetaInclude.setBehavior(BucketSpec::Behavior::kInclude);
+
+    unpacker.setBucketSpec(std::move(specMetaExclude));
     ASSERT_TRUE(unpacker.includeMetaField());
-    unpacker.setBucketSpecAndBehavior(std::move(specMetaInclude),
-                                      BucketUnpacker::Behavior::kInclude);
+    unpacker.setBucketSpec(std::move(specMetaInclude));
     ASSERT_FALSE(unpacker.includeMetaField());
 }
 
+TEST_F(BucketUnpackerTest, EraseUnneededComputedMetaProjFieldsWithInclusiveProject) {
+    auto bucket = fromjson(R"(
+{
+    control: {version: 1},
+    data: {
+        _id: {'0':4, '1':5, '2':6},
+        time: {'0':4, '1': 5, '2': 6}
+    }
+})");
+    std::set<std::string> unpackerFields{kUserDefinedTimeName.toString()};
+    auto unpacker = makeBucketUnpacker(unpackerFields,
+                                       BucketSpec::Behavior::kInclude,
+                                       std::move(bucket),
+                                       kUserDefinedMetaName.toString());
+
+    // Add fields to '_computedMetaProjFields'.
+    unpacker.addComputedMetaProjFields({"hello"_sd, "bye"_sd});
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("hello"));
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("bye"));
+
+    auto spec = unpacker.bucketSpec();
+    std::set<std::string> includeFields{kUserDefinedTimeName.toString(), "bye"};
+    spec.setFieldSet(includeFields);
+    spec.setBehavior(BucketSpec::Behavior::kInclude);
+
+    // This calls eraseUnneededComputedMetaProjFields().
+    unpacker.setBucketSpec(std::move(spec));
+    // As "hello" was not in the includes, it should be removed.
+    ASSERT_FALSE(unpacker.bucketSpec().computedMetaProjFields().contains("hello"));
+    // As "bye" was in the includes, it should still be in '_computedMetaProjFields'.
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("bye"));
+}
+
+TEST_F(BucketUnpackerTest, EraseUnneededComputedMetaProjFieldsWithExclusiveProject) {
+    auto bucket = fromjson(R"(
+{
+    control: {version: 1},
+    data: {
+        _id: {'0':4, '1':5, '2':6},
+        time: {'0':4, '1': 5, '2': 6}
+    }
+})");
+    std::set<std::string> unpackerFields{kUserDefinedTimeName.toString()};
+    auto unpacker = makeBucketUnpacker(unpackerFields,
+                                       BucketSpec::Behavior::kInclude,
+                                       std::move(bucket),
+                                       kUserDefinedMetaName.toString());
+
+    // Add fields to '_computedMetaProjFields'.
+    unpacker.addComputedMetaProjFields({"hello"_sd, "bye"_sd});
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("hello"));
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("bye"));
+
+    auto spec = unpacker.bucketSpec();
+    std::set<std::string> excludeFields{kUserDefinedTimeName.toString(), "bye"};
+    spec.setFieldSet(excludeFields);
+    spec.setBehavior(BucketSpec::Behavior::kExclude);
+
+    // This calls eraseUnneededComputedMetaProjFields().
+    unpacker.setBucketSpec(std::move(spec));
+    // As "hello" was not excluded, it should still exist.
+    ASSERT_TRUE(unpacker.bucketSpec().computedMetaProjFields().contains("hello"));
+    // As "bye" was in the excludes, it should be removed from '_computedMetaProjFields'.
+    ASSERT_FALSE(unpacker.bucketSpec().computedMetaProjFields().contains("bye"));
+}
+
 TEST_F(BucketUnpackerTest, DetermineIncludeTimeField) {
     auto bucket = fromjson(R"(
 {
@@ -677,21 +734,25 @@ TEST_F(BucketUnpackerTest, DetermineIncludeTimeField) {
 })");
     std::set<std::string> unpackerFields{kUserDefinedTimeName.toString()};
     auto unpacker = makeBucketUnpacker(unpackerFields,
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(bucket),
                                        kUserDefinedMetaName.toString());
 
     std::set<std::string> includeFields{kUserDefinedTimeName.toString()};
-    auto includeSpec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(includeFields)};
+    auto includeSpec = BucketSpec{kUserDefinedTimeName.toString(),
+                                  kUserDefinedMetaName.toString(),
+                                  std::move(includeFields),
+                                  BucketSpec::Behavior::kInclude};
     // This calls determineIncludeTimeField.
-    unpacker.setBucketSpecAndBehavior(std::move(includeSpec), BucketUnpacker::Behavior::kInclude);
+    unpacker.setBucketSpec(std::move(includeSpec));
     ASSERT_TRUE(unpacker.includeTimeField());
 
     std::set<std::string> excludeFields{kUserDefinedTimeName.toString()};
-    auto excludeSpec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(excludeFields)};
-    unpacker.setBucketSpecAndBehavior(std::move(excludeSpec), BucketUnpacker::Behavior::kExclude);
+    auto excludeSpec = BucketSpec{kUserDefinedTimeName.toString(),
+                                  kUserDefinedMetaName.toString(),
+                                  std::move(excludeFields),
+                                  BucketSpec::Behavior::kExclude};
+    unpacker.setBucketSpec(std::move(excludeSpec));
     ASSERT_FALSE(unpacker.includeTimeField());
 }
 
@@ -700,30 +761,33 @@ TEST_F(BucketUnpackerTest, DetermineIncludeFieldIncludeMode) {
     std::string excludedMeasurementField = "measurementField2";
     std::set<std::string> fields{kUserDefinedTimeName.toString(), includedMeasurementField};
 
-    auto bucket = Document{{"_id", 1},
-                           {"control", Document{{"version", 1}}},
-                           {"meta", Document{{"m1", 999}, {"m2", 9999}}},
-                           {"data", Document{}}}
-                      .toBson();
+    auto bucket = Document{
+        {"_id", 1},
+        {"control", Document{{"version", 1}}},
+        {"meta", Document{{"m1", 999}, {"m2", 9999}}},
+        {"data",
+         Document{}}}.toBson();
 
-    auto spec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(fields)};
+    auto spec = BucketSpec{kUserDefinedTimeName.toString(),
+                           kUserDefinedMetaName.toString(),
+                           std::move(fields),
+                           BucketSpec::Behavior::kInclude};
 
     BucketUnpacker includeUnpacker;
-    includeUnpacker.setBucketSpecAndBehavior(std::move(spec), BucketUnpacker::Behavior::kInclude);
+    includeUnpacker.setBucketSpec(std::move(spec));
     // Need to call reset so that the private method calculateFieldsToIncludeExcludeDuringUnpack()
     // is called, and _unpackFieldsToIncludeExclude gets filled with fields.
     includeUnpacker.reset(std::move(bucket));
     // Now the spec knows which fields to include/exclude.
 
     ASSERT_TRUE(determineIncludeField(kUserDefinedTimeName,
-                                      BucketUnpacker::Behavior::kInclude,
+                                      BucketSpec::Behavior::kInclude,
                                       includeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
     ASSERT_TRUE(determineIncludeField(includedMeasurementField,
-                                      BucketUnpacker::Behavior::kInclude,
+                                      BucketSpec::Behavior::kInclude,
                                       includeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
     ASSERT_FALSE(determineIncludeField(excludedMeasurementField,
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        includeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
 }
 
@@ -732,27 +796,30 @@ TEST_F(BucketUnpackerTest, DetermineIncludeFieldExcludeMode) {
     std::string excludedMeasurementField = "measurementField2";
     std::set<std::string> fields{kUserDefinedTimeName.toString(), includedMeasurementField};
 
-    auto bucket = Document{{"_id", 1},
-                           {"control", Document{{"version", 1}}},
-                           {"meta", Document{{"m1", 999}, {"m2", 9999}}},
-                           {"data", Document{}}}
-                      .toBson();
+    auto bucket = Document{
+        {"_id", 1},
+        {"control", Document{{"version", 1}}},
+        {"meta", Document{{"m1", 999}, {"m2", 9999}}},
+        {"data",
+         Document{}}}.toBson();
 
-    auto spec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(fields)};
+    auto spec = BucketSpec{kUserDefinedTimeName.toString(),
+                           kUserDefinedMetaName.toString(),
+                           std::move(fields),
+                           BucketSpec::Behavior::kExclude};
 
     BucketUnpacker excludeUnpacker;
-    excludeUnpacker.setBucketSpecAndBehavior(std::move(spec), BucketUnpacker::Behavior::kExclude);
+    excludeUnpacker.setBucketSpec(std::move(spec));
     excludeUnpacker.reset(std::move(bucket));
 
     ASSERT_FALSE(determineIncludeField(kUserDefinedTimeName,
-                                       BucketUnpacker::Behavior::kExclude,
+                                       BucketSpec::Behavior::kExclude,
                                        excludeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
     ASSERT_FALSE(determineIncludeField(includedMeasurementField,
-                                       BucketUnpacker::Behavior::kExclude,
+                                       BucketSpec::Behavior::kExclude,
                                        excludeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
     ASSERT_TRUE(determineIncludeField(excludedMeasurementField,
-                                      BucketUnpacker::Behavior::kExclude,
+                                      BucketSpec::Behavior::kExclude,
                                       excludeUnpacker.fieldsToIncludeExcludeDuringUnpack()));
 }
 
@@ -770,9 +837,11 @@ auto expectedTimestampObjSize(int32_t rowKeyOffset, int32_t n) {
 TEST_F(BucketUnpackerTest, ExtractSingleMeasurement) {
     std::set<std::string> fields{
         "_id", kUserDefinedMetaName.toString(), kUserDefinedTimeName.toString(), "a", "b"};
-    auto spec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(fields)};
-    auto unpacker = BucketUnpacker{std::move(spec), BucketUnpacker::Behavior::kInclude};
+    auto spec = BucketSpec{kUserDefinedTimeName.toString(),
+                           kUserDefinedMetaName.toString(),
+                           std::move(fields),
+                           BucketSpec::Behavior::kInclude};
+    auto unpacker = BucketUnpacker{std::move(spec)};
 
     auto d1 = dateFromISOString("2020-02-17T00:00:00.000Z").getValue();
     auto d2 = dateFromISOString("2020-02-17T01:00:00.000Z").getValue();
@@ -815,9 +884,11 @@ TEST_F(BucketUnpackerTest, ExtractSingleMeasurement) {
 TEST_F(BucketUnpackerTest, ExtractSingleMeasurementSparse) {
     std::set<std::string> fields{
         "_id", kUserDefinedMetaName.toString(), kUserDefinedTimeName.toString(), "a", "b"};
-    auto spec = BucketSpec{
-        kUserDefinedTimeName.toString(), kUserDefinedMetaName.toString(), std::move(fields)};
-    auto unpacker = BucketUnpacker{std::move(spec), BucketUnpacker::Behavior::kInclude};
+    auto spec = BucketSpec{kUserDefinedTimeName.toString(),
+                           kUserDefinedMetaName.toString(),
+                           std::move(fields),
+                           BucketSpec::Behavior::kInclude};
+    auto unpacker = BucketUnpacker{std::move(spec)};
 
     auto d1 = dateFromISOString("2020-02-17T00:00:00.000Z").getValue();
     auto d2 = dateFromISOString("2020-02-17T01:00:00.000Z").getValue();
@@ -899,13 +970,12 @@ TEST_F(BucketUnpackerTest, TamperedCompressedCountLess) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto compressedBucket =
-        timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-            .compressedBucket;
+        timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket;
     // Reduce the count by one to be 1.
     auto modifiedCompressedBucket = modifyCompressedBucketElementCount(*compressedBucket, -1);
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(modifiedCompressedBucket),
                                        kUserDefinedMetaName.toString());
 
@@ -935,13 +1005,12 @@ TEST_F(BucketUnpackerTest, TamperedCompressedCountMore) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto compressedBucket =
-        timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-            .compressedBucket;
+        timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket;
     // Increase the count by one to be 3.
     auto modifiedCompressedBucket = modifyCompressedBucketElementCount(*compressedBucket, 1);
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(modifiedCompressedBucket),
                                        kUserDefinedMetaName.toString());
 
@@ -971,13 +1040,12 @@ TEST_F(BucketUnpackerTest, TamperedCompressedCountMissing) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto compressedBucket =
-        timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-            .compressedBucket;
+        timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket;
     // Remove the count field
     auto modifiedCompressedBucket = modifyCompressedBucketElementCount(*compressedBucket, 0);
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(modifiedCompressedBucket),
                                        kUserDefinedMetaName.toString());
 
@@ -1008,14 +1076,13 @@ TEST_F(BucketUnpackerTest, TamperedCompressedElementMismatchDataField) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto compressedBucket =
-        timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-            .compressedBucket;
+        timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket;
     // Remove an element in the "a" field.
     auto modifiedCompressedBucket =
         modifyCompressedBucketRemoveLastInField(*compressedBucket, "a"_sd);
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(modifiedCompressedBucket),
                                        kUserDefinedMetaName.toString());
 
@@ -1044,14 +1111,13 @@ TEST_F(BucketUnpackerTest, TamperedCompressedElementMismatchTimeField) {
         "a:{'0':1, '1':2}, b:{'1':1}}}");
 
     auto compressedBucket =
-        timeseries::compressBucket(bucket, "time"_sd, {}, /*eligibleForReopening=*/false, false)
-            .compressedBucket;
+        timeseries::compressBucket(bucket, "time"_sd, {}, false).compressedBucket;
     // Remove an element in the time field
     auto modifiedCompressedBucket =
         modifyCompressedBucketRemoveLastInField(*compressedBucket, "time"_sd);
 
     auto unpacker = makeBucketUnpacker(std::move(fields),
-                                       BucketUnpacker::Behavior::kInclude,
+                                       BucketSpec::Behavior::kInclude,
                                        std::move(modifiedCompressedBucket),
                                        kUserDefinedMetaName.toString());
 
