@@ -88,8 +88,11 @@ const StringDataSet Document::allMetadataFieldNames{Document::metaFieldTextScore
                                                     Document::metaFieldGeoNearPoint,
                                                     Document::metaFieldSearchScore,
                                                     Document::metaFieldSearchHighlights,
+                                                    Document::metaFieldSearchSortValues,
                                                     Document::metaFieldIndexKey,
-                                                    Document::metaFieldSearchScoreDetails};
+                                                    Document::metaFieldSearchScoreDetails,
+                                                    Document::metaFieldVectorSearchScore,
+                                                    Document::metaFieldSearchSequenceToken};
 
 DocumentStorageIterator::DocumentStorageIterator(DocumentStorage* storage, BSONObjIterator bsonIt)
     : _bsonIt(std::move(bsonIt)),
@@ -130,7 +133,7 @@ bool DocumentStorageIterator::shouldSkipDeleted() {
 
         // If we strip the metadata see if a field name matches the known list. All metadata fields
         // start with '$' so optimize for a quick bailout.
-        if (_storage->stripMetadata() && fieldName[0] == '$' &&
+        if (_storage->bsonHasMetadata() && !fieldName.empty() && fieldName[0] == '$' &&
             Document::allMetadataFieldNames.contains(fieldName)) {
             return true;
         }
@@ -343,8 +346,8 @@ void DocumentStorage::reserveFields(size_t expectedFields) {
 }
 
 intrusive_ptr<DocumentStorage> DocumentStorage::clone() const {
-    auto out =
-        make_intrusive<DocumentStorage>(_bson, _stripMetadata, _modified, _numBytesFromBSONInCache);
+    auto out = make_intrusive<DocumentStorage>(
+        _bson, _bsonHasMetadata, _modified, _numBytesFromBSONInCache);
 
     if (_cache) {
         // Make a copy of the buffer with the fields.
@@ -390,10 +393,10 @@ DocumentStorage::~DocumentStorage() {
     }
 }
 
-void DocumentStorage::reset(const BSONObj& bson, bool stripMetadata) {
+void DocumentStorage::reset(const BSONObj& bson, bool bsonHasMetadata) {
     _bson = bson;
     _numBytesFromBSONInCache = 0;
-    _stripMetadata = stripMetadata;
+    _bsonHasMetadata = bsonHasMetadata;
     _modified = false;
     _snapshottedSize = 0;
 
@@ -411,9 +414,27 @@ void DocumentStorage::reset(const BSONObj& bson, bool stripMetadata) {
     _metadataFields = DocumentMetadataFields{};
 }
 
-void DocumentStorage::fillCache() const {
+Document DocumentStorage::shred() const {
+    MutableDocument md;
+    // Iterate raw bson if possible. This avoids caching all of the values in a doc that might get
+    // thrown away.
+    if (!isModified() && !bsonHasMetadata()) {
+        for (const auto& elem : _bson) {
+            md[elem.fieldNameStringData()] = Value(elem).shred();
+        }
+    } else {
+        for (DocumentStorageIterator it = iterator(); !it.atEnd(); it.advance()) {
+            const auto& valueElem = it.get();
+            md[it.fieldName()] = valueElem.val.shred();
+        }
+    }
+    md.setMetadata(DocumentMetadataFields(metadata()));
+    return md.freeze();
+}
+
+void DocumentStorage::loadIntoCache() const {
     for (DocumentStorageIterator it = iterator(); !it.atEnd(); it.advance()) {
-        it->val.fillCache();
+        it.get();
     }
 }
 
@@ -422,11 +443,13 @@ void DocumentStorage::loadLazyMetadata() const {
         return;
     }
 
+    bool oldModified = _metadataFields.isModified();
+
     BSONObjIterator it(_bson);
     while (it.more()) {
         BSONElement elem(it.next());
         auto fieldName = elem.fieldNameStringData();
-        if (fieldName[0] == '$') {
+        if (!fieldName.empty() && fieldName[0] == '$') {
             if (fieldName == Document::metaFieldTextScore) {
                 _metadataFields.setTextScore(elem.Double());
             } else if (fieldName == Document::metaFieldSearchScore) {
@@ -462,10 +485,17 @@ void DocumentStorage::loadLazyMetadata() const {
                 _metadataFields.setIndexKey(elem.Obj());
             } else if (fieldName == Document::metaFieldSearchScoreDetails) {
                 _metadataFields.setSearchScoreDetails(elem.Obj());
+            } else if (fieldName == Document::metaFieldSearchSortValues) {
+                _metadataFields.setSearchSortValues(elem.Obj());
+            } else if (fieldName == Document::metaFieldVectorSearchScore) {
+                _metadataFields.setVectorSearchScore(elem.Double());
+            } else if (fieldName == Document::metaFieldSearchSequenceToken) {
+                _metadataFields.setSearchSequenceToken(Value(elem));
             }
         }
     }
 
+    _metadataFields.setModified(oldModified);
     _haveLazyLoadedMetadata = true;
 }
 
@@ -515,22 +545,11 @@ void Document::toBson(BSONObjBuilder* builder, size_t recursionLevel) const {
     }
 }
 
-BSONObj Document::toBson() const {
-    if (!storage().isModified() && !storage().stripMetadata()) {
-        return storage().bsonObj();
-    }
-
-    BSONObjBuilder bb;
-    toBson(&bb);
-    return bb.obj();
-}
-
 boost::optional<BSONObj> Document::toBsonIfTriviallyConvertible() const {
-    if (!storage().isModified() && !storage().stripMetadata()) {
+    if (isTriviallyConvertible()) {
         return storage().bsonObj();
-    } else {
-        return boost::none;
     }
+    return boost::none;
 }
 
 constexpr StringData Document::metaFieldTextScore;
@@ -541,35 +560,44 @@ constexpr StringData Document::metaFieldGeoNearPoint;
 constexpr StringData Document::metaFieldSearchScore;
 constexpr StringData Document::metaFieldSearchHighlights;
 constexpr StringData Document::metaFieldSearchScoreDetails;
+constexpr StringData Document::metaFieldSearchSortValues;
+constexpr StringData Document::metaFieldVectorSearchScore;
 
-BSONObj Document::toBsonWithMetaData() const {
-    BSONObjBuilder bb;
-    toBson(&bb);
+void Document::toBsonWithMetaData(BSONObjBuilder* builder) const {
+    toBson(builder);
     if (!metadata()) {
-        return bb.obj();
+        return;
     }
 
     if (metadata().hasTextScore())
-        bb.append(metaFieldTextScore, metadata().getTextScore());
+        builder->append(metaFieldTextScore, metadata().getTextScore());
     if (metadata().hasRandVal())
-        bb.append(metaFieldRandVal, metadata().getRandVal());
+        builder->append(metaFieldRandVal, metadata().getRandVal());
     if (metadata().hasSortKey())
-        bb.append(metaFieldSortKey,
-                  DocumentMetadataFields::serializeSortKey(metadata().isSingleElementKey(),
-                                                           metadata().getSortKey()));
+        builder->append(metaFieldSortKey,
+                        DocumentMetadataFields::serializeSortKey(metadata().isSingleElementKey(),
+                                                                 metadata().getSortKey()));
     if (metadata().hasGeoNearDistance())
-        bb.append(metaFieldGeoNearDistance, metadata().getGeoNearDistance());
+        builder->append(metaFieldGeoNearDistance, metadata().getGeoNearDistance());
     if (metadata().hasGeoNearPoint())
-        metadata().getGeoNearPoint().addToBsonObj(&bb, metaFieldGeoNearPoint);
+        metadata().getGeoNearPoint().addToBsonObj(builder, metaFieldGeoNearPoint);
     if (metadata().hasSearchScore())
-        bb.append(metaFieldSearchScore, metadata().getSearchScore());
+        builder->append(metaFieldSearchScore, metadata().getSearchScore());
     if (metadata().hasSearchHighlights())
-        metadata().getSearchHighlights().addToBsonObj(&bb, metaFieldSearchHighlights);
+        metadata().getSearchHighlights().addToBsonObj(builder, metaFieldSearchHighlights);
     if (metadata().hasIndexKey())
-        bb.append(metaFieldIndexKey, metadata().getIndexKey());
+        builder->append(metaFieldIndexKey, metadata().getIndexKey());
     if (metadata().hasSearchScoreDetails())
-        bb.append(metaFieldSearchScoreDetails, metadata().getSearchScoreDetails());
-    return bb.obj();
+        builder->append(metaFieldSearchScoreDetails, metadata().getSearchScoreDetails());
+    if (metadata().hasSearchSortValues()) {
+        builder->append(metaFieldSearchSortValues, metadata().getSearchSortValues());
+    }
+    if (metadata().hasSearchSequenceToken()) {
+        metadata().getSearchSequenceToken().addToBsonObj(builder, metaFieldSearchSequenceToken);
+    }
+    if (metadata().hasVectorSearchScore()) {
+        builder->append(metaFieldVectorSearchScore, metadata().getVectorSearchScore());
+    }
 }
 
 Document Document::fromBsonWithMetaData(const BSONObj& bson) {
