@@ -31,6 +31,7 @@
 
 #include "mongo/db/exec/document_value/value.h"
 
+#include <absl/hash/hash.h>
 #include <boost/functional/hash.hpp>
 #include <cmath>
 #include <limits>
@@ -74,7 +75,7 @@ void ValueStorage::verifyRefCountingIfShould() const {
         case NumberLong:
         case NumberDouble:
             // the above types never reference external data
-            verify(!refCounter);
+            MONGO_verify(!refCounter);
             break;
 
         case String:
@@ -285,6 +286,27 @@ Value::Value(const vector<Document>& vec) : _storage(Array) {
     _storage.putVector(std::move(storageVec));
 }
 
+Value::Value(const SafeNum& value) : _storage(value.type()) {
+    switch (value.type()) {
+        case EOO:
+            break;
+        case NumberInt:
+            _storage.intValue = value._value.int32Val;
+            break;
+        case NumberLong:
+            _storage.longValue = value._value.int64Val;
+            break;
+        case NumberDouble:
+            _storage.doubleValue = value._value.doubleVal;
+            break;
+        case NumberDecimal:
+            _storage.putDecimal(Decimal128(value._value.decimalVal));
+            break;
+        default:
+            MONGO_UNREACHABLE;
+    }
+}
+
 Value Value::createIntOrLong(long long longValue) {
     int intValue = longValue;
     if (intValue != longValue) {
@@ -317,12 +339,12 @@ double Value::getDouble() const {
     if (type == NumberDecimal)
         return _storage.getDecimal().toDouble();
 
-    verify(type == NumberDouble);
+    MONGO_verify(type == NumberDouble);
     return _storage.doubleValue;
 }
 
 Document Value::getDocument() const {
-    verify(getType() == Object);
+    MONGO_verify(getType() == Object);
     return _storage.getDocument();
 }
 
@@ -400,7 +422,7 @@ BSONObjBuilder& operator<<(BSONObjBuilderValueStream& builder, const Value& val)
             return builder.builder();
         }
     }
-    verify(false);
+    MONGO_verify(false);
 }
 
 void Value::addToBsonObj(BSONObjBuilder* builder,
@@ -487,7 +509,7 @@ bool Value::coerceToBool() const {
         case NumberDecimal:
             return !_storage.getDecimal().isZero();
     }
-    verify(false);
+    MONGO_verify(false);
 }
 
 namespace {
@@ -862,9 +884,20 @@ int Value::compare(const Value& rL,
             return l->scope.woCompare(r->scope);
         }
     }
-    verify(false);
+    MONGO_verify(false);
 }
 
+namespace {
+/**
+ * Hashes the given 'StringData', combines the resulting hash with 'seed', and returns the result.
+ */
+size_t hashStringData(StringData sd, size_t seed) {
+    size_t strHash = absl::Hash<absl::string_view>{}(absl::string_view(sd.rawData(), sd.size()));
+    boost::hash_combine(seed, strHash);
+    return seed;
+}
+}  // namespace
+
 void Value::hash_combine(size_t& seed,
                          const StringData::ComparatorInterface* stringComparator) const {
     BSONType type = getType();
@@ -936,7 +969,7 @@ void Value::hash_combine(size_t& seed,
         case Code:
         case Symbol: {
             StringData sd = getRawData();
-            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
+            seed = hashStringData(sd, seed);
             break;
         }
 
@@ -945,7 +978,7 @@ void Value::hash_combine(size_t& seed,
             if (stringComparator) {
                 stringComparator->hash_combine(seed, sd);
             } else {
-                MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
+                seed = hashStringData(sd, seed);
             }
             break;
         }
@@ -969,14 +1002,14 @@ void Value::hash_combine(size_t& seed,
 
         case BinData: {
             StringData sd = getRawData();
-            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
+            seed = hashStringData(sd, seed);
             boost::hash_combine(seed, _storage.binDataType());
             break;
         }
 
         case RegEx: {
             StringData sd = getRawData();
-            MurmurHash3_x86_32(sd.rawData(), sd.size(), seed, &seed);
+            seed = hashStringData(sd, seed);
             break;
         }
 
@@ -1170,7 +1203,7 @@ size_t Value::getApproximateSize() const {
         case Undefined:
             return sizeof(Value);
     }
-    verify(false);
+    MONGO_verify(false);
 }
 
 string Value::toString() const {
@@ -1251,17 +1284,20 @@ ostream& operator<<(ostream& out, const Value& val) {
     }
 
     // Not in default case to trigger better warning if a case is missing
-    verify(false);
+    MONGO_verify(false);
 }
 
-void Value::fillCache() const {
+Value Value::shred() const {
     if (isObject()) {
-        getDocument().fillCache();
+        return Value(getDocument().shred());
     } else if (isArray()) {
+        std::vector<Value> values;
         for (auto&& val : getArray()) {
-            val.fillCache();
+            values.push_back(val.shred());
         }
+        return Value(values);
     }
+    return Value(*this);
 }
 
 void Value::serializeForSorter(BufBuilder& buf) const {
@@ -1432,7 +1468,7 @@ Value Value::deserializeForSorter(BufReader& buf, const SorterDeserializeSetting
             return Value(std::move(array));
         }
     }
-    verify(false);
+    MONGO_verify(false);
 }
 
 void Value::serializeForIDL(StringData fieldName, BSONObjBuilder* builder) const {
