@@ -38,6 +38,7 @@
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
+#include "mongo/bson/unordered_fields_bsonobj_comparator.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/document_value/value.h"
@@ -55,11 +56,12 @@ using std::vector;
 
 auto createProjectionExecutor(const BSONObj& spec, const ProjectionPolicies& policies) {
     const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    auto projection = projection_ast::parse(expCtx, spec, policies);
+    auto projection = projection_ast::parseAndAnalyze(expCtx, spec, policies);
     auto builderParams = BuilderParamsBitSet{kDefaultBuilderParams};
-    builderParams.reset(kAllowFastPath);
     auto executor = buildProjectionExecutor(expCtx, &projection, policies, builderParams);
-    invariant(executor->getType() == TransformerInterface::TransformerType::kExclusionProjection);
+    tassert(7241742,
+            "projection executor must be exclusion projections",
+            executor->getType() == TransformerInterface::TransformerType::kExclusionProjection);
     return executor;
 }
 
@@ -106,6 +108,43 @@ TEST(ExclusionProjectionExecutionTest, ShouldSerializeToEquivalentProjection) {
     ASSERT_VALUE_EQ(serialization["x"].getDocument()["y"], Value(false));
 }
 
+TEST(ExclusionProjectionExecutionTest, ShouldSerializeWithTopLevelID) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(BSON("a" << 0 << "b" << 0));
+    auto serialization = exclusion->serializeTransformation(boost::none);
+    ASSERT_VALUE_EQ(serialization["a"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"], Value(false));
+    ASSERT_VALUE_EQ(serialization["_id"], Value(true));
+
+    exclusion = makeExclusionProjectionWithDefaultPolicies(
+        BSON("a" << 0 << "b" << BSON("c" << 0 << "d" << 0)));
+    serialization = exclusion->serializeTransformation(boost::none);
+    ASSERT_VALUE_EQ(serialization["a"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"]["c"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"]["d"], Value(false));
+    ASSERT_VALUE_EQ(serialization["_id"], Value(true));
+    ASSERT_VALUE_EQ(serialization["b"]["_id"], Value());
+
+    exclusion = makeExclusionProjectionWithDefaultIdExclusion(BSON("a" << false << "b" << false));
+    serialization = exclusion->serializeTransformation(boost::none);
+    ASSERT_VALUE_EQ(serialization["a"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"], Value(false));
+    ASSERT_VALUE_EQ(serialization["_id"], Value(false));
+
+    exclusion = makeExclusionProjectionWithDefaultIdExclusion(
+        BSON("a" << false << "b" << false << "_id" << false));
+    serialization = exclusion->serializeTransformation(boost::none);
+    ASSERT_VALUE_EQ(serialization["a"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"], Value(false));
+    ASSERT_VALUE_EQ(serialization["_id"], Value(false));
+
+    exclusion = makeExclusionProjectionWithDefaultIdExclusion(
+        BSON("a" << false << "b" << false << "_id" << true));
+    serialization = exclusion->serializeTransformation(boost::none);
+    ASSERT_VALUE_EQ(serialization["a"], Value(false));
+    ASSERT_VALUE_EQ(serialization["b"], Value(false));
+    ASSERT_VALUE_EQ(serialization["_id"], Value(true));
+}
+
 TEST(ExclusionProjectionExecutionTest, ShouldNotAddAnyDependencies) {
     // An exclusion projection will cause the stage to return DepsTracker::State::SEE_NEXT, meaning
     // it doesn't strictly require any fields.
@@ -272,22 +311,20 @@ TEST(ExclusionProjectionExecutionTest, ShouldNotCreateSubDocIfDottedExcludedFiel
 TEST(ExclusionProjectionExecutionTest, ShouldApplyDottedExclusionToEachElementInArray) {
     auto exclusion = makeExclusionProjectionWithDefaultPolicies(BSON("a.b" << false));
 
-    std::vector<Value> nestedValues = {
-        Value(1),
-        Value(Document{}),
-        Value(Document{{"b", 1}}),
-        Value(Document{{"b", 1}, {"c", 2}}),
-        Value(vector<Value>{}),
-        Value(vector<Value>{Value(1), Value(Document{{"c", 1}, {"b", 1}})})};
-    std::vector<Value> expectedNestedValues = {
-        Value(1),
-        Value(Document{}),
-        Value(Document{}),
-        Value(Document{{"c", 2}}),
-        Value(vector<Value>{}),
-        Value(vector<Value>{Value(1), Value(Document{{"c", 1}})})};
-    auto result = exclusion->applyTransformation(Document{{"a", nestedValues}});
-    auto expectedResult = Document{{"a", expectedNestedValues}};
+    auto result = exclusion->applyTransformation(Document{{"a",
+                                                           {1,
+                                                            Document{},
+                                                            Document{{"b", 1}},
+                                                            Document{{"b", 1}, {"c", 2}},
+                                                            vector<Value>{},
+                                                            {1, Document{{"c", 1}, {"b", 1}}}}}});
+    auto expectedResult = Document{{"a",
+                                    {1,
+                                     Document{},
+                                     Document{},
+                                     Document{{"c", 2}},
+                                     vector<Value>{},
+                                     {1, Document{{"c", 1}}}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
@@ -327,7 +364,8 @@ TEST(ExclusionProjectionExecutionTest, ShouldEvaluateMetaExpressions) {
                                                             "i: {$meta: 'recordId'}, "
                                                             "j: {$meta: 'indexKey'}, "
                                                             "k: {$meta: 'sortKey'}, "
-                                                            "l: {$meta: 'searchScoreDetails'}}"));
+                                                            "l: {$meta: 'searchScoreDetails'}, "
+                                                            "m: {$meta: 'vectorSearchScore'}}"));
 
     MutableDocument inputDocBuilder(Document{{"a", 1}, {"b", 2}});
     inputDocBuilder.metadata().setTextScore(0.0);
@@ -341,6 +379,7 @@ TEST(ExclusionProjectionExecutionTest, ShouldEvaluateMetaExpressions) {
     inputDocBuilder.metadata().setSortKey(Value{Document{{"bar", 8}}}, true);
     inputDocBuilder.metadata().setSearchScoreDetails(BSON("scoreDetails"
                                                           << "foo"));
+    inputDocBuilder.metadata().setVectorSearchScore(9.0);
     Document inputDoc = inputDocBuilder.freeze();
 
     auto result = exclusion->applyTransformation(inputDoc);
@@ -348,7 +387,7 @@ TEST(ExclusionProjectionExecutionTest, ShouldEvaluateMetaExpressions) {
     ASSERT_DOCUMENT_EQ(result,
                        Document{fromjson("{b: 2, c: 0.0, d: 1.0, e: 2.0, f: 'foo', g: 3.0, "
                                          "h: [4, 5], i: 6, j: {foo: 7}, k: [{bar: 8}],"
-                                         "l: {scoreDetails: 'foo'}}")});
+                                         "l: {scoreDetails: 'foo'}, m: 9.0}")});
 }
 
 TEST(ExclusionProjectionExecutionTest, ShouldAddMetaExpressionsToDependencies) {
@@ -362,18 +401,20 @@ TEST(ExclusionProjectionExecutionTest, ShouldAddMetaExpressionsToDependencies) {
                                                             "i: {$meta: 'recordId'}, "
                                                             "j: {$meta: 'indexKey'}, "
                                                             "k: {$meta: 'sortKey'}, "
-                                                            "l: {$meta: 'searchScoreDetails'}}"));
+                                                            "l: {$meta: 'searchScoreDetails'}}, "
+                                                            "m: {$meta: 'vectorSearchScore'}"));
 
     DepsTracker deps;
     exclusion->addDependencies(&deps);
 
     ASSERT_EQ(deps.fields.size(), 0UL);
 
-    // We do not add the dependencies for searchScore, searchHighlights, or searchScoreDetails
-    // because those values are not stored in the collection (or in mongod at all).
+    // We do not add the dependencies for searchScore, searchHighlights, searchScoreDetails, or
+    // distance because those values are not stored in the collection (or in mongod at all).
     ASSERT_FALSE(deps.metadataDeps()[DocumentMetadataFields::kSearchScore]);
     ASSERT_FALSE(deps.metadataDeps()[DocumentMetadataFields::kSearchHighlights]);
     ASSERT_FALSE(deps.metadataDeps()[DocumentMetadataFields::kSearchScoreDetails]);
+    ASSERT_FALSE(deps.metadataDeps()[DocumentMetadataFields::kVectorSearchScore]);
 
     ASSERT_TRUE(deps.metadataDeps()[DocumentMetadataFields::kTextScore]);
     ASSERT_TRUE(deps.metadataDeps()[DocumentMetadataFields::kRandVal]);
@@ -467,18 +508,14 @@ TEST(ExclusionProjectionExecutionTest, ShouldRecurseNestedArraysByDefault) {
     auto exclusion = makeExclusionProjectionWithDefaultPolicies(BSON("a.b" << false));
 
     // {a: [1, {b: 2, c: 3}, [{b: 4, c: 5}], {d: 6}]} => {a: [1, {c: 3}, [{c: 5}], {d: 6}]}
-    auto result = exclusion->applyTransformation(
-        Document{{"a",
-                  vector<Value>{Value(1),
-                                Value(Document{{"b", 2}, {"c", 3}}),
-                                Value(vector<Value>{Value(Document{{"b", 4}, {"c", 5}})}),
-                                Value(Document{{"d", 6}})}}});
+    auto result = exclusion->applyTransformation(Document{{"a",
+                                                           {1,
+                                                            Document{{"b", 2}, {"c", 3}},
+                                                            vector{Document{{"b", 4}, {"c", 5}}},
+                                                            Document{{"d", 6}}}}});
 
-    auto expectedResult = Document{{"a",
-                                    vector<Value>{Value(1),
-                                                  Value(Document{{"c", 3}}),
-                                                  Value(vector<Value>{Value(Document{{"c", 5}})}),
-                                                  Value(Document{{"d", 6}})}}};
+    auto expectedResult =
+        Document{{"a", {1, Document{{"c", 3}}, vector{Document{{"c", 5}}}, Document{{"d", 6}}}}};
 
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
@@ -488,19 +525,14 @@ TEST(ExclusionProjectionExecutionTest, ShouldNotRecurseNestedArraysForNoRecurseP
 
     // {a: [1, {b: 2, c: 3}, [{b: 4, c: 5}], {d: 6}]} => {a: [1, {c: 3}, [{b: 4, c: 5}], {d:
     // 6}]}
-    auto result = exclusion->applyTransformation(
-        Document{{"a",
-                  vector<Value>{Value(1),
-                                Value(Document{{"b", 2}, {"c", 3}}),
-                                Value(vector<Value>{Value(Document{{"b", 4}, {"c", 5}})}),
-                                Value(Document{{"d", 6}})}}});
+    auto result = exclusion->applyTransformation(Document{{"a",
+                                                           {1,
+                                                            Document{{"b", 2}, {"c", 3}},
+                                                            vector{Document{{"b", 4}, {"c", 5}}},
+                                                            Document{{"d", 6}}}}});
 
-    auto expectedResult =
-        Document{{"a",
-                  vector<Value>{Value(1),
-                                Value(Document{{"c", 3}}),
-                                Value(vector<Value>{Value(Document{{"b", 4}, {"c", 5}})}),
-                                Value(Document{{"d", 6}})}}};
+    auto expectedResult = Document{
+        {"a", {1, Document{{"c", 3}}, vector{Document{{"b", 4}, {"c", 5}}}, Document{{"d", 6}}}}};
 
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
@@ -509,17 +541,84 @@ TEST(ExclusionProjectionExecutionTest, ShouldNotRetainNestedArraysIfNoRecursionN
     auto exclusion = makeExclusionProjectionWithNoArrayRecursion(BSON("a" << false));
 
     // {a: [1, {b: 2, c: 3}, [{b: 4, c: 5}], {d: 6}]} => {}
-    const auto inputDoc =
-        Document{{"a",
-                  vector<Value>{Value(1),
-                                Value(Document{{"b", 2}, {"c", 3}}),
-                                Value(vector<Value>{Value(Document{{"b", 4}, {"c", 5}})}),
-                                Value(Document{{"d", 6}})}}};
+    const auto inputDoc = Document{{"a",
+                                    {1,
+                                     Document{{"b", 2}, {"c", 3}},
+                                     vector{Document{{"b", 4}, {"c", 5}}},
+                                     Document{{"d", 6}}}}};
 
     auto result = exclusion->applyTransformation(inputDoc);
     const auto expectedResult = Document{};
 
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
+
+//
+// extractProjectOnFieldAndRename() tests.
+//
+TEST(ExclusionProjectionExecutionTest, ShouldExtractEntireProjectOnRootField) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(fromjson("{a: 0}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("a", "b");
+
+    ASSERT_TRUE(allExtracted);
+    ASSERT_BSONOBJ_EQ(fromjson("{b: false}"), extractedProj);
+}
+
+TEST(ExclusionProjectionExecutionTest, ShouldExtractEntireProjectOnMultipleSubfields) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(fromjson("{'a.c': 0, 'a.d': 0}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("a", "b");
+
+    ASSERT_TRUE(allExtracted);
+    const UnorderedFieldsBSONObjComparator kComparator;
+    ASSERT_EQ(kComparator.compare(fromjson("{b: {c: false, d: false}}"), extractedProj), 0);
+}
+
+TEST(ExclusionProjectionExecutionTest, ShouldExtractPartOfProjectOnRootField) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(fromjson("{a: 0, c: 0}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("a", "b");
+
+    ASSERT_FALSE(allExtracted);
+    ASSERT_BSONOBJ_EQ(fromjson("{b: false}"), extractedProj);
+    ASSERT_BSONOBJ_EQ(fromjson("{c: false, _id: true}"),
+                      exclusion->serializeTransformation(boost::none).toBson());
+}
+
+TEST(ExclusionProjectionExecutionTest, ShouldExtractPartOfProjectOnSubfields) {
+    auto exclusion =
+        makeExclusionProjectionWithDefaultPolicies(fromjson("{'x.y': 0, 'x.z': 0, 'c.d': 0}}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("x", "w");
+
+    ASSERT_FALSE(allExtracted);
+    const UnorderedFieldsBSONObjComparator kComparator;
+    ASSERT_EQ(kComparator.compare(fromjson("{w: {y: false, z: false}}"), extractedProj), 0);
+    ASSERT_BSONOBJ_EQ(fromjson("{c: {d: false}, _id: true}"),
+                      exclusion->serializeTransformation(boost::none).toBson());
+}
+
+TEST(ExclusionProjectionExecutionTest, ShouldCorrectlyLeaveRemainderWithExcludedId) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(fromjson("{a: 0, _id: 0}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("a", "b");
+
+    ASSERT_FALSE(allExtracted);
+    ASSERT_BSONOBJ_EQ(fromjson("{b: false}"), extractedProj);
+    ASSERT_BSONOBJ_EQ(fromjson("{_id: false}"),
+                      exclusion->serializeTransformation(boost::none).toBson());
+}
+
+TEST(ExclusionProjectionExecutionTest, ShouldNotExtractWhenFieldIsNotPresent) {
+    auto exclusion = makeExclusionProjectionWithDefaultPolicies(fromjson("{a: {b: 0}}"));
+
+    auto [extractedProj, allExtracted] = exclusion->extractProjectOnFieldAndRename("b", "c");
+
+    ASSERT_FALSE(allExtracted);
+    ASSERT_BSONOBJ_EQ(fromjson("{}"), extractedProj);
+    ASSERT_BSONOBJ_EQ(fromjson("{a: {b: false}, _id: true}"),
+                      exclusion->serializeTransformation(boost::none).toBson());
+}
 }  // namespace
 }  // namespace mongo::projection_executor
