@@ -33,255 +33,16 @@
 #include <string>
 #include <vector>
 
+#include "mongo/db/exec/sbe/abt/named_slots.h"
 #include "mongo/db/exec/sbe/util/debug_print.h"
 #include "mongo/db/exec/sbe/values/slot.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
-#include "mongo/stdx/unordered_map.h"
-#include "mongo/util/string_map.h"
 
 namespace mongo {
 namespace sbe {
-using SpoolBuffer = std::vector<value::MaterializedRow>;
 
-/**
- * A holder for slots and accessors which are used in a PlanStage tree but:
- *  - Cannot be made constants due to restrictions on the lifetime of such values (e.g., they're
- *    singleton instances owned somewhere else).
- *  - Can be changed in runtime outside of the PlanStage tree (e.g., a resume recordId changed by a
- *    PlanExecutor).
- *
- * A RuntimeEnvironment object is created once per an execution thread. That means that each
- * producer and consumer in a parallel plan will have their own compilation environment, with their
- * own slot accessors. However, slot accessors in each of such environment will access shared data,
- * which is the same across all environments.
- *
- * To avoid data races, the values stored in the runtime environment are considered read-only when
- * used with a parallel plan. An attempt to change any slot with 'resetValue' will result in a user
- * exception.
- *
- * If the runtime environment is used in a serial plan, modifications of the slots is allowed.
- */
-class RuntimeEnvironment {
-public:
-    RuntimeEnvironment() = default;
-    RuntimeEnvironment(RuntimeEnvironment&&) = delete;
-    RuntimeEnvironment& operator=(const RuntimeEnvironment&) = delete;
-    RuntimeEnvironment& operator=(const RuntimeEnvironment&&) = delete;
-    ~RuntimeEnvironment();
-
-    class Accessor final : public value::SlotAccessor {
-    public:
-        Accessor(RuntimeEnvironment* env, size_t index) : _env{env}, _index{index} {}
-
-        std::pair<value::TypeTags, value::Value> getViewOfValue() const override {
-            return {_env->_state->typeTags[_index], _env->_state->vals[_index]};
-        }
-
-        std::pair<value::TypeTags, value::Value> copyOrMoveValue() override {
-            // Always make a copy.
-            return copyValue(_env->_state->typeTags[_index], _env->_state->vals[_index]);
-        }
-
-        std::pair<value::TypeTags, value::Value> copyOrMoveValue() const {
-            // Always make a copy.
-            return copyValue(_env->_state->typeTags[_index], _env->_state->vals[_index]);
-        }
-
-        void reset(bool owned, value::TypeTags tag, value::Value val) {
-            release();
-
-            _env->_state->typeTags[_index] = tag;
-            _env->_state->vals[_index] = val;
-            _env->_state->owned[_index] = owned;
-        }
-
-    private:
-        void release() {
-            if (_env->_state->owned[_index]) {
-                releaseValue(_env->_state->typeTags[_index], _env->_state->vals[_index]);
-                _env->_state->owned[_index] = false;
-            }
-        }
-
-        RuntimeEnvironment* const _env;
-        const size_t _index;
-    };
-
-    /**
-     * Registers and returns a SlotId for the given slot 'name'. The 'slotIdGenerator' is used
-     * to generate a new SlotId for the given slot 'name', which is then registered with this
-     * environment by creating a new SlotAccessor. The value 'val' is then stored within the
-     * SlotAccessor and the newly generated SlotId is returned.
-     *
-     * Both owned and unowned values can be stored in the runtime environment.
-     *
-     * A user exception is raised if this slot 'name' has been already registered.
-     */
-    value::SlotId registerSlot(StringData name,
-                               value::TypeTags tag,
-                               value::Value val,
-                               bool owned,
-                               value::SlotIdGenerator* slotIdGenerator);
-
-    /**
-     * Same as above, but allows to register an unnamed slot.
-     */
-    value::SlotId registerSlot(value::TypeTags tag,
-                               value::Value val,
-                               bool owned,
-                               value::SlotIdGenerator* slotIdGenerator);
-
-    /**
-     * Returns a SlotId registered for the given slot 'name'. If the slot with the specified name
-     * hasn't been registered, a user exception is raised.
-     */
-    value::SlotId getSlot(StringData name);
-
-    /**
-     * Returns a SlotId registered for the given slot 'name'. If the slot with the specified name
-     * hasn't been registered, boost::none is returned.
-     */
-    boost::optional<value::SlotId> getSlotIfExists(StringData name);
-
-    /**
-     * Store the given value in the specified slot within this runtime environment instance.
-     *
-     * A user exception is raised if the SlotId is not registered within this environment, or
-     * if this environment is used with a parallel plan.
-     */
-    void resetSlot(value::SlotId slot, value::TypeTags tag, value::Value val, bool owned);
-
-    /**
-     * Returns a SlotAccessor for the given SlotId which must be previously registered within this
-     * Environment by invoking 'registerSlot' method.
-     *
-     * A user exception is raised if the SlotId is not registered within this environment.
-     */
-    Accessor* getAccessor(value::SlotId slot);
-
-    /**
-     * Make a copy of this environment. The new environment will have its own set of SlotAccessors
-     * pointing to the same shared data holding slot values.
-     *
-     * To create a copy of the runtime environment for a parallel execution plan, please use
-     * makeCopyForParallelUse() method. This will result in this environment being converted to a
-     * parallel environment, as well as the newly created copy.
-     */
-    std::unique_ptr<RuntimeEnvironment> makeCopyForParallelUse();
-    std::unique_ptr<RuntimeEnvironment> makeCopy() const;
-
-    /**
-     * Make a "deep" copy of this environment. The new environment will have its own set of
-     * SlotAccessors pointing to data copied from this RuntimeEnvironment. All the slot values are
-     * made owned by the new environment as much as possible. There could be some uncopyable types
-     * which can not be owned by the new environment, e.g. TimeZoneDatabase.
-     */
-    std::unique_ptr<RuntimeEnvironment> makeDeepCopy() const;
-
-    /**
-     * Dumps all the slots currently defined in this environment into the given string builder.
-     */
-    void debugString(StringBuilder* builder);
-
-private:
-    RuntimeEnvironment(const RuntimeEnvironment&);
-
-    struct State {
-        size_t pushSlot(value::SlotId slot) {
-            auto index = vals.size();
-
-            typeTags.push_back(value::TypeTags::Nothing);
-            vals.push_back(0);
-            owned.push_back(false);
-
-            auto [_, inserted] = slots.emplace(slot, index);
-            uassert(4946302, str::stream() << "duplicate environment slot: " << slot, inserted);
-            return index;
-        }
-
-        void nameSlot(StringData name, value::SlotId slot) {
-            uassert(5645901, str::stream() << "undefined slot: " << slot, slots.count(slot));
-            auto [_, inserted] = namedSlots.emplace(name, slot);
-            uassert(5645902, str::stream() << "duplicate named slot: " << name, inserted);
-        }
-
-        std::unique_ptr<State> makeCopyWithoutValues() {
-            auto state = std::make_unique<State>();
-            state->namedSlots = namedSlots;
-            state->slots = slots;
-
-            // Populate slot values with default value.
-            state->typeTags.resize(typeTags.size(), value::TypeTags::Nothing);
-            state->vals.resize(vals.size(), 0);
-            state->owned.resize(owned.size(), false);
-
-            return state;
-        }
-
-        StringMap<value::SlotId> namedSlots;
-        value::SlotMap<size_t> slots;
-
-        std::vector<value::TypeTags> typeTags;
-        std::vector<value::Value> vals;
-        std::vector<bool> owned;
-    };
-
-    void emplaceAccessor(value::SlotId slot, size_t index) {
-        _accessors.emplace(slot, Accessor{this, index});
-    }
-
-    std::shared_ptr<State> _state{std::make_shared<State>()};
-    value::SlotMap<Accessor> _accessors;
-    bool _isSmp{false};
-
-    friend class Accessor;
-};
-
-class PlanStage;
-struct CompileCtx {
-    CompileCtx(std::unique_ptr<RuntimeEnvironment> env) : env{std::move(env)} {}
-
-    value::SlotAccessor* getAccessor(value::SlotId slot);
-
-    RuntimeEnvironment::Accessor* getRuntimeEnvAccessor(value::SlotId slotId) {
-        return env->getAccessor(slotId);
-    }
-
-    std::shared_ptr<SpoolBuffer> getSpoolBuffer(SpoolId spool);
-
-    void pushCorrelated(value::SlotId slot, value::SlotAccessor* accessor);
-    void popCorrelated();
-
-    /**
-     * Make a copy of this CompileCtx. The underlying RuntimeEnvironment will also be copied.
-     *
-     * To create a copy of the underlying runtime environment for a parallel execution plan, please
-     * use makeCopyForParallelUse() method. This will result in the environment in this CompileCtx
-     * being converted to a parallel environment, as well as the newly created copy.
-     */
-    CompileCtx makeCopyForParallelUse();
-    CompileCtx makeCopy() const;
-
-    /**
-     * Root plan stage is used to resolve slot accessors introduced by PlanStage (optional).
-     * - if specified, the root plan stage will be used to resolve the slot accessor.
-     * - otherwise, if null, default context accessor resolution rules will be used.
-     */
-    PlanStage* root{nullptr};
-
-    value::SlotAccessor* accumulator{nullptr};
-    std::vector<std::pair<value::SlotId, value::SlotAccessor*>> correlated;
-    stdx::unordered_map<SpoolId, std::shared_ptr<SpoolBuffer>> spoolBuffers;
-    bool aggExpression{false};
-
-private:
-    // Any data that a PlanStage needs from the RuntimeEnvironment should not be accessed directly
-    // but insteady by looking up the corresponding slots. These slots are set up during the process
-    // of building PlanStages, so the PlanStages themselves should never need to add new slots to
-    // the RuntimeEnvironment.
-    std::unique_ptr<RuntimeEnvironment> env;
-};
+struct CompileCtx;
 
 /**
  * This is an abstract base class of all expression types in SBE. The expression types derived form
@@ -310,9 +71,7 @@ public:
     /**
      * Returns bytecode directly executable by VM.
      */
-    std::unique_ptr<vm::CodeFragment> compile(CompileCtx& ctx) const {
-        return std::make_unique<vm::CodeFragment>(compileDirect(ctx));
-    }
+    std::unique_ptr<vm::CodeFragment> compile(CompileCtx& ctx) const;
 
     virtual vm::CodeFragment compileDirect(CompileCtx& ctx) const = 0;
 
@@ -331,6 +90,17 @@ public:
         return dynamic_cast<T*>(this);
     }
 
+    /**
+     * Utility for casting to derived types.
+     */
+    template <typename T>
+    const T* as() const {
+        return dynamic_cast<const T*>(this);
+    }
+
+    // For printing from an interactive debugger.
+    std::string toString() const;
+
 protected:
     Vector _nodes;
 
@@ -343,11 +113,20 @@ protected:
         }
     }
 
-private:
-    // For printing from an interactive debugger.
-    std::string toString() const;
+    template <typename P>
+    void collectDescendants(P&& expandPredicate, std::vector<const EExpression*>* acc) const {
+        if (expandPredicate(this)) {
+            for (auto& child : _nodes) {
+                child->collectDescendants(expandPredicate, acc);
+            }
+        } else {
+            acc->push_back(this);
+        }
+    }
 };
 
+using SlotExprPairVector = std::vector<std::pair<value::SlotId, std::unique_ptr<EExpression>>>;
+
 template <typename T, typename... Args>
 inline std::unique_ptr<EExpression> makeE(Args&&... args) {
     return std::make_unique<T>(std::forward<Args>(args)...);
@@ -364,20 +143,30 @@ inline auto makeEs(Ts&&... pack) {
 
 namespace detail {
 // base case
-inline void makeEM_unwind(value::SlotMap<std::unique_ptr<EExpression>>& result,
-                          value::SlotId slot,
-                          std::unique_ptr<EExpression> expr) {
-    result.emplace(slot, std::move(expr));
+template <typename R>
+inline void makeSlotExprPairHelper(R& result,
+                                   value::SlotId slot,
+                                   std::unique_ptr<EExpression> expr) {
+    if constexpr (std::is_same_v<R, value::SlotMap<std::unique_ptr<EExpression>>>) {
+        result.emplace(slot, std::move(expr));
+    } else {
+        result.push_back({slot, std::move(expr)});
+    }
 }
 
 // recursive case
-template <typename... Ts>
-inline void makeEM_unwind(value::SlotMap<std::unique_ptr<EExpression>>& result,
-                          value::SlotId slot,
-                          std::unique_ptr<EExpression> expr,
-                          Ts&&... rest) {
-    result.emplace(slot, std::move(expr));
-    makeEM_unwind(result, std::forward<Ts>(rest)...);
+template <typename R, typename... Ts>
+inline void makeSlotExprPairHelper(R& result,
+                                   value::SlotId slot,
+                                   std::unique_ptr<EExpression> expr,
+                                   Ts&&... rest) {
+    if constexpr (std::is_same_v<R, value::SlotMap<std::unique_ptr<EExpression>>>) {
+        result.emplace(slot, std::move(expr));
+    } else {
+        static_assert(std::is_same_v<R, SlotExprPairVector>);
+        result.push_back({slot, std::move(expr)});
+    }
+    makeSlotExprPairHelper(result, std::forward<Ts>(rest)...);
 }
 }  // namespace detail
 
@@ -386,7 +175,7 @@ auto makeEM(Ts&&... pack) {
     value::SlotMap<std::unique_ptr<EExpression>> result;
     if constexpr (sizeof...(pack) > 0) {
         result.reserve(sizeof...(Ts) / 2);
-        detail::makeEM_unwind(result, std::forward<Ts>(pack)...);
+        detail::makeSlotExprPairHelper(result, std::forward<Ts>(pack)...);
     }
     return result;
 }
@@ -399,6 +188,16 @@ auto makeSV(Args&&... args) {
     return v;
 }
 
+template <typename... Ts>
+auto makeSlotExprPairVec(Ts&&... pack) {
+    SlotExprPairVector v;
+    if constexpr (sizeof...(pack) > 0) {
+        v.reserve(sizeof...(Ts) / 2);
+        detail::makeSlotExprPairHelper(v, std::forward<Ts>(pack)...);
+    }
+    return v;
+}
+
 /**
  * This is a constant expression. It assumes the ownership of the input constant.
  */
@@ -448,6 +247,16 @@ public:
     size_t estimateSize() const final {
         return sizeof(*this);
     }
+    boost::optional<FrameId> getFrameId() const {
+        return _frameId;
+    }
+    value::SlotId getSlotId() const {
+        return _var;
+    }
+
+    bool isMoveFrom() const {
+        return _moveFrom;
+    }
 
 private:
     value::SlotId _var;
@@ -469,6 +278,10 @@ public:
         logicAnd,
         logicOr,
 
+        // Nothing-handling operation. This is short-circuiting like logicOr,
+        // but it checks Nothing / non-Nothing instead of false / true.
+        fillEmpty,
+
         // Math operations.
         add,
         sub,
@@ -515,6 +328,10 @@ public:
 
     size_t estimateSize() const final;
 
+private:
+    std::vector<const EExpression*> collectOrClauses() const;
+    std::vector<const EExpression*> collectAndClauses() const;
+
 private:
     Op _op;
 };
