@@ -27,11 +27,13 @@
  *    it in the license file.
  */
 
+#include "mongo/db/exec/sbe/sbe_unittest.h"
 #include "mongo/db/exec/sbe/values/bson.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/sbe/vm/vm.h"
+#include "mongo/db/exec/sbe/vm/vm_printer.h"
+#include "mongo/unittest/golden_test.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/pcre.h"
 
 namespace mongo::sbe {
 
@@ -243,7 +245,7 @@ TEST(SBEVM, Add) {
         vm::CodeFragment code;
         code.appendConstVal(tagInt32, valInt32);
         code.appendConstVal(tagInt64, valInt64);
-        code.appendAdd();
+        code.appendAdd({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -261,7 +263,7 @@ TEST(SBEVM, Add) {
         vm::CodeFragment code;
         code.appendConstVal(tagInt32, valInt32);
         code.appendConstVal(tagDouble, valDouble);
-        code.appendAdd();
+        code.appendAdd({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -278,7 +280,7 @@ TEST(SBEVM, Add) {
         vm::CodeFragment code;
         code.appendConstVal(tagDecimal, valDecimal);
         code.appendConstVal(tagDouble, valDouble);
-        code.appendAdd();
+        code.appendAdd({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -304,7 +306,7 @@ TEST(SBEVM, CompareBinData) {
                             value::bitcastFrom<const char*>(operands[0].value()));
         code.appendConstVal(value::TypeTags::bsonBinData,
                             value::bitcastFrom<const char*>(operands[1].value()));
-        code.appendCmp3w();
+        code.appendCmp3w({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -324,7 +326,7 @@ TEST(SBEVM, CompareBinData) {
                             value::bitcastFrom<const char*>(operands[0].value()));
         code.appendConstVal(value::TypeTags::bsonBinData,
                             value::bitcastFrom<const char*>(operands[1].value()));
-        code.appendCmp3w();
+        code.appendCmp3w({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -344,7 +346,7 @@ TEST(SBEVM, CompareBinData) {
                             value::bitcastFrom<const char*>(operands[0].value()));
         code.appendConstVal(value::TypeTags::bsonBinData,
                             value::bitcastFrom<const char*>(operands[1].value()));
-        code.appendCmp3w();
+        code.appendCmp3w({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -368,7 +370,7 @@ TEST(SBEVM, CompareBinData) {
                             value::bitcastFrom<const char*>(operands[0].value()));
         code.appendConstVal(value::TypeTags::bsonBinData,
                             value::bitcastFrom<const char*>(operands[1].value()));
-        code.appendCmp3w();
+        code.appendCmp3w({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -392,7 +394,7 @@ TEST(SBEVM, CompareBinData) {
                             value::bitcastFrom<const char*>(operands[0].value()));
         code.appendConstVal(value::TypeTags::bsonBinData,
                             value::bitcastFrom<const char*>(operands[1].value()));
-        code.appendCmp3w();
+        code.appendCmp3w({}, {});
 
         vm::ByteCode interpreter;
         auto [owned, tag, val] = interpreter.run(&code);
@@ -420,83 +422,43 @@ TEST(SBEVM, ConvertBinDataToBsonObj) {
     ASSERT_EQ(originalBinData.woCompare(convertedBinData), 0);
 }
 
-namespace {
+TEST(SBEVM, CodeFragmentToStringSanity) {
+    vm::CodeFragment code;
+    auto ptr2str = [](const void* ptr) {
+        std::stringstream ss;
+        ss << ptr;
+        return ss.str();
+    };
 
-// The hex representation of memory addresses in the output of CodeFragment::toString() differs on
-// Linux and Windows machines so 'addrPattern' is used to cover both cases.
-static const std::string kLinuxAddrPattern{"(0x[a-f0-9]+)"};
-static const std::string kWindowsAddrPattern{"([A-F0-9]+)"};
-static const std::string kAddrPattern{"(" + kLinuxAddrPattern + "|" + kWindowsAddrPattern + ")"};
+    code.appendDiv({}, {});
+    std::string instrs = code.toString();
 
-// The beginning of the output from CodeFragment::toString() gives a range of the addresses that
-// 'pcPointer' will traverse.
-static const std::string kPcPointerRangePattern{"(\\[" + kAddrPattern + ")-(" + kAddrPattern +
-                                                ")\\])"};
-
-/**
- * Creates a pcre pattern to match the instructions in the output of CodeFragment::toString(). Any
- * arguments must be passed in a single comma separated string, and no arguments can be represented
- * using an empty string.
- */
-std::string instrPattern(std::string op, std::string args) {
-    return "(" + kAddrPattern + ": " + op + "\\(" + args + "\\); )";
-}
-}  // namespace
-
-TEST(SBEVM, CodeFragmentToString) {
-    {
-        vm::CodeFragment code;
-        std::string toStringPattern{kPcPointerRangePattern + "( )"};
-
-        code.appendDiv();
-        toStringPattern += instrPattern("div", "");
-        code.appendMul();
-        toStringPattern += instrPattern("mul", "");
-        code.appendAdd();
-        toStringPattern += instrPattern("add", "");
-
-        std::string instrs = code.toString();
-
-        static const pcre::Regex validToStringOutput{toStringPattern};
-
-        ASSERT_TRUE(!!validToStringOutput.matchView(instrs));
-    }
+    ASSERT_TRUE(instrs.find("[" + ptr2str(code.instrs().data()) + "]: div") >= 0);
 }
 
-TEST(SBEVM, CodeFragmentToStringArgs) {
-    {
-        vm::CodeFragment code;
-        std::string toStringPattern{kAddrPattern};
+TEST(SBEVM, CodeFragmentPrintStable) {
+    GoldenTestContext ctx(&goldenTestConfigSbe);
+    ctx.printTestHeader(GoldenTestContext::HeaderFormat::Text);
 
-        code.appendFillEmpty(vm::Instruction::True);
-        toStringPattern += instrPattern("fillEmptyConst", "k: True");
-        code.appendFillEmpty(vm::Instruction::Null);
-        toStringPattern += instrPattern("fillEmptyConst", "k: Null");
-        code.appendFillEmpty(vm::Instruction::False);
-        toStringPattern += instrPattern("fillEmptyConst", "k: False");
+    auto& os = ctx.outStream();
 
-        code.appendTraverseP(0xAA);
-        auto offsetP = 0xAA - code.instrs().size();
-        toStringPattern += instrPattern("traversePConst", "offset: " + std::to_string(offsetP));
-        code.appendTraverseF(0xBB, vm::Instruction::True);
-        auto offsetF = 0xBB - code.instrs().size();
-        toStringPattern +=
-            instrPattern("traverseFConst", "k: True, offset: " + std::to_string(offsetF));
+    vm::CodeFragment code;
+    code.appendFillEmpty(vm::Instruction::Null);
+    code.appendFillEmpty(vm::Instruction::False);
+    code.appendFillEmpty(vm::Instruction::True);
+    code.appendTraverseP(0xAA, vm::Instruction::Nothing);
+    code.appendTraverseP(0xAA, vm::Instruction::Int32One);
+    code.appendTraverseF(0xBB, vm::Instruction::True);
+    code.appendGetField({}, "Hello world!"_sd);
+    code.appendAdd({}, {});
 
-        auto [tag, val] = value::makeNewString("Hello world!");
-        value::ValueGuard guard{tag, val};
-        code.appendGetField(tag, val);
-        toStringPattern += instrPattern("getFieldConst", "value: \"Hello world!\"");
+    TimeZoneDatabase timezoneDB;
+    code.appendDateTrunc(
+        TimeUnit::day, 1, timezoneDB.getTimeZone("America/New_York"_sd), DayOfWeek::monday);
 
-        code.appendAdd();
-        toStringPattern += instrPattern("add", "");
-
-        std::string instrs = code.toString();
-
-        static const pcre::Regex validToStringOutput{toStringPattern};
-
-        ASSERT_TRUE(!!validToStringOutput.matchView(instrs));
-    }
+    vm::CodeFragmentPrinter printer(vm::CodeFragmentPrinter::PrintFormat::Stable);
+    printer.print(os, code);
+    os << std::endl;
 }
 
 namespace {
