@@ -35,7 +35,6 @@
 #include "mongo/db/exec/sbe/values/slot.h"
 #include "mongo/db/exec/sbe/values/value.h"
 #include "mongo/db/exec/scoped_timer.h"
-#include "mongo/db/exec/scoped_timer_factory.h"
 #include "mongo/db/exec/trial_run_tracker.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/plan_yield_policy.h"
@@ -124,19 +123,35 @@ public:
      * Ensures that accessor owns the underlying BSON value, which can be potentially owned by
      * storage.
      */
-    static void prepareForYielding(value::OwnedValueAccessor& accessor) {
-        auto [tag, value] = accessor.getViewOfValue();
-        if (shouldCopyValue(tag)) {
-            accessor.makeOwned();
+    void prepareForYielding(value::OwnedValueAccessor& accessor, bool isAccessible) {
+        if (isAccessible) {
+            auto [tag, value] = accessor.getViewOfValue();
+            if (shouldCopyValue(tag)) {
+                accessor.makeOwned();
+            }
+        } else {
+#if defined(MONGO_CONFIG_DEBUG_BUILD)
+            auto [tag, val] = value::getPoisonValue();
+            accessor.reset(false, tag, val);
+#endif
         }
     }
 
-    static void prepareForYielding(value::MaterializedRow& row) {
-        for (size_t idx = 0; idx < row.size(); idx++) {
-            auto [tag, value] = row.getViewOfValue(idx);
-            if (shouldCopyValue(tag)) {
-                row.makeOwned(idx);
+    void prepareForYielding(value::MaterializedRow& row, bool isAccessible) {
+        if (isAccessible) {
+            for (size_t idx = 0; idx < row.size(); idx++) {
+                auto [tag, value] = row.getViewOfValue(idx);
+                if (shouldCopyValue(tag)) {
+                    row.makeOwned(idx);
+                }
             }
+        } else {
+#if defined(MONGO_CONFIG_DEBUG_BUILD)
+            auto [tag, val] = value::getPoisonValue();
+            for (size_t idx = 0; idx < row.size(); idx++) {
+                row.reset(idx, false, tag, val);
+            }
+#endif
         }
     }
 
@@ -151,17 +166,23 @@ public:
      * The 'relinquishCursor' parameter indicates whether cursors should be reset and all data
      * should be copied.
      *
+     * When 'relinquishCursor' is true, the 'disableSlotAccess' parameter indicates whether this
+     * stage is allowed to discard slot state before saving. When 'relinquishCursor' is false, the
+     * 'disableSlotAccess' parameter has no effect.
+     *
      * TODO SERVER-59620: Remove the 'relinquishCursor' parameter once all callers pass 'false'.
      */
-    void saveState(bool relinquishCursor) {
+    void saveState(bool relinquishCursor, bool disableSlotAccess = false) {
         auto stage = static_cast<T*>(this);
         stage->_commonStats.yields++;
+
+        if (relinquishCursor && disableSlotAccess) {
+            stage->disableSlotAccess();
+        }
+
         stage->doSaveState(relinquishCursor);
-        // Save the children in a right to left order so dependent stages (i.e. one using correlated
-        // slots) are saved first.
-        auto& children = stage->_children;
-        for (auto it = children.rbegin(); it != children.rend(); it++) {
-            (*it)->saveState(relinquishCursor);
+        if (!stage->_children.empty()) {
+            saveChildrenState(relinquishCursor, disableSlotAccess);
         }
 
 #if defined(MONGO_CONFIG_DEBUG_BUILD)
@@ -215,6 +236,25 @@ protected:
     SaveState _saveState{SaveState::kNotSaved};
 #endif
 
+    virtual void saveChildrenState(bool relinquishCursor, bool disableSlotAccess) {
+        // clang-format off
+        static const StringDataSet propagateSet = {
+            "branch", "cfilter", "efilter", "exchangep", "filter",   "limit", "limitskip",
+            "lspool", "mkbson",  "mkobj",   "project",   "traverse", "union", "unique"};
+        // clang-format on
+
+        auto stage = static_cast<T*>(this);
+        if (!propagateSet.count(stage->getCommonStats()->stageType)) {
+            disableSlotAccess = false;
+        }
+
+        // Save the children in a right to left order so dependent stages (i.e. one using correlated
+        // slots) are saved first.
+        for (auto it = stage->_children.rbegin(); it != stage->_children.rend(); ++it) {
+            (*it)->saveState(relinquishCursor, disableSlotAccess);
+        }
+    }
+
     static bool shouldCopyValue(value::TypeTags tag) {
         switch (tag) {
             case value::TypeTags::Array:
@@ -361,8 +401,8 @@ public:
         invariant(durationCount<Microseconds>(_commonStats.executionTime.executionTimeEstimate) ==
                   0);
 
-        if (internalMeasureQueryExecutionTimeInMicroseconds.load()) {
-            _commonStats.executionTime.precision = QueryExecTimerPrecision::kMicros;
+        if (internalMeasureQueryExecutionTimeInNanoseconds.load()) {
+            _commonStats.executionTime.precision = QueryExecTimerPrecision::kNanos;
         } else {
             _commonStats.executionTime.precision = QueryExecTimerPrecision::kMillis;
         }
@@ -387,13 +427,16 @@ public:
         _participateInTrialRunTracking = false;
     }
 
+    bool slotsAccessible() const {
+        return _slotsAccessible;
+    }
+
 protected:
     PlanState trackPlanState(PlanState state) {
         if (state == PlanState::IS_EOF) {
             _commonStats.isEOF = true;
             _slotsAccessible = false;
         } else {
-            invariant(state == PlanState::ADVANCED);
             _commonStats.advances++;
             _slotsAccessible = true;
         }
@@ -405,19 +448,25 @@ protected:
         _slotsAccessible = false;
     }
 
-    bool slotsAccessible() const {
-        return _slotsAccessible;
-    }
-
     /**
      * Returns an optional timer which is used to collect time spent executing the current stage.
      * May return boost::none if it is not necessary to collect timing info.
      */
     boost::optional<ScopedTimer> getOptTimer(OperationContext* opCtx) {
-        if (opCtx) {
-            return scoped_timer_factory::make(opCtx->getServiceContext(),
-                                              _commonStats.executionTime.precision,
-                                              &_commonStats.executionTime.executionTimeEstimate);
+        if (opCtx && _commonStats.executionTime.precision != QueryExecTimerPrecision::kNoTiming) {
+
+            if (MONGO_likely(_commonStats.executionTime.precision ==
+                             QueryExecTimerPrecision::kMillis)) {
+                return boost::optional<ScopedTimer>(
+                    boost::in_place_init,
+                    &_commonStats.executionTime.executionTimeEstimate,
+                    opCtx->getServiceContext()->getFastClockSource());
+            } else {
+                return boost::optional<ScopedTimer>(
+                    boost::in_place_init,
+                    &_commonStats.executionTime.executionTimeEstimate,
+                    opCtx->getServiceContext()->getTickSource());
+            }
         }
 
         return boost::none;
