@@ -33,6 +33,8 @@
 
 #include "mongo/base/compare_numbers.h"
 #include "mongo/db/exec/js_function.h"
+#include "mongo/db/exec/sbe/makeobj_spec.h"
+#include "mongo/db/exec/sbe/size_estimator.h"
 #include "mongo/db/exec/sbe/values/bson.h"
 #include "mongo/db/exec/sbe/values/sort_spec.h"
 #include "mongo/db/exec/sbe/values/value_builder.h"
@@ -146,28 +148,65 @@ std::pair<TypeTags, Value> makeCopyPcreRegex(const pcre::Regex& regex) {
     return {TypeTags::pcreRegex, bitcastFrom<pcre::Regex*>(regexCopy.release())};
 }
 
+std::pair<TypeTags, Value> makeCopyTimeZone(const TimeZone& tz) {
+    auto tzCopy = std::make_unique<TimeZone>(tz);
+    return {TypeTags::timeZone, bitcastFrom<TimeZone*>(tzCopy.release())};
+}
+
 KeyString::Value SortSpec::generateSortKey(const BSONObj& obj, const CollatorInterface* collator) {
-    KeyStringSet keySet;
-    SharedBufferFragmentBuilder allocator(KeyString::HeapBuilder::kHeapAllocatorDefaultBytes);
-    const bool skipMultikey = false;
-    MultikeyPaths* multikeyPaths = nullptr;
-    _keyGen.getKeys(allocator, obj, skipMultikey, &keySet, multikeyPaths, collator);
+    _sortKeyGen.setCollator(collator);
+    return _sortKeyGen.computeSortKeyString(obj);
+}
 
-    // When 'isSparse' is false, BtreeKeyGenerator::getKeys() is guaranteed to insert at least
-    // one key into 'keySet', so this assertion should always be true.
-    tassert(5037000, "BtreeKeyGenerator failed to generate key", !keySet.empty());
+value::SortKeyComponentVector* SortSpec::generateSortKeyComponentVector(
+    FastTuple<bool, value::TypeTags, value::Value> obj, const CollatorInterface* collator) {
+    auto [objOwned, objTag, objVal] = obj;
+    ValueGuard guard(objOwned, objTag, objVal);
 
-    // Return the first KeyString in the set.
-    return std::move(*keySet.extract_sequence().begin());
+    // While this function accepts any type of object, for now we simply convert everything
+    // to BSON. In the future, we may change this function to avoid the conversion.
+    auto bsonObj = [&, objTag = objTag, objVal = objVal, objOwned = objOwned]() {
+        if (objTag == value::TypeTags::bsonObject) {
+            if (objOwned) {
+                // Take ownership of the temporary object here.
+                _tempVal.emplace(objTag, objVal);
+                guard.reset();
+            }
+            return BSONObj{value::bitcastTo<const char*>(objVal)};
+        } else if (objTag == value::TypeTags::Object) {
+            BSONObjBuilder objBuilder;
+            bson::convertToBsonObj(objBuilder, value::getObjectView(objVal));
+            _tempObj = objBuilder.obj();
+            return _tempObj;
+        } else {
+            MONGO_UNREACHABLE_TASSERT(7103703);
+        }
+    }();
+
+
+    _sortKeyGen.setCollator(collator);
+    // Use the generic API for getting an array of bson elements representing the
+    // sort key.
+    _sortKeyGen.generateSortKeyComponentVector(bsonObj, &_localBsonEltStorage);
+
+    // Convert this array of BSONElements into the SBE SortKeyComponentVector type.
+    {
+        size_t i = 0;
+        for (auto& elt : _localBsonEltStorage) {
+            _localSortKeyComponentStorage.elts[i++] = bson::convertFrom<true>(elt);
+        }
+    }
+    return &_localSortKeyComponentStorage;
 }
 
 BtreeKeyGenerator SortSpec::initKeyGen() const {
-    tassert(
-        5037003, "SortSpec should not be passed an empty sort pattern", !_sortPattern.isEmpty());
+    tassert(5037003,
+            "SortSpec should not be passed an empty sort pattern",
+            !_sortPatternBson.isEmpty());
 
     std::vector<const char*> fields;
     std::vector<BSONElement> fixed;
-    for (auto&& elem : _sortPattern) {
+    for (auto&& elem : _sortPatternBson) {
         fields.push_back(elem.fieldName());
 
         // BtreeKeyGenerator's constructor's first parameter (the 'fields' vector) and second
@@ -181,15 +220,15 @@ BtreeKeyGenerator SortSpec::initKeyGen() const {
 
     const bool isSparse = false;
     auto version = KeyString::Version::kLatestVersion;
-    auto ordering = Ordering::make(_sortPattern);
+    auto ordering = Ordering::make(_sortPatternBson);
 
     return {std::move(fields), std::move(fixed), isSparse, version, ordering};
 }
 
 size_t SortSpec::getApproximateSize() const {
     auto size = sizeof(SortSpec);
-    size += _sortPattern.isOwned() ? _sortPattern.objsize() : 0;
-    size += _keyGen.getApproximateSize() - sizeof(_keyGen);
+    size += _sortKeyGen.getApproximateSize();
+    size += _sortPatternBson.isOwned() ? _sortPatternBson.objsize() : 0;
     return size;
 }
 
@@ -213,6 +252,11 @@ std::pair<TypeTags, Value> makeCopySortSpec(const SortSpec& ss) {
     return {TypeTags::sortSpec, ssCopy};
 }
 
+std::pair<TypeTags, Value> makeCopyMakeObjSpec(const MakeObjSpec& mos) {
+    auto mosCopy = bitcastFrom<MakeObjSpec*>(new MakeObjSpec(mos));
+    return {TypeTags::makeObjSpec, mosCopy};
+}
+
 std::pair<TypeTags, Value> makeCopyCollator(const CollatorInterface& collator) {
     auto collatorCopy = bitcastFrom<CollatorInterface*>(collator.clone().release());
     return {TypeTags::collator, collatorCopy};
@@ -291,14 +335,17 @@ void releaseValueDeep(TypeTags tag, Value val) noexcept {
         case TypeTags::sortSpec:
             delete getSortSpecView(val);
             break;
+        case TypeTags::makeObjSpec:
+            delete getMakeObjSpecView(val);
+            break;
         case TypeTags::collator:
             delete getCollatorView(val);
             break;
         case TypeTags::indexBounds:
             delete getIndexBoundsView(val);
             break;
-        case TypeTags::classicMatchExpresion:
-            delete getClassicMatchExpressionView(val);
+        case TypeTags::timeZone:
+            delete getTimeZoneView(val);
             break;
         default:
             break;
@@ -527,6 +574,11 @@ std::size_t hashValue(TypeTags tag, Value val, const CollatorInterface* collator
                 hashCombine(hashInit(), abslHash(cws.code)),
                 hashValue(TypeTags::bsonObject, bitcastFrom<const char*>(cws.scope)));
         }
+        case TypeTags::timeZone: {
+            auto timezone = getTimeZoneView(val);
+            return hashCombine(hashCombine(hashInit(), abslHash(timezone->getTzInfo())),
+                               abslHash(timezone->getUtcOffset().count()));
+        }
         default:
             break;
     }
@@ -780,17 +832,17 @@ std::pair<TypeTags, Value> compareValue(TypeTags lhsTag,
     }
 }
 
-bool isNaN(TypeTags tag, Value val) {
+bool isNaN(TypeTags tag, Value val) noexcept {
     return (tag == TypeTags::NumberDouble && std::isnan(bitcastTo<double>(val))) ||
         (tag == TypeTags::NumberDecimal && bitcastTo<Decimal128>(val).isNaN());
 }
 
-bool isInfinity(TypeTags tag, Value val) {
+bool isInfinity(TypeTags tag, Value val) noexcept {
     return (tag == TypeTags::NumberDouble && std::isinf(bitcastTo<double>(val))) ||
         (tag == TypeTags::NumberDecimal && bitcastTo<Decimal128>(val).isInfinite());
 }
 
-void ArraySet::push_back(TypeTags tag, Value val) {
+bool ArraySet::push_back(TypeTags tag, Value val) {
     if (tag != TypeTags::Nothing) {
         ValueGuard guard{tag, val};
         auto [it, inserted] = _values.insert({tag, val});
@@ -798,7 +850,11 @@ void ArraySet::push_back(TypeTags tag, Value val) {
         if (inserted) {
             guard.reset();
         }
+
+        return inserted;
     }
+
+    return false;
 }
 
 std::pair<TypeTags, Value> ArrayEnumerator::getViewOfValue() const {
@@ -807,8 +863,7 @@ std::pair<TypeTags, Value> ArrayEnumerator::getViewOfValue() const {
     } else if (_arraySet) {
         return {_iter->first, _iter->second};
     } else {
-        auto sv = bson::fieldNameView(_arrayCurrent);
-        return bson::convertFrom<true>(_arrayCurrent, _arrayEnd, sv.size());
+        return bson::convertFrom<true>(_arrayCurrent, _arrayEnd, _fieldNameSize);
     }
 }
 
@@ -826,12 +881,14 @@ bool ArrayEnumerator::advance() {
 
         return _iter != _arraySet->values().end();
     } else {
-        if (*_arrayCurrent != 0) {
-            auto sv = bson::fieldNameView(_arrayCurrent);
-            _arrayCurrent = bson::advance(_arrayCurrent, sv.size());
+        if (_arrayCurrent != _arrayEnd - 1) {
+            _arrayCurrent = bson::advance(_arrayCurrent, _fieldNameSize);
+            if (_arrayCurrent != _arrayEnd - 1) {
+                _fieldNameSize = TinyStrHelpers::strlen(bson::fieldNameRaw(_arrayCurrent));
+            }
         }
 
-        return *_arrayCurrent != 0;
+        return _arrayCurrent != _arrayEnd - 1;
     }
 }
 
@@ -839,7 +896,7 @@ std::pair<TypeTags, Value> ObjectEnumerator::getViewOfValue() const {
     if (_object) {
         return _object->getAt(_index);
     } else {
-        auto sv = bson::fieldNameView(_objectCurrent);
+        auto sv = bson::fieldNameAndLength(_objectCurrent);
         return bson::convertFrom<true>(_objectCurrent, _objectEnd, sv.size());
     }
 }
@@ -853,7 +910,7 @@ bool ObjectEnumerator::advance() {
         return _index < _object->size();
     } else {
         if (*_objectCurrent != 0) {
-            auto sv = bson::fieldNameView(_objectCurrent);
+            auto sv = bson::fieldNameAndLength(_objectCurrent);
             _objectCurrent = bson::advance(_objectCurrent, sv.size());
         }
 
@@ -871,7 +928,7 @@ StringData ObjectEnumerator::getFieldName() const {
         }
     } else {
         if (*_objectCurrent != 0) {
-            return bson::fieldNameView(_objectCurrent);
+            return bson::fieldNameAndLength(_objectCurrent);
         } else {
             return ""_sd;
         }
