@@ -116,11 +116,12 @@ public:
         return {};
     }
 
-    virtual void addRequiredPrivileges(const std::string& dbname,
-                                       const BSONObj& cmdObj,
-                                       std::vector<Privilege>* out) const {
-        // Command is testing-only, and can only be enabled at command line.  Hence, no auth
-        // check needed.
+    Status checkAuthForOperation(OperationContext*,
+                                 const DatabaseName&,
+                                 const BSONObj&) const override {
+        // Command is testing-only, and can only be enabled at command line.
+        //  Hence, no auth check needed.
+        return Status::OK();
     }
 
     bool run(OperationContext* opCtx,
@@ -139,7 +140,8 @@ public:
             return false;
         }
 
-        const NamespaceString nss(dbName, collElt.String());
+        const NamespaceString nss(
+            NamespaceStringUtil::parseNamespaceFromRequest(dbName, collElt.String()));
         uassert(ErrorCodes::InvalidNamespace,
                 str::stream() << nss.toString() << " is not a valid namespace",
                 nss.isValid());
@@ -240,7 +242,7 @@ public:
                 std::unique_ptr<MatchExpression> me = std::move(statusWithMatcher.getValue());
                 // exprs is what will wind up deleting this.
                 matcher = me.get();
-                verify(nullptr != matcher);
+                MONGO_verify(nullptr != matcher);
                 exprs->push_back(std::move(me));
             } else if (argsTag == e.fieldName()) {
                 nodeArgs = argObj;
