@@ -32,6 +32,7 @@
 
 #include <cmath>
 #include <memory>
+#include <s2polygonbuilder.h>
 #include <string>
 #include <vector>
 
@@ -40,7 +41,6 @@
 #include "mongo/db/jsobj.h"
 #include "mongo/util/str.h"
 #include "mongo/util/transitional_tools_do_not_use/vector_spooling.h"
-#include "third_party/s2/s2polygonbuilder.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kGeo
 
@@ -51,19 +51,24 @@ namespace mongo {
 
 namespace dps = ::mongo::dotted_path_support;
 
-static Status parseFlatPoint(const BSONElement& elem, Point* out, bool allowAddlFields = false) {
+// Convenience function to extract flat point coordinates from enclosing element.
+// Note, coordinate elements must not outlive the parent element.
+Status GeoParser::parseFlatPointCoordinates(const BSONElement& elem,
+                                            BSONElement& x,
+                                            BSONElement& y,
+                                            bool allowAddlFields /* = false */) {
     if (!elem.isABSONObj()) {
         return BAD_VALUE("Point must be an array or object, instead got type "
                          << typeName(elem.type()));
     }
 
     BSONObjIterator it(elem.Obj());
-    BSONElement x = it.next();
+    x = it.next();
     if (!x.isNumber()) {
         return BAD_VALUE("Point must only contain numeric elements, instead got type "
                          << typeName(x.type()));
     }
-    BSONElement y = it.next();
+    y = it.next();
     if (!y.isNumber()) {
         return BAD_VALUE("Point must only contain numeric elements, instead got type "
                          << typeName(y.type()));
@@ -71,6 +76,16 @@ static Status parseFlatPoint(const BSONElement& elem, Point* out, bool allowAddl
     if (!allowAddlFields && it.more()) {
         return BAD_VALUE("Point must only contain two numeric elements");
     }
+    return Status::OK();
+}
+
+static Status parseFlatPoint(const BSONElement& elem, Point* out, bool allowAddlFields = false) {
+    BSONElement x, y;
+    auto status = GeoParser::parseFlatPointCoordinates(elem, x, y, allowAddlFields);
+    if (!status.isOK()) {
+        return status;
+    }
+
     out->x = x.number();
     out->y = y.number();
     // Point coordinates must be finite numbers, neither NaN or infinite.
@@ -803,20 +818,23 @@ GeoParser::GeoJSONType GeoParser::parseGeoJSONType(const BSONObj& obj) {
     if (String != type.type()) {
         return GeoParser::GEOJSON_UNKNOWN;
     }
-    const string& typeString = type.String();
-    if (GEOJSON_TYPE_POINT == typeString) {
+    return geoJSONTypeStringToEnum(type.checkAndGetStringData());
+}
+
+GeoParser::GeoJSONType GeoParser::geoJSONTypeStringToEnum(StringData type) {
+    if (GEOJSON_TYPE_POINT == type) {
         return GeoParser::GEOJSON_POINT;
-    } else if (GEOJSON_TYPE_LINESTRING == typeString) {
+    } else if (GEOJSON_TYPE_LINESTRING == type) {
         return GeoParser::GEOJSON_LINESTRING;
-    } else if (GEOJSON_TYPE_POLYGON == typeString) {
+    } else if (GEOJSON_TYPE_POLYGON == type) {
         return GeoParser::GEOJSON_POLYGON;
-    } else if (GEOJSON_TYPE_MULTI_POINT == typeString) {
+    } else if (GEOJSON_TYPE_MULTI_POINT == type) {
         return GeoParser::GEOJSON_MULTI_POINT;
-    } else if (GEOJSON_TYPE_MULTI_LINESTRING == typeString) {
+    } else if (GEOJSON_TYPE_MULTI_LINESTRING == type) {
         return GeoParser::GEOJSON_MULTI_LINESTRING;
-    } else if (GEOJSON_TYPE_MULTI_POLYGON == typeString) {
+    } else if (GEOJSON_TYPE_MULTI_POLYGON == type) {
         return GeoParser::GEOJSON_MULTI_POLYGON;
-    } else if (GEOJSON_TYPE_GEOMETRY_COLLECTION == typeString) {
+    } else if (GEOJSON_TYPE_GEOMETRY_COLLECTION == type) {
         return GeoParser::GEOJSON_GEOMETRY_COLLECTION;
     }
     return GeoParser::GEOJSON_UNKNOWN;
