@@ -27,7 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -42,6 +41,9 @@
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/unittest.h"
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
+
 using namespace mongo;
 using std::cout;
 using std::endl;
@@ -70,9 +72,9 @@ std::string dumpMultikeyPaths(const MultikeyPaths& multikeyPaths) {
     std::stringstream ss;
 
     ss << "[ ";
-    for (const auto multikeyComponents : multikeyPaths) {
+    for (const auto& multikeyComponents : multikeyPaths) {
         ss << "[ ";
-        for (const auto multikeyComponent : multikeyComponents) {
+        for (const auto& multikeyComponent : multikeyComponents) {
             ss << multikeyComponent << " ";
         }
         ss << "] ";
@@ -116,12 +118,8 @@ bool testKeygen(const BSONObj& kp,
         fixed.push_back(BSONElement());
     }
 
-    auto keyGen = std::make_unique<BtreeKeyGenerator>(fieldNames,
-                                                      fixed,
-                                                      sparse,
-                                                      collator,
-                                                      KeyString::Version::kLatestVersion,
-                                                      Ordering::make(BSONObj()));
+    auto keyGen = std::make_unique<BtreeKeyGenerator>(
+        fieldNames, fixed, sparse, KeyString::Version::kLatestVersion, Ordering::make(BSONObj()));
 
     auto runTest = [&](bool skipMultikey) {
         //
@@ -129,9 +127,10 @@ bool testKeygen(const BSONObj& kp,
         // indexed fields that would cause the index to be multikey as a result of inserting
         // 'actualKeys'.
         //
+        SharedBufferFragmentBuilder allocator(BufBuilder::kDefaultInitSizeBytes);
         KeyStringSet actualKeys;
         MultikeyPaths actualMultikeyPaths;
-        keyGen->getKeys(obj, skipMultikey, &actualKeys, &actualMultikeyPaths);
+        keyGen->getKeys(allocator, obj, skipMultikey, &actualKeys, &actualMultikeyPaths, collator);
 
         //
         // Check that the results match the expected result.
@@ -186,7 +185,7 @@ TEST(BtreeKeyGeneratorTest, GetIdKeyFromObject) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 'foo'}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -196,7 +195,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromObjectSimple) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 5}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -206,7 +205,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromObjectDotted) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 4}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -254,7 +253,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromArrayFirstElement) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': 3, '': 2}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release(), keyString3.release()};
-    MultikeyPaths expectedMultikeyPaths{{0U}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{{0U}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -268,7 +267,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromArraySecondElement) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': 5, '': 3}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release(), keyString3.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}, {0U}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}, {0U}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -337,7 +336,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysArraySubobjectCompoundIndex) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': 3, '': 99}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release(), keyString3.release()};
-    MultikeyPaths expectedMultikeyPaths{{0U}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{{0U}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -374,7 +373,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysMissingField) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': null}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -395,7 +394,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromCompound) {
                                      fromjson("{'': 'a', '': 'b'}"),
                                      Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -406,7 +405,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromCompoundMissing) {
                                      fromjson("{'': 'a', '': null}"),
                                      Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -558,7 +557,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromMultiEmptyArray) {
                                       fromjson("{'': 1, '': undefined}"),
                                       Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.getValueCopy(), keyString2.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}, {0U}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}, {0U}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 
     genKeysFrom = fromjson("{a: 1, b: [1]}");
@@ -649,7 +648,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromSparseEmptyArray) {
     BSONObj keyPattern = fromjson("{'a.b': 1}");
     BSONObj genKeysFrom = fromjson("{a:1}");
     KeyStringSet expectedKeys;
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, sparse));
 
     genKeysFrom = fromjson("{a:[]}");
@@ -665,7 +664,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromSparseEmptyArraySecond) {
     BSONObj keyPattern = fromjson("{z: 1, 'a.b': 1}");
     BSONObj genKeysFrom = fromjson("{a:1}");
     KeyStringSet expectedKeys;
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, sparse));
 
     genKeysFrom = fromjson("{a:[]}");
@@ -705,7 +704,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromIndexedArrayIndex) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': undefined}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.getValueCopy()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 
     genKeysFrom = fromjson("{a:[[1]]}");
@@ -748,7 +747,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromDoubleIndexedArrayIndex) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': undefined}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 
     genKeysFrom = fromjson("{a:[[]]}");
@@ -768,7 +767,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromDoubleIndexedArrayIndex) {
     genKeysFrom = fromjson("{a:[[[]]]}");
     expectedKeys.clear();
     expectedKeys.insert(keyString3.release());
-    expectedMultikeyPaths = {std::set<size_t>{}};
+    expectedMultikeyPaths = {MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -782,7 +781,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromObjectWithinArray) {
     KeyString::HeapBuilder keyString3(
         KeyString::Version::kLatestVersion, fromjson("{'': undefined}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.getValueCopy()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 
     genKeysFrom = fromjson("{a:[{b:[1]}]}");
@@ -838,7 +837,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysTrailingPositionalElementIsSingletonArray) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': [3]}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -848,7 +847,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysTrailingPositionalElementIsEmptyArray) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': undefined}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -868,7 +867,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFromArrayWithinObjectWithinArray) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 1}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1045,7 +1044,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysPositionalKeyPatternNestedArray2) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': [0, 1, 2]}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1080,7 +1079,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysPositionalKeyPatternNestedArray5) {
     KeyString::HeapBuilder keyString2(
         KeyString::Version::kLatestVersion, fromjson("{'': 6}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release(), keyString2.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{0U}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{0U}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1123,7 +1122,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysRepeatedFieldName) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 2}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1151,7 +1150,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysLastPathPieceEmpty) {
     KeyString::HeapBuilder keyString2(
         KeyString::Version::kLatestVersion, fromjson("{'': {'': 2}}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString1.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 
     genKeysFrom = fromjson("{a: {'': 2}}");
@@ -1166,7 +1165,7 @@ TEST(BtreeKeyGeneratorTest, GetKeysFirstPathPieceEmpty) {
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': null}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1209,7 +1208,7 @@ TEST(BtreeKeyGeneratorTest, KeyPattern_a_0_0_b_Extracts_b_ElementInsideSingleton
     KeyString::HeapBuilder keyString(
         KeyString::Version::kLatestVersion, fromjson("{'': 1}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1290,7 +1289,7 @@ TEST(BtreeKeyGeneratorTest, PositionalKeyPatternNestedArrays6) {
                                       Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{
         keyString1.release(), keyString2.release(), keyString3.release(), keyString4.release()};
-    MultikeyPaths expectedMultikeyPaths{{0U}, {0U, 1U}, {2U}, {0U}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{{0U}, {0U, 1U}, {2U}, {0U}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1314,7 +1313,7 @@ TEST(BtreeKeyGeneratorTest, PositionalKeyPatternNestedArrays7) {
                                       Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{
         keyString1.release(), keyString2.release(), keyString3.release(), keyString4.release()};
-    MultikeyPaths expectedMultikeyPaths{{0U}, {0U, 1U}, {2U}, {0U}, std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{{0U}, {0U, 1U}, {2U}, {0U}, MultikeyComponents{}};
     ASSERT(testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths));
 }
 
@@ -1325,7 +1324,7 @@ TEST(BtreeKeyGeneratorTest, GetCollationAwareIdKeyFromObject) {
         KeyString::Version::kLatestVersion, fromjson("{'': 'oof'}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1337,7 +1336,7 @@ TEST(BtreeKeyGeneratorTest, GetCollationAwareKeysFromObjectSimple) {
         KeyString::Version::kLatestVersion, fromjson("{'': 'oof'}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1349,7 +1348,7 @@ TEST(BtreeKeyGeneratorTest, GetCollationAwareKeysFromObjectDotted) {
         KeyString::Version::kLatestVersion, fromjson("{'': 'oof'}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1377,7 +1376,7 @@ TEST(BtreeKeyGeneratorTest, CollatorDoesNotAffectNonStringIdKey) {
         KeyString::Version::kLatestVersion, fromjson("{'': 5}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1389,7 +1388,7 @@ TEST(BtreeKeyGeneratorTest, CollatorDoesNotAffectNonStringKeys) {
         KeyString::Version::kLatestVersion, fromjson("{'': 5}"), Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1402,7 +1401,7 @@ TEST(BtreeKeyGeneratorTest, GetCollationAwareKeysFromNestedObject) {
                                      Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
@@ -1415,7 +1414,7 @@ TEST(BtreeKeyGeneratorTest, GetCollationAwareKeysFromNestedArray) {
                                      Ordering::make(BSONObj()));
     KeyStringSet expectedKeys{keyString.release()};
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    MultikeyPaths expectedMultikeyPaths{std::set<size_t>{}};
+    MultikeyPaths expectedMultikeyPaths{MultikeyComponents{}};
     ASSERT(
         testKeygen(keyPattern, genKeysFrom, expectedKeys, expectedMultikeyPaths, false, &collator));
 }
