@@ -56,7 +56,8 @@ private:
 public:
     UnencodedCellView toUnencodedCellView(const SplitCellView& view) {
         // Fill builder with bson-encoded elements from view.
-        auto cursor = view.subcellValuesGenerator(ValueEncoder{&builder});
+        ValueEncoder encoder{&builder};
+        auto cursor = view.subcellValuesGenerator(&encoder);
         while (cursor.nextValue()) {
             // Work done in ValueEncoder::operator() rather than here.
         }
@@ -310,7 +311,7 @@ void basicTests(int line,
     for (auto&& [keyGen, expectedPaths] : expected) {
         StringMap<UnencodedCellValue_ForTest> expected;
         // Create expected by retrieving flags and vals from expected paths
-        for (auto path : expectedPaths) {
+        for (const auto& path : expectedPaths) {
             expected.insert({path, pathMap.find(path)->second});
         }
         // Add in the RowID column. Since it is always the same, tests shouldn't include it.
@@ -336,7 +337,7 @@ std::unique_ptr<ColumnKeyGenerator> makeKeyGen(BSONObj columnstoreProjection = B
 
 ProjPairVector expectedProjPairs(std::vector<std::pair<BSONObj, StringSet>> bsonPairs) {
     ProjPairVector projPairs;
-    for (auto [projection, fields] : bsonPairs) {
+    for (const auto& [projection, fields] : bsonPairs) {
         projPairs.push_back({makeKeyGen(projection), fields});
     }
     return projPairs;
@@ -896,18 +897,20 @@ TEST(ColKeyGen, DuplicateFieldTests) {
                    {"", {"", "", kHasDuplicateFields}},
                },
                expectedProjPairs({{{}, {""}}}));
-    // This behaves the same as {a:[{b:[1,3]}, {b:2}]} as far as a.b can tell.
+    // Although no object has "b" as a duplicate field, at least one a.b path goes through a
+    // duplicate "a" field.
     basicTests(__LINE__,
                R"({a: [{b:[1, 3]}], a: [null, {b:[2]}]})",
                {
                    {"a", {"", "", kHasDuplicateFields}},
-                   {"a.b", {"1,3,2", "[{[|1]{[", kIsSparse}},
+                   {"a.b", {"", "", kHasDuplicateFields}},
                },
                expectedProjPairs({{{}, {"a", "a.b"}},
                                   {BSON("c" << false), {"a", "a.b"}},
                                   {BSON("a.b" << true), {"a", "a.b"}},
                                   {BSON("a.b" << false), {"a"}}}));
-    // These tests only have one a.b path.
+    // The a.b path does not marked as duplicate here, because all paths a.b go through only the
+    // first "a" object.
     basicTests(__LINE__,
                R"({a: [{b:1}], a: 2})",
                {
@@ -922,7 +925,7 @@ TEST(ColKeyGen, DuplicateFieldTests) {
                R"({a: 1, a: [{b:2}]})",
                {
                    {"a", {"", "", kHasDuplicateFields}},
-                   {"a.b", {"2", "[", kIsSparse}},
+                   {"a.b", {"", "", kHasDuplicateFields}},
                },
                expectedProjPairs({{{}, {"a", "a.b"}},
                                   {BSON("c" << false), {"a", "a.b"}},
@@ -942,12 +945,33 @@ TEST(ColKeyGen, DuplicateFieldTests) {
                R"({a: [1], a: [{b:2}]})",
                {
                    {"a", {"", "", kHasDuplicateFields}},
-                   {"a.b", {"2", "[", kIsSparse}},
+                   {"a.b", {"", "", kHasDuplicateFields}},
                },
                expectedProjPairs({{{}, {"a", "a.b"}},
                                   {BSON("c" << false), {"a", "a.b"}},
                                   {BSON("a.b" << true), {"a", "a.b"}},
                                   {BSON("a.b" << false), {"a"}}}));
+    basicTests(__LINE__,
+               R"({a: [1], b: [{b: 2}], a: [{b:3}]})",
+               {
+                   {"a", {"", "", kHasDuplicateFields}},
+                   {"a.b", {"", "", kHasDuplicateFields}},
+                   {"a.b", {"", "", kHasDuplicateFields}},
+                   {"b", {"", "[o", kHasSubPath}},
+                   {"b.b", {"2", "["}},
+               },
+               expectedProjPairs({{{}, {"a", "a.b", "b", "b.b"}},
+                                  {BSON("c" << false), {"a", "a.b", "b", "b.b"}},
+                                  {BSON("a.b" << true), {"a", "a.b"}},
+                                  {BSON("b.b" << true), {"b", "b.b"}}}));
+    basicTests(__LINE__,
+               R"({a: { b: { c: 42 }, b: [ 42 ] } })",
+               {{"a", {"", "", kHasSubPath}},
+                {"a.b", {"", "", kHasDuplicateFields}},
+                {"a.b.c", {"42", "", kIsSparse}}},
+               expectedProjPairs({
+                   {{}, {"a", "a.b", "a.b.c"}},
+               }));
 }
 
 void updateTest(
@@ -960,7 +984,7 @@ void updateTest(
     std::vector<BSONElement> elems;
     for (auto&& [keyGen, expectedPaths] : projPairs) {
         StringMap<std::pair<ColumnKeyGenerator::DiffAction, UnencodedCellValue_ForTest>> expected;
-        for (auto path : expectedPaths) {
+        for (const auto& path : expectedPaths) {
             expected.insert({path, pathMap.find(path)->second});
         }
         StringSet seenPaths;
