@@ -120,6 +120,47 @@ void appendSysInfo(BSONObjBuilder* obj) {
 #endif
 }
 
+StatusWith<repl::ReplSettings> populateReplSettings(const moe::Environment& params) {
+    repl::ReplSettings replSettings;
+
+    if (params.count("replication.serverless")) {
+        if (params.count("replication.replSet") || params.count("replication.replSetName")) {
+            return Status(ErrorCodes::BadValue,
+                          "serverless cannot be used with replSet or replSetName options");
+        }
+        // Starting a node in "serverless" mode implies it uses a replSet.
+        replSettings.setServerlessMode();
+    } else if (params.count("replication.replSet")) {
+        /* seed list of hosts for the repl set */
+        replSettings.setReplSetString(params["replication.replSet"].as<std::string>().c_str());
+    } else if (params.count("replication.replSetName")) {
+        // "replSetName" is previously removed if "replSet" and "replSetName" are both found to be
+        // set by the user. Therefore, we only need to check for it if "replSet" in not found.
+        replSettings.setReplSetString(params["replication.replSetName"].as<std::string>().c_str());
+    }
+
+    if (params.count("replication.oplogSizeMB")) {
+        long long x = params["replication.oplogSizeMB"].as<int>();
+        if (x <= 0) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream() << "bad --oplogSize, arg must be greater than 0,"
+                                           "found: "
+                                        << x);
+        }
+        // note a small size such as x==1 is ok for an arbiter.
+        if (x > 1000 && sizeof(void*) == 4) {
+            StringBuilder sb;
+            sb << "--oplogSize of " << x
+               << "MB is too big for 32 bit version. Use 64 bit build instead.";
+            return Status(ErrorCodes::BadValue, sb.str());
+        }
+        replSettings.setOplogSizeBytes(x * 1024 * 1024);
+        invariant(replSettings.getOplogSizeBytes() > 0);
+    }
+
+    return replSettings;
+}
+
 }  // namespace
 
 bool handlePreValidationMongodOptions(const moe::Environment& params,
@@ -406,15 +447,6 @@ Status storeMongodOptions(const moe::Environment& params) {
 
     if (params.count("storage.syncPeriodSecs")) {
         storageGlobalParams.syncdelay = params["storage.syncPeriodSecs"].as<double>();
-        storageGlobalParams.checkpointDelaySecs =
-            static_cast<size_t>(params["storage.syncPeriodSecs"].as<double>());
-
-        if (storageGlobalParams.syncdelay < 0 ||
-            storageGlobalParams.syncdelay > StorageGlobalParams::kMaxSyncdelaySecs) {
-            return Status(ErrorCodes::BadValue,
-                          str::stream() << "syncdelay out of allowed range (0-"
-                                        << StorageGlobalParams::kMaxSyncdelaySecs << "s)");
-        }
     }
 
     if (params.count("storage.directoryPerDB")) {
@@ -477,26 +509,26 @@ Status storeMongodOptions(const moe::Environment& params) {
                           str::stream() << "Cannot specify both --repair and --restore");
         }
     }
-
-    repl::ReplSettings replSettings;
-    if (params.count("replication.serverless")) {
-        if (params.count("replication.replSet") || params.count("replication.replSetName")) {
-            return Status(ErrorCodes::BadValue,
-                          "serverless cannot be used with replSet or replSetName options");
-        }
-        // Starting a node in "serverless" mode implies it uses a replSet.
-        replSettings.setServerlessMode();
+    if (params.count("magicRestore") && params["magicRestore"].as<bool>() == true) {
+        storageGlobalParams.magicRestore = 1;
     }
-    if (params.count("replication.replSet")) {
-        /* seed list of hosts for the repl set */
-        replSettings.setReplSetString(params["replication.replSet"].as<std::string>().c_str());
-    } else if (params.count("replication.replSetName")) {
-        // "replSetName" is previously removed if "replSet" and "replSetName" are both found to be
-        // set by the user. Therefore, we only need to check for it if "replSet" in not found.
-        replSettings.setReplSetString(params["replication.replSetName"].as<std::string>().c_str());
+
+    const auto replSettingsWithStatus = populateReplSettings(params);
+    if (!replSettingsWithStatus.isOK())
+        return replSettingsWithStatus.getStatus();
+    const repl::ReplSettings& replSettings(replSettingsWithStatus.getValue());
+
+    if (replSettings.usingReplSets()) {
+        if ((params.count("security.authorization") &&
+             params["security.authorization"].as<std::string>() == "enabled") &&
+            !serverGlobalParams.startupClusterAuthMode.x509Only() &&
+            serverGlobalParams.keyFile.empty()) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream() << "security.keyFile is required when authorization is "
+                                           "enabled with replica sets");
+        }
     } else {
-        // If neither "replication.replSet" nor "replication.replSetName" is set, then we are in
-        // standalone mode.
+        // If we are not using a replica set, then we are in standalone mode.
         //
         // A standalone node does not use the oplog collection, so special truncation handling for
         // the capped collection is unnecessary.
@@ -506,23 +538,12 @@ Status storeMongodOptions(const moe::Environment& params) {
         // a replica set member could be deleted. Replication can need history older than the last
         // checkpoint to support transactions.
         //
-        // Note: we only use this to defer oplog collection truncation via OplogStones in WT. Non-WT
-        // storage engines will continue to perform regular capped collection handling for the oplog
-        // collection, regardless of this parameter setting.
+        // Note: we only use this to defer oplog collection truncation via OplogTruncateMarkers in
+        // WT. Non-WT storage engines will continue to perform regular capped collection handling
+        // for the oplog collection, regardless of this parameter setting.
         storageGlobalParams.allowOplogTruncation = false;
     }
 
-    if (replSettings.usingReplSets() &&
-        (params.count("security.authorization") &&
-         params["security.authorization"].as<std::string>() == "enabled") &&
-        !serverGlobalParams.startupClusterAuthMode.x509Only() &&
-        serverGlobalParams.keyFile.empty()) {
-        return Status(
-            ErrorCodes::BadValue,
-            str::stream()
-                << "security.keyFile is required when authorization is enabled with replica sets");
-    }
-
     serverGlobalParams.enableMajorityReadConcern = true;
 
     if (storageGlobalParams.engineSetByUser && (storageGlobalParams.engine == "devnull")) {
@@ -543,25 +564,6 @@ Status storeMongodOptions(const moe::Environment& params) {
         }
     }
 
-    if (params.count("replication.oplogSizeMB")) {
-        long long x = params["replication.oplogSizeMB"].as<int>();
-        if (x <= 0) {
-            return Status(ErrorCodes::BadValue,
-                          str::stream() << "bad --oplogSize, arg must be greater than 0,"
-                                           "found: "
-                                        << x);
-        }
-        // note a small size such as x==1 is ok for an arbiter.
-        if (x > 1000 && sizeof(void*) == 4) {
-            StringBuilder sb;
-            sb << "--oplogSize of " << x
-               << "MB is too big for 32 bit version. Use 64 bit build instead.";
-            return Status(ErrorCodes::BadValue, sb.str());
-        }
-        replSettings.setOplogSizeBytes(x * 1024 * 1024);
-        invariant(replSettings.getOplogSizeBytes() > 0);
-    }
-
     if (params.count("storage.oplogMinRetentionHours")) {
         storageGlobalParams.oplogMinRetentionHours.store(
             params["storage.oplogMinRetentionHours"].as<double>());
@@ -593,19 +595,13 @@ Status storeMongodOptions(const moe::Environment& params) {
                 return Status(ErrorCodes::BadValue, sb.str());
             }
         }
-    } else {
-        if (serverGlobalParams.port < 0 || serverGlobalParams.port > 65535) {
-            return Status(ErrorCodes::BadValue, "bad --port number");
-        }
     }
     if (params.count("sharding.clusterRole")) {
         auto clusterRoleParam = params["sharding.clusterRole"].as<std::string>();
-        const bool replicationEnabled = params.count("replication.replSet") ||
-            params.count("replication.replSetName") || params.count("replication.serverless");
         // Force to set up the node as a replica set, unless we're a shard and we're using queryable
         // backup mode.
         if ((clusterRoleParam == "configsvr" || !params.count("storage.queryableBackupMode")) &&
-            !replicationEnabled) {
+            !replSettings.usingReplSets()) {
             return Status(ErrorCodes::BadValue,
                           str::stream() << "Cannot start a " << clusterRoleParam
                                         << " as a standalone server. Please use the option "
@@ -700,4 +696,15 @@ Status storeMongodOptions(const moe::Environment& params) {
     return Status::OK();
 }
 
+namespace {
+std::function<ExitCode(ServiceContext* svcCtx)> _magicRestoreMainFn = nullptr;
+}
+
+void setMagicRestoreMain(std::function<ExitCode(ServiceContext* svcCtx)> magicRestoreMainFn) {
+    _magicRestoreMainFn = magicRestoreMainFn;
+}
+std::function<ExitCode(ServiceContext* svcCtx)> getMagicRestoreMain() {
+    return _magicRestoreMainFn;
+}
+
 }  // namespace mongo
