@@ -42,6 +42,7 @@
 #include "mongo/db/service_context_test_fixture.h"
 #include "mongo/db/session/logical_session_id.h"
 #include "mongo/logv2/log.h"
+#include "mongo/logv2/log_debug.h"
 #include "mongo/stdx/future.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/transport/session.h"
@@ -79,7 +80,7 @@ public:
 class OperationContextTest : public ServiceContextTest {
 public:
     auto makeClient(std::string desc = "OperationContextTest",
-                    transport::SessionHandle session = nullptr) {
+                    std::shared_ptr<transport::Session> session = nullptr) {
         return getServiceContext()->makeClient(desc, session);
     }
 };
@@ -729,74 +730,6 @@ TEST_F(OperationDeadlineTests, DuringWaitMaxTimeExpirationDominatesUntilExpirati
     ASSERT_TRUE(opCtx->getCancellationToken().isCanceled());
 }
 
-TEST_F(OperationDeadlineTests,
-       MaxTimeExpirationIgnoredWhenIgnoringInterruptsExceptReplStateChange) {
-    auto opCtx = client->makeOperationContext();
-    opCtx->setDeadlineByDate(mockClock->now(), ErrorCodes::MaxTimeMSExpired);
-    auto m = MONGO_MAKE_LATCH();
-    stdx::condition_variable cv;
-    stdx::unique_lock<Latch> lk(m);
-    ASSERT_FALSE(opCtx->getCancellationToken().isCanceled());
-    opCtx->setIgnoreInterruptsExceptForReplStateChange(true);
-    // Advance the clock so the MaxTimeMS is hit before the timeout.
-    mockClock->advance(Milliseconds(100));
-    ASSERT_FALSE(
-        opCtx->waitForConditionOrInterruptUntil(cv, lk, mockClock->now(), [] { return false; }));
-    ASSERT_FALSE(opCtx->getCancellationToken().isCanceled());
-}
-
-TEST_F(OperationDeadlineTests,
-       AlreadyExpiredMaxTimeIgnoredWhenIgnoringInterruptsExceptReplStateChange) {
-    auto opCtx = client->makeOperationContext();
-    opCtx->setDeadlineByDate(mockClock->now(), ErrorCodes::MaxTimeMSExpired);
-    auto m = MONGO_MAKE_LATCH();
-    stdx::condition_variable cv;
-    stdx::unique_lock<Latch> lk(m);
-    ASSERT_FALSE(opCtx->getCancellationToken().isCanceled());
-    ASSERT_THROWS_CODE(opCtx->waitForConditionOrInterruptUntil(
-                           cv, lk, mockClock->now() + Seconds(1), [] { return false; }),
-                       DBException,
-                       ErrorCodes::MaxTimeMSExpired);
-
-    ASSERT_EQ(ErrorCodes::MaxTimeMSExpired, opCtx->checkForInterruptNoAssert());
-    ASSERT_TRUE(opCtx->getCancellationToken().isCanceled());
-    opCtx->setIgnoreInterruptsExceptForReplStateChange(true);
-    ASSERT_OK(opCtx->checkForInterruptNoAssert());
-    // Advance the clock so the MaxTimeMS is hit before the timeout.
-    mockClock->advance(Milliseconds(100));
-    ASSERT_FALSE(
-        opCtx->waitForConditionOrInterruptUntil(cv, lk, mockClock->now(), [] { return false; }));
-    ASSERT_TRUE(opCtx->getCancellationToken().isCanceled());
-}
-
-TEST_F(OperationDeadlineTests,
-       MaxTimeRespectedAfterReplStateChangeWhenIgnoringInterruptsExceptReplStateChange) {
-    auto opCtx = client->makeOperationContext();
-    opCtx->setDeadlineByDate(mockClock->now(), ErrorCodes::MaxTimeMSExpired);
-    auto m = MONGO_MAKE_LATCH();
-    stdx::condition_variable cv;
-    stdx::unique_lock<Latch> lk(m);
-    ASSERT_FALSE(opCtx->getCancellationToken().isCanceled());
-    ASSERT_THROWS_CODE(opCtx->waitForConditionOrInterruptUntil(
-                           cv, lk, mockClock->now() + Seconds(1), [] { return false; }),
-                       DBException,
-                       ErrorCodes::MaxTimeMSExpired);
-
-    ASSERT_EQ(ErrorCodes::MaxTimeMSExpired, opCtx->checkForInterruptNoAssert());
-    ASSERT_TRUE(opCtx->getCancellationToken().isCanceled());
-    opCtx->setIgnoreInterruptsExceptForReplStateChange(true);
-    ASSERT_OK(opCtx->checkForInterruptNoAssert());
-    opCtx->markKilled(ErrorCodes::InterruptedDueToReplStateChange);
-    ASSERT_EQ(ErrorCodes::MaxTimeMSExpired, opCtx->checkForInterruptNoAssert());
-    // Advance the clock so the MaxTimeMS is hit before the timeout.
-    mockClock->advance(Milliseconds(100));
-    ASSERT_THROWS_CODE(
-        opCtx->waitForConditionOrInterruptUntil(cv, lk, mockClock->now(), [] { return false; }),
-        DBException,
-        ErrorCodes::MaxTimeMSExpired);
-    ASSERT_TRUE(opCtx->getCancellationToken().isCanceled());
-}
-
 class ThreadedOperationDeadlineTests : public OperationDeadlineTests {
 public:
     using CvPred = std::function<bool()>;
@@ -1158,7 +1091,7 @@ TEST_F(OperationContextTest, TestActiveClientOperationsForClientsWithoutSession)
 
 TEST_F(OperationContextTest, TestActiveClientOperations) {
     transport::TransportLayerMock transportLayer;
-    transport::SessionHandle session = transportLayer.createSession();
+    std::shared_ptr<transport::Session> session = transportLayer.createSession();
 
     auto serviceCtx = getServiceContext();
     auto client = serviceCtx->makeClient("OperationContextTest", session);
