@@ -26,7 +26,6 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#include "mongo/platform/basic.h"
 
 #include "mongo/db/ops/insert.h"
 
@@ -34,20 +33,17 @@
 
 #include "mongo/bson/bson_depth.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/commands/feature_compatibility_version_parser.h"
+#include "mongo/db/mongod_options_storage_gen.h"
 #include "mongo/db/query/dbref.h"
 #include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/update/storage_validation.h"
 #include "mongo/db/vector_clock_mutable.h"
-#include "mongo/db/views/durable_view_catalog.h"
 #include "mongo/util/fail_point.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
-
-using std::string;
-
 namespace {
 
 /**
@@ -86,11 +82,20 @@ Status validateDepth(const BSONObj& obj) {
 
 StatusWith<BSONObj> fixDocumentForInsert(OperationContext* opCtx,
                                          const BSONObj& doc,
+                                         bool bypassEmptyTsReplacement,
                                          bool* containsDotsAndDollarsField) {
     bool validationDisabled = DocumentValidationSettings::get(opCtx).isInternalValidationDisabled();
 
     if (!validationDisabled) {
-        if (doc.objsize() > BSONObjMaxUserSize)
+        // 'gAllowDocumentsGreaterThanMaxUserSize' should only ever be enabled when restoring a node
+        // from a backup. For some restores, we re-insert whole oplog entries from a
+        // source cluster to a destination cluster. Some generated oplog entries may exceed the user
+        // maximum due to entry metadata, and therefore we should skip BSON size validation for
+        // these inserts. Note that we should only skip the size check when inserting oplog entries
+        // into the oplog and not when inserting user documents. The oplog entries to insert have
+        // already been validated for size on the source cluster, and were successfully inserted
+        // into the source oplog.
+        if (doc.objsize() > BSONObjMaxUserSize && !gAllowDocumentsGreaterThanMaxUserSize)
             return StatusWith<BSONObj>(ErrorCodes::BadValue,
                                        str::stream() << "object to insert too large"
                                                      << ". size in bytes: " << doc.objsize()
@@ -122,7 +127,8 @@ StatusWith<BSONObj> fixDocumentForInsert(OperationContext* opCtx,
             }
 
             if (!validationDisabled) {
-                if (e.type() == bsonTimestamp && e.timestampValue() == 0) {
+                if (!bypassEmptyTsReplacement && e.type() == bsonTimestamp &&
+                    e.timestampValue() == 0) {
                     // we replace Timestamp(0,0) at the top level with a correct value
                     // in the fast pass, we just mark that we want to swap
                     hasTimestampToFix = true;
@@ -170,7 +176,8 @@ StatusWith<BSONObj> fixDocumentForInsert(OperationContext* opCtx,
         BSONElement e = i.next();
         if (hadId && e.fieldNameStringData() == "_id") {
             // no-op
-        } else if (e.type() == bsonTimestamp && e.timestampValue() == 0) {
+        } else if (!bypassEmptyTsReplacement && e.type() == bsonTimestamp &&
+                   e.timestampValue() == 0) {
             auto nextTime = VectorClockMutable::get(opCtx)->tickClusterTime(1);
             b.append(e.fieldName(), nextTime.asTimestamp());
         } else {
@@ -211,17 +218,18 @@ Status userAllowedCreateNS(OperationContext* opCtx, const NamespaceString& ns) {
                       str::stream() << "Invalid collection name: " << ns.coll());
     }
 
-    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer && !ns.isOnInternalDb()) {
+    if (serverGlobalParams.clusterRole.exclusivelyHasConfigRole() && !ns.isOnInternalDb()) {
         return Status(ErrorCodes::InvalidNamespace,
                       str::stream()
-                          << "Can't create user databases on a --configsvr instance " << ns);
+                          << "Can't create user databases on a dedicated --configsvr instance "
+                          << ns);
     }
 
     if (ns.isSystemDotProfile()) {
         return Status::OK();
     }
 
-    if (ns.isSystem() && !ns.isLegalClientSystemNS(serverGlobalParams.featureCompatibility)) {
+    if (ns.isSystem() && !ns.isLegalClientSystemNS()) {
         return Status(ErrorCodes::InvalidNamespace,
                       str::stream() << "Invalid system namespace: " << ns);
     }
@@ -243,7 +251,7 @@ Status userAllowedCreateNS(OperationContext* opCtx, const NamespaceString& ns) {
             return Status::OK();
         }
 
-        if (ns.isConfigDB() && ns.isLegalClientSystemNS(serverGlobalParams.featureCompatibility)) {
+        if (ns.isConfigDB() && ns.isLegalClientSystemNS()) {
             return Status::OK();
         }
 
