@@ -294,15 +294,6 @@ public:
      */
     static void checkValueTypeOrMissing(Value v, StringData fieldName, BSONType expectedType);
 
-    /**
-     * Extracts the resume token from the given spec. If a 'startAtOperationTime' is specified,
-     * returns the equivalent high-watermark token. This method should only ever be called on a spec
-     * where one of 'resumeAfter', 'startAfter', or 'startAtOperationTime' is populated.
-     */
-    static ResumeTokenData resolveResumeTokenFromSpec(
-        const boost::intrusive_ptr<ExpressionContext>& expCtx,
-        const DocumentSourceChangeStreamSpec& spec);
-
     /**
      * For a change stream with no resume information supplied by the user, returns the clusterTime
      * at which the new stream should begin scanning the oplog.
