@@ -70,11 +70,11 @@ repl::OplogEntry makeOplogEntry(
     boost::optional<repl::RetryImageEnum> needsRetryImage = boost::none) {
     return {
         repl::DurableOplogEntry(opTime,                           // optime
-                                boost::none,                      // hash
                                 opType,                           // opType
                                 nss,                              // namespace
                                 uuid,                             // uuid
                                 boost::none,                      // fromMigrate
+                                boost::none,                      // checkExistenceForDiffInsert
                                 repl::OplogEntry::kOplogVersion,  // version
                                 oField,                           // o
                                 boost::none,                      // o2
@@ -130,16 +130,17 @@ TEST_F(FindAndModifyImageLookupTest, NoopWhenEntryDoesNotHaveNeedsRetryImageFiel
     const auto stmtId = 1;
     const auto opTime = repl::OpTime(Timestamp(2, 1), 1);
     const auto preImageOpTime = repl::OpTime(Timestamp(1, 1), 1);
-    const auto oplogEntryBson = makeOplogEntry(opTime,
-                                               repl::OpTypeEnum::kNoop,
-                                               NamespaceString("test.foo"),
-                                               UUID::gen(),
-                                               BSON("a" << 1),
-                                               sessionInfo,
-                                               {stmtId},
-                                               preImageOpTime)
-                                    .getEntry()
-                                    .toBSON();
+    const auto oplogEntryBson =
+        makeOplogEntry(opTime,
+                       repl::OpTypeEnum::kNoop,
+                       NamespaceString::createNamespaceString_forTest("test.foo"),
+                       UUID::gen(),
+                       BSON("a" << 1),
+                       sessionInfo,
+                       {stmtId},
+                       preImageOpTime)
+            .getEntry()
+            .toBSON();
     auto mock = DocumentSourceMock::createForTest(Document(oplogEntryBson), getExpCtx());
     imageLookup->setSource(mock.get());
     // Mock out the foreign collection.
@@ -164,19 +165,20 @@ TEST_F(FindAndModifyImageLookupTest, ShouldNotForgeImageEntryWhenImageDocMissing
     sessionInfo.setTxnNumber(1);
     const auto stmtId = 1;
     const auto opTime = repl::OpTime(Timestamp(2, 1), 1);
-    const auto oplogEntryBson = makeOplogEntry(opTime,
-                                               repl::OpTypeEnum::kUpdate,
-                                               NamespaceString("test.foo"),
-                                               UUID::gen(),
-                                               BSON("a" << 1),
-                                               sessionInfo,
-                                               {stmtId},
-                                               boost::none /* preImageOpTime */,
-                                               boost::none /* postImageOpTime */,
-                                               repl::RetryImageEnum::kPreImage)
-                                    .getEntry()
-                                    .toBSON();
-    auto mock = DocumentSourceMock::createForTest(Document(oplogEntryBson), getExpCtx());
+    const auto oplogEntryDoc =
+        Document(makeOplogEntry(opTime,
+                                repl::OpTypeEnum::kUpdate,
+                                NamespaceString::createNamespaceString_forTest("test.foo"),
+                                UUID::gen(),
+                                BSON("a" << 1),
+                                sessionInfo,
+                                {stmtId},
+                                boost::none /* preImageOpTime */,
+                                boost::none /* postImageOpTime */,
+                                repl::RetryImageEnum::kPreImage)
+                     .getEntry()
+                     .toBSON());
+    auto mock = DocumentSourceMock::createForTest(oplogEntryDoc, getExpCtx());
     imageLookup->setSource(mock.get());
 
     // Mock out the foreign collection.
@@ -185,8 +187,11 @@ TEST_F(FindAndModifyImageLookupTest, ShouldNotForgeImageEntryWhenImageDocMissing
 
     auto next = imageLookup->getNext();
     ASSERT_TRUE(next.isAdvanced());
-    Document expected = Document(oplogEntryBson);
-    ASSERT_DOCUMENT_EQ(next.releaseDocument(), expected);
+    // The needsRetryImage field should have been stripped even though we are not forging an image
+    // entry.
+    MutableDocument expected{oplogEntryDoc};
+    expected.remove(repl::OplogEntryBase::kNeedsRetryImageFieldName);
+    ASSERT_DOCUMENT_EQ(next.releaseDocument(), expected.freeze());
 
     ASSERT_TRUE(imageLookup->getNext().isEOF());
     ASSERT_TRUE(imageLookup->getNext().isEOF());
@@ -202,19 +207,20 @@ TEST_F(FindAndModifyImageLookupTest, ShouldNotForgeImageEntryWhenImageDocHasDiff
     const auto stmtId = 1;
     const auto ts = Timestamp(2, 1);
     const auto opTime = repl::OpTime(ts, 1);
-    const auto oplogEntryBson = makeOplogEntry(opTime,
-                                               repl::OpTypeEnum::kUpdate,
-                                               NamespaceString("test.foo"),
-                                               UUID::gen(),
-                                               BSON("a" << 1),
-                                               sessionInfo,
-                                               {stmtId},
-                                               boost::none /* preImageOpTime */,
-                                               boost::none /* postImageOpTime */,
-                                               repl::RetryImageEnum::kPreImage)
-                                    .getEntry()
-                                    .toBSON();
-    auto mock = DocumentSourceMock::createForTest(Document(oplogEntryBson), getExpCtx());
+    const auto oplogEntryDoc =
+        Document(makeOplogEntry(opTime,
+                                repl::OpTypeEnum::kUpdate,
+                                NamespaceString::createNamespaceString_forTest("test.foo"),
+                                UUID::gen(),
+                                BSON("a" << 1),
+                                sessionInfo,
+                                {stmtId},
+                                boost::none /* preImageOpTime */,
+                                boost::none /* postImageOpTime */,
+                                repl::RetryImageEnum::kPreImage)
+                     .getEntry()
+                     .toBSON());
+    auto mock = DocumentSourceMock::createForTest(oplogEntryDoc, getExpCtx());
     imageLookup->setSource(mock.get());
 
     // Create an 'ImageEntry' with a higher 'txnNumber'.
@@ -231,8 +237,11 @@ TEST_F(FindAndModifyImageLookupTest, ShouldNotForgeImageEntryWhenImageDocHasDiff
 
     auto next = imageLookup->getNext();
     ASSERT_TRUE(next.isAdvanced());
-    Document expected = Document(oplogEntryBson);
-    ASSERT_DOCUMENT_EQ(next.releaseDocument(), expected);
+    // The needsRetryImage field should have been stripped even though we are not forging an image
+    // entry.
+    MutableDocument expected{oplogEntryDoc};
+    expected.remove(repl::OplogEntryBase::kNeedsRetryImageFieldName);
+    ASSERT_DOCUMENT_EQ(next.releaseDocument(), expected.freeze());
 
     ASSERT_TRUE(imageLookup->getNext().isEOF());
     ASSERT_TRUE(imageLookup->getNext().isEOF());
@@ -255,7 +264,7 @@ TEST_F(FindAndModifyImageLookupTest, ShouldForgeImageEntryWhenMatchingImageDocIs
         sessionInfo.setTxnNumber(txnNum);
         const auto ts = Timestamp(2, 1);
         const auto opTime = repl::OpTime(ts, 1);
-        const auto nss = NamespaceString("test.foo");
+        const auto nss = NamespaceString::createNamespaceString_forTest("test.foo");
         const auto uuid = UUID::gen();
 
         // Define a findAndModify/update oplog entry with the 'needsRetryImage' field set.
@@ -343,7 +352,7 @@ TEST_F(FindAndModifyImageLookupTest, ShouldForgeImageEntryWhenMatchingImageDocIs
         const auto applyOpsOpTime = repl::OpTime(applyOpsTs, 1);
         const auto commitTxnTs = Timestamp(3, 1);
         const auto commitTxnTsFieldName = CommitTransactionOplogObject::kCommitTimestampFieldName;
-        const auto nss = NamespaceString("test.foo");
+        const auto nss = NamespaceString::createNamespaceString_forTest("test.foo");
         const auto uuid = UUID::gen();
 
         // Define an applyOps oplog entry containing a findAndModify/update operation entry with
@@ -437,5 +446,86 @@ TEST_F(FindAndModifyImageLookupTest, ShouldForgeImageEntryWhenMatchingImageDocIs
         ASSERT_TRUE(imageLookup->getNext().isEOF());
     }
 }
+TEST_F(FindAndModifyImageLookupTest,
+       ShouldNotForgeImageEntryWhenMatchingImageDocIsNotFoundApplyOpsOp) {
+    auto imageLookup = DocumentSourceFindAndModifyImageLookup::create(
+        getExpCtx(), true /* includeCommitTransactionTimestamp */);
+    const auto sessionId = makeLogicalSessionIdWithTxnNumberAndUUIDForTest();
+    const auto txnNum = 1LL;
+    OperationSessionInfo sessionInfo;
+    sessionInfo.setSessionId(sessionId);
+    sessionInfo.setTxnNumber(txnNum);
+    const auto stmtId = 1;
+    const auto applyOpsTs = Timestamp(2, 1);
+    const auto applyOpsOpTime = repl::OpTime(applyOpsTs, 1);
+    const auto commitTxnTs = Timestamp(3, 1);
+    const auto commitTxnTsFieldName = CommitTransactionOplogObject::kCommitTimestampFieldName;
+    const auto nss = NamespaceString::createNamespaceString_forTest("test.foo");
+    const auto uuid = UUID::gen();
+
+    // Define an applyOps oplog entry containing a findAndModify/update operation entry with
+    // the 'needsRetryImage' field set.
+    auto insertOp = repl::MutableOplogEntry::makeInsertOperation(
+        nss, uuid, BSON("_id" << 0 << "a" << 0), BSON("_id" << 0));
+    auto updateOp = repl::MutableOplogEntry::makeUpdateOperation(
+        nss, uuid, BSON("$set" << BSON("a" << 1)), BSON("_id" << 1));
+    updateOp.setStatementIds({{stmtId}});
+    updateOp.setNeedsRetryImage(repl::RetryImageEnum::kPreImage);
+    BSONObjBuilder applyOpsBuilder;
+    applyOpsBuilder.append("applyOps", BSON_ARRAY(insertOp.toBSON() << updateOp.toBSON()));
+    auto oplogEntryUUID = UUID::gen();
+    auto oplogEntryBson = makeOplogEntry(applyOpsOpTime,
+                                         repl::OpTypeEnum::kCommand,
+                                         {},
+                                         oplogEntryUUID,
+                                         applyOpsBuilder.obj(),
+                                         sessionInfo,
+                                         {})
+                              .getEntry()
+                              .toBSON()
+                              .addFields(BSON(commitTxnTsFieldName << commitTxnTs));
+
+    auto mock = DocumentSourceMock::createForTest(Document(oplogEntryBson), getExpCtx());
+    imageLookup->setSource(mock.get());
+
+    // Mock out the foreign collection.
+    getExpCtx()->mongoProcessInterface =
+        std::make_unique<MockMongoInterface>(std::vector<Document>{Document{}});
+
+    // The next doc should be the doc for original applyOps oplog entry but the
+    // findAndModify/update operation entry should have 'needsRetryImage' field removed.
+    auto next = imageLookup->getNext();
+    const auto downConvertedOplogEntryBson = next.releaseDocument().toBson();
+
+    auto updateOpWithoutNeedsRetryImage = repl::MutableOplogEntry::makeUpdateOperation(
+        nss, uuid, BSON("$set" << BSON("a" << 1)), BSON("_id" << 1));
+    updateOpWithoutNeedsRetryImage.setStatementIds({{stmtId}});
+    BSONObjBuilder applyOpsWithoutNeedsRetryImageBuilder;
+    applyOpsWithoutNeedsRetryImageBuilder.append(
+        "applyOps", BSON_ARRAY(insertOp.toBSON() << updateOpWithoutNeedsRetryImage.toBSON()));
+    auto expectedOplogEntryBson = makeOplogEntry(applyOpsOpTime,
+                                                 repl::OpTypeEnum::kCommand,
+                                                 {},
+                                                 oplogEntryUUID,
+                                                 applyOpsWithoutNeedsRetryImageBuilder.obj(),
+                                                 sessionInfo,
+                                                 {})
+                                      .getEntry()
+                                      .toBSON()
+                                      .addFields(BSON(commitTxnTsFieldName << commitTxnTs));
+
+    ASSERT_BSONOBJ_EQ(expectedOplogEntryBson, downConvertedOplogEntryBson);
+
+    auto applyOpsInfo = repl::ApplyOpsCommandInfo::parse(
+        downConvertedOplogEntryBson.getObjectField(repl::OplogEntry::kObjectFieldName));
+    auto operationDocs = applyOpsInfo.getOperations();
+    ASSERT_EQ(operationDocs.size(), 2U);
+
+    ASSERT_BSONOBJ_EQ(operationDocs[0], insertOp.toBSON());
+
+    ASSERT_TRUE(imageLookup->getNext().isEOF());
+    ASSERT_TRUE(imageLookup->getNext().isEOF());
+    ASSERT_TRUE(imageLookup->getNext().isEOF());
+}
 }  // namespace
 }  // namespace mongo
