@@ -46,6 +46,7 @@
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/sort_reorder_helpers.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/stats/counters.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/logv2/log.h"
 
@@ -70,7 +71,8 @@ NamespaceString parseGraphLookupFromAndResolveNamespace(const BSONElement& elem,
             elem.type() == String || elem.type() == Object);
 
     if (elem.type() == BSONType::String) {
-        NamespaceString fromNss(defaultDb, elem.valueStringData());
+        NamespaceString fromNss(
+            NamespaceStringUtil::parseNamespaceFromRequest(defaultDb, elem.valueStringData()));
         uassert(ErrorCodes::InvalidNamespace,
                 str::stream() << "invalid $graphLookup namespace: " << fromNss.ns(),
                 fromNss.isValid());
@@ -78,14 +80,17 @@ NamespaceString parseGraphLookupFromAndResolveNamespace(const BSONElement& elem,
     }
 
     // Valdate the db and coll names.
-    auto spec =
-        NamespaceSpec::parse(IDLParserContext{elem.fieldNameStringData()}, elem.embeddedObject());
-    // TODO SERVER-62491 Use system tenantId to construct nss.
-    auto nss = NamespaceString(spec.getDb().value_or(""), spec.getColl().value_or(""));
+    auto spec = NamespaceSpec::parse(
+        IDLParserContext{elem.fieldNameStringData(), false /* apiStrict */, defaultDb.tenantId()},
+        elem.embeddedObject());
+
+    auto nss = NamespaceStringUtil::parseNamespaceFromRequest(spec.getDb().value_or(DatabaseName()),
+                                                              spec.getColl().value_or(""));
+
     uassert(ErrorCodes::FailedToParse,
             str::stream()
                 << "$graphLookup with syntax {from: {db:<>, coll:<>},..} is not supported for db: "
-                << nss.db() << " and coll: " << nss.coll(),
+                << nss.dbName().toStringForErrorMsg() << " and coll: " << nss.coll(),
             nss == NamespaceString::kTenantMigrationOplogView);
     return nss;
 }
@@ -478,7 +483,7 @@ void DocumentSourceGraphLookUp::performSearch() {
 
     // If _startWith evaluates to an array, treat each value as a separate starting point.
     if (startingValue.isArray()) {
-        for (auto value : startingValue.getArray()) {
+        for (const auto& value : startingValue.getArray()) {
             _frontier.insert(value);
             _frontierUsageBytes += value.getApproximateSize();
         }
@@ -494,7 +499,7 @@ void DocumentSourceGraphLookUp::performSearch() {
         // throw a custom exception.
         if (auto staleInfo = ex.extraInfo<StaleConfigInfo>(); staleInfo &&
             staleInfo->getVersionWanted() &&
-            staleInfo->getVersionWanted() != ChunkVersion::UNSHARDED()) {
+            staleInfo->getVersionWanted() != ShardVersion::UNSHARDED()) {
             uassert(3904801,
                     "Cannot run $graphLookup with a sharded foreign collection in a transaction",
                     foreignShardedGraphLookupAllowed());
@@ -576,47 +581,57 @@ void DocumentSourceGraphLookUp::checkMemoryUsage() {
     _cache.evictDownTo(_maxMemoryUsageBytes - _frontierUsageBytes - _visitedUsageBytes);
 }
 
-void DocumentSourceGraphLookUp::serializeToArray(
-    std::vector<Value>& array, boost::optional<ExplainOptions::Verbosity> explain) const {
+void DocumentSourceGraphLookUp::serializeToArray(std::vector<Value>& array,
+                                                 const SerializationOptions& opts) const {
     // Do not include tenantId in serialized 'from' namespace.
     auto fromValue = (pExpCtx->ns.db() == _from.db())
-        ? Value(_from.coll())
-        : Value(Document{{"db", _from.dbName().db()}, {"coll", _from.coll()}});
+        ? Value(opts.serializeIdentifier(_from.coll()))
+        : Value(Document{{"db", opts.serializeIdentifier(_from.dbName().db())},
+                         {"coll", opts.serializeIdentifier(_from.coll())}});
 
     // Serialize default options.
-    MutableDocument spec(DOC("from" << fromValue << "as" << _as.fullPath() << "connectToField"
-                                    << _connectToField.fullPath() << "connectFromField"
-                                    << _connectFromField.fullPath() << "startWith"
-                                    << _startWith->serialize(false)));
+    MutableDocument spec(DOC("from" << fromValue << "as" << opts.serializeFieldPath(_as)
+                                    << "connectToField" << opts.serializeFieldPath(_connectToField)
+                                    << "connectFromField"
+                                    << opts.serializeFieldPath(_connectFromField) << "startWith"
+                                    << _startWith->serialize(opts)));
 
     // depthField is optional; serialize it if it was specified.
     if (_depthField) {
-        spec["depthField"] = Value(_depthField->fullPath());
+        spec["depthField"] = Value(opts.serializeFieldPath(*_depthField));
     }
 
     if (_maxDepth) {
-        spec["maxDepth"] = Value(*_maxDepth);
+        spec["maxDepth"] = Value(opts.serializeLiteral(*_maxDepth));
     }
 
     if (_additionalFilter) {
-        spec["restrictSearchWithMatch"] = Value(*_additionalFilter);
+        if (opts.transformIdentifiers ||
+            opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
+            auto matchExpr =
+                uassertStatusOK(MatchExpressionParser::parse(*_additionalFilter, pExpCtx));
+            spec["restrictSearchWithMatch"] = Value(matchExpr->serialize(opts));
+        } else {
+            spec["restrictSearchWithMatch"] = Value(*_additionalFilter);
+        }
     }
 
     // If we are explaining, include an absorbed $unwind inside the $graphLookup specification.
-    if (_unwind && explain) {
+    if (_unwind && opts.verbosity) {
         const boost::optional<FieldPath> indexPath = (*_unwind)->indexPath();
         spec["unwinding"] =
             Value(DOC("preserveNullAndEmptyArrays"
-                      << (*_unwind)->preserveNullAndEmptyArrays() << "includeArrayIndex"
-                      << (indexPath ? Value((*indexPath).fullPath()) : Value())));
+                      << opts.serializeLiteral((*_unwind)->preserveNullAndEmptyArrays())
+                      << "includeArrayIndex"
+                      << (indexPath ? Value(opts.serializeFieldPath(*indexPath)) : Value())));
     }
 
     array.push_back(Value(DOC(getSourceName() << spec.freeze())));
 
     // If we are not explaining, the output of this method must be parseable, so serialize our
     // $unwind into a separate stage.
-    if (_unwind && !explain) {
-        (*_unwind)->serializeToArray(array);
+    if (_unwind && !opts.verbosity) {
+        (*_unwind)->serializeToArray(array, opts);
     }
 }
 
@@ -628,6 +643,10 @@ void DocumentSourceGraphLookUp::reattachToOperationContext(OperationContext* opC
     _fromExpCtx->opCtx = opCtx;
 }
 
+bool DocumentSourceGraphLookUp::validateOperationContext(const OperationContext* opCtx) const {
+    return getContext()->opCtx == opCtx && _fromExpCtx->opCtx == opCtx;
+}
+
 DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     NamespaceString from,
@@ -654,6 +673,10 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
       _unwind(unwindSrc),
       _variables(expCtx->variables),
       _variablesParseState(expCtx->variablesParseState.copyWith(_variables.useIdGenerator())) {
+    if (!_from.isOnInternalDb()) {
+        globalOpCounters.gotNestedAggregate();
+    }
+
     const auto& resolvedNamespace = pExpCtx->getResolvedNamespace(_from);
     _fromExpCtx = pExpCtx->copyForSubPipeline(resolvedNamespace.ns, resolvedNamespace.uuid);
     _fromExpCtx->inLookup = true;
@@ -668,10 +691,7 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
 DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
     const DocumentSourceGraphLookUp& original,
     const boost::intrusive_ptr<ExpressionContext>& newExpCtx)
-    : DocumentSource(
-          kStageName,
-          newExpCtx ? newExpCtx
-                    : original.pExpCtx->copyWith(original.pExpCtx->ns, original.pExpCtx->uuid)),
+    : DocumentSource(kStageName, newExpCtx),
       _from(original._from),
       _as(original._as),
       _connectFromField(original._connectFromField),
@@ -690,7 +710,8 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
       _variables(original._variables),
       _variablesParseState(original._variablesParseState.copyWith(_variables.useIdGenerator())) {
     if (original._unwind) {
-        _unwind = static_cast<DocumentSourceUnwind*>(original._unwind.value()->clone().get());
+        _unwind =
+            static_cast<DocumentSourceUnwind*>(original._unwind.value()->clone(pExpCtx).get());
     }
 }
 
