@@ -64,8 +64,8 @@ public:
                                 ChangeStreamRequirement::kDenylist);
     }
 
-    Value serialize(
-        boost::optional<ExplainOptions::Verbosity> explain = boost::none) const override;
+
+    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final override;
 
     boost::optional<DistributedPlanLogic> distributedPlanLogic() override {
         return boost::none;
