@@ -68,11 +68,10 @@ const char* DocumentSourceMatch::getSourceName() const {
     return kStageName.rawData();
 }
 
-Value DocumentSourceMatch::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
-    if (explain) {
-        BSONObjBuilder builder;
-        _expression->serialize(&builder);
-        return Value(DOC(getSourceName() << Document(builder.obj())));
+Value DocumentSourceMatch::serialize(const SerializationOptions& opts) const {
+    if (opts.verbosity || opts.transformIdentifiers ||
+        opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
+        return Value(DOC(getSourceName() << Document(_expression->serialize(opts))));
     }
     return Value(DOC(getSourceName() << Document(getQuery())));
 }
@@ -94,11 +93,12 @@ DocumentSource::GetNextResult DocumentSourceMatch::doGetNext() {
     auto nextInput = pSource->getNext();
     for (; nextInput.isAdvanced(); nextInput = pSource->getNext()) {
         // MatchExpression only takes BSON documents, so we have to make one. As an optimization,
-        // only serialize the fields we need to do the match.
+        // only serialize the fields we need to do the match. Specify BSONObj::LargeSizeTrait so
+        // that matching against a large document mid-pipeline does not throw a BSON max-size error.
         BSONObj toMatch = _dependencies.needWholeDocument
-            ? nextInput.getDocument().toBson()
-            : document_path_support::documentToBsonWithPaths(nextInput.getDocument(),
-                                                             _dependencies.fields);
+            ? nextInput.getDocument().toBson<BSONObj::LargeSizeTrait>()
+            : document_path_support::documentToBsonWithPaths<BSONObj::LargeSizeTrait>(
+                  nextInput.getDocument(), _dependencies.fields);
 
         if (_expression->matchesBSON(toMatch)) {
             return nextInput;
@@ -267,7 +267,6 @@ Document redactSafePortionDollarOps(BSONObj expr) {
             }
 
             // These are never allowed
-            case PathAcceptingKeyword::ENCRYPTED_BETWEEN:
             case PathAcceptingKeyword::EXISTS:
             case PathAcceptingKeyword::GEO_INTERSECTS:
             case PathAcceptingKeyword::GEO_NEAR:
@@ -276,6 +275,7 @@ Document redactSafePortionDollarOps(BSONObj expr) {
             case PathAcceptingKeyword::INTERNAL_EXPR_GTE:
             case PathAcceptingKeyword::INTERNAL_EXPR_LT:
             case PathAcceptingKeyword::INTERNAL_EXPR_LTE:
+            case PathAcceptingKeyword::INTERNAL_EQ_HASHED_KEY:
             case PathAcceptingKeyword::INTERNAL_SCHEMA_ALL_ELEM_MATCH_FROM_INDEX:
             case PathAcceptingKeyword::INTERNAL_SCHEMA_BIN_DATA_ENCRYPTED_TYPE:
             case PathAcceptingKeyword::INTERNAL_SCHEMA_BIN_DATA_SUBTYPE:
@@ -450,15 +450,11 @@ DocumentSourceMatch::splitSourceByFunc(const OrderedPathSet& fields,
     // the corresponding BSONObj may not exist. Therefore, we take each of these expressions,
     // serialize them, and then re-parse them, constructing new BSON that is owned by the
     // DocumentSourceMatch.
-    BSONObjBuilder firstBob;
-    newExpr.first->serialize(&firstBob);
-    auto firstMatch = DocumentSourceMatch::create(firstBob.obj(), pExpCtx);
+    auto firstMatch = DocumentSourceMatch::create(newExpr.first->serialize(), pExpCtx);
 
     intrusive_ptr<DocumentSourceMatch> secondMatch;
     if (newExpr.second) {
-        BSONObjBuilder secondBob;
-        newExpr.second->serialize(&secondBob);
-        secondMatch = DocumentSourceMatch::create(secondBob.obj(), pExpCtx);
+        secondMatch = DocumentSourceMatch::create(newExpr.second->serialize(), pExpCtx);
     }
 
     return {std::move(firstMatch), std::move(secondMatch)};
@@ -470,8 +466,9 @@ boost::intrusive_ptr<DocumentSourceMatch> DocumentSourceMatch::descendMatchOnPat
     const intrusive_ptr<ExpressionContext>& expCtx) {
     expression::mapOver(matchExpr, [&descendOn](MatchExpression* node, std::string path) -> void {
         // Cannot call this method on a $match including a $elemMatch.
-        invariant(node->matchType() != MatchExpression::ELEM_MATCH_OBJECT &&
-                  node->matchType() != MatchExpression::ELEM_MATCH_VALUE);
+        tassert(9224700,
+                "The given match expression has a node that represents a partial path.",
+                !MatchExpression::isInternalNodeWithPath(node->matchType()));
         // Only leaf and array match expressions have a path.
         if (node->getCategory() != MatchExpression::MatchCategory::kLeaf &&
             node->getCategory() != MatchExpression::MatchCategory::kArrayMatching) {
@@ -479,7 +476,10 @@ boost::intrusive_ptr<DocumentSourceMatch> DocumentSourceMatch::descendMatchOnPat
         }
 
         auto leafPath = node->path();
-        invariant(expression::isPathPrefixOf(descendOn, leafPath));
+        tassert(9224701,
+                str::stream() << "Expected '" << redact(descendOn) << "' to be a prefix of '"
+                              << redact(leafPath) << "', but it is not.",
+                expression::isPathPrefixOf(descendOn, leafPath));
 
         auto newPath = leafPath.substr(descendOn.size() + 1);
         if (node->getCategory() == MatchExpression::MatchCategory::kLeaf) {
@@ -491,9 +491,7 @@ boost::intrusive_ptr<DocumentSourceMatch> DocumentSourceMatch::descendMatchOnPat
         }
     });
 
-    BSONObjBuilder query;
-    matchExpr->serialize(&query);
-    return new DocumentSourceMatch(query.obj(), expCtx);
+    return new DocumentSourceMatch(matchExpr->serialize(), expCtx);
 }
 
 std::pair<boost::intrusive_ptr<DocumentSourceMatch>, boost::intrusive_ptr<DocumentSourceMatch>>
