@@ -48,8 +48,9 @@ public:
 
     // A descriptor for a merge strategy. Holds a merge strategy function and a set of actions the
     // client should be authorized to perform in order to be able to execute a merge operation using
-    // this merge strategy. If a 'BatchTransform' function is provided, it will be called when
-    // constructing a batch object to transform updates.
+    // this merge strategy. Additionally holds a 'BatchedCommandGenerator' that will initialize a
+    // BatchedWriteRequest for executing the batch write. If a 'BatchTransform' function is
+    // provided, it will be called when constructing a batch object to transform updates.
     struct MergeStrategyDescriptor {
         using WhenMatched = MergeWhenMatchedModeEnum;
         using WhenNotMatched = MergeWhenNotMatchedModeEnum;
@@ -62,13 +63,19 @@ public:
                                                  const WriteConcernOptions&,
                                                  boost::optional<OID>,
                                                  BatchedObjects&&,
+                                                 BatchedCommandRequest&&,
                                                  UpsertType upsert)>;
 
+        // A function object that will be invoked to generate a BatchedCommandRequest.
+        using BatchedCommandGenerator = std::function<BatchedCommandRequest(
+            const boost::intrusive_ptr<ExpressionContext>&, const NamespaceString&)>;
+
         MergeMode mode;
         ActionSet actions;
         MergeStrategy strategy;
         BatchTransform transform;
         UpsertType upsertType;
+        BatchedCommandGenerator batchedCommandGenerator;
     };
 
     /**
@@ -128,7 +135,7 @@ public:
 
     boost::optional<DistributedPlanLogic> distributedPlanLogic() final;
 
-    Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
+    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final override;
 
     /**
      * Creates a new $merge stage from the given arguments.
@@ -141,7 +148,7 @@ public:
         boost::optional<BSONObj> letVariables,
         boost::optional<std::vector<BSONObj>> pipeline,
         std::set<FieldPath> mergeOnFields,
-        boost::optional<ChunkVersion> targetCollectionVersion);
+        boost::optional<ChunkVersion> targetCollectionPlacementVersion);
 
     /**
      * Parses a $merge stage from the user-supplied BSON.
@@ -155,13 +162,13 @@ public:
 
     void initialize() override {
         // This implies that the stage will soon start to write, so it's safe to verify the target
-        // collection version. This is done here instead of parse time since it requires that locks
-        // are not held.
-        if (!pExpCtx->inMongos && _targetCollectionVersion) {
-            // If mongos has sent us a target shard version, we need to be sure we are prepared to
-            // act as a router which is at least as recent as that mongos.
+        // collection placement version. This is done here instead of parse time since it requires
+        // that locks are not held.
+        if (!pExpCtx->inMongos && _targetCollectionPlacementVersion) {
+            // If mongos has sent us a target placement version, we need to be sure we are prepared
+            // to act as a router which is at least as recent as that mongos.
             pExpCtx->mongoProcessInterface->checkRoutingInfoEpochOrThrow(
-                pExpCtx, getOutputNs(), *_targetCollectionVersion);
+                pExpCtx, getOutputNs(), *_targetCollectionPlacementVersion);
         }
     }
 
@@ -178,7 +185,7 @@ public:
 private:
     /**
      * Builds a new $merge stage which will merge all documents into 'outputNs'. If
-     * 'targetCollectionVersion' is provided then processing will stop with an error if the
+     * 'targetCollectionPlacementVersion' is provided then processing will stop with an error if the
      * collection's epoch changes during the course of execution. This is used as a mechanism to
      * prevent the shard key from changing.
      */
@@ -188,14 +195,15 @@ private:
                         boost::optional<BSONObj> letVariables,
                         boost::optional<std::vector<BSONObj>> pipeline,
                         std::set<FieldPath> mergeOnFields,
-                        boost::optional<ChunkVersion> targetCollectionVersion);
+                        boost::optional<ChunkVersion> targetCollectionPlacementVersion);
 
     /**
      * Creates an UpdateModification object from the given 'doc' to be used with the batched update.
      */
     auto makeBatchUpdateModification(const Document& doc) const {
         return _pipeline ? write_ops::UpdateModification(*_pipeline)
-                         : write_ops::UpdateModification::parseFromClassicUpdate(doc.toBson());
+                         : write_ops::UpdateModification(
+                               doc.toBson(), write_ops::UpdateModification::ReplacementTag{});
     }
 
     /**
@@ -218,13 +226,15 @@ private:
         return bob.obj();
     }
 
-    void spill(BatchedObjects&& batch) override;
+    void spill(BatchedCommandRequest&& bcr, BatchedObjects&& batch) override;
+
+    BatchedCommandRequest initializeBatchedWriteRequest() const override;
 
     void waitWhileFailPointEnabled() override;
 
     std::pair<BatchObject, int> makeBatchObject(Document&& doc) const override;
 
-    boost::optional<ChunkVersion> _targetCollectionVersion;
+    boost::optional<ChunkVersion> _targetCollectionPlacementVersion;
 
     // A merge descriptor contains a merge strategy function describing how to merge two
     // collections, as well as some other metadata needed to perform the merge operation. This is
