@@ -37,6 +37,8 @@
 #include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/pipeline/document_path_support.h"
+#include "mongo/db/timeseries/catalog_helper.h"
+#include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/destructor_guard.h"
@@ -61,7 +63,13 @@ DocumentSourceOut::~DocumentSourceOut() {
         // Make sure we drop the temp collection if anything goes wrong. Errors are ignored
         // here because nothing can be done about them. Additionally, if this fails and the
         // collection is left behind, it will be cleaned up next time the server is started.
-        if (_tempNs.size()) {
+
+        // If creating a time-series collection, we must drop the "real" buckets collection, if
+        // anything goes wrong creating the view.
+
+        // If creating a time-series collection, '_tempNs' is translated to include the
+        // "system.buckets" prefix.
+        if (_tempNs.size() || (_timeseries && !_timeseriesStateConsistent)) {
             auto cleanupClient =
                 pExpCtx->opCtx->getServiceContext()->makeClient("$out_replace_coll_cleanup");
             AlternativeClientRegion acr(cleanupClient);
@@ -71,51 +79,102 @@ DocumentSourceOut::~DocumentSourceOut() {
 
             DocumentSourceWriteBlock writeBlock(cleanupOpCtx.get());
 
-            pExpCtx->mongoProcessInterface->dropCollection(cleanupOpCtx.get(), _tempNs);
+            auto deleteNs = _tempNs.size() ? _tempNs : makeBucketNsIfTimeseries(getOutputNs());
+            pExpCtx->mongoProcessInterface->dropCollection(cleanupOpCtx.get(), deleteNs);
         });
 }
 
-NamespaceString DocumentSourceOut::parseNsFromElem(const BSONElement& spec,
-                                                   const DatabaseName& defaultDB) {
+DocumentSourceOutSpec DocumentSourceOut::parseOutSpecAndResolveTargetNamespace(
+    const BSONElement& spec, const DatabaseName& defaultDB) {
+    DocumentSourceOutSpec outSpec;
     if (spec.type() == BSONType::String) {
-        return NamespaceString(defaultDB, spec.valueStringData());
+        outSpec.setColl(spec.valueStringData());
+        outSpec.setDb(defaultDB.db());
     } else if (spec.type() == BSONType::Object) {
-        auto nsObj = spec.Obj();
-        uassert(16994,
-                str::stream() << "If an object is passed to " << kStageName
-                              << " it must have exactly 2 fields: 'db' and 'coll'",
-                nsObj.nFields() == 2 && nsObj.hasField("coll") && nsObj.hasField("db"));
-        return NamespaceString(defaultDB.tenantId(), nsObj["db"].String(), nsObj["coll"].String());
+        outSpec = mongo::DocumentSourceOutSpec::parse(IDLParserContext(kStageName),
+                                                      spec.embeddedObject());
     } else {
         uassert(16990,
                 "{} only supports a string or object argument, but found {}"_format(
                     kStageName, typeName(spec.type())),
                 spec.type() == BSONType::String);
     }
-    MONGO_UNREACHABLE;
+
+    return outSpec;
+}
+
+NamespaceString DocumentSourceOut::makeBucketNsIfTimeseries(const NamespaceString& ns) {
+    return _timeseries ? ns.makeTimeseriesBucketsNamespace() : ns;
 }
 
 std::unique_ptr<DocumentSourceOut::LiteParsed> DocumentSourceOut::LiteParsed::parse(
     const NamespaceString& nss, const BSONElement& spec) {
+    auto outSpec = parseOutSpecAndResolveTargetNamespace(spec, nss.dbName());
+    NamespaceString targetNss = NamespaceStringUtil::parseNamespaceFromRequest(
+        nss.dbName().tenantId(), outSpec.getDb(), outSpec.getColl());
 
-    NamespaceString targetNss = parseNsFromElem(spec, nss.dbName());
     uassert(ErrorCodes::InvalidNamespace,
             "Invalid {} target namespace, {}"_format(kStageName, targetNss.ns()),
             targetNss.isValid());
     return std::make_unique<DocumentSourceOut::LiteParsed>(spec.fieldName(), std::move(targetNss));
 }
 
+boost::optional<TimeseriesOptions> DocumentSourceOut::validateTimeseries() {
+    const NamespaceString& outNs = getOutputNs();
+    auto existingOpts = mongo::timeseries::getTimeseriesOptions(pExpCtx->opCtx, outNs, true);
+
+    // If the user did not specify the 'timeseries' option in the input, but the target namespace is
+    // a time-series collection, then we can fetch the time-series options from the
+    // CollectionCatalog and treat this operation as a write to time-series collection. If the user
+    // did specify 'timeseries' options and the target namespace exists, then the options should
+    // match.
+    if (!_timeseries) {
+        return existingOpts;
+    }
+
+    if (existingOpts) {
+        uassert(7406103,
+                str::stream() << "Time-series options inputted must match the existing time-series "
+                                 "collection. Received: "
+                              << _timeseries->toBSON().toString()
+                              << "Found: " << existingOpts->toBSON().toString(),
+                timeseries::optionsAreEqual(_timeseries.value(), existingOpts.value()));
+    } else {
+        auto collection = CollectionCatalog::get(pExpCtx->opCtx)
+                              ->lookupCollectionByNamespace(pExpCtx->opCtx, outNs);
+        uassert(7268700,
+                "Cannot create a time-series collection from a non time-series collection.",
+                !collection);
+        auto view = CollectionCatalog::get(pExpCtx->opCtx)->lookupView(pExpCtx->opCtx, outNs);
+        uassert(
+            7268703, "Cannot create a time-series collection from a non time-series view.", !view);
+    }
+    return _timeseries;
+}
+
 void DocumentSourceOut::initialize() {
     DocumentSourceWriteBlock writeBlock(pExpCtx->opCtx);
 
-    const auto& outputNs = getOutputNs();
-    // We will write all results into a temporary collection, then rename the temporary collection
-    // to be the target collection once we are done.
-    // Note that this temporary collection name is used by MongoMirror and thus should not be
-    // changed without consultation.
-    _tempNs = NamespaceString(outputNs.tenantId(),
-                              str::stream() << outputNs.dbName().toString() << ".tmp.agg_out."
-                                            << UUID::gen());
+    // Must be called before all other functions, since sets the value of '_timeseries', which the
+    // rest of the function heavily relies on.
+    _timeseries = validateTimeseries();
+
+    uassert(7406100,
+            "$out to time-series collections is only supported on FCV greater than or equal to 7.1",
+            feature_flags::gFeatureFlagAggOutTimeseries.isEnabled(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) ||
+                !_timeseries);
+
+    const NamespaceString& outputNs = makeBucketNsIfTimeseries(getOutputNs());
+
+    // We will write all results into a temporary collection, then rename the temporary
+    // collection to be the target collection once we are done. Note that this temporary
+    // collection name is used by MongoMirror and thus should not be changed without
+    // consultation.
+    _tempNs = NamespaceStringUtil::parseNamespaceFromRequest(
+        getOutputNs().tenantId(),
+        str::stream() << getOutputNs().dbName().toString() << "."
+                      << NamespaceString::kOutTmpCollectionPrefix << UUID::gen());
 
     // Save the original collection options and index specs so we can check they didn't change
     // during computation.
@@ -137,12 +196,25 @@ void DocumentSourceOut::initialize() {
         BSONObjBuilder cmd;
         cmd << "create" << _tempNs.coll();
         cmd << "temp" << true;
-        cmd.appendElementsUnique(_originalOutOptions);
-
+        if (_timeseries) {
+            // Append the original collection options without the 'validator' and 'clusteredIndex'
+            // fields since these fields are invalid with the 'timeseries' field and will be
+            // recreated when the buckets collection is created.
+            _originalOutOptions.isEmpty()
+                ? cmd << DocumentSourceOutSpec::kTimeseriesFieldName << _timeseries->toBSON()
+                : cmd.appendElementsUnique(_originalOutOptions.removeFields(
+                      StringDataSet{"clusteredIndex", "validator"}));
+        } else {
+            cmd.appendElementsUnique(_originalOutOptions);
+        }
         pExpCtx->mongoProcessInterface->createCollection(
             pExpCtx->opCtx, _tempNs.dbName(), cmd.done());
     }
 
+    // After creating the tmp collection we should update '_tempNs' to represent the buckets
+    // collection if the collection is time-series.
+    _tempNs = makeBucketNsIfTimeseries(_tempNs);
+
     CurOpFailpointHelpers::waitWhileFailPointEnabled(
         &outWaitAfterTempCollectionCreation,
         pExpCtx->opCtx,
@@ -170,7 +242,15 @@ void DocumentSourceOut::initialize() {
 void DocumentSourceOut::finalize() {
     DocumentSourceWriteBlock writeBlock(pExpCtx->opCtx);
 
-    const auto& outputNs = getOutputNs();
+    uassert(7406101,
+            "$out to time-series collections is only supported on FCV greater than or equal to 7.1",
+            feature_flags::gFeatureFlagAggOutTimeseries.isEnabled(
+                serverGlobalParams.featureCompatibility.acquireFCVSnapshot()) ||
+                !_timeseries);
+
+    // If the collection is time-series, we must rename to the "real" buckets collection.
+    const NamespaceString& outputNs = makeBucketNsIfTimeseries(getOutputNs());
+
     pExpCtx->mongoProcessInterface->renameIfOptionsAndIndexesHaveNotChanged(pExpCtx->opCtx,
                                                                             _tempNs,
                                                                             outputNs,
@@ -181,11 +261,32 @@ void DocumentSourceOut::finalize() {
 
     // The rename succeeded, so the temp collection no longer exists.
     _tempNs = {};
+
+    _timeseriesStateConsistent = false;
+    // If the collection is time-series, try to create the view.
+    if (_timeseries) {
+        BSONObjBuilder cmd;
+        cmd << "create" << getOutputNs().coll();
+        cmd << DocumentSourceOutSpec::kTimeseriesFieldName << _timeseries->toBSON();
+        pExpCtx->mongoProcessInterface->createTimeseriesView(
+            pExpCtx->opCtx, getOutputNs(), cmd.done(), _timeseries.value());
+    }
+
+    // Creating the view succeeded, so the boolean should be set to true.
+    _timeseriesStateConsistent = true;
+}
+
+BatchedCommandRequest DocumentSourceOut::initializeBatchedWriteRequest() const {
+    // Note that our insert targets '_tempNs' (or the associated timeseries view) since we will
+    // never write to 'outputNs' directly.
+    const auto& targetNss = _timeseries ? _tempNs.getTimeseriesViewNamespace() : _tempNs;
+    return DocumentSourceWriter::makeInsertCommand(targetNss, pExpCtx->bypassDocumentValidation);
 }
 
 boost::intrusive_ptr<DocumentSource> DocumentSourceOut::create(
-    NamespaceString outputNs, const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-
+    NamespaceString outputNs,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    boost::optional<TimeseriesOptions> timeseries) {
     uassert(ErrorCodes::OperationNotSupportedInTransaction,
             "{} cannot be used in a transaction"_format(kStageName),
             !expCtx->opCtx->inMultiDocumentTransaction());
@@ -201,20 +302,25 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceOut::create(
     uassert(31321,
             "Can't {} to internal database: {}"_format(kStageName, outputNs.db()),
             !outputNs.isOnInternalDb());
-
-    return new DocumentSourceOut(std::move(outputNs), expCtx);
+    return new DocumentSourceOut(std::move(outputNs), std::move(timeseries), expCtx);
 }
 
 boost::intrusive_ptr<DocumentSource> DocumentSourceOut::createFromBson(
     BSONElement elem, const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-    auto targetNS = parseNsFromElem(elem, expCtx->ns.dbName());
-    return create(targetNS, expCtx);
+    auto outSpec = parseOutSpecAndResolveTargetNamespace(elem, expCtx->ns.dbName());
+    NamespaceString targetNss = NamespaceStringUtil::parseNamespaceFromRequest(
+        expCtx->ns.dbName().tenantId(), outSpec.getDb(), outSpec.getColl());
+    return create(std::move(targetNss), expCtx, std::move(outSpec.getTimeseries()));
 }
 
-Value DocumentSourceOut::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
-    // Do not include the tenantId in the serialized 'outputNs'.
-    return Value(
-        DOC(kStageName << DOC("db" << _outputNs.dbName().db() << "coll" << _outputNs.coll())));
+Value DocumentSourceOut::serialize(const SerializationOptions& opts) const {
+    BSONObjBuilder bob;
+    DocumentSourceOutSpec spec;
+    spec.setDb(_outputNs.dbName().db());
+    spec.setColl(_outputNs.coll());
+    spec.setTimeseries(_timeseries);
+    spec.serialize(&bob, opts);
+    return Value(Document{{kStageName, bob.done()}});
 }
 
 void DocumentSourceOut::waitWhileFailPointEnabled() {
@@ -223,8 +329,9 @@ void DocumentSourceOut::waitWhileFailPointEnabled() {
         pExpCtx->opCtx,
         "hangWhileBuildingDocumentSourceOutBatch",
         []() {
-            LOGV2(20902,
-                  "Hanging aggregation due to 'hangWhileBuildingDocumentSourceOutBatch' failpoint");
+            LOGV2(
+                20902,
+                "Hanging aggregation due to  'hangWhileBuildingDocumentSourceOutBatch' failpoint");
         });
 }
 
