@@ -80,6 +80,27 @@ std::unique_ptr<Pipeline, PipelineDeleter> buildPipelineFromViewDefinition(
 
 }  // namespace
 
+DocumentSourceUnionWith::DocumentSourceUnionWith(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    std::unique_ptr<Pipeline, PipelineDeleter> pipeline)
+    : DocumentSource(kStageName, expCtx), _pipeline(std::move(pipeline)) {
+    if (!_pipeline->getContext()->ns.isOnInternalDb()) {
+        globalOpCounters.gotNestedAggregate();
+    }
+    _pipeline->getContext()->inUnionWith = true;
+}
+
+DocumentSourceUnionWith::DocumentSourceUnionWith(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    NamespaceString unionNss,
+    std::vector<BSONObj> pipeline)
+    : DocumentSourceUnionWith(expCtx,
+                              buildPipelineFromViewDefinition(
+                                  expCtx, expCtx->getResolvedNamespace(unionNss), pipeline)) {
+    _userNss = std::move(unionNss);
+    _userPipeline = std::move(pipeline);
+}
+
 DocumentSourceUnionWith::~DocumentSourceUnionWith() {
     if (_pipeline && _pipeline->getContext()->explain) {
         _pipeline->dispose(pExpCtx->opCtx);
@@ -127,12 +148,14 @@ std::unique_ptr<DocumentSourceUnionWith::LiteParsed> DocumentSourceUnionWith::Li
     NamespaceString unionNss;
     boost::optional<LiteParsedPipeline> liteParsedPipeline;
     if (spec.type() == BSONType::String) {
-        unionNss = NamespaceString(nss.dbName(), spec.valueStringData());
+        unionNss =
+            NamespaceStringUtil::parseNamespaceFromRequest(nss.dbName(), spec.valueStringData());
     } else {
         auto unionWithSpec =
             UnionWithSpec::parse(IDLParserContext(kStageName), spec.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceString(nss.dbName(), *unionWithSpec.getColl());
+            unionNss = NamespaceStringUtil::parseNamespaceFromRequest(nss.dbName(),
+                                                                      *unionWithSpec.getColl());
         } else {
             // If no collection specified, it must have $documents as first field in pipeline.
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
@@ -185,12 +208,14 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceUnionWith::createFromBson(
     NamespaceString unionNss;
     std::vector<BSONObj> pipeline;
     if (elem.type() == BSONType::String) {
-        unionNss = NamespaceString(expCtx->ns.dbName(), elem.valueStringData());
+        unionNss = NamespaceStringUtil::parseNamespaceFromRequest(expCtx->ns.dbName(),
+                                                                  elem.valueStringData());
     } else {
         auto unionWithSpec =
             UnionWithSpec::parse(IDLParserContext(kStageName), elem.embeddedObject());
         if (unionWithSpec.getColl()) {
-            unionNss = NamespaceString(expCtx->ns.dbName(), *unionWithSpec.getColl());
+            unionNss = NamespaceStringUtil::parseNamespaceFromRequest(expCtx->ns.dbName(),
+                                                                      *unionWithSpec.getColl());
         } else {
             // if no collection specified, it must have $documents as first field in pipeline
             validateUnionWithCollectionlessPipeline(unionWithSpec.getPipeline());
@@ -199,9 +224,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceUnionWith::createFromBson(
         pipeline = unionWithSpec.getPipeline().value_or(std::vector<BSONObj>{});
     }
     return make_intrusive<DocumentSourceUnionWith>(
-        expCtx,
-        buildPipelineFromViewDefinition(
-            expCtx, expCtx->getResolvedNamespace(std::move(unionNss)), std::move(pipeline)));
+        expCtx, std::move(unionNss), std::move(pipeline));
 }
 
 DocumentSource::GetNextResult DocumentSourceUnionWith::doGetNext() {
@@ -277,14 +300,10 @@ MONGO_COMPILER_NOINLINE void DocumentSourceUnionWith::logShardedViewFound(
 Pipeline::SourceContainer::iterator DocumentSourceUnionWith::doOptimizeAt(
     Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
     auto duplicateAcrossUnion = [&](auto&& nextStage) {
-        _pipeline->addFinalSource(nextStage->clone());
+        _pipeline->addFinalSource(nextStage->clone(_pipeline->getContext()));
         // Apply the same rewrite to the cached pipeline if available.
         if (pExpCtx->explain >= ExplainOptions::Verbosity::kExecStats) {
-            auto cloneForExplain = nextStage->clone();
-            if (!_cachedPipeline.empty()) {
-                cloneForExplain->setSource(_cachedPipeline.back().get());
-            }
-            _cachedPipeline.push_back(std::move(cloneForExplain));
+            _pushedDownStages.push_back(nextStage->serialize().getDocument().toBson());
         }
         auto newStageItr = container->insert(itr, std::move(nextStage));
         container->erase(std::next(itr));
@@ -317,14 +336,16 @@ void DocumentSourceUnionWith::doDispose() {
 
         if (!_pipeline->getContext()->explain) {
             _pipeline->dispose(pExpCtx->opCtx);
+            _userPipeline.clear();
+            _pushedDownStages.clear();
             _pipeline.reset();
         }
     }
 }
 
-Value DocumentSourceUnionWith::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
+Value DocumentSourceUnionWith::serialize(const SerializationOptions& opts) const {
     auto collectionless = _pipeline->getContext()->ns.isCollectionlessAggregateNS();
-    if (explain) {
+    if (opts.verbosity) {
         // There are several different possible states depending on the explain verbosity as well as
         // the other stages in the pipeline:
         //  * If verbosity is queryPlanner, then the sub-pipeline should be untouched and we can
@@ -334,44 +355,63 @@ Value DocumentSourceUnionWith::serialize(boost::optional<ExplainOptions::Verbosi
         //  $limit stage after the $unionWith which results in only reading from the base collection
         //  branch and not the sub-pipeline.
         Pipeline* pipeCopy = nullptr;
-        if (*explain == ExplainOptions::Verbosity::kQueryPlanner) {
+        if (*opts.verbosity == ExplainOptions::Verbosity::kQueryPlanner) {
             pipeCopy = Pipeline::create(_pipeline->getSources(), _pipeline->getContext()).release();
-        } else if (*explain >= ExplainOptions::Verbosity::kExecStats &&
+        } else if (*opts.verbosity >= ExplainOptions::Verbosity::kExecStats &&
                    _executionState > ExecutionProgress::kIteratingSource) {
+            std::vector<BSONObj> recoveredPipeline;
             // We've either exhausted the sub-pipeline or at least started iterating it. Use the
-            // cached pipeline to get the explain output since the '_pipeline' may have been
-            // modified for any optimizations or pushdowns into the initial $cursor stage.
-            pipeCopy = Pipeline::create(_cachedPipeline, _pipeline->getContext()).release();
+            // cached user pipeline and pushed down stages to get the explain output since the
+            // '_pipeline' may have been modified for any optimizations or pushdowns into the
+            // initial $cursor stage.
+            recoveredPipeline.reserve(_userPipeline.size() + _pushedDownStages.size());
+            std::move(
+                _userPipeline.begin(), _userPipeline.end(), std::back_inserter(recoveredPipeline));
+            std::move(_pushedDownStages.begin(),
+                      _pushedDownStages.end(),
+                      std::back_inserter(recoveredPipeline));
+            pipeCopy = Pipeline::parse(recoveredPipeline, _pipeline->getContext()).release();
         } else {
             // The plan does not require reading from the sub-pipeline, so just include the
             // serialization in the explain output.
             BSONArrayBuilder bab;
-            for (auto&& stage : _pipeline->serialize(explain))
+            for (auto&& stage : _pipeline->serialize(opts))
                 bab << stage;
             auto spec = collectionless
                 ? DOC("pipeline" << bab.arr())
-                : DOC("coll" << _pipeline->getContext()->ns.coll() << "pipeline" << bab.arr());
+                : DOC("coll" << opts.serializeIdentifier(_pipeline->getContext()->ns.coll())
+                             << "pipeline" << bab.arr());
             return Value(DOC(getSourceName() << spec));
         }
 
         invariant(pipeCopy);
         BSONObj explainLocal =
-            pExpCtx->mongoProcessInterface->preparePipelineAndExplain(pipeCopy, *explain);
+            pExpCtx->mongoProcessInterface->preparePipelineAndExplain(pipeCopy, *opts.verbosity);
         LOGV2_DEBUG(4553501, 3, "$unionWith attached cursor to pipeline for explain");
         // We expect this to be an explanation of a pipeline -- there should only be one field.
         invariant(explainLocal.nFields() == 1);
 
-        auto spec = collectionless ? DOC("pipeline" << explainLocal.firstElement())
-                                   : DOC("coll" << _pipeline->getContext()->ns.coll() << "pipeline"
-                                                << explainLocal.firstElement());
+        auto spec = collectionless
+            ? DOC("pipeline" << explainLocal.firstElement())
+            : DOC("coll" << opts.serializeIdentifier(_pipeline->getContext()->ns.coll())
+                         << "pipeline" << explainLocal.firstElement());
         return Value(DOC(getSourceName() << spec));
     } else {
-        BSONArrayBuilder bab;
-        for (auto&& stage : _pipeline->serialize())
-            bab << stage;
-        auto spec = collectionless
-            ? DOC("pipeline" << bab.arr())
-            : DOC("coll" << _pipeline->getContext()->ns.coll() << "pipeline" << bab.arr());
+        // Query shapes must reflect the original, unresolved and unoptimized pipeline, so we need a
+        // special case here if we are serializing the stage for that purpose. Otherwise, we should
+        // return the current (optimized) pipeline for introspection with explain, etc.
+        auto serializedPipeline = [&]() -> std::vector<BSONObj> {
+            if (opts.transformIdentifiers ||
+                opts.literalPolicy != LiteralSerializationPolicy::kUnchanged) {
+                return Pipeline::parse(_userPipeline, _pipeline->getContext())
+                    ->serializeToBson(opts);
+            }
+            return _pipeline->serializeToBson(opts);
+        }();
+
+        auto spec = collectionless ? DOC("pipeline" << serializedPipeline)
+                                   : DOC("coll" << opts.serializeIdentifier(_userNss.coll())
+                                                << "pipeline" << serializedPipeline);
         return Value(DOC(getSourceName() << spec));
     }
 }
@@ -424,6 +464,11 @@ void DocumentSourceUnionWith::reattachToOperationContext(OperationContext* opCtx
     }
 }
 
+bool DocumentSourceUnionWith::validateOperationContext(const OperationContext* opCtx) const {
+    return getContext()->opCtx == opCtx &&
+        (!_pipeline || _pipeline->validateOperationContext(opCtx));
+}
+
 void DocumentSourceUnionWith::addInvolvedCollections(
     stdx::unordered_set<NamespaceString>* collectionNames) const {
     collectionNames->insert(_pipeline->getContext()->ns);
