@@ -34,15 +34,18 @@
 #include <algorithm>
 
 #include "mongo/base/error_codes.h"
+#include "mongo/base/exact_cast.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/accumulator.h"
+#include "mongo/db/pipeline/change_stream_helpers.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_merge.h"
 #include "mongo/db/pipeline/document_source_out.h"
+#include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/search_helper.h"
@@ -108,14 +111,32 @@ void validateTopLevelPipeline(const Pipeline& pipeline) {
 
         // If the first stage is a $changeStream stage, then all stages in the pipeline must be
         // either $changeStream stages or allowlisted as being able to run in a change stream.
-        if (firstStageConstraints.isChangeStreamStage()) {
-            for (auto&& source : sources) {
-                uassert(ErrorCodes::IllegalOperation,
-                        str::stream() << source->getSourceName()
-                                      << " is not permitted in a $changeStream pipeline",
-                        source->constraints().isAllowedInChangeStream());
+        const bool isChangeStream = firstStageConstraints.isChangeStreamStage();
+        // Record whether any of the stages in the pipeline is a $changeStreamSplitLargeEvent.
+        bool hasChangeStreamSplitLargeEventStage = false;
+        for (auto&& source : sources) {
+            uassert(ErrorCodes::IllegalOperation,
+                    str::stream() << source->getSourceName()
+                                  << " is not permitted in a $changeStream pipeline",
+                    !(isChangeStream && !source->constraints().isAllowedInChangeStream()));
+            // Check whether any stages must only be run in a change stream pipeline.
+            uassert(ErrorCodes::IllegalOperation,
+                    str::stream() << source->getSourceName()
+                                  << " can only be used in a $changeStream pipeline",
+                    !(source->constraints().requiresChangeStream() && !isChangeStream));
+            // Check whether this is a change stream split stage.
+            if ("$changeStreamSplitLargeEvent"_sd == source->getSourceName()) {
+                hasChangeStreamSplitLargeEventStage = true;
             }
         }
+        auto expCtx = pipeline.getContext();
+        auto spec = isChangeStream ? expCtx->changeStreamSpec : boost::none;
+        auto hasSplitEventResumeToken = spec &&
+            change_stream::resolveResumeTokenFromSpec(expCtx, *spec).fragmentNum.has_value();
+        uassert(ErrorCodes::ChangeStreamFatalError,
+                "To resume from a split event, the $changeStream pipeline must include a "
+                "$changeStreamSplitLargeEvent stage",
+                !(hasSplitEventResumeToken && !hasChangeStreamSplitLargeEventStage));
     }
 
     // Verify that usage of $searchMeta and $search is legal.
@@ -158,11 +179,12 @@ Pipeline::~Pipeline() {
 
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::clone(
     const boost::intrusive_ptr<ExpressionContext>& newExpCtx) const {
+    auto expCtx = newExpCtx ? newExpCtx : getContext();
     SourceContainer clonedStages;
     for (auto&& stage : _sources) {
-        clonedStages.push_back(stage->clone(newExpCtx));
+        clonedStages.push_back(stage->clone(expCtx));
     }
-    return create(clonedStages, newExpCtx ? newExpCtx : getContext());
+    return create(clonedStages, expCtx);
 }
 
 template <class T>
@@ -234,39 +256,43 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::create(
 }
 
 void Pipeline::validateCommon(bool alreadyOptimized) const {
-    size_t i = 0;
-
     uassert(ErrorCodes::FailedToParse,
             str::stream() << "Pipeline length must be no longer than "
                           << internalPipelineLengthLimit << " stages",
             static_cast<int>(_sources.size()) <= internalPipelineLengthLimit);
 
-    for (auto&& stage : _sources) {
+    checkValidOperationContext();
+
+    // Keep track of stages which can only appear once.
+    std::set<StringData> singleUseStages;
+
+    for (auto sourceIter = _sources.begin(); sourceIter != _sources.end(); ++sourceIter) {
+        auto& stage = *sourceIter;
         auto constraints = stage->constraints(_splitState);
 
         // Verify that all stages adhere to their PositionRequirement constraints.
         uassert(40602,
                 str::stream() << stage->getSourceName()
                               << " is only valid as the first stage in a pipeline",
-                !(constraints.requiredPosition == PositionRequirement::kFirst && i != 0));
-        uassert(40603,
-                str::stream() << stage->getSourceName()
-                              << " is only valid as the first stage in an optimized pipeline",
-                !(alreadyOptimized &&
-                  constraints.requiredPosition == PositionRequirement::kFirstAfterOptimization &&
-                  i != 0));
+                !(constraints.requiredPosition == PositionRequirement::kFirst &&
+                  sourceIter != _sources.begin()));
 
+        // TODO SERVER-73790: use PositionRequirement::kCustom to validate $match.
         auto matchStage = dynamic_cast<DocumentSourceMatch*>(stage.get());
         uassert(17313,
                 "$match with $text is only allowed as the first pipeline stage",
-                !(i != 0 && matchStage && matchStage->isTextQuery()));
+                !(sourceIter != _sources.begin() && matchStage && matchStage->isTextQuery()));
 
         uassert(40601,
                 str::stream() << stage->getSourceName()
                               << " can only be the final stage in the pipeline",
                 !(constraints.requiredPosition == PositionRequirement::kLast &&
-                  i != _sources.size() - 1));
-        ++i;
+                  std::next(sourceIter) != _sources.end()));
+
+        // If the stage has a special requirement about its position, validate it.
+        if (constraints.requiredPosition == PositionRequirement::kCustom) {
+            stage->validatePipelinePosition(alreadyOptimized, sourceIter, _sources);
+        }
 
         // Verify that we are not attempting to run a mongoS-only stage on mongoD.
         uassert(40644,
@@ -278,6 +304,17 @@ void Pipeline::validateCommon(bool alreadyOptimized) const {
             str::stream() << "Stage not supported inside of a multi-document transaction: "
                           << stage->getSourceName(),
             !(pCtx->opCtx->inMultiDocumentTransaction() && !constraints.isAllowedInTransaction()));
+
+        // Verify that a stage which can only appear once doesn't appear more than that.
+        uassert(7183900,
+                str::stream() << stage->getSourceName() << " can only be used once in the pipeline",
+                !(constraints.canAppearOnlyOnceInPipeline &&
+                  !singleUseStages.insert(stage->getSourceName()).second));
+
+        tassert(7355707,
+                "If a stage is broadcast to all shard servers then it must be a data source.",
+                constraints.hostRequirement != HostTypeRequirement::kAllShardServers ||
+                    !constraints.requiresInputDocSource);
     }
 }
 
@@ -286,21 +323,29 @@ void Pipeline::optimizePipeline() {
     if (MONGO_unlikely(disablePipelineOptimization.shouldFail())) {
         return;
     }
-
     optimizeContainer(&_sources);
+    optimizeEachStage(&_sources);
 }
 
 void Pipeline::optimizeContainer(SourceContainer* container) {
-    SourceContainer optimizedSources;
-
     SourceContainer::iterator itr = container->begin();
     try {
         while (itr != container->end()) {
             invariant((*itr).get());
             itr = (*itr).get()->optimizeAt(itr, container);
         }
+    } catch (DBException& ex) {
+        ex.addContext("Failed to optimize pipeline");
+        throw;
+    }
 
-        // Once we have reached our final number of stages, optimize each individually.
+    stitch(container);
+}
+
+void Pipeline::optimizeEachStage(SourceContainer* container) {
+    SourceContainer optimizedSources;
+    try {
+        // We should have our final number of stages. Optimize each individually.
         for (auto&& source : *container) {
             if (auto out = source->optimize()) {
                 optimizedSources.push_back(out);
@@ -341,6 +386,9 @@ void Pipeline::detachFromOperationContext() {
     for (auto&& source : _sources) {
         source->detachFromOperationContext();
     }
+
+    // Check for a null operation context to make sure that all children detached correctly.
+    checkValidOperationContext();
 }
 
 void Pipeline::reattachToOperationContext(OperationContext* opCtx) {
@@ -349,6 +397,24 @@ void Pipeline::reattachToOperationContext(OperationContext* opCtx) {
     for (auto&& source : _sources) {
         source->reattachToOperationContext(opCtx);
     }
+
+    checkValidOperationContext();
+}
+
+bool Pipeline::validateOperationContext(const OperationContext* opCtx) const {
+    return std::all_of(_sources.begin(), _sources.end(), [this, opCtx](const auto& s) {
+        // All sources in a pipeline must share its expression context. Subpipelines may have a
+        // different expression context, but must point to the same operation context. Let the
+        // sources validate this themselves since they don't all have the same subpipelines, etc.
+        return s->getContext() == getContext() && s->validateOperationContext(opCtx);
+    });
+}
+
+void Pipeline::checkValidOperationContext() const {
+    tassert(7406000,
+            str::stream()
+                << "All DocumentSources and subpipelines must have the same operation context",
+            validateOperationContext(getContext()->opCtx));
 }
 
 void Pipeline::dispose(OperationContext* opCtx) {
@@ -400,11 +466,19 @@ bool Pipeline::needsMongosMerger() const {
     });
 }
 
+bool Pipeline::needsAllShardServers() const {
+    return std::any_of(_sources.begin(), _sources.end(), [&](const auto& stage) {
+        return stage->constraints().resolvedHostTypeRequirement(pCtx) ==
+            HostTypeRequirement::kAllShardServers;
+    });
+}
+
 bool Pipeline::needsShard() const {
     return std::any_of(_sources.begin(), _sources.end(), [&](const auto& stage) {
         auto hostType = stage->constraints().resolvedHostTypeRequirement(pCtx);
         return (hostType == HostTypeRequirement::kAnyShard ||
-                hostType == HostTypeRequirement::kPrimaryShard);
+                hostType == HostTypeRequirement::kPrimaryShard ||
+                hostType == HostTypeRequirement::kAllShardServers);
     });
 }
 
@@ -450,20 +524,20 @@ stdx::unordered_set<NamespaceString> Pipeline::getInvolvedCollections() const {
 }
 
 vector<Value> Pipeline::serializeContainer(const SourceContainer& container,
-                                           boost::optional<ExplainOptions::Verbosity> explain) {
+                                           boost::optional<const SerializationOptions&> opts) {
     vector<Value> serializedSources;
     for (auto&& source : container) {
-        source->serializeToArray(serializedSources, explain);
+        source->serializeToArray(serializedSources, opts ? opts.get() : SerializationOptions());
     }
     return serializedSources;
 }
-vector<Value> Pipeline::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
-    return serializeContainer(_sources, explain);
+
+vector<Value> Pipeline::serialize(boost::optional<const SerializationOptions&> opts) const {
+    return serializeContainer(_sources, opts);
 }
 
-vector<BSONObj> Pipeline::serializeToBson(
-    boost::optional<ExplainOptions::Verbosity> explain) const {
-    const auto serialized = serialize(explain);
+vector<BSONObj> Pipeline::serializeToBson(boost::optional<const SerializationOptions&> opts) const {
+    const auto serialized = serialize(opts);
     std::vector<BSONObj> asBson;
     asBson.reserve(serialized.size());
     for (auto&& stage : serialized) {
@@ -504,16 +578,16 @@ boost::optional<Document> Pipeline::getNext() {
                               : boost::optional<Document>{nextResult.releaseDocument()};
 }
 
-vector<Value> Pipeline::writeExplainOps(ExplainOptions::Verbosity verbosity) const {
+vector<Value> Pipeline::writeExplainOps(const SerializationOptions& opts) const {
     vector<Value> array;
     for (auto&& stage : _sources) {
         auto beforeSize = array.size();
-        stage->serializeToArray(array, verbosity);
+        stage->serializeToArray(array, opts);
         auto afterSize = array.size();
         // Append execution stats to the serialized stage if the specified verbosity is
         // 'executionStats' or 'allPlansExecution'.
         invariant(afterSize - beforeSize == 1u);
-        if (verbosity >= ExplainOptions::Verbosity::kExecStats) {
+        if (*opts.verbosity >= ExplainOptions::Verbosity::kExecStats) {
             auto serializedStage = array.back();
             array.back() = appendCommonExecStats(serializedStage, stage->getCommonStats());
         }
@@ -554,6 +628,7 @@ DepsTracker Pipeline::getDependenciesForContainer(
     // us to call 'deps.setNeedsMetadata()' without throwing.
     DepsTracker deps(unavailableMetadata.get_value_or(DepsTracker::kNoMetadata));
 
+    OrderedPathSet generatedPaths;
     bool hasUnsupportedStage = false;
     bool knowAllFields = false;
     bool knowAllMeta = false;
@@ -573,10 +648,33 @@ DepsTracker Pipeline::getDependenciesForContainer(
         // If we ever saw an unsupported stage, don't bother continuing to track field and metadata
         // deps: we already have to assume the pipeline depends on everything.
         if (!hasUnsupportedStage && !knowAllFields) {
-            deps.fields.insert(localDeps.fields.begin(), localDeps.fields.end());
+            for (const auto& field : localDeps.fields) {
+                // If a field was generated within the pipeline, we don't need to count it as a
+                // dependency of the pipeline as a whole when it is used in later stages.
+                if (!expression::containsDependency({field}, generatedPaths)) {
+                    deps.fields.emplace(field);
+                }
+            }
             if (localDeps.needWholeDocument)
                 deps.needWholeDocument = true;
             knowAllFields = status & DepsTracker::State::EXHAUSTIVE_FIELDS;
+
+            // Check if this stage modifies any fields that we should track for use by later stages.
+            // Fields which are part of exclusion projections should not be marked as generated,
+            // despite them being modified.
+            auto localGeneratedPaths = source->getModifiedPaths();
+            auto isExclusionProjection = [&]() {
+                const auto projStage =
+                    exact_pointer_cast<DocumentSourceSingleDocumentTransformation*>(source.get());
+                return projStage &&
+                    projStage->getType() ==
+                    TransformerInterface::TransformerType::kExclusionProjection;
+            };
+            if (localGeneratedPaths.type == DocumentSource::GetModPathsReturn::Type::kFiniteSet &&
+                !isExclusionProjection()) {
+                auto newPathNames = localGeneratedPaths.getNewNames();
+                generatedPaths.insert(newPathNames.begin(), newPathNames.end());
+            }
         }
 
         if (!hasUnsupportedStage && !knowAllMeta) {
@@ -609,7 +707,8 @@ Status Pipeline::_pipelineCanRunOnMongoS() const {
         auto hostRequirement = constraints.resolvedHostTypeRequirement(pCtx);
 
         const bool needsShard = (hostRequirement == HostTypeRequirement::kAnyShard ||
-                                 hostRequirement == HostTypeRequirement::kPrimaryShard);
+                                 hostRequirement == HostTypeRequirement::kPrimaryShard ||
+                                 hostRequirement == HostTypeRequirement::kAllShardServers);
 
         const bool mustWriteToDisk =
             (constraints.diskRequirement == DiskUseRequirement::kWritesPersistentData);
@@ -696,17 +795,31 @@ boost::intrusive_ptr<DocumentSource> Pipeline::popFrontWithNameAndCriteria(
     return popFront();
 }
 
+void Pipeline::appendPipeline(std::unique_ptr<Pipeline, PipelineDeleter> otherPipeline) {
+    auto& otherPipelineSources = otherPipeline->getSources();
+    while (!otherPipelineSources.empty()) {
+        _sources.push_back(std::move(otherPipelineSources.front()));
+        otherPipelineSources.pop_front();
+    }
+    constexpr bool alreadyOptimized = false;
+    validateCommon(alreadyOptimized);
+    stitch();
+}
+
+
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
     const std::vector<BSONObj>& rawPipeline,
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const MakePipelineOptions opts) {
     auto pipeline = Pipeline::parse(rawPipeline, expCtx, opts.validator);
 
+    bool alreadyOptimized = opts.alreadyOptimized;
+
     if (opts.optimize) {
         pipeline->optimizePipeline();
+        alreadyOptimized = true;
     }
 
-    constexpr bool alreadyOptimized = true;
     pipeline->validateCommon(alreadyOptimized);
 
     if (opts.attachCursorSource) {
@@ -714,39 +827,61 @@ std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
             pipeline.release(), opts.shardTargetingPolicy, std::move(opts.readConcern));
     }
 
+    // After parsing the pipeline to detect if $$USER_ROLES is referenced, set the value of
+    // $$USER_ROLES for the pipeline.
+    expCtx->setUserRoles();
+
     return pipeline;
 }
 
+std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipeline(
+    AggregateCommandRequest& aggRequest,
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    boost::optional<BSONObj> shardCursorsSortSpec,
+    const MakePipelineOptions opts) {
+    tassert(7393500, "AttachCursorSource must be set to true.", opts.attachCursorSource);
+
+    boost::optional<BSONObj> readConcern;
+    // If readConcern is set on opts and aggRequest, assert they are equal.
+    if (opts.readConcern && aggRequest.getReadConcern()) {
+        tassert(7393501,
+                "Read concern on aggRequest and makePipelineOpts must match.",
+                opts.readConcern->binaryEqual(*aggRequest.getReadConcern()));
+        readConcern = aggRequest.getReadConcern();
+    } else {
+        readConcern = aggRequest.getReadConcern() ? aggRequest.getReadConcern() : opts.readConcern;
+    }
+
+    auto pipeline = Pipeline::parse(aggRequest.getPipeline(), expCtx, opts.validator);
+    if (opts.optimize) {
+        pipeline->optimizePipeline();
+    }
+
+    constexpr bool alreadyOptimized = true;
+    pipeline->validateCommon(alreadyOptimized);
+    aggRequest.setPipeline(pipeline->serializeToBson());
+
+    return expCtx->mongoProcessInterface->attachCursorSourceToPipeline(aggRequest,
+                                                                       pipeline.release(),
+                                                                       expCtx,
+                                                                       shardCursorsSortSpec,
+                                                                       opts.shardTargetingPolicy,
+                                                                       std::move(readConcern));
+}
+
 Pipeline::SourceContainer::iterator Pipeline::optimizeEndOfPipeline(
     Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
     // We must create a new SourceContainer representing the subsection of the pipeline we wish to
     // optimize, since otherwise calls to optimizeAt() will overrun these limits.
     auto endOfPipeline = Pipeline::SourceContainer(std::next(itr), container->end());
     Pipeline::optimizeContainer(&endOfPipeline);
+    Pipeline::optimizeEachStage(&endOfPipeline);
     container->erase(std::next(itr), container->end());
     container->splice(std::next(itr), endOfPipeline);
 
     return std::next(itr);
 }
 
-Pipeline::SourceContainer::iterator Pipeline::optimizeAtEndOfPipeline(
-    Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
-    if (itr == container->end()) {
-        return itr;
-    }
-    itr = std::next(itr);
-    try {
-        while (itr != container->end()) {
-            invariant((*itr).get());
-            itr = (*itr).get()->optimizeAt(itr, container);
-        }
-    } catch (DBException& ex) {
-        ex.addContext("Failed to optimize pipeline");
-        throw;
-    }
-    return itr;
-}
-
 std::unique_ptr<Pipeline, PipelineDeleter> Pipeline::makePipelineFromViewDefinition(
     const boost::intrusive_ptr<ExpressionContext>& subPipelineExpCtx,
     ExpressionContext::ResolvedNamespace resolvedNs,
