@@ -50,6 +50,7 @@
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/index/index_descriptor.h"
+#include "mongo/db/multitenancy_gen.h"
 #include "mongo/db/pipeline/document_source_cursor.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/pipeline_d.h"
@@ -58,6 +59,7 @@
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/sbe_plan_cache.h"
 #include "mongo/db/repl/primary_only_service.h"
+#include "mongo/db/s/query_analysis_writer.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/transaction_coordinator_curop.h"
 #include "mongo/db/s/transaction_coordinator_worker_curop_repository.h"
@@ -67,12 +69,19 @@
 #include "mongo/db/stats/storage_stats.h"
 #include "mongo/db/storage/backup_cursor_hooks.h"
 #include "mongo/db/storage/durable_catalog.h"
+#include "mongo/db/timeseries/catalog_helper.h"
+#include "mongo/db/timeseries/timeseries_constants.h"
+#include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/db/transaction/transaction_history_iterator.h"
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/db/transaction/transaction_participant_resource_yielder.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/query/document_source_merge_cursors.h"
+#include "mongo/s/query_analysis_sample_tracker.h"
+#include "mongo/s/query_analysis_sampler_util.h"
+#include "mongo/util/database_name_util.h"
+#include "mongo/util/namespace_string_util.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -142,13 +151,14 @@ void listDurableCatalog(OperationContext* opCtx,
             continue;
         }
 
-        NamespaceString ns(obj.getStringField("ns"));
+        NamespaceString ns(NamespaceString::parseFromStringExpectTenantIdInMultitenancyMode(
+            obj.getStringField("ns")));
         if (ns.isSystemDotViews()) {
             systemViewsNamespaces->push_back(ns);
         }
 
         BSONObjBuilder builder;
-        builder.append("db", ns.db());
+        builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
         builder.append("name", ns.coll());
         builder.append("type", "collection");
         if (!shardName.empty()) {
@@ -172,14 +182,12 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
                                                                   const NamespaceString& ns,
                                                                   StringData host,
                                                                   bool addShardName) {
-    AutoGetCollectionForReadCommandMaybeLockFree collection(opCtx, ns);
+    AutoGetCollectionForReadMaybeLockFree collection(opCtx, ns);
 
     std::vector<Document> indexStats;
     if (!collection) {
-        LOGV2_DEBUG(23881,
-                    2,
-                    "Collection not found on index stats retrieval: {ns_ns}",
-                    "ns_ns"_attr = ns.ns());
+        LOGV2_DEBUG(
+            23881, 2, "Collection not found on index stats retrieval: {ns_ns}", "ns_ns"_attr = ns);
         return indexStats;
     }
 
@@ -211,15 +219,22 @@ std::vector<Document> CommonMongodProcessInterface::getIndexStats(OperationConte
         auto entry = idxCatalog->getEntry(idx);
         doc["spec"] = Value(idx->infoObj());
 
-        // Not all indexes in the CollectionIndexUsageTracker may be visible or consistent with our
-        // snapshot. For this reason, it is unsafe to check `isReady` on the entry, which
-        // asserts that the index's in-memory state is consistent with our snapshot.
-        if (!entry->isPresentInMySnapshot(opCtx)) {
-            continue;
-        }
+        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+        if (feature_flags::gPointInTimeCatalogLookups.isEnabledAndIgnoreFCVUnsafe()) {
+            if (!entry->isReady(opCtx)) {
+                doc["building"] = Value(true);
+            }
+        } else {
+            // Not all indexes in the CollectionIndexUsageTracker may be visible or consistent with
+            // our snapshot. For this reason, it is unsafe to check `isReady` on the entry, which
+            // asserts that the index's in-memory state is consistent with our snapshot.
+            if (!entry->isPresentInMySnapshot(opCtx)) {
+                continue;
+            }
 
-        if (!entry->isReadyInMySnapshot(opCtx)) {
-            doc["building"] = Value(true);
+            if (!entry->isReadyInMySnapshot(opCtx)) {
+                doc["building"] = Value(true);
+            }
         }
 
         indexStats.push_back(doc.freeze());
@@ -250,10 +265,9 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
         AutoGetCollectionForReadCommandMaybeLockFree collLock(
             opCtx,
             systemViewsNamespaces.front(),
-            AutoGetCollectionViewMode::kViewsForbidden,
-            Date_t::max(),
-            AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
-            {++systemViewsNamespaces.cbegin(), systemViewsNamespaces.cend()});
+            AutoGetCollection::Options{}.secondaryNssOrUUIDs(
+                {++systemViewsNamespaces.cbegin(), systemViewsNamespaces.cend()}),
+            AutoStatsTracker::LogMode::kUpdateTopAndCurOp);
 
         // If the primary collection is not available, it means the information from parsing
         // _mdb_catalog is no longer valid. Therefore, we restart this process from the top.
@@ -268,17 +282,17 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
         std::vector<NamespaceStringOrUUID> systemViewsNamespacesFromSecondCatalogRead;
         listDurableCatalog(
             opCtx, getShardName(opCtx), &docs, &systemViewsNamespacesFromSecondCatalogRead);
-        if (!std::equal(
-                systemViewsNamespaces.cbegin(),
-                systemViewsNamespaces.cend(),
-                systemViewsNamespacesFromSecondCatalogRead.cbegin(),
-                [](const auto& lhs, const auto& rhs) { return *lhs.nss() == *rhs.nss(); })) {
+        if (!std::equal(systemViewsNamespaces.cbegin(),
+                        systemViewsNamespaces.cend(),
+                        systemViewsNamespacesFromSecondCatalogRead.cbegin(),
+                        [](const auto& lhs, const auto& rhs) { return lhs.nss() == rhs.nss(); })) {
             continue;
         }
 
         for (const auto& svns : systemViewsNamespaces) {
-            auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByNamespaceForRead(
-                opCtx, *svns.nss());
+            // Hold reference to the catalog for collection lookup without locks to be safe.
+            auto catalog = CollectionCatalog::get(opCtx);
+            auto collection = catalog->lookupCollectionByNamespace(opCtx, svns.nss());
             if (!collection) {
                 continue;
             }
@@ -287,11 +301,13 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
             while (auto record = cursor->next()) {
                 BSONObj obj = record->data.releaseToBson();
 
-                NamespaceString ns(obj.getStringField("_id"));
-                NamespaceString viewOnNs(ns.db(), obj.getStringField("viewOn"));
+                NamespaceString ns(NamespaceStringUtil::deserialize(svns.nss().tenantId(),
+                                                                    obj.getStringField("_id")));
+                NamespaceString viewOnNs(NamespaceStringUtil::parseNamespaceFromDoc(
+                    ns.dbName(), obj.getStringField("viewOn")));
 
                 BSONObjBuilder builder;
-                builder.append("db", ns.db());
+                builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
                 builder.append("name", ns.coll());
                 if (viewOnNs.isTimeseriesBucketsCollection()) {
                     builder.append("type", "timeseries");
@@ -312,48 +328,62 @@ std::deque<BSONObj> CommonMongodProcessInterface::listCatalog(OperationContext*
 }
 
 boost::optional<BSONObj> CommonMongodProcessInterface::getCatalogEntry(
-    OperationContext* opCtx, const NamespaceString& ns) const {
-    Lock::GlobalLock globalLock{opCtx, MODE_IS};
+    OperationContext* opCtx,
+    const NamespaceString& ns,
+    const boost::optional<UUID>& collUUID) const {
 
-    auto rs = DurableCatalog::get(opCtx)->getRecordStore();
-    if (!rs) {
+    // Perform an AutoGetCollection. This will verify that the collection still exists at the given
+    // read concern. If it doesn't and the aggregation has specified a UUID then this acquisition
+    // will fail.
+    AutoGetCollectionForRead coll{opCtx, ns, AutoGetCollection::Options{}.expectedUUID(collUUID)};
+
+    const auto& collPtr = coll.getCollection();
+
+    if (!collPtr) {
         return boost::none;
     }
 
-    auto cursor = rs->getCursor(opCtx);
-    while (auto record = cursor->next()) {
-        auto obj = record->data.toBson();
-        if (NamespaceString{obj.getStringField("ns")} != ns) {
-            continue;
-        }
+    auto obj = DurableCatalog::get(opCtx)->getCatalogEntry(opCtx, collPtr->getCatalogId());
 
-        BSONObjBuilder builder;
-        builder.append("db", ns.db());
-        builder.append("name", ns.coll());
-        builder.append("type", "collection");
-        if (auto shardName = getShardName(opCtx); !shardName.empty()) {
-            builder.append("shard", shardName);
-        }
-        builder.appendElements(obj);
-
-        return builder.obj();
+    BSONObjBuilder builder;
+    builder.append("db", DatabaseNameUtil::serialize(ns.dbName()));
+    builder.append("name", ns.coll());
+    builder.append("type", "collection");
+    if (auto shardName = getShardName(opCtx); !shardName.empty()) {
+        builder.append("shard", shardName);
     }
+    builder.appendElements(obj);
 
-    return boost::none;
+    return builder.obj();
 }
 
 void CommonMongodProcessInterface::appendLatencyStats(OperationContext* opCtx,
                                                       const NamespaceString& nss,
                                                       bool includeHistograms,
                                                       BSONObjBuilder* builder) const {
-    Top::get(opCtx->getServiceContext()).appendLatencyStats(nss, includeHistograms, builder);
+    auto catalog = CollectionCatalog::get(opCtx);
+    auto view = catalog->lookupView(opCtx, nss);
+    if (!view) {
+        AutoGetCollectionForRead collection(opCtx, nss);
+        bool redactForQE =
+            (collection && collection->getCollectionOptions().encryptedFieldConfig) ||
+            nss.isFLE2StateCollection();
+        if (!redactForQE) {
+            Top::get(opCtx->getServiceContext())
+                .appendLatencyStats(nss, includeHistograms, builder);
+        }
+    } else {
+        Top::get(opCtx->getServiceContext()).appendLatencyStats(nss, includeHistograms, builder);
+    }
 }
 
-Status CommonMongodProcessInterface::appendStorageStats(OperationContext* opCtx,
-                                                        const NamespaceString& nss,
-                                                        const StorageStatsSpec& spec,
-                                                        BSONObjBuilder* builder) const {
-    return appendCollectionStorageStats(opCtx, nss, spec, builder);
+Status CommonMongodProcessInterface::appendStorageStats(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const StorageStatsSpec& spec,
+    BSONObjBuilder* builder,
+    const boost::optional<BSONObj>& filterObj) const {
+    return appendCollectionStorageStats(opCtx, nss, spec, builder, filterObj);
 }
 
 Status CommonMongodProcessInterface::appendRecordCount(OperationContext* opCtx,
@@ -371,21 +401,25 @@ Status CommonMongodProcessInterface::appendQueryExecStats(OperationContext* opCt
                 str::stream() << "Collection [" << nss.toString() << "] not found."};
     }
 
-    auto collectionScanStats =
-        CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-            .getCollectionScanStats();
-
-    dassert(collectionScanStats.collectionScans <=
-            static_cast<unsigned long long>(std::numeric_limits<long long>::max()));
-    dassert(collectionScanStats.collectionScansNonTailable <=
-            static_cast<unsigned long long>(std::numeric_limits<long long>::max()));
-    builder->append("queryExecStats",
-                    BSON("collectionScans" << BSON(
-                             "total" << static_cast<long long>(collectionScanStats.collectionScans)
-                                     << "nonTailable"
-                                     << static_cast<long long>(
-                                            collectionScanStats.collectionScansNonTailable))));
+    bool redactForQE =
+        collection->getCollectionOptions().encryptedFieldConfig || nss.isFLE2StateCollection();
+    if (!redactForQE) {
+        auto collectionScanStats =
+            CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
+                .getCollectionScanStats();
 
+        dassert(collectionScanStats.collectionScans <=
+                static_cast<unsigned long long>(std::numeric_limits<long long>::max()));
+        dassert(collectionScanStats.collectionScansNonTailable <=
+                static_cast<unsigned long long>(std::numeric_limits<long long>::max()));
+        builder->append(
+            "queryExecStats",
+            BSON("collectionScans"
+                 << BSON("total" << static_cast<long long>(collectionScanStats.collectionScans)
+                                 << "nonTailable"
+                                 << static_cast<long long>(
+                                        collectionScanStats.collectionScansNonTailable))));
+    }
     return Status::OK();
 }
 
@@ -407,7 +441,8 @@ BSONObj CommonMongodProcessInterface::getCollectionOptions(OperationContext* opC
 }
 
 std::unique_ptr<Pipeline, PipelineDeleter>
-CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(Pipeline* ownedPipeline) {
+CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(
+    Pipeline* ownedPipeline, boost::optional<const AggregateCommandRequest&> aggRequest) {
     auto expCtx = ownedPipeline->getContext();
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline(ownedPipeline,
                                                         PipelineDeleter(expCtx->opCtx));
@@ -421,30 +456,45 @@ CommonMongodProcessInterface::attachCursorSourceToPipelineForLocalRead(Pipeline*
         return pipeline;
     }
 
-    boost::optional<AutoGetCollectionForReadCommandMaybeLockFree> autoColl;
-    const NamespaceStringOrUUID nsOrUUID = expCtx->uuid
-        ? NamespaceStringOrUUID{expCtx->ns.db().toString(), *expCtx->uuid}
-        : expCtx->ns;
+    if (expCtx->eligibleForSampling()) {
+        if (auto sampleId = analyze_shard_key::tryGenerateSampleId(
+                expCtx->opCtx, expCtx->ns, analyze_shard_key::SampledCommandNameEnum::kAggregate)) {
+            auto [_, letParameters] =
+                expCtx->variablesParseState.transitionalCompatibilitySerialize(expCtx->variables);
+            analyze_shard_key::QueryAnalysisWriter::get(expCtx->opCtx)
+                ->addAggregateQuery(*sampleId,
+                                    expCtx->ns,
+                                    pipeline->getInitialQuery(),
+                                    expCtx->getCollatorBSON(),
+                                    letParameters)
+                .getAsync([](auto) {});
+        }
+    }
 
     // Reparse 'pipeline' to discover whether there are secondary namespaces that we need to lock
     // when constructing our query executor.
     auto lpp = LiteParsedPipeline(expCtx->ns, pipeline->serializeToBson());
     std::vector<NamespaceStringOrUUID> secondaryNamespaces = lpp.getForeignExecutionNamespaces();
 
-    autoColl.emplace(expCtx->opCtx,
-                     nsOrUUID,
-                     AutoGetCollectionViewMode::kViewsForbidden,
-                     Date_t::max(),
-                     AutoStatsTracker::LogMode::kUpdateTop,
-                     secondaryNamespaces);
+    AutoGetCollectionForReadCommandMaybeLockFree autoColl(
+        expCtx->opCtx,
+        expCtx->ns,
+        AutoGetCollection::Options{}.secondaryNssOrUUIDs(secondaryNamespaces),
+        AutoStatsTracker::LogMode::kUpdateTop);
+
+    uassert(ErrorCodes::NamespaceNotFound,
+            fmt::format("collection '{}' does not match the expected uuid",
+                        expCtx->ns.toStringForErrorMsg()),
+            !expCtx->uuid || (autoColl && autoColl->uuid() == expCtx->uuid));
 
     MultipleCollectionAccessor holder{expCtx->opCtx,
-                                      &autoColl->getCollection(),
-                                      autoColl->getNss(),
-                                      autoColl->isAnySecondaryNamespaceAViewOrSharded(),
+                                      &autoColl.getCollection(),
+                                      autoColl.getNss(),
+                                      autoColl.isAnySecondaryNamespaceAViewOrSharded(),
                                       secondaryNamespaces};
+    auto resolvedAggRequest = aggRequest ? &aggRequest.get() : nullptr;
     PipelineD::buildAndAttachInnerQueryExecutorToPipeline(
-        holder, expCtx->ns, nullptr, pipeline.get());
+        holder, expCtx->ns, resolvedAggRequest, pipeline.get());
 
     return pipeline;
 }
@@ -478,15 +528,15 @@ boost::optional<Document> CommonMongodProcessInterface::doLookupSingleDocument(
         auto foreignExpCtx = expCtx->copyWith(
             nss,
             collectionUUID,
-            _getCollectionDefaultCollator(expCtx->opCtx, nss.db(), collectionUUID));
+            _getCollectionDefaultCollator(expCtx->opCtx, nss.dbName(), collectionUUID));
 
         // If we are here, we are either executing the pipeline normally or running in one of the
         // execution stat explain verbosities. In either case, we disable explain on the foreign
         // context so that we actually retrieve the document.
         foreignExpCtx->explain = boost::none;
-
         pipeline = Pipeline::makePipeline({BSON("$match" << documentKey)}, foreignExpCtx, opts);
-    } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
+    } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>& ex) {
+        LOGV2_DEBUG(6726700, 1, "Namespace not found while looking up document", "error"_attr = ex);
         return boost::none;
     }
 
@@ -540,6 +590,9 @@ std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStat
     };
 
     const auto predicate = [&matchExp](const BSONObj& obj) {
+        if (obj.hasField("securityLevel")) {
+            return false;
+        }
         return !matchExp ? true : matchExp->matchesBSON(obj);
     };
 
@@ -554,22 +607,21 @@ std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStat
     auto planCacheEntries =
         planCache->getMatchingStats({} /* cacheKeyFilterFunc */, serializer, predicate);
 
-    if (feature_flags::gFeatureFlagSbeFull.isEnabledAndIgnoreFCV()) {
-        // Retrieve plan cache entries from the SBE plan cache.
-        const auto cacheKeyFilter = [uuid = collection->uuid(),
-                                     collVersion = collQueryInfo.getPlanCacheInvalidatorVersion()](
-                                        const sbe::PlanCacheKey& key) {
-            // Only fetch plan cache entries with keys matching given UUID and collectionVersion.
-            return uuid == key.getMainCollectionState().uuid &&
-                collVersion == key.getMainCollectionState().version;
-        };
+    // Retrieve plan cache entries from the SBE plan cache.
+    const auto cacheKeyFilter = [uuid = collection->uuid(),
+                                 collVersion = collQueryInfo.getPlanCacheInvalidatorVersion()](
+                                    const sbe::PlanCacheKey& key) {
+        // Only fetch plan cache entries with keys matching given UUID and collectionVersion.
+        return uuid == key.getMainCollectionState().uuid &&
+            collVersion == key.getMainCollectionState().version;
+    };
 
-        auto planCacheEntriesSBE =
-            sbe::getPlanCache(opCtx).getMatchingStats(cacheKeyFilter, serializer, predicate);
+    auto planCacheEntriesSBE =
+        sbe::getPlanCache(opCtx).getMatchingStats(cacheKeyFilter, serializer, predicate);
+
+    planCacheEntries.insert(
+        planCacheEntries.end(), planCacheEntriesSBE.begin(), planCacheEntriesSBE.end());
 
-        planCacheEntries.insert(
-            planCacheEntries.end(), planCacheEntriesSBE.begin(), planCacheEntriesSBE.end());
-    }
 
     return planCacheEntries;
 }
@@ -620,6 +672,10 @@ BSONObj CommonMongodProcessInterface::_reportCurrentOpForClient(
     OperationContext* clientOpCtx = client->getOperationContext();
 
     if (clientOpCtx) {
+        if (CurOp::get(clientOpCtx)->getShouldOmitDiagnosticInformation()) {
+            return builder.obj();
+        }
+
         if (auto txnParticipant = TransactionParticipant::get(clientOpCtx)) {
             txnParticipant.reportUnstashedState(clientOpCtx, &builder);
         }
@@ -680,12 +736,19 @@ void CommonMongodProcessInterface::_reportCurrentOpsForIdleSessions(
     });
 }
 
+void CommonMongodProcessInterface::_reportCurrentOpsForQueryAnalysis(
+    OperationContext* opCtx, std::vector<BSONObj>* ops) const {
+    if (analyze_shard_key::supportsPersistingSampledQueries(opCtx)) {
+        analyze_shard_key::QueryAnalysisSampleTracker::get(opCtx).reportForCurrentOp(ops);
+    }
+}
+
 std::unique_ptr<CollatorInterface> CommonMongodProcessInterface::_getCollectionDefaultCollator(
-    OperationContext* opCtx, StringData dbName, UUID collectionUUID) {
+    OperationContext* opCtx, const DatabaseName& dbName, UUID collectionUUID) {
     auto it = _collatorCache.find(collectionUUID);
     if (it == _collatorCache.end()) {
         auto collator = [&]() -> std::unique_ptr<CollatorInterface> {
-            AutoGetCollection autoColl(opCtx, {dbName.toString(), collectionUUID}, MODE_IS);
+            AutoGetCollection autoColl(opCtx, {dbName, collectionUUID}, MODE_IS);
             if (!autoColl.getCollection()) {
                 // This collection doesn't exist, so assume a nullptr default collation
                 return nullptr;
@@ -714,15 +777,15 @@ std::pair<std::set<FieldPath>, boost::optional<ChunkVersion>>
 CommonMongodProcessInterface::ensureFieldsUniqueOrResolveDocumentKey(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     boost::optional<std::set<FieldPath>> fieldPaths,
-    boost::optional<ChunkVersion> targetCollectionVersion,
+    boost::optional<ChunkVersion> targetCollectionPlacementVersion,
     const NamespaceString& outputNs) const {
     uassert(51123,
             "Unexpected target chunk version specified",
-            !targetCollectionVersion || expCtx->fromMongos);
+            !targetCollectionPlacementVersion || expCtx->fromMongos);
 
     if (!fieldPaths) {
         uassert(51124, "Expected fields to be provided from mongos", !expCtx->fromMongos);
-        return {std::set<FieldPath>{"_id"}, targetCollectionVersion};
+        return {std::set<FieldPath>{"_id"}, targetCollectionPlacementVersion};
     }
 
     // Make sure the 'fields' array has a supporting index. Skip this check if the command is sent
@@ -732,60 +795,7 @@ CommonMongodProcessInterface::ensureFieldsUniqueOrResolveDocumentKey(
                 "Cannot find index to verify that join fields will be unique",
                 fieldsHaveSupportingUniqueIndex(expCtx, outputNs, *fieldPaths));
     }
-    return {*fieldPaths, targetCollectionVersion};
-}
-
-write_ops::InsertCommandRequest CommonMongodProcessInterface::buildInsertOp(
-    const NamespaceString& nss, std::vector<BSONObj>&& objs, bool bypassDocValidation) {
-    write_ops::InsertCommandRequest insertOp(nss);
-    insertOp.setDocuments(std::move(objs));
-    insertOp.setWriteCommandRequestBase([&] {
-        write_ops::WriteCommandRequestBase wcb;
-        wcb.setOrdered(false);
-        wcb.setBypassDocumentValidation(bypassDocValidation);
-        return wcb;
-    }());
-    return insertOp;
-}
-
-write_ops::UpdateCommandRequest CommonMongodProcessInterface::buildUpdateOp(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    const NamespaceString& nss,
-    BatchedObjects&& batch,
-    UpsertType upsert,
-    bool multi) {
-    write_ops::UpdateCommandRequest updateOp(nss);
-    updateOp.setUpdates([&] {
-        std::vector<write_ops::UpdateOpEntry> updateEntries;
-        for (auto&& obj : batch) {
-            updateEntries.push_back([&] {
-                write_ops::UpdateOpEntry entry;
-                auto&& [q, u, c] = obj;
-                entry.setQ(std::move(q));
-                entry.setU(std::move(u));
-                entry.setC(std::move(c));
-                entry.setUpsert(upsert != UpsertType::kNone);
-                entry.setUpsertSupplied(
-                    {{entry.getUpsert(), upsert == UpsertType::kInsertSuppliedDoc}});
-                entry.setMulti(multi);
-                return entry;
-            }());
-        }
-        return updateEntries;
-    }());
-    updateOp.setWriteCommandRequestBase([&] {
-        write_ops::WriteCommandRequestBase wcb;
-        wcb.setOrdered(false);
-        wcb.setBypassDocumentValidation(expCtx->bypassDocumentValidation);
-        return wcb;
-    }());
-    auto [constants, letParams] =
-        expCtx->variablesParseState.transitionalCompatibilitySerialize(expCtx->variables);
-    updateOp.setLegacyRuntimeConstants(std::move(constants));
-    if (!letParams.isEmpty()) {
-        updateOp.setLet(std::move(letParams));
-    }
-    return updateOp;
+    return {*fieldPaths, targetCollectionPlacementVersion};
 }
 
 BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
@@ -797,14 +807,8 @@ BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
 
     BSONObjBuilder newCmd;
     newCmd.append("internalRenameIfOptionsAndIndexesMatch", 1);
-    // TODO SERVER-62114 Change to check for upgraded FCV rather than feature flag
-    if (gFeatureFlagRequireTenantID.isEnabled(serverGlobalParams.featureCompatibility)) {
-        newCmd.append("from", sourceNs.ns());
-        newCmd.append("to", targetNs.ns());
-    } else {
-        newCmd.append("from", sourceNs.toStringWithTenantId());
-        newCmd.append("to", targetNs.toStringWithTenantId());
-    }
+    newCmd.append("from", NamespaceStringUtil::serialize(sourceNs));
+    newCmd.append("to", NamespaceStringUtil::serialize(targetNs));
     newCmd.append("collectionOptions", originalCollectionOptions);
     BSONArrayBuilder indexArrayBuilder(newCmd.subarrayStart("indexes"));
     for (auto&& index : originalIndexes) {
@@ -814,6 +818,27 @@ BSONObj CommonMongodProcessInterface::_convertRenameToInternalRename(
     return newCmd.obj();
 }
 
+void CommonMongodProcessInterface::_handleTimeseriesCreateError(const DBException& ex,
+                                                                OperationContext* opCtx,
+                                                                const NamespaceString& ns,
+                                                                TimeseriesOptions userOpts) {
+    // If we receive a NamespaceExists error for a time-series view that has the same
+    // specification as the time-series view we wanted to create, we should not throw an
+    // error. The user is allowed to overwrite an existing time-series collection when
+    // entering this function.
+    auto view = CollectionCatalog::get(opCtx)->lookupView(opCtx, ns);
+    // Confirming the error is NamespaceExists and that there is a time-series view in that
+    // namespace.
+    if (ex.code() != ErrorCodes::NamespaceExists || !view || !view->timeseries()) {
+        throw;
+    }
+    // Confirming the time-series options of the existing view are the same as expected.
+    auto timeseriesOpts = mongo::timeseries::getTimeseriesOptions(opCtx, ns, true);
+    if (!timeseriesOpts || !mongo::timeseries::optionsAreEqual(timeseriesOpts.value(), userOpts)) {
+        throw;
+    }
+}
+
 void CommonMongodProcessInterface::writeRecordsToRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     RecordStore* rs,
@@ -875,9 +900,9 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocumentLoca
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const NamespaceString& nss,
     const Document& documentKey) {
-    AutoGetCollectionForRead autoColl(expCtx->opCtx, nss);
+    AutoGetCollectionForReadMaybeLockFree autoColl(expCtx->opCtx, nss);
     BSONObj document;
-    if (!Helpers::findById(expCtx->opCtx, nss.ns(), documentKey.toBson(), document)) {
+    if (!Helpers::findById(expCtx->opCtx, nss, documentKey.toBson(), document)) {
         return boost::none;
     }
     return Document(document).getOwned();
