@@ -60,7 +60,8 @@ public:
     std::deque<BSONObj> listCatalog(OperationContext* opCtx) const final;
 
     boost::optional<BSONObj> getCatalogEntry(OperationContext* opCtx,
-                                             const NamespaceString& ns) const final;
+                                             const NamespaceString& ns,
+                                             const boost::optional<UUID>& collUUID) const final;
 
     void appendLatencyStats(OperationContext* opCtx,
                             const NamespaceString& nss,
@@ -69,7 +70,8 @@ public:
     Status appendStorageStats(OperationContext* opCtx,
                               const NamespaceString& nss,
                               const StorageStatsSpec& spec,
-                              BSONObjBuilder* builder) const final;
+                              BSONObjBuilder* builder,
+                              const boost::optional<BSONObj>& filterObj) const final;
     Status appendRecordCount(OperationContext* opCtx,
                              const NamespaceString& nss,
                              BSONObjBuilder* builder) const final;
@@ -78,7 +80,8 @@ public:
                                 BSONObjBuilder* builder) const final override;
     BSONObj getCollectionOptions(OperationContext* opCtx, const NamespaceString& nss) override;
     std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipelineForLocalRead(
-        Pipeline* pipeline) final;
+        Pipeline* pipeline,
+        boost::optional<const AggregateCommandRequest&> aggRequest = boost::none) final;
     std::string getShardName(OperationContext* opCtx) const final;
 
     bool inShardedEnvironment(OperationContext* opCtx) const final;
@@ -103,10 +106,11 @@ public:
     std::unique_ptr<ResourceYielder> getResourceYielder(StringData cmdName) const final;
 
     std::pair<std::set<FieldPath>, boost::optional<ChunkVersion>>
-    ensureFieldsUniqueOrResolveDocumentKey(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                           boost::optional<std::set<FieldPath>> fieldPaths,
-                                           boost::optional<ChunkVersion> targetCollectionVersion,
-                                           const NamespaceString& outputNs) const final;
+    ensureFieldsUniqueOrResolveDocumentKey(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        boost::optional<std::set<FieldPath>> fieldPaths,
+        boost::optional<ChunkVersion> targetCollectionPlacementVersion,
+        const NamespaceString& outputNs) const final;
 
     std::unique_ptr<TemporaryRecordStore> createTemporaryRecordStore(
         const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const final;
@@ -142,23 +146,6 @@ protected:
         const Document& documentKey,
         MakePipelineOptions opts);
 
-    /**
-     * Builds an ordered insert op on namespace 'nss' and documents to be written 'objs'.
-     */
-    write_ops::InsertCommandRequest buildInsertOp(const NamespaceString& nss,
-                                                  std::vector<BSONObj>&& objs,
-                                                  bool bypassDocValidation);
-
-    /**
-     * Builds an ordered update op on namespace 'nss' with update entries contained in 'batch'.
-     */
-    write_ops::UpdateCommandRequest buildUpdateOp(
-        const boost::intrusive_ptr<ExpressionContext>& expCtx,
-        const NamespaceString& nss,
-        BatchedObjects&& batch,
-        UpsertType upsert,
-        bool multi);
-
     BSONObj _reportCurrentOpForClient(OperationContext* opCtx,
                                       Client* client,
                                       CurrentOpTruncateMode truncateOps,
@@ -177,6 +164,9 @@ protected:
                                                  CurrentOpSessionsMode sessionMode,
                                                  std::vector<BSONObj>* ops) const final;
 
+    void _reportCurrentOpsForQueryAnalysis(OperationContext* opCtx,
+                                           std::vector<BSONObj>* ops) const final;
+
     /**
      * Converts a renameCollection command into an internalRenameIfOptionsAndIndexesMatch command.
      */
@@ -186,6 +176,11 @@ protected:
                                            const BSONObj& originalCollectionOptions,
                                            const std::list<BSONObj>& originalIndexes);
 
+    void _handleTimeseriesCreateError(const DBException& ex,
+                                      OperationContext* opCtx,
+                                      const NamespaceString& ns,
+                                      TimeseriesOptions userOpts);
+
 private:
     /**
      * Looks up the collection default collator for the collection given by 'collectionUUID'. A
@@ -195,7 +190,7 @@ private:
      * collation.
      */
     std::unique_ptr<CollatorInterface> _getCollectionDefaultCollator(OperationContext* opCtx,
-                                                                     StringData dbName,
+                                                                     const DatabaseName& dbName,
                                                                      UUID collectionUUID);
 
     std::map<UUID, std::unique_ptr<const CollatorInterface>> _collatorCache;
