@@ -29,6 +29,7 @@
 
 #include "mongo/db/pipeline/variables.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/client.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_dependencies.h"
@@ -41,6 +42,21 @@
 
 namespace mongo {
 
+namespace {
+
+// We need to be careful when serializing values, e.g. to populate the 'let' parameter of a command
+// to be sent over the wire. First, missing values should be serialied as $$REMOVE, otherwise they
+// might be incorrectly omitted or serialized as empty objects ({}). Also, we should wrap values in
+// $literal to avoid a scenario like the following: suppose we had a user-defined 'let' specified as
+// {let: {a: {$literal: "$notAFieldName"}}}. On mongos, this will be evaluated to the string
+// "$notAFieldName". When we serialize it again for the shard commands, it must appear as {$literal:
+// "$notAFieldName"}, not simply "$notAFieldName", since the latter will be treated as a field name
+// by mongods.
+Value serializeValue(Value val) {
+    return val.missing() ? Value("$$REMOVE"_sd) : Value(DOC("$literal" << val));
+}
+}  // namespace
+
 using namespace std::string_literals;
 
 constexpr Variables::Id Variables::kRootId;
@@ -53,6 +69,7 @@ constexpr StringData kClusterTimeName = "CLUSTER_TIME"_sd;
 constexpr StringData kJsScopeName = "JS_SCOPE"_sd;
 constexpr StringData kIsMapReduceName = "IS_MR"_sd;
 constexpr StringData kSearchMetaName = "SEARCH_META"_sd;
+constexpr StringData kUserRolesName = "USER_ROLES"_sd;
 
 const StringMap<Variables::Id> Variables::kBuiltinVarNameToId = {
     {kRootName.rawData(), kRootId},
@@ -61,7 +78,8 @@ const StringMap<Variables::Id> Variables::kBuiltinVarNameToId = {
     {kClusterTimeName.rawData(), kClusterTimeId},
     {kJsScopeName.rawData(), kJsScopeId},
     {kIsMapReduceName.rawData(), kIsMapReduceId},
-    {kSearchMetaName.rawData(), kSearchMetaId}};
+    {kSearchMetaName.rawData(), kSearchMetaId},
+    {kUserRolesName.rawData(), kUserRolesId}};
 
 const std::map<Variables::Id, std::string> Variables::kIdToBuiltinVarName = {
     {kRootId, kRootName.rawData()},
@@ -70,7 +88,8 @@ const std::map<Variables::Id, std::string> Variables::kIdToBuiltinVarName = {
     {kClusterTimeId, kClusterTimeName.rawData()},
     {kJsScopeId, kJsScopeName.rawData()},
     {kIsMapReduceId, kIsMapReduceName.rawData()},
-    {kSearchMetaId, kSearchMetaName.rawData()}};
+    {kSearchMetaId, kSearchMetaName.rawData()},
+    {kUserRolesId, kUserRolesName.rawData()}};
 
 const std::map<StringData, std::function<void(const Value&)>> Variables::kSystemVarValidators = {
     {kNowName,
@@ -94,11 +113,18 @@ const std::map<StringData, std::function<void(const Value&)>> Variables::kSystem
                                << typeName(value.getType()),
                  value.getType() == BSONType::Object);
      }},
-    {kIsMapReduceName, [](const auto& value) {
+    {kIsMapReduceName,
+     [](const auto& value) {
          uassert(ErrorCodes::TypeMismatch,
                  str::stream() << "$$IS_MR must have a bool value, found "
                                << typeName(value.getType()),
                  value.getType() == BSONType::Bool);
+     }},
+    {kUserRolesName, [](const auto& value) {
+         uassert(ErrorCodes::TypeMismatch,
+                 str::stream() << "$$USER_ROLES must have an array value, found "
+                               << typeName(value.getType()),
+                 value.getType() == BSONType::Array);
      }}};
 
 void Variables::setValue(Id id, const Value& value, bool isConstant) {
@@ -158,6 +184,7 @@ Value Variables::getValue(Id id, const Document& root) const {
             case Variables::kClusterTimeId:
             case Variables::kJsScopeId:
             case Variables::kIsMapReduceId:
+            case Variables::kUserRolesId:
                 if (auto it = _definitions.find(id); it != _definitions.end()) {
                     return it->second.value;
                 }
@@ -205,6 +232,9 @@ void Variables::setLegacyRuntimeConstants(const LegacyRuntimeConstants& constant
     if (constants.getIsMapReduce()) {
         _definitions[kIsMapReduceId] = {Value(*constants.getIsMapReduce()), constant};
     }
+    if (constants.getUserRoles()) {
+        _definitions[kUserRolesId] = {Value(constants.getUserRoles().value()), constant};
+    }
 }
 
 void Variables::setDefaultRuntimeConstants(OperationContext* opCtx) {
@@ -214,7 +244,11 @@ void Variables::setDefaultRuntimeConstants(OperationContext* opCtx) {
 void Variables::appendSystemVariables(BSONObjBuilder& bob) const {
     for (auto&& [name, id] : kBuiltinVarNameToId) {
         if (hasValue(id)) {
-            bob << name << getValue(id);
+            // We should serialize the system variables using $literal (as we do with the
+            // user-defined variables) so that they get parsed the same way (for example, not using
+            // $literal to parse a variable value that is an array causes the expression context to
+            // be marked as SBE incompatible through ExpressionArray).
+            bob << name << Value(DOC("$literal" << getValue(id)));
         }
     }
 }
@@ -322,6 +356,16 @@ LegacyRuntimeConstants Variables::transitionalExtractRuntimeConstants() const {
                     extracted.setIsMapReduce(value.getBool());
                     break;
                 }
+                case kUserRolesId: {
+                    invariant(value.getType() == BSONType::Array);
+                    BSONArrayBuilder bab;
+                    for (const auto& val : value.getArray()) {
+                        invariant(val.getType() == BSONType::Object);
+                        bab.append(val.getDocument().toBson());
+                    }
+                    extracted.setUserRoles(bab.arr());
+                    break;
+                }
                 default:
                     MONGO_UNREACHABLE;
             }
@@ -330,8 +374,29 @@ LegacyRuntimeConstants Variables::transitionalExtractRuntimeConstants() const {
     return extracted;
 }
 
+void Variables::defineUserRoles(OperationContext* opCtx) {
+    auto* as = AuthorizationSession::get(opCtx->getClient());
+
+    auto roleNames =
+        as->isImpersonating() ? as->getImpersonatedRoleNames() : as->getAuthenticatedRoleNames();
+    // Marshall current effective user roles into an array of
+    // {_id: ..., db: ..., role: ...} objects for the $$USER_ROLES variable.
+    BSONArrayBuilder builder;
+    for (; roleNames.more(); roleNames.next()) {
+        BSONObjBuilder bob(builder.subobjStart());
+
+        bob.append("_id"_sd, roleNames->getUnambiguousName());
+        bob.append("role"_sd, roleNames->getRole());
+        bob.append("db"_sd, roleNames->getDB());
+        bob.doneFast();
+    }
+
+    _definitions[kUserRolesId] = {Value(builder.arr()), true /* isConst */};
+}
+
 Variables::Id VariablesParseState::defineVariable(StringData name) {
-    // Caller should have validated before hand by using variableValidationvalidateNameForUserWrite.
+    // Caller should have validated before hand by using
+    // variableValidation::validateNameForUserWrite.
     massert(17275,
             "Can't redefine a non-user-writable variable",
             Variables::kBuiltinVarNameToId.find(name) == Variables::kBuiltinVarNameToId.end());
@@ -375,8 +440,9 @@ std::set<Variables::Id> VariablesParseState::getDefinedVariableIDs() const {
 BSONObj VariablesParseState::serialize(const Variables& vars) const {
     auto bob = BSONObjBuilder{};
     for (auto&& [var_name, id] : _variables)
-        if (vars.hasValue(id))
-            bob << var_name << Value(DOC("$literal" << vars.getValue(id)));
+        if (vars.hasValue(id)) {
+            bob << var_name << serializeValue(vars.getValue(id));
+        }
 
     // System variables have to be added separately since the variable IDs are reserved and not
     // allocated like normal variables, and so not present in '_variables'.
@@ -388,8 +454,9 @@ std::pair<LegacyRuntimeConstants, BSONObj> VariablesParseState::transitionalComp
     const Variables& vars) const {
     auto bob = BSONObjBuilder{};
     for (auto&& [var_name, id] : _variables)
-        if (vars.hasValue(id))
-            bob << var_name << Value(DOC("$literal" << vars.getValue(id)));
+        if (vars.hasValue(id)) {
+            bob << var_name << serializeValue(vars.getValue(id));
+        }
 
     return {vars.transitionalExtractRuntimeConstants(), bob.obj()};
 }
