@@ -44,10 +44,10 @@
 #include "mongo/db/pipeline/document_source_lookup.h"
 #include "mongo/db/query/analyze_regex.h"
 #include "mongo/db/query/projection.h"
-#include "mongo/db/query/query_feature_flags_gen.h"
-#include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/query/query_decorations.h"
 #include "mongo/db/query/tree_walker.h"
 #include "mongo/logv2/log.h"
+#include "mongo/platform/overflow_arithmetic.h"
 #include "mongo/util/base64.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
@@ -78,33 +78,6 @@ bool isQueryNegatingEqualToNull(const mongo::MatchExpression* tree) {
 
 namespace {
 
-// Delimiters for cache key encoding.
-const char kEncodeChildrenBegin = '[';
-const char kEncodeChildrenEnd = ']';
-const char kEncodeChildrenSeparator = ',';
-const char kEncodeCollationSection = '#';
-const char kEncodeProjectionSection = '|';
-const char kEncodeProjectionRequirementSeparator = '-';
-const char kEncodeRegexFlagsSeparator = '/';
-const char kEncodeSortSection = '~';
-const char kEncodeEngineSection = '@';
-const char kEncodePipelineSection = '^';
-
-// These special bytes are used in the encoding of auto-parameterized match expressions in the SBE
-// plan cache key.
-
-// Precedes the id number of a parameter marker.
-const char kEncodeParamMarker = '?';
-// Precedes the encoding of a constant when that constant has not been auto-paramterized. The
-// constant is typically encoded as a BSON type byte followed by a BSON value (without the
-// BSONElement's field name).
-const char kEncodeConstantLiteralMarker = ':';
-// Precedes a byte which encodes the bounds tightness associated with a predicate. The structure of
-// the plan (i.e. presence of filters) is affected by bounds tightness. Therefore, if different
-// parameter values can result in different tightnesses, this must be explicitly encoded into the
-// plan cache key.
-const char kEncodeBoundsTightnessDiscriminator = ':';
-
 /**
  * AppendChar provides the compiler with a type for a "appendChar(...)" member function.
  */
@@ -130,15 +103,11 @@ void encodeUserString(StringData s, BuilderType* builder) {
             case kEncodeChildrenBegin:
             case kEncodeChildrenEnd:
             case kEncodeChildrenSeparator:
-            case kEncodeCollationSection:
-            case kEncodeProjectionSection:
+            case kEncodeSectionDelimiter:
             case kEncodeProjectionRequirementSeparator:
             case kEncodeRegexFlagsSeparator:
-            case kEncodeSortSection:
-            case kEncodeEngineSection:
             case kEncodeParamMarker:
             case kEncodeConstantLiteralMarker:
-            case kEncodePipelineSection:
             case '\\':
                 if constexpr (hasAppendChar<BuilderType>) {
                     builder->appendChar('\\');
@@ -263,6 +232,9 @@ const char* encodeMatchType(MatchExpression::MatchType mt) {
         case MatchExpression::INTERNAL_EXPR_LTE:
             return "ele";
 
+        case MatchExpression::INTERNAL_EQ_HASHED_KEY:
+            return "ieh";
+
         case MatchExpression::INTERNAL_SCHEMA_ALL_ELEM_MATCH_FROM_INDEX:
             return "internalSchemaAllElemMatchFromIndex";
 
@@ -411,13 +383,13 @@ char encodeEnum(T val) {
 }
 
 void encodeCollation(const CollatorInterface* collation, StringBuilder* keyBuilder) {
+    *keyBuilder << kEncodeSectionDelimiter;
     if (!collation) {
         return;
     }
 
     const Collation& spec = collation->getSpec();
 
-    *keyBuilder << kEncodeCollationSection;
     *keyBuilder << spec.getLocale();
     *keyBuilder << spec.getCaseLevel();
 
@@ -437,11 +409,11 @@ void encodeCollation(const CollatorInterface* collation, StringBuilder* keyBuild
 
 void encodePipeline(const std::vector<std::unique_ptr<InnerPipelineStageInterface>>& pipeline,
                     BufBuilder* bufBuilder) {
-    bufBuilder->appendChar(kEncodePipelineSection);
+    bufBuilder->appendChar(kEncodeSectionDelimiter);
     for (auto& stage : pipeline) {
         std::vector<Value> serializedArray;
         if (auto lookupStage = dynamic_cast<DocumentSourceLookUp*>(stage->documentSource())) {
-            lookupStage->serializeToArray(serializedArray, boost::none);
+            lookupStage->serializeToArray(serializedArray);
             tassert(6443201,
                     "$lookup stage isn't serialized to a single bson object",
                     serializedArray.size() == 1 && serializedArray[0].getType() == Object);
@@ -487,7 +459,9 @@ void encodeRegexFlagsForMatch(RegexIterator first, RegexIterator last, StringBui
 // Helper overload to prepare a vector of unique_ptrs for the heavy-lifting function above.
 void encodeRegexFlagsForMatch(const std::vector<std::unique_ptr<RegexMatchExpression>>& regexes,
                               StringBuilder* keyBuilder) {
-    const auto transformFunc = [](const auto& regex) { return regex.get(); };
+    const auto transformFunc = [](const auto& regex) {
+        return regex.get();
+    };
     encodeRegexFlagsForMatch(boost::make_transform_iterator(regexes.begin(), transformFunc),
                              boost::make_transform_iterator(regexes.end(), transformFunc),
                              keyBuilder);
@@ -568,12 +542,11 @@ void encodeKeyForMatch(const MatchExpression* tree, StringBuilder* keyBuilder) {
  * FindCommandRequest.
  */
 void encodeKeyForSort(const BSONObj& sortObj, StringBuilder* keyBuilder) {
+    *keyBuilder << kEncodeSectionDelimiter;
     if (sortObj.isEmpty()) {
         return;
     }
 
-    *keyBuilder << kEncodeSortSection;
-
     BSONObjIterator it(sortObj);
     while (it.more()) {
         BSONElement elt = it.next();
@@ -608,6 +581,7 @@ void encodeKeyForSort(const BSONObj& sortObj, StringBuilder* keyBuilder) {
  * expressions), the projection section is empty.
  */
 void encodeKeyForProj(const projection_ast::Projection* proj, StringBuilder* keyBuilder) {
+    *keyBuilder << kEncodeSectionDelimiter;
     if (!proj || proj->requiresDocument()) {
         // Don't encode anything for the projection section to indicate the entire document is
         // required.
@@ -622,10 +596,6 @@ void encodeKeyForProj(const projection_ast::Projection* proj, StringBuilder* key
         return;
     }
 
-    // If the projection just re-writes the entire document and has no dependencies on the original
-    // document (e.g. {a: "foo", _id: 0}), then just include the projection delimiter.
-    *keyBuilder << kEncodeProjectionSection;
-
     // Encode the fields required by the projection in order.
     bool isFirst = true;
     for (auto&& requiredField : requiredFields) {
@@ -645,17 +615,49 @@ void encodeKeyForProj(const projection_ast::Projection* proj, StringBuilder* key
     }
 }
 
-void encodeFindCommandRequest(const FindCommandRequest& findCommand, BufBuilder* bufBuilder) {
-    if (auto skip = findCommand.getSkip()) {
-        bufBuilder->appendNum(*skip);
-    } else {
-        bufBuilder->appendNum(0);
+/**
+ * Approximate the number of documents to be processed into a small, medium or large category. Best
+ * plans for limit: 10 and limit: 1000 may be different. This allows us to cache different plans for
+ * different cases without unbounded growth of plan cache for each skip and limit value.
+ */
+char getLimitSkipCategory(OperationContext* opCtx,
+                          boost::optional<int64_t> skip,
+                          boost::optional<int64_t> limit) {
+    if (limit.value_or(0) == 1 && !skip) {
+        return '1';
     }
-    if (auto limit = findCommand.getLimit()) {
-        bufBuilder->appendNum(*limit);
-    } else {
-        bufBuilder->appendNum(0);
+
+    size_t limitSkipSum;
+    bool hasOverflowed = overflow::add(static_cast<size_t>(skip.value_or(0)),
+                                       static_cast<size_t>(limit.value_or(0)),
+                                       &limitSkipSum);
+    if (hasOverflowed) {
+        return 'l';
     }
+    size_t planEvaluationMaxResults =
+        QueryKnobConfiguration::decoration(opCtx).getPlanEvaluationMaxResultsForOp();
+    if (limitSkipSum < planEvaluationMaxResults) {
+        return 's';
+    } else if (limitSkipSum < 10 * planEvaluationMaxResults) {
+        return 'm';
+    } else {
+        return 'l';
+    }
+}
+
+void encodeLimitSkip(const CanonicalQuery& cq, BufBuilder* bufBuilder) {
+    boost::optional<int64_t> skip = cq.getFindCommandRequest().getSkip();
+    boost::optional<int64_t> limit = cq.getFindCommandRequest().getLimit();
+    if (!limit && !skip) {
+        return;
+    }
+    bufBuilder->appendChar(skip ? 1 : 0);
+    bufBuilder->appendChar(limit ? 1 : 0);
+    bufBuilder->appendChar(getLimitSkipCategory(cq.getOpCtx(), skip, limit));
+}
+
+void encodeFindCommandRequest(const CanonicalQuery& cq, BufBuilder* bufBuilder) {
+    encodeLimitSkip(cq, bufBuilder);
 
     // Encode a OptionalBool value - 'n' if the value is not specified, 't' for true, and 'f' for
     // false.
@@ -668,6 +670,7 @@ void encodeFindCommandRequest(const FindCommandRequest& findCommand, BufBuilder*
             bufBuilder->appendChar('f');
         }
     };
+    const auto& findCommand = cq.getFindCommandRequest();
     encodeOptionalBool(findCommand.getAllowDiskUse());
     encodeOptionalBool(findCommand.getReturnKey());
     encodeOptionalBool(findCommand.getRequestResumeToken());
@@ -693,7 +696,7 @@ void encodeFindCommandRequest(const FindCommandRequest& findCommand, BufBuilder*
 
 namespace canonical_query_encoder {
 
-CanonicalQuery::QueryShapeString encode(const CanonicalQuery& cq) {
+CanonicalQuery::QueryShapeString encodeClassic(const CanonicalQuery& cq) {
     StringBuilder keyBuilder;
     encodeKeyForMatch(cq.root(), &keyBuilder);
     encodeKeyForSort(cq.getFindCommandRequest().getSort(), &keyBuilder);
@@ -702,7 +705,13 @@ CanonicalQuery::QueryShapeString encode(const CanonicalQuery& cq) {
 
     // This encoding can be removed once the classic query engine reaches EOL and SBE is used
     // exclusively for all query execution.
-    keyBuilder << kEncodeEngineSection << (cq.getForceClassicEngine() ? "f" : "t");
+    keyBuilder << kEncodeSectionDelimiter << (cq.getForceClassicEngine() ? "f" : "t");
+
+    // The apiStrict flag can cause the query to see different set of indexes. For example, all
+    // sparse indexes will be ignored with apiStrict is used.
+    const bool apiStrict =
+        cq.getOpCtx() && APIParameters::get(cq.getOpCtx()).getAPIStrict().value_or(false);
+    keyBuilder << (apiStrict ? "t" : "f");
 
     return keyBuilder.str();
 }
@@ -718,10 +727,10 @@ namespace {
 class MatchExpressionSbePlanCacheKeySerializationVisitor final
     : public MatchExpressionConstVisitor {
 public:
-    explicit MatchExpressionSbePlanCacheKeySerializationVisitor(BufBuilder* builder)
-        : _builder(builder) {
-        invariant(_builder);
-    }
+    explicit MatchExpressionSbePlanCacheKeySerializationVisitor(OperationContext* opCtx,
+                                                                BufBuilder* builder,
+                                                                bool hasSort)
+        : _opCtx(opCtx), _builder(builder), _hasSort(hasSort) {}
 
     void visit(const BitsAllClearMatchExpression* expr) final {
         encodeBitTestExpression(expr);
@@ -762,6 +771,20 @@ public:
 
     void visit(const InMatchExpression* expr) final {
         encodeSingleParamPathNode(expr);
+        // Encode the number of unique $in values as part of the plan cache key. If the query is
+        // optimized by exploding for sort, the number of unique elements in $in determines how many
+        // merge branches we get in the query plan. If there is no sort, this is not necessary.
+        if (expr->getInputParamId() && _hasSort) {
+            size_t maxScansToExplode =
+                QueryKnobConfiguration::decoration(_opCtx).getMaxScansToExplodeForOp();
+            // Assume that $in have n elements.
+            // If n is less than or equal to maxScansToExplode, then it is possible that explode for
+            // sort optimization will be used, so we need to add n to plan cache key.
+            // If n is greater than maxScansToExplode, then we can't explode it for sort. So we can
+            // use the same value of (maxScansToExplode + 1) for all queries, so they can share a
+            // plan cache entry.
+            _builder->appendNum(std::min(maxScansToExplode + 1, expr->getEqualities().size()));
+        }
     }
 
     void visit(const ModMatchExpression* expr) final {
@@ -841,6 +864,7 @@ public:
     void visit(const AlwaysFalseMatchExpression* expr) final {}
     void visit(const AlwaysTrueMatchExpression* expr) final {}
     void visit(const AndMatchExpression* expr) final {}
+    void visit(const ElemMatchValueMatchExpression* matchExpr) final {}
     void visit(const ElemMatchObjectMatchExpression* matchExpr) final {}
     void visit(const NorMatchExpression* expr) final {}
     void visit(const NotMatchExpression* expr) final {}
@@ -853,11 +877,12 @@ public:
     void visit(const InternalExprLTEMatchExpression* expr) final {}
     void visit(const InternalExprLTMatchExpression* expr) final {}
 
+
     /**
      * These node types are not yet supported in SBE.
      */
-    void visit(const ElemMatchValueMatchExpression* matchExpr) final {
-        MONGO_UNREACHABLE_TASSERT(6142110);
+    void visit(const InternalEqHashedKey* expr) final {
+        MONGO_UNREACHABLE_TASSERT(7281402);
     }
     void visit(const GeoMatchExpression* expr) final {
         MONGO_UNREACHABLE_TASSERT(6142111);
@@ -934,9 +959,6 @@ public:
     void visit(const TwoDPtInAnnulusExpression* expr) final {
         MONGO_UNREACHABLE_TASSERT(6142133);
     }
-    void visit(const EncryptedBetweenMatchExpression* expr) final {
-        MONGO_UNREACHABLE_TASSERT(6762801);
-    }
 
 private:
     /**
@@ -1037,7 +1059,14 @@ private:
         _builder->appendBuf(elem.value(), elem.valuesize());
     }
 
+    OperationContext* const _opCtx;
     BufBuilder* const _builder;
+    // Whether there is a sort absorbed by the Canonical query. Note: '_hasSort' is true only when
+    // there is a sort that can be used for explode for sort optimization. For a $match stage,
+    // '_hasSort' should be false since $match does not perform index selection. In cases where a
+    // $sort is not absorbed by the canonical query '_hasSort' should be false since we only perform
+    // explode for sort using the sort in canonical query.
+    bool _hasSort;
 };
 
 /**
@@ -1049,8 +1078,10 @@ private:
  */
 class MatchExpressionSbePlanCacheKeySerializationWalker {
 public:
-    explicit MatchExpressionSbePlanCacheKeySerializationWalker(BufBuilder* builder)
-        : _builder{builder}, _visitor{_builder} {
+    explicit MatchExpressionSbePlanCacheKeySerializationWalker(OperationContext* opCtx,
+                                                               BufBuilder* builder,
+                                                               bool hasSort)
+        : _builder{builder}, _visitor{opCtx, _builder, hasSort} {
         invariant(_builder);
     }
 
@@ -1087,16 +1118,16 @@ private:
  * following property: Two match expression trees which are identical after auto-parameterization
  * have the same key, otherwise the keys must differ.
  */
-void encodeKeyForAutoParameterizedMatchSBE(MatchExpression* matchExpr, BufBuilder* builder) {
-    MatchExpressionSbePlanCacheKeySerializationWalker walker{builder};
+void encodeKeyForAutoParameterizedMatchSBE(OperationContext* opCtx,
+                                           MatchExpression* matchExpr,
+                                           BufBuilder* builder,
+                                           bool hasSort) {
+    MatchExpressionSbePlanCacheKeySerializationWalker walker{opCtx, builder, hasSort};
     tree_walker::walk<true, MatchExpression>(matchExpr, &walker);
 }
 }  // namespace
 
 std::string encodeSBE(const CanonicalQuery& cq) {
-    tassert(6512900,
-            "using the SBE plan cache key encoding requires SBE to be fully enabled",
-            feature_flags::gFeatureFlagSbeFull.isEnabledAndIgnoreFCV());
     tassert(6142104,
             "attempting to encode SBE plan cache key for SBE-incompatible query",
             cq.isSbeCompatible());
@@ -1104,6 +1135,7 @@ std::string encodeSBE(const CanonicalQuery& cq) {
     const auto& filter = cq.getQueryObj();
     const auto& proj = cq.getFindCommandRequest().getProjection();
     const auto& sort = cq.getFindCommandRequest().getSort();
+    const auto& hint = cq.getFindCommandRequest().getHint();
 
     StringBuilder strBuilder;
     encodeKeyForSort(sort, &strBuilder);
@@ -1113,16 +1145,33 @@ std::string encodeSBE(const CanonicalQuery& cq) {
     // A constant for reserving buffer size. It should be large enough to reserve the space required
     // to encode various properties from the FindCommandRequest and query knobs.
     const int kBufferSizeConstant = 200;
-    size_t bufSize =
-        filter.objsize() + proj.objsize() + strBuilderEncoded.size() + kBufferSizeConstant;
+    size_t bufSize = filter.objsize() + proj.objsize() + strBuilderEncoded.size() + hint.objsize() +
+        kBufferSizeConstant;
 
     BufBuilder bufBuilder(bufSize);
-    encodeKeyForAutoParameterizedMatchSBE(cq.root(), &bufBuilder);
-
+    encodeKeyForAutoParameterizedMatchSBE(cq.getOpCtx(), cq.root(), &bufBuilder, !sort.isEmpty());
     bufBuilder.appendBuf(proj.objdata(), proj.objsize());
     bufBuilder.appendStr(strBuilderEncoded, false /* includeEndingNull */);
+    bufBuilder.appendChar(kEncodeSectionDelimiter);
+    if (!hint.isEmpty()) {
+        bufBuilder.appendBuf(hint.objdata(), hint.objsize());
+    }
+    bufBuilder.appendChar(kEncodeSectionDelimiter);
+    bufBuilder.appendChar(cq.getForceGenerateRecordId() ? 1 : 0);
+    bufBuilder.appendChar(cq.isCountLike() ? 1 : 0);
+    // The apiStrict flag can cause the query to see different set of indexes. For example, all
+    // sparse indexes will be ignored with apiStrict is used.
+    const bool apiStrict =
+        cq.getOpCtx() && APIParameters::get(cq.getOpCtx()).getAPIStrict().value_or(false);
+    bufBuilder.appendChar(apiStrict ? 1 : 0);
 
-    encodeFindCommandRequest(cq.getFindCommandRequest(), &bufBuilder);
+    // We can wind up with different query plans for aggregate commands if 'needsMerge' is set or
+    // not. For instance, when 'needsMerge' is true, $group queries will produce partial aggregates
+    // as output, and complete output otherwise.
+    const bool needsMerge = cq.getExpCtx()->needsMerge;
+    bufBuilder.appendChar(needsMerge ? 1 : 0);
+
+    encodeFindCommandRequest(cq, &bufBuilder);
 
     encodePipeline(cq.pipeline(), &bufBuilder);
 
@@ -1139,6 +1188,8 @@ CanonicalQuery::PlanCacheCommandKey encodeForPlanCacheCommand(const CanonicalQue
     // be encoded.
     if (!cq.getFindCommandRequest().getCollation().isEmpty()) {
         encodeCollation(cq.getCollator(), &keyBuilder);
+    } else {
+        keyBuilder << kEncodeSectionDelimiter;
     }
 
     return keyBuilder.str();
