@@ -32,15 +32,18 @@
 
 #include <cmath>
 #include <limits>
+#include <s2.h>
+#include <s2cell.h>
+#include <s2regioncoverer.h>
 
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsontypes.h"
 #include "mongo/db/geo/geoconstants.h"
-#include "mongo/db/geo/s2.h"
 #include "mongo/db/index/expression_params.h"
 #include "mongo/db/index/s2_common.h"
 #include "mongo/db/matcher/expression_geo.h"
 #include "mongo/db/matcher/expression_internal_bucket_geo_within.h"
+#include "mongo/db/matcher/expression_internal_eq_hashed_key.h"
 #include "mongo/db/matcher/expression_internal_expr_comparison.h"
 #include "mongo/db/query/analyze_regex.h"
 #include "mongo/db/query/collation/collation_index_key.h"
@@ -52,8 +55,6 @@
 #include "mongo/db/query/planner_wildcard_helpers.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/logv2/log.h"
-#include "third_party/s2/s2cell.h"
-#include "third_party/s2/s2regioncoverer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
@@ -269,9 +270,9 @@ void IndexBoundsBuilder::translateAndUnion(const MatchExpression* expr,
 
 bool typeMatch(const BSONObj& obj) {
     BSONObjIterator it(obj);
-    verify(it.more());
+    MONGO_verify(it.more());
     BSONElement first = it.next();
-    verify(it.more());
+    MONGO_verify(it.more());
     BSONElement second = it.next();
     return first.canonicalType() == second.canonicalType();
 }
@@ -301,8 +302,15 @@ void IndexBoundsBuilder::translate(const MatchExpression* expr,
     // adjusted regardless of the predicate. Having filled out the initial bounds, we apply any
     // necessary changes to the tightness here.
     if (index.type == IndexType::INDEX_WILDCARD) {
-        *tightnessOut =
-            wcp::translateWildcardIndexBoundsAndTightness(index, *tightnessOut, oilOut, ietBuilder);
+        // Check if 'elt' is the wildcard field.
+        BSONElement wildcardElt = wcp::getWildcardField(index);
+
+        // Adjust index bounds and tightness only if the index bounds generated are for the wildcard
+        // field.
+        if (wildcardElt.fieldNameStringData() == elt.fieldNameStringData()) {
+            *tightnessOut = wcp::translateWildcardIndexBoundsAndTightness(
+                index, *tightnessOut, oilOut, ietBuilder);
+        }
     }
 }
 
@@ -685,7 +693,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         BSONObjBuilder bob;
         buildBoundsForQueryElementForLT(dataElt, index.collator, &bob);
         BSONObj dataObj = bob.done().getOwned();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         bool inclusiveBounds = dataElt.type() == BSONType::Array;
         Interval interval =
             makeRangeInterval(dataObj,
@@ -773,7 +781,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         BSONObjBuilder bob;
         buildBoundsForQueryElementForLT(dataElt, index.collator, &bob);
         BSONObj dataObj = bob.done().getOwned();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
 
         bool inclusiveBounds = dataElt.type() == BSONType::Array || typeMatch(dataObj);
         const Interval interval = makeRangeInterval(
@@ -845,7 +853,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         BSONObjBuilder bob;
         buildBoundsForQueryElementForGT(dataElt, index.collator, &bob);
         BSONObj dataObj = bob.done().getOwned();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         bool inclusiveBounds = dataElt.type() == BSONType::Array;
         Interval interval =
             makeRangeInterval(dataObj,
@@ -934,7 +942,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         BSONObjBuilder bob;
         buildBoundsForQueryElementForGT(dataElt, index.collator, &bob);
         BSONObj dataObj = bob.done().getOwned();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         bool inclusiveBounds = dataElt.type() == BSONType::Array || typeMatch(dataObj);
         const Interval interval = makeRangeInterval(
             dataObj, IndexBounds::makeBoundInclusionFromBoundBools(true, inclusiveBounds));
@@ -967,6 +975,24 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         Interval interval = makeRangeInterval(dataObj, BoundInclusion::kIncludeBothStartAndEndKeys);
         oilOut->intervals.push_back(interval);
         *tightnessOut = getInequalityPredicateTightness(interval, dataElt, index);
+    } else if (MatchExpression::INTERNAL_EQ_HASHED_KEY == expr->matchType()) {
+        ON_BLOCK_EXIT([ietBuilder, oilOut] {
+            if (ietBuilder != nullptr) {
+                ietBuilder->addConst(*oilOut);
+            }
+        });
+
+        tassert(7281403, "Expected a hashed index", index.type == INDEX_HASHED);
+
+        const auto* node = static_cast<const InternalEqHashedKey*>(expr);
+        BSONObj dataObj = BSON("" << node->getData());
+
+        Interval interval = makePointInterval(dataObj);
+        oilOut->intervals.push_back(interval);
+
+        // Technically this could be EXACT_FETCH, if such a thing existed. But we don't need to
+        // optimize this that much.
+        *tightnessOut = IndexBoundsBuilder::INEXACT_FETCH;
     } else if (MatchExpression::REGEX == expr->matchType()) {
         const RegexMatchExpression* rme = static_cast<const RegexMatchExpression*>(expr);
         translateRegex(rme, index, oilOut, tightnessOut);
@@ -979,7 +1005,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         bob.appendMinForType("", NumberDouble);
         bob.appendMaxForType("", NumberDouble);
         BSONObj dataObj = bob.obj();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         oilOut->intervals.push_back(
             makeRangeInterval(dataObj, BoundInclusion::kIncludeBothStartAndEndKeys));
         *tightnessOut = IndexBoundsBuilder::INEXACT_COVERED;
@@ -1083,14 +1109,14 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
     } else if (MatchExpression::GEO == expr->matchType()) {
         const GeoMatchExpression* gme = static_cast<const GeoMatchExpression*>(expr);
         if ("2dsphere" == elt.valueStringDataSafe()) {
-            verify(gme->getGeoExpression().getGeometry().hasS2Region());
+            MONGO_verify(gme->getGeoExpression().getGeometry().hasS2Region());
             const S2Region& region = gme->getGeoExpression().getGeometry().getS2Region();
             S2IndexingParams indexParams;
             ExpressionParams::initialize2dsphereParams(index.infoObj, index.collator, &indexParams);
             ExpressionMapping::cover2dsphere(region, indexParams, oilOut);
             *tightnessOut = IndexBoundsBuilder::INEXACT_FETCH;
         } else if ("2d" == elt.valueStringDataSafe()) {
-            verify(gme->getGeoExpression().getGeometry().hasR2Region());
+            MONGO_verify(gme->getGeoExpression().getGeometry().hasR2Region());
             const R2Region& region = gme->getGeoExpression().getGeometry().getR2Region();
 
             ExpressionMapping::cover2d(
@@ -1101,7 +1127,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
             LOGV2_WARNING(20934,
                           "Planner error trying to build geo bounds for an index element",
                           "element"_attr = elt.toString());
-            verify(0);
+            MONGO_verify(0);
         }
     } else if (MatchExpression::INTERNAL_BUCKET_GEO_WITHIN == expr->matchType()) {
         const InternalBucketGeoWithinMatchExpression* ibgwme =
@@ -1125,7 +1151,7 @@ void IndexBoundsBuilder::_translatePredicate(const MatchExpression* expr,
         LOGV2_WARNING(20935,
                       "Planner error while trying to build bounds for expression",
                       "expression"_attr = redact(expr->debugString()));
-        verify(0);
+        MONGO_verify(0);
     }
 }
 
@@ -1136,9 +1162,9 @@ Interval IndexBoundsBuilder::makeRangeInterval(const BSONObj& obj, BoundInclusio
     ret.startInclusive = IndexBounds::isStartIncludedInBound(boundInclusion);
     ret.endInclusive = IndexBounds::isEndIncludedInBound(boundInclusion);
     BSONObjIterator it(obj);
-    verify(it.more());
+    MONGO_verify(it.more());
     ret.start = it.next();
-    verify(it.more());
+    MONGO_verify(it.more());
     ret.end = it.next();
     return ret;
 }
@@ -1164,7 +1190,7 @@ void IndexBoundsBuilder::intersectize(const OrderedIntervalList& oilA, OrderedIn
         }
 
         Interval::IntervalComparison cmp = oilAIntervals[oilAIdx].compare(oilBIntervals[oilBIdx]);
-        verify(Interval::INTERVAL_UNKNOWN != cmp);
+        MONGO_verify(Interval::INTERVAL_UNKNOWN != cmp);
 
         if (cmp == Interval::INTERVAL_PRECEDES || cmp == Interval::INTERVAL_PRECEDES_COULD_UNION) {
             // oilAIntervals is before oilBIntervals. move oilAIntervals forward.
@@ -1216,7 +1242,7 @@ void IndexBoundsBuilder::unionize(OrderedIntervalList* oilOut) {
         Interval::IntervalComparison cmp = iv[i].compare(iv[i + 1]);
 
         // This means our sort didn't work.
-        verify(Interval::INTERVAL_SUCCEEDS != cmp);
+        MONGO_verify(Interval::INTERVAL_SUCCEEDS != cmp);
 
         // Intervals are correctly ordered.
         if (Interval::INTERVAL_PRECEDES == cmp) {
@@ -1319,7 +1345,7 @@ void IndexBoundsBuilder::translateRegex(const RegexMatchExpression* rme,
         bob.appendMinForType("", String);
         bob.appendMaxForType("", String);
         BSONObj dataObj = bob.obj();
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         oilOut->intervals.push_back(
             makeRangeInterval(dataObj, BoundInclusion::kIncludeStartKeyOnly));
     }
@@ -1350,7 +1376,7 @@ void IndexBoundsBuilder::translateEquality(const BSONElement& data,
             dataObj = ExpressionMapping::hash(dataObj.firstElement());
         }
 
-        verify(dataObj.isOwned());
+        MONGO_verify(dataObj.isOwned());
         oil->intervals.push_back(makePointInterval(dataObj));
 
         if (isHashed) {
