@@ -76,6 +76,12 @@ bool ConstEval::optimize(ABT& n) {
     return _changed;
 }
 
+void ConstEval::constFold(ABT& n) {
+    VariableEnvironment env = VariableEnvironment::build(n);
+    ConstEval instance(env);
+    instance.optimize(n);
+}
+
 void ConstEval::removeUnusedEvalNodes() {
     for (auto&& [k, v] : _projectRefs) {
         if (v.size() == 0) {
@@ -86,8 +92,7 @@ void ConstEval::removeUnusedEvalNodes() {
             // Do not inline nodes which can become Sargable.
             // TODO: consider caching.
             // TODO: consider deriving IndexingAvailability.
-            if (!_disableSargableInlining ||
-                !convertExprToPartialSchemaReq(k->getProjection(), false /*isFilterContext*/)) {
+            if (!_canInlineEval || _canInlineEval(*k)) {
                 // Schedule node inlining as there is exactly one reference.
                 _singleRef.emplace(v.front());
                 _changed = true;
@@ -119,6 +124,9 @@ void ConstEval::transport(ABT& n, const Variable& var) {
             // This is a indirection to another variable. So we can skip, but first remember that we
             // inlined this variable so that we won't try to replace it with a common expression and
             // revert the inlining.
+            if (_renamedProj) {
+                _renamedProj(var.name(), variable->name());
+            }
             _inlinedDefs.emplace(def.definition);
             swapAndUpdate(n, def.definition);
         } else if (_singleRef.erase(&var)) {
@@ -188,6 +196,23 @@ void ConstEval::transport(ABT& n, const LambdaApplication& app, ABT& lam, ABT& a
     }
 }
 
+void ConstEval::transport(ABT& n, const UnaryOp& op, ABT& child) {
+    switch (op.op()) {
+        case Operations::Not: {
+            if (const auto childConst = child.cast<Constant>();
+                childConst && childConst->isValueBool()) {
+                swapAndUpdate(n, Constant::boolean(!childConst->getValueBool()));
+            }
+            break;
+        }
+
+            // Could also constant fold arithmetic negation.
+
+        default:
+            break;
+    }
+}
+
 // Specific transport for binary operation
 // The const correctness is probably wrong (as const ABT& lhs, const ABT& rhs does not work for
 // some reason but we can fix it later).
@@ -305,6 +330,9 @@ void ConstEval::transport(ABT& n, const BinaryOp& op, ABT& lhs, ABT& rhs) {
 
                     const auto [compareTag, compareVal] =
                         sbe::value::compareValue(lhsTag, lhsVal, rhsTag, rhsVal);
+                    uassert(7086701,
+                            "Invalid comparison result",
+                            compareTag == sbe::value::TypeTags::NumberInt32);
                     const auto cmpVal = sbe::value::bitcastTo<int32_t>(compareVal);
 
                     switch (op.op()) {
@@ -426,6 +454,14 @@ void ConstEval::transport(ABT& n, const FunctionCall& op, std::vector<ABT>& args
             auto [tag, val] = sbe::value::makeCopyArray(array);
             swapAndUpdate(n, make<Constant>(tag, val));
         }
+    } else if (op.name() == "traverseP") {
+        // TraverseP with an identity lambda. Replace with the input.
+        if (const auto* lambdaPtr = args.at(1).cast<LambdaAbstraction>()) {
+            if (const auto* varPtr = lambdaPtr->getBody().cast<Variable>();
+                varPtr != nullptr && varPtr->name() == lambdaPtr->varName()) {
+                swapAndUpdate(n, args.front());
+            }
+        }
     }
 }
 
@@ -444,6 +480,20 @@ void ConstEval::transport(ABT& n, const If& op, ABT& cond, ABT& thenBranch, ABT&
     }
 }
 
+void ConstEval::transport(ABT& n, const EvalPath& op, ABT& path, ABT& input) {
+    if (const auto* pathConstPtr = path.cast<PathConstant>()) {
+        // PathConst does not depend on its parent, so replace with the PathConst's child.
+        swapAndUpdate(n, pathConstPtr->getConstant());
+    }
+}
+
+void ConstEval::transport(ABT& n, const EvalFilter& op, ABT& path, ABT& input) {
+    if (const auto* pathConstPtr = path.cast<PathConstant>()) {
+        // PathConst does not depend on its parent, so replace with the PathConst's child.
+        swapAndUpdate(n, pathConstPtr->getConstant());
+    }
+}
+
 void ConstEval::prepare(ABT&, const PathTraverse&) {
     ++_inCostlyCtx;
 }
@@ -484,11 +534,23 @@ void ConstEval::transport(ABT&, const LambdaAbstraction&, ABT&) {
     --_inCostlyCtx;
 }
 
+void ConstEval::transport(ABT& n, const FilterNode& op, ABT& child, ABT& expr) {
+    if (expr == Constant::boolean(true)) {
+        // Remove trivially true filter.
+
+        // First, pull out the child and put in a blackhole.
+        auto result = std::exchange(child, make<Blackhole>());
+
+        // Replace the filter node itself with the extracted child.
+        swapAndUpdate(n, std::move(result));
+    }
+}
+
 void ConstEval::transport(ABT& n, const EvaluationNode& op, ABT& child, ABT& expr) {
     if (_noRefProj.erase(&op)) {
         // The evaluation node is unused so replace it with its own child.
-        if (_erasedProjNames != nullptr) {
-            _erasedProjNames->insert(op.getProjectionName());
+        if (_erasedProj) {
+            _erasedProj(op.getProjectionName());
         }
 
         // First, pull out the child and put in a blackhole.
