@@ -34,6 +34,7 @@
 
 #include "mongo/db/query/index_tag.h"
 #include "mongo/db/query/indexability.h"
+#include "mongo/db/query/query_planner_common.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/string_map.h"
 
@@ -60,8 +61,8 @@ std::string getPathPrefix(std::string path) {
  * is a predicate that is required to use an index.
  */
 bool expressionRequiresIndex(const MatchExpression* node) {
-    return CanonicalQuery::countNodes(node, MatchExpression::GEO_NEAR) > 0 ||
-        CanonicalQuery::countNodes(node, MatchExpression::TEXT) > 0;
+    return QueryPlannerCommon::countNodes(node, MatchExpression::GEO_NEAR) > 0 ||
+        QueryPlannerCommon::countNodes(node, MatchExpression::TEXT) > 0;
 }
 
 size_t getPathLength(const MatchExpression* expr) {
@@ -263,7 +264,8 @@ PlanEnumerator::PlanEnumerator(const PlanEnumeratorParams& params)
       _ixisect(params.intersect),
       _enumerateOrChildrenLockstep(params.enumerateOrChildrenLockstep),
       _orLimit(params.maxSolutionsPerOr),
-      _intersectLimit(params.maxIntersectPerAnd) {}
+      _intersectLimit(params.maxIntersectPerAnd),
+      _disableOrPushdown(params.disableOrPushdown) {}
 
 PlanEnumerator::~PlanEnumerator() {
     typedef stdx::unordered_map<MemoID, NodeAssignment*> MemoMap;
@@ -339,6 +341,8 @@ string PlanEnumerator::NodeAssignment::toString() const {
         str::stream ss;
         ss << "ALL OF (lockstep): {";
         ss << "\n\ttotalEnumerated: " << lockstepOrAssignment->totalEnumerated;
+        ss << "\n\texhaustedLockstepIteration: "
+           << lockstepOrAssignment->exhaustedLockstepIteration;
         ss << "\n\tsubnodes: [ ";
         for (auto&& node : lockstepOrAssignment->subnodes) {
             ss << "\n\t\t{";
@@ -376,7 +380,7 @@ unique_ptr<MatchExpression> PlanEnumerator::getNext() {
     // Tag with our first solution.
     tagMemo(memoIDForNode(_root));
 
-    unique_ptr<MatchExpression> tree(_root->shallowClone());
+    unique_ptr<MatchExpression> tree(_root->clone());
     tagForSort(tree.get());
 
     _root->resetTag();
@@ -397,9 +401,9 @@ void PlanEnumerator::allocateAssignment(MatchExpression* expr,
     size_t newID = _memo.size() + 1;
 
     // Shouldn't be anything there already.
-    verify(_nodeToId.end() == _nodeToId.find(expr));
+    MONGO_verify(_nodeToId.end() == _nodeToId.find(expr));
     _nodeToId[expr] = newID;
-    verify(_memo.end() == _memo.find(newID));
+    MONGO_verify(_memo.end() == _memo.find(newID));
     NodeAssignment* newAssignment = new NodeAssignment();
     _memo[newID] = newAssignment;
     *assign = newAssignment;
@@ -494,7 +498,7 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
         NodeAssignment* assign;
         allocateAssignment(node, &assign, &myMemoID);
 
-        assign->arrayAssignment.reset(aa.release());
+        assign->arrayAssignment = std::move(aa);
         return true;
     } else if (Indexability::nodeCanUseIndexOnOwnField(node) ||
                Indexability::isBoundsGeneratingNot(node) ||
@@ -528,10 +532,14 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
         // preds to 'indexedPreds'. Adding the mandatory preds directly to 'indexedPreds' would lead
         // to problems such as pulling a predicate beneath an OR into a set joined by an AND.
         getIndexedPreds(node, childContext, &indexedPreds);
-        // Pass in the indexed predicates as outside predicates when prepping the subnodes.
+        // Pass in the indexed predicates as outside predicates when prepping the subnodes. But if
+        // match expression optimization is disabled, skip this part: we don't want to do
+        // OR-pushdown because it relies on the expression being canonicalized.
         auto childContextCopy = childContext;
-        for (auto pred : indexedPreds) {
-            childContextCopy.outsidePreds[pred] = OutsidePredRoute{};
+        if (MONGO_likely(!_disableOrPushdown)) {
+            for (auto pred : indexedPreds) {
+                childContextCopy.outsidePreds[pred] = OutsidePredRoute{};
+            }
         }
         if (!prepSubNodes(node, childContextCopy, &subnodes, &mandatorySubnodes)) {
             return false;
@@ -835,6 +843,13 @@ void PlanEnumerator::assignPredicate(
     MatchExpression* pred,
     size_t position,
     OneIndexAssignment* indexAssignment) {
+    if (MONGO_unlikely(_disableOrPushdown)) {
+        // If match expression optimization is disabled, we also disable OR-pushdown,
+        // so we should never get 'outsidePreds' here.
+        tassert(7059700,
+                "Tried to do OR-pushdown despite disableMatchExpressionOptimization",
+                outsidePreds.empty());
+    }
     if (outsidePreds.find(pred) != outsidePreds.end()) {
         OrPushdownTag::Destination dest;
         dest.route = outsidePreds.at(pred).route;
@@ -1289,6 +1304,8 @@ void PlanEnumerator::getIndexedPreds(MatchExpression* node,
                                      std::vector<MatchExpression*>* indexedPreds) {
     if (Indexability::nodeCanUseIndexOnOwnField(node)) {
         RelevantTag* rt = static_cast<RelevantTag*>(node->getTag());
+        tassert(9074700, "RelevantTag is not assigned to the match expression node", rt != nullptr);
+
         if (context.elemMatchExpr) {
             // If we're in an $elemMatch context, store the
             // innermost parent $elemMatch, as well as the
@@ -1305,7 +1322,7 @@ void PlanEnumerator::getIndexedPreds(MatchExpression* node,
         indexedPreds->push_back(node);
     } else if (Indexability::isBoundsGeneratingNot(node)) {
         getIndexedPreds(node->getChild(0), context, indexedPreds);
-    } else if (MatchExpression::ELEM_MATCH_OBJECT == node->matchType()) {
+    } else if (Indexability::isBoundsGeneratingElemMatchObject(node)) {
         PrepMemoContext childContext;
         childContext.elemMatchExpr = node;
         for (size_t i = 0; i < node->numChildren(); ++i) {
@@ -1319,7 +1336,6 @@ void PlanEnumerator::getIndexedPreds(MatchExpression* node,
 }
 
 bool PlanEnumerator::prepSubNodes(MatchExpression* node,
-
                                   PrepMemoContext context,
                                   vector<MemoID>* subnodesOut,
                                   vector<MemoID>* mandatorySubnodes) {
@@ -1354,9 +1370,13 @@ bool PlanEnumerator::prepSubNodes(MatchExpression* node,
             childContext.elemMatchExpr = child;
             childContext.outsidePreds = context.outsidePreds;
             markTraversedThroughElemMatchObj(&childContext);
-            prepSubNodes(child, childContext, subnodesOut, mandatorySubnodes);
+            if (!prepSubNodes(child, childContext, subnodesOut, mandatorySubnodes)) {
+                return false;
+            }
         } else if (MatchExpression::AND == child->matchType()) {
-            prepSubNodes(child, context, subnodesOut, mandatorySubnodes);
+            if (!prepSubNodes(child, context, subnodesOut, mandatorySubnodes)) {
+                return false;
+            }
         }
     }
     return true;
@@ -1626,7 +1646,7 @@ void PlanEnumerator::compound(const vector<MatchExpression*>& tryCompound,
 void PlanEnumerator::tagMemo(size_t id) {
     LOGV2_DEBUG(20944, 5, "Tagging memoID", "id"_attr = id);
     NodeAssignment* assign = _memo[id];
-    verify(nullptr != assign);
+    MONGO_verify(nullptr != assign);
 
     if (nullptr != assign->orAssignment) {
         OrAssignment* oa = assign->orAssignment.get();
@@ -1643,7 +1663,7 @@ void PlanEnumerator::tagMemo(size_t id) {
         tagMemo(aa->subnodes[aa->counter]);
     } else if (nullptr != assign->andAssignment) {
         AndAssignment* aa = assign->andAssignment.get();
-        verify(aa->counter < aa->choices.size());
+        MONGO_verify(aa->counter < aa->choices.size());
 
         const AndEnumerableState& aes = aa->choices[aa->counter];
 
@@ -1677,7 +1697,7 @@ void PlanEnumerator::tagMemo(size_t id) {
             }
         }
     } else {
-        verify(0);
+        MONGO_verify(0);
     }
 }
 
@@ -1691,8 +1711,8 @@ bool PlanEnumerator::LockstepOrAssignment::allIdentical() const {
     return true;
 }
 
-bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(
-    size_t totalEnumerated) const {
+bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(size_t totalEnumerated,
+                                                                       size_t orLimit) const {
     if (totalEnumerated == 0 || !exhaustedLockstepIteration) {
         return false;
     }
@@ -1703,6 +1723,11 @@ bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(
             return false;  // Haven't yet looped over this child entirely, not ready yet.
         }
         totalPossibleEnumerations *= subnode.maxIterCount.value();
+        // If 'totalPossibleEnumerations' reaches the limit, we can just shortcut it. Otherwise,
+        // 'totalPossibleEnumerations' could overflow if we have a large $or.
+        if (totalPossibleEnumerations >= orLimit) {
+            return false;
+        }
     }
 
     // If we're able to compute a total number expected enumerations, we must have already cycled
@@ -1739,7 +1764,7 @@ bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
         }
         // Edge case: if every child has only one option available, we are already finished
         // enumerating.
-        if (assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
+        if (assignment->shouldResetBeforeProceeding(assignment->totalEnumerated, _orLimit)) {
             assignment->exhaustedLockstepIteration = false;
             return true;  // We're back at the beginning, no need to reset.
         }
@@ -1766,27 +1791,35 @@ bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
         }
     };
     advanceOnce();
-    while (assignment->allIdentical()) {
-        // All sub-nodes have the same enumeration state, skip this one since we already did
-        // it above. This is expected to happen pretty often. For example, if we have two subnodes
-        // each enumerating two states, we'd expect the order to be: 00, 11 (these two iterated
-        // above), then 00 (skipped by falling through above after finishing lockstep iteration),
-        // then 10, 11 (skipped here), 00 (skipped here), then finally 01.
+    if (assignment->allIdentical()) {
+        // All sub-nodes have the same enumeration state, skip this one since we already did it
+        // above. This is expected to happen pretty often. For example, if we have two subnodes each
+        // enumerating two states, we'd expect the order to be: 00, 11 (these two iterated above),
+        // then 00 (skipped here when we fall through after finishing lockstep iteration), then 10,
+        // 01, then finally 11 (skipped here).
+        //
+        // In this example, when we finally roll back to 00, enumeration is complete. We will fall
+        // through the code below which is responsible for resetting all enumeration state to the
+        // starting point (which need not reset the child nodes in this case because they already
+        // all rolled back to the starting point of 00). Finally, we return true to indicate that
+        // all possibilities have been enumerated.
         advanceOnce();
     }
 
     // This special ordering is tricky to reset. Because it iterates the sub nodes in such a
     // unique order, it can be difficult to know when it has actually finished iterating. Our
     // strategy is just to compute a total and go back to the beginning once we hit that total.
-    if (!assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
+    if (!assignment->shouldResetBeforeProceeding(assignment->totalEnumerated, _orLimit)) {
         return false;
     }
     // Reset!
     for (auto&& subnode : assignment->subnodes) {
-        while (!nextMemo(subnode.memoId)) {
-            // Keep advancing till it rolls over.
+        if (subnode.iterationCount != 0) {
+            while (!nextMemo(subnode.memoId)) {
+                // Keep advancing till it rolls over.
+            }
+            subnode.iterationCount = 0;
         }
-        subnode.iterationCount = 0;
     }
     assignment->exhaustedLockstepIteration = false;
     return true;
@@ -1794,7 +1827,7 @@ bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
 
 bool PlanEnumerator::nextMemo(size_t id) {
     NodeAssignment* assign = _memo[id];
-    verify(nullptr != assign);
+    MONGO_verify(nullptr != assign);
 
     if (nullptr != assign->orAssignment) {
         OrAssignment* oa = assign->orAssignment.get();
