@@ -40,16 +40,47 @@
 
 namespace mongo {
 
-PlanYieldPolicy::PlanYieldPolicy(YieldPolicy policy,
+PlanYieldPolicy::PlanYieldPolicy(OperationContext* opCtx,
+                                 YieldPolicy policy,
                                  ClockSource* cs,
                                  int yieldIterations,
                                  Milliseconds yieldPeriod,
                                  const Yieldable* yieldable,
                                  std::unique_ptr<const YieldPolicyCallbacks> callbacks)
-    : _policy(policy),
+    : _policy(getPolicyOverrideForOperation(opCtx, policy)),
       _yieldable(yieldable),
       _callbacks(std::move(callbacks)),
-      _elapsedTracker(cs, yieldIterations, yieldPeriod) {}
+      _elapsedTracker(cs, yieldIterations, yieldPeriod) {
+    invariant(!_yieldable || _yieldable->yieldable() ||
+              policy == YieldPolicy::WRITE_CONFLICT_RETRY_ONLY || policy == YieldPolicy::NO_YIELD ||
+              policy == YieldPolicy::INTERRUPT_ONLY || policy == YieldPolicy::ALWAYS_TIME_OUT ||
+              policy == YieldPolicy::ALWAYS_MARK_KILLED);
+}
+
+PlanYieldPolicy::YieldPolicy PlanYieldPolicy::getPolicyOverrideForOperation(
+    OperationContext* opCtx, PlanYieldPolicy::YieldPolicy desired) {
+    // We may have a null opCtx in testing.
+    if (MONGO_unlikely(!opCtx)) {
+        return desired;
+    }
+    // Multi-document transactions cannot yield locks or snapshots. We convert to a non-yielding
+    // interruptible plan.
+    if (opCtx->inMultiDocumentTransaction() &&
+        (desired == YieldPolicy::YIELD_AUTO || desired == YieldPolicy::YIELD_MANUAL ||
+         desired == YieldPolicy::WRITE_CONFLICT_RETRY_ONLY)) {
+        return YieldPolicy::INTERRUPT_ONLY;
+    }
+
+    // If the state of our locks held is not yieldable at all, we will assume this is an internal
+    // operation that should not be interrupted or yielded.
+    // TODO: SERVER-76238 Evaluate if we can make everything INTERRUPT_ONLY instead.
+    if (!opCtx->lockState()->canSaveLockState() &&
+        (desired == YieldPolicy::YIELD_AUTO || desired == YieldPolicy::YIELD_MANUAL)) {
+        return YieldPolicy::NO_YIELD;
+    }
+
+    return desired;
+}
 
 bool PlanYieldPolicy::shouldYieldOrInterrupt(OperationContext* opCtx) {
     if (_policy == YieldPolicy::INTERRUPT_ONLY) {
@@ -95,8 +126,7 @@ Status PlanYieldPolicy::yieldOrInterrupt(OperationContext* opCtx,
 
             try {
                 saveState(opCtx);
-            } catch (const WriteConflictException&) {
-                // Saving the state of an execution plan must never throw WCE.
+            } catch (const StorageUnavailableException&) {
                 MONGO_UNREACHABLE;
             }
 
@@ -128,7 +158,7 @@ Status PlanYieldPolicy::yieldOrInterrupt(OperationContext* opCtx,
 
             restoreState(opCtx, yieldable);
             return Status::OK();
-        } catch (const WriteConflictException&) {
+        } catch (const StorageUnavailableException&) {
             if (_callbacks) {
                 _callbacks->handledWriteConflict(opCtx);
             }
@@ -149,42 +179,32 @@ void PlanYieldPolicy::performYield(OperationContext* opCtx,
                                    std::function<void()> whileYieldingFn) {
     // Things have to happen here in a specific order:
     //   * Release 'yieldable'.
-    //   * Release lock mgr locks.
+    //   * Abandon the current storage engine snapshot.
     //   * Check for interrupt if the yield policy requires.
-    //   * Abondon the query's current storage engine snapshot.
-    //   * Reacquire lock mgr locks.
+    //   * Release lock manager locks.
+    //   * Reacquire lock manager locks.
     //   * Restore 'yieldable'.
-    Locker* locker = opCtx->lockState();
+    invariant(_policy == YieldPolicy::YIELD_AUTO || _policy == YieldPolicy::YIELD_MANUAL);
 
-    if (locker->isGlobalLockedRecursively()) {
-        // No purpose in yielding if the locks are recursively held and cannot be released.
-        return;
-    }
-
-    // Since the locks are not recursively held, this is a top level operation and we can safely
-    // clear the 'yieldable' state before unlocking and then re-establish it after re-locking.
+    // If we are here, the caller has guaranteed locks are not recursively held. This is a top level
+    // operation and we can safely clear the 'yieldable' state before unlocking and then
+    // re-establish it after re-locking.
     yieldable.yield();
 
-    Locker::LockSnapshot snapshot;
-    auto unlocked = locker->saveLockStateAndUnlock(&snapshot);
+    // Release any storage engine resources. This requires holding a global lock to correctly
+    // synchronize with states such as shutdown and rollback.
+    opCtx->recoveryUnit()->abandonSnapshot();
 
-    // After all steps to relinquish locks and save the execution plan have been taken, check
-    // for interrupt. This is the main interrupt check during query execution. Yield points and
-    // interrupt points are one and the same.
+    // Check for interrupt before releasing locks. This avoids the complexities of having to
+    // re-acquire locks to clean up when we are interrupted. This is the main interrupt check during
+    // query execution. Yield points and interrupt points are one and the same.
     if (getPolicy() == PlanYieldPolicy::YieldPolicy::YIELD_AUTO) {
         opCtx->checkForInterrupt();  // throws
     }
 
-    if (!unlocked) {
-        // Nothing was unlocked. Recursively held locks are not the only reason locks cannot be
-        // released. Restore the 'yieldable' state before returning.
-        yieldable.restore();
-        return;
-    }
-
-    // Top-level locks are freed, release any potential low-level (storage engine-specific
-    // locks). If we are yielding, we are at a safe place to do so.
-    opCtx->recoveryUnit()->abandonSnapshot();
+    Locker* locker = opCtx->lockState();
+    Locker::LockSnapshot snapshot;
+    locker->saveLockStateAndUnlock(&snapshot);
 
     if (_callbacks) {
         _callbacks->duringYield(opCtx);
