@@ -190,39 +190,118 @@ bool hintMatchesClusterKey(const boost::optional<ClusteredCollectionInfo>& clust
  * Returns whether the hintedIndex matches the columnstore index.
  */
 bool hintMatchesColumnStoreIndex(const BSONObj& hintObj, const ColumnIndexEntry& columnStoreIndex) {
-    // TODO SERVER-68400: Should be possible to have some other keypattern.
     return hintMatchesNameOrPattern(
         hintObj, columnStoreIndex.identifier.catalogName, columnStoreIndex.keyPattern);
 }
 
 /**
- * Returns the dependencies for the CanoncialQuery, split by those needed to answer the filter,
- * and those needed for "everything else" which is the project and sort.
+ * Returns the dependencies for the CanonicalQuery, split by those needed to answer the filter,
+ * and those needed for "everything else", e.g. project, sort and shard filter.
  */
-std::pair<DepsTracker, DepsTracker> computeDeps(const QueryPlannerParams& params,
-                                                const CanonicalQuery& query) {
+std::pair<DepsTracker /* filter */, DepsTracker /* other */> computeDeps(
+    const QueryPlannerParams& params, const CanonicalQuery& query) {
     DepsTracker filterDeps;
     match_expression::addDependencies(query.root(), &filterDeps);
     DepsTracker outputDeps;
-    if (!query.getProj() || query.getProj()->requiresDocument()) {
+    if ((!query.getProj() || query.getProj()->requiresDocument()) && !query.isCountLike()) {
         outputDeps.needWholeDocument = true;
         return {std::move(filterDeps), std::move(outputDeps)};
     }
-    outputDeps.fields = query.getProj()->getRequiredFields();
-    if (auto sortPattern = query.getSortPattern()) {
-        sortPattern->addDependencies(&outputDeps);
-    }
     if (params.options & QueryPlannerParams::INCLUDE_SHARD_FILTER) {
         for (auto&& field : params.shardKey) {
             outputDeps.fields.emplace(field.fieldNameStringData());
         }
     }
+    if (query.isCountLike()) {
+        // If this is a count, we won't have required projections, but may still need to output the
+        // shard filter.
+        return {std::move(filterDeps), std::move(outputDeps)};
+    }
+
+    const auto& reqFields = query.getProj()->getRequiredFields();
+    outputDeps.fields.insert(reqFields.begin(), reqFields.end());
+
+    if (auto sortPattern = query.getSortPattern()) {
+        sortPattern->addDependencies(&outputDeps);
+    }
     // There's no known way a sort would depend on the whole document, and we already verified
     // that the projection doesn't depend on the whole document.
     tassert(6430503, "Unexpectedly required entire object", !outputDeps.needWholeDocument);
     return {std::move(filterDeps), std::move(outputDeps)};
 }
 
+/**
+ * Determines whether a column scan should be used given information about the query and collection.
+ * We are specifically interested in whether a column scan is likely to perform better than a
+ * collection scan. Column scan should be used if the following conditions are met:
+ *
+ * (|referenced fields| < maxNumFields) &&
+ * (|filtered fields| >= minNumColumnFilters ||
+ *     uncompressedCollectionSize >= minCollectionSize ||
+ *     avgDocSize >= minAvgDocSize)
+ *
+ * In words: we will use column scan if the query is reading fewer than the max number of fields,
+ * and at least one of the following is true: we are pushing down filters on a large number of
+ * fields, the collection does not fit in memory, or the average document size is large.
+ *
+ * All of the thresholds listed (referenced fields, column filters, collection size, and average
+ * document size) can be adjusted via query knobs.
+ */
+Status querySatisfiesCsiPlanningHeuristics(size_t nReferencedFields,
+                                           size_t nFilteredFields,
+                                           const QueryPlannerParams& plannerParams) {
+    // Check that we are reading fewer than the max number of fields allowed for column scan.
+    const int maxNumFields = nFilteredFields > 0
+        ? internalQueryMaxNumberOfFieldsToChooseFilteredColumnScan.load()
+        : internalQueryMaxNumberOfFieldsToChooseUnfilteredColumnScan.load();
+    if (static_cast<int>(nReferencedFields) > maxNumFields) {
+        return Status{ErrorCodes::Error{6430508},
+                      str::stream()
+                          << "query referenced too many fields to use column scan. nReferenced="
+                          << nReferencedFields << ", limit=" << maxNumFields};
+    }
+
+    const auto columnFilterThreshold = internalQueryColumnScanMinNumColumnFilters.load();
+    if (static_cast<int>(nFilteredFields) >= columnFilterThreshold) {
+        // We have enough column filters to make column scan worth it, regardless of the
+        // collection/document size.
+        return Status::OK();
+    }
+
+    const auto numDocs = plannerParams.collectionStats.noOfRecords;
+    const auto uncompressedDataSizeBytes = plannerParams.collectionStats.approximateDataSizeBytes;
+
+    // Check if the entire uncompressed collection is greater than our min collection size
+    // threshold, or if it can fit in memory if the min size is unspecified.
+    const auto collectionSizeThresholdBytes = [&]() {
+        const auto configuredThresholdBytes = internalQueryColumnScanMinCollectionSizeBytes.load();
+        // If there is no threshold specified (== -1), use available memory size.
+        return configuredThresholdBytes == -1 ? plannerParams.availableMemoryBytes
+                                              : configuredThresholdBytes;
+    }();
+    if (uncompressedDataSizeBytes >= collectionSizeThresholdBytes) {
+        // The collection is larger than memory/the configured threshold - use column scan.
+        return Status::OK();
+    }
+
+    // If we got here, the query/collection doesn't meet any of our other thresholds. Check if the
+    // average document size is greater than our threshold for using column scan.
+    const auto docSizeThresholdBytes = internalQueryColumnScanMinAvgDocSizeBytes.load();
+    auto avgDocSizeBytes =
+        numDocs > 0 ? uncompressedDataSizeBytes / static_cast<double>(numDocs) : 0.0;
+    if (avgDocSizeBytes >= docSizeThresholdBytes) {
+        return Status::OK();
+    }
+
+    return {ErrorCodes::Error{6995600},
+            str::stream() << "query did not pass heuristics for using column scan. "
+                          << "nFilteredFields: " << nFilteredFields << " < "
+                          << columnFilterThreshold << ", collectionSizeBytes: "
+                          << uncompressedDataSizeBytes << " < " << collectionSizeThresholdBytes
+                          << ", avgDocSizeBytes: " << avgDocSizeBytes << " < "
+                          << docSizeThresholdBytes};
+}
+
 Status computeColumnScanIsPossibleStatus(const CanonicalQuery& query,
                                          const QueryPlannerParams& params) {
     if (params.columnStoreIndexes.empty()) {
@@ -267,25 +346,6 @@ std::unique_ptr<QuerySolution> makeColumnScanPlan(
                                               std::move(residualPredicate)));
 }
 
-/**
- * A helper function which applies a heuristic to determine if a COLUMN_SCAN plan would examine few
- * enough fields to be considered faster than a COLLSCAN.
- */
-Status checkColumnScanFieldLimits(
-    size_t nReferencedFields,
-    const StringMap<std::unique_ptr<MatchExpression>>& filterSplitByColumn) {
-
-    const int maxNumFields = filterSplitByColumn.size() > 0
-        ? internalQueryMaxNumberOfFieldsToChooseFilteredColumnScan.load()
-        : internalQueryMaxNumberOfFieldsToChooseUnfilteredColumnScan.load();
-    if (static_cast<int>(nReferencedFields) > maxNumFields) {
-        return Status{ErrorCodes::Error{6430508},
-                      str::stream() << "referenced too many fields. nReferenced="
-                                    << nReferencedFields << ", limit=" << maxNumFields};
-    }
-    return Status::OK();
-}
-
 bool checkProjectionCoversQuery(OrderedPathSet& fields, const ColumnIndexEntry& columnStoreIndex) {
     const auto projectedFields = projection_executor_utils::applyProjectionToFields(
         columnStoreIndex.indexPathProjection->exec(), fields);
@@ -300,7 +360,7 @@ bool checkProjectionCoversQuery(OrderedPathSet& fields, const ColumnIndexEntry&
  */
 std::pair<int, const ColumnIndexEntry*> getValidColumnIndex(
     OrderedPathSet& fields, const std::vector<ColumnIndexEntry>& columnStoreIndexes) {
-    const ColumnIndexEntry* chosenIndex;
+    const ColumnIndexEntry* chosenIndex = nullptr;
     int numValid = 0;
     for (const auto& columnStoreIndex : columnStoreIndexes) {
         if (checkProjectionCoversQuery(fields, columnStoreIndex)) {
@@ -343,6 +403,11 @@ StatusWith<std::unique_ptr<QuerySolution>> tryToBuildColumnScan(
                 str::stream() << "cannot use columnstore index because the query requires paths "
                                  "which are a prefix of each other: "
                               << set_util::setToString(allFieldsReferenced)};
+    } else if (expression::containsEmptyPaths(allFieldsReferenced)) {
+        return {
+            ErrorCodes::Error{6549400},
+            str::stream() << "cannot use columnstore index because the query requires empty paths: "
+                          << set_util::setToString(allFieldsReferenced)};
     }
 
     // Ensures that hinted index is eligible for the column scan.
@@ -377,12 +442,21 @@ StatusWith<std::unique_ptr<QuerySolution>> tryToBuildColumnScan(
     StringMap<std::unique_ptr<MatchExpression>> filterSplitByColumn;
     std::tie(filterSplitByColumn, residualPredicate) =
         expression::splitMatchExpressionForColumns(query.root());
-    auto fieldLimitStatus =
-        checkColumnScanFieldLimits(allFieldsReferenced.size(), filterSplitByColumn);
+    auto heuristicsStatus = querySatisfiesCsiPlanningHeuristics(
+        allFieldsReferenced.size(), filterSplitByColumn.size(), params);
+
+    if (heuristicsStatus.isOK() || hintedIndex) {
+        // We have a hint, or collection stats and dependencies that indicate a column scan is
+        // likely better than a collection scan. Build it and return it.
+
+        // Since the residual predicate must be applied after the column scan, we need to include
+        // the dependencies among the output fields.
+        DepsTracker residualDeps;
+        if (residualPredicate) {
+            match_expression::addDependencies(residualPredicate.get(), &residualDeps);
+            outputDeps.fields = set_util::setUnion(outputDeps.fields, residualDeps.fields);
+        }
 
-    if (fieldLimitStatus.isOK() || hintedIndex) {
-        // We have a hint, or few enough dependencies that we suspect a column scan is still
-        // better than a collection scan. Build it and return it.
         return makeColumnScanPlan(query,
                                   params,
                                   columnStoreIndex,
@@ -392,8 +466,13 @@ StatusWith<std::unique_ptr<QuerySolution>> tryToBuildColumnScan(
                                   std::move(filterSplitByColumn),
                                   std::move(residualPredicate));
     }
-    return Status{ErrorCodes::Error{6298502}, "columnstore index is not applicable for this query"};
+    return heuristicsStatus;
 }
+
+bool collscanIsBounded(const CollectionScanNode* collscan) {
+    return collscan->minRecord || collscan->maxRecord;
+}
+
 }  // namespace
 
 using std::numeric_limits;
@@ -448,9 +527,6 @@ string optionString(size_t options) {
             case QueryPlannerParams::INDEX_INTERSECTION:
                 ss << "INDEX_INTERSECTION ";
                 break;
-            case QueryPlannerParams::IS_COUNT:
-                ss << "IS_COUNT ";
-                break;
             case QueryPlannerParams::GENERATE_COVERED_IXSCANS:
                 ss << "GENERATE_COVERED_IXSCANS ";
                 break;
@@ -463,9 +539,6 @@ string optionString(size_t options) {
             case QueryPlannerParams::STRICT_DISTINCT_ONLY:
                 ss << "STRICT_DISTINCT_ONLY ";
                 break;
-            case QueryPlannerParams::PRESERVE_RECORD_ID:
-                ss << "PRESERVE_RECORD_ID ";
-                break;
             case QueryPlannerParams::ASSERT_MIN_TS_HAS_NOT_FALLEN_OFF_OPLOG:
                 ss << "ASSERT_MIN_TS_HAS_NOT_FALLEN_OFF_OPLOG ";
                 break;
@@ -478,6 +551,9 @@ string optionString(size_t options) {
             case QueryPlannerParams::GENERATE_PER_COLUMN_FILTERS:
                 ss << "GENERATE_PER_COLUMN_FILTERS ";
                 break;
+            case QueryPlannerParams::STRICT_NO_TABLE_SCAN:
+                ss << "STRICT_NO_TABLE_SCAN ";
+                break;
             case QueryPlannerParams::DEFAULT:
                 MONGO_UNREACHABLE;
                 break;
@@ -613,13 +689,58 @@ static BSONObj finishMaxObj(const IndexEntry& indexEntry,
     }
 }
 
+bool providesSort(const CanonicalQuery& query, const BSONObj& kp) {
+    return query.getFindCommandRequest().getSort().isPrefixOf(
+        kp, SimpleBSONElementComparator::kInstance);
+}
+
+/**
+ * Determine whether this query has a sort that can be provided by the clustered index, if so, which
+ * direction the scan should be. If the collection is not clustered, or the sort cannot be provided,
+ * returns 'boost::none'.
+ */
+boost::optional<int> determineClusteredScanDirection(const CanonicalQuery& query,
+                                                     const QueryPlannerParams& params) {
+    if (params.clusteredInfo && query.getSortPattern() &&
+        CollatorInterface::collatorsMatch(params.clusteredCollectionCollator,
+                                          query.getCollator())) {
+        auto kp = clustered_util::getSortPattern(params.clusteredInfo->getIndexSpec());
+        if (providesSort(query, kp)) {
+            return 1;
+        } else if (providesSort(query, QueryPlannerCommon::reverseSortObj(kp))) {
+            return -1;
+        }
+    }
+
+    return boost::none;
+}
+
+/**
+ * Determine the direction of the scan needed for the query. Defaults to 1 unless this is a
+ * clustered collection and we have a sort that can be provided by the clustered index.
+ */
+int determineCollscanDirection(const CanonicalQuery& query, const QueryPlannerParams& params) {
+    return determineClusteredScanDirection(query, params).value_or(1);
+}
+
+std::pair<std::unique_ptr<QuerySolution>, const CollectionScanNode*> buildCollscanSolnWithNode(
+    const CanonicalQuery& query, bool tailable, const QueryPlannerParams& params, int direction) {
+    std::unique_ptr<QuerySolutionNode> solnRoot(
+        QueryPlannerAccess::makeCollectionScan(query, tailable, params, direction));
+    const auto* collscanNode = checked_cast<const CollectionScanNode*>(solnRoot.get());
+    return std::make_pair(
+        QueryPlannerAnalysis::analyzeDataAccess(query, params, std::move(solnRoot)), collscanNode);
+}
+
 std::unique_ptr<QuerySolution> buildCollscanSoln(const CanonicalQuery& query,
                                                  bool tailable,
                                                  const QueryPlannerParams& params,
-                                                 int direction = 1) {
-    std::unique_ptr<QuerySolutionNode> solnRoot(
-        QueryPlannerAccess::makeCollectionScan(query, tailable, params, direction));
-    return QueryPlannerAnalysis::analyzeDataAccess(query, params, std::move(solnRoot));
+                                                 boost::optional<int> direction = boost::none) {
+    return buildCollscanSolnWithNode(query,
+                                     tailable,
+                                     params,
+                                     direction.value_or(determineCollscanDirection(query, params)))
+        .first;
 }
 
 std::unique_ptr<QuerySolution> buildWholeIXSoln(
@@ -635,11 +756,6 @@ std::unique_ptr<QuerySolution> buildWholeIXSoln(
     return QueryPlannerAnalysis::analyzeDataAccess(query, params, std::move(solnRoot));
 }
 
-bool providesSort(const CanonicalQuery& query, const BSONObj& kp) {
-    return query.getFindCommandRequest().getSort().isPrefixOf(
-        kp, SimpleBSONElementComparator::kInstance);
-}
-
 StatusWith<std::unique_ptr<PlanCacheIndexTree>> QueryPlanner::cacheDataFromTaggedTree(
     const MatchExpression* const taggedTree, const vector<IndexEntry>& relevantIndices) {
     if (!taggedTree) {
@@ -685,13 +801,13 @@ StatusWith<std::unique_ptr<PlanCacheIndexTree>> QueryPlanner::cacheDataFromTagge
 
             std::unique_ptr<IndexEntry> indexEntry =
                 std::make_unique<IndexEntry>(relevantIndices[itag->index]);
-            indexTree->entry.reset(indexEntry.release());
+            indexTree->entry = std::move(indexEntry);
             indexTree->index_pos = itag->pos;
             indexTree->canCombineBounds = itag->canCombineBounds;
         }
 
         for (const auto& dest : orPushdownTag->getDestinations()) {
-            IndexTag* indexTag = static_cast<IndexTag*>(dest.tagData.get());
+            IndexTag* indexTag = checked_cast<IndexTag*>(dest.tagData.get());
             PlanCacheIndexTree::OrPushdown orPushdown{relevantIndices[indexTag->index].identifier,
                                                       indexTag->pos,
                                                       indexTag->canCombineBounds,
@@ -725,7 +841,7 @@ Status QueryPlanner::tagAccordingToCache(MatchExpression* filter,
 
     // We're tagging the tree here, so it shouldn't have
     // any tags hanging off yet.
-    verify(nullptr == filter->getTag());
+    MONGO_verify(nullptr == filter->getTag());
 
     if (filter->numChildren() != indexTree->children.size()) {
         str::stream ss;
@@ -805,7 +921,7 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
     } else if (SolutionCacheData::COLLSCAN_SOLN == winnerCacheData.solnType) {
         // The cached solution is a collection scan. We don't cache collscans
         // with tailable==true, hence the false below.
-        auto soln = buildCollscanSoln(query, false, params);
+        auto soln = buildCollscanSoln(query, false, params, winnerCacheData.wholeIXSolnDir);
         if (!soln) {
             return Status(ErrorCodes::NoQueryExecutionPlans,
                           "plan cache error: collection scan soln");
@@ -819,7 +935,7 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
     // cases, and we proceed by using the PlanCacheIndexTree to tag the query tree.
 
     // Create a copy of the expression tree.  We use cachedSoln to annotate this with indices.
-    unique_ptr<MatchExpression> clone = query.root()->shallowClone();
+    unique_ptr<MatchExpression> clone = query.root()->clone();
 
     LOGV2_DEBUG(20963,
                 5,
@@ -827,10 +943,11 @@ StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::planFromCache(
                 "filter"_attr = redact(clone->debugString()),
                 "cacheData"_attr = redact(winnerCacheData.toString()));
 
-    stdx::unordered_set<string> fields;
+    RelevantFieldIndexMap fields;
     QueryPlannerIXSelect::getFields(query.root(), &fields);
+    // We will not cache queries with 'hint'.
     std::vector<IndexEntry> expandedIndexes =
-        QueryPlannerIXSelect::expandIndexes(fields, params.indices);
+        QueryPlannerIXSelect::expandIndexes(fields, params.indices, false /* indexHinted */);
 
     // Map from index name to index number.
     map<IndexEntry::Identifier, size_t> indexMap;
@@ -892,13 +1009,41 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> singleSolution(
     return {std::move(out)};
 }
 
-bool canTableScan(const QueryPlannerParams& params) {
-    return !(params.options & QueryPlannerParams::NO_TABLE_SCAN);
+// If no table scan option is set the planner may not return any plan containing a collection scan.
+// Yet clusteredIdxScans are still allowed as they are not a full collection scan but a bounded
+// collection scan.
+bool noTableScan(const QueryPlannerParams& params) {
+    return (params.options & QueryPlannerParams::NO_TABLE_SCAN);
+}
+
+// Used internally if the planner should also avoid retruning a plan containing a clusteredIDX scan.
+bool noTableAndClusteredIDXScan(const QueryPlannerParams& params) {
+    return (params.options & QueryPlannerParams::STRICT_NO_TABLE_SCAN);
+}
+
+bool isClusteredScan(QuerySolutionNode* node) {
+    if (node->getType() == STAGE_COLLSCAN) {
+        auto collectionScanSolnNode = dynamic_cast<CollectionScanNode*>(node);
+        return (collectionScanSolnNode->doClusteredCollectionScan());
+    }
+    return false;
+}
+
+// Check if this is a real coll scan or a hidden ClusteredIDX scan.
+bool isColusteredIDXScanSoln(QuerySolution* collscanSoln) {
+    if (collscanSoln->root()->getType() == STAGE_SHARDING_FILTER) {
+        auto child = collscanSoln->root()->children.begin();
+        return isClusteredScan(child->get());
+    }
+    if (collscanSoln->root()->getType() == STAGE_COLLSCAN) {
+        return isClusteredScan(collscanSoln->root());
+    }
+    return false;
 }
 
 StatusWith<std::vector<std::unique_ptr<QuerySolution>>> attemptCollectionScan(
     const CanonicalQuery& query, bool isTailable, const QueryPlannerParams& params) {
-    if (!canTableScan(params)) {
+    if (noTableScan(params)) {
         return Status(ErrorCodes::NoQueryExecutionPlans,
                       "not allowed to output a collection scan because 'notablescan' is enabled");
     }
@@ -971,13 +1116,6 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> handleClusteredScanHint(
 
 StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     const CanonicalQuery& query, const QueryPlannerParams& params) {
-    // It's a little silly to ask for a count and for owned data. This could indicate a bug
-    // earlier on.
-    tassert(5397500,
-            "Count and owned data requested",
-            !((params.options & QueryPlannerParams::IS_COUNT) &&
-              (params.options & QueryPlannerParams::RETURN_OWNED_DATA)));
-
     LOGV2_DEBUG(20967,
                 5,
                 "Beginning planning",
@@ -1005,10 +1143,10 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
             "query is tailable so must do a collection scan");
     }
 
-    // Hints require us to only consider the hinted index. If index filters in the query settings
-    // were used to override the allowed indices for planning, we should not use the hinted index
-    // requested in the query.
-    boost::optional<BSONObj> hintedIndexBson;
+    // Hints require us to only consider the hinted index. If index filters in the query
+    // settings were used to override the allowed indices for planning, we should not use the
+    // hinted index requested in the query.
+    boost::optional<BSONObj> hintedIndexBson = boost::none;
     if (!params.indexFiltersApplied) {
         if (auto hintObj = query.getFindCommandRequest().getHint(); !hintObj.isEmpty()) {
             hintedIndexBson = hintObj;
@@ -1064,13 +1202,14 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
     }
 
     // Figure out what fields we care about.
-    stdx::unordered_set<string> fields;
+    RelevantFieldIndexMap fields;
     QueryPlannerIXSelect::getFields(query.root(), &fields);
     for (auto&& field : fields) {
-        LOGV2_DEBUG(20970, 5, "Predicate over field", "field"_attr = field);
+        LOGV2_DEBUG(20970, 5, "Predicate over field", "field"_attr = field.first);
     }
 
-    fullIndexList = QueryPlannerIXSelect::expandIndexes(fields, std::move(fullIndexList));
+    fullIndexList = QueryPlannerIXSelect::expandIndexes(
+        fields, std::move(fullIndexList), hintedIndexBson != boost::none);
     std::vector<IndexEntry> relevantIndices;
 
     if (!hintedIndexEntry) {
@@ -1253,7 +1392,7 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
             // Store the plan cache index tree before calling prepareForAccessingPlanning(), so
             // that the PlanCacheIndexTree has the same sort as the MatchExpression used to
             // generate the plan cache key.
-            std::unique_ptr<MatchExpression> clone(nextTaggedTree->shallowClone());
+            std::unique_ptr<MatchExpression> clone(nextTaggedTree->clone());
             std::unique_ptr<PlanCacheIndexTree> cacheData;
             auto statusWithCacheData = cacheDataFromTaggedTree(clone.get(), relevantIndices);
             if (!statusWithCacheData.isOK()) {
@@ -1428,37 +1567,6 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
                 }
             }
         }
-
-        // The base index is sorted on some key, so it's possible we might want to use
-        // a collection scan to provide the sort requested
-        if (params.clusteredInfo) {
-            if (CollatorInterface::collatorsMatch(params.clusteredCollectionCollator,
-                                                  query.getCollator())) {
-                auto kp = clustered_util::getSortPattern(params.clusteredInfo->getIndexSpec());
-                int direction = 0;
-                if (providesSort(query, kp)) {
-                    direction = 1;
-                } else if (providesSort(query, QueryPlannerCommon::reverseSortObj(kp))) {
-                    direction = -1;
-                }
-
-                if (direction != 0) {
-                    auto soln = buildCollscanSoln(query, isTailable, params, direction);
-                    if (soln) {
-                        LOGV2_DEBUG(6082401,
-                                    5,
-                                    "Planner: outputting soln that uses clustered index to "
-                                    "provide sort");
-                        SolutionCacheData* scd = new SolutionCacheData();
-                        scd->solnType = SolutionCacheData::COLLSCAN_SOLN;
-                        scd->wholeIXSolnDir = direction;
-
-                        soln->cacheData.reset(scd);
-                        out.push_back(std::move(soln));
-                    }
-                }
-            }
-        }
     }
 
     // If a projection exists, there may be an index that allows for a covered plan, even if
@@ -1511,11 +1619,13 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
 
     // No indexed plans?  We must provide a collscan if possible or else we can't run the query.
     bool collScanRequired = 0 == out.size();
-    if (collScanRequired && !canTableScan(params)) {
+    if (collScanRequired && noTableAndClusteredIDXScan(params)) {
         return Status(ErrorCodes::NoQueryExecutionPlans,
                       "No indexed plans available, and running with 'notablescan'");
     }
 
+    bool clusteredCollection = params.clusteredInfo.has_value();
+
     // geoNear and text queries *require* an index.
     // Also, if a hint is specified it indicates that we MUST use it.
     bool possibleToCollscan =
@@ -1525,31 +1635,53 @@ StatusWith<std::vector<std::unique_ptr<QuerySolution>>> QueryPlanner::plan(
         return Status(ErrorCodes::NoQueryExecutionPlans, "No query solutions");
     }
 
-    if (possibleToCollscan && (collscanRequested || collScanRequired)) {
-        auto collscan = buildCollscanSoln(query, isTailable, params);
-        if (!collscan && collScanRequired) {
+    bool isClusteredIDXScan = false;
+    if (possibleToCollscan && (collscanRequested || collScanRequired || clusteredCollection)) {
+        auto clusteredScanDirection = determineClusteredScanDirection(query, params);
+        auto direction = clusteredScanDirection.value_or(1);
+        auto [collscanSoln, collscanNode] =
+            buildCollscanSolnWithNode(query, isTailable, params, direction);
+        if (!collscanSoln && collScanRequired) {
             return Status(ErrorCodes::NoQueryExecutionPlans,
                           "Failed to build collection scan soln");
         }
-        if (collscan) {
+        isClusteredIDXScan = isColusteredIDXScanSoln(collscanSoln.get());
+        // We consider collection scan in the following cases:
+        // 1. collScanRequested - specifically requested by caller.
+        // 2. collScanRequired - there are no other possible plans, so we fallback to full scan.
+        // 3. collscanIsBounded - collection is clustered and clustered index is used.
+        // 4. clusteredScanDirection - collection is clustered and sort, provided by clustered
+        // index, is used
+        if (collscanSoln &&
+            (collscanRequested || collScanRequired || collscanIsBounded(collscanNode) ||
+             clusteredScanDirection)) {
             LOGV2_DEBUG(20984,
                         5,
                         "Planner: outputting a collection scan",
-                        "collectionScan"_attr = redact(collscan->toString()));
+                        "collectionScan"_attr = redact(collscanSoln->toString()));
             SolutionCacheData* scd = new SolutionCacheData();
             scd->solnType = SolutionCacheData::COLLSCAN_SOLN;
-            collscan->cacheData.reset(scd);
-            out.push_back(std::move(collscan));
+            scd->wholeIXSolnDir = direction;
+            collscanSoln->cacheData.reset(scd);
+            out.push_back(std::move(collscanSoln));
         }
     }
 
+    // Make sure to respect the notablescan option. A clustered IDX scan is allowed even under a
+    // NOTABLE option. Only in the case of a strict NOTABLE scan option a clustered IDX scan is not
+    // allowed. This option is used in mongoS for shardPruning.
     invariant(out.size() > 0);
+    if (collScanRequired && noTableScan(params) && !isClusteredIDXScan) {
+        return Status(ErrorCodes::NoQueryExecutionPlans,
+                      "No indexed plans available, and running with 'notablescan'");
+    }
     return {std::move(out)};
 }
 
 /**
- * The 'query' might contain parts of aggregation pipeline. For now, we plan those separately and
- * later attach the agg portion of the plan to the solution(s) for the "find" part of the query.
+ * The 'query' might contain parts of aggregation pipeline. For now, we plan those separately
+ * and later attach the agg portion of the plan to the solution(s) for the "find" part of the
+ * query.
  */
 std::unique_ptr<QuerySolution> QueryPlanner::extendWithAggPipeline(
     const CanonicalQuery& query,
@@ -1576,9 +1708,9 @@ std::unique_ptr<QuerySolution> QueryPlanner::extendWithAggPipeline(
         if (lookupStage) {
             tassert(6369000,
                     "This $lookup stage should be compatible with SBE",
-                    lookupStage->sbeCompatible());
+                    lookupStage->sbeCompatibility() != SbeCompatibility::notCompatible);
             auto [strategy, idxEntry] = QueryPlannerAnalysis::determineLookupStrategy(
-                lookupStage->getFromNs().toString(),
+                lookupStage->getFromNs(),
                 lookupStage->getForeignField()->fullPath(),
                 secondaryCollInfos,
                 query.getExpCtx()->allowDiskUse,
@@ -1601,7 +1733,12 @@ std::unique_ptr<QuerySolution> QueryPlanner::extendWithAggPipeline(
     }
 
     solution->extendWith(std::move(solnForAgg));
-    return QueryPlannerAnalysis::removeInclusionProjectionBelowGroup(std::move(solution));
+
+    solution = QueryPlannerAnalysis::removeInclusionProjectionBelowGroup(std::move(solution));
+
+    QueryPlannerAnalysis::removeUselessColumnScanRowStoreExpression(*solution->root());
+
+    return std::move(solution);
 }
 
 StatusWith<std::unique_ptr<QuerySolution>> QueryPlanner::choosePlanForSubqueries(
@@ -1715,7 +1852,7 @@ StatusWith<QueryPlanner::SubqueriesPlanningResult> QueryPlanner::planSubqueries(
     invariant(query.root()->matchType() == MatchExpression::OR);
     invariant(query.root()->numChildren(), "Cannot plan subqueries for an $or with no children");
 
-    SubqueriesPlanningResult planningResult{query.root()->shallowClone()};
+    SubqueriesPlanningResult planningResult{query.root()->clone()};
     for (size_t i = 0; i < params.indices.size(); ++i) {
         const IndexEntry& ie = params.indices[i];
         const auto insertionRes = planningResult.indexMap.insert(std::make_pair(ie.identifier, i));
@@ -1729,13 +1866,13 @@ StatusWith<QueryPlanner::SubqueriesPlanningResult> QueryPlanner::planSubqueries(
         planningResult.branches.push_back(
             std::make_unique<SubqueriesPlanningResult::BranchPlanningResult>());
         auto branchResult = planningResult.branches.back().get();
-        auto orChild = planningResult.orExpression->getChild(i);
 
         // Turn the i-th child into its own query.
-        auto statusWithCQ = CanonicalQuery::canonicalize(opCtx, query, orChild);
+        auto statusWithCQ = CanonicalQuery::makeForSubplanner(opCtx, query, i);
         if (!statusWithCQ.isOK()) {
             str::stream ss;
-            ss << "Can't canonicalize subchild " << orChild->debugString() << " "
+            ss << "Can't canonicalize subchild "
+               << planningResult.orExpression->getChild(i)->debugString() << " "
                << statusWithCQ.getStatus().reason();
             return Status(ErrorCodes::BadValue, ss);
         }
