@@ -40,7 +40,8 @@ namespace {
 
 
 TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesQuery) {
-    params.options = QueryPlannerParams::IS_COUNT;
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(BSON("x" << 5));
     ASSERT_EQUALS(getNumSolutions(), 1U);
@@ -48,7 +49,8 @@ TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesQuery
 }
 
 TEST_F(QueryPlannerTest, PlannerAddsFetchToIxscanForCountWhenFetchFilterNonempty) {
-    params.options = QueryPlannerParams::IS_COUNT;
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
     addIndex(BSON("x" << 1));
     runQuery(BSON("y" << 3 << "x" << 5));
     ASSERT_EQUALS(getNumSolutions(), 1U);
@@ -57,6 +59,102 @@ TEST_F(QueryPlannerTest, PlannerAddsFetchToIxscanForCountWhenFetchFilterNonempty
         "{pattern: {x: 1}, bounds: {x: [[5,5,true,true]]}}}}}");
 }
 
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenIndexSatisfiesNullQuery) {
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    addIndex(BSON("x" << 1));
+    runQuery(fromjson("{x: null}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanWhenIndexSatisfiesNullAndOtherQuery) {
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    addIndex(BSON("x" << 1));
+    runQuery(fromjson("{x: {$in: [null, 2]}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true], [2, 2, true, true]]}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerAddsFetchForCountWhenMultikeyIndexSatisfiesNullQuery) {
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    MultikeyPaths multikeyPaths{{0U}};
+    addIndex(fromjson("{x: 1}"), multikeyPaths);
+    runQuery(fromjson("{x: null}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{fetch: {filter: {x: null}, node: {ixscan: "
+        "{pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanForCountWhenMultikeyIndexSatisfiesNullEmptyQuery) {
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    MultikeyPaths multikeyPaths{{0U}};
+    addIndex(fromjson("{x: 1}"), multikeyPaths);
+    runQuery(fromjson("{x: {$in: [null, []]}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true], [[], [], true, true]]}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanWhenMultikeyIndexSatisfiesNullEmptyAndOtherQuery) {
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
+    MultikeyPaths multikeyPaths{{0U}};
+    addIndex(fromjson("{x: 1}"), multikeyPaths);
+    runQuery(fromjson("{x: {$in: [null, [], 2]}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{ixscan: {pattern: {x: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true], [2, 2, true, true], [[], [], "
+        "true, true]]}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanFindWhenIndexSatisfiesNullQuery) {
+    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    addIndex(fromjson("{x: 1, _id: 1}"));
+    runQuerySortProj(fromjson("{x: null}}"), BSONObj(), fromjson("{_id: 1}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{proj: {spec: {_id: 1}, node:"
+        "{ixscan: {pattern: {x: 1, _id: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerAddsFetchForFindWhenMultikeyIndexSatisfiesNullQuery) {
+    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    MultikeyPaths multikeyPaths{{0U}, MultikeyComponents{}};
+    addIndex(fromjson("{x: 1, _id: 1}"), multikeyPaths);
+    runQuerySortProj(fromjson("{x: null}}"), BSONObj(), fromjson("{_id: 1}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{proj: {spec: {_id: 1}, node:"
+        "{fetch: {filter: {x: null}, node: {ixscan: "
+        "{pattern: {x: 1, _id: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true]]}}}}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerUsesCoveredIxscanFindWhenMultikeyIndexSatisfiesNullEmptyQuery) {
+    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
+    MultikeyPaths multikeyPaths{{0U}, MultikeyComponents{}};
+    addIndex(fromjson("{x: 1, _id: 1}"), multikeyPaths);
+    runQuerySortProj(fromjson("{x: {$in: [null, []]}}}"), BSONObj(), fromjson("{_id: 1}}"));
+    ASSERT_EQUALS(getNumSolutions(), 1U);
+    assertSolutionExists(
+        "{proj: {spec: {_id: 1}, node:"
+        "{ixscan: {pattern: {x: 1, _id: 1}, bounds: "
+        "{x: [[undefined,undefined,true,true],[null,null,true,true], [[], [], true, true]]}}}}}");
+}
+
 //
 // Sparse indices, SERVER-8067
 // Each index in this block of tests is sparse.
@@ -161,8 +259,9 @@ TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseMultikeyIndex)
         "{fetch: {filter: {a: {$not: {$in: [null, []]}}}, node: {ixscan: "
         "{pattern: {a: 1}, bounds: "
         "{a: ["
-        "['MinKey', undefined, true, true],"
-        "[null, 'MaxKey', false, true]"
+        "['MinKey', undefined, true, false],"
+        "[null, [], false, false],"
+        "[[], 'MaxKey', false, true]"
         "]}}}}}");
 }
 
@@ -176,8 +275,9 @@ TEST_F(QueryPlannerTest, NinListWithOnlyNullAndEmptyArrayShouldUseIndex) {
         "{fetch: {node: {ixscan: "
         "{pattern: {a: 1}, bounds: "
         "{a: ["
-        "['MinKey', undefined, true, true],"
-        "[null, 'MaxKey', false, true]"
+        "['MinKey', undefined, true, false],"
+        "[null, [], false, false],"
+        "[[], 'MaxKey', false, true]"
         "]}}}}}");
 }
 
@@ -200,6 +300,12 @@ TEST_F(QueryPlannerTest, NinListWithNonEmptyArrayShouldNotUseIndex) {
     assertHasOnlyCollscan();
 }
 
+TEST_F(QueryPlannerTest, NotLtEmptyArrayShouldNotUseIndex) {
+    addIndex(fromjson("{a: 1}"), true);
+    runQuery(fromjson("{a: { $not: { $lt: [] } } }"));
+    assertHasOnlyCollscan();
+}
+
 TEST_F(QueryPlannerTest, SparseIndexCannotSupportEqualsNull) {
     addIndex(BSON("i" << 1),
              false,  // multikey
@@ -208,29 +314,46 @@ TEST_F(QueryPlannerTest, SparseIndexCannotSupportEqualsNull) {
 
     runQuery(fromjson("{i: {$eq: null}}"));
     assertHasOnlyCollscan();
-}
-
-// TODO: SERVER-37164: The semantics of {$gte: null} and {$lte: null} are inconsistent with and
-// without a sparse index. It is unclear whether or not a sparse index _should_ be able to support
-// these operations.
-TEST_F(QueryPlannerTest, SparseIndexCanSupportGTEOrLTENull) {
-    params.options &= ~QueryPlannerParams::INCLUDE_COLLSCAN;
-    addIndex(BSON("i" << 1),
-             false,  // multikey
-             true    // sparse
-    );
 
     runQuery(fromjson("{i: {$gte: null}}"));
-    assertNumSolutions(1U);
-    assertSolutionExists(
-        "{fetch: {filter: {i: {$gte: null}}, node: {ixscan: {pattern: "
-        "{i: 1}, bounds: {i: [[undefined,undefined,true,true], [null,null,true,true]]}}}}}");
+    assertHasOnlyCollscan();
 
     runQuery(fromjson("{i: {$lte: null}}"));
-    assertNumSolutions(1U);
-    assertSolutionExists(
-        "{fetch: {filter: {i: {$lte: null}}, node: {ixscan: {pattern: "
-        "{i: 1}, bounds: {i: [[undefined,undefined,true,true], [null,null,true,true]]}}}}}");
+    assertHasOnlyCollscan();
+}
+
+TEST_F(QueryPlannerTest, PlannerCanUseIndexesWithSameKeyButDifferentSparseProperty) {
+    // Create two indexes on the same key pattern; one sparse, the other non-sparse. This is
+    // permitted because the 'sparse' property is part of the index signature.
+    addIndex(fromjson("{a: 1}"), /*multikey*/ false, /*sparse*/ false);
+    addIndex(fromjson("{a: 1}"), /*multikey*/ false, /*sparse*/ true, /*unique*/ false, "a_sparse");
+
+    runQuery(fromjson("{a: 1}"));
+
+    // Plan #1: FETCH > IXSCAN with a_1 index.
+    // Plan #2: FETCH > IXSCAN with a_sparse index.
+    // Plan #3: COLLSCAN with filter a == 1.
+    assertNumSolutions(3U);
+
+    // There must be a solution that uses the sparse index "a_sparse".
+    assertSolutionExists("{fetch: {node: {ixscan: {name: \"a_sparse\"}}}}");
+}
+
+TEST_F(QueryPlannerTest, PlannerCanUseIndexesWithSameKeyButDifferentUniqueProperty) {
+    // Create two indexes on the same key pattern; one unique, the other non-unique. This is
+    // permitted because the 'unique' property is part of the index signature.
+    addIndex(fromjson("{a: 1}"), /*multikey*/ false, /*sparse*/ false, /*unique*/ false);
+    addIndex(fromjson("{a: 1}"), /*multikey*/ false, /*sparse*/ false, /*unique*/ true, "a_unique");
+
+    runQuery(fromjson("{a: 1}"));
+
+    // Plan #1: FETCH > IXSCAN with a_1 index.
+    // Plan #2: FETCH > IXSCAN with a_unique index.
+    // Plan #3: COLLSCAN with filter a == 1.
+    assertNumSolutions(3U);
+
+    // There must be a solution that uses the unique index "a_unique".
+    assertSolutionExists("{fetch: {node: {ixscan: {name: \"a_unique\"}}}}");
 }
 
 //
@@ -494,12 +617,9 @@ TEST_F(QueryPlannerTest, IndexBoundsAndWithNestedOr) {
 
     // It's kind of iffy to look for indexed solutions so we don't...
     size_t matches = 0;
-    matches += numSolutionMatches(
-        "{cscan: {dir: 1, filter: "
-        "{$or: [{a: 2, a:1}, {a: 3, a:1}]}}}");
-    matches += numSolutionMatches(
-        "{cscan: {dir: 1, filter: "
-        "{$and: [{$or: [{a: 2}, {a: 3}]}, {a: 1}]}}}");
+    matches += numSolutionMatches("{cscan: {dir: 1, filter: {$or: [{a: 2, a:1}, {a: 3, a:1}]}}}");
+    matches +=
+        numSolutionMatches("{cscan: {dir: 1, filter: {$and: [{a: {$in: [2, 3]}}, {a: 1}]}}}");
     ASSERT_GREATER_THAN_OR_EQUALS(matches, 1U);
 }
 
@@ -510,7 +630,7 @@ TEST_F(QueryPlannerTest, IndexBoundsIndexedSort) {
     assertNumSolutions(2U);
     assertSolutionExists(
         "{sort: {pattern: {a:1}, limit: 0, type: 'simple', node: "
-        "{cscan: {filter: {$or:[{a:1},{a:2}]}, dir: 1}}}}");
+        "{cscan: {filter: {a: {$in: [1,2]}}, dir: 1}}}}");
     assertSolutionExists(
         "{fetch: {filter: null, node: {ixscan: {filter: null, "
         "pattern: {a:1}, bounds: {a: [[1,1,true,true], [2,2,true,true]]}}}}}");
@@ -523,7 +643,7 @@ TEST_F(QueryPlannerTest, IndexBoundsUnindexedSort) {
     assertNumSolutions(2U);
     assertSolutionExists(
         "{sort: {pattern: {b:1}, limit: 0, type: 'simple', node: "
-        "{cscan: {filter: {$or:[{a:1},{a:2}]}, dir: 1}}}}");
+        "{cscan: {filter: {a: {$in: [1,2]}}, dir: 1}}}}");
     assertSolutionExists(
         "{sort: {pattern: {b:1}, limit: 0, type: 'simple', node: {fetch: "
         "{filter: null, node: {ixscan: {filter: null, "
