@@ -56,7 +56,7 @@ protected:
     }
 
     void addWildcardIndex(BSONObj keyPattern,
-                          const std::set<std::string>& multikeyPathSet = {},
+                          const OrderedPathSet& multikeyPathSet = {},
                           BSONObj wildcardProjection = BSONObj{},
                           MatchExpression* partialFilterExpr = nullptr,
                           CollatorInterface* collator = nullptr,
@@ -75,6 +75,7 @@ protected:
 
         params.indices.push_back({std::move(keyPattern),
                                   IndexType::INDEX_WILDCARD,
+                                  IndexDescriptor::kLatestIndexVersion,
                                   isMultikey,
                                   {},  // multikeyPaths
                                   std::move(multikeyFieldRefs),
@@ -94,7 +95,9 @@ protected:
 // General planning tests.
 //
 
-DEATH_TEST_F(QueryPlannerWildcardTest, CannotExpandPreExpandedWildcardIndexEntry, "Invariant") {
+DEATH_TEST_F(QueryPlannerWildcardTest,
+             CannotExpandPreExpandedWildcardIndexEntry,
+             "Tripwire assertion") {
     addWildcardIndex(BSON("$**" << 1));
     ASSERT_EQ(params.indices.size(), 2U);
 
@@ -104,7 +107,7 @@ DEATH_TEST_F(QueryPlannerWildcardTest, CannotExpandPreExpandedWildcardIndexEntry
     ASSERT_EQ(expandedIndex.size(), 1U);
     params.indices.push_back(expandedIndex.front());
 
-    // Now run a query. This will invariant when the planner expands the expanded index.
+    // Now run a query. This will tassert when the planner expands the expanded index.
     runQuery(fromjson("{a: 1}"));
 }
 
@@ -568,7 +571,7 @@ TEST_F(QueryPlannerWildcardTest, OrWithOneRegularAndOneWildcardIndexPathUsesTwoI
 
 TEST_F(QueryPlannerWildcardTest, BasicSkip) {
     addWildcardIndex(BSON("$**" << 1));
-    runQuerySkipNToReturn(BSON("a" << 5), 8, 0);
+    runQuerySkipLimit(BSON("a" << 5), 8, 0);
 
     assertNumSolutions(1U);
     assertSolutionExists(
@@ -579,7 +582,7 @@ TEST_F(QueryPlannerWildcardTest, BasicSkip) {
 
 TEST_F(QueryPlannerWildcardTest, CoveredSkip) {
     addWildcardIndex(BSON("$**" << 1));
-    runQuerySortProjSkipNToReturn(fromjson("{a: 5}"), BSONObj(), fromjson("{_id: 0, a: 1}"), 8, 0);
+    runQuerySortProjSkipLimit(fromjson("{a: 5}"), BSONObj(), fromjson("{_id: 0, a: 1}"), 8, 0);
 
     assertNumSolutions(1U);
     assertSolutionExists(
@@ -590,7 +593,7 @@ TEST_F(QueryPlannerWildcardTest, CoveredSkip) {
 
 TEST_F(QueryPlannerWildcardTest, BasicLimit) {
     addWildcardIndex(BSON("$**" << 1));
-    runQuerySkipNToReturn(BSON("a" << 5), 0, -5);
+    runQuerySkipLimit(BSON("a" << 5), 0, 5);
 
     assertNumSolutions(1U);
     assertSolutionExists(
@@ -621,7 +624,8 @@ TEST_F(QueryPlannerWildcardTest, DottedFieldCovering) {
 }
 
 TEST_F(QueryPlannerWildcardTest, CoveredIxscanForCountOnIndexedPath) {
-    params.options = QueryPlannerParams::IS_COUNT;
+    params.options = QueryPlannerParams::DEFAULT;
+    setIsCountLike();
     addWildcardIndex(BSON("$**" << 1));
     runQuery(fromjson("{a: 5}"));
 
@@ -1832,9 +1836,13 @@ TEST_F(QueryPlannerWildcardTest, CanAnswerInContainingEmptyObjectWhenPathIsMulti
 }
 
 TEST_F(QueryPlannerWildcardTest, CanProduceSortMergePlanWithWildcardIndex) {
+    auto defaultMaxOr = internalQueryEnumerationMaxOrSolutions.load();
+    ON_BLOCK_EXIT([&] { internalQueryEnumerationMaxOrSolutions.store(defaultMaxOr); });
+    internalQueryEnumerationMaxOrSolutions.store(5);
     addWildcardIndex(BSON("$**" << 1));
     addIndex(BSON("a" << 1 << "b" << 1));
-    runQueryAsCommand(fromjson("{filter: {$or: [{a: 1, b: 1}, {b: 2}]}, sort: {b: -1}}"));
+    runQueryAsCommand(
+        fromjson("{find: 'testns', filter: {$or: [{a: 1, b: 1}, {b: 2}]}, sort: {b: -1}}"));
     assertNumSolutions(3U);
     assertSolutionExists(
         "{fetch: {filter: null, node: {mergeSort: {nodes: ["
@@ -1858,6 +1866,9 @@ TEST_F(QueryPlannerWildcardTest, CanProduceSortMergePlanWithWildcardIndex) {
 }
 
 TEST_F(QueryPlannerWildcardTest, ContainedOrPushdownWorksWithWildcardIndex) {
+    auto defaultMaxOr = internalQueryEnumerationMaxOrSolutions.load();
+    ON_BLOCK_EXIT([&] { internalQueryEnumerationMaxOrSolutions.store(defaultMaxOr); });
+    internalQueryEnumerationMaxOrSolutions.store(5);
     addWildcardIndex(BSON("$**" << 1));
     addIndex(BSON("a" << 1 << "b" << 1));
     runQuery(fromjson("{a: 1, $or: [{c: 2}, {b: 3}]}"));
