@@ -230,7 +230,7 @@ std::pair<SlotId /* keyValueSlot */, std::unique_ptr<sbe::PlanStage>> buildForei
 
         SlotId getFieldSlot = slotIdGenerator.generate();
         currentStage = makeProjectStage(
-            move(currentStage), nodeId, getFieldSlot, std::move(getFieldFromObject));
+            std::move(currentStage), nodeId, getFieldSlot, std::move(getFieldFromObject));
         keyValueSlot = getFieldSlot;
 
         // For the terminal array we will do the extra work of adding the array itself to the stream
@@ -322,6 +322,25 @@ std::pair<SlotId /* keyValueSlot */, std::unique_ptr<sbe::PlanStage>> buildForei
     return {keyValueSlot, std::move(currentStage)};
 }
 
+std::pair<SlotId /* keyValuesSetSlot */, EvalStage> replaceEmptySetWithNullArray(
+    EvalStage innerStage,
+    SlotId innerRecordSlot,
+    SlotIdGenerator& slotIdGenerator,
+    const PlanNodeId nodeId) {
+    auto [arrayWithNullTag, arrayWithNullVal] = makeNewArray();
+    auto arrayWithNull = makeConstant(arrayWithNullTag, arrayWithNullVal);
+    value::Array* arrayWithNullView = getArrayView(arrayWithNullVal);
+    arrayWithNullView->push_back(TypeTags::Null, 0);
+    auto nonEmptySetSlot = slotIdGenerator.generate();
+    return {nonEmptySetSlot,
+            makeProject(std::move(innerStage),
+                        nodeId,
+                        nonEmptySetSlot,
+                        makeE<EIf>(makeFunction("isArrayEmpty", makeVariable(innerRecordSlot)),
+                                   std::move(arrayWithNull),
+                                   makeVariable(innerRecordSlot)))};
+}
+
 // Creates stages for traversing path 'fp' in the record from 'inputSlot'. Puts the set of key
 // values into 'keyValuesSetSlot. For example, if the record in the 'inputSlot' is:
 //     {a: [{b:[1,[2,3]]}, {b:4}, {b:1}, {b:2}]},
@@ -345,12 +364,15 @@ std::pair<SlotId /* keyValuesSetSlot */, std::unique_ptr<sbe::PlanStage>> buildK
     // Re-pack the individual key values into a set. We don't cap "addToSet" here because its size
     // is bounded by the size of the record.
     SlotId keyValuesSetSlot = slotIdGenerator.generate();
+    SlotId spillSlot = slotIdGenerator.generate();
     EvalStage packedKeyValuesStage = makeHashAgg(
         EvalStage{std::move(keyValuesStage), SlotVector{}},
         makeSV(), /* groupBy slots - "none" means creating a single group */
-        makeEM(keyValuesSetSlot, makeFunction("addToSet"_sd, makeVariable(keyValueSlot))),
+        makeAggExprVector(
+            keyValuesSetSlot, nullptr, makeFunction("addToSet"_sd, makeVariable(keyValueSlot))),
         boost::none /* we group _all_ key values into a single set, so collator is irrelevant */,
         allowDiskUse,
+        makeSlotExprPairVec(spillSlot, makeFunction("aggSetUnion"_sd, makeVariable(spillSlot))),
         nodeId);
 
     // The set in 'keyValuesSetSlot' might end up empty if the localField contained only missing and
@@ -359,35 +381,21 @@ std::pair<SlotId /* keyValuesSetSlot */, std::unique_ptr<sbe::PlanStage>> buildK
     // that contains a single 'null' value. The set of foreign key values also can be empty but it
     // should produce no matches so we leave it empty.
     if (joinSide == JoinSide::Local) {
-        auto [arrayWithNullTag, arrayWithNullVal] = makeNewArray();
-        std::unique_ptr<EExpression> arrayWithNull =
-            makeConstant(arrayWithNullTag, arrayWithNullVal);
-        value::Array* arrayWithNullView = getArrayView(arrayWithNullVal);
-        arrayWithNullView->push_back(TypeTags::Null, 0);
-
-        std::unique_ptr<EExpression> isNonEmptySetExpr =
-            makeBinaryOp(EPrimBinary::greater,
-                         makeFunction("getArraySize", makeVariable(keyValuesSetSlot)),
-                         makeConstant(TypeTags::NumberInt32, 0));
-
-        SlotId nonEmptySetSlot = slotIdGenerator.generate();
-        packedKeyValuesStage = makeProject(std::move(packedKeyValuesStage),
-                                           nodeId,
-                                           nonEmptySetSlot,
-                                           makeE<EIf>(std::move(isNonEmptySetExpr),
-                                                      makeVariable(keyValuesSetSlot),
-                                                      std::move(arrayWithNull)));
-        keyValuesSetSlot = nonEmptySetSlot;
+        std::tie(keyValuesSetSlot, packedKeyValuesStage) = replaceEmptySetWithNullArray(
+            std::move(packedKeyValuesStage),  // NOLINT(bugprone-use-after-move)
+            keyValuesSetSlot,
+            slotIdGenerator,
+            nodeId);
     }
 
     // Attach the set of key values to the original local record.
-    std::unique_ptr<sbe::PlanStage> nljLocalWithKeyValuesSet =
-        makeS<LoopJoinStage>(std::move(inputStage),
-                             std::move(packedKeyValuesStage.stage),
-                             makeSV(recordSlot) /* outerProjects */,
-                             makeSV(recordSlot) /* outerCorrelated */,
-                             nullptr /* predicate */,
-                             nodeId);
+    std::unique_ptr<sbe::PlanStage> nljLocalWithKeyValuesSet = makeS<LoopJoinStage>(
+        std::move(inputStage),
+        packedKeyValuesStage.extractStage(nodeId),  // NOLINT(bugprone-use-after-move)
+        makeSV(recordSlot) /* outerProjects */,
+        makeSV(recordSlot) /* outerCorrelated */,
+        nullptr /* predicate */,
+        nodeId);
 
     return {keyValuesSetSlot, std::move(nljLocalWithKeyValuesSet)};
 }
@@ -406,15 +414,21 @@ std::pair<SlotId /* resultSlot */, std::unique_ptr<sbe::PlanStage>> buildForeign
     // are no matches, return an empty array.
     const int sizeCap = internalLookupStageIntermediateDocumentMaxSizeBytes.load();
     SlotId accumulatorSlot = slotIdGenerator.generate();
+    SlotId spillSlot = slotIdGenerator.generate();
     innerBranch = makeHashAgg(
         std::move(innerBranch),
         makeSV(), /* groupBy slots */
-        makeEM(accumulatorSlot,
-               makeFunction("addToArrayCapped"_sd,
-                            makeVariable(foreignRecordSlot),
-                            makeConstant(TypeTags::NumberInt32, sizeCap))),
+        makeAggExprVector(accumulatorSlot,
+                          nullptr,
+                          makeFunction("addToArrayCapped"_sd,
+                                       makeVariable(foreignRecordSlot),
+                                       makeConstant(TypeTags::NumberInt32, sizeCap))),
         {} /* collatorSlot, no collation here because we want to return all matches "as is" */,
         allowDiskUse,
+        makeSlotExprPairVec(spillSlot,
+                            makeFunction("aggConcatArraysCapped",
+                                         makeVariable(spillSlot),
+                                         makeConstant(TypeTags::NumberInt32, sizeCap))),
         nodeId);
 
     // 'accumulatorSlot' is either Nothing or contains an array of size two, where the front element
@@ -446,7 +460,7 @@ std::pair<SlotId /* resultSlot */, std::unique_ptr<sbe::PlanStage>> buildForeign
 
     SlotId unionOutputSlot = slotIdGenerator.generate();
     EvalStage unionStage =
-        makeUnion(makeVector(EvalStage{std::move(innerBranch.stage), SlotVector{}},
+        makeUnion(makeVector(EvalStage{innerBranch.extractStage(nodeId), SlotVector{}},
                              EvalStage{std::move(emptyArrayStage), SlotVector{}}),
                   {makeSV(matchedRecordsSlot), makeSV(emptyArraySlot)} /* inputs */,
                   makeSV(unionOutputSlot),
@@ -454,54 +468,127 @@ std::pair<SlotId /* resultSlot */, std::unique_ptr<sbe::PlanStage>> buildForeign
 
     return std::make_pair(
         unionOutputSlot,
-        std::move(makeLimitSkip(std::move(unionStage), nodeId, 1 /* limit */).stage));
+        makeLimitSkip(std::move(unionStage), nodeId, 1 /* limit */).extractStage(nodeId));
 }
 
-// Creates stages that extract key values from the given foreign record, compares them to the local
-// key values and groups the matching records into an array.
+/**
+ * Build keys set for NLJ foreign side using traverseF expression. Creates stages that extract key
+ * values from the given foreign record, compares them to the local key values and groups the
+ * matching records into an array.
+ *
+ * The traverseF expression will iterate each key value, including terminal arrays as
+ * a whole value, and compare it against local key set 'localKeySlot'. For example,
+ * if the record in the 'foreignRecordSlot' is:
+ *     {a: [{b:[1,[2,3]]}, {b:4}, {b:1}, {b:2}]},
+ * path "a.b" will be iterated as: 1, [2,3], [1, [2, 3]], 4, 1, 2.
+ * Scalars inside arrays on the path are skipped, that is, if the record in the 'foreignRecordSlot'
+ * is:  {a: [42, {b:{c:1}}, {b: [41,42,{c:2}]}, {b:42}, {b:{c:3}}]},
+ * path "a.b.c" will be iterated as: 1, 2, null, 3.
+ * Replaces other missing terminals with 'null', that is, if the record in the 'foreignRecordSlot'
+ * is:  {a: [{b:1}, {b:[]}, {no_b:42}, {b:2}]},
+ * path "a.b" will be iterated as: 1, [], null, 2.
+ *
+ * Here is an example plan for the NLJ inner side:
+ * limit 1
+ * union [unionOutputSlot] [
+ *   branch0[projOutputSlot]
+ *     project [projOutputSlot = getElement(groupSlot, 0)]
+ *     group [] [groupSlot = addToArrayCapped(foreignRecordSlot, 104857600)]
+ *     filter {traverseF (
+ *       let [
+ *           l11.0 = fillEmpty (getField (foreignRecordSlot, "a"), null)
+ *       ]
+ *       in
+ *           if typeMatch (l11.0, 24)
+ *           then l11.0
+ *           else Nothing
+ *       , lambda(l3.0) {
+ *           if fillEmpty (isObject (l3.0), true)
+ *           then traverseF (
+ *             fillEmpty (getField (l3.0, "b"), null), lambda(l2.0) {isMember (l2.0, localKeySlot)},
+ *             true),
+ *           else false
+ *        }, false)}
+ *     scan foreignRecordSlot recordIdSlot none none none none [] @uuid true false
+ * branch1[emptySlot] project [emptySlot = []] limit 1 coscan
+ * ]
+ */
 std::pair<SlotId /* matched docs */, std::unique_ptr<sbe::PlanStage>> buildForeignMatches(
-    SlotId localKeysSetSlot,
+    SlotId localKeySlot,
     std::unique_ptr<sbe::PlanStage> foreignStage,
     SlotId foreignRecordSlot,
     const FieldPath& foreignFieldName,
     boost::optional<SlotId> collatorSlot,
     const PlanNodeId nodeId,
     SlotIdGenerator& slotIdGenerator,
+    FrameIdGenerator& frameIdGenerator,
     bool allowDiskUse) {
-    // 'buildForeignKeysStream()' does not take a child stage. To connect the extraction of key
-    // values it does to the 'foreignStage' we'll use an 'nlj', inner branch of which will do the
-    // key value extraction and filtering. It's possible that the same foreign record contains
-    // multiple matching values and to avoid duplication of the record we must limit the inner
-    // branch to '1'.
-    auto [foreignValueSlot, currentRootStage] =
-        buildForeignKeysStream(foreignRecordSlot, foreignFieldName, nodeId, slotIdGenerator);
+    auto frameId = frameIdGenerator.generate();
+    auto lambdaArg = makeVariable(frameId, 0);
+    auto filter = collatorSlot
+        ? makeFunction("collIsMember"_sd,
+                       makeVariable(*collatorSlot),
+                       lambdaArg->clone(),
+                       makeVariable(localKeySlot))
+        : makeFunction("isMember"_sd, lambdaArg->clone(), makeVariable(localKeySlot));
 
-    currentRootStage =
-        makeLimitTree(makeS<FilterStage<false /*IsConst*/>>(
-                          std::move(currentRootStage),
-                          collatorSlot ? makeFunction("collIsMember",
-                                                      makeVariable(*collatorSlot),
-                                                      makeVariable(foreignValueSlot),
-                                                      makeVariable(localKeysSetSlot))
-                                       : makeFunction("isMember",
-                                                      makeVariable(foreignValueSlot),
-                                                      makeVariable(localKeysSetSlot)),
-                          nodeId),
-                      nodeId,
-                      1 /* take the foreign record once, even if multiple key values match */);
+    // Recursively create traverseF expressions to iterate elements in 'foreignRecordSlot' with path
+    // 'foreignFieldName', and check if key is in set 'localKeySlot'.
+    //
+    // If a non-terminal field is an array, we will ignore any element that is not an object inside
+    // the array.
+    const int32_t foreignPathLength = foreignFieldName.getPathLength();
+    for (int32_t i = foreignPathLength - 1; i >= 0; --i) {
+        auto arrayLambda = makeE<ELocalLambda>(frameId, std::move(filter));
+        frameId = frameIdGenerator.generate();
+        lambdaArg = i == 0 ? makeVariable(foreignRecordSlot) : makeVariable(frameId, 0);
 
-    currentRootStage = makeS<LoopJoinStage>(std::move(foreignStage) /* outer */,
-                                            std::move(currentRootStage) /* inner */,
-                                            makeSV(foreignRecordSlot) /* outerProjects */,
-                                            makeSV(foreignRecordSlot) /* outerCorrelated */,
-                                            nullptr,
-                                            nodeId);
+        auto getFieldOrNull = makeFillEmptyNull(makeFunction(
+            "getField"_sd, lambdaArg->clone(), makeConstant(foreignFieldName.getFieldName(i))));
+
+        // Non object/array field will be converted into Nothing, passing along recursive traverseF
+        // and will be treated as null to compared against local key set.
+        if (i != foreignPathLength - 1) {
+            getFieldOrNull = makeLocalBind(
+                &frameIdGenerator,
+                [&](sbe::EVariable var) {
+                    return sbe::makeE<sbe::EIf>(
+                        makeFunction("typeMatch"_sd,
+                                     var.clone(),
+                                     makeConstant(sbe::value::TypeTags::NumberInt64,
+                                                  sbe::value::bitcastFrom<int64_t>(
+                                                      getBSONTypeMask(BSONType::Array) |
+                                                      getBSONTypeMask(BSONType::Object)))),
+                        var.clone(),
+                        makeConstant(sbe::value::TypeTags::Nothing, 0));
+                },
+                std::move(getFieldOrNull));
+        }
+
+        filter = makeFunction(
+            "traverseF"_sd,
+            std::move(getFieldOrNull),
+            std::move(arrayLambda),
+            makeConstant(TypeTags::Boolean, i == foreignPathLength - 1) /*compareArray*/);
+
+        if (i > 0) {
+            // Ignoring the nulls produced by missing field in array.
+            filter =
+                sbe::makeE<sbe::EIf>(makeBinaryOp(sbe::EPrimBinary::fillEmpty,
+                                                  makeFunction("isObject"_sd, lambdaArg->clone()),
+                                                  makeConstant(TypeTags::Boolean, true)),
+                                     std::move(filter),
+                                     makeConstant(TypeTags::Boolean, false));
+        }
+    }
 
     // Group the matched foreign documents into a list.
     // It creates a union stage internally so that when there's no matching foreign records, an
     // empty array will be returned.
     return buildForeignMatchedArray(
-        EvalStage{std::move(currentRootStage), makeSV(foreignRecordSlot)},
+        makeFilter<false /*IsConst*/>(EvalStage{std::move(foreignStage), makeSV(foreignRecordSlot)},
+                                      std::move(filter),
+                                      nodeId),
         foreignRecordSlot,
         nodeId,
         slotIdGenerator,
@@ -515,10 +602,11 @@ std::pair<SlotId /* matched docs */, std::unique_ptr<sbe::PlanStage>> buildNljLo
     const FieldPath& localFieldName,
     std::unique_ptr<sbe::PlanStage> foreignStage,
     SlotId foreignRecordSlot,
-    StringData foreignFieldName,
+    const FieldPath& foreignFieldName,
     boost::optional<SlotId> collatorSlot,
     const PlanNodeId nodeId,
-    SlotIdGenerator& slotIdGenerator) {
+    SlotIdGenerator& slotIdGenerator,
+    FrameIdGenerator& frameIdGenerator) {
     CurOp::get(state.opCtx)->debug().nestedLoopJoin += 1;
 
     // Build the outer branch that produces the set of local key values.
@@ -540,6 +628,7 @@ std::pair<SlotId /* matched docs */, std::unique_ptr<sbe::PlanStage>> buildNljLo
                                                                     collatorSlot,
                                                                     nodeId,
                                                                     slotIdGenerator,
+                                                                    frameIdGenerator,
                                                                     state.allowDiskUse);
 
     // 'innerRootStage' should not participate in trial run tracking as the number of reads that
@@ -556,7 +645,6 @@ std::pair<SlotId /* matched docs */, std::unique_ptr<sbe::PlanStage>> buildNljLo
                              makeSV(localKeySlot) /* outerCorrelated */,
                              nullptr /* predicate */,
                              nodeId);
-
     return {matchedRecordsSlot, std::move(nlj)};
 }
 
@@ -615,7 +703,6 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
     const FieldPath& foreignFieldName,
     const CollectionPtr& foreignColl,
     const IndexEntry& index,
-    StringMap<const IndexAccessMethod*>& iamMap,
     PlanYieldPolicySBE* yieldPolicy,
     boost::optional<SlotId> collatorSlot,
     const PlanNodeId nodeId,
@@ -636,7 +723,6 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
         foreignColl->getIndexCatalog()->getEntry(indexDescriptor)->accessMethod()->asSortedData();
     const auto indexVersion = indexAccessMethod->getSortedDataInterface()->getKeyStringVersion();
     const auto indexOrdering = indexAccessMethod->getSortedDataInterface()->getOrdering();
-    iamMap.insert({indexName, indexAccessMethod});
 
     // Build the outer branch that produces the correlated local key slot.
     auto [localKeysSetSlot, localKeysSetStage] = buildKeySet(JoinSide::Local,
@@ -707,7 +793,7 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
         makeProjectStage(makeLimitCoScanTree(nodeId, 1),
                          nodeId,
                          arrayBranchOutput,
-                         makeFunction("fillEmpty",
+                         makeBinaryOp(sbe::EPrimBinary::fillEmpty,
                                       makeFunction("getElement",
                                                    makeVariable(singleLocalValueSlot),
                                                    makeConstant(TypeTags::NumberInt32, 0)),
@@ -735,24 +821,27 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
         makeSV(valueForIndexBounds),
         nodeId);
 
-    // For hashed indexes, we need to hash value before computing keystrings.
     if (index.type == INDEX_HASHED) {
-        auto rawValueSlot = valueForIndexBounds;
-        valueForIndexBounds = slotIdGenerator.generate();
-        valueGeneratorStage =
-            makeProjectStage(std::move(valueGeneratorStage),
-                             nodeId,
-                             valueForIndexBounds,
-                             makeFunction("shardHash", makeVariable(rawValueSlot)));
+        // For hashed indexes, we need to hash the value before computing keystrings iff the
+        // lookup's "foreignField" is the hashed field in this index.
+        const BSONElement elt = index.keyPattern.getField(foreignFieldName.fullPath());
+        if (elt.valueStringDataSafe() == IndexNames::HASHED) {
+            auto rawValueSlot = valueForIndexBounds;
+            valueForIndexBounds = slotIdGenerator.generate();
+            valueGeneratorStage =
+                makeProjectStage(std::move(valueGeneratorStage),
+                                 nodeId,
+                                 valueForIndexBounds,
+                                 makeFunction("shardHash", makeVariable(rawValueSlot)));
+        }
     }
 
     // Calculate the low key and high key of each individual local field. They are stored in
     // 'lowKeySlot' and 'highKeySlot', respectively. These two slots will be made available in
-    // the loop join stage to perform index seek. We also set 'indexIdSlot' and
-    // 'indexKeyPatternSlot' constants for the seek stage later to perform consistency check.
+    // the loop join stage to perform index seek. We also set the 'indexKeyPatternSlot' constant
+    // for the seek stage later to perform consistency check.
     auto lowKeySlot = slotIdGenerator.generate();
     auto highKeySlot = slotIdGenerator.generate();
-    auto indexIdSlot = slotIdGenerator.generate();
     auto indexKeyPatternSlot = slotIdGenerator.generate();
     auto [_, indexKeyPatternValue] =
         copyValue(TypeTags::bsonObject, bitcastFrom<const char*>(index.keyPattern.objdata()));
@@ -779,8 +868,6 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
                          makeNewKeyStringCall(KeyString::Discriminator::kExclusiveBefore),
                          highKeySlot,
                          makeNewKeyStringCall(KeyString::Discriminator::kExclusiveAfter),
-                         indexIdSlot,
-                         makeConstant(indexName),
                          indexKeyPatternSlot,
                          makeConstant(value::TypeTags::bsonObject, indexKeyPatternValue));
 
@@ -801,25 +888,27 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
     auto foreignRecordIdSlot = slotIdGenerator.generate();
     auto indexKeySlot = slotIdGenerator.generate();
     auto snapshotIdSlot = slotIdGenerator.generate();
-    auto ixScanStage = makeS<IndexScanStage>(foreignCollUUID,
-                                             indexName,
-                                             true /* forward */,
-                                             indexKeySlot,
-                                             foreignRecordIdSlot,
-                                             snapshotIdSlot,
-                                             IndexKeysInclusionSet{} /* indexKeysToInclude */,
-                                             makeSV() /* vars */,
-                                             makeVariable(lowKeySlot),
-                                             makeVariable(highKeySlot),
-                                             yieldPolicy,
-                                             nodeId);
+    auto indexIdentSlot = slotIdGenerator.generate();
+    auto ixScanStage = makeS<SimpleIndexScanStage>(foreignCollUUID,
+                                                   indexName,
+                                                   true /* forward */,
+                                                   indexKeySlot,
+                                                   foreignRecordIdSlot,
+                                                   snapshotIdSlot,
+                                                   indexIdentSlot,
+                                                   IndexKeysInclusionSet{} /* indexKeysToInclude */,
+                                                   makeSV() /* vars */,
+                                                   makeVariable(lowKeySlot),
+                                                   makeVariable(highKeySlot),
+                                                   yieldPolicy,
+                                                   nodeId);
 
     // Loop join the low key and high key generation with the index seek stage to produce the
     // foreign record id to seek.
     auto ixScanNljStage =
         makeS<LoopJoinStage>(std::move(indexBoundKeyStage),
                              std::move(ixScanStage),
-                             makeSV(indexIdSlot, indexKeyPatternSlot) /* outerProjects */,
+                             makeSV(indexKeyPatternSlot) /* outerProjects */,
                              makeSV(lowKeySlot, highKeySlot) /* outerCorrelated */,
                              nullptr /* predicate */,
                              nodeId);
@@ -838,20 +927,22 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
 
     // Loop join the foreign record id produced by the index seek on the outer side with seek
     // stage on the inner side to get matched foreign documents. The foreign documents are
-    // stored in 'foreignRecordSlot'. We also pass in 'snapshotIdSlot', 'indexIdSlot',
-    // 'indexKeySlot' and 'indexKeyPatternSlot' to perform index consistency check during the
-    // seek.
-    auto [foreignRecordSlot, __, scanNljStage] = makeLoopJoinForFetch(std::move(ixScanNljStage),
-                                                                      foreignRecordIdSlot,
-                                                                      snapshotIdSlot,
-                                                                      indexIdSlot,
-                                                                      indexKeySlot,
-                                                                      indexKeyPatternSlot,
-                                                                      foreignColl,
-                                                                      iamMap,
-                                                                      nodeId,
-                                                                      makeSV() /* slotsToForward */,
-                                                                      slotIdGenerator);
+    // stored in 'foreignRecordSlot'. We also pass in 'snapshotIdSlot', 'indexIdentSlot',
+    // 'indexKeySlot' and 'indexKeyPatternSlot' to perform index consistency check during the seek.
+    auto foreignRecordSlot = slotIdGenerator.generate();
+    auto scanNljStage = makeLoopJoinForFetch(std::move(ixScanNljStage),
+                                             foreignRecordSlot,
+                                             slotIdGenerator.generate() /* unused recordId slot */,
+                                             std::vector<std::string>{},
+                                             makeSV(),
+                                             foreignRecordIdSlot,
+                                             snapshotIdSlot,
+                                             indexIdentSlot,
+                                             indexKeySlot,
+                                             indexKeyPatternSlot,
+                                             foreignColl,
+                                             nodeId,
+                                             makeSV() /* slotsToForward */);
 
     // 'buildForeignMatches()' filters the foreign records, returned by the index scan, to match
     // those in 'localKeysSetSlot'. This is necessary because some values are encoded with the same
@@ -864,6 +955,7 @@ std::pair<SlotId, std::unique_ptr<sbe::PlanStage>> buildIndexJoinLookupStage(
                                                                      collatorSlot,
                                                                      nodeId,
                                                                      slotIdGenerator,
+                                                                     frameIdGenerator,
                                                                      state.allowDiskUse);
 
     // 'foreignGroupStage' should not participate in trial run tracking as the number of reads
@@ -933,8 +1025,10 @@ std::pair<SlotId /*matched docs*/, std::unique_ptr<sbe::PlanStage>> buildHashJoi
     // Add a projection that makes so that empty array is returned if no foreign row were matched.
     auto innerResultSlot = slotIdGenerator.generate();
     auto [emptyArrayTag, emptyArrayVal] = makeNewArray();
-    std::unique_ptr<EExpression> innerResultProjection = makeFunction(
-        "fillEmpty"_sd, makeVariable(lookupAggSlot), makeConstant(emptyArrayTag, emptyArrayVal));
+    std::unique_ptr<EExpression> innerResultProjection =
+        makeBinaryOp(sbe::EPrimBinary::fillEmpty,
+                     makeVariable(lookupAggSlot),
+                     makeConstant(emptyArrayTag, emptyArrayVal));
 
     std::unique_ptr<sbe::PlanStage> resultStage =
         makeProjectStage(std::move(hl), nodeId, innerResultSlot, std::move(innerResultProjection));
@@ -953,7 +1047,8 @@ std::pair<SlotId /*matched docs*/, std::unique_ptr<sbe::PlanStage>> buildLookupS
     const FieldPath& foreignFieldName,
     boost::optional<SlotId> collatorSlot,
     const PlanNodeId nodeId,
-    SlotIdGenerator& slotIdGenerator) {
+    SlotIdGenerator& slotIdGenerator,
+    FrameIdGenerator& frameIdGenerator) {
     switch (lookupStrategy) {
         case EqLookupNode::LookupStrategy::kNestedLoopJoin:
             return buildNljLookupStage(state,
@@ -962,10 +1057,11 @@ std::pair<SlotId /*matched docs*/, std::unique_ptr<sbe::PlanStage>> buildLookupS
                                        localFieldName,
                                        std::move(foreignStage),
                                        foreignRecordSlot,
-                                       foreignFieldName.fullPath(),
+                                       foreignFieldName,
                                        collatorSlot,
                                        nodeId,
-                                       slotIdGenerator);
+                                       slotIdGenerator,
+                                       frameIdGenerator);
         case EqLookupNode::LookupStrategy::kHashJoin:
             return buildHashJoinLookupStage(state,
                                             std::move(localStage),
@@ -1063,9 +1159,6 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> SlotBasedStageBuilder
         _state.data->foreignHashJoinCollections.emplace(eqLookupNode->foreignCollection);
     }
 
-    // $lookup creates its own output documents.
-    _shouldProduceRecordIdSlot = false;
-
     auto localReqs = reqs.copy().set(kResult);
     auto [localStage, localOutputs] = build(eqLookupNode->children[0].get(), localReqs);
     SlotId localDocumentSlot = localOutputs.get(PlanStageSlots::kResult);
@@ -1099,7 +1192,6 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> SlotBasedStageBuilder
                                                  eqLookupNode->joinFieldForeign,
                                                  foreignColl,
                                                  *eqLookupNode->idxEntry,
-                                                 _data.iamMap,
                                                  _yieldPolicy,
                                                  collatorSlot,
                                                  eqLookupNode->nodeId(),
@@ -1119,7 +1211,7 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> SlotBasedStageBuilder
                                                      foreignResultSlot,
                                                      foreignRecordIdSlot,
                                                      boost::none /* snapshotIdSlot */,
-                                                     boost::none /* indexIdSlot */,
+                                                     boost::none /* indexIdentSlot */,
                                                      boost::none /* indexKeySlot */,
                                                      boost::none /* indexKeyPatternSlot */,
                                                      boost::none /* tsSlot */,
@@ -1141,7 +1233,8 @@ std::pair<std::unique_ptr<sbe::PlanStage>, PlanStageSlots> SlotBasedStageBuilder
                                         eqLookupNode->joinFieldForeign,
                                         collatorSlot,
                                         eqLookupNode->nodeId(),
-                                        _slotIdGenerator);
+                                        _slotIdGenerator,
+                                        _frameIdGenerator);
             }
             default:
                 MONGO_UNREACHABLE_TASSERT(5842605);
