@@ -66,8 +66,8 @@ boost::optional<RWConcernDefault> readWriteConcernDefaultsCacheLookupMongoD(
                                    getPersistedDefaultRWConcernDocument(opCtx));
 }
 
-void readWriteConcernDefaultsMongodStartupChecks(OperationContext* opCtx) {
-    if (serverGlobalParams.clusterRole == ClusterRole::ShardServer) {
+void readWriteConcernDefaultsMongodStartupChecks(OperationContext* opCtx, bool isReplicaSet) {
+    if (serverGlobalParams.clusterRole.exclusivelyHasShardRole()) {
         DBDirectClient client(opCtx);
         const auto numPersistedDocuments =
             client.count(NamespaceString::kConfigSettingsNamespace,
@@ -86,6 +86,30 @@ void readWriteConcernDefaultsMongodStartupChecks(OperationContext* opCtx) {
                 "configSettingsNamespace"_attr = NamespaceString::kConfigSettingsNamespace);
         }
     }
+
+    // We used to allow setting CWWC with only 'wtimeout,' and it defaulted the 'w' field to 1.
+    // Therefore, we issue a warning on startup if a node has CWWC with 'w' field set to 1 and
+    // 'wtimeout' is greater than 0.
+    // Can't retrieve CWWC if 'failRWCDefaultsLookup' is enabled.
+    if (!MONGO_unlikely(failRWCDefaultsLookup.shouldFail()) &&
+        (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer) ||
+         serverGlobalParams.clusterRole.has(ClusterRole::None)) &&
+        isReplicaSet && ReadWriteConcernDefaults::get(opCtx).isCWWCSet(opCtx)) {
+        auto wcDefault =
+            ReadWriteConcernDefaults::get(opCtx).getDefault(opCtx).getDefaultWriteConcern();
+        if (wcDefault && stdx::holds_alternative<int64_t>(wcDefault->w) &&
+            stdx::get<int64_t>(wcDefault->w) == 1 &&
+            wcDefault->wTimeout != WriteConcernOptions::kNoTimeout) {
+            LOGV2_WARNING_OPTIONS(
+                8036300,
+                {logv2::LogTag::kStartupWarnings},
+                "The default cluster-wide write concern is configured with the 'w' field set to 1 "
+                "and 'wtimeout' set to a value greater than 0, although it's important to note "
+                "that 'wtimeout' is only applicable for 'w' values greater than 1."
+                "Please verify if this is intentional; if not, consider making the appropriate "
+                "adjustments.");
+        }
+    }
 }
 
 }  // namespace mongo
