@@ -42,6 +42,7 @@
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
 #include "mongo/unittest/death_test.h"
+#include "mongo/unittest/log_test.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/fail_point.h"
@@ -79,7 +80,7 @@ public:
     }
 
     NamespaceString getStateDocumentsNS() const override {
-        return NamespaceString("admin", "test_service");
+        return NamespaceString::createNamespaceString_forTest("admin", "test_service");
     }
 
     ThreadPool::Limits getThreadPoolLimits() const override {
@@ -258,9 +259,14 @@ public:
 
                 DBDirectClient client(opCtx.get());
                 if (targetState == State::kDone) {
-                    client.remove("admin.test_service", _id);
+                    client.remove(
+                        NamespaceString::createNamespaceString_forTest("admin.test_service"), _id);
                 } else {
-                    client.update("admin.test_service", _id, newStateDoc, true /*upsert*/);
+                    client.update(
+                        NamespaceString::createNamespaceString_forTest("admin.test_service"),
+                        _id,
+                        newStateDoc,
+                        true /*upsert*/);
                 }
             } catch (const DBException& e) {
                 _documentWriteException.setError(e.toStatus());
@@ -289,7 +295,7 @@ private:
         auto opCtx = opCtxHolder.get();
         DBDirectClient client(opCtx);
         BSONObj result;
-        client.runCommand(nss.db().toString(),
+        client.runCommand(nss.dbName(),
                           BSON("createIndexes"
                                << nss.coll().toString() << "indexes"
                                << BSON_ARRAY(BSON("key" << BSON("x" << 1) << "name"
@@ -318,8 +324,8 @@ public:
 
         _service = _registry->lookupServiceByName("TestService");
         ASSERT(_service);
-        auto serviceByNamespace =
-            _registry->lookupServiceByNamespace(NamespaceString("admin.test_service"));
+        auto serviceByNamespace = _registry->lookupServiceByNamespace(
+            NamespaceString::createNamespaceString_forTest("admin.test_service"));
         ASSERT_EQ(_service, serviceByNamespace);
 
         _testExecutor = makeTestExecutor();
@@ -521,16 +527,21 @@ TEST_F(PrimaryOnlyServiceTest, LookupInstance) {
     ASSERT(instance.get());
     ASSERT_EQ(0, instance->getID());
 
-    auto instance2 = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
+    auto [instance2, isPausedOrShutdown2] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    ASSERT_TRUE(instance2);
+    ASSERT_FALSE(isPausedOrShutdown2);
 
-    ASSERT_EQ(instance.get(), instance2.get());
+    ASSERT_EQ(instance.get(), instance2.value().get());
 
     TestServiceHangDuringInitialization.setMode(FailPoint::off);
     instance->getCompletionFuture().get();
 
     // Shouldn't be able to look up instance after it has completed running.
-    auto instance3 = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
-    ASSERT_FALSE(instance3.has_value());
+    auto [instance3, isPausedOrShutdown3] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    ASSERT_FALSE(instance3);
+    ASSERT_FALSE(isPausedOrShutdown3);
 }
 
 TEST_F(PrimaryOnlyServiceTest, LookupInstanceInterruptible) {
@@ -565,9 +576,11 @@ TEST_F(PrimaryOnlyServiceTest, LookupInstanceHoldingISLock) {
         opCtx->setAlwaysInterruptAtStepDownOrUp_UNSAFE();
         ASSERT_FALSE(opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
 
-        auto instance2 =
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-        ASSERT_EQ(instance.get(), instance2.get());
+        auto [instance2, isPausedOrShutdown] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_TRUE(instance2);
+        ASSERT_FALSE(isPausedOrShutdown);
+        ASSERT_EQ(instance.get(), instance2.value().get());
     }
 
     TestServiceHangDuringInitialization.setMode(FailPoint::off);
@@ -587,9 +600,11 @@ TEST_F(PrimaryOnlyServiceTest, LookupInstanceHoldingIXLock) {
     {
         Lock::GlobalLock lk(opCtx.get(), MODE_IX);
         ASSERT_FALSE(opCtx->shouldAlwaysInterruptAtStepDownOrUp());
-        auto instance2 =
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-        ASSERT_EQ(instance.get(), instance2.get());
+        auto [instance2, isPausedOrShutdown] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_TRUE(instance2);
+        ASSERT_FALSE(isPausedOrShutdown);
+        ASSERT_EQ(instance.get(), instance2.value().get());
     }
 
     TestServiceHangDuringInitialization.setMode(FailPoint::off);
@@ -632,9 +647,11 @@ TEST_F(PrimaryOnlyServiceTest, LookupInstanceAfterStepDownReturnsNone) {
 
     stepDown();
 
-    auto instance2 = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    auto [instance2, isPausedOrShutdown] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
 
-    ASSERT_EQ(instance2, boost::none);
+    ASSERT_FALSE(instance2);
+    ASSERT_TRUE(isPausedOrShutdown);
 
     TestServiceHangDuringInitialization.setMode(FailPoint::off);
     ASSERT_EQ(ErrorCodes::Interrupted, instance->getCompletionFuture().getNoThrow());
@@ -655,9 +672,11 @@ TEST_F(PrimaryOnlyServiceTest, LookupInstanceAfterShutDownReturnsNone) {
 
     shutdown();
 
-    auto instance2 = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    auto [instance2, isPausedOrShutdown] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
 
-    ASSERT_EQ(instance2, boost::none);
+    ASSERT_FALSE(instance2);
+    ASSERT_TRUE(isPausedOrShutdown);
 
     ASSERT_EQ(ErrorCodes::Interrupted, instance->getCompletionFuture().getNoThrow());
 }
@@ -839,8 +858,10 @@ TEST_F(PrimaryOnlyServiceTest, StepDownBeforePersisted) {
 
     auto opCtx = makeOperationContext();
     // Since the Instance never wrote its state document, it shouldn't be recreated on stepUp.
-    auto recreatedInstance = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
-    ASSERT(!recreatedInstance.has_value());
+    auto [recreatedInstance, isPausedOrShutdown] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    ASSERT_FALSE(recreatedInstance);
+    ASSERT_FALSE(isPausedOrShutdown);
 }
 
 TEST_F(PrimaryOnlyServiceTest, RecreateInstanceOnStepUp) {
@@ -866,11 +887,13 @@ TEST_F(PrimaryOnlyServiceTest, RecreateInstanceOnStepUp) {
 
     {
         auto opCtx = makeOperationContext();
-        auto recreatedInstance =
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-        ASSERT_EQ(TestService::State::kOne, recreatedInstance->getInitialState());
+        auto [recreatedInstance, isPausedOrShutdown] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_TRUE(recreatedInstance);
+        ASSERT_FALSE(isPausedOrShutdown);
+        ASSERT_EQ(TestService::State::kOne, (*recreatedInstance)->getInitialState());
         TestServiceHangDuringStateTwo.waitForTimesEntered(++stateTwoFPTimesEntered);
-        ASSERT_EQ(TestService::State::kTwo, recreatedInstance->getState());
+        ASSERT_EQ(TestService::State::kTwo, (*recreatedInstance)->getState());
     }
 
     stepDown();
@@ -884,17 +907,19 @@ TEST_F(PrimaryOnlyServiceTest, RecreateInstanceOnStepUp) {
 
     {
         auto opCtx = makeOperationContext();
-        auto recreatedInstance =
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-        ASSERT_EQ(TestService::State::kTwo, recreatedInstance->getInitialState());
-        TestServiceHangDuringStateOne.setMode(FailPoint::off);
-        recreatedInstance->getCompletionFuture().get();
-        ASSERT_EQ(TestService::State::kDone, recreatedInstance->getState());
-
-
-        auto nonExistentInstance =
+        auto [recreatedInstance, isPausedOrShutdown1] =
             TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
-        ASSERT(!nonExistentInstance.has_value());
+        ASSERT_TRUE(recreatedInstance);
+        ASSERT_FALSE(isPausedOrShutdown1);
+        ASSERT_EQ(TestService::State::kTwo, (*recreatedInstance)->getInitialState());
+        TestServiceHangDuringStateOne.setMode(FailPoint::off);
+        (*recreatedInstance)->getCompletionFuture().get();
+        ASSERT_EQ(TestService::State::kDone, (*recreatedInstance)->getState());
+
+        auto [nonExistentInstance, isPausedOrShutdown2] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_FALSE(nonExistentInstance);
+        ASSERT_FALSE(isPausedOrShutdown2);
     }
 
     stepDown();
@@ -904,9 +929,10 @@ TEST_F(PrimaryOnlyServiceTest, RecreateInstanceOnStepUp) {
         auto opCtx = makeOperationContext();
         // No Instance should be recreated since the previous run completed successfully and deleted
         // its state document.
-        auto nonExistentInstance =
+        auto [nonExistentInstance, isPausedOrShutdown] =
             TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
-        ASSERT(!nonExistentInstance.has_value());
+        ASSERT_FALSE(nonExistentInstance);
+        ASSERT_FALSE(isPausedOrShutdown);
     }
 }
 
@@ -962,13 +988,16 @@ TEST_F(PrimaryOnlyServiceTest, StepDownBeforeRebuildingInstances) {
     TestServiceHangDuringStateOne.waitForTimesEntered(++stateOneFPTimesEntered);
 
     auto opCtx = makeOperationContext();
-    auto instance = TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-    ASSERT_EQ(TestService::State::kOne, instance->getInitialState());
-    ASSERT_EQ(TestService::State::kOne, instance->getState());
+    auto [instance, isPausedOrShutdown] =
+        TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+    ASSERT_TRUE(instance);
+    ASSERT_FALSE(isPausedOrShutdown);
+    ASSERT_EQ(TestService::State::kOne, (*instance)->getInitialState());
+    ASSERT_EQ(TestService::State::kOne, (*instance)->getState());
 
     TestServiceHangDuringStateOne.setMode(FailPoint::off);
 
-    instance->getCompletionFuture().get();
+    (*instance)->getCompletionFuture().get();
 }
 
 TEST_F(PrimaryOnlyServiceTest, RecreateInstancesFails) {
@@ -1012,8 +1041,10 @@ TEST_F(PrimaryOnlyServiceTest, RecreateInstancesFails) {
         // After stepping down we are in a consistent state again, but cannot create or lookup
         // instances because we are not primary.
         auto opCtx = makeOperationContext();
-        ASSERT_FALSE(
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).has_value());
+        auto [instance, isPausedOrShutdown] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_FALSE(instance);
+        ASSERT_TRUE(isPausedOrShutdown);
         ASSERT_THROWS_CODE(TestService::Instance::getOrCreate(
                                opCtx.get(), _service, BSON("_id" << 0 << "state" << 0)),
                            DBException,
@@ -1030,13 +1061,15 @@ TEST_F(PrimaryOnlyServiceTest, RecreateInstancesFails) {
     {
         // Instance should be recreated successfully.
         auto opCtx = makeOperationContext();
-        auto instance =
-            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0)).value();
-        ASSERT_EQ(TestService::State::kOne, instance->getInitialState());
-        ASSERT_EQ(TestService::State::kOne, instance->getState());
+        auto [instance, isPausedOrShutdown] =
+            TestService::Instance::lookup(opCtx.get(), _service, BSON("_id" << 0));
+        ASSERT_TRUE(instance);
+        ASSERT_FALSE(isPausedOrShutdown);
+        ASSERT_EQ(TestService::State::kOne, (*instance)->getInitialState());
+        ASSERT_EQ(TestService::State::kOne, (*instance)->getState());
         TestServiceHangDuringStateOne.setMode(FailPoint::off);
-        instance->getCompletionFuture().get();
-        ASSERT_EQ(TestService::State::kDone, instance->getState());
+        (*instance)->getCompletionFuture().get();
+        ASSERT_EQ(TestService::State::kDone, (*instance)->getState());
     }
 }
 
@@ -1147,3 +1180,66 @@ TEST_F(PrimaryOnlyServiceTest, StateTransitionFromRebuildingShouldWakeUpConditio
     stepUpThread.join();
     lookUpInstanceThread.join();
 }
+
+TEST_F(PrimaryOnlyServiceTest, PrimaryOnlyServiceLogSlowServices) {
+    primaryOnlyServiceTestStepUpWaitForRebuildComplete.setMode(FailPoint::alwaysOn);
+    // We change slow stepUp threshold to a extremely large value to avoid test failure in some slow
+    // test environments.
+    const auto oldSlowServiceOnStepUpCompleteThresholdMS =
+        repl::slowServiceOnStepUpCompleteThresholdMS.load();
+    const auto oldSlowTotalOnStepUpCompleteThresholdMS =
+        repl::slowTotalOnStepUpCompleteThresholdMS.load();
+    repl::slowServiceOnStepUpCompleteThresholdMS.store(1000000);
+    repl::slowTotalOnStepUpCompleteThresholdMS.store(1000000);
+    std::string slowSingleServiceStepUpCompleteMsg =
+        "Duration spent in PrimaryOnlyServiceRegistry::onStepUpComplete for service "
+        "exceeded slowServiceOnStepUpCompleteThresholdMS";
+    std::string slowTotalTimeStepUpCompleteMsg =
+        "Duration spent in PrimaryOnlyServiceRegistry::onStepUpComplete for all services "
+        "exceeded slowTotalOnStepUpCompleteThresholdMS";
+
+    stepDown();
+
+    // First test a stepUp with no hanging. We shouldn't log anything.
+    startCapturingLogMessages();
+    stepUp();
+    stopCapturingLogMessages();
+    ASSERT_EQ(0, countTextFormatLogLinesContaining(slowSingleServiceStepUpCompleteMsg));
+    ASSERT_EQ(0, countTextFormatLogLinesContaining(slowTotalTimeStepUpCompleteMsg));
+
+    // Reset thresholds.
+    repl::slowServiceOnStepUpCompleteThresholdMS.store(oldSlowServiceOnStepUpCompleteThresholdMS);
+    repl::slowTotalOnStepUpCompleteThresholdMS.store(oldSlowTotalOnStepUpCompleteThresholdMS);
+
+    // Use the "PrimaryOnlyServiceHangBeforeLaunchingStepUpLogic", which is encountered
+    // via PrimaryOnlyServiceRegistry::onStepUpComplete, to simulate a slow primary-only
+    // service.
+    auto doStepUpWithHang = [this](long long ms) {
+        stepDown();
+
+        auto stepUpTimesEntered =
+            PrimaryOnlyServiceHangBeforeLaunchingStepUpLogic.setMode(FailPoint::alwaysOn);
+
+        // Start an async task to step up, which will block on the fail point.
+        auto stepUpFuture = ExecutorFuture<void>(_testExecutor).then([this]() { stepUp(); });
+
+        // Once stepUp has hit the failpoint, sleep for the specified amount of ms so the service
+        // hangs, before disabling the failpoint and allowing stepUp to proceed.
+        PrimaryOnlyServiceHangBeforeLaunchingStepUpLogic.waitForTimesEntered(++stepUpTimesEntered);
+        sleepmillis(ms);
+        PrimaryOnlyServiceHangBeforeLaunchingStepUpLogic.setMode(FailPoint::off);
+        stepUpFuture.wait();
+    };
+
+    // Hang for long enough to generate a message about the single service being slow.
+    startCapturingLogMessages();
+    doStepUpWithHang(repl::slowServiceOnStepUpCompleteThresholdMS.load() + 1);
+    stopCapturingLogMessages();
+    ASSERT_EQ(1, countTextFormatLogLinesContaining(slowSingleServiceStepUpCompleteMsg));
+
+    // Hang for long enough to generate a message about the total time being slow.
+    startCapturingLogMessages();
+    doStepUpWithHang(repl::slowTotalOnStepUpCompleteThresholdMS.load() + 1);
+    stopCapturingLogMessages();
+    ASSERT_EQ(1, countTextFormatLogLinesContaining(slowTotalTimeStepUpCompleteMsg));
+}
