@@ -36,6 +36,7 @@
 #include "mongo/base/status_with.h"
 #include "mongo/client/connection_string.h"
 #include "mongo/db/repl/member_config.h"
+#include "mongo/db/repl/repl_set_config_gen.h"
 #include "mongo/db/repl/repl_set_tag.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/util/string_map.h"
@@ -107,16 +108,57 @@ private:
     long long _term;
 };
 
+class ReplSetConfig;
+using ReplSetConfigPtr = std::shared_ptr<ReplSetConfig>;
+
+/**
+ * This class is used for mutating the ReplicaSetConfig.  Call ReplSetConfig::getMutable()
+ * to get a mutable copy, mutate it, and use the ReplSetConfig(MutableReplSetConfig&&) constructor
+ * to get a usable immutable config from it.
+ */
+class MutableReplSetConfig : public ReplSetConfigBase {
+public:
+    ReplSetConfigSettings& getMutableSettings() {
+        invariant(ReplSetConfigBase::getSettings());
+        // TODO(SERVER-47937): Get rid of the const_cast when the IDL supports that.
+        return const_cast<ReplSetConfigSettings&>(*ReplSetConfigBase::getSettings());
+    }
+
+    /**
+     * Adds 'newlyAdded=true' to the MemberConfig of the specified member.
+     */
+    void addNewlyAddedFieldForMember(MemberId memberId);
+
+    /**
+     * Removes the 'newlyAdded' field from the MemberConfig of the specified member.
+     */
+    void removeNewlyAddedFieldForMember(MemberId memberId);
+
+    /**
+     * Sets the member config's 'secondaryDelaySecs' field to the default value of 0.
+     */
+    void setSecondaryDelaySecsFieldDefault(MemberId memberId);
+
+protected:
+    MutableReplSetConfig() = default;
+
+    /**
+     * Returns a pointer to a mutable MemberConfig.
+     */
+    MemberConfig* _findMemberByID(MemberId id);
+
+    ReplSetConfigPtr _recipientConfig;
+};
+
 /**
  * Representation of the configuration information about a particular replica set.
  */
-class ReplSetConfig {
+class ReplSetConfig : private MutableReplSetConfig {
 public:
     typedef std::vector<MemberConfig>::const_iterator MemberIterator;
 
-    static const std::string kConfigServerFieldName;
-    static const std::string kVersionFieldName;
-    static const std::string kTermFieldName;
+    using ReplSetConfigBase::kConfigServerFieldName;
+    using ReplSetConfigBase::kConfigTermFieldName;
     static constexpr char kMajorityWriteConcernModeName[] = "$majority";
     static constexpr char kVotingMembersWriteConcernModeName[] = "$votingMembers";
     static constexpr char kConfigMajorityWriteConcernModeName[] = "$configMajority";
@@ -124,14 +166,14 @@ public:
 
     // If this field is present, a repair operation potentially modified replicated data. This
     // should never be included in a valid configuration document.
-    static const std::string kRepairedFieldName;
+    using ReplSetConfigBase::kRepairedFieldName;
 
     /**
-     * Inline `kMaxMembers` to allow others (e.g, `WriteConcernOptions`) use
+     * Inline `kMaxMembers` and `kMaxVotingMembers` to allow others (e.g, `WriteConcernOptions`) use
      * the constant without linking to `repl_set_config.cpp`.
      */
     inline static const size_t kMaxMembers = 50;
-    static const size_t kMaxVotingMembers = 7;
+    inline static const size_t kMaxVotingMembers = 7;
     static const Milliseconds kInfiniteCatchUpTimeout;
     static const Milliseconds kCatchUpDisabled;
     static const Milliseconds kCatchUpTakeoverDisabled;
@@ -143,22 +185,51 @@ public:
     static const bool kDefaultChainingAllowed;
     static const Milliseconds kDefaultCatchUpTakeoverDelay;
 
-    /**
-     * Initializes this ReplSetConfig from the contents of "cfg".
-     * Sets _replicaSetId to "defaultReplicaSetId" if a replica set ID is not specified in "cfg".
-     * If forceTerm is not boost::none, sets _term to the given term. Otherwise, parses term from
-     * config BSON.
-     */
-    Status initialize(const BSONObj& cfg,
-                      boost::optional<long long> forceTerm = boost::none,
-                      OID defaultReplicaSetId = OID());
+    // Methods inherited from the base IDL class.  Do not include any setters here.
+    using ReplSetConfigBase::getConfigServer;
+    using ReplSetConfigBase::getConfigTerm;
+    using ReplSetConfigBase::getConfigVersion;
+    using ReplSetConfigBase::getProtocolVersion;
+    using ReplSetConfigBase::getReplSetName;
+    using ReplSetConfigBase::getWriteConcernMajorityShouldJournal;
+    using ReplSetConfigBase::serialize;
 
     /**
-     * Same as the generic initialize() above except will default "configsvr" setting to the value
-     * of serverGlobalParams.configsvr.
-     * Sets _term to kInitialTerm.
+     * Constructor used for converting a mutable config to an immutable one.
      */
-    Status initializeForInitiate(const BSONObj& cfg);
+    explicit ReplSetConfig(MutableReplSetConfig&& base);
+
+    ReplSetConfig() {
+        // This is not defaultable in the IDL.
+        // SERVER-47938 would make it possible to be defaulted.
+
+        setSettings(ReplSetConfigSettings());
+        _setRequiredFields();
+    }
+    /**
+     * Initializes a new ReplSetConfig from the contents of "cfg".
+     * Sets replicaSetId to "defaultReplicaSetId" if a replica set ID is not specified in "cfg";
+     * If forceTerm is not boost::none, sets _term to the given term. Otherwise, uses term from
+     * config BSON.
+     *
+     * Parse errors are reported via exceptions.
+     */
+    static ReplSetConfig parse(const BSONObj& cfg,
+                               boost::optional<long long> forceTerm = boost::none,
+                               OID defaultReplicaSetId = OID());
+
+    /**
+     * Same as the generic parse() above except will default "configsvr" setting to the value
+     * of serverGlobalParams.configsvr.
+     * Sets term to kInitialTerm.
+     * Sets replicaSetId to "newReplicaSetId", which must be set.
+     */
+    static ReplSetConfig parseForInitiate(const BSONObj& cfg, OID newReplicaSetId);
+
+    /**
+     * Override ReplSetConfigBase::toBSON to conditionally include the recipient config.
+     */
+    BSONObj toBSON() const;
 
     /**
      * Returns true if this object has been successfully initialized or copied from
@@ -173,6 +244,12 @@ public:
      */
     Status validate() const;
 
+    /**
+     * Performs basic consistency checks on the replica set configuration, but does not fail on
+     * IP addresses in split horizon configuration
+     */
+    Status validateAllowingSplitHorizonIP() const;
+
     /**
      * Checks if this configuration can satisfy the given write concern.
      *
@@ -183,46 +260,11 @@ public:
      */
     Status checkIfWriteConcernCanBeSatisfied(const WriteConcernOptions& writeConcern) const;
 
-    /**
-     * Gets and sets the version of this configuration.
-     *
-     * The version number sequences configurations of the replica set, so that
-     * nodes may distinguish between "older" and "newer" configurations.
-     */
-    long long getConfigVersion() const {
-        return _version;
-    }
-
-    void setConfigVersion(long long version) {
-        _version = version;
-    }
-
-    /**
-     * Gets and sets the term of this configuration.
-     *
-     * The configuration term is the term of the primary that originally created this configuration.
-     * Configurations in a replica set are totally ordered by their term and configuration version.
-     */
-    long long getConfigTerm() const {
-        return _term;
-    }
-
-    void setConfigTerm(long long term) {
-        _term = term;
-    }
-
     /**
      * Gets the (version, term) pair of this configuration.
      */
     ConfigVersionAndTerm getConfigVersionAndTerm() const {
-        return ConfigVersionAndTerm(_version, _term);
-    }
-
-    /**
-     * Gets the name (_id field value) of the replica set described by this configuration.
-     */
-    const std::string& getReplSetName() const {
-        return _replSetName;
+        return ConfigVersionAndTerm(getConfigVersion(), getConfigTerm());
     }
 
     /**
@@ -236,7 +278,7 @@ public:
      * Gets the number of members in this configuration.
      */
     int getNumMembers() const {
-        return _members.size();
+        return getMembers().size();
     }
 
     /**
@@ -248,18 +290,18 @@ public:
      * Gets a begin iterator over the MemberConfigs stored in this ReplSetConfig.
      */
     MemberIterator membersBegin() const {
-        return _members.begin();
+        return getMembers().begin();
     }
 
     /**
      * Gets an end iterator over the MemberConfigs stored in this ReplSetConfig.
      */
     MemberIterator membersEnd() const {
-        return _members.end();
+        return getMembers().end();
     }
 
     const std::vector<MemberConfig>& members() const {
-        return _members;
+        return getMembers();
     }
 
     /**
@@ -267,7 +309,7 @@ public:
      */
     std::vector<MemberConfig> votingMembers() const {
         std::vector<MemberConfig> votingMembers;
-        for (const MemberConfig& m : _members) {
+        for (const MemberConfig& m : getMembers()) {
             if (m.getNumVotes() > 0) {
                 votingMembers.push_back(m);
             }
@@ -308,7 +350,7 @@ public:
      * Gets the default write concern for the replica set described by this configuration.
      */
     const WriteConcernOptions& getDefaultWriteConcern() const {
-        return _defaultWriteConcern;
+        return getSettings()->getDefaultWriteConcern();
     }
 
     /**
@@ -323,7 +365,7 @@ public:
      * run for election.
      */
     Milliseconds getElectionTimeoutPeriod() const {
-        return _electionTimeoutPeriod;
+        return Milliseconds(getSettings()->getElectionTimeoutMillis());
     }
 
     /**
@@ -331,7 +373,7 @@ public:
      * nodes in the replica set.
      */
     Seconds getHeartbeatTimeoutPeriod() const {
-        return _heartbeatTimeoutPeriod;
+        return Seconds(getSettings()->getHeartbeatTimeoutSecs());
     }
 
     /**
@@ -340,14 +382,14 @@ public:
      * Seconds object.
      */
     Milliseconds getHeartbeatTimeoutPeriodMillis() const {
-        return _heartbeatTimeoutPeriod;
+        return duration_cast<Milliseconds>(getHeartbeatTimeoutPeriod());
     }
 
     /**
      * Gets the timeout to wait for a primary to catch up its oplog.
      */
     Milliseconds getCatchUpTimeoutPeriod() const {
-        return _catchUpTimeoutPeriod;
+        return Milliseconds(getSettings()->getCatchUpTimeoutMillis());
     }
 
     /**
@@ -368,7 +410,7 @@ public:
      * Returns true if automatic (not explicitly set) chaining is allowed.
      */
     bool isChainingAllowed() const {
-        return _chainingAllowed;
+        return getSettings()->getChainingAllowed();
     }
 
     /**
@@ -376,21 +418,6 @@ public:
      */
     bool isLocalHostAllowed() const;
 
-    /**
-     * Returns whether or not majority write concerns should implicitly journal, if j has not been
-     * explicitly set.
-     */
-    bool getWriteConcernMajorityShouldJournal() const {
-        return _writeConcernMajorityJournalDefault;
-    }
-
-    /**
-     * Returns true if this replica set is for use as a config server replica set.
-     */
-    bool isConfigServer() const {
-        return _configServer;
-    }
-
     /**
      * Returns a ReplSetTag with the given "key" and "value", or an invalid
      * tag if the configuration describes no such tag.
@@ -404,6 +431,15 @@ public:
      */
     StatusWith<ReplSetTagPattern> findCustomWriteMode(StringData patternName) const;
 
+    /**
+     * Returns a pattern constructed from a raw set of tags provided as the `w` value
+     * of a write concern.
+     *
+     * @returns `ErrorCodes::NoSuchKey` if a tag was provided which is not found in
+     * the local tag config.
+     */
+    StatusWith<ReplSetTagPattern> makeCustomWriteMode(const WTags& wTags) const;
+
     /**
      * Returns the "tags configuration" for this replicaset.
      *
@@ -414,9 +450,9 @@ public:
     }
 
     /**
-     * Returns the config as a BSONObj.
+     * Returns the config as a BSONObj. Omits 'newlyAdded' fields.
      */
-    BSONObj toBSON() const;
+    BSONObj toBSONWithoutNewlyAdded() const;
 
     /**
      * Returns a vector of strings which are the names of the WriteConcernModes.
@@ -439,30 +475,20 @@ public:
         return _writeMajority;
     }
 
-    /**
-     * Gets the protocol version for this configuration.
-     *
-     * The protocol version number currently determines what election protocol is used by the
-     * cluster; 1 is the default.
-     */
-    long long getProtocolVersion() const {
-        return _protocolVersion;
-    }
-
     /**
      * Returns true if this configuration contains a valid replica set ID.
      * This ID is set at creation and is used to disambiguate replica set configurations that may
      * have the same replica set name (_id field) but meant for different replica set instances.
      */
     bool hasReplicaSetId() const {
-        return _replicaSetId.isSet();
+        return getSettings()->getReplicaSetId() != boost::none;
     }
 
     /**
      * Returns replica set ID.
      */
     OID getReplicaSetId() const {
-        return _replicaSetId;
+        return getSettings()->getReplicaSetId() ? *getSettings()->getReplicaSetId() : OID();
     }
 
     /**
@@ -476,7 +502,7 @@ public:
      * sees that it is more caught up than the current primary.
      */
     Milliseconds getCatchUpTakeoverDelay() const {
-        return _catchUpTakeoverDelay;
+        return Milliseconds(getSettings()->getCatchUpTakeoverDelayMillis());
     }
 
     /**
@@ -489,6 +515,16 @@ public:
      */
     bool containsArbiter() const;
 
+    /**
+     * Returns a mutable (but not directly usable) copy of the config.
+     */
+    MutableReplSetConfig getMutable() const;
+
+    /**
+     * Returns true if implicit default write concern should be majority.
+     */
+    bool isImplicitDefaultWriteConcernMajority() const;
+
     /**
      * Returns true if the config consists of a Primary-Secondary-Arbiter (PSA) architecture.
      */
@@ -496,11 +532,44 @@ public:
         return getNumMembers() == 3 && getNumDataBearingMembers() == 2;
     }
 
+    /**
+     * Returns true if the getLastErrorDefaults has been customized.
+     */
+    bool containsCustomizedGetLastErrorDefaults() const;
+
+    /**
+     * Returns Status::OK if write concern is valid for this config, or appropriate status
+     * otherwise.
+     */
+    Status validateWriteConcern(const WriteConcernOptions& writeConcern) const;
+
+    /**
+     * Returns true if this config is a split config, which is determined by checking if it contains
+     * a recipient config for a shard split operation.
+     */
+    bool isSplitConfig() const;
+
+    /**
+     * Returns the config for the recipient during a tenant split operation, if it exists.
+     */
+    ReplSetConfigPtr getRecipientConfig() const;
+
+    /**
+     * Compares the write concern modes with another config and returns 'true' if they are
+     * identical.
+     */
+    bool areWriteConcernModesTheSame(ReplSetConfig* otherConfig) const;
+
 private:
     /**
-     * Parses the "settings" subdocument of a replica set configuration.
+     * Sets replica set ID to 'defaultReplicaSetId' if 'cfg' does not contain an ID.
+     * Sets term to kInitialTerm for initiate.
+     * Sets term to forceTerm if it is not boost::none. Otherwise, parses term from 'cfg'.
      */
-    Status _parseSettingsSubdocument(const BSONObj& settings);
+    ReplSetConfig(const BSONObj& cfg,
+                  bool forInitiate,
+                  boost::optional<long long> forceTerm,
+                  OID defaultReplicaSetId);
 
     /**
      * Calculates and stores the majority for electing a primary (_majorityVoteCount).
@@ -513,46 +582,36 @@ private:
     void _addInternalWriteConcernModes();
 
     /**
-     * Populate _connectionString based on the contents of _members and _replSetName.
+     * Populate _connectionString based on the contents of members and replSetName.
      */
     void _initializeConnectionString();
 
     /**
-     * Sets replica set ID to 'defaultReplicaSetId' if forInitiate is false and 'cfg' does not
-     * contain an ID.
-     * Sets _term to kInitialTerm for initiate.
-     * Sets _term to forceTerm if it is not boost::none. Otherwise, parses term from 'cfg'.
+     * Sets the required fields of the IDL object.
      */
-    Status _initialize(const BSONObj& cfg,
-                       bool forInitiate,
+    void _setRequiredFields();
+
+    /**
+     * Performs basic consistency checks on the replica set configuration.
+     */
+    Status _validate(bool allowSplitHorizonIP) const;
+
+    /**
+     * Common code used by constructors
+     */
+    Status _initialize(bool forInitiate,
                        boost::optional<long long> forceTerm,
                        OID defaultReplicaSetId);
 
     bool _isInitialized = false;
-    long long _version = 1;
-    long long _term = OpTime::kUninitializedTerm;
-    std::string _replSetName;
-    std::vector<MemberConfig> _members;
-    WriteConcernOptions _defaultWriteConcern;
-    Milliseconds _electionTimeoutPeriod = kDefaultElectionTimeoutPeriod;
-    Milliseconds _heartbeatInterval = kDefaultHeartbeatInterval;
-    Seconds _heartbeatTimeoutPeriod = kDefaultHeartbeatTimeoutPeriod;
-    Milliseconds _catchUpTimeoutPeriod = kDefaultCatchUpTimeoutPeriod;
-    Milliseconds _catchUpTakeoverDelay = kDefaultCatchUpTakeoverDelay;
-    bool _chainingAllowed = kDefaultChainingAllowed;
-    bool _writeConcernMajorityJournalDefault = false;
     int _majorityVoteCount = 0;
     int _writableVotingMembersCount = 0;
     int _writeMajority = 0;
     int _totalVotingMembers = 0;
     ReplSetTagConfig _tagConfig;
     StringMap<ReplSetTagPattern> _customWriteConcernModes;
-    long long _protocolVersion = 1;
-    bool _configServer = false;
-    OID _replicaSetId;
     ConnectionString _connectionString;
 };
 
-
 }  // namespace repl
 }  // namespace mongo
