@@ -35,7 +35,6 @@
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/lock_state.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/storage_interface.h"
@@ -193,8 +192,7 @@ OpTime ReplicationConsistencyMarkersImpl::getMinValid(OperationContext* opCtx) c
 }
 
 void ReplicationConsistencyMarkersImpl::setMinValid(OperationContext* opCtx,
-                                                    const OpTime& minValid,
-                                                    bool alwaysAllowUntimestampedWrite) {
+                                                    const OpTime& minValid) {
     LOGV2_DEBUG(21289,
                 3,
                 "setting minvalid to exactly: {minValidString}({minValidBSON})",
@@ -206,11 +204,6 @@ void ReplicationConsistencyMarkersImpl::setMinValid(OperationContext* opCtx,
                             << minValid.getTimestamp() << MinValidDocument::kMinValidTermFieldName
                             << minValid.getTerm()));
 
-    // This method is only used with storage engines that do not support recover to stable
-    // timestamp. As a result, their timestamps do not matter.
-    invariant(alwaysAllowUntimestampedWrite ||
-              !opCtx->getServiceContext()->getStorageEngine()->supportsRecoverToStableTimestamp());
-
     _updateMinValidDocument(opCtx, update);
 }
 
@@ -424,11 +417,12 @@ ReplicationConsistencyMarkersImpl::refreshOplogTruncateAfterPointIfPrimary(
     }
     ON_BLOCK_EXIT([&] { opCtx->recoveryUnit()->setPrepareConflictBehavior(originalBehavior); });
 
-    // Exempt storage ticket acquisition in order to avoid starving upstream requests waiting
-    // for durability. SERVER-60682 is an example with more pending prepared transactions than
-    // storage tickets; the transaction coordinator could not persist the decision and
-    // had to unnecessarily wait for prepared transactions to expire to make forward progress.
-    SkipTicketAcquisitionForLock skipTicketAcquisition(opCtx);
+    // Exempt waiting for storage ticket acquisition in order to avoid starving upstream requests
+    // waiting for durability. SERVER-60682 is an example with more pending prepared transactions
+    // than storage tickets; the transaction coordinator could not persist the decision and had to
+    // unnecessarily wait for prepared transactions to expire to make forward progress.
+    ScopedAdmissionPriorityForLock setTicketAquisition(opCtx->lockState(),
+                                                       AdmissionContext::Priority::kImmediate);
 
     // The locks necessary to write to the oplog truncate after point's collection and read from the
     // oplog collection must be taken up front so that the mutex can also be taken around both
@@ -497,7 +491,8 @@ ReplicationConsistencyMarkersImpl::refreshOplogTruncateAfterPointIfPrimary(
 }
 
 Status ReplicationConsistencyMarkersImpl::createInternalCollections(OperationContext* opCtx) {
-    for (auto nss : std::vector<NamespaceString>({_oplogTruncateAfterPointNss, _minValidNss})) {
+    for (const auto& nss :
+         std::vector<NamespaceString>({_oplogTruncateAfterPointNss, _minValidNss})) {
         auto status = _storageInterface->createCollection(opCtx, nss, CollectionOptions());
         if (!status.isOK() && status.code() != ErrorCodes::NamespaceExists) {
             return {ErrorCodes::CannotCreateCollection,
@@ -512,8 +507,7 @@ void ReplicationConsistencyMarkersImpl::setInitialSyncIdIfNotSet(OperationContex
     auto status =
         _storageInterface->createCollection(opCtx, _initialSyncIdNss, CollectionOptions());
     if (!status.isOK() && status.code() != ErrorCodes::NamespaceExists) {
-        LOGV2_FATAL(
-            4608500, "Failed to create collection", "namespace"_attr = _initialSyncIdNss.ns());
+        LOGV2_FATAL(4608500, "Failed to create collection", logAttrs(_initialSyncIdNss));
         fassertFailedWithStatus(4608502, status);
     }
 
