@@ -43,8 +43,10 @@
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog/local_oplog_info.h"
+#include "mongo/db/catalog_shard_feature_flag_gen.h"
 #include "mongo/db/change_stream_change_collection_manager.h"
 #include "mongo/db/change_stream_pre_images_collection_manager.h"
+#include "mongo/db/change_stream_serverless_helpers.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands/feature_compatibility_version.h"
 #include "mongo/db/commands/rwc_defaults_commands_gen.h"
@@ -54,7 +56,6 @@
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/free_mon/free_mon_mongod.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/logical_time_validator.h"
@@ -77,11 +78,9 @@
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/s/balancer/balancer.h"
-#include "mongo/db/s/chunk_splitter.h"
 #include "mongo/db/s/config/index_on_config.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/migration_util.h"
-#include "mongo/db/s/periodic_balancer_config_refresher.h"
 #include "mongo/db/s/periodic_sharded_index_consistency_checker.h"
 #include "mongo/db/s/resharding/resharding_donor_recipient_common.h"
 #include "mongo/db/s/shard_local.h"
@@ -111,6 +110,7 @@
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/cluster_identity_loader.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/transport/service_entry_point.h"
 #include "mongo/util/assert_util.h"
@@ -130,15 +130,7 @@ namespace mongo {
 namespace repl {
 namespace {
 
-const char localDbName[] = "local";
-// TODO SERVER-62491 Use SystemTenantId
-const auto configDatabaseName = DatabaseName(boost::none, localDbName);
-const auto lastVoteDatabaseName = localDbName;
-const char meCollectionName[] = "local.me";
-const auto meDatabaseName = localDbName;
-const char tsFieldName[] = "ts";
-
-const NamespaceString configCollectionNS{"local", "system.replset"};
+const char kLocalDbName[] = "local";
 
 MONGO_FAIL_POINT_DEFINE(dropPendingCollectionReaperHang);
 
@@ -430,7 +422,8 @@ Status ReplicationCoordinatorExternalStateImpl::initializeReplSetStorage(Operati
                                {
                                    // Writes to 'local.system.replset' must be untimestamped.
                                    WriteUnitOfWork wuow(opCtx);
-                                   Helpers::putSingleton(opCtx, configCollectionNS, config);
+                                   Helpers::putSingleton(
+                                       opCtx, NamespaceString::kSystemReplSetNamespace, config);
                                    wuow.commit();
                                }
                                {
@@ -459,7 +452,8 @@ Status ReplicationCoordinatorExternalStateImpl::initializeReplSetStorage(Operati
 
 void ReplicationCoordinatorExternalStateImpl::onDrainComplete(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
-    invariant(!opCtx->shouldParticipateInFlowControl());
+    invariant(opCtx->lockState()->getAdmissionPriority() == AdmissionContext::Priority::kImmediate,
+              "Replica Set state changes are critical to the cluster and should not be throttled");
 
     if (_oplogBuffer) {
         _oplogBuffer->exitDrainMode();
@@ -468,9 +462,11 @@ void ReplicationCoordinatorExternalStateImpl::onDrainComplete(OperationContext*
 
 OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationContext* opCtx) {
     invariant(opCtx->lockState()->isRSTLExclusive());
-    invariant(!opCtx->shouldParticipateInFlowControl());
+    invariant(opCtx->lockState()->getAdmissionPriority() == AdmissionContext::Priority::kImmediate,
+              "Replica Set state changes are critical to the cluster and should not be throttled");
 
-    MongoDSessionCatalog::onStepUp(opCtx);
+    auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+    mongoDSessionCatalog->onStepUp(opCtx);
 
     invariant(
         _replicationProcess->getConsistencyMarkers()->getOplogTruncateAfterPoint(opCtx).isNull());
@@ -506,6 +502,10 @@ OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationC
                  << ReplicationCoordinator::newPrimaryMsg));
         wuow.commit();
     });
+    // As far as the storage system is concerned, we're still secondary here, and will be until we
+    // change readWriteAbility.  So new and resumed lock-free reads will read from lastApplied.  We
+    // just advanced lastApplied by writing the no-op, so we need to signal oplog waiters.
+    signalOplogWaiters();
     const auto loadLastOpTimeAndWallTimeResult = loadLastOpTimeAndWallTime(opCtx);
     fassert(28665, loadLastOpTimeAndWallTimeResult);
     auto opTimeToReturn = loadLastOpTimeAndWallTimeResult.getValue().opTime;
@@ -531,8 +531,6 @@ OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationC
 
     IndexBuildsCoordinator::get(opCtx)->onStepUp(opCtx);
 
-    notifyFreeMonitoringOnTransitionToPrimary();
-
     // It is only necessary to check the system indexes on the first transition to primary.
     // On subsequent transitions to primary the indexes will have already been created.
     static std::once_flag verifySystemIndexesOnce;
@@ -544,7 +542,7 @@ OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationC
     });
 
     // Create the pre-images collection if it doesn't exist yet in the non-serverless environment.
-    if (!ChangeStreamChangeCollectionManager::isChangeCollectionsModeActive()) {
+    if (!change_stream_serverless_helpers::isChangeCollectionsModeActive()) {
         ChangeStreamPreImagesCollectionManager::createPreImagesCollection(
             opCtx, boost::none /* tenantId */);
     }
@@ -562,13 +560,17 @@ StatusWith<BSONObj> ReplicationCoordinatorExternalStateImpl::loadLocalConfigDocu
     OperationContext* opCtx) {
     try {
         return writeConflictRetry(
-            opCtx, "load replica set config", configCollectionNS.ns(), [opCtx] {
+            opCtx,
+            "load replica set config",
+            NamespaceString::kSystemReplSetNamespace.ns(),
+            [opCtx] {
                 BSONObj config;
-                if (!Helpers::getSingleton(opCtx, configCollectionNS, config)) {
+                if (!Helpers::getSingleton(
+                        opCtx, NamespaceString::kSystemReplSetNamespace, config)) {
                     return StatusWith<BSONObj>(
                         ErrorCodes::NoMatchingDocument,
                         "Did not find replica set configuration document in {}"_format(
-                            configCollectionNS.toString()));
+                            NamespaceString::kSystemReplSetNamespace.toString()));
                 }
                 return StatusWith<BSONObj>(config);
             });
@@ -581,29 +583,30 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalConfigDocument(Operati
                                                                          const BSONObj& config,
                                                                          bool writeOplog) {
     try {
-        writeConflictRetry(opCtx, "save replica set config", configCollectionNS.ns(), [&] {
-            {
-                // Writes to 'local.system.replset' must be untimestamped.
-                WriteUnitOfWork wuow(opCtx);
-                Lock::DBLock dbWriteLock(opCtx, configDatabaseName, MODE_X);
-                Helpers::putSingleton(opCtx, configCollectionNS, config);
-                wuow.commit();
-            }
+        writeConflictRetry(
+            opCtx, "save replica set config", NamespaceString::kSystemReplSetNamespace.ns(), [&] {
+                {
+                    // Writes to 'local.system.replset' must be untimestamped.
+                    WriteUnitOfWork wuow(opCtx);
+                    AutoGetCollection coll(opCtx, NamespaceString::kSystemReplSetNamespace, MODE_X);
+                    Helpers::putSingleton(opCtx, NamespaceString::kSystemReplSetNamespace, config);
+                    wuow.commit();
+                }
 
-            if (writeOplog) {
-                // The no-op write doesn't affect the correctness of the safe reconfig protocol and
-                // so it doesn't have to be written in the same WUOW as the config write. In fact,
-                // the no-op write is only needed for some corner cases where the committed snapshot
-                // is dropped after a force reconfig that changes the config content or a safe
-                // reconfig that changes writeConcernMajorityJournalDefault.
-                WriteUnitOfWork wuow(opCtx);
-                auto msgObj = BSON("msg"
-                                   << "Reconfig set"
-                                   << "version" << config["version"]);
-                _service->getOpObserver()->onOpMessage(opCtx, msgObj);
-                wuow.commit();
-            }
-        });
+                if (writeOplog) {
+                    // The no-op write doesn't affect the correctness of the safe reconfig protocol
+                    // and so it doesn't have to be written in the same WUOW as the config write. In
+                    // fact, the no-op write is only needed for some corner cases where the
+                    // committed snapshot is dropped after a force reconfig that changes the config
+                    // content or a safe reconfig that changes writeConcernMajorityJournalDefault.
+                    WriteUnitOfWork wuow(opCtx);
+                    auto msgObj = BSON("msg"
+                                       << "Reconfig set"
+                                       << "version" << config["version"]);
+                    _service->getOpObserver()->onOpMessage(opCtx, msgObj);
+                    wuow.commit();
+                }
+            });
         return Status::OK();
     } catch (const DBException& ex) {
         return ex.toStatus();
@@ -612,13 +615,14 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalConfigDocument(Operati
 
 Status ReplicationCoordinatorExternalStateImpl::replaceLocalConfigDocument(
     OperationContext* opCtx, const BSONObj& config) try {
-    writeConflictRetry(opCtx, "replace replica set config", configCollectionNS.ns(), [&] {
-        WriteUnitOfWork wuow(opCtx);
-        Lock::DBLock dbWriteLock(opCtx, configDatabaseName, MODE_X);
-        Helpers::emptyCollection(opCtx, configCollectionNS);
-        Helpers::putSingleton(opCtx, configCollectionNS, config);
-        wuow.commit();
-    });
+    writeConflictRetry(
+        opCtx, "replace replica set config", NamespaceString::kSystemReplSetNamespace.ns(), [&] {
+            WriteUnitOfWork wuow(opCtx);
+            AutoGetCollection coll(opCtx, NamespaceString::kSystemReplSetNamespace, MODE_X);
+            Helpers::emptyCollection(opCtx, NamespaceString::kSystemReplSetNamespace);
+            Helpers::putSingleton(opCtx, NamespaceString::kSystemReplSetNamespace, config);
+            wuow.commit();
+        });
     return Status::OK();
 } catch (const DBException& ex) {
     return ex.toStatus();
@@ -686,15 +690,24 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalLastVoteDocument(
     OperationContext* opCtx, const LastVote& lastVote) {
     BSONObj lastVoteObj = lastVote.toBSON();
 
-    // Writes that are part of elections should not be throttled.
-    invariant(!opCtx->shouldParticipateInFlowControl());
+    invariant(opCtx->lockState()->getAdmissionPriority() == AdmissionContext::Priority::kImmediate,
+              "Writes that are part of elections should not be throttled");
 
     try {
         // If we are casting a vote in a new election immediately after stepping down, we
         // don't want to have this process interrupted due to us stepping down, since we
         // want to be able to cast our vote for a new primary right away. Both the write's lock
         // acquisition and the "waitUntilDurable" lock acquisition must be uninterruptible.
-        UninterruptibleLockGuard noInterrupt(opCtx->lockState());
+        //
+        // It is not safe to take an uninterruptible lock during STARTUP2, so we only take this lock
+        // if we are primary or secondary.  We do not have the RSTL but that is OK because we never
+        // move in to STARTUP2 from PRIMARY or SECONDARY, so the consequence of a stale state is
+        // only that we don't take an uninterruptible lock when we should.
+        auto* replCoord = ReplicationCoordinator::get(opCtx);
+
+        boost::optional<UninterruptibleLockGuard> noInterrupt;
+        if (replCoord->isInPrimaryOrSecondaryState_UNSAFE())
+            noInterrupt.emplace(opCtx->lockState());
 
         Status status = writeConflictRetry(
             opCtx,
@@ -767,7 +780,7 @@ StatusWith<OpTimeAndWallTime> ReplicationCoordinatorExternalStateImpl::loadLastO
         BSONObj oplogEntry;
 
         if (!writeConflictRetry(
-                opCtx, "Load last opTime", NamespaceString::kRsOplogNamespace.ns().c_str(), [&] {
+                opCtx, "Load last opTime", NamespaceString::kRsOplogNamespace.ns(), [&] {
                     return Helpers::getLast(opCtx, NamespaceString::kRsOplogNamespace, oplogEntry);
                 })) {
             return StatusWith<OpTimeAndWallTime>(ErrorCodes::NoMatchingDocument,
@@ -813,14 +826,20 @@ void ReplicationCoordinatorExternalStateImpl::onStepDownHook() {
 }
 
 void ReplicationCoordinatorExternalStateImpl::_shardingOnStepDownHook() {
-    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
+    if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
         PeriodicShardedIndexConsistencyChecker::get(_service).onStepDown();
         TransactionCoordinatorService::get(_service)->onStepDown();
-    } else if (ShardingState::get(_service)->enabled()) {
-        ChunkSplitter::get(_service).onStepDown();
-        CatalogCacheLoader::get(_service).onStepDown();
-        PeriodicBalancerConfigRefresher::get(_service).onStepDown();
-        TransactionCoordinatorService::get(_service)->onStepDown();
+    }
+
+    if (ShardingState::get(_service)->enabled()) {
+        if (!serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
+            // Called earlier for config servers.
+            TransactionCoordinatorService::get(_service)->onStepDown();
+            CatalogCacheLoader::get(_service).onStepDown();
+            // (Ignore FCV check): TODO(SERVER-75389): add why FCV is ignored here.
+        } else if (gFeatureFlagTransitionToCatalogShard.isEnabledAndIgnoreFCVUnsafe()) {
+            CatalogCacheLoader::get(_service).onStepDown();
+        }
     }
 
     if (auto validator = LogicalTimeValidator::get(_service)) {
@@ -841,7 +860,11 @@ void ReplicationCoordinatorExternalStateImpl::_stopAsyncUpdatesOfAndClearOplogTr
     // available, which may have to wait for the ticket refresher to run, which in turn blocks
     // on the repl _mutex to check whether we are primary or not: this is a deadlock because
     // stepdown already holds the repl _mutex!
-    FlowControl::Bypass flowControlBypass(opCtx);
+    // As opCtx does not expose a method to allow skipping flow control on purpose we mark the
+    // operation as having Immediate priority. This will skip flow control and ticket acquisition.
+    // It is fine to do this since the system is essentially shutting down at this point.
+    ScopedAdmissionPriorityForLock priority(opCtx->lockState(),
+                                            AdmissionContext::Priority::kImmediate);
 
     // Tell the system to stop updating the oplogTruncateAfterPoint asynchronously and to go
     // back to using last applied to update repl's durable timestamp instead of the truncate
@@ -876,7 +899,7 @@ void ReplicationCoordinatorExternalStateImpl::_stopAsyncUpdatesOfAndClearOplogTr
 
 void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook(
     OperationContext* opCtx) {
-    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
+    if (serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
         Status status = ShardingCatalogManager::get(opCtx)->initializeConfigDatabaseIfNeeded(opCtx);
         if (!status.isOK() && status != ErrorCodes::AlreadyInitialized) {
             // If the node is shutting down or it lost quorum just as it was becoming primary,
@@ -895,15 +918,17 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
 
         if (status.isOK()) {
             // Load the clusterId into memory. Use local readConcern, since we can't use
-            // majority readConcern in drain mode because the global lock prevents replication.
-            // This is safe, since if the clusterId write is rolled back, any writes that depend
-            // on it will also be rolled back.
+            // majority/snapshot readConcern in drain mode because the global lock prevents
+            // replication. This is safe, since if the clusterId write is rolled back, any writes
+            // that depend on it will also be rolled back.
             //
             // Since we *just* wrote the cluster ID to the config.version document (via the call
             // to ShardingCatalogManager::initializeConfigDatabaseIfNeeded above), this read can
             // only meaningfully fail if the node is shutting down.
             status = ClusterIdentityLoader::get(opCtx)->loadClusterId(
-                opCtx, repl::ReadConcernLevel::kLocalReadConcern);
+                opCtx,
+                ShardingCatalogManager::get(opCtx)->localCatalogClient(),
+                repl::ReadConcernLevel::kLocalReadConcern);
 
             if (ErrorCodes::isShutdownError(status.code())) {
                 return;
@@ -917,10 +942,17 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
 
         PeriodicShardedIndexConsistencyChecker::get(_service).onStepUp(_service);
         TransactionCoordinatorService::get(_service)->onStepUp(opCtx);
-    } else if (serverGlobalParams.clusterRole == ClusterRole::ShardServer) {
+
+        // (Ignore FCV check): TODO(SERVER-75389): add why FCV is ignored here.
+        if (gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe() &&
+            gFeatureFlagTransitionToCatalogShard.isEnabledAndIgnoreFCVUnsafe()) {
+            CatalogCacheLoader::get(_service).onStepUp();
+        }
+    }
+    if (serverGlobalParams.clusterRole.has(ClusterRole::ShardServer)) {
         if (ShardingState::get(opCtx)->enabled()) {
-            Status status = ShardingStateRecovery::recover(opCtx);
             VectorClockMutable::get(opCtx)->recoverDirect(opCtx);
+            Status status = ShardingStateRecovery_DEPRECATED::recover(opCtx);
 
             // If the node is shutting down or it lost quorum just as it was becoming primary, don't
             // run the sharding onStepUp machinery. The onStepDown counterpart to these methods is
@@ -931,10 +963,11 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
             }
             fassert(40107, status);
 
-            CatalogCacheLoader::get(_service).onStepUp();
-            ChunkSplitter::get(_service).onStepUp();
-            PeriodicBalancerConfigRefresher::get(_service).onStepUp(_service);
-            TransactionCoordinatorService::get(_service)->onStepUp(opCtx);
+            if (!serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer)) {
+                // Called earlier for config servers.
+                TransactionCoordinatorService::get(_service)->onStepUp(opCtx);
+                CatalogCacheLoader::get(_service).onStepUp();
+            }
 
             const auto configsvrConnStr =
                 Grid::get(opCtx)->shardRegistry()->getConfigShard()->getConnString();
@@ -944,7 +977,11 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
             // Note, these must be done after the configOpTime is recovered via
             // ShardingStateRecovery::recover above, because they may trigger filtering metadata
             // refreshes which should use the recovered configOpTime.
-            migrationutil::resubmitRangeDeletionsOnStepUp(_service);
+            // (Ignore FCV check): This feature flag doesn't have upgrade/downgrade concern. The
+            // feature flag is used to turn on new range deleter on startup.
+            if (!mongo::feature_flags::gRangeDeleterService.isEnabledAndIgnoreFCVUnsafe()) {
+                migrationutil::resubmitRangeDeletionsOnStepUp(_service);
+            }
             migrationutil::resumeMigrationCoordinationsOnStepUp(opCtx);
             migrationutil::resumeMigrationRecipientsOnStepUp(opCtx);
 
@@ -977,48 +1014,64 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
                                         "shard's first transition to primary"));
         }
 
-        // Create indexes in config.shard.indexes if needed.
-        indexStatus = sharding_util::createGlobalIndexesIndexes(opCtx);
-        if (!indexStatus.isOK()) {
-            // If the node is shutting down or it lost quorum just as it was becoming primary,
-            // don't run the sharding onStepUp machinery. The onStepDown counterpart to these
-            // methods is already idempotent, so the machinery will remain in the stepped down
-            // state.
-            if (ErrorCodes::isShutdownError(indexStatus.code()) ||
-                ErrorCodes::isNotPrimaryError(indexStatus.code())) {
-                return;
+        // (Ignore FCV check): TODO(SERVER-75389): add why FCV is ignored here.
+        if (mongo::feature_flags::gGlobalIndexesShardingCatalog.isEnabledAndIgnoreFCVUnsafe()) {
+            // Create indexes in config.shard.indexes if needed.
+            indexStatus = sharding_util::createShardingIndexCatalogIndexes(opCtx);
+            if (!indexStatus.isOK()) {
+                // If the node is shutting down or it lost quorum just as it was becoming primary,
+                // don't run the sharding onStepUp machinery. The onStepDown counterpart to these
+                // methods is already idempotent, so the machinery will remain in the stepped down
+                // state.
+                if (ErrorCodes::isShutdownError(indexStatus.code()) ||
+                    ErrorCodes::isNotPrimaryError(indexStatus.code())) {
+                    return;
+                }
+                fassertFailedWithStatus(
+                    6280501,
+                    indexStatus.withContext(str::stream()
+                                            << "Failed to create index on "
+                                            << NamespaceString::kShardIndexCatalogNamespace
+                                            << " on shard's first transition to primary"));
             }
-            fassertFailedWithStatus(
-                6280501,
-                indexStatus.withContext(str::stream()
-                                        << "Failed to create index on "
-                                        << NamespaceString::kShardIndexCatalogNamespace
-                                        << " on shard's first transition to primary"));
-        }
 
-        // Create indexes in config.shard.collections if needed.
-        indexStatus = sharding_util::createShardCollectionCatalogIndexes(opCtx);
-        if (!indexStatus.isOK()) {
-            // If the node is shutting down or it lost quorum just as it was becoming primary,
-            // don't run the sharding onStepUp machinery. The onStepDown counterpart to these
-            // methods is already idempotent, so the machinery will remain in the stepped down
-            // state.
-            if (ErrorCodes::isShutdownError(indexStatus.code()) ||
-                ErrorCodes::isNotPrimaryError(indexStatus.code())) {
-                return;
+            // Create indexes in config.shard.collections if needed.
+            indexStatus = sharding_util::createShardCollectionCatalogIndexes(opCtx);
+            if (!indexStatus.isOK()) {
+                // If the node is shutting down or it lost quorum just as it was becoming primary,
+                // don't run the sharding onStepUp machinery. The onStepDown counterpart to these
+                // methods is already idempotent, so the machinery will remain in the stepped down
+                // state.
+                if (ErrorCodes::isShutdownError(indexStatus.code()) ||
+                    ErrorCodes::isNotPrimaryError(indexStatus.code())) {
+                    return;
+                }
+                fassertFailedWithStatus(
+                    6711907,
+                    indexStatus.withContext(str::stream()
+                                            << "Failed to create index on "
+                                            << NamespaceString::kShardCollectionCatalogNamespace
+                                            << " on shard's first transition to primary"));
             }
-            fassertFailedWithStatus(
-                6711907,
-                indexStatus.withContext(str::stream()
-                                        << "Failed to create index on "
-                                        << NamespaceString::kShardCollectionCatalogNamespace
-                                        << " on shard's first transition to primary"));
         }
-    } else {  // unsharded
+    }
+    if (serverGlobalParams.clusterRole.has(ClusterRole::None)) {  // unsharded
         if (auto validator = LogicalTimeValidator::get(_service)) {
             validator->enableKeyGenerator(opCtx, true);
         }
     }
+
+    // (Ignore FCV check): TODO(SERVER-75389): add why FCV is ignored here.
+    if (gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe() &&
+        serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer) &&
+        !ShardingState::get(opCtx)->enabled()) {
+        // Note this must be called after the config server has created the cluster ID and also
+        // after the onStepUp logic for the shard role because this triggers sharding state
+        // initialization which will transition some components into the "primary" state, like the
+        // TransactionCoordinatorService, and they would fail if the onStepUp logic attempted the
+        // same transition.
+        ShardingCatalogManager::get(opCtx)->installConfigShardIdentityDocument(opCtx);
+    }
 }
 
 void ReplicationCoordinatorExternalStateImpl::signalApplierToChooseNewSyncSource() {
@@ -1076,14 +1129,14 @@ void ReplicationCoordinatorExternalStateImpl::_dropAllTempCollections(OperationC
     for (const auto& dbName : dbNames) {
         // The local db is special because it isn't replicated. It is cleared at startup even on
         // replica set members.
-        if (dbName.db() == NamespaceString::kLocalDb)
+        if (dbName == DatabaseName::kLocal)
             continue;
 
         LOGV2_DEBUG(21309,
                     2,
                     "Removing temporary collections from {db}",
                     "Removing temporary collections",
-                    "db"_attr = dbName);
+                    logAttrs(dbName));
         Lock::DBLock dbLock(opCtx, dbName, MODE_IX);
         clearTempCollections(opCtx, dbName);
     }
@@ -1229,21 +1282,19 @@ void ReplicationCoordinatorExternalStateImpl::setupNoopWriter(Seconds waitTime)
 
 bool ReplicationCoordinatorExternalStateImpl::isShardPartOfShardedCluster(
     OperationContext* opCtx) const {
-    return serverGlobalParams.clusterRole == ClusterRole::ShardServer &&
+    return serverGlobalParams.clusterRole.has(ClusterRole::ShardServer) &&
         ShardingState::get(opCtx)->enabled();
 }
 
 bool ReplicationCoordinatorExternalStateImpl::isCWWCSetOnConfigShard(
     OperationContext* opCtx) const {
     GetDefaultRWConcern configsvrRequest;
-    // Empty tenant id is acceptable here as command's tenant id will not be serialized to BSON.
-    // TODO SERVER-62491: Use system tenant id.
-    configsvrRequest.setDbName(DatabaseName(boost::none, NamespaceString::kAdminDb));
+    configsvrRequest.setDbName(DatabaseName::kAdmin);
     auto cmdResponse = uassertStatusOK(
         Grid::get(opCtx)->shardRegistry()->getConfigShard()->runCommandWithFixedRetryAttempts(
             opCtx,
             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-            NamespaceString::kAdminDb.toString(),
+            DatabaseName::kAdmin.toString(),
             configsvrRequest.toBSON({}),
             Shard::RetryPolicy::kIdempotent));
 
