@@ -36,7 +36,6 @@
 
 #include "mongo/base/status.h"
 #include "mongo/bson/timestamp.h"
-#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
 #include "mongo/db/repl/delayable_timeout_callback.h"
 #include "mongo/db/repl/initial_syncer.h"
@@ -113,7 +112,8 @@ public:
 
     virtual bool inQuiesceMode() const override;
 
-    virtual void shutdown(OperationContext* opCtx) override;
+    virtual void shutdown(OperationContext* opCtx,
+                          BSONObjBuilder* shutdownTimeElapsedBuilder) override;
 
     virtual const ReplSettings& getSettings() const override;
 
@@ -239,6 +239,8 @@ public:
 
     virtual std::vector<MemberConfig> getConfigVotingMembers() const override;
 
+    virtual size_t getNumConfigVotingMembers() const override;
+
     virtual std::int64_t getConfigTerm() const override;
 
     virtual std::int64_t getConfigVersion() const override;
@@ -294,7 +296,8 @@ public:
     virtual Status doOptimizedReconfig(OperationContext* opCtx, GetNewConfigFn) override;
 
     virtual Status awaitConfigCommitment(OperationContext* opCtx,
-                                         bool waitForOplogCommitment) override;
+                                         bool waitForOplogCommitment,
+                                         long long term) override;
 
     virtual Status processReplSetInitiate(OperationContext* opCtx,
                                           const BSONObj& configObj,
@@ -409,8 +412,8 @@ public:
         boost::optional<TopologyVersion> clientTopologyVersion,
         boost::optional<Date_t> deadline) override;
 
-    virtual StatusWith<OpTime> getLatestWriteOpTime(OperationContext* opCtx) const
-        noexcept override;
+    virtual StatusWith<OpTime> getLatestWriteOpTime(
+        OperationContext* opCtx) const noexcept override;
 
     virtual HostAndPort getCurrentPrimaryHostAndPort() const override;
 
@@ -426,6 +429,8 @@ public:
 
     virtual void recordIfCWWCIsSetOnConfigServerOnStartup(OperationContext* opCtx) final;
 
+    virtual SplitPrepareSessionManager* getSplitPrepareSessionManager() override;
+
     // ================== Test support API ===================
 
     /**
@@ -468,6 +473,11 @@ public:
      */
     executor::TaskExecutor::CallbackHandle getCatchupTakeoverCbh_forTest() const;
 
+    /**
+     * Returns the cached horizon topology version from most recent SplitHorizonChange.
+     */
+    int64_t getLastHorizonChange_forTest() const;
+
     /**
      * Simple wrappers around _setLastOptimeForMember to make it easier to test.
      */
@@ -589,6 +599,8 @@ public:
      */
     WriteConcernTagChanges* getWriteConcernTagChanges() override;
 
+    bool isRetryableWrite(OperationContext* opCtx) const override;
+
 private:
     using CallbackFn = executor::TaskExecutor::CallbackFn;
 
@@ -765,6 +777,9 @@ private:
 
     class WaiterList {
     public:
+        WaiterList() = delete;
+        WaiterList(Atomic64Metric& waiterCountMetric);
+
         // Adds waiter into the list.
         void add_inlock(const OpTime& opTime, SharedWaiterHandle waiter);
         // Adds a waiter into the list and returns the future of the waiter's promise.
@@ -782,8 +797,13 @@ private:
         void setErrorAll_inlock(Status status);
 
     private:
+        void _updateMetric_inlock();
+
         // Waiters sorted by OpTime.
         std::multimap<OpTime, SharedWaiterHandle> _waiters;
+        // We keep a separate count outside _waiters.size() in order to avoid having to
+        // take a lock to read the metric.
+        Atomic64Metric& _waiterCountMetric;
     };
 
     enum class HeartbeatState { kScheduled = 0, kSent = 1 };
@@ -1360,7 +1380,9 @@ private:
     void _scheduleHeartbeatReconfig(WithLock lk, const ReplSetConfig& newConfig);
 
     /**
-     * Determines if the provided config is a split config, and validates it for installation.
+     * Accepts a ReplSetConfig and resolves it either to itself, or the embedded shard split
+     * recipient config if it's present and self is a shard split recipient. Returns a tuple of the
+     * resolved config and a boolean indicating whether a recipient config was found.
      */
     std::tuple<StatusWith<ReplSetConfig>, bool> _resolveConfigToApply(const ReplSetConfig& config);
 
@@ -1368,7 +1390,8 @@ private:
      * Method to write a configuration transmitted via heartbeat message to stable storage.
      */
     void _heartbeatReconfigStore(const executor::TaskExecutor::CallbackArgs& cbd,
-                                 const ReplSetConfig& newConfig);
+                                 const ReplSetConfig& newConfig,
+                                 bool isSplitRecipientConfig = false);
 
     /**
      * Conclusion actions of a heartbeat-triggered reconfiguration.
@@ -1376,7 +1399,7 @@ private:
     void _heartbeatReconfigFinish(const executor::TaskExecutor::CallbackArgs& cbData,
                                   const ReplSetConfig& newConfig,
                                   StatusWith<int> myIndex,
-                                  bool isRecipientConfig);
+                                  bool isSplitRecipientConfig);
 
     /**
      * Calculates the time (in millis) left in quiesce mode and converts the value to int64.
@@ -1564,6 +1587,12 @@ private:
      */
     void _enterDrainMode_inlock();
 
+    /**
+     * Enter drain mode which does not result in a primary stepup. Returns a future which becomes
+     * ready when the oplog buffers have completed draining.
+     */
+    Future<void> _drainForShardSplit();
+
     /**
      * Waits for the config state to leave kConfigStartingUp, which indicates that start() has
      * finished.
@@ -1710,8 +1739,8 @@ private:
     // Set to true when we are in the process of shutting down replication.
     bool _inShutdown;  // (M)
 
-    // Election ID of the last election that resulted in this node becoming primary.
-    OID _electionId;  // (M)
+    // The term of the last election that resulted in this node becoming primary.
+    AtomicWord<int64_t> _electionIdTerm;  // (S)
 
     // Used to signal threads waiting for changes to _memberState.
     stdx::condition_variable _memberStateChange;  // (M)
@@ -1836,7 +1865,10 @@ private:
     // The cached value of the 'counter' field in the server's TopologyVersion.
     AtomicWord<int64_t> _cachedTopologyVersionCounter;  // (S)
 
-    // This should be set during sharding initialization.
+    // The cached value of the topology from the most recent SplitHorizonChange.
+    int64_t _lastHorizonTopologyChange{-1};  // (M)
+
+    // This should be set during sharding initialization except on config shard.
     boost::optional<bool> _wasCWWCSetOnConfigServerOnStartup;
 
     InitialSyncerInterface::OnCompletionFn _onCompletion;
@@ -1844,6 +1876,12 @@ private:
     // Construct used to synchronize default write concern changes with config write concern
     // changes.
     WriteConcernTagChangesImpl _writeConcernTagChanges;
+
+    // An optional promise created when entering drain mode for shard split.
+    boost::optional<Promise<void>> _finishedDrainingPromise;  // (M)
+
+    // Pointer to the SplitPrepareSessionManager owned by this ReplicationCoordinator.
+    SplitPrepareSessionManager _splitSessionManager;  // (S)
 };
 
 }  // namespace repl
