@@ -75,7 +75,7 @@ public:
 
     virtual bool inQuiesceMode() const;
 
-    virtual void shutdown(OperationContext* opCtx);
+    virtual void shutdown(OperationContext* opCtx, BSONObjBuilder* shutdownTimeElapsedBuilder);
 
     virtual void appendDiagnosticBSON(BSONObjBuilder* bob) override {}
 
@@ -205,6 +205,8 @@ public:
 
     virtual std::vector<MemberConfig> getConfigVotingMembers() const override;
 
+    virtual size_t getNumConfigVotingMembers() const override;
+
     virtual std::int64_t getConfigTerm() const override;
 
     virtual std::int64_t getConfigVersion() const override;
@@ -261,7 +263,9 @@ public:
 
     virtual Status doOptimizedReconfig(OperationContext* opCtx, GetNewConfigFn getNewConfig);
 
-    Status awaitConfigCommitment(OperationContext* opCtx, bool waitForOplogCommitment);
+    Status awaitConfigCommitment(OperationContext* opCtx,
+                                 bool waitForOplogCommitment,
+                                 long long term);
 
     virtual Status processReplSetInitiate(OperationContext* opCtx,
                                           const BSONObj& configObj,
@@ -391,8 +395,8 @@ public:
         const SplitHorizon::Parameters& horizonParams,
         boost::optional<TopologyVersion> clientTopologyVersion) override;
 
-    virtual StatusWith<OpTime> getLatestWriteOpTime(OperationContext* opCtx) const
-        noexcept override;
+    virtual StatusWith<OpTime> getLatestWriteOpTime(
+        OperationContext* opCtx) const noexcept override;
 
     virtual HostAndPort getCurrentPrimaryHostAndPort() const override;
 
@@ -421,6 +425,21 @@ public:
 
     virtual WriteConcernTagChanges* getWriteConcernTagChanges() override;
 
+    virtual SplitPrepareSessionManager* getSplitPrepareSessionManager() override;
+
+    /**
+     * If this is true, the mock will update the "committed snapshot" everytime the "last applied"
+     * is updated. That behavior can be disabled for tests that need more control over what's
+     * majority committed.
+     */
+    void setUpdateCommittedSnapshot(bool val) {
+        _updateCommittedSnapshot = val;
+    }
+
+    bool isRetryableWrite(OperationContext* opCtx) const override {
+        return false;
+    }
+
 private:
     void _setMyLastAppliedOpTimeAndWallTime(WithLock lk,
                                             const OpTimeAndWallTime& opTimeAndWallTime);
@@ -450,6 +469,9 @@ private:
     bool _resetLastOpTimesCalled = false;
     bool _alwaysAllowWrites = false;
     bool _canAcceptNonLocalWrites = false;
+
+    SplitPrepareSessionManager _splitSessionManager;
+    bool _updateCommittedSnapshot = true;
 };
 
 }  // namespace repl
