@@ -334,9 +334,7 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
     // We support only recovery from stable checkpoints during initial sync.
     invariant(!_duringInitialSync || recoveryTS);
 
-    // Initialize the cached pointer to the oplog collection.
-    acquireOplogCollectionForLogging(opCtx);
-
+    boost::optional<Timestamp> stableTimestamp = boost::none;
     if (recoveryTS || startupRecoveryForRestore) {
         if (startupRecoveryForRestore && !recoveryTS) {
             LOGV2_WARNING(5576601,
@@ -347,8 +345,7 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
 
         // We pass in "none" for the stable timestamp so that recoverFromOplog asks storage
         // for the recoveryTimestamp just like on replica set recovery.
-        const auto stableTimestamp = boost::none;
-        recoverFromOplog(opCtx, stableTimestamp);
+        stableTimestamp = recoverFromOplog(opCtx, boost::none);
     } else {
         if (gTakeUnstableCheckpointOnShutdown) {
             // Ensure 'recoverFromOplogAsStandalone' with 'takeUnstableCheckpointOnShutdown'
@@ -368,7 +365,10 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
 
     if (!_duringInitialSync) {
         // Initial sync will reconstruct prepared transactions when it is completely done.
-        reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
+        reconstructPreparedTransactions(opCtx,
+                                        stableTimestamp
+                                            ? OplogApplication::Mode::kStableRecovering
+                                            : OplogApplication::Mode::kUnstableRecovering);
     }
 }
 
@@ -384,9 +384,6 @@ void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Time
                             "Cannot use 'recoverToOplogTimestamp' without a stable checkpoint");
     }
 
-    // Initialize the cached pointer to the oplog collection.
-    acquireOplogCollectionForLogging(opCtx);
-
     // This may take an IS lock on the oplog collection.
     _truncateOplogIfNeededAndThenClearOplogTruncateAfterPoint(opCtx, &recoveryTS);
 
@@ -430,14 +427,14 @@ void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Time
         invariant(appliedUpTo <= endPoint);
     }
 
-    reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
+    reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kStableRecovering);
 }
 
-void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
-                                               boost::optional<Timestamp> stableTimestamp) try {
+boost::optional<Timestamp> ReplicationRecoveryImpl::recoverFromOplog(
+    OperationContext* opCtx, boost::optional<Timestamp> stableTimestamp) try {
     if (_consistencyMarkers->getInitialSyncFlag(opCtx)) {
         LOGV2(21542, "No recovery needed. Initial sync flag set");
-        return;  // Initial Sync will take over so no cleanup is needed.
+        return stableTimestamp;  // Initial Sync will take over so no cleanup is needed.
     }
 
     const auto serviceCtx = getGlobalServiceContext();
@@ -479,7 +476,7 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
         // Oplog is empty. There are no oplog entries to apply, so we exit recovery and go into
         // initial sync.
         LOGV2(21543, "No oplog entries to apply for recovery. Oplog is empty");
-        return;
+        return stableTimestamp;
     }
     fassert(40290, topOfOplogSW);
     const auto topOfOplog = topOfOplogSW.getValue();
@@ -493,6 +490,7 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
         _recoverFromUnstableCheckpoint(
             opCtx, _consistencyMarkers->getAppliedThrough(opCtx), topOfOplog);
     }
+    return stableTimestamp;
 } catch (...) {
     LOGV2_FATAL_CONTINUE(21570,
                          "Caught exception during replication recovery: {error}",
@@ -705,6 +703,10 @@ Timestamp ReplicationRecoveryImpl::_applyOplogOperations(OperationContext* opCtx
 
     RecoveryOplogApplierStats stats;
 
+    auto oplogApplicationMode = (recoveryMode == RecoveryMode::kStartupFromStableTimestamp ||
+                                 recoveryMode == RecoveryMode::kRollbackFromStableTimestamp)
+        ? OplogApplication::Mode::kStableRecovering
+        : OplogApplication::Mode::kUnstableRecovering;
     auto writerPool = makeReplWriterPool();
     auto* replCoord = ReplicationCoordinator::get(opCtx);
     OplogApplierImpl oplogApplier(nullptr,
@@ -713,7 +715,7 @@ Timestamp ReplicationRecoveryImpl::_applyOplogOperations(OperationContext* opCtx
                                   replCoord,
                                   _consistencyMarkers,
                                   _storageInterface,
-                                  OplogApplier::Options(OplogApplication::Mode::kRecovering),
+                                  OplogApplier::Options(oplogApplicationMode),
                                   writerPool.get());
 
     OplogApplier::BatchLimits batchLimits;
@@ -820,7 +822,7 @@ void ReplicationRecoveryImpl::_truncateOplogTo(OperationContext* opCtx,
     // Find an oplog entry <= truncateAfterTimestamp.
     boost::optional<BSONObj> truncateAfterOplogEntryBSON =
         _storageInterface->findOplogEntryLessThanOrEqualToTimestamp(
-            opCtx, oplogCollection, truncateAfterTimestamp);
+            opCtx, CollectionPtr(oplogCollection), truncateAfterTimestamp);
     if (!truncateAfterOplogEntryBSON) {
         LOGV2_FATAL_NOTRACE(40296,
                             "Reached end of oplog looking for an oplog entry lte to "
