@@ -87,8 +87,8 @@ BaseCloner::AfterStageBehavior TenantDatabaseCloner::listCollectionsStage() {
     // This will be set after a successful listCollections command.
     _operationTime = Timestamp();
 
-    auto collectionInfos =
-        getClient()->getCollectionInfos(_dbName, ListCollectionsFilter::makeTypeCollectionFilter());
+    auto collectionInfos = getClient()->getCollectionInfos(
+        DatabaseName(boost::none, _dbName), ListCollectionsFilter::makeTypeCollectionFilter());
 
     // Do a majority read on the sync source to make sure the collections listed exist on a majority
     // of nodes in the set. We do not check the rollbackId - rollback would lead to the sync source
@@ -115,7 +115,8 @@ BaseCloner::AfterStageBehavior TenantDatabaseCloner::listCollectionsStage() {
 
     BSONObj readResult;
     BSONObj cmd = ClonerUtils::buildMajorityWaitRequest(_operationTime);
-    getClient()->runCommand("admin", cmd, readResult, QueryOption_SecondaryOk);
+    getClient()->runCommand(
+        DatabaseName(boost::none, "admin"), cmd, readResult, QueryOption_SecondaryOk);
     uassertStatusOKWithContext(
         getStatusFromCommandResult(readResult),
         "TenantDatabaseCloner failed to get listCollections result majority-committed");
@@ -151,7 +152,7 @@ BaseCloner::AfterStageBehavior TenantDatabaseCloner::listCollectionsStage() {
             LOGV2_DEBUG(4881602,
                         1,
                         "Database cloner skipping 'system' collection",
-                        "namespace"_attr = collectionNamespace.ns(),
+                        logAttrs(collectionNamespace),
                         "tenantId"_attr = _tenantId);
             continue;
         }
@@ -197,8 +198,8 @@ BaseCloner::AfterStageBehavior TenantDatabaseCloner::listExistingCollectionsStag
     long long approxTotalDBSizeOnDisk = 0;
 
     std::vector<UUID> clonedCollectionUUIDs;
-    auto collectionInfos =
-        client.getCollectionInfos(_dbName, ListCollectionsFilter::makeTypeCollectionFilter());
+    auto collectionInfos = client.getCollectionInfos(
+        DatabaseName(boost::none, _dbName), ListCollectionsFilter::makeTypeCollectionFilter());
     for (auto&& info : collectionInfos) {
         ListCollectionResult result;
         try {
@@ -218,13 +219,14 @@ BaseCloner::AfterStageBehavior TenantDatabaseCloner::listExistingCollectionsStag
                         "Tenant database cloner skipping 'system' collection",
                         "migrationId"_attr = getSharedData()->getMigrationId(),
                         "tenantId"_attr = _tenantId,
-                        "namespace"_attr = collectionNamespace.ns());
+                        logAttrs(collectionNamespace));
             continue;
         }
         clonedCollectionUUIDs.emplace_back(result.getInfo().getUuid());
 
         BSONObj res;
-        client.runCommand(_dbName, BSON("collStats" << result.getName()), res);
+        client.runCommand(
+            DatabaseName(boost::none, _dbName), BSON("collStats" << result.getName()), res);
         if (auto status = getStatusFromCommandResult(res); !status.isOK()) {
             LOGV2_WARNING(5522901,
                           "Skipping recording of data size metrics for database due to failure "
@@ -302,7 +304,7 @@ void TenantDatabaseCloner::postStage() {
         _stats.collectionStats.reserve(_collections.size());
         for (const auto& coll : _collections) {
             _stats.collectionStats.emplace_back();
-            _stats.collectionStats.back().ns = coll.first.ns();
+            _stats.collectionStats.back().ns = coll.first.ns().toString();
         }
     }
     for (const auto& coll : _collections) {
@@ -325,12 +327,12 @@ void TenantDatabaseCloner::postStage() {
             LOGV2_DEBUG(4881600,
                         1,
                         "Tenant collection clone finished",
-                        "namespace"_attr = sourceNss,
+                        logAttrs(sourceNss),
                         "tenantId"_attr = _tenantId);
         } else {
             LOGV2_ERROR(4881601,
                         "Tenant collection clone failed",
-                        "namespace"_attr = sourceNss,
+                        logAttrs(sourceNss),
                         "error"_attr = collStatus.toString(),
                         "tenantId"_attr = _tenantId);
             auto message = collStatus.withContext(str::stream() << "Error cloning collection '"
