@@ -38,21 +38,24 @@
 
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj_comparator_interface.h"
+#include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/db/s/sharding_config_server_parameters_gen.h"
 #include "mongo/db/s/sharding_util.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/bits.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog_cache.h"
+#include "mongo/s/chunk_manager.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/request_types/get_stats_for_balancing_gen.h"
 #include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/util/str.h"
+#include "mongo/util/timer.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
+MONGO_FAIL_POINT_DEFINE(overrideStatsForBalancingBatchSize);
 
 namespace mongo {
 
@@ -67,39 +70,14 @@ StatusWith<DistributionStatus> createCollectionDistributionStatus(
     const NamespaceString& nss,
     const ShardStatisticsVector& allShards,
     const ChunkManager& chunkMgr) {
-    ShardToChunksMap shardToChunksMap;
 
-    // Makes sure there is an entry in shardToChunksMap for every shard, so empty shards will also
-    // be accounted for
-    for (const auto& stat : allShards) {
-        shardToChunksMap[stat.shardId];
-    }
-
-    chunkMgr.forEachChunk([&](const auto& chunkEntry) {
-        ChunkType chunk;
-        chunk.setCollectionUUID(chunkMgr.getUUID());
-        chunk.setMin(chunkEntry.getMin());
-        chunk.setMax(chunkEntry.getMax());
-        chunk.setJumbo(chunkEntry.isJumbo());
-        chunk.setShard(chunkEntry.getShardId());
-        chunk.setVersion(chunkEntry.getLastmod());
-
-        shardToChunksMap[chunkEntry.getShardId()].push_back(chunk);
-
-        return true;
-    });
-
-    const auto& keyPattern = chunkMgr.getShardKeyPattern().getKeyPattern();
-
-    // Cache the collection zones
-    auto swZoneInfo = ZoneInfo::getZonesForCollection(opCtx, nss, keyPattern);
+    auto swZoneInfo =
+        createCollectionZoneInfo(opCtx, nss, chunkMgr.getShardKeyPattern().getKeyPattern());
     if (!swZoneInfo.isOK()) {
         return swZoneInfo.getStatus();
     }
 
-    DistributionStatus distribution(nss, std::move(shardToChunksMap), swZoneInfo.getValue());
-
-    return {std::move(distribution)};
+    return {DistributionStatus{nss, std::move(swZoneInfo.getValue()), chunkMgr}};
 }
 
 stdx::unordered_map<NamespaceString, CollectionDataSizeInfoForBalancing>
@@ -136,7 +114,7 @@ getDataSizeInfoForCollections(OperationContext* opCtx,
     const auto executor = Grid::get(opCtx)->getExecutorPool()->getFixedExecutor();
     const auto responsesFromShards =
         sharding_util::sendCommandToShards(opCtx,
-                                           NamespaceString::kAdminDb.toString(),
+                                           DatabaseName::kAdmin.toString(),
                                            reqObj,
                                            shardIds,
                                            executor,
@@ -175,7 +153,8 @@ getDataSizeInfoForCollections(OperationContext* opCtx,
 
 CollectionDataSizeInfoForBalancing getDataSizeInfoForCollection(OperationContext* opCtx,
                                                                 const NamespaceString& nss) {
-    const auto coll = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
+    const auto catalogClient = ShardingCatalogManager::get(opCtx)->localCatalogClient();
+    const auto coll = catalogClient->getCollection(opCtx, nss);
     std::vector<CollectionType> vec{coll};
     return std::move(getDataSizeInfoForCollections(opCtx, vec).at(nss));
 }
@@ -191,9 +170,9 @@ class SplitCandidatesBuffer {
     SplitCandidatesBuffer& operator=(const SplitCandidatesBuffer&) = delete;
 
 public:
-    SplitCandidatesBuffer(NamespaceString nss, ChunkVersion collectionVersion)
+    SplitCandidatesBuffer(NamespaceString nss, ChunkVersion collectionPlacementVersion)
         : _nss(std::move(nss)),
-          _collectionVersion(collectionVersion),
+          _collectionPlacementVersion(collectionPlacementVersion),
           _chunkSplitPoints(SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<SplitInfo>()) {
     }
 
@@ -207,7 +186,7 @@ public:
             _chunkSplitPoints.emplace(chunk.getMin(),
                                       SplitInfo(chunk.getShardId(),
                                                 _nss,
-                                                _collectionVersion,
+                                                _collectionPlacementVersion,
                                                 chunk.getLastmod(),
                                                 chunk.getMin(),
                                                 chunk.getMax(),
@@ -234,9 +213,9 @@ public:
     }
 
 private:
-    // Namespace and expected collection version
+    // Namespace and expected collection placement version
     const NamespaceString _nss;
-    const ChunkVersion _collectionVersion;
+    const ChunkVersion _collectionPlacementVersion;
 
     // Chunk min key and split vector associated with that chunk
     BSONObjIndexedMap<SplitInfo> _chunkSplitPoints;
@@ -247,12 +226,12 @@ private:
  * range boundaries.
  */
 void getSplitCandidatesToEnforceZoneRanges(const ChunkManager& cm,
-                                           const DistributionStatus& distribution,
+                                           const ZoneInfo& zoneInfo,
                                            SplitCandidatesBuffer* splitCandidates) {
     const auto& globalMax = cm.getShardKeyPattern().getKeyPattern().globalMax();
 
     // For each zone range, find chunks that need to be split.
-    for (const auto& zoneRangeEntry : distribution.zoneRanges()) {
+    for (const auto& zoneRangeEntry : zoneInfo.zoneRanges()) {
         const auto& zoneRange = zoneRangeEntry.second;
 
         const auto chunkAtZoneMin = cm.findIntersectingChunkWithSimpleCollation(zoneRange.min);
@@ -277,65 +256,10 @@ void getSplitCandidatesToEnforceZoneRanges(const ChunkManager& cm,
     }
 }
 
-/**
- * If the number of chunks as given by the ChunkManager is less than the configured minimum
- * number of chunks for the sessions collection (minNumChunksForSessionsCollection), calculates
- * split points that evenly partition the key space into N ranges (where N is
- * minNumChunksForSessionsCollection rounded up to the next power of 2), and populates
- * splitCandidates with chunk and splitPoint pairs for chunks that need to split.
- */
-void getSplitCandidatesForSessionsCollection(OperationContext* opCtx,
-                                             const ChunkManager& cm,
-                                             SplitCandidatesBuffer* splitCandidates) {
-    const auto minNumChunks = minNumChunksForSessionsCollection.load();
-
-    if (cm.numChunks() >= minNumChunks) {
-        return;
-    }
-
-    // Use the next power of 2 as the target number of chunks.
-    const size_t numBits = 64 - countLeadingZeros64(minNumChunks - 1);
-    const uint32_t numChunks = 1 << numBits;
-
-    // Compute split points for _id.id that partition the UUID 128-bit data space into numChunks
-    // equal ranges. Since the numChunks is a power of 2, the split points are the permutations
-    // of the prefix numBits right-padded with 0's.
-    std::vector<BSONObj> splitPoints;
-    for (uint32_t i = 1; i < numChunks; i++) {
-        // Start with a buffer of 0's.
-        std::array<uint8_t, 16> buf{0b0};
-
-        // Left-shift i to fill the remaining bits in the prefix 32 bits with 0's.
-        const uint32_t high = i << (CHAR_BIT * 4 - numBits);
-
-        // Fill the prefix 4 bytes with high's bytes.
-        buf[0] = static_cast<uint8_t>(high >> CHAR_BIT * 3);
-        buf[1] = static_cast<uint8_t>(high >> CHAR_BIT * 2);
-        buf[2] = static_cast<uint8_t>(high >> CHAR_BIT * 1);
-        buf[3] = static_cast<uint8_t>(high);
-
-        ConstDataRange cdr(buf.data(), sizeof(buf));
-        splitPoints.push_back(BSON("_id" << BSON("id" << UUID::fromCDR(cdr))));
-    }
-
-    // For each split point, find a chunk that needs to be split.
-    for (auto& splitPoint : splitPoints) {
-        const auto chunkAtSplitPoint = cm.findIntersectingChunkWithSimpleCollation(splitPoint);
-        invariant(chunkAtSplitPoint.getMax().woCompare(splitPoint) > 0);
-
-        if (chunkAtSplitPoint.getMin().woCompare(splitPoint)) {
-            splitCandidates->addSplitPoint(chunkAtSplitPoint, splitPoint);
-        }
-    }
-
-    return;
-}
-
 }  // namespace
 
-BalancerChunkSelectionPolicyImpl::BalancerChunkSelectionPolicyImpl(ClusterStatistics* clusterStats,
-                                                                   BalancerRandomSource& random)
-    : _clusterStats(clusterStats), _random(random) {}
+BalancerChunkSelectionPolicyImpl::BalancerChunkSelectionPolicyImpl(ClusterStatistics* clusterStats)
+    : _clusterStats(clusterStats) {}
 
 BalancerChunkSelectionPolicyImpl::~BalancerChunkSelectionPolicyImpl() = default;
 
@@ -348,14 +272,16 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
 
     const auto& shardStats = shardStatsStatus.getValue();
 
-    auto collections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, {});
+    const auto catalogClient = ShardingCatalogManager::get(opCtx)->localCatalogClient();
+    auto collections = catalogClient->getCollections(opCtx, {});
     if (collections.empty()) {
         return SplitInfoVector{};
     }
 
     SplitInfoVector splitCandidates;
 
-    std::shuffle(collections.begin(), collections.end(), _random);
+    auto client = opCtx->getClient();
+    std::shuffle(collections.begin(), collections.end(), client->getPrng().urbg());
 
     for (const auto& coll : collections) {
         const NamespaceString& nss(coll.getNss());
@@ -365,19 +291,11 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
             // Namespace got dropped before we managed to get to it, so just skip it
             continue;
         } else if (!candidatesStatus.isOK()) {
-            if (nss == NamespaceString::kLogicalSessionsNamespace) {
-                LOGV2_WARNING(4562402,
-                              "Unable to split sessions collection chunks",
-                              "error"_attr = candidatesStatus.getStatus());
-
-            } else {
-                LOGV2_WARNING(
-                    21852,
-                    "Unable to enforce zone range policy for collection {namespace}: {error}",
-                    "Unable to enforce zone range policy for collection",
-                    "namespace"_attr = nss.ns(),
-                    "error"_attr = candidatesStatus.getStatus());
-            }
+            LOGV2_WARNING(21852,
+                          "Unable to enforce zone range policy for collection {namespace}: {error}",
+                          "Unable to enforce zone range policy for collection",
+                          logAttrs(nss),
+                          "error"_attr = candidatesStatus.getStatus());
 
             continue;
         }
@@ -404,86 +322,176 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
 }
 
 StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMove(
-    OperationContext* opCtx, stdx::unordered_set<ShardId>* usedShards) {
-    auto shardStatsStatus = _clusterStats->getStats(opCtx);
-    if (!shardStatsStatus.isOK()) {
-        return shardStatsStatus.getStatus();
-    }
+    OperationContext* opCtx,
+    const std::vector<ClusterStatistics::ShardStatistics>& shardStats,
+    stdx::unordered_set<ShardId>* availableShards,
+    stdx::unordered_set<NamespaceString>* imbalancedCollectionsCachePtr) {
+    invariant(availableShards);
+    invariant(imbalancedCollectionsCachePtr);
 
-    const auto& shardStats = shardStatsStatus.getValue();
-
-    if (shardStats.size() < 2) {
+    if (availableShards->size() < 2) {
         return MigrateInfoVector{};
     }
 
-    auto collections = Grid::get(opCtx)->catalogClient()->getCollections(opCtx, {});
+    Timer chunksSelectionTimer;
+
+    const auto catalogClient = ShardingCatalogManager::get(opCtx)->localCatalogClient();
+    auto collections = catalogClient->getCollections(opCtx,
+                                                     {},
+                                                     repl::ReadConcernLevel::kMajorityReadConcern,
+                                                     BSON(CollectionType::kNssFieldName << 1));
     if (collections.empty()) {
         return MigrateInfoVector{};
     }
 
     MigrateInfoVector candidateChunks;
 
-    std::shuffle(collections.begin(), collections.end(), _random);
+    const uint32_t kStatsForBalancingBatchSize = [&]() {
+        auto batchSize = 100U;
+        overrideStatsForBalancingBatchSize.execute([&batchSize](const BSONObj& data) {
+            batchSize = data["size"].numberInt();
+            LOGV2(7617200, "Overriding collections batch size", "size"_attr = batchSize);
+        });
+        return batchSize;
+    }();
 
-    static constexpr auto kStatsForBalancingBatchSize = 20;
+    const uint32_t kMaxCachedCollectionsSize = 0.75 * kStatsForBalancingBatchSize;
 
-    std::vector<CollectionType> collBatch;
-    for (auto collIt = collections.begin(); collIt != collections.end();) {
-        const auto& coll = *(collIt++);
+    // Lambda function used to get a CollectionType leveraging the `collections` vector
+    // The `collections` vector must be sorted by nss when it is called
+    auto getCollectionTypeByNss = [&collections](const NamespaceString& nss)
+        -> std::pair<boost::optional<CollectionType>, std::vector<CollectionType>::iterator> {
+        // Using a lower_bound to perform a binary search on the `collections` vector
+        const auto collIt =
+            std::lower_bound(collections.begin(),
+                             collections.end(),
+                             nss,
+                             [](const CollectionType& coll, const NamespaceString& ns) {
+                                 return coll.getNss() < ns;
+                             });
+
+        if (collIt == collections.end() || collIt->getNss() != nss) {
+            return std::make_pair(boost::none, collections.end());
+        }
+        return std::make_pair(*collIt, collIt);
+    };
+
+    // Lambda function to check if a collection is explicitly disabled for balancing
+    const auto canBalanceCollection = [](const CollectionType& coll) -> bool {
         if (!coll.getAllowBalance() || !coll.getAllowMigrations() || !coll.getPermitMigrations() ||
             coll.getDefragmentCollection()) {
             LOGV2_DEBUG(5966401,
                         1,
                         "Not balancing explicitly disabled collection",
-                        "namespace"_attr = coll.getNss(),
+                        logAttrs(coll.getNss()),
                         "allowBalance"_attr = coll.getAllowBalance(),
                         "allowMigrations"_attr = coll.getAllowMigrations(),
                         "permitMigrations"_attr = coll.getPermitMigrations(),
                         "defragmentCollection"_attr = coll.getDefragmentCollection());
-        } else {
-            collBatch.push_back(coll);
+            return false;
         }
+        return true;
+    };
 
-        if (collBatch.size() < kStatsForBalancingBatchSize && collIt != collections.end()) {
-            // keep Accumulating in the batch
-            continue;
-        }
+    // Lambda function to select migrate candidates from a batch of collections
+    const auto processBatch = [&](std::vector<CollectionType>& collBatch) {
+        const auto collsDataSizeInfo = getDataSizeInfoForCollections(opCtx, collBatch);
 
-        boost::optional<stdx::unordered_map<NamespaceString, CollectionDataSizeInfoForBalancing>>
-            collsDataSizeInfo;
-        if (feature_flags::gBalanceAccordingToDataSize.isEnabled(
-                serverGlobalParams.featureCompatibility)) {
-            collsDataSizeInfo.emplace(getDataSizeInfoForCollections(opCtx, collBatch));
-        }
+        auto client = opCtx->getClient();
+        std::shuffle(collBatch.begin(), collBatch.end(), client->getPrng().urbg());
+        for (const auto& coll : collBatch) {
 
-        for (const auto& collFromBatch : collBatch) {
-            const auto& nss = collFromBatch.getNss();
-
-            boost::optional<CollectionDataSizeInfoForBalancing> optDataSizeInfo;
-            if (collsDataSizeInfo.has_value()) {
-                optDataSizeInfo.emplace(std::move(collsDataSizeInfo->at(nss)));
+            if (availableShards->size() < 2) {
+                break;
             }
 
-            auto candidatesStatus = _getMigrateCandidatesForCollection(
-                opCtx, nss, shardStats, optDataSizeInfo, usedShards);
-            if (candidatesStatus == ErrorCodes::NamespaceNotFound) {
+            const auto& nss = coll.getNss();
+
+            auto swMigrateCandidates = _getMigrateCandidatesForCollection(
+                opCtx, nss, shardStats, collsDataSizeInfo.at(nss), availableShards);
+            if (swMigrateCandidates == ErrorCodes::NamespaceNotFound) {
                 // Namespace got dropped before we managed to get to it, so just skip it
+                imbalancedCollectionsCachePtr->erase(nss);
                 continue;
-            } else if (!candidatesStatus.isOK()) {
+            } else if (!swMigrateCandidates.isOK()) {
                 LOGV2_WARNING(21853,
                               "Unable to balance collection",
-                              "namespace"_attr = nss.ns(),
-                              "error"_attr = candidatesStatus.getStatus());
+                              logAttrs(nss),
+                              "error"_attr = swMigrateCandidates.getStatus());
                 continue;
             }
 
             candidateChunks.insert(
                 candidateChunks.end(),
-                std::make_move_iterator(candidatesStatus.getValue().first.begin()),
-                std::make_move_iterator(candidatesStatus.getValue().first.end()));
+                std::make_move_iterator(swMigrateCandidates.getValue().first.begin()),
+                std::make_move_iterator(swMigrateCandidates.getValue().first.end()));
+
+            const auto& migrateCandidates = swMigrateCandidates.getValue().first;
+            if (migrateCandidates.empty()) {
+                imbalancedCollectionsCachePtr->erase(nss);
+            } else if (imbalancedCollectionsCachePtr->size() < kMaxCachedCollectionsSize) {
+                imbalancedCollectionsCachePtr->insert(nss);
+            }
+        }
+    };
+
+    // To assess if a collection has chunks to migrate, we need to ask shards the size of that
+    // collection. For efficiency, we ask for a batch of collections per every shard request instead
+    // of a single request per collection
+    std::vector<CollectionType> collBatch;
+
+    // The first batch is partially filled by the imbalanced cached collections
+    for (auto imbalancedNssIt = imbalancedCollectionsCachePtr->begin();
+         imbalancedNssIt != imbalancedCollectionsCachePtr->end();) {
+
+        const auto& [imbalancedColl, collIt] = getCollectionTypeByNss(*imbalancedNssIt);
+
+        if (!imbalancedColl.has_value() || !canBalanceCollection(imbalancedColl.value())) {
+            // The collection was dropped or is no longer enabled for balancing.
+            imbalancedCollectionsCachePtr->erase(imbalancedNssIt++);
+            continue;
         }
 
-        collBatch.clear();
+        collBatch.push_back(imbalancedColl.value());
+        ++imbalancedNssIt;
+
+        // Remove the collection from the whole list of collections to avoid processing it twice
+        collections.erase(collIt);
+    }
+
+    // Iterate all the remaining collections randomly
+    auto client = opCtx->getClient();
+    std::shuffle(collections.begin(), collections.end(), client->getPrng().urbg());
+    for (const auto& coll : collections) {
+
+        if (canBalanceCollection(coll)) {
+            collBatch.push_back(coll);
+        }
+
+        if (collBatch.size() == kStatsForBalancingBatchSize) {
+            processBatch(collBatch);
+            if (availableShards->size() < 2) {
+                return candidateChunks;
+            }
+            collBatch.clear();
+        }
+
+        const auto maxTimeMs = balancerChunksSelectionTimeoutMs.load();
+        if (candidateChunks.size() > 0 && chunksSelectionTimer.millis() > maxTimeMs) {
+            LOGV2_DEBUG(
+                7100900,
+                1,
+                "Exceeded max time while searching for candidate chunks to migrate in this round.",
+                "maxTime"_attr = Milliseconds(maxTimeMs),
+                "chunksSelectionTime"_attr = chunksSelectionTimer.elapsed(),
+                "numCandidateChunks"_attr = candidateChunks.size());
+
+            return candidateChunks;
+        }
+    }
+
+    if (collBatch.size() > 0) {
+        processBatch(collBatch);
     }
 
     return candidateChunks;
@@ -500,18 +508,21 @@ StatusWith<MigrateInfosWithReason> BalancerChunkSelectionPolicyImpl::selectChunk
 
     // Used to check locally if the collection exists, it should trow NamespaceNotFound if it
     // doesn't.
-    Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
+    ShardingCatalogManager::get(opCtx)->localCatalogClient()->getCollection(opCtx, nss);
 
-    stdx::unordered_set<ShardId> usedShards;
+    stdx::unordered_set<ShardId> availableShards;
+    std::transform(shardStats.begin(),
+                   shardStats.end(),
+                   std::inserter(availableShards, availableShards.end()),
+                   [](const ClusterStatistics::ShardStatistics& shardStatistics) -> ShardId {
+                       return shardStatistics.shardId;
+                   });
 
-    boost::optional<CollectionDataSizeInfoForBalancing> optCollDataSizeInfo;
-    if (feature_flags::gBalanceAccordingToDataSize.isEnabled(
-            serverGlobalParams.featureCompatibility)) {
-        optCollDataSizeInfo.emplace(getDataSizeInfoForCollection(opCtx, nss));
-    }
 
-    auto candidatesStatus = _getMigrateCandidatesForCollection(
-        opCtx, nss, shardStats, optCollDataSizeInfo, &usedShards);
+    const auto dataSizeInfo = getDataSizeInfoForCollection(opCtx, nss);
+
+    auto candidatesStatus =
+        _getMigrateCandidatesForCollection(opCtx, nss, shardStats, dataSizeInfo, &availableShards);
     if (!candidatesStatus.isOK()) {
         return candidatesStatus.getStatus();
     }
@@ -531,12 +542,13 @@ BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* op
     const auto& shardStats = shardStatsStatus.getValue();
 
     auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithPlacementRefresh(opCtx,
+                                                                                              nss);
     if (!routingInfoStatus.isOK()) {
         return routingInfoStatus.getStatus();
     }
 
-    const auto& cm = routingInfoStatus.getValue();
+    const auto& [cm, _] = routingInfoStatus.getValue();
 
     const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
     if (!collInfoStatus.isOK()) {
@@ -545,85 +557,39 @@ BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* op
 
     const DistributionStatus& distribution = collInfoStatus.getValue();
 
-    return BalancerPolicy::balanceSingleChunk(chunk, shardStats, distribution);
-}
+    const auto dataSizeInfo = getDataSizeInfoForCollection(opCtx, nss);
 
-Status BalancerChunkSelectionPolicyImpl::checkMoveAllowed(OperationContext* opCtx,
-                                                          const ChunkType& chunk,
-                                                          const ShardId& newShardId) {
-    auto shardStatsStatus = _clusterStats->getStats(opCtx);
-    if (!shardStatsStatus.isOK()) {
-        return shardStatsStatus.getStatus();
-    }
-
-    const CollectionType collection = Grid::get(opCtx)->catalogClient()->getCollection(
-        opCtx, chunk.getCollectionUUID(), repl::ReadConcernLevel::kLocalReadConcern);
-    const auto& nss = collection.getNss();
-
-
-    auto shardStats = std::move(shardStatsStatus.getValue());
-
-    auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
-    if (!routingInfoStatus.isOK()) {
-        return routingInfoStatus.getStatus();
-    }
-
-    const auto& cm = routingInfoStatus.getValue();
-
-    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
-    if (!collInfoStatus.isOK()) {
-        return collInfoStatus.getStatus();
-    }
-
-    const DistributionStatus& distribution = collInfoStatus.getValue();
-
-    auto newShardIterator =
-        std::find_if(shardStats.begin(),
-                     shardStats.end(),
-                     [&newShardId](const ClusterStatistics::ShardStatistics& stat) {
-                         return stat.shardId == newShardId;
-                     });
-    if (newShardIterator == shardStats.end()) {
-        return {ErrorCodes::ShardNotFound,
-                str::stream() << "Unable to find constraints information for shard " << newShardId
-                              << ". Move to this shard will be disallowed."};
-    }
-
-    return BalancerPolicy::isShardSuitableReceiver(*newShardIterator,
-                                                   distribution.getZoneForChunk(chunk));
+    return BalancerPolicy::balanceSingleChunk(chunk, shardStats, distribution, dataSizeInfo);
 }
 
 StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::_getSplitCandidatesForCollection(
     OperationContext* opCtx, const NamespaceString& nss, const ShardStatisticsVector& shardStats) {
     auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithPlacementRefresh(opCtx,
+                                                                                              nss);
     if (!routingInfoStatus.isOK()) {
         return routingInfoStatus.getStatus();
     }
 
-    const auto& cm = routingInfoStatus.getValue();
+    const auto& [cm, _] = routingInfoStatus.getValue();
 
-    const auto collInfoStatus = createCollectionDistributionStatus(opCtx, nss, shardStats, cm);
-    if (!collInfoStatus.isOK()) {
-        return collInfoStatus.getStatus();
+    const auto swCollZoneInfo =
+        createCollectionZoneInfo(opCtx, nss, cm.getShardKeyPattern().getKeyPattern());
+    if (!swCollZoneInfo.isOK()) {
+        return swCollZoneInfo.getStatus();
     }
-
-    const DistributionStatus& distribution = collInfoStatus.getValue();
+    const auto& collZoneInfo = swCollZoneInfo.getValue();
 
     // Accumulate split points for the same chunk together
     SplitCandidatesBuffer splitCandidates(nss, cm.getVersion());
 
-    if (nss == NamespaceString::kLogicalSessionsNamespace) {
-        if (!distribution.zones().empty()) {
-            LOGV2_WARNING(4562401,
-                          "Ignoring zones for the sessions collection",
-                          "zones"_attr = distribution.zones());
-        }
-
-        getSplitCandidatesForSessionsCollection(opCtx, cm, &splitCandidates);
+    if (nss == NamespaceString::kLogicalSessionsNamespace && !collZoneInfo.allZones().empty()) {
+        LOGV2_WARNING(4562401,
+                      "Ignoring zones for the internal sessions collection.",
+                      "nss"_attr = NamespaceString::kLogicalSessionsNamespace,
+                      "zones"_attr = collZoneInfo.allZones());
     } else {
-        getSplitCandidatesToEnforceZoneRanges(cm, distribution, &splitCandidates);
+        getSplitCandidatesToEnforceZoneRanges(cm, collZoneInfo, &splitCandidates);
     }
 
     return splitCandidates.done();
@@ -634,15 +600,16 @@ BalancerChunkSelectionPolicyImpl::_getMigrateCandidatesForCollection(
     OperationContext* opCtx,
     const NamespaceString& nss,
     const ShardStatisticsVector& shardStats,
-    const boost::optional<CollectionDataSizeInfoForBalancing>& collDataSizeInfo,
-    stdx::unordered_set<ShardId>* usedShards) {
+    const CollectionDataSizeInfoForBalancing& collDataSizeInfo,
+    stdx::unordered_set<ShardId>* availableShards) {
     auto routingInfoStatus =
-        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
+        Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithPlacementRefresh(opCtx,
+                                                                                              nss);
     if (!routingInfoStatus.isOK()) {
         return routingInfoStatus.getStatus();
     }
 
-    const auto& cm = routingInfoStatus.getValue();
+    const auto& [cm, _] = routingInfoStatus.getValue();
 
     const auto& shardKeyPattern = cm.getShardKeyPattern().getKeyPattern();
 
@@ -653,7 +620,7 @@ BalancerChunkSelectionPolicyImpl::_getMigrateCandidatesForCollection(
 
     const DistributionStatus& distribution = collInfoStatus.getValue();
 
-    for (const auto& zoneRangeEntry : distribution.zoneRanges()) {
+    for (const auto& zoneRangeEntry : distribution.getZoneInfo().zoneRanges()) {
         const auto& zoneRange = zoneRangeEntry.second;
 
         const auto chunkAtZoneMin = cm.findIntersectingChunkWithSimpleCollation(zoneRange.min);
@@ -692,7 +659,7 @@ BalancerChunkSelectionPolicyImpl::_getMigrateCandidatesForCollection(
         shardStats,
         distribution,
         collDataSizeInfo,
-        usedShards,
+        availableShards,
         Grid::get(opCtx)->getBalancerConfiguration()->attemptToBalanceJumboChunks());
 }
 
