@@ -66,7 +66,7 @@ public:
             uassert(ErrorCodes::IllegalOperation,
                     str::stream() << ConfigsvrBalancerCollectionStatus::kCommandName
                                   << " can only be run on config servers",
-                    serverGlobalParams.clusterRole == ClusterRole::ConfigServer);
+                    serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer));
 
             const NamespaceString& nss = ns();
 
