@@ -40,6 +40,7 @@
 #include "mongo/db/s/transaction_coordinator_service.h"
 #include "mongo/db/session/logical_session_cache_noop.h"
 #include "mongo/db/session/session_catalog_mongod.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_shard.h"
@@ -58,8 +59,8 @@ protected:
 
         ConfigServerTestFixture::setUp();
         DBDirectClient client(operationContext());
-        client.createCollection(NamespaceString::kSessionTransactionsTableNamespace.ns());
-        client.createIndexes(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+        client.createCollection(NamespaceString::kSessionTransactionsTableNamespace);
+        client.createIndexes(NamespaceString::kSessionTransactionsTableNamespace,
                              {MongoDSessionCatalog::getConfigTxnPartialIndexSpec()});
 
         ReadWriteConcernDefaults::create(getServiceContext(), _lookupMock.getFetchDefaultsFn());
@@ -107,7 +108,9 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
         migratedChunk.setCollectionUUID(collUUID);
         migratedChunk.setVersion(origVersion);
         migratedChunk.setShard(shard0.getName());
-        migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+        migratedChunk.setOnCurrentShardSince(Timestamp(100, 0));
+        migratedChunk.setHistory(
+            {ChunkHistory(*migratedChunk.getOnCurrentShardSince(), shard0.getName())});
         migratedChunk.setMin(BSON("a" << 1));
         migratedChunk.setMax(BSON("a" << 10));
 
@@ -117,7 +120,9 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
         controlChunk.setCollectionUUID(collUUID);
         controlChunk.setVersion(origVersion);
         controlChunk.setShard(shard0.getName());
-        controlChunk.setHistory({ChunkHistory(Timestamp(50, 0), shard0.getName())});
+        controlChunk.setOnCurrentShardSince(Timestamp(50, 0));
+        controlChunk.setHistory(
+            {ChunkHistory(*controlChunk.getOnCurrentShardSince(), shard0.getName())});
         controlChunk.setMin(BSON("a" << 10));
         controlChunk.setMax(BSON("a" << 20));
         controlChunk.setJumbo(true);
@@ -125,26 +130,27 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
 
     setupCollection(kNamespace, kKeyPattern, {migratedChunk, controlChunk});
 
-    Timestamp validAfter{101, 0};
-    BSONObj versions = assertGet(ShardingCatalogManager::get(operationContext())
-                                     ->commitChunkMigration(operationContext(),
-                                                            kNamespace,
-                                                            migratedChunk,
-                                                            migratedChunk.getVersion().epoch(),
-                                                            collTimestamp,
-                                                            ShardId(shard0.getName()),
-                                                            ShardId(shard1.getName()),
-                                                            validAfter));
+    const auto currentTime = VectorClock::get(getServiceContext())->getTime();
+    const auto expectedValidAfter = currentTime.clusterTime().asTimestamp();
+
+    auto versions = assertGet(ShardingCatalogManager::get(operationContext())
+                                  ->commitChunkMigration(operationContext(),
+                                                         kNamespace,
+                                                         migratedChunk,
+                                                         migratedChunk.getVersion().epoch(),
+                                                         collTimestamp,
+                                                         ShardId(shard0.getName()),
+                                                         ShardId(shard1.getName())));
 
     // Verify the versions returned match expected values.
-    auto mver = ChunkVersion::parse(versions["shardVersion"]);
+    auto mver = versions.shardPlacementVersion;
     ASSERT_EQ(ChunkVersion(
                   {migratedChunk.getVersion().epoch(), migratedChunk.getVersion().getTimestamp()},
                   {migratedChunk.getVersion().majorVersion() + 1, 1}),
               mver);
 
-    // Verify that a collection version is returned
-    auto cver = ChunkVersion::parse(versions["collectionVersion"]);
+    // Verify that a collection placement version is returned
+    auto cver = versions.collectionPlacementVersion;
     ASSERT_TRUE(mver.isOlderOrEqualThan(cver));
 
     // Verify the chunks ended up in the right shards.
@@ -154,7 +160,8 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
 
     // The migrated chunk's history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
-    ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(expectedValidAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(expectedValidAfter, *chunkDoc0.getOnCurrentShardSince());
 
     auto chunkDoc1 = uassertStatusOK(
         getChunkDoc(operationContext(), controlChunk.getMin(), collEpoch, collTimestamp));
@@ -166,6 +173,8 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
               chunkDoc1.getHistory().front().getValidAfter());
     ASSERT_EQ(controlChunk.getHistory().front().getShard(),
               chunkDoc1.getHistory().front().getShard());
+    ASSERT(chunkDoc1.getOnCurrentShardSince().has_value());
+    ASSERT_EQ(controlChunk.getOnCurrentShardSince(), chunkDoc1.getOnCurrentShardSince());
     ASSERT(chunkDoc1.getJumbo());
 }
 
@@ -192,7 +201,8 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
     chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
-    chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    chunk0.setOnCurrentShardSince(Timestamp(100, 0));
+    chunk0.setHistory({ChunkHistory(*chunk0.getOnCurrentShardSince(), shard0.getName())});
 
     // apportion
     auto chunkMin = BSON("a" << 1);
@@ -201,28 +211,28 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
     chunk0.setMax(chunkMax);
 
     setupCollection(kNamespace, kKeyPattern, {chunk0});
+    const auto currentTime = VectorClock::get(getServiceContext())->getTime();
+    const auto expectedValidAfter = currentTime.clusterTime().asTimestamp();
 
-    Timestamp validAfter{101, 0};
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   origVersion.epoch(),
+                                   collTimestamp,
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
 
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                origVersion.epoch(),
-                                                                collTimestamp,
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
-
-    ASSERT_OK(resultBSON.getStatus());
+    ASSERT_OK(result.getStatus());
 
     // Verify the version returned matches expected value.
-    BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parse(versions["shardVersion"]);
+    auto versions = result.getValue();
+    auto mver = versions.shardPlacementVersion;
     ASSERT_EQ(ChunkVersion({origVersion.epoch(), origVersion.getTimestamp()}, {0, 0}), mver);
 
-    // Verify that a collection version is returned
-    auto cver = ChunkVersion::parse(versions["collectionVersion"]);
+    // Verify that a collection placement version is returned
+    auto cver = versions.collectionPlacementVersion;
     ASSERT_EQ(ChunkVersion({collEpoch, collTimestamp}, {origMajorVersion + 1, 0}), cver);
 
     // Verify the chunk ended up in the right shard.
@@ -231,7 +241,8 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
     ASSERT_EQ("shard1", chunkDoc0.getShard().toString());
     // The history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
-    ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(expectedValidAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(expectedValidAfter, *chunkDoc0.getOnCurrentShardSince());
 }
 
 TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
@@ -257,7 +268,8 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
     chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
-    chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    chunk0.setOnCurrentShardSince(Timestamp(100, 0));
+    chunk0.setHistory({ChunkHistory(*chunk0.getOnCurrentShardSince(), shard0.getName())});
 
     // apportion
     auto chunkMin = BSON("a" << 1);
@@ -268,23 +280,26 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
     setupCollection(kNamespace, kKeyPattern, {chunk0});
 
     // Make the time distance between the last history element large enough.
-    Timestamp validAfter{200, 0};
+    const auto currentTime = VectorClock::get(getServiceContext())->getTime();
+    const auto currentClusterTime = currentTime.clusterTime().asTimestamp();
+    const auto updatedClusterTime = LogicalTime(currentClusterTime + Timestamp(200.0).asULL());
+    VectorClock::get(getServiceContext())->advanceClusterTime_forTest(updatedClusterTime);
 
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                origVersion.epoch(),
-                                                                collTimestamp,
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   origVersion.epoch(),
+                                   collTimestamp,
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
 
-    ASSERT_OK(resultBSON.getStatus());
+    ASSERT_OK(result.getStatus());
 
     // Verify the version returned matches expected value.
-    BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parse(versions["shardVersion"]);
+    auto versions = result.getValue();
+    auto mver = versions.shardPlacementVersion;
     ASSERT_EQ(ChunkVersion({origVersion.epoch(), origVersion.getTimestamp()}, {0, 0}), mver);
 
     // Verify the chunk ended up in the right shard.
@@ -294,7 +309,8 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
 
     // The new history entry should be added, but the old one preserved.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
-    ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(updatedClusterTime.asTimestamp(), chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(updatedClusterTime.asTimestamp(), *chunkDoc0.getOnCurrentShardSince());
 }
 
 TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
@@ -318,7 +334,8 @@ TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
     chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
-    chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    chunk0.setOnCurrentShardSince(Timestamp(100, 1));
+    chunk0.setHistory({ChunkHistory(*chunk0.getOnCurrentShardSince(), shard0.getName())});
 
     // apportion
     auto chunkMin = BSON("a" << 1);
@@ -328,20 +345,21 @@ TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
 
     setupCollection(kNamespace, kKeyPattern, {chunk0});
 
-    // Make the time before the last change to trigger the failure.
-    Timestamp validAfter{99, 0};
+    // Ensure that the current cluster time is earlier than the timestamp associated to the chunk
+    // being migrated.
+    VectorClock::get(getServiceContext())->resetVectorClock_forTest();
 
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                origVersion.epoch(),
-                                                                origVersion.getTimestamp(),
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   origVersion.epoch(),
+                                   origVersion.getTimestamp(),
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
 
-    ASSERT_EQ(ErrorCodes::IncompatibleShardingMetadata, resultBSON.getStatus());
+    ASSERT_EQ(ErrorCodes::IncompatibleShardingMetadata, result.getStatus());
 }
 
 TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
@@ -384,19 +402,17 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
 
     setupCollection(kNamespace, kKeyPattern, {chunk0, chunk1});
 
-    Timestamp validAfter{1};
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   OID::gen(),
+                                   Timestamp(52),
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
 
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                OID::gen(),
-                                                                Timestamp(52),
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
-
-    ASSERT_EQ(ErrorCodes::StaleEpoch, resultBSON.getStatus());
+    ASSERT_EQ(ErrorCodes::StaleEpoch, result.getStatus());
 }
 
 TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
@@ -441,19 +457,17 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
     // get version from the control chunk this time
     setupCollection(kNamespace, kKeyPattern, {chunk1, chunk0});
 
-    Timestamp validAfter{1};
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   origVersion.epoch(),
+                                   origVersion.getTimestamp(),
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
 
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                origVersion.epoch(),
-                                                                origVersion.getTimestamp(),
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
-
-    ASSERT_EQ(ErrorCodes::StaleEpoch, resultBSON.getStatus());
+    ASSERT_EQ(ErrorCodes::StaleEpoch, result.getStatus());
 }
 
 TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks) {
@@ -479,7 +493,8 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
-    chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    chunk0.setOnCurrentShardSince(Timestamp(100, 0));
+    chunk0.setHistory({ChunkHistory(*chunk0.getOnCurrentShardSince(), shard0.getName())});
 
     // apportion
     auto chunkMin = BSON("a" << 1);
@@ -498,26 +513,29 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     chunk1.setMax(chunkMaxax);
 
     Timestamp ctrlChunkValidAfter = Timestamp(50, 0);
-    chunk1.setHistory({ChunkHistory(ctrlChunkValidAfter, shard1.getName())});
+    chunk1.setOnCurrentShardSince(ctrlChunkValidAfter);
+    chunk1.setHistory({ChunkHistory(*chunk1.getOnCurrentShardSince(), shard1.getName())});
 
     setupCollection(kNamespace, kKeyPattern, {chunk0, chunk1});
 
-    Timestamp validAfter{101, 0};
-    StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
-                                         ->commitChunkMigration(operationContext(),
-                                                                kNamespace,
-                                                                chunk0,
-                                                                origVersion.epoch(),
-                                                                origVersion.getTimestamp(),
-                                                                ShardId(shard0.getName()),
-                                                                ShardId(shard1.getName()),
-                                                                validAfter);
+    const auto currentTime = VectorClock::get(getServiceContext())->getTime();
+    const auto expectedValidAfter = currentTime.clusterTime().asTimestamp();
 
-    ASSERT_OK(resultBSON.getStatus());
+    StatusWith<ShardingCatalogManager::ShardAndCollectionPlacementVersions> result =
+        ShardingCatalogManager::get(operationContext())
+            ->commitChunkMigration(operationContext(),
+                                   kNamespace,
+                                   chunk0,
+                                   origVersion.epoch(),
+                                   origVersion.getTimestamp(),
+                                   ShardId(shard0.getName()),
+                                   ShardId(shard1.getName()));
+
+    ASSERT_OK(result.getStatus());
 
     // Verify the versions returned match expected values.
-    BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parse(versions["shardVersion"]);
+    auto versions = result.getValue();
+    auto mver = versions.shardPlacementVersion;
     ASSERT_EQ(ChunkVersion({origVersion.epoch(), origVersion.getTimestamp()}, {0, 0}), mver);
 
     // Verify the chunks ended up in the right shards.
@@ -527,7 +545,9 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
 
     // The migrated chunk's history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
-    ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(chunkDoc0.getHistory().front().getValidAfter(), *chunkDoc0.getOnCurrentShardSince());
+    ASSERT_EQ(expectedValidAfter, chunkDoc0.getHistory().front().getValidAfter());
+    ASSERT_EQ(expectedValidAfter, *chunkDoc0.getOnCurrentShardSince());
 
     auto chunkDoc1 =
         uassertStatusOK(getChunkDoc(operationContext(), chunkMax, collEpoch, collTimestamp));
@@ -537,6 +557,7 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     // The control chunk's history should be unchanged.
     ASSERT_EQ(1UL, chunkDoc1.getHistory().size());
     ASSERT_EQ(ctrlChunkValidAfter, chunkDoc1.getHistory().front().getValidAfter());
+    ASSERT_EQ(ctrlChunkValidAfter, *chunkDoc1.getOnCurrentShardSince());
 }
 
 TEST_F(CommitChunkMigrate, RejectMissingChunkVersion) {
@@ -559,7 +580,9 @@ TEST_F(CommitChunkMigrate, RejectMissingChunkVersion) {
     migratedChunk.setName(OID::gen());
     migratedChunk.setCollectionUUID(collUUID);
     migratedChunk.setShard(shard0.getName());
-    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    migratedChunk.setHistory(
+        {ChunkHistory(*migratedChunk.getOnCurrentShardSince(), shard0.getName())});
     migratedChunk.setMin(BSON("a" << 1));
     migratedChunk.setMax(BSON("a" << 10));
 
@@ -568,13 +591,14 @@ TEST_F(CommitChunkMigrate, RejectMissingChunkVersion) {
     currentChunk.setCollectionUUID(collUUID);
     currentChunk.setVersion(origVersion);
     currentChunk.setShard(shard0.getName());
-    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    currentChunk.setHistory(
+        {ChunkHistory(*currentChunk.getOnCurrentShardSince(), shard0.getName())});
     currentChunk.setMin(BSON("a" << 1));
     currentChunk.setMax(BSON("a" << 10));
 
     setupCollection(kNamespace, kKeyPattern, {currentChunk});
 
-    Timestamp validAfter{101, 0};
     ASSERT_THROWS_CODE(ShardingCatalogManager::get(operationContext())
                            ->commitChunkMigration(operationContext(),
                                                   kNamespace,
@@ -582,8 +606,7 @@ TEST_F(CommitChunkMigrate, RejectMissingChunkVersion) {
                                                   origVersion.epoch(),
                                                   origVersion.getTimestamp(),
                                                   ShardId(shard0.getName()),
-                                                  ShardId(shard1.getName()),
-                                                  validAfter),
+                                                  ShardId(shard1.getName())),
                        DBException,
                        4683300);
 }
@@ -609,7 +632,9 @@ TEST_F(CommitChunkMigrate, RejectOlderChunkVersion) {
     migratedChunk.setCollectionUUID(collUUID);
     migratedChunk.setVersion(origVersion);
     migratedChunk.setShard(shard0.getName());
-    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    migratedChunk.setHistory(
+        {ChunkHistory(*migratedChunk.getOnCurrentShardSince(), shard0.getName())});
     migratedChunk.setMin(BSON("a" << 1));
     migratedChunk.setMax(BSON("a" << 10));
 
@@ -620,13 +645,14 @@ TEST_F(CommitChunkMigrate, RejectOlderChunkVersion) {
     currentChunk.setCollectionUUID(collUUID);
     currentChunk.setVersion(currentChunkVersion);
     currentChunk.setShard(shard0.getName());
-    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    currentChunk.setHistory(
+        {ChunkHistory(*currentChunk.getOnCurrentShardSince(), shard0.getName())});
     currentChunk.setMin(BSON("a" << 1));
     currentChunk.setMax(BSON("a" << 10));
 
     setupCollection(kNamespace, kKeyPattern, {currentChunk});
 
-    Timestamp validAfter{101, 0};
     auto result = ShardingCatalogManager::get(operationContext())
                       ->commitChunkMigration(operationContext(),
                                              kNamespace,
@@ -634,11 +660,10 @@ TEST_F(CommitChunkMigrate, RejectOlderChunkVersion) {
                                              origVersion.epoch(),
                                              origVersion.getTimestamp(),
                                              ShardId(shard0.getName()),
-                                             ShardId(shard1.getName()),
-                                             validAfter);
+                                             ShardId(shard1.getName()));
 
-    ASSERT_NOT_OK(result);
-    ASSERT_EQ(result, ErrorCodes::ConflictingOperationInProgress);
+    ASSERT_NOT_OK(result.getStatus());
+    ASSERT_EQ(result.getStatus(), ErrorCodes::ConflictingOperationInProgress);
 }
 
 TEST_F(CommitChunkMigrate, RejectMismatchedEpoch) {
@@ -661,7 +686,9 @@ TEST_F(CommitChunkMigrate, RejectMismatchedEpoch) {
     migratedChunk.setCollectionUUID(collUUID);
     migratedChunk.setVersion(origVersion);
     migratedChunk.setShard(shard0.getName());
-    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    migratedChunk.setHistory(
+        {ChunkHistory(*migratedChunk.getOnCurrentShardSince(), shard0.getName())});
     migratedChunk.setMin(BSON("a" << 1));
     migratedChunk.setMax(BSON("a" << 10));
 
@@ -672,13 +699,14 @@ TEST_F(CommitChunkMigrate, RejectMismatchedEpoch) {
     currentChunk.setCollectionUUID(collUUID);
     currentChunk.setVersion(currentChunkVersion);
     currentChunk.setShard(shard0.getName());
-    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setOnCurrentShardSince(Timestamp(100, 0));
+    currentChunk.setHistory(
+        {ChunkHistory(*currentChunk.getOnCurrentShardSince(), shard0.getName())});
     currentChunk.setMin(BSON("a" << 1));
     currentChunk.setMax(BSON("a" << 10));
 
     setupCollection(kNamespace, kKeyPattern, {currentChunk});
 
-    Timestamp validAfter{101, 0};
     auto result = ShardingCatalogManager::get(operationContext())
                       ->commitChunkMigration(operationContext(),
                                              kNamespace,
@@ -686,11 +714,10 @@ TEST_F(CommitChunkMigrate, RejectMismatchedEpoch) {
                                              origVersion.epoch(),
                                              origVersion.getTimestamp(),
                                              ShardId(shard0.getName()),
-                                             ShardId(shard1.getName()),
-                                             validAfter);
+                                             ShardId(shard1.getName()));
 
-    ASSERT_NOT_OK(result);
-    ASSERT_EQ(result, ErrorCodes::StaleEpoch);
+    ASSERT_NOT_OK(result.getStatus());
+    ASSERT_EQ(result.getStatus(), ErrorCodes::StaleEpoch);
 }
 
 class CommitMoveRangeTest : public CommitChunkMigrate {
@@ -710,6 +737,8 @@ public:
         chunk.setVersion(version);
         chunk.setShard(shardID);
         chunk.setHistory(history);
+        if (!history.empty())
+            chunk.setOnCurrentShardSince(history.front().getValidAfter());
         chunk.setMin(min);
         chunk.setMax(max);
 
@@ -717,7 +746,8 @@ public:
     }
 
     /*
-     * Setup the collection with `numberOfChunks` contiguous chunks covering all the shard key space
+     * Setup the collection with `numberOfChunks` contiguous chunks covering all the shard key
+     space
      */
     void setupCollectionWithNChunks(int numberOfChunks) {
         invariant(numberOfChunks > 0);
@@ -725,7 +755,6 @@ public:
         uint32_t currentMajorVersion = 1;
         int historyTimestampSecond = 100;
 
-        std::vector<ChunkHistory> history;
         std::vector<BSONObj> chunksMin = {kKeyPattern.globalMin()};
         for (int i = 10; i < numberOfChunks * 10; i += 10) {
             chunksMin.push_back(BSON("x" << i));
@@ -738,8 +767,8 @@ public:
             const auto shardId = _shardIds.at(i % 2);  // Shard owning the chunk
             ChunkVersion version =
                 ChunkVersion({_collEpoch, _collTimestamp}, {currentMajorVersion++, 0});
-            history.insert(history.begin(),
-                           {ChunkHistory(Timestamp(historyTimestampSecond++, 0), shardId)});
+            std::vector<ChunkHistory> history{
+                ChunkHistory(Timestamp(historyTimestampSecond++, 0), shardId)};
             ChunkType chunk = createChunk(_collUUID, min, max, version, shardId, history);
             chunks.push_back(chunk);
         }
@@ -758,16 +787,11 @@ public:
                                const ChunkType& migratedChunk,
                                const bool expectLeftSplit,
                                const bool expectRightSplit) {
-        Timestamp validAfter = [&]() {
-            auto currValidAfter = migratedChunk.getHistory().at(0).getValidAfter();
-            return Timestamp(currValidAfter.getSecs() + 100, 0);
-        }();
-
         const auto donor = migratedChunk.getShard();
         const auto recipient =
             migratedChunk.getShard() == _shardIds.at(0) ? _shardIds.at(1) : _shardIds.at(0);
 
-        auto collVersionBefore = [&]() {
+        auto collPlacementVersionBefore = [&]() {
             const auto chunkDoc = uassertStatusOK(
                 findOneOnConfigCollection(operationContext(),
                                           ChunkType::ConfigNS,
@@ -778,6 +802,9 @@ public:
             return chunk.getVersion();
         }();
 
+        const auto currentTime = VectorClock::get(getServiceContext())->getTime();
+        const auto expectedValidAfter = currentTime.clusterTime().asTimestamp();
+
         uassertStatusOK(ShardingCatalogManager::get(operationContext())
                             ->commitChunkMigration(operationContext(),
                                                    kNamespace,
@@ -785,8 +812,7 @@ public:
                                                    migratedChunk.getVersion().epoch(),
                                                    migratedChunk.getVersion().getTimestamp(),
                                                    donor,
-                                                   recipient,
-                                                   validAfter));
+                                                   recipient));
 
         // Verify the new chunk is on the recipient shard
         {
@@ -799,12 +825,14 @@ public:
             ASSERT(migratedChunk.getMax().woCompare(newChunk.getMax()) == 0);
 
             // The migrated chunk's version must have been bumped
-            ASSERT_EQ(newChunk.getVersion().majorVersion(), collVersionBefore.majorVersion() + 1);
+            ASSERT_EQ(newChunk.getVersion().majorVersion(),
+                      collPlacementVersionBefore.majorVersion() + 1);
             ASSERT_EQ(0, newChunk.getVersion().minorVersion());
 
             // The migrated chunk's history should have been updated with a new `validAfter` entry
             ASSERT_EQ(origChunk.getHistory().size() + 1, newChunk.getHistory().size());
-            ASSERT_EQ(validAfter, newChunk.getHistory().front().getValidAfter());
+            ASSERT_EQ(expectedValidAfter, newChunk.getHistory().front().getValidAfter());
+            ASSERT_EQ(expectedValidAfter, *newChunk.getOnCurrentShardSince());
 
             // The migrated chunk's history must inherit the previous chunk's history
             assertSameHistories(std::vector<ChunkHistory>(newChunk.getHistory().begin() + 1,
@@ -821,6 +849,7 @@ public:
                                             migratedChunk.getVersion().epoch(),
                                             migratedChunk.getVersion().getTimestamp()));
             ASSERT_EQ(donor, leftSplitChunk.getShard());
+            ASSERT_EQ(origChunk.getOnCurrentShardSince(), leftSplitChunk.getOnCurrentShardSince());
 
             // The min of the split chunk must be the min of the original chunk
             ASSERT(leftSplitChunk.getMin().woCompare(origChunk.getMin()) == 0);
@@ -829,7 +858,7 @@ public:
             ASSERT(leftSplitChunk.getMax().woCompare(migratedChunk.getMin()) == 0);
 
             // The major and minor versions of the left split chunk must have been bumped
-            ASSERT_EQ(collVersionBefore.majorVersion() + 1,
+            ASSERT_EQ(collPlacementVersionBefore.majorVersion() + 1,
                       leftSplitChunk.getVersion().majorVersion());
             ASSERT_EQ(expectedMinVersion++, leftSplitChunk.getVersion().minorVersion());
 
@@ -845,6 +874,7 @@ public:
                                             migratedChunk.getVersion().epoch(),
                                             migratedChunk.getVersion().getTimestamp()));
             ASSERT_EQ(donor, rightSplitChunk.getShard());
+            ASSERT_EQ(origChunk.getOnCurrentShardSince(), rightSplitChunk.getOnCurrentShardSince());
 
             // The min of the right split chunk must fit the max of the new chunk
             ASSERT(rightSplitChunk.getMin().woCompare(migratedChunk.getMax()) == 0);
@@ -853,7 +883,7 @@ public:
             ASSERT(rightSplitChunk.getMax().woCompare(origChunk.getMax()) == 0);
 
             // The major and minor versions of the right split chunk must have been bumped
-            ASSERT_EQ(collVersionBefore.majorVersion() + 1,
+            ASSERT_EQ(collPlacementVersionBefore.majorVersion() + 1,
                       rightSplitChunk.getVersion().majorVersion());
             ASSERT_EQ(expectedMinVersion++, rightSplitChunk.getVersion().minorVersion());
 
