@@ -30,15 +30,17 @@
 #include "mongo/db/s/move_primary_source_manager.h"
 
 #include "mongo/client/connpool.h"
-#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/repl/change_stream_oplog_notification.h"
+#include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/shard_metadata_util.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_state_recovery.h"
 #include "mongo/db/s/sharding_statistics.h"
 #include "mongo/db/s/type_shard_database.h"
+#include "mongo/db/vector_clock_mutable.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/catalog_cache.h"
@@ -61,8 +63,8 @@ MovePrimarySourceManager::MovePrimarySourceManager(OperationContext* opCtx,
                                                    ShardId& toShard)
     : _requestArgs(std::move(requestArgs)),
       _dbname(dbname),
-      _fromShard(std::move(fromShard)),
-      _toShard(std::move(toShard)),
+      _fromShard(fromShard),
+      _toShard(toShard),
       _critSecReason(BSON("command"
                           << "movePrimary"
                           << "dbName" << _dbname << "fromShard" << fromShard << "toShard"
@@ -90,8 +92,8 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
     auto logChangeCheckedStatus = ShardingLogging::get(opCtx)->logChangeChecked(
         opCtx,
         "movePrimary.start",
-        _dbname.toString(),
-        _buildMoveLogEntry(_dbname.toString(), _fromShard.toString(), _toShard.toString()),
+        _dbname,
+        _buildMoveLogEntry(_dbname, _fromShard.toString(), _toShard.toString()),
         ShardingCatalogClient::kMajorityWriteConcern);
 
     if (!logChangeCheckedStatus.isOK()) {
@@ -104,10 +106,9 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
         AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_X);
         invariant(autoDb.ensureDbExists(opCtx), getNss().toString());
 
-        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
-
-        dss->setMovePrimarySourceManager(opCtx, this, dssLock);
+        auto scopedDss =
+            DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, getNss().dbName());
+        scopedDss->setMovePrimaryInProgress(opCtx);
     }
 
     _state = kCloning;
@@ -156,7 +157,8 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
     ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     // Mark the shard as running a critical operation that requires recovery on crash.
-    auto startMetadataOpStatus = ShardingStateRecovery::startMetadataOp(opCtx);
+    // TODO (SERVER-60110): Remove once 7.0 becomes last LTS.
+    auto startMetadataOpStatus = ShardingStateRecovery_DEPRECATED::startMetadataOp(opCtx);
     if (!startMetadataOpStatus.isOK()) {
         return startMetadataOpStatus;
     }
@@ -173,11 +175,11 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
                                     << " was dropped during the movePrimary operation.");
         }
 
-        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
+        auto scopedDss =
+            DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, getNss().dbName());
 
         // IMPORTANT: After this line, the critical section is in place and needs to be signaled
-        dss->enterCriticalSectionCatchUpPhase(opCtx, dssLock, _critSecReason);
+        scopedDss->enterCriticalSectionCatchUpPhase(opCtx, _critSecReason);
     }
 
     _state = kCriticalSection;
@@ -224,14 +226,14 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
                                     << " was dropped during the movePrimary operation.");
         }
 
-        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
+        auto scopedDss =
+            DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, getNss().dbName());
 
         // Read operations must begin to wait on the critical section just before we send the
         // commit operation to the config server
-        dss->enterCriticalSectionCommitPhase(opCtx, dssLock, _critSecReason);
+        scopedDss->enterCriticalSectionCommitPhase(opCtx, _critSecReason);
 
-        expectedDbVersion = DatabaseHolder::get(opCtx)->getDbVersion(opCtx, _dbname);
+        expectedDbVersion = scopedDss->getDbVersion(opCtx);
     }
 
     auto commitStatus = [&]() {
@@ -256,7 +258,7 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
             opCtx,
             "movePrimary.validating",
             getNss().ns(),
-            _buildMoveLogEntry(_dbname.toString(), _fromShard.toString(), _toShard.toString()),
+            _buildMoveLogEntry(_dbname, _fromShard.toString(), _toShard.toString()),
             ShardingCatalogClient::kMajorityWriteConcern);
 
         if ((ErrorCodes::isInterruption(validateStatus.code()) ||
@@ -274,7 +276,8 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
         // metadata for this database, forcing subsequent callers to do a full refresh. Check if
         // this node can accept writes for this collection as a proxy for it being primary.
         if (!validateStatus.isOK()) {
-            UninterruptibleLockGuard noInterrupt(opCtx->lockState());
+            // TODO (SERVER-71444): Fix to be interruptible or document exception.
+            UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
             AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_IX);
 
             if (!autoDb.getDb()) {
@@ -284,8 +287,9 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
             }
 
             if (!repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, getNss())) {
-                DatabaseHolder::get(opCtx)->clearDbInfo(
-                    opCtx, DatabaseName(boost::none, getNss().toString()));
+                auto scopedDss = DatabaseShardingState::assertDbLockedAndAcquireExclusive(
+                    opCtx, getNss().dbName());
+                scopedDss->clearDbInfo(opCtx);
                 uassertStatusOK(validateStatus.withContext(
                     str::stream() << "Unable to verify movePrimary commit for database: "
                                   << getNss().ns()
@@ -317,8 +321,8 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
     uassertStatusOK(ShardingLogging::get(opCtx)->logChangeChecked(
         opCtx,
         "movePrimary.commit",
-        _dbname.toString(),
-        _buildMoveLogEntry(_dbname.toString(), _fromShard.toString(), _toShard.toString()),
+        _dbname,
+        _buildMoveLogEntry(_dbname, _fromShard.toString(), _toShard.toString()),
         ShardingCatalogClient::kMajorityWriteConcern));
 
     scopedGuard.dismiss();
@@ -338,15 +342,18 @@ Status MovePrimarySourceManager::_commitOnConfig(OperationContext* opCtx,
                 "toShard"_attr = _toShard,
                 "expectedDbVersion"_attr = expectedDbVersion);
 
+    notifyChangeStreamsOnMovePrimary(opCtx, getNss().dbName(), _fromShard, _toShard);
+
     const auto commitStatus = [&] {
-        ConfigsvrCommitMovePrimary commitRequest(_dbname, expectedDbVersion, _toShard);
-        commitRequest.setDbName(NamespaceString::kAdminDb);
+        ConfigsvrCommitMovePrimary commitRequest(
+            DatabaseName{_dbname}, expectedDbVersion, _toShard);
+        commitRequest.setDbName(DatabaseName::kAdmin);
 
         const auto commitResponse =
             Grid::get(opCtx)->shardRegistry()->getConfigShard()->runCommandWithFixedRetryAttempts(
                 opCtx,
                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                NamespaceString::kAdminDb.toString(),
+                DatabaseName::kAdmin.toString(),
                 CommandHelpers::appendMajorityWriteConcern(commitRequest.toBSON({})),
                 Shard::RetryPolicy::kIdempotent);
 
@@ -370,6 +377,9 @@ Status MovePrimarySourceManager::_commitOnConfig(OperationContext* opCtx,
               "Error committing movePrimary",
               "db"_attr = _dbname,
               "error"_attr = redact(commitStatus));
+        // Try to emit a second notification to reverse the effect of the one notified before the
+        // commit attempt.
+        notifyChangeStreamsOnMovePrimary(opCtx, getNss().dbName(), _toShard, _fromShard);
         return commitStatus;
     }
 
@@ -456,13 +466,13 @@ Status MovePrimarySourceManager::cleanStaleData(OperationContext* opCtx) {
     DBDirectClient client(opCtx);
     for (auto& coll : _clonedColls) {
         BSONObj dropCollResult;
-        client.runCommand(_dbname.toString(), BSON("drop" << coll.coll()), dropCollResult);
+        client.runCommand(DatabaseName{_dbname}, BSON("drop" << coll.coll()), dropCollResult);
         Status dropStatus = getStatusFromCommandResult(dropCollResult);
         if (!dropStatus.isOK()) {
             LOGV2(22045,
                   "Failed to drop cloned collection {namespace} in movePrimary: {error}",
                   "Failed to drop cloned collection in movePrimary",
-                  "namespace"_attr = coll,
+                  logAttrs(coll),
                   "error"_attr = redact(dropStatus));
         }
     }
@@ -479,8 +489,8 @@ void MovePrimarySourceManager::cleanupOnError(OperationContext* opCtx) {
     ShardingLogging::get(opCtx)->logChange(
         opCtx,
         "movePrimary.error",
-        _dbname.toString(),
-        _buildMoveLogEntry(_dbname.toString(), _fromShard.toString(), _toShard.toString()),
+        _dbname,
+        _buildMoveLogEntry(_dbname, _fromShard.toString(), _toShard.toString()),
         ShardingCatalogClient::kMajorityWriteConcern);
 
     try {
@@ -502,21 +512,27 @@ void MovePrimarySourceManager::_cleanup(OperationContext* opCtx) {
 
     {
         // Unregister from the database's sharding state if we're still registered.
-        UninterruptibleLockGuard noInterrupt(opCtx->lockState());
+        // TODO (SERVER-71444): Fix to be interruptible or document exception.
+        UninterruptibleLockGuard noInterrupt(opCtx->lockState());  // NOLINT.
         AutoGetDb autoDb(opCtx, getNss().dbName(), MODE_IX);
 
-        auto dss = DatabaseShardingState::get(opCtx, getNss().db());
-        dss->clearMovePrimarySourceManager(opCtx);
-        DatabaseHolder::get(opCtx)->clearDbInfo(opCtx,
-                                                DatabaseName(boost::none, getNss().toString()));
+        auto scopedDss =
+            DatabaseShardingState::assertDbLockedAndAcquireExclusive(opCtx, getNss().dbName());
+        scopedDss->unsetMovePrimaryInProgress(opCtx);
+        scopedDss->clearDbInfo(opCtx);
+
         // Leave the critical section if we're still registered.
-        dss->exitCriticalSection(opCtx, _critSecReason);
+        scopedDss->exitCriticalSection(opCtx, _critSecReason);
     }
 
     if (_state == kCriticalSection || _state == kCloneCompleted) {
         // Clear the 'minOpTime recovery' document so that the next time a node from this shard
         // becomes a primary, it won't have to recover the config server optime.
-        ShardingStateRecovery::endMetadataOp(opCtx);
+        // TODO (SERVER-60110): Remove once 7.0 becomes last LTS.
+        ShardingStateRecovery_DEPRECATED::endMetadataOp(opCtx);
+
+        // Checkpoint the vector clock to ensure causality in the event of a crash or shutdown.
+        VectorClockMutable::get(opCtx)->waitForDurableConfigTime().get(opCtx);
     }
 
     // If we're in the kCloneCompleted state, then we need to do the last step of cleaning up
