@@ -50,9 +50,8 @@ namespace {
 
 using namespace fmt::literals;
 
-const NamespaceString kRemoteOplogNss{"local.oplog.rs"};
-const NamespaceString kLocalOplogBufferNss{"{}.{}xxx.yyy"_format(
-    NamespaceString::kConfigDb, NamespaceString::kReshardingLocalOplogBufferPrefix)};
+const NamespaceString kLocalOplogBufferNss = NamespaceString::createNamespaceString_forTest(
+    "config", "{}xxx.yyy"_format(NamespaceString::kReshardingLocalOplogBufferPrefix));
 
 // A mock TransactionHistoryIterator to support DSReshardingIterateTransaction.
 class MockTransactionHistoryIterator : public TransactionHistoryIteratorBase {
@@ -135,6 +134,16 @@ public:
         return pipeline;
     }
 
+    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+        const AggregateCommandRequest& aggRequest,
+        Pipeline* pipeline,
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        boost::optional<BSONObj> shardCursorsSortSpec = boost::none,
+        ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
+        boost::optional<BSONObj> readConcern = boost::none) final {
+        return attachCursorSourceToPipeline(pipeline, shardTargetingPolicy, std::move(readConcern));
+    }
+
     std::unique_ptr<TransactionHistoryIteratorBase> createTransactionHistoryIterator(
         repl::OpTime time) const {
         return std::unique_ptr<TransactionHistoryIteratorBase>(
@@ -233,11 +242,11 @@ repl::DurableOplogEntry makeApplyOpsOplog(std::vector<BSONObj> operations,
     }
 
     return {opTime,
-            boost::none /* hash */,
             repl::OpTypeEnum::kCommand,
             {},
             UUID::gen(),
             false /* fromMigrate */,
+            boost::none,  // checkExistenceForDiffInsert
             0 /* version */,
             applyOpsBuilder.obj(), /* o */
             boost::none,           /* o2 */
@@ -266,7 +275,8 @@ boost::intrusive_ptr<ExpressionContextForTest> createExpressionContext(Operation
     boost::intrusive_ptr<ExpressionContextForTest> expCtx(
         new ExpressionContextForTest(opCtx, kLocalOplogBufferNss));
     expCtx->setResolvedNamespace(kLocalOplogBufferNss, {kLocalOplogBufferNss, {}});
-    expCtx->setResolvedNamespace(kRemoteOplogNss, {kRemoteOplogNss, {}});
+    expCtx->setResolvedNamespace(NamespaceString::kRsOplogNamespace,
+                                 {NamespaceString::kRsOplogNamespace, {}});
     return expCtx;
 }
 
@@ -359,7 +369,7 @@ protected:
         std::deque<DocumentSource::GetNextResult> pipelineSource) {
         // Set up the oplog collection state for $lookup and $graphLookup calls.
         auto expCtx = createExpressionContext();
-        expCtx->ns = kRemoteOplogNss;
+        expCtx->ns = NamespaceString::kRsOplogNamespace;
         expCtx->mongoProcessInterface = std::make_shared<MockMongoInterface>(pipelineSource);
 
         auto pipeline = resharding::createOplogFetchingPipelineForResharding(
@@ -521,7 +531,7 @@ TEST_F(ReshardingAggTest, VerifyPipelineOutputHasOplogSchema) {
                                                                 Document(deleteOplog.toBSON())};
 
     boost::intrusive_ptr<ExpressionContext> expCtx = createExpressionContext();
-    expCtx->ns = kRemoteOplogNss;
+    expCtx->ns = NamespaceString::kRsOplogNamespace;
     expCtx->mongoProcessInterface = std::make_shared<MockMongoInterface>(pipelineSource);
 
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline =
@@ -622,7 +632,7 @@ TEST_F(ReshardingAggTest, VerifyPipelinePreparedTxn) {
 
     boost::intrusive_ptr<ExpressionContext> expCtx = createExpressionContext();
     // Set up the oplog collection state for $lookup and $graphLookup calls.
-    expCtx->ns = kRemoteOplogNss;
+    expCtx->ns = NamespaceString::kRsOplogNamespace;
     expCtx->mongoProcessInterface = std::make_shared<MockMongoInterface>(pipelineSource);
 
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline =
@@ -667,7 +677,7 @@ TEST_F(ReshardingAggTest, VerifyPipelinePreparedTxn) {
     ASSERT(!pipeline->getNext());
 }
 
-TEST_F(ReshardingAggTest, VerifyPipelineAtomicApplyOps) {
+TEST_F(ReshardingAggTest, VerifyPipelineApplyOps) {
     const auto oplogBSON = fromjson(R"({
         "op": "c",
         "ns": "test.$cmd",
@@ -705,7 +715,6 @@ TEST_F(ReshardingAggTest, VerifyPipelineAtomicApplyOps) {
 
     auto pipeline = createPipeline({Document(oplogBSON)});
 
-    // We don't need to support atomic applyOps in the resharding pipeline; we filter them out.
     ASSERT(!pipeline->getNext());
 }
 
@@ -1443,14 +1452,14 @@ TEST_F(ReshardingAggWithStorageTest, RetryableFindAndModifyWithImageLookup) {
     imageEntry.setImage(preImage);
 
     DBDirectClient client(opCtx());
-    client.insert(NamespaceString::kConfigImagesNamespace.ns(), imageEntry.toBSON());
+    client.insert(NamespaceString::kConfigImagesNamespace, imageEntry.toBSON());
 
     repl::DurableOplogEntry oplog(opTime,
-                                  boost::none /* hash */,
                                   repl::OpTypeEnum::kUpdate,
                                   kCrudNs,
                                   kCrudUUID,
                                   false /* fromMigrate */,
+                                  boost::none,  // checkExistenceForDiffInsert
                                   0 /* version */,
                                   BSON("$set" << BSON("y" << 1)), /* o1 */
                                   BSON("_id" << 2),               /* o2 */
@@ -1560,7 +1569,7 @@ TEST_F(ReshardingAggWithStorageTest,
     imageEntry.setImage(preImage);
 
     DBDirectClient client(opCtx());
-    client.insert(NamespaceString::kConfigImagesNamespace.ns(), imageEntry.toBSON());
+    client.insert(NamespaceString::kConfigImagesNamespace, imageEntry.toBSON());
 
     auto createPipeline = [&](ReshardingDonorOplogId startAt) {
         std::deque<DocumentSource::GetNextResult> pipelineSource{
