@@ -72,6 +72,16 @@ public:
         return pipeline;
     }
 
+    std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
+        const AggregateCommandRequest& aggRequest,
+        Pipeline* pipeline,
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        boost::optional<BSONObj> shardCursorsSortSpec = boost::none,
+        ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
+        boost::optional<BSONObj> readConcern = boost::none) final {
+        return attachCursorSourceToPipeline(pipeline, shardTargetingPolicy, std::move(readConcern));
+    }
+
 private:
     std::deque<DocumentSource::GetNextResult> _mockResults;
 };
@@ -101,9 +111,9 @@ protected:
 
         getCatalogCacheMock()->setChunkManagerReturnValue(
             createChunkManager(newShardKeyPattern, configCacheChunksData));
-
-        _pipeline = _cloner->makePipeline(
+        auto [rawPipeline, expCtx] = _cloner->makeRawPipeline(
             operationContext(), std::make_shared<MockMongoInterface>(configCacheChunksData));
+        _pipeline = Pipeline::parse(std::move(rawPipeline), std::move(expCtx));
 
         _pipeline->addInitialSource(
             DocumentSourceMock::createForTest(sourceCollectionData, _pipeline->getContext()));
@@ -152,8 +162,7 @@ protected:
                                                epoch,
                                                Timestamp(1, 1),
                                                boost::none /* timeseriesFields */,
-                                               boost::none,
-                                               boost::none /* chunkSizeBytes */,
+                                               boost::none /* reshardingFields */,
                                                false,
                                                chunks);
 
@@ -172,7 +181,7 @@ protected:
         std::function<void(std::unique_ptr<SeekableRecordCursor>)> verifyFunction) {
         initializePipelineTest(shardKey, recipientShard, collectionData, configData);
         auto opCtx = operationContext();
-        AutoGetCollection tempColl{opCtx, tempNss, MODE_IS};
+        AutoGetCollection tempColl{opCtx, tempNss, MODE_IX};
         while (_cloner->doOneBatch(operationContext(), *_pipeline)) {
             ASSERT_EQ(tempColl->numRecords(opCtx), _metrics->getDocumentsProcessedCount());
             ASSERT_EQ(tempColl->dataSize(opCtx), _metrics->getBytesWrittenCount());
@@ -185,7 +194,8 @@ protected:
     }
 
 protected:
-    const NamespaceString _sourceNss = NamespaceString("test"_sd, "collection_being_resharded"_sd);
+    const NamespaceString _sourceNss =
+        NamespaceString::createNamespaceString_forTest("test"_sd, "collection_being_resharded"_sd);
     const NamespaceString tempNss =
         resharding::constructTemporaryReshardingNss(_sourceNss.db(), _sourceUUID);
     const UUID _sourceUUID = UUID::gen();
@@ -245,8 +255,7 @@ TEST_F(ReshardingCollectionClonerTest, MaxKeyChunk) {
         Doc(fromjson("{_id: {x: {$minKey: 1}}, max: {x: 0.0}, shard: 'myShardName'}")),
         Doc(fromjson("{_id: {x: 0.0}, max: {x: {$maxKey: 1}}, shard: 'shard2' }")),
     };
-    // TODO SERVER-67529: Change expected documents to 4.
-    constexpr auto kExpectedCopiedCount = 3;
+    constexpr auto kExpectedCopiedCount = 4;
     const auto verify = [](auto cursor) {
         auto next = cursor->next();
         ASSERT(next);
@@ -263,12 +272,10 @@ TEST_F(ReshardingCollectionClonerTest, MaxKeyChunk) {
         ASSERT_BSONOBJ_BINARY_EQ(BSON("_id" << 5 << "x" << 0.001 << "$sortKey" << BSON_ARRAY(5)),
                                  next->data.toBson());
 
-        // TODO SERVER-67529: Enable after ChunkManager can handle documents with $maxKey.
-        // next = cursor->next();
-        // ASSERT(next);
-        // ASSERT_BSONOBJ_BINARY_EQ(
-        //     BSON("_id" << 6 << "x" << MAXKEY << "$sortKey" << BSON_ARRAY(6)),
-        //     next->data.toBson());
+        next = cursor->next();
+        ASSERT(next);
+        ASSERT_BSONOBJ_BINARY_EQ(BSON("_id" << 6 << "x" << MAXKEY << "$sortKey" << BSON_ARRAY(6)),
+                                 next->data.toBson());
 
         ASSERT_FALSE(cursor->next());
     };
