@@ -44,13 +44,12 @@
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_mock.h"
 #include "mongo/db/s/operation_sharding_state.h"
-#include "mongo/db/s/recoverable_critical_section_service.h"
 #include "mongo/db/s/resharding/resharding_change_event_o2_field_gen.h"
 #include "mongo/db/s/resharding/resharding_data_copy_util.h"
 #include "mongo/db/s/resharding/resharding_donor_service.h"
 #include "mongo/db/s/resharding/resharding_service_test_helpers.h"
 #include "mongo/db/s/resharding/resharding_util.h"
-#include "mongo/db/s/sharding_ddl_util.h"
+#include "mongo/db/s/sharding_recovery_service.h"
 #include "mongo/db/service_context.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
@@ -66,7 +65,8 @@ namespace {
 using DonorStateTransitionController =
     resharding_service_test_helpers::StateTransitionController<DonorStateEnum>;
 using OpObserverForTest =
-    resharding_service_test_helpers::OpObserverForTest<DonorStateEnum, ReshardingDonorDocument>;
+    resharding_service_test_helpers::StateTransitionControllerOpObserver<DonorStateEnum,
+                                                                         ReshardingDonorDocument>;
 using PauseDuringStateTransitions =
     resharding_service_test_helpers::PauseDuringStateTransitions<DonorStateEnum>;
 
@@ -86,20 +86,8 @@ public:
                                    const BSONObj& query,
                                    const BSONObj& update) override {}
 
-    void clearFilteringMetadata(OperationContext* opCtx,
-                                const NamespaceString& sourceNss,
-                                const NamespaceString& tempReshardingNss) override {}
-};
-
-class DonorOpObserverForTest : public OpObserverForTest {
-public:
-    DonorOpObserverForTest(std::shared_ptr<DonorStateTransitionController> controller)
-        : OpObserverForTest(std::move(controller),
-                            NamespaceString::kDonorReshardingOperationsNamespace) {}
-
-    DonorStateEnum getState(const ReshardingDonorDocument& donorDoc) override {
-        return donorDoc.getMutableState().getState();
-    }
+    void refreshCollectionPlacementInfo(OperationContext* opCtx,
+                                        const NamespaceString& sourceNss) override {}
 };
 
 class ReshardingDonorServiceForTest : public ReshardingDonorService {
@@ -138,7 +126,12 @@ public:
         repl::StorageInterface::set(serviceContext, std::move(storageMock));
 
         _controller = std::make_shared<DonorStateTransitionController>();
-        _opObserverRegistry->addObserver(std::make_unique<DonorOpObserverForTest>(_controller));
+        _opObserverRegistry->addObserver(std::make_unique<OpObserverForTest>(
+            _controller,
+            NamespaceString::kDonorReshardingOperationsNamespace,
+            [](const ReshardingDonorDocument& donorDoc) {
+                return donorDoc.getMutableState().getState();
+            }));
     }
 
     DonorStateTransitionController* controller() {
@@ -171,8 +164,7 @@ public:
     void createSourceCollection(OperationContext* opCtx, const ReshardingDonorDocument& donorDoc) {
         CollectionOptions options;
         options.uuid = donorDoc.getSourceUUID();
-        mongo::sharding_ddl_util::ensureCollectionDroppedNoChangeEvent(opCtx,
-                                                                       donorDoc.getSourceNss());
+        resharding::data_copy::ensureCollectionDropped(opCtx, donorDoc.getSourceNss());
         resharding::data_copy::ensureCollectionExists(opCtx, donorDoc.getSourceNss(), options);
     }
 
@@ -180,8 +172,7 @@ public:
                                              const ReshardingDonorDocument& donorDoc) {
         CollectionOptions options;
         options.uuid = donorDoc.getReshardingUUID();
-        mongo::sharding_ddl_util::ensureCollectionDroppedNoChangeEvent(
-            opCtx, donorDoc.getTempReshardingNss());
+        resharding::data_copy::ensureCollectionDropped(opCtx, donorDoc.getTempReshardingNss());
         resharding::data_copy::ensureCollectionExists(
             opCtx, donorDoc.getTempReshardingNss(), options);
     }
@@ -426,8 +417,10 @@ TEST_F(ReshardingDonorServiceTest, StepDownStepUpEachTransition) {
                     DonorStateMachine::insertStateDocument(opCtx.get(), doc);
                     return DonorStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
                 } else {
-                    auto maybeDonor = DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
-                    ASSERT_TRUE(bool(maybeDonor));
+                    auto [maybeDonor, isPausedOrShutdown] =
+                        DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
+                    ASSERT_TRUE(maybeDonor);
+                    ASSERT_FALSE(isPausedOrShutdown);
 
                     // Allow the transition to prevState to succeed on this primary-only service
                     // instance.
@@ -488,8 +481,10 @@ TEST_F(ReshardingDonorServiceTest, StepDownStepUpEachTransition) {
         }
 
         // Finally complete the operation and ensure its success.
-        auto maybeDonor = DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
-        ASSERT_TRUE(bool(maybeDonor));
+        auto [maybeDonor, isPausedOrShutdown] =
+            DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(maybeDonor);
+        ASSERT_FALSE(isPausedOrShutdown);
 
         auto donor = *maybeDonor;
         stateTransitionsGuard.unset(DonorStateEnum::kDone);
@@ -500,6 +495,55 @@ TEST_F(ReshardingDonorServiceTest, StepDownStepUpEachTransition) {
     }
 }
 
+TEST_F(ReshardingDonorServiceTest, ReportForCurrentOpAfterCompletion) {
+    const auto donorState = DonorStateEnum::kDonatingInitialData;
+
+    PauseDuringStateTransitions stateTransitionsGuard{controller(), donorState};
+    auto doc = makeStateDocument(false /* isAlsoRecipient */);
+    auto instanceId =
+        BSON(ReshardingDonorDocument::kReshardingUUIDFieldName << doc.getReshardingUUID());
+
+    auto opCtx = makeOperationContext();
+    auto donor = [&] {
+        createSourceCollection(opCtx.get(), doc);
+
+        DonorStateMachine::insertStateDocument(opCtx.get(), doc);
+        return DonorStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
+    }();
+
+    // Step down before the transition to state can complete.
+    stateTransitionsGuard.wait(donorState);
+    stepDown();
+    stateTransitionsGuard.unset(donorState);
+
+    // At this point, the resharding metrics will have been unregistered from the cumulative metrics
+    ASSERT_EQ(donor->getCompletionFuture().getNoThrow(),
+              ErrorCodes::InterruptedDueToReplStateChange);
+
+    // Now call step up. The old donor object has not yet been destroyed because we still hold
+    // a shared pointer to it ('donor') - this can happen in production after a failover if a
+    // state machine is slow to clean up.
+    stepUp(opCtx.get());
+
+    // Assert that the old donor object will return a currentOp report, because the resharding
+    // metrics still exist on the coordinator object itelf.
+    ASSERT(donor->reportForCurrentOp(MongoProcessInterface::CurrentOpConnectionsMode::kExcludeIdle,
+                                     MongoProcessInterface::CurrentOpSessionsMode::kIncludeIdle));
+
+    // Ensure the new donor started up successfully (and thus, registered new resharding metrics),
+    // despite the "zombie" state machine still existing.
+    auto [maybeDonor, isPausedOrShutdown] =
+        DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
+    ASSERT_TRUE(maybeDonor);
+    ASSERT_FALSE(isPausedOrShutdown);
+    auto newDonor = *maybeDonor;
+    ASSERT_NE(donor, newDonor);
+
+    // No need to finish the resharding op, so we just cancel the op.
+    newDonor->abort(false);
+    ASSERT_OK(newDonor->getCompletionFuture().getNoThrow());
+}
+
 DEATH_TEST_REGEX_F(ReshardingDonorServiceTest, CommitFn, "4457001.*tripwire") {
     auto doc = makeStateDocument(false /* isAlsoRecipient */);
     auto opCtx = makeOperationContext();
@@ -619,8 +663,10 @@ TEST_F(ReshardingDonorServiceTest, CompletesWithStepdownAfterAbort) {
         donor.reset();
         stepUp(opCtx.get());
 
-        auto maybeDonor = DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
-        ASSERT_TRUE(bool(maybeDonor));
+        auto [maybeDonor, isPausedOrShutdown] =
+            DonorStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(maybeDonor);
+        ASSERT_FALSE(isPausedOrShutdown);
 
         donor = *maybeDonor;
         doneTransitionGuard.reset();
@@ -758,7 +804,7 @@ TEST_F(ReshardingDonorServiceTest, RestoreMetricsOnKBlockingWrites) {
 
     // This acquires the critical section required by resharding donor machine when it is in
     // kBlockingWrites.
-    RecoverableCriticalSectionService::get(opCtx.get())
+    ShardingRecoveryService::get(opCtx.get())
         ->acquireRecoverableCriticalSectionBlockWrites(opCtx.get(),
                                                        doc.getSourceNss(),
                                                        BSON("command"
