@@ -30,11 +30,11 @@
 
 #include "mongo/platform/basic.h"
 
+#include "mongo/db/s/metrics/sharding_data_transform_cumulative_metrics.h"
+#include "mongo/db/s/metrics/sharding_data_transform_metrics_test_fixture.h"
 #include "mongo/db/s/resharding/resharding_metrics.h"
 #include "mongo/db/s/resharding/resharding_service_test_helpers.h"
 #include "mongo/db/s/resharding/resharding_util.h"
-#include "mongo/db/s/sharding_data_transform_cumulative_metrics.h"
-#include "mongo/db/s/sharding_data_transform_metrics_test_fixture.h"
 #include "mongo/unittest/unittest.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
@@ -43,6 +43,7 @@
 namespace mongo {
 namespace {
 
+using TimedPhase = ReshardingMetrics::TimedPhase;
 constexpr auto kRunningTime = Seconds(12345);
 constexpr auto kResharding = "resharding";
 const auto kShardKey = BSON("newKey" << 1);
@@ -425,7 +426,7 @@ TEST_F(ReshardingMetricsTest, RecipientIncrementFetchedOplogEntries) {
 
     auto report = metrics->reportForCurrentOp();
     ASSERT_EQ(report.getIntField("oplogEntriesFetched"), 0);
-    metrics->onOplogEntriesFetched(50, Milliseconds(1));
+    metrics->onOplogEntriesFetched(50);
 
     report = metrics->reportForCurrentOp();
     ASSERT_EQ(report.getIntField("oplogEntriesFetched"), 50);
@@ -458,10 +459,10 @@ TEST_F(ReshardingMetricsTest, RecipientReportsRemainingTime) {
     const auto kIncrementSecs = durationCount<Seconds>(kIncrement);
     const auto kExpectedTotal = kIncrementSecs * 8;
     metrics->setDocumentsToProcessCounts(0, kOpsPerIncrement * 4);
-    metrics->onOplogEntriesFetched(kOpsPerIncrement * 4, Milliseconds(1));
+    metrics->onOplogEntriesFetched(kOpsPerIncrement * 4);
 
     // During cloning.
-    metrics->onCopyingBegin();
+    metrics->setStartFor(TimedPhase::kCloning, getClockSource()->now());
     metrics->onDocumentsProcessed(0, kOpsPerIncrement, Milliseconds(1));
     clock->advance(kIncrement);
     auto report = metrics->reportForCurrentOp();
@@ -477,8 +478,8 @@ TEST_F(ReshardingMetricsTest, RecipientReportsRemainingTime) {
     // During applying.
     metrics->onDocumentsProcessed(0, kOpsPerIncrement, Milliseconds(1));
     clock->advance(kIncrement);
-    metrics->onCopyingEnd();
-    metrics->onApplyingBegin();
+    metrics->setEndFor(TimedPhase::kCloning, getClockSource()->now());
+    metrics->setStartFor(TimedPhase::kApplying, getClockSource()->now());
     metrics->onOplogEntriesApplied(kOpsPerIncrement);
     clock->advance(kIncrement);
     report = metrics->reportForCurrentOp();
@@ -494,7 +495,7 @@ TEST_F(ReshardingMetricsTest, RecipientReportsRemainingTime) {
     // Done.
     metrics->onOplogEntriesApplied(kOpsPerIncrement);
     clock->advance(kIncrement);
-    metrics->onApplyingEnd();
+    metrics->setEndFor(TimedPhase::kApplying, getClockSource()->now());
     report = metrics->reportForCurrentOp();
     ASSERT_EQ(report.getIntField("remainingOperationTimeEstimatedSecs"), 0);
 }
@@ -518,13 +519,43 @@ TEST_F(ReshardingMetricsTest, RecipientRestoreAppliedOplogEntries) {
     ASSERT_EQ(report.getIntField("oplogEntriesApplied"), 30);
 }
 
+TEST_F(ReshardingMetricsTest, CurrentOpReportsCopyingTime) {
+    runTimeReportTest<ReshardingMetrics>(
+        "CurrentOpReportsCopyingTime",
+        {Role::kRecipient, Role::kCoordinator},
+        "totalCopyTimeElapsedSecs",
+        [this](ReshardingMetrics* metrics) {
+            metrics->setStartFor(TimedPhase::kCloning, getClockSource()->now());
+        },
+        [this](ReshardingMetrics* metrics) {
+            metrics->setEndFor(TimedPhase::kCloning, getClockSource()->now());
+        });
+}
+
 TEST_F(ReshardingMetricsTest, CurrentOpReportsApplyingTime) {
     runTimeReportTest<ReshardingMetrics>(
         "CurrentOpReportsApplyingTime",
         {Role::kRecipient, Role::kCoordinator},
         "totalApplyTimeElapsedSecs",
-        [](ReshardingMetrics* metrics) { metrics->onApplyingBegin(); },
-        [](ReshardingMetrics* metrics) { metrics->onApplyingEnd(); });
+        [this](ReshardingMetrics* metrics) {
+            metrics->setStartFor(TimedPhase::kApplying, getClockSource()->now());
+        },
+        [this](ReshardingMetrics* metrics) {
+            metrics->setEndFor(TimedPhase::kApplying, getClockSource()->now());
+        });
+}
+
+TEST_F(ReshardingMetricsTest, CurrentOpReportsCriticalSectionTime) {
+    runTimeReportTest<ReshardingMetrics>(
+        "CurrentOpReportsCriticalSectionTime",
+        {Role::kDonor, Role::kCoordinator},
+        "totalCriticalSectionTimeElapsedSecs",
+        [this](ReshardingMetrics* metrics) {
+            metrics->setStartFor(TimedPhase::kCriticalSection, getClockSource()->now());
+        },
+        [this](ReshardingMetrics* metrics) {
+            metrics->setEndFor(TimedPhase::kCriticalSection, getClockSource()->now());
+        });
 }
 
 TEST_F(ReshardingMetricsTest, RecipientEstimatesNoneOnNewInstance) {
@@ -568,21 +599,23 @@ TEST_F(ReshardingMetricsTest, OnDeleteAppliedIncrementsCumulativeMetrics) {
 
 TEST_F(ReshardingMetricsTest, OnOplogFetchedIncrementsCumulativeMetricsFetchedCount) {
     createMetricsAndAssertIncrementsCumulativeMetricsField(
-        [](auto metrics) { metrics->onOplogEntriesFetched(1, Milliseconds{0}); },
+        [](auto metrics) { metrics->onOplogEntriesFetched(1); },
         Section::kActive,
         "oplogEntriesFetched");
 }
 
-TEST_F(ReshardingMetricsTest, OnOplogFetchedIncrementsCumulativeMetricsFetchedBatchCount) {
+TEST_F(ReshardingMetricsTest,
+       OnBatchRetrievedDuringOplogFetchingIncrementsCumulativeMetricsFetchedBatchCount) {
     createMetricsAndAssertIncrementsCumulativeMetricsField(
-        [](auto metrics) { metrics->onOplogEntriesFetched(0, Milliseconds{0}); },
+        [](auto metrics) { metrics->onBatchRetrievedDuringOplogFetching(Milliseconds{0}); },
         Section::kLatencies,
         "oplogFetchingTotalRemoteBatchesRetrieved");
 }
 
-TEST_F(ReshardingMetricsTest, OnOplogFetchedIncrementsCumulativeMetricsFetchedBatchTime) {
+TEST_F(ReshardingMetricsTest,
+       OnBatchRetrievedDuringOplogFetchingIncrementsCumulativeMetricsFetchedBatchTime) {
     createMetricsAndAssertIncrementsCumulativeMetricsField(
-        [](auto metrics) { metrics->onOplogEntriesFetched(0, Milliseconds{1}); },
+        [](auto metrics) { metrics->onBatchRetrievedDuringOplogFetching(Milliseconds{1}); },
         Section::kLatencies,
         "oplogFetchingTotalRemoteBatchRetrievalTimeMillis");
 }
@@ -639,8 +672,7 @@ TEST_F(ReshardingMetricsTest, OnApplyingBatchAppliedIncrementsCumulativeMetricsB
 TEST_F(ReshardingMetricsTest, OnStateTransitionFromNoneInformsCumulativeMetrics) {
     createMetricsAndAssertIncrementsCumulativeMetricsField(
         [](auto metrics) {
-            metrics->onStateTransition(
-                boost::none, ReshardingMetrics::CoordinatorState{CoordinatorStateEnum::kApplying});
+            metrics->onStateTransition(boost::none, CoordinatorStateEnum::kApplying);
         },
         Section::kCurrentInSteps,
         "countInstancesInCoordinatorState4Applying");
@@ -648,7 +680,7 @@ TEST_F(ReshardingMetricsTest, OnStateTransitionFromNoneInformsCumulativeMetrics)
 
 TEST_F(ReshardingMetricsTest, OnStateTransitionToNoneInformsCumulativeMetrics) {
     auto metrics = createInstanceMetrics(getClockSource(), UUID::gen(), Role::kCoordinator);
-    auto state = ReshardingMetrics::CoordinatorState{CoordinatorStateEnum::kApplying};
+    auto state = CoordinatorStateEnum::kApplying;
     metrics->onStateTransition(boost::none, state);
     assertDecrementsCumulativeMetricsField(
         metrics.get(),
@@ -662,8 +694,8 @@ TEST_F(ReshardingMetricsTest, OnStateTransitionToNoneInformsCumulativeMetrics) {
 
 TEST_F(ReshardingMetricsTest, OnStateTransitionInformsCumulativeMetrics) {
     auto metrics = createInstanceMetrics(getClockSource(), UUID::gen(), Role::kCoordinator);
-    auto initialState = ReshardingMetrics::CoordinatorState{CoordinatorStateEnum::kApplying};
-    auto nextState = ReshardingMetrics::CoordinatorState{CoordinatorStateEnum::kBlockingWrites};
+    auto initialState = CoordinatorStateEnum::kApplying;
+    auto nextState = CoordinatorStateEnum::kBlockingWrites;
     metrics->onStateTransition(boost::none, initialState);
     assertAltersCumulativeMetrics(
         metrics.get(),
