@@ -42,9 +42,9 @@
 #include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/read_concern_level.h"
+#include "mongo/db/s/metrics/sharding_data_transform_cumulative_metrics.h"
 #include "mongo/db/s/resharding/resharding_metrics.h"
 #include "mongo/db/s/resharding/resharding_util.h"
-#include "mongo/db/s/sharding_data_transform_cumulative_metrics.h"
 #include "mongo/db/storage/write_unit_of_work.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/logv2/log.h"
@@ -194,23 +194,22 @@ ExecutorFuture<void> ReshardingOplogFetcher::_reschedule(
 }
 
 bool ReshardingOplogFetcher::iterate(Client* client, CancelableOperationContextFactory factory) {
-    std::shared_ptr<Shard> targetShard;
-    {
-        auto opCtxRaii = factory.makeOperationContext(client);
-        opCtxRaii->checkForInterrupt();
-
-        StatusWith<std::shared_ptr<Shard>> swDonor =
-            Grid::get(opCtxRaii.get())->shardRegistry()->getShard(opCtxRaii.get(), _donorShard);
-        if (!swDonor.isOK()) {
-            LOGV2_WARNING(5127203,
-                          "Error finding shard in registry, retrying.",
-                          "error"_attr = swDonor.getStatus());
-            return true;
-        }
-        targetShard = swDonor.getValue();
-    }
-
     try {
+        std::shared_ptr<Shard> targetShard;
+        {
+            auto opCtxRaii = factory.makeOperationContext(client);
+            opCtxRaii->checkForInterrupt();
+
+            StatusWith<std::shared_ptr<Shard>> swDonor =
+                Grid::get(opCtxRaii.get())->shardRegistry()->getShard(opCtxRaii.get(), _donorShard);
+            if (!swDonor.isOK()) {
+                LOGV2_WARNING(5127203,
+                              "Error finding shard in registry, retrying.",
+                              "error"_attr = swDonor.getStatus());
+                return true;
+            }
+            targetShard = swDonor.getValue();
+        }
         return consume(client, factory, targetShard.get());
     } catch (const ExceptionFor<ErrorCodes::OplogQueryMinTsMissing>&) {
         LOGV2_ERROR(
@@ -236,7 +235,7 @@ void ReshardingOplogFetcher::_ensureCollection(Client* client,
 
     // Create the destination collection if necessary.
     writeConflictRetry(opCtx, "createReshardingLocalOplogBuffer", nss.toString(), [&] {
-        const CollectionPtr coll =
+        const Collection* coll =
             CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
         if (coll) {
             return;
@@ -316,8 +315,9 @@ bool ReshardingOplogFetcher::consume(Client* client,
         [this, &batchesProcessed, &moreToCome, &opCtxRaii, &batchFetchTimer, factory](
             const std::vector<BSONObj>& batch,
             const boost::optional<BSONObj>& postBatchResumeToken) {
-            _env->metrics()->onOplogEntriesFetched(batch.size(),
-                                                   Milliseconds(batchFetchTimer.millis()));
+            _env->metrics()->onOplogEntriesFetched(batch.size());
+            _env->metrics()->onBatchRetrievedDuringOplogFetching(
+                Milliseconds(batchFetchTimer.millis()));
 
             ThreadClient client(fmt::format("ReshardingFetcher-{}-{}",
                                             _reshardingUUID.toString(),
@@ -394,7 +394,7 @@ bool ReshardingOplogFetcher::consume(Client* client,
 
                     // Also include synthetic oplog in the fetched count so it can match up with the
                     // total oplog applied count in the end.
-                    _env->metrics()->onOplogEntriesFetched(1, Milliseconds(0));
+                    _env->metrics()->onOplogEntriesFetched(1);
 
                     auto [p, f] = makePromiseFuture<void>();
                     {
