@@ -47,7 +47,6 @@
 #include "mongo/db/s/resharding/resharding_recipient_service.h"
 #include "mongo/db/s/resharding/resharding_recipient_service_external_state.h"
 #include "mongo/db/s/resharding/resharding_service_test_helpers.h"
-#include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/db/service_context.h"
 #include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/logv2/log.h"
@@ -64,9 +63,8 @@ using RecipientStateTransitionController =
     resharding_service_test_helpers::StateTransitionController<RecipientStateEnum>;
 using PauseDuringStateTransitions =
     resharding_service_test_helpers::PauseDuringStateTransitions<RecipientStateEnum>;
-using OpObserverForTest =
-    resharding_service_test_helpers::OpObserverForTest<RecipientStateEnum,
-                                                       ReshardingRecipientDocument>;
+using OpObserverForTest = resharding_service_test_helpers::
+    StateTransitionControllerOpObserver<RecipientStateEnum, ReshardingRecipientDocument>;
 const ShardId recipientShardId{"myShardId"};
 
 class ExternalStateForTest : public ReshardingRecipientService::RecipientStateMachineExternalState {
@@ -77,8 +75,8 @@ public:
 
     void refreshCatalogCache(OperationContext* opCtx, const NamespaceString& nss) override {}
 
-    ChunkManager getShardedCollectionRoutingInfo(OperationContext* opCtx,
-                                                 const NamespaceString& nss) override {
+    CollectionRoutingInfo getShardedCollectionRoutingInfo(OperationContext* opCtx,
+                                                          const NamespaceString& nss) override {
         invariant(nss == _sourceNss);
 
         const OID epoch = OID::gen();
@@ -97,14 +95,21 @@ public:
                                                Timestamp(1, 1),
                                                boost::none /* timeseriesFields */,
                                                boost::none /* reshardingFields */,
-                                               boost::none /* chunkSizeBytes */,
                                                true /* allowMigrations */,
                                                chunks);
+        IndexCatalogTypeMap shardingIndexesCatalogMap;
+        shardingIndexesCatalogMap.emplace(
+            "randomKey_1",
+            IndexCatalogType(
+                "randomKey_1", BSON("randomKey" << 1), BSONObj(), Timestamp(1, 0), _sourceUUID));
 
-        return ChunkManager(_someDonorId,
-                            DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
-                            _makeStandaloneRoutingTableHistory(std::move(rt)),
-                            boost::none /* clusterTime */);
+        return CollectionRoutingInfo{
+            ChunkManager(_someDonorId,
+                         DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
+                         _makeStandaloneRoutingTableHistory(std::move(rt)),
+                         boost::none /* clusterTime */),
+            ShardingIndexesCatalogCache(CollectionIndexes(_sourceUUID, Timestamp(1, 0)),
+                                        std::move(shardingIndexesCatalogMap))};
     }
 
     MigrationDestinationManager::CollectionOptionsAndUUID getCollectionOptions(
@@ -127,6 +132,11 @@ public:
         return {std::vector<BSONObj>{}, BSONObj()};
     }
 
+    boost::optional<ShardingIndexesCatalogCache> getCollectionIndexInfoWithRefresh(
+        OperationContext* opCtx, const NamespaceString& nss) {
+        return boost::none;
+    }
+
     void withShardVersionRetry(OperationContext* opCtx,
                                const NamespaceString& nss,
                                StringData reason,
@@ -152,23 +162,13 @@ private:
 
     const StringData _currentShardKey = "oldKey";
 
-    const NamespaceString _sourceNss{"sourcedb", "sourcecollection"};
+    const NamespaceString _sourceNss =
+        NamespaceString::createNamespaceString_forTest("sourcedb", "sourcecollection");
     const UUID _sourceUUID = UUID::gen();
 
     const ShardId _someDonorId{"myDonorId"};
 };
 
-class RecipientOpObserverForTest : public OpObserverForTest {
-public:
-    RecipientOpObserverForTest(std::shared_ptr<RecipientStateTransitionController> controller)
-        : OpObserverForTest(std::move(controller),
-                            NamespaceString::kRecipientReshardingOperationsNamespace) {}
-
-    RecipientStateEnum getState(const ReshardingRecipientDocument& recipientDoc) override {
-        return recipientDoc.getMutableState().getState();
-    }
-};
-
 class DataReplicationForTest : public ReshardingDataReplicationInterface {
 public:
     SemiFuture<void> runUntilStrictlyConsistent(
@@ -236,7 +236,12 @@ public:
         repl::StorageInterface::set(serviceContext, std::move(storageMock));
 
         _controller = std::make_shared<RecipientStateTransitionController>();
-        _opObserverRegistry->addObserver(std::make_unique<RecipientOpObserverForTest>(_controller));
+        _opObserverRegistry->addObserver(std::make_unique<OpObserverForTest>(
+            _controller,
+            NamespaceString::kRecipientReshardingOperationsNamespace,
+            [](const ReshardingRecipientDocument& stateDoc) {
+                return stateDoc.getMutableState().getState();
+            }));
     }
 
     RecipientStateTransitionController* controller() {
@@ -257,7 +262,8 @@ public:
                                          ShardId{"donor3"}},
                                         durationCount<Milliseconds>(Milliseconds{5}));
 
-        NamespaceString sourceNss("sourcedb", "sourcecollection");
+        NamespaceString sourceNss =
+            NamespaceString::createNamespaceString_forTest("sourcedb", "sourcecollection");
         auto sourceUUID = UUID::gen();
         auto commonMetadata = CommonReshardingMetadata(
             UUID::gen(),
@@ -275,8 +281,7 @@ public:
                                 const ReshardingRecipientDocument& recipientDoc) {
         CollectionOptions options;
         options.uuid = recipientDoc.getSourceUUID();
-        mongo::sharding_ddl_util::ensureCollectionDroppedNoChangeEvent(opCtx,
-                                                                       recipientDoc.getSourceNss());
+        resharding::data_copy::ensureCollectionDropped(opCtx, recipientDoc.getSourceNss());
         resharding::data_copy::ensureCollectionExists(opCtx, recipientDoc.getSourceNss(), options);
     }
 
@@ -284,8 +289,7 @@ public:
                                         const ReshardingRecipientDocument& recipientDoc) {
         CollectionOptions options;
         options.uuid = recipientDoc.getReshardingUUID();
-        mongo::sharding_ddl_util::ensureCollectionDroppedNoChangeEvent(
-            opCtx, recipientDoc.getTempReshardingNss());
+        resharding::data_copy::ensureCollectionDropped(opCtx, recipientDoc.getTempReshardingNss());
         resharding::data_copy::ensureCollectionExists(
             opCtx, recipientDoc.getTempReshardingNss(), options);
     }
@@ -310,6 +314,15 @@ public:
         ASSERT_TRUE(bool(recipientColl->isEmpty(opCtx)));
     }
 
+    ReshardingRecipientDocument getStateDoc(OperationContext* opCtx) {
+        DBDirectClient client(opCtx);
+
+        auto doc =
+            client.findOne(NamespaceString::kRecipientReshardingOperationsNamespace, BSONObj{});
+        IDLParserContext errCtx("reshardingRecipientFromTest");
+        return ReshardingRecipientDocument::parse(errCtx, doc);
+    }
+
 private:
     TypeCollectionRecipientFields _makeRecipientFields(
         const ReshardingRecipientDocument& recipientDoc) {
@@ -354,14 +367,64 @@ TEST_F(ReshardingRecipientServiceTest, CanTransitionThroughEachStateToCompletion
               "Running case",
               "test"_attr = _agent.getTestName(),
               "isAlsoDonor"_attr = isAlsoDonor);
+        auto removeRecipientDocFailpoint =
+            globalFailPointRegistry().find("removeRecipientDocFailpoint");
+        auto timesEnteredFailPoint = removeRecipientDocFailpoint->setMode(FailPoint::alwaysOn);
         auto doc = makeStateDocument(isAlsoDonor);
         auto opCtx = makeOperationContext();
         RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
         auto recipient = RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
 
         notifyToStartCloning(opCtx.get(), *recipient, doc);
+
         notifyReshardingCommitting(opCtx.get(), *recipient, doc);
 
+        removeRecipientDocFailpoint->waitForTimesEntered(timesEnteredFailPoint + 1);
+
+        // Search metrics in the state document and verify they are valid and the same as the ones
+        // in memory.
+        auto persistedRecipientDocument = getStateDoc(opCtx.get());
+
+        Date_t copyBegin = recipient->getMetrics()
+                               .getStartFor(ReshardingMetrics::TimedPhase::kCloning)
+                               .value_or(Date_t::min());
+        Date_t copyEnd = recipient->getMetrics()
+                             .getEndFor(ReshardingMetrics::TimedPhase::kCloning)
+                             .value_or(Date_t::min());
+        Date_t applyBegin = recipient->getMetrics()
+                                .getStartFor(ReshardingMetrics::TimedPhase::kApplying)
+                                .value_or(Date_t::min());
+        Date_t applyEnd = recipient->getMetrics()
+                              .getEndFor(ReshardingMetrics::TimedPhase::kApplying)
+                              .value_or(Date_t::min());
+
+        auto copyBeginDoc = persistedRecipientDocument.getMetrics()->getDocumentCopy()->getStart();
+        auto copyEndDoc = persistedRecipientDocument.getMetrics()->getDocumentCopy()->getStop();
+        auto applyBeginDoc =
+            persistedRecipientDocument.getMetrics()->getOplogApplication()->getStart();
+        auto applyEndDoc =
+            persistedRecipientDocument.getMetrics()->getOplogApplication()->getStop();
+
+        ASSERT_NE(copyBegin, Date_t::min());
+        ASSERT_NE(copyEnd, Date_t::min());
+        ASSERT_NE(applyBegin, Date_t::min());
+        ASSERT_NE(applyEnd, Date_t::min());
+        ASSERT_LTE(copyBegin, copyEnd);
+        ASSERT_LTE(applyBegin, applyEnd);
+
+        ASSERT_TRUE(copyBeginDoc.has_value());
+        ASSERT_EQ(copyBegin, copyBeginDoc.get());
+
+        ASSERT_TRUE(copyEndDoc.has_value());
+        ASSERT_EQ(copyEnd, copyEndDoc.get());
+
+        ASSERT_TRUE(applyBeginDoc.has_value());
+        ASSERT_EQ(applyBegin, applyBeginDoc.get());
+
+        ASSERT_TRUE(applyEndDoc.has_value());
+        ASSERT_EQ(applyEnd, applyEndDoc.get());
+
+        removeRecipientDocFailpoint->setMode(FailPoint::off);
         ASSERT_OK(recipient->getCompletionFuture().getNoThrow());
         checkStateDocumentRemoved(opCtx.get());
     }
@@ -397,9 +460,10 @@ TEST_F(ReshardingRecipientServiceTest, StepDownStepUpEachTransition) {
                     RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
                     return RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
                 } else {
-                    auto maybeRecipient =
+                    auto [maybeRecipient, isPausedOrShutdown] =
                         RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-                    ASSERT_TRUE(bool(maybeRecipient));
+                    ASSERT_TRUE(maybeRecipient);
+                    ASSERT_FALSE(isPausedOrShutdown);
 
                     // Allow the transition to prevState to succeed on this primary-only service
                     // instance.
@@ -436,8 +500,7 @@ TEST_F(ReshardingRecipientServiceTest, StepDownStepUpEachTransition) {
             stateTransitionsGuard.wait(state);
             stepDown();
 
-            ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-                      ErrorCodes::InterruptedDueToReplStateChange);
+            ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
 
             prevState = state;
 
@@ -446,8 +509,10 @@ TEST_F(ReshardingRecipientServiceTest, StepDownStepUpEachTransition) {
         }
 
         // Finally complete the operation and ensure its success.
-        auto maybeRecipient = RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-        ASSERT_TRUE(bool(maybeRecipient));
+        auto [maybeRecipient, isPausedOrShutdown] =
+            RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(maybeRecipient);
+        ASSERT_FALSE(isPausedOrShutdown);
 
         auto recipient = *maybeRecipient;
 
@@ -458,6 +523,55 @@ TEST_F(ReshardingRecipientServiceTest, StepDownStepUpEachTransition) {
     }
 }
 
+TEST_F(ReshardingRecipientServiceTest, ReportForCurrentOpAfterCompletion) {
+    const auto recipientState = RecipientStateEnum::kCreatingCollection;
+
+    PauseDuringStateTransitions stateTransitionsGuard{controller(), recipientState};
+    auto doc = makeStateDocument(false /* isAlsoDonor */);
+    auto instanceId =
+        BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName << doc.getReshardingUUID());
+    auto opCtx = makeOperationContext();
+
+    auto recipient = [&] {
+        RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
+        return RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
+    }();
+
+    notifyToStartCloning(opCtx.get(), *recipient, doc);
+
+    // Step down before the transition to state can complete.
+    stateTransitionsGuard.wait(recipientState);
+    stepDown();
+    stateTransitionsGuard.unset(recipientState);
+
+    // At this point, the resharding metrics will have been unregistered from the cumulative metrics
+    ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
+
+    // Now call step up. The old recipient object has not yet been destroyed because we still hold
+    // a shared pointer to it ('recipient') - this can happen in production after a failover if a
+    // state machine is slow to clean up.
+    stepUp(opCtx.get());
+
+    // Assert that the old recipient object will return a currentOp report, because the resharding
+    // metrics still exist on the coordinator object itelf.
+    ASSERT(
+        recipient->reportForCurrentOp(MongoProcessInterface::CurrentOpConnectionsMode::kExcludeIdle,
+                                      MongoProcessInterface::CurrentOpSessionsMode::kIncludeIdle));
+
+    // Ensure the new recipient started up successfully (and thus, registered new resharding
+    // metrics), despite the "zombie" state machine still existing.
+    auto [maybeRecipient, isPausedOrShutdown] =
+        RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
+    ASSERT_TRUE(maybeRecipient);
+    ASSERT_FALSE(isPausedOrShutdown);
+    auto newRecipient = *maybeRecipient;
+    ASSERT_NE(recipient, newRecipient);
+
+    // No need to finish the resharding op, so we just cancel the op.
+    newRecipient->abort(false);
+    ASSERT_OK(newRecipient->getCompletionFuture().getNoThrow());
+}
+
 TEST_F(ReshardingRecipientServiceTest, OpCtxKilledWhileRestoringMetrics) {
     for (bool isAlsoDonor : {false, true}) {
         LOGV2(5992701,
@@ -492,8 +606,10 @@ TEST_F(ReshardingRecipientServiceTest, OpCtxKilledWhileRestoringMetrics) {
         stepUp(opCtx.get());
 
         // After the failpoint is disabled, the operation should succeed.
-        auto maybeRecipient = RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-        ASSERT_TRUE(bool(maybeRecipient));
+        auto [maybeRecipient, isPausedOrShutdown] =
+            RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(maybeRecipient);
+        ASSERT_FALSE(isPausedOrShutdown);
         recipient = *maybeRecipient;
         notifyReshardingCommitting(opCtx.get(), *recipient, doc);
         ASSERT_OK(recipient->getCompletionFuture().getNoThrow());
@@ -547,14 +663,15 @@ TEST_F(ReshardingRecipientServiceTest, DropsTemporaryReshardingCollectionOnAbort
         doneTransitionGuard->wait(RecipientStateEnum::kDone);
         stepDown();
 
-        ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-                  ErrorCodes::InterruptedDueToReplStateChange);
+        ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
 
         recipient.reset();
         stepUp(opCtx.get());
 
-        auto maybeRecipient = RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-        ASSERT_TRUE(bool(maybeRecipient));
+        auto [maybeRecipient, isPausedOrShutdown] =
+            RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(maybeRecipient);
+        ASSERT_FALSE(isPausedOrShutdown);
         recipient = *maybeRecipient;
 
         doneTransitionGuard.reset();
@@ -632,8 +749,7 @@ TEST_F(ReshardingRecipientServiceTest, WritesNoopOplogEntryOnReshardDoneCatchUp)
 
     stepDown();
     doneTransitionGuard.reset();
-    ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-              ErrorCodes::InterruptedDueToReplStateChange);
+    ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
 
     DBDirectClient client(opCtx.get());
     NamespaceString sourceNss =
@@ -679,8 +795,7 @@ TEST_F(ReshardingRecipientServiceTest, WritesNoopOplogEntryForImplicitShardColle
 
     stepDown();
     doneTransitionGuard.reset();
-    ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-              ErrorCodes::InterruptedDueToReplStateChange);
+    ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
 
     DBDirectClient client(opCtx.get());
     NamespaceString sourceNss =
@@ -800,9 +915,10 @@ TEST_F(ReshardingRecipientServiceTest, RestoreMetricsAfterStepUp) {
                 RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
                 return RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
             } else {
-                auto maybeRecipient =
+                auto [maybeRecipient, isPausedOrShutdown] =
                     RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-                ASSERT_TRUE(bool(maybeRecipient));
+                ASSERT_TRUE(maybeRecipient);
+                ASSERT_FALSE(isPausedOrShutdown);
 
                 // Allow the transition to prevState to succeed on this primary-only service
                 // instance.
@@ -897,8 +1013,7 @@ TEST_F(ReshardingRecipientServiceTest, RestoreMetricsAfterStepUp) {
         }
         stepDown();
 
-        ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-                  ErrorCodes::InterruptedDueToReplStateChange);
+        ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(), ErrorCodes::CallbackCanceled);
 
         prevState = state;
         if (state == RecipientStateEnum::kApplying ||
