@@ -37,6 +37,7 @@
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/persistent_task_store.h"
 #include "mongo/db/pipeline/process_interface/shardsvr_process_interface.h"
+#include "mongo/db/query/cursor_response.h"
 #include "mongo/db/repl/storage_interface_impl.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/resharding/resharding_server_parameters_gen.h"
@@ -47,6 +48,7 @@
 #include "mongo/db/session/logical_session_cache_noop.h"
 #include "mongo/db/session/session_catalog_mongod.h"
 #include "mongo/db/session/session_txn_record_gen.h"
+#include "mongo/db/transaction/session_catalog_mongod_transaction_interface_impl.h"
 #include "mongo/db/transaction/transaction_participant.h"
 #include "mongo/db/vector_clock_metadata_hook.h"
 #include "mongo/executor/network_interface_factory.h"
@@ -80,10 +82,9 @@ class ReshardingTxnClonerTest : public ShardServerTestFixture {
         auto mockLoader = std::make_unique<CatalogCacheLoaderMock>();
 
         // The config database's primary shard is always config, and it is always sharded.
-        mockLoader->setDatabaseRefreshReturnValue(
-            DatabaseType{NamespaceString::kConfigDb.toString(),
-                         ShardId::kConfigServerId,
-                         DatabaseVersion::makeFixed()});
+        mockLoader->setDatabaseRefreshReturnValue(DatabaseType{DatabaseName::kConfig.toString(),
+                                                               ShardId::kConfigServerId,
+                                                               DatabaseVersion::makeFixed()});
 
         // The config.transactions collection is always unsharded.
         mockLoader->setCollectionRefreshReturnValue(
@@ -108,7 +109,8 @@ class ReshardingTxnClonerTest : public ShardServerTestFixture {
         // onStepUp() relies on the storage interface to create the config.transactions table.
         repl::StorageInterface::set(getServiceContext(),
                                     std::make_unique<repl::StorageInterfaceImpl>());
-        MongoDSessionCatalog::onStepUp(operationContext());
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(operationContext());
+        mongoDSessionCatalog->onStepUp(operationContext());
         LogicalSessionCache::set(getServiceContext(), std::make_unique<LogicalSessionCacheNoop>());
     }
 
@@ -130,7 +132,9 @@ class ReshardingTxnClonerTest : public ShardServerTestFixture {
             StaticCatalogClient(std::vector<ShardId> shardIds) : _shardIds(std::move(shardIds)) {}
 
             StatusWith<repl::OpTimeWith<std::vector<ShardType>>> getAllShards(
-                OperationContext* opCtx, repl::ReadConcernLevel readConcern) override {
+                OperationContext* opCtx,
+                repl::ReadConcernLevel readConcern,
+                bool excludeDraining) override {
                 std::vector<ShardType> shardTypes;
                 for (const auto& shardId : _shardIds) {
                     const ConnectionString cs = ConnectionString::forReplicaSet(
@@ -143,6 +147,15 @@ class ReshardingTxnClonerTest : public ShardServerTestFixture {
                 return repl::OpTimeWith<std::vector<ShardType>>(shardTypes);
             }
 
+            std::pair<CollectionType, std::vector<IndexCatalogType>>
+            getCollectionAndShardingIndexCatalogEntries(
+                OperationContext* opCtx,
+                const NamespaceString& nss,
+                const repl::ReadConcernArgs& readConcern) override {
+                uasserted(ErrorCodes::NamespaceNotFound,
+                          str::stream() << "Collection " << nss.ns() << " not found");
+            }
+
         private:
             const std::vector<ShardId> _shardIds;
         };
@@ -227,6 +240,21 @@ protected:
         });
     }
 
+    void onKillCursorsRequest(CursorId cursorId) {
+        // Handle the 'killCursors' command.
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            ASSERT(request.cmdObj["killCursors"]);
+            auto cursors = request.cmdObj["cursors"];
+            ASSERT_EQ(cursors.type(), BSONType::Array);
+            auto cursorsArray = cursors.Array();
+            ASSERT_FALSE(cursorsArray.empty());
+            ASSERT_EQ(cursorsArray[0].Long(), cursorId);
+            // The AsyncResultsMerger doesn't actually inspect the response of the killCursors, so
+            // we don't have to put anything except {ok: true}.
+            return BSON("ok" << true);
+        });
+    }
+
     void seedTransactionOnRecipient(LogicalSessionId sessionId,
                                     TxnNumber txnNum,
                                     bool multiDocTxn) {
@@ -238,7 +266,8 @@ protected:
             opCtx->setInMultiDocumentTransaction();
         }
 
-        MongoDOperationContextSession ocs(opCtx);
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+        auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
 
         auto txnParticipant = TransactionParticipant::get(opCtx);
         ASSERT(txnParticipant);
@@ -279,7 +308,8 @@ protected:
     void checkTxnHasNotBeenUpdated(LogicalSessionId sessionId, TxnNumber txnNum) {
         auto opCtx = operationContext();
         opCtx->setLogicalSessionId(sessionId);
-        MongoDOperationContextSession ocs(opCtx);
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+        auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
 
         DBDirectClient client(operationContext());
@@ -386,7 +416,8 @@ protected:
         opCtx->setLogicalSessionId(std::move(lsid));
         opCtx->setTxnNumber(txnNumber);
 
-        MongoDOperationContextSession ocs(opCtx);
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+        auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
         txnParticipant.beginOrContinue(
             opCtx, {txnNumber}, false /* autocommit */, true /* startTransaction */);
@@ -402,7 +433,8 @@ protected:
         opCtx->setLogicalSessionId(std::move(lsid));
         opCtx->setTxnNumber(txnNumber);
 
-        MongoDOperationContextSession ocs(opCtx);
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+        auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
         txnParticipant.beginOrContinue(
             opCtx, {txnNumber}, false /* autocommit */, true /* startTransaction */);
@@ -434,7 +466,8 @@ protected:
         opCtx->setLogicalSessionId(std::move(lsid));
         opCtx->setTxnNumber(txnNumber);
 
-        MongoDOperationContextSession ocs(opCtx);
+        auto mongoDSessionCatalog = MongoDSessionCatalog::get(opCtx);
+        auto ocs = mongoDSessionCatalog->checkOutSession(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
         txnParticipant.beginOrContinue(
             opCtx, {txnNumber}, false /* autocommit */, boost::none /* startTransaction */);
@@ -798,6 +831,8 @@ TEST_F(ReshardingTxnClonerTest, ClonerStoresProgressMultipleBatches) {
     auto status = future.getNoThrow();
     ASSERT_EQ(status, ErrorCodes::CallbackCanceled);
 
+    onKillCursorsRequest(CursorId{123});
+
     // After the first batch, the progress document should contain the lsid of the last document in
     // that batch.
     ASSERT_FALSE(getTxnCloningProgress(kTwoSourceIdList[0]));
@@ -851,6 +886,8 @@ TEST_F(ReshardingTxnClonerTest, ClonerStoresProgressResume) {
     auto status = future.getNoThrow();
     ASSERT_EQ(status, ErrorCodes::CallbackCanceled);
 
+    onKillCursorsRequest(CursorId{123});
+
     // The stored progress should be unchanged.
     ASSERT_FALSE(getTxnCloningProgress(kTwoSourceIdList[0]));
     ASSERT_EQ(*getProgressLsid(kTwoSourceIdList[1]), firstLsid);
