@@ -27,9 +27,6 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/db/s/resharding/resharding_util.h"
 
 #include <fmt/format.h>
@@ -61,7 +58,6 @@
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kResharding
 
-
 namespace mongo {
 namespace resharding {
 
@@ -144,7 +140,7 @@ std::set<ShardId> getRecipientShards(OperationContext* opCtx,
                                      const UUID& reshardingUUID) {
     const auto& tempNss = constructTemporaryReshardingNss(sourceNss.db(), reshardingUUID);
     auto* catalogCache = Grid::get(opCtx)->catalogCache();
-    auto cm = uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, tempNss));
+    auto [cm, _] = uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, tempNss));
 
     uassert(ErrorCodes::NamespaceNotSharded,
             str::stream() << "Expected collection " << tempNss << " to be sharded",
@@ -171,7 +167,7 @@ void checkForHolesAndOverlapsInChunks(std::vector<ReshardedChunk>& chunks,
                                                         keyPattern.globalMax()));
 
     boost::optional<BSONObj> prevMax = boost::none;
-    for (auto chunk : chunks) {
+    for (const auto& chunk : chunks) {
         if (prevMax) {
             uassert(ErrorCodes::BadValue,
                     "Chunk ranges must be contiguous",
@@ -217,7 +213,7 @@ void checkForOverlappingZones(std::vector<ReshardingZoneType>& zones) {
         });
 
     boost::optional<BSONObj> prevMax = boost::none;
-    for (auto zone : zones) {
+    for (const auto& zone : zones) {
         if (prevMax) {
             uassert(ErrorCodes::BadValue,
                     "Zone ranges must not overlap",
@@ -228,10 +224,13 @@ void checkForOverlappingZones(std::vector<ReshardingZoneType>& zones) {
 }
 
 std::vector<BSONObj> buildTagsDocsFromZones(const NamespaceString& tempNss,
-                                            const std::vector<ReshardingZoneType>& zones) {
+                                            std::vector<ReshardingZoneType>& zones,
+                                            const ShardKeyPattern& shardKey) {
     std::vector<BSONObj> tags;
     tags.reserve(zones.size());
-    for (const auto& zone : zones) {
+    for (auto& zone : zones) {
+        zone.setMin(shardKey.getKeyPattern().extendRangeBound(zone.getMin(), false));
+        zone.setMax(shardKey.getKeyPattern().extendRangeBound(zone.getMax(), false));
         ChunkRange range(zone.getMin(), zone.getMax());
         TagsType tag(tempNss, zone.getZone().toString(), range);
         tags.push_back(tag.toBSON());
@@ -352,14 +351,13 @@ bool isFinalOplog(const repl::OplogEntry& oplog, UUID reshardingUUID) {
 }
 
 NamespaceString getLocalOplogBufferNamespace(UUID existingUUID, ShardId donorShardId) {
-    return NamespaceString("config.localReshardingOplogBuffer.{}.{}"_format(
-        existingUUID.toString(), donorShardId.toString()));
+    return NamespaceString::makeReshardingLocalOplogBufferNSS(existingUUID,
+                                                              donorShardId.toString());
 }
 
 NamespaceString getLocalConflictStashNamespace(UUID existingUUID, ShardId donorShardId) {
-    return NamespaceString{NamespaceString::kConfigDb,
-                           "localReshardingConflictStash.{}.{}"_format(existingUUID.toString(),
-                                                                       donorShardId.toString())};
+    return NamespaceString::makeReshardingLocalConflictStashNSS(existingUUID,
+                                                                donorShardId.toString());
 }
 
 void doNoopWrite(OperationContext* opCtx, StringData opStr, const NamespaceString& nss) {
