@@ -32,7 +32,6 @@
 
 #include "mongo/db/s/transaction_coordinator_service.h"
 
-#include "mongo/db/internal_transactions_feature_flag_gen.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/transaction_coordinator_document_gen.h"
 #include "mongo/db/s/transaction_coordinator_params_gen.h"
@@ -48,6 +47,8 @@
 namespace mongo {
 namespace {
 
+MONGO_FAIL_POINT_DEFINE(hangBeforeTxnCoordinatorOnStepUpWork);
+
 const auto transactionCoordinatorServiceDecoration =
     ServiceContext::declareDecoration<TransactionCoordinatorService>();
 
@@ -198,6 +199,16 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
             .scheduleWorkIn(
                 recoveryDelayForTesting,
                 [catalogAndScheduler = _catalogAndScheduler](OperationContext* opCtx) {
+                    if (MONGO_unlikely(hangBeforeTxnCoordinatorOnStepUpWork.shouldFail())) {
+                        LOGV2(8288301, "Hit hangBeforeTxnCoordinatorOnStepUpWork failpoint");
+                        hangBeforeTxnCoordinatorOnStepUpWork.pauseWhileSet(opCtx);
+                    }
+
+                    // Skip ticket acquisition in order to prevent possible deadlock when
+                    // participants are in the prepared state. See SERVER-82883 and SERVER-60682.
+                    ScopedAdmissionPriorityForLock skipTicketAcquisition(
+                        opCtx->lockState(), AdmissionContext::Priority::kImmediate);
+
                     auto& replClientInfo = repl::ReplClientInfo::forClient(opCtx->getClient());
                     replClientInfo.setLastOpToSystemLastOpTime(opCtx);
 
@@ -217,7 +228,6 @@ void TransactionCoordinatorService::onStepUp(OperationContext* opCtx,
                                             WriteConcernOptions::kNoTimeout},
                         &unusedWCResult));
 
-                    FlowControl::Bypass flowControlBypass(opCtx);
                     auto coordinatorDocs = txn::readAllCoordinatorDocs(opCtx);
 
                     LOGV2(22452,
