@@ -100,9 +100,8 @@ void setGlobalServiceContext(ServiceContext::UniqueServiceContext&& serviceConte
 
 ServiceContext::ServiceContext()
     : _opIdRegistry(UniqueOperationIdRegistry::create()),
-      _tickSource(std::make_unique<SystemTickSource>()),
+      _tickSource(makeSystemTickSource()),
       _fastClockSource(std::make_unique<SystemClockSource>()),
-      _clockTickSource(_fastClockSource.get()),
       _preciseClockSource(std::make_unique<SystemClockSource>()) {}
 
 
@@ -170,8 +169,8 @@ void onCreate(T* object, const ObserversContainer& observers) {
 
 }  // namespace
 
-ServiceContext::UniqueClient ServiceContext::makeClient(std::string desc,
-                                                        transport::SessionHandle session) {
+ServiceContext::UniqueClient ServiceContext::makeClient(
+    std::string desc, std::shared_ptr<transport::Session> session) {
     std::unique_ptr<Client> client(new Client(std::move(desc), this, std::move(session)));
     onCreate(client.get(), _clientObservers);
     {
@@ -213,7 +212,6 @@ void ServiceContext::setTickSource(std::unique_ptr<TickSource> newSource) {
 }
 
 void ServiceContext::setFastClockSource(std::unique_ptr<ClockSource> newSource) {
-    _clockTickSource.setClockSource(newSource.get());
     _fastClockSource = std::move(newSource);
 }
 
@@ -258,7 +256,10 @@ ServiceContext::UniqueOperationContext ServiceContext::makeOperationContext(Clie
     onCreate(opCtx.get(), _clientObservers);
     ScopeGuard onCreateGuard([&] { onDestroy(opCtx.get(), _clientObservers); });
 
-    invariant(opCtx->lockState(), ProcessInfo().getProcessName());
+    invariant(
+        opCtx->lockState(),
+        str::stream() << "No lock state configured. This could be a missing build dependency. "
+                      << ProcessInfo().getProcessName());
 
     if (!opCtx->recoveryUnit()) {
         opCtx->setRecoveryUnit(std::make_unique<RecoveryUnitNoop>(),
@@ -295,7 +296,7 @@ ServiceContext::UniqueOperationContext ServiceContext::makeOperationContext(Clie
     batonGuard.dismiss();
 
     {
-        stdx::lock_guard lk(_mutex);
+        stdx::lock_guard lk(_clientByOpIdMutex);
         bool clientByOperationContextInsertionSuccessful =
             _clientByOperationId.insert({opCtx->getOpID(), client}).second;
         invariant(clientByOperationContextInsertionSuccessful);
@@ -319,7 +320,8 @@ void ServiceContext::OperationContextDeleter::operator()(OperationContext* opCtx
 }
 
 LockedClient ServiceContext::getLockedClient(OperationId id) {
-    stdx::lock_guard lk(_mutex);
+    stdx::lock_guard lk(_clientByOpIdMutex);
+
     auto it = _clientByOperationId.find(id);
     if (it == _clientByOperationId.end()) {
         return {};
@@ -396,7 +398,7 @@ void ServiceContext::_delistOperation(OperationContext* opCtx) noexcept {
     // its client to prevent situations that another thread could use the service context to get a
     // hold of an `opCtx` that has been removed from its client.
     {
-        stdx::lock_guard lk(_mutex);
+        stdx::lock_guard lk(_clientByOpIdMutex);
         if (_clientByOperationId.erase(opCtx->getOpID()) != 1) {
             // Another thread has already delisted this `opCtx`.
             return;
@@ -492,7 +494,8 @@ ServiceContext::ConstructorActionRegisterer::ConstructorActionRegisterer(
     ConstructorAction constructor,
     DestructorAction destructor) {
     if (!destructor)
-        destructor = [](ServiceContext*) {};
+        destructor = [](ServiceContext*) {
+        };
     _registerer.emplace(
         std::move(name),
         [this, constructor, destructor](InitializerContext*) {
