@@ -41,7 +41,6 @@
 #include "mongo/db/catalog/database_holder_impl.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/global_settings.h"
-#include "mongo/db/index/index_access_method_factory_impl.h"
 #include "mongo/db/index_builds_coordinator_mongod.h"
 #include "mongo/db/op_observer/op_observer_registry.h"
 #include "mongo/db/repl/repl_settings.h"
@@ -49,7 +48,9 @@
 #include "mongo/db/s/collection_sharding_state_factory_shard.h"
 #include "mongo/db/service_entry_point_mongod.h"
 #include "mongo/db/storage/control/storage_control.h"
+#include "mongo/db/storage/execution_control/concurrency_adjustment_parameters_gen.h"
 #include "mongo/db/storage/storage_engine_init.h"
+#include "mongo/db/storage/storage_engine_parameters_gen.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
@@ -64,6 +65,11 @@ namespace mongo {
 ServiceContextMongoDTest::ServiceContextMongoDTest(Options options)
     : _journalListener(std::move(options._journalListener)),
       _tempDir("service_context_d_test_fixture") {
+    gStorageEngineConcurrencyAdjustmentAlgorithm = "fixedConcurrentTransactions";
+
+    if (options._forceDisableTableLogging) {
+        storageGlobalParams.forceDisableTableLogging = true;
+    }
 
     if (options._useReplSettings) {
         repl::ReplSettings replSettings;
@@ -94,6 +100,15 @@ ServiceContextMongoDTest::ServiceContextMongoDTest(Options options)
     }
 
     auto const serviceContext = getServiceContext();
+
+    // Authorization manager override must occur before creating new threads like JournalFlusher
+    if (options._mockAuthzExternalState) {
+        _authzExternalState = options._mockAuthzExternalState.get();
+        auto uniqueAuthzManager = std::make_unique<AuthorizationManagerImpl>(
+            serviceContext, std::move(options._mockAuthzExternalState));
+        AuthorizationManager::set(serviceContext, std::move(uniqueAuthzManager));
+    }
+
     if (options._useMockClock) {
         // Copied from dbtests.cpp. DBTests sets up a controlled mock clock while
         // ServiceContextMongoDTest uses the system clock. Tests moved from dbtests to unittests may
@@ -135,10 +150,10 @@ ServiceContextMongoDTest::ServiceContextMongoDTest(Options options)
     // for faster startup.
     auto opCtx = serviceContext->makeOperationContext(getClient());
     initializeStorageEngine(opCtx.get(), options._initFlags);
+
     StorageControl::startStorageControls(serviceContext, true /*forTestOnly*/);
 
     DatabaseHolder::set(serviceContext, std::make_unique<DatabaseHolderImpl>());
-    IndexAccessMethodFactory::set(serviceContext, std::make_unique<IndexAccessMethodFactoryImpl>());
     Collection::Factory::set(serviceContext, std::make_unique<CollectionImpl::FactoryImpl>());
     IndexBuildsCoordinator::set(serviceContext, std::make_unique<IndexBuildsCoordinatorMongod>());
     CollectionShardingStateFactory::set(
@@ -174,6 +189,8 @@ ServiceContextMongoDTest::~ServiceContextMongoDTest() {
     std::swap(storageGlobalParams.repair, _stashedStorageParams.repair);
     std::swap(serverGlobalParams.enableMajorityReadConcern,
               _stashedServerParams.enableMajorityReadConcern);
+
+    storageGlobalParams.reset();
 }
 
 void ServiceContextMongoDTest::tearDown() {
