@@ -66,7 +66,7 @@ class LogicalSessionIdTest : public ServiceContextTest {
 public:
     AuthzManagerExternalStateMock* managerState;
     transport::TransportLayerMock transportLayer;
-    transport::SessionHandle session = transportLayer.createSession();
+    std::shared_ptr<transport::Session> session = transportLayer.createSession();
     ServiceContext::UniqueOperationContext _opCtx;
     AuthorizationSession* authzSession;
 
@@ -111,7 +111,7 @@ public:
                                            << "db"
                                            << "test"))),
             BSONObj()));
-        ASSERT_OK(authzSession->addAndAuthorizeUser(_opCtx.get(), un));
+        ASSERT_OK(authzSession->addAndAuthorizeUser(_opCtx.get(), {un, boost::none}, boost::none));
         return authzSession->lookupUser(un);
     }
 
@@ -126,7 +126,7 @@ public:
                                            << "db"
                                            << "admin"))),
             BSONObj()));
-        ASSERT_OK(authzSession->addAndAuthorizeUser(_opCtx.get(), un));
+        ASSERT_OK(authzSession->addAndAuthorizeUser(_opCtx.get(), {un, boost::none}, boost::none));
         return authzSession->lookupUser(un);
     }
 };
