@@ -34,7 +34,6 @@
 
 #include <memory>
 
-#include "mongo/db/internal_transactions_feature_flag_gen.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/session/logical_session_id_helpers.h"
@@ -53,6 +52,16 @@ const auto operationSessionDecoration =
 
 MONGO_FAIL_POINT_DEFINE(hangAfterIncrementingNumWaitingToCheckOut);
 
+std::string provenanceToString(SessionCatalog::Provenance provenance) {
+    switch (provenance) {
+        case SessionCatalog::Provenance::kRouter:
+            return "router";
+        case SessionCatalog::Provenance::kParticipant:
+            return "participant";
+    }
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace
 
 SessionCatalog::~SessionCatalog() {
@@ -315,10 +324,11 @@ SessionCatalog::SessionRuntimeInfo* SessionCatalog::_getOrCreateSessionRuntimeIn
     return sri;
 }
 
-void SessionCatalog::_releaseSession(SessionRuntimeInfo* sri,
-                                     Session* session,
-                                     boost::optional<KillToken> killToken,
-                                     boost::optional<TxnNumber> clientTxnNumberStarted) {
+void SessionCatalog::_releaseSession(
+    SessionRuntimeInfo* sri,
+    Session* session,
+    boost::optional<KillToken> killToken,
+    boost::optional<TxnNumberAndProvenance> clientTxnNumberStarted) {
     stdx::unique_lock<Latch> ul(_mutex);
 
     // Make sure we have exactly the same session on the map and that it is still associated with an
@@ -341,29 +351,34 @@ void SessionCatalog::_releaseSession(SessionRuntimeInfo* sri,
 
     std::vector<LogicalSessionId> eagerlyReapedSessions;
     if (clientTxnNumberStarted.has_value()) {
+        auto [txnNumber, provenance] = *clientTxnNumberStarted;
+
         // Since the given txnNumber successfully started, we know any child sessions with older
         // txnNumbers can be discarded. This needed to wait until a transaction started because that
         // can fail, e.g. if the active transaction is prepared.
+        auto workerFn = _makeSessionWorkerFnForEagerReap(service, txnNumber, provenance);
         auto numReaped = stdx::erase_if(sri->childSessions, [&](auto&& it) {
             ObservableSession osession(ul, sri, &it.second);
-            if (it.first.getTxnNumber() && *it.first.getTxnNumber() < *clientTxnNumberStarted) {
-                osession.markForReap(ObservableSession::ReapMode::kExclusive);
-            }
+            workerFn(osession);
 
-            bool willReap = osession._shouldBeReaped();
+            bool willReap = osession._shouldBeReaped() &&
+                (osession._reapMode == ObservableSession::ReapMode::kExclusive);
             if (willReap) {
                 eagerlyReapedSessions.push_back(std::move(it.first));
             }
             return willReap;
         });
 
+        sri->lastClientTxnNumberStarted = txnNumber;
+
         LOGV2_DEBUG(6685200,
                     4,
                     "Erased child sessions",
                     "releasedLsid"_attr = session->getSessionId(),
-                    "clientTxnNumber"_attr = *clientTxnNumberStarted,
+                    "clientTxnNumber"_attr = txnNumber,
                     "childSessionsRemaining"_attr = sri->childSessions.size(),
-                    "numReaped"_attr = numReaped);
+                    "numReaped"_attr = numReaped,
+                    "provenance"_attr = provenanceToString(provenance));
     }
 
     invariant(ul);
@@ -374,6 +389,19 @@ void SessionCatalog::_releaseSession(SessionRuntimeInfo* sri,
     }
 }
 
+SessionCatalog::ScanSessionsCallbackFn SessionCatalog::_defaultMakeSessionWorkerFnForEagerReap(
+    ServiceContext* service, TxnNumber clientTxnNumberStarted, Provenance provenance) {
+    return [clientTxnNumberStarted](ObservableSession& osession) {
+        // If a higher txnNumber has been seen for a client and started a transaction, assume any
+        // child sessions for lower transactions have been superseded and can be reaped.
+        const auto& transactionSessionId = osession.getSessionId();
+        if (transactionSessionId.getTxnNumber() &&
+            *transactionSessionId.getTxnNumber() < clientTxnNumberStarted) {
+            osession.markForReap(ObservableSession::ReapMode::kExclusive);
+        }
+    };
+}
+
 Session* SessionCatalog::SessionRuntimeInfo::getSession(WithLock, const LogicalSessionId& lsid) {
     if (isParentSessionId(lsid)) {
         // We should have already compared the parent lsid when we found this SRI.
@@ -394,16 +422,9 @@ SessionCatalog::KillToken ObservableSession::kill(ErrorCodes::Error reason) cons
     ++_sri->killsRequested;
 
     if (firstKiller && hasCurrentOperation()) {
-        // Interrupt the current OperationContext if its running on the transaction session
-        // that is being killed or if we are killing the parent transaction session.
         invariant(_clientLock.owns_lock());
-        const auto checkedOutLsid = _sri->checkoutOpCtx->getLogicalSessionId();
-        const auto lsidToKill = getSessionId();
-        const bool isKillingParentSession = isParentSessionId(lsidToKill);
-        if (isKillingParentSession || (checkedOutLsid == lsidToKill)) {
-            const auto serviceContext = _sri->checkoutOpCtx->getServiceContext();
-            serviceContext->killOperation(_clientLock, _sri->checkoutOpCtx, reason);
-        }
+        const auto serviceContext = _sri->checkoutOpCtx->getServiceContext();
+        serviceContext->killOperation(_clientLock, _sri->checkoutOpCtx, reason);
     }
 
     return SessionCatalog::KillToken(getSessionId());
@@ -521,9 +542,10 @@ void OperationContextSession::checkOut(OperationContext* opCtx) {
     checkedOutSession.emplace(std::move(scopedCheckedOutSession));
 }
 
-void OperationContextSession::observeNewTxnNumberStarted(OperationContext* opCtx,
-                                                         const LogicalSessionId& lsid,
-                                                         TxnNumber txnNumber) {
+void OperationContextSession::observeNewTxnNumberStarted(
+    OperationContext* opCtx,
+    const LogicalSessionId& lsid,
+    SessionCatalog::TxnNumberAndProvenance txnNumberAndProvenance) {
     auto& checkedOutSession = operationSessionDecoration(opCtx);
     invariant(checkedOutSession);
 
@@ -531,7 +553,8 @@ void OperationContextSession::observeNewTxnNumberStarted(OperationContext* opCtx
                 4,
                 "Observing new retryable write number started on session",
                 "lsid"_attr = lsid,
-                "txnNumber"_attr = txnNumber);
+                "txnNumber"_attr = txnNumberAndProvenance.first,
+                "provenance"_attr = txnNumberAndProvenance.second);
 
     const auto& checkedOutLsid = (*checkedOutSession)->getSessionId();
     if (isParentSessionId(lsid)) {
@@ -541,7 +564,7 @@ void OperationContextSession::observeNewTxnNumberStarted(OperationContext* opCtx
         // parent. This is safe because both share the same SessionRuntimeInfo.
         dassert(lsid == checkedOutLsid || lsid == *getParentSessionId(checkedOutLsid));
 
-        checkedOutSession->observeNewClientTxnNumberStarted(txnNumber);
+        checkedOutSession->observeNewClientTxnNumberStarted(txnNumberAndProvenance);
     } else if (isInternalSessionForRetryableWrite(lsid)) {
         // Observing a new internal transaction on a retryable session.
 
@@ -549,7 +572,8 @@ void OperationContextSession::observeNewTxnNumberStarted(OperationContext* opCtx
         // directly.
         dassert(lsid == checkedOutLsid);
 
-        checkedOutSession->observeNewClientTxnNumberStarted(*lsid.getTxnNumber());
+        checkedOutSession->observeNewClientTxnNumberStarted(
+            {*lsid.getTxnNumber(), txnNumberAndProvenance.second});
     }
 }
 
