@@ -100,6 +100,7 @@ public:
     IntWrapper getOwned() const {
         return *this;
     }
+    void makeOwned() {}
 
     std::string toString() const {
         return std::to_string(_i);
@@ -117,10 +118,10 @@ enum Direction { ASC = 1, DESC = -1 };
 class IWComparator {
 public:
     IWComparator(Direction dir = ASC) : _dir(dir) {}
-    int operator()(const IWPair& lhs, const IWPair& rhs) const {
-        if (lhs.first == rhs.first)
+    int operator()(const IntWrapper& lhs, const IntWrapper& rhs) const {
+        if (lhs == rhs)
             return 0;
-        if (lhs.first < rhs.first)
+        if (lhs < rhs)
             return -1 * _dir;
         return 1 * _dir;
     }
@@ -165,7 +166,7 @@ public:
         return false;
     }
     Data next() {
-        verify(false);
+        MONGO_verify(false);
     }
     const Data& current() {
         MONGO_UNREACHABLE;
@@ -176,7 +177,7 @@ class LimitIterator : public IWIterator {
 public:
     LimitIterator(long long limit, std::shared_ptr<IWIterator> source)
         : _remaining(limit), _source(source) {
-        verify(limit > 0);
+        MONGO_verify(limit > 0);
     }
 
     void openSource() {}
@@ -186,7 +187,7 @@ public:
         return _remaining && _source->more();
     }
     Data next() {
-        verify(more());
+        MONGO_verify(more());
         _remaining--;
         return _source->next();
     }
@@ -350,6 +351,7 @@ public:
         ASSERT_EQ(sorterFileStats.opened.load(), 2);
         ASSERT_EQ(sorterFileStats.closed.load(), 2);
         ASSERT_LTE(sorterTracker.bytesSpilled.load(), currentFileSize);
+        ASSERT_LTE(sorterFileStats.bytesSpilled(), currentFileSize);
 
         ASSERT(boost::filesystem::is_empty(tempDir.path()));
     }
@@ -484,7 +486,7 @@ public:
                 std::shared_ptr<IWSorter> sorter = makeSorter(opts, IWComparator(ASC));
                 addData(sorter.get());
                 ASSERT_ITERATORS_EQUIVALENT(done(sorter.get()), correct());
-                ASSERT_EQ(numAdded(), sorter->numSorted());
+                ASSERT_EQ(numAdded(), sorter->stats().numSorted());
                 if (assertRanges) {
                     assertRangeInfo(sorter, opts);
                 }
@@ -493,7 +495,7 @@ public:
                 std::shared_ptr<IWSorter> sorter = makeSorter(opts, IWComparator(DESC));
                 addData(sorter.get());
                 ASSERT_ITERATORS_EQUIVALENT(done(sorter.get()), correctReverse());
-                ASSERT_EQ(numAdded(), sorter->numSorted());
+                ASSERT_EQ(numAdded(), sorter->stats().numSorted());
                 if (assertRanges) {
                     assertRangeInfo(sorter, opts);
                 }
@@ -881,7 +883,7 @@ TEST_F(SorterMakeFromExistingRangesTest, SkipFileCheckingOnEmptyRanges) {
     ASSERT_EQ(0, sorter->stats().spilledRanges());
 
     auto iter = std::unique_ptr<IWIterator>(sorter->done());
-    ASSERT_EQ(0, sorter->numSorted());
+    ASSERT_EQ(0, sorter->stats().numSorted());
 
     iter->openSource();
     ASSERT_FALSE(iter->more());
@@ -931,7 +933,7 @@ TEST_F(SorterMakeFromExistingRangesTest, CorruptedFile) {
 
     // The number of spills is set when NoLimitSorter is constructed from existing ranges.
     ASSERT_EQ(makeSampleRanges().size(), sorter->stats().spilledRanges());
-    ASSERT_EQ(0, sorter->numSorted());
+    ASSERT_EQ(0, sorter->stats().numSorted());
 
     // 16817 - error reading file.
     ASSERT_THROWS_CODE(sorter->done(), DBException, 16817);
@@ -961,7 +963,7 @@ TEST_F(SorterMakeFromExistingRangesTest, RoundTrip) {
         state = sorterBeforeShutdown->persistDataForShutdown();
         ASSERT_FALSE(state.fileName.empty());
         ASSERT_EQUALS(1U, state.ranges.size()) << state.ranges.size();
-        ASSERT_EQ(1, sorterBeforeShutdown->numSorted());
+        ASSERT_EQ(1, sorterBeforeShutdown->stats().numSorted());
     }
 
     // On restart, reconstruct sorter from persisted state.
@@ -976,7 +978,7 @@ TEST_F(SorterMakeFromExistingRangesTest, RoundTrip) {
     sorter->add(pairInsertedAfterStartup.first, pairInsertedAfterStartup.second);
 
     // Technically this sorter has not sorted anything. It is just merging files.
-    ASSERT_EQ(0, sorter->numSorted());
+    ASSERT_EQ(0, sorter->stats().numSorted());
 
     // Read data from sorter.
     {
@@ -1039,7 +1041,7 @@ public:
         Key operator()(Key k, const Doc&) const {
             return k - 10;
         }
-        Document serialize() const {
+        Document serialize(const SerializationOptions& opts = {}) const {
             MONGO_UNREACHABLE;
         }
     };
@@ -1047,7 +1049,7 @@ public:
         Key operator()(Key k, const Doc&) const {
             return k + 10;
         }
-        Document serialize() const {
+        Document serialize(const SerializationOptions& opts = {}) const {
             MONGO_UNREACHABLE;
         }
     };
@@ -1061,7 +1063,9 @@ public:
      */
     std::vector<Doc> sort(std::vector<Doc> input, int expectedSize = -1) {
         std::vector<Doc> output;
-        auto push = [&](Doc doc) { output.push_back(doc); };
+        auto push = [&](Doc doc) {
+            output.push_back(doc);
+        };
 
         for (auto&& doc : input) {
             sorter->add(doc.time, doc);
