@@ -56,6 +56,18 @@ namespace {
 
 #define TRANSPARENT_HUGE_PAGES_DIR "/sys/kernel/mm/transparent_hugepage"
 
+inline bool isValidOpMode(const std::string& opMode, const std::string& param) noexcept {
+    if (param == kTHPEnabledParameter) {
+        return opMode == "always" || opMode == "madvise" || opMode == "never";
+    }
+    if (param == kTHPDefragParameter) {
+        return opMode == "always" || opMode == "madvise" || opMode == "never" ||
+            opMode == "defer" || opMode == "defer+madvise";
+    }
+
+    return false;
+}
+
 }  // namespace
 
 using std::ios_base;
@@ -105,8 +117,8 @@ StatusWith<std::string> StartupWarningsMongod::readTransparentHugePagesParameter
                                                << ((ifs.eof()) ? "EOF" : errorMessage(ec)));
         }
 
-        std::string::size_type posBegin = line.find("[");
-        std::string::size_type posEnd = line.find("]");
+        std::string::size_type posBegin = line.find('[');
+        std::string::size_type posEnd = line.find(']');
         if (posBegin == string::npos || posEnd == string::npos || posBegin >= posEnd) {
             return StatusWith<std::string>(ErrorCodes::FailedToParse,
                                            str::stream() << "cannot parse line: '" << line << "'");
@@ -120,7 +132,7 @@ StatusWith<std::string> StartupWarningsMongod::readTransparentHugePagesParameter
         }
 
         // Check against acceptable values of opMode.
-        if (opMode != "always" && opMode != "madvise" && opMode != "never") {
+        if (!isValidOpMode(opMode, parameter)) {
             return StatusWith<std::string>(
                 ErrorCodes::BadValue,
                 str::stream()
@@ -240,17 +252,18 @@ void logMongodStartupWarnings(const StorageGlobalParams& storageParams,
 
     // Transparent Hugepages checks
     StatusWith<std::string> transparentHugePagesEnabledResult =
-        StartupWarningsMongod::readTransparentHugePagesParameter("enabled");
+        StartupWarningsMongod::readTransparentHugePagesParameter(kTHPEnabledParameter);
     bool shouldWarnAboutDefragAlways = false;
     if (transparentHugePagesEnabledResult.isOK()) {
         if (transparentHugePagesEnabledResult.getValue() == "always") {
             // If we do not have hugepages enabled, we don't need to warn about its features
             shouldWarnAboutDefragAlways = true;
 
-            LOGV2_WARNING_OPTIONS(22178,
-                                  {logv2::LogTag::kStartupWarnings},
-                                  TRANSPARENT_HUGE_PAGES_DIR
-                                  "/enabled is 'always'. We suggest setting it to 'never'");
+            LOGV2_WARNING_OPTIONS(
+                22178,
+                {logv2::LogTag::kStartupWarnings},
+                TRANSPARENT_HUGE_PAGES_DIR
+                "/enabled is 'always'. We suggest setting it to 'never' in this binary version");
         }
     } else if (transparentHugePagesEnabledResult.getStatus().code() !=
                ErrorCodes::NonExistentPath) {
@@ -262,14 +275,15 @@ void logMongodStartupWarnings(const StorageGlobalParams& storageParams,
     }
 
     StatusWith<std::string> transparentHugePagesDefragResult =
-        StartupWarningsMongod::readTransparentHugePagesParameter("defrag");
+        StartupWarningsMongod::readTransparentHugePagesParameter(kTHPDefragParameter);
     if (transparentHugePagesDefragResult.isOK()) {
         if (shouldWarnAboutDefragAlways &&
             transparentHugePagesDefragResult.getValue() == "always") {
-            LOGV2_WARNING_OPTIONS(22181,
-                                  {logv2::LogTag::kStartupWarnings},
-                                  TRANSPARENT_HUGE_PAGES_DIR
-                                  "/defrag is 'always'. We suggest setting it to 'never'");
+            LOGV2_WARNING_OPTIONS(
+                22181,
+                {logv2::LogTag::kStartupWarnings},
+                TRANSPARENT_HUGE_PAGES_DIR
+                "/defrag is 'always'. We suggest setting it to 'never' in this binary version");
         }
     } else if (transparentHugePagesDefragResult.getStatus().code() != ErrorCodes::NonExistentPath) {
         LOGV2_WARNING_OPTIONS(22204,
