@@ -31,7 +31,6 @@
 
 #include <map>
 
-#include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/platform/atomic_word.h"
@@ -78,6 +77,10 @@ public:
         _checkWrap(&OpCounters::_queryDeprecated, 1);
     }
 
+    void gotNestedAggregate() {
+        _checkWrap(&OpCounters::_nestedAggregate, 1);
+    }
+
     BSONObj getObj() const;
 
     // These opcounters record operations that would fail if we were fully enforcing our consistency
@@ -117,6 +120,9 @@ public:
     const AtomicWord<long long>* getCommand() const {
         return &*_command;
     }
+    const AtomicWord<long long>* getNestedAggregate() const {
+        return &*_nestedAggregate;
+    }
     const AtomicWord<long long>* getInsertOnExistingDoc() const {
         return &*_insertOnExistingDoc;
     }
@@ -133,7 +139,15 @@ public:
         return &*_acceptableErrorInCommand;
     }
 
+    // Reset all counters. To used for testing purposes only.
+    void resetForTest() {
+        _reset();
+    }
+
 private:
+    // Reset all counters.
+    void _reset();
+
     // Increment member `counter` by `n`, resetting all counters if it was > 2^60.
     void _checkWrap(CacheExclusive<AtomicWord<long long>> OpCounters::*counter, int n);
 
@@ -143,6 +157,7 @@ private:
     CacheExclusive<AtomicWord<long long>> _delete;
     CacheExclusive<AtomicWord<long long>> _getmore;
     CacheExclusive<AtomicWord<long long>> _command;
+    CacheExclusive<AtomicWord<long long>> _nestedAggregate;
 
     CacheExclusive<AtomicWord<long long>> _insertOnExistingDoc;
     CacheExclusive<AtomicWord<long long>> _updateOnMissingDoc;
@@ -248,6 +263,8 @@ public:
 
     void incSaslSupportedMechanismsReceived();
 
+    void incAuthenticationCumulativeTime(long long micros);
+
     void append(BSONObjBuilder*);
 
     void initializeMechanismMap(const std::vector<std::string>&);
@@ -270,7 +287,7 @@ private:
     using MechanismMap = std::map<std::string, MechanismData>;
 
     AtomicWord<long long> _saslSupportedMechanismsReceived;
-
+    AtomicWord<long long> _authenticationCumulativeMicros;
     // Mechanism maps are initialized at startup to contain all
     // mechanisms known to authenticationMechanisms setParam.
     // After that they are kept to a fixed size.
@@ -316,31 +333,40 @@ public:
         auto& debug = curop->debug();
         const BSONObj& cmdObj = curop->opDescription();
         auto cmdName = cmdObj.firstElementFieldNameStringData();
-        if (cmdName == "find" && debug.classicEngineUsed) {
-            if (debug.classicEngineUsed.get()) {
-                classicFindQueryCounter.increment();
-            } else {
-                sbeFindQueryCounter.increment();
+
+        if (cmdName == "find") {
+            switch (debug.queryFramework) {
+                case PlanExecutor::QueryFramework::kClassicOnly:
+                    classicFindQueryCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kSBEOnly:
+                    sbeFindQueryCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kCQF:
+                    cqfFindQueryCounter.increment();
+                    break;
+                default:
+                    break;
             }
-        } else if (cmdName == "aggregate" && debug.classicEngineUsed && debug.documentSourceUsed) {
-            if (debug.classicEngineUsed.get()) {
-                if (debug.documentSourceUsed.get()) {
-                    classicHybridAggregationCounter.increment();
-                } else {
+        } else if (cmdName == "aggregate") {
+            switch (debug.queryFramework) {
+                case PlanExecutor::QueryFramework::kClassicOnly:
                     classicOnlyAggregationCounter.increment();
-                }
-            } else {
-                if (debug.documentSourceUsed.get()) {
-                    sbeHybridAggregationCounter.increment();
-                } else {
+                    break;
+                case PlanExecutor::QueryFramework::kClassicHybrid:
+                    classicHybridAggregationCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kSBEOnly:
                     sbeOnlyAggregationCounter.increment();
-                }
-            }
-        } else if (debug.cqfUsed) {
-            if (cmdName == "find") {
-                cqfFindQueryCounter.increment();
-            } else {
-                cqfAggregationQueryCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kSBEHybrid:
+                    sbeHybridAggregationCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kCQF:
+                    cqfAggregationQueryCounter.increment();
+                    break;
+                case PlanExecutor::QueryFramework::kUnknown:
+                    break;
             }
         }
     }
@@ -367,22 +393,138 @@ class LookupPushdownCounters {
 public:
     LookupPushdownCounters() = default;
 
-    void incrementLookupCounters(OpDebug& debug) {
-        nestedLoopJoin.increment(debug.nestedLoopJoin);
-        indexedLoopJoin.increment(debug.indexedLoopJoin);
-        hashLookup.increment(debug.hashLookup);
-        hashLookupSpillToDisk.increment(debug.hashLookupSpillToDisk);
+    void incrementLookupCountersPerQuery(int nestedLoopJoin, int indexedLoopJoin, int hashLookup) {
+        nestedLoopJoinCounter.increment(nestedLoopJoin);
+        indexedLoopJoinCounter.increment(indexedLoopJoin);
+        hashLookupCounter.increment(hashLookup);
+    }
+
+    void incrementLookupCountersPerSpilling(int64_t spillToDisk, int64_t spillToDiskBytes) {
+        hashLookupSpillToDisk.increment(spillToDisk);
+        hashLookupSpillToDiskBytes.increment(spillToDiskBytes);
     }
 
     // Counters for lookup join strategies.
-    CounterMetric nestedLoopJoin{"query.lookup.nestedLoopJoin"};
-    CounterMetric indexedLoopJoin{"query.lookup.indexedLoopJoin"};
-    CounterMetric hashLookup{"query.lookup.hashLookup"};
+    CounterMetric nestedLoopJoinCounter{"query.lookup.nestedLoopJoin"};
+    CounterMetric indexedLoopJoinCounter{"query.lookup.indexedLoopJoin"};
+    CounterMetric hashLookupCounter{"query.lookup.hashLookup"};
     // Counter tracking hashLookup spills in lookup stages that get pushed down.
     CounterMetric hashLookupSpillToDisk{"query.lookup.hashLookupSpillToDisk"};
+    // Counter tracking hashLookup spilled bytes in lookup stages that get pushed down.
+    CounterMetric hashLookupSpillToDiskBytes{"query.lookup.hashLookupSpillToDiskBytes"};
 };
 extern LookupPushdownCounters lookupPushdownCounters;
 
+class SortCounters {
+public:
+    SortCounters() = default;
+
+    void incrementSortCountersPerQuery(int64_t bytesSorted, int64_t keysSorted) {
+        sortTotalBytesCounter.increment(bytesSorted);
+        sortTotalKeysCounter.increment(keysSorted);
+    }
+
+    void incrementSortCountersPerSpilling(int64_t sortSpills, int64_t sortSpillBytes) {
+        sortSpillsCounter.increment(sortSpills);
+        sortSpillBytesCounter.increment(sortSpillBytes);
+    }
+
+    // Counters tracking sort stats across all engines
+    // The total number of spills from sort stages
+    CounterMetric sortSpillsCounter{"query.sort.spillToDisk"};
+    // The total bytes spilled. This is the storage size after compression.
+    CounterMetric sortSpillBytesCounter{"query.sort.spillToDiskBytes"};
+    // The number of keys that we've sorted.
+    CounterMetric sortTotalKeysCounter{"query.sort.totalKeysSorted"};
+    // The amount of data we've sorted in bytes
+    CounterMetric sortTotalBytesCounter{"query.sort.totalBytesSorted"};
+};
+extern SortCounters sortCounters;
+
+class GroupCounters {
+public:
+    GroupCounters() = default;
+
+    void incrementGroupCountersPerSpilling(int64_t spills,
+                                           int64_t spilledBytes,
+                                           int64_t spilledRecords) {
+        groupSpills.increment(spills);
+        groupSpilledBytes.increment(spilledBytes);
+        groupSpilledRecords.increment(spilledRecords);
+    }
+
+    void incrementGroupCountersPerQuery(int64_t spilledDataStorageSize) {
+        groupSpilledDataStorageSize.increment(spilledDataStorageSize);
+    }
+
+    // The total number of spills from group stages.
+    CounterMetric groupSpills{"query.group.spills"};
+    // The total number of bytes spilled from group stages. The spilled stroage size after
+    // compression might be different from the bytes spilled.
+    CounterMetric groupSpilledBytes{"query.group.spilledBytes"};
+    // The number of records spilled.
+    CounterMetric groupSpilledRecords{"query.group.spilledRecords"};
+    // The size of the file or RecordStore spilled to disk, updated after all spilling happened.
+    CounterMetric groupSpilledDataStorageSize{"query.group.spilledDataStorageSize"};
+};
+extern GroupCounters groupCounters;
+
+/** Counters tracking setWindowFields stats across all execution engines. */
+class SetWindowFieldsCounters {
+public:
+    SetWindowFieldsCounters() = default;
+    SetWindowFieldsCounters(SetWindowFieldsCounters&) = delete;
+    SetWindowFieldsCounters& operator=(const SetWindowFieldsCounters&) = delete;
+
+    void incrementSetWindowFieldsCountersPerSpilling(int64_t spills,
+                                                     int64_t spilledBytes,
+                                                     int64_t spilledRecords) {
+        setWindowFieldsSpills.increment(spills);
+        setWindowFieldsSpilledBytes.increment(spilledBytes);
+        setWindowFieldsSpilledRecords.increment(spilledRecords);
+    }
+
+    // Counter tracking setWindowFields spills.
+    CounterMetric setWindowFieldsSpills{"query.setWindowFields.spills"};
+    // Counter tracking setWindowFields spilled bytes. The spilled storage size after compression
+    // might be different from the bytes spilled.
+    CounterMetric setWindowFieldsSpilledBytes{"query.setWindowFields.spilledBytes"};
+    // Counter tracking setWindowFields spilled record number.
+    CounterMetric setWindowFieldsSpilledRecords{"query.setWindowFields.spilledRecords"};
+};
+extern SetWindowFieldsCounters setWindowFieldsCounters;
+
+/**
+ * A common class which holds various counters related to Classic and SBE plan caches.
+ */
+class PlanCacheCounters {
+public:
+    PlanCacheCounters() = default;
+
+    void incrementClassicHitsCounter() {
+        classicHits.increment();
+    }
+
+    void incrementClassicMissesCounter() {
+        classicMisses.increment();
+    }
+
+    void incrementSbeHitsCounter() {
+        sbeHits.increment();
+    }
+
+    void incrementSbeMissesCounter() {
+        sbeMisses.increment();
+    }
+
+private:
+    CounterMetric classicHits{"query.planCache.classic.hits"};
+    CounterMetric classicMisses{"query.planCache.classic.misses"};
+    CounterMetric sbeHits{"query.planCache.sbe.hits"};
+    CounterMetric sbeMisses{"query.planCache.sbe.misses"};
+};
+extern PlanCacheCounters planCacheCounters;
+
 /**
  * Generic class for counters of expressions inside various MQL statements.
  */
@@ -415,6 +557,49 @@ private:
     StringMap<std::unique_ptr<ExprCounter>> operatorCountersExprMap = {};
 };
 
+class ValidatorCounters {
+public:
+    ValidatorCounters() {
+        _validatorCounterMap["create"] = std::make_unique<ValidatorCounter>("create");
+        _validatorCounterMap["collMod"] = std::make_unique<ValidatorCounter>("collMod");
+    }
+
+    void incrementCounters(const StringData cmdName,
+                           const BSONObj& validator,
+                           bool parsingSucceeded) {
+        if (!validator.isEmpty()) {
+            auto validatorCounter = _validatorCounterMap.find(cmdName);
+            tassert(7139200,
+                    str::stream() << "The validator counters are not support for the command: "
+                                  << cmdName,
+                    validatorCounter != _validatorCounterMap.end());
+            validatorCounter->second->totalCounter.increment();
+
+            if (!parsingSucceeded) {
+                validatorCounter->second->failedCounter.increment();
+            }
+            if (validator.hasField("$jsonSchema")) {
+                validatorCounter->second->jsonSchemaCounter.increment();
+            }
+        }
+    }
+
+private:
+    struct ValidatorCounter {
+        ValidatorCounter(const StringData name)
+            : totalCounter{"commands." + name + ".validator.total"},
+              failedCounter{"commands." + name + ".validator.failed"},
+              jsonSchemaCounter{"commands." + name + ".validator.jsonSchema"} {}
+        CounterMetric totalCounter;
+        CounterMetric failedCounter;
+        CounterMetric jsonSchemaCounter;
+    };
+
+    StringMap<std::unique_ptr<ValidatorCounter>> _validatorCounterMap = {};
+};
+
+extern ValidatorCounters validatorCounters;
+
 // Global counters for expressions inside aggregation pipelines.
 extern OperatorCounters operatorCountersAggExpressions;
 // Global counters for match expressions.
@@ -428,5 +613,10 @@ extern OperatorCounters operatorCountersWindowAccumulatorExpressions;
 extern CounterMetric updateManyCount;
 // Track the number of deleteMany calls.
 extern CounterMetric deleteManyCount;
-
+// Track the number of internal retryable writes
+extern CounterMetric internalRetryableWriteCount;
+// Track the number of external retryable writes
+extern CounterMetric externalRetryableWriteCount;
+// Track the number of internal transactions for retryable writes
+extern CounterMetric retryableInternalTransactionCount;
 }  // namespace mongo
