@@ -48,7 +48,8 @@ MONGO_INIT_REGISTER_ERROR_EXTRA_INFO(DuplicateKeyErrorInfo);
 DuplicateKeyErrorInfo::DuplicateKeyErrorInfo(const BSONObj& keyPattern,
                                              const BSONObj& keyValue,
                                              const BSONObj& collation,
-                                             FoundValue&& foundValue)
+                                             FoundValue&& foundValue,
+                                             boost::optional<RecordId> duplicateRid)
     : _keyPattern(keyPattern.getOwned()),
       _keyValue(keyValue.getOwned()),
       _collation(collation.getOwned()),
@@ -56,6 +57,9 @@ DuplicateKeyErrorInfo::DuplicateKeyErrorInfo(const BSONObj& keyPattern,
     if (auto foundValueObj = stdx::get_if<BSONObj>(&_foundValue)) {
         _foundValue = foundValueObj->getOwned();
     }
+    if (duplicateRid) {
+        _duplicateRid = *duplicateRid;
+    }
 }
 
 void DuplicateKeyErrorInfo::serialize(BSONObjBuilder* bob) const {
@@ -94,17 +98,20 @@ void DuplicateKeyErrorInfo::serialize(BSONObjBuilder* bob) const {
         bob->append("collation", _collation);
     }
 
-    stdx::visit(
-        OverloadedVisitor{
-            [](stdx::monostate) {},
-            [bob](const RecordId& rid) { rid.serializeToken("foundValue", bob); },
-            [bob](const BSONObj& obj) {
-                if (obj.objsize() < BSONObjMaxUserSize / 2) {
-                    bob->append("foundValue", obj);
-                }
-            },
-        },
-        _foundValue);
+    stdx::visit(OverloadedVisitor{
+                    [](stdx::monostate) {},
+                    [bob](const RecordId& rid) { rid.serializeToken("foundValue", bob); },
+                    [bob](const BSONObj& obj) {
+                        if (obj.objsize() < BSONObjMaxUserSize / 2) {
+                            bob->append("foundValue", obj);
+                        }
+                    },
+                },
+                _foundValue);
+
+    if (_duplicateRid) {
+        _duplicateRid->serializeToken("duplicateRid", bob);
+    }
 }
 
 std::shared_ptr<const ErrorExtraInfo> DuplicateKeyErrorInfo::parse(const BSONObj& obj) {
@@ -146,8 +153,13 @@ std::shared_ptr<const ErrorExtraInfo> DuplicateKeyErrorInfo::parse(const BSONObj
         }
     }
 
+    boost::optional<RecordId> duplicateRid;
+    if (auto duplicateRidElt = obj["duplicateRid"]) {
+        duplicateRid = RecordId::deserializeToken(duplicateRidElt);
+    }
+
     return std::make_shared<DuplicateKeyErrorInfo>(
-        keyPattern, keyValue, collation, std::move(foundValue));
+        keyPattern, keyValue, collation, std::move(foundValue), duplicateRid);
 }
 
 }  // namespace mongo
