@@ -31,6 +31,7 @@
 
 #include <boost/optional.hpp>
 #include <functional>
+#include <set>
 
 #include "mongo/bson/mutable/damage_vector.h"
 #include "mongo/db/exec/collection_scan_common.h"
@@ -212,6 +213,11 @@ public:
     virtual void saveUnpositioned() {
         save();
     }
+
+    virtual uint64_t getCheckpointId() const {
+        uasserted(ErrorCodes::CommandNotSupported,
+                  "The current storage engine does not support checkpoint ids");
+    }
 };
 
 /**
@@ -231,14 +237,12 @@ public:
      *
      * NOTE: Waiting threads can be signaled by calling kill or notify* methods.
      */
-    void waitUntil(uint64_t prevVersion, Date_t deadline) const;
+    void waitUntil(OperationContext* opCtx, uint64_t prevVersion, Date_t deadline) const;
 
     /**
      * Returns the version for use as an additional wake condition when used above.
      */
-    uint64_t getVersion() const {
-        return _version;
-    }
+    uint64_t getVersion() const;
 
     /**
      * Cancels the notifier if the collection is dropped/invalidated, and wakes all waiting.
@@ -280,12 +284,12 @@ private:
  * This class must be thread-safe. In addition, for storage engines implementing the KVEngine some
  * methods must be thread safe, see DurableCatalog.
  */
-class RecordStore : public Ident {
+class RecordStore {
     RecordStore(const RecordStore&) = delete;
     RecordStore& operator=(const RecordStore&) = delete;
 
 public:
-    RecordStore(StringData ns, StringData identName, bool isCapped);
+    RecordStore(boost::optional<UUID> uuid, StringData identName, bool isCapped);
     virtual ~RecordStore() {}
 
     // META
@@ -293,18 +297,31 @@ public:
     // name of the RecordStore implementation
     virtual const char* name() const = 0;
 
-    const std::string& ns() const {
-        return _ns;
-    }
-
-    void setNs(NamespaceString ns) {
-        _ns = ns.ns();
+    boost::optional<UUID> uuid() const {
+        return _uuid;
     }
 
     bool isTemp() const {
-        return ns().size() == 0;
+        return !_uuid.has_value();
     }
 
+    std::shared_ptr<Ident> getSharedIdent() const {
+        return _ident;
+    }
+
+    const std::string& getIdent() const {
+        return _ident->getIdent();
+    }
+
+    void setIdent(std::shared_ptr<Ident> newIdent) {
+        _ident = std::move(newIdent);
+    }
+
+    /**
+     * Get the namespace this RecordStore is associated with.
+     */
+    virtual std::string ns(OperationContext* opCtx) const = 0;
+
     /**
      * The key format for this RecordStore's RecordIds.
      *
@@ -392,7 +409,8 @@ public:
     RecordData dataFor(OperationContext* opCtx, const RecordId& loc) const {
         RecordData data;
         invariant(findRecord(opCtx, loc, &data),
-                  str::stream() << "Didn't find RecordId " << loc << " in record store " << ns());
+                  str::stream() << "Didn't find RecordId " << loc << " in record store "
+                                << ns(opCtx));
         return data;
     }
 
@@ -490,8 +508,13 @@ public:
 
     /**
      * Prints any storage engine provided metadata for the record with 'recordId'.
+     *
+     * If provided, saves any valid timestamps (startTs, startDurableTs, stopTs, stopDurableTs)
+     * related to this record in 'recordTimestamps'.
      */
-    virtual void printRecordMetadata(OperationContext* opCtx, const RecordId& recordId) const = 0;
+    virtual void printRecordMetadata(OperationContext* opCtx,
+                                     const RecordId& recordId,
+                                     std::set<Timestamp>* recordTimestamps) const = 0;
 
     /**
      * Returns a new cursor over this record store.
@@ -522,10 +545,24 @@ public:
     // higher level
 
     /**
-     * removes all Records
+     * Removes all Records.
      */
     Status truncate(OperationContext* opCtx);
 
+    /**
+     * Removes all Records in the range [minRecordId, maxRecordId] inclusive of both. The hint*
+     * arguments serve as a hint to the record store of how much data will be truncated. This is
+     * necessary for some implementations to avoid reading the data between the two RecordIds in
+     * order to update numRecords and dataSize correctly. Implementations are free to ignore the
+     * hints if they have a way of obtaining the correct values without the help of external
+     * callers.
+     */
+    Status rangeTruncate(OperationContext* opCtx,
+                         const RecordId& minRecordId = RecordId(),
+                         const RecordId& maxRecordId = RecordId(),
+                         int64_t hintDataSizeIncrement = 0,
+                         int64_t hintNumRecordsIncrement = 0);
+
     /**
      * Truncate documents newer than the document at 'end' from the capped
      * collection.  The collection cannot be completely emptied using this
@@ -548,15 +585,6 @@ public:
         return false;
     }
 
-    /**
-     * If compact() supports online compaction.
-     *
-     * Only called if compactSupported() returns true.
-     */
-    virtual bool supportsOnlineCompaction() const {
-        MONGO_UNREACHABLE;
-    }
-
     /**
      * Attempt to reduce the storage space used by this RecordStore.
      *
@@ -568,9 +596,7 @@ public:
      * Performs record store specific validation to ensure consistency of underlying data
      * structures. If corruption is found, details of the errors will be in the results parameter.
      */
-    virtual void validate(OperationContext* opCtx,
-                          ValidateResults* results,
-                          BSONObjBuilder* output) {}
+    virtual void validate(OperationContext* opCtx, bool full, ValidateResults* results) {}
 
     /**
      * @param scaleSize - amount by which to scale size metrics
@@ -618,6 +644,14 @@ public:
      */
     void waitForAllEarlierOplogWritesToBeVisible(OperationContext* opCtx) const;
 
+    /**
+     * Reserve a range of contiguous RecordIds. Returns the first valid RecordId in the range. Must
+     * only be called on a RecordStore with KeyFormat::Long.
+     */
+    virtual void reserveRecordIds(OperationContext* opCtx,
+                                  std::vector<RecordId>* out,
+                                  size_t nRecords) = 0;
+
     /**
      * Called after a repair operation is run with the recomputed numRecords and dataSize.
      */
@@ -628,32 +662,33 @@ public:
     /**
      * Storage engines can choose whether to support changing the oplog size online.
      */
-    virtual Status updateOplogSize(long long newOplogSize) {
+    virtual Status updateOplogSize(OperationContext* opCtx, long long newOplogSize) {
         return Status(ErrorCodes::CommandNotSupported,
                       "This storage engine does not support updateOplogSize");
     }
 
     /**
      * Returns false if the oplog was dropped while waiting for a deletion request.
-     * This should only be called if StorageEngine::supportsOplogStones() is true.
-     * Storage engines supporting oplog stones must implement this function.
+     * This should only be called if StorageEngine::supportsOplogTruncateMarkers() is true.
+     * Storage engines supporting oplog truncate markers must implement this function.
      */
     virtual bool yieldAndAwaitOplogDeletionRequest(OperationContext* opCtx) {
         MONGO_UNREACHABLE;
     }
 
     /**
-     * This should only be called if StorageEngine::supportsOplogStones() is true.
-     * Storage engines supporting oplog stones must implement this function.
+     * This should only be called if StorageEngine::supportsOplogTruncateMarkers() is true.
+     * Storage engines supporting oplog truncate markers must implement this function.
      */
     virtual void reclaimOplog(OperationContext* opCtx) {
         MONGO_UNREACHABLE;
     }
 
     /**
-     * This should only be called if StorageEngine::supportsOplogStones() is true.
-     * Storage engines supporting oplog stones must implement this function.
-     * Populates `builder` with various statistics pertaining to oplog stones and oplog truncation.
+     * This should only be called if StorageEngine::supportsOplogTruncateMarkers() is true.
+     * Storage engines supporting oplog truncate markers must implement this function.
+     * Populates `builder` with various statistics pertaining to oplog truncate markers and oplog
+     * truncation.
      */
     virtual void getOplogTruncateStats(BSONObjBuilder& builder) const {
         MONGO_UNREACHABLE;
@@ -705,6 +740,11 @@ protected:
         const char* damageSource,
         const mutablebson::DamageVector& damages) = 0;
     virtual Status doTruncate(OperationContext* opCtx) = 0;
+    virtual Status doRangeTruncate(OperationContext* opCtx,
+                                   const RecordId& minRecordId,
+                                   const RecordId& maxRecordId,
+                                   int64_t hintDataSizeDiff,
+                                   int64_t hintNumRecordsDiff) = 0;
     virtual void doCappedTruncateAfter(OperationContext* opCtx,
                                        const RecordId& end,
                                        bool inclusive,
@@ -721,7 +761,9 @@ protected:
 
     virtual void waitForAllEarlierOplogWritesToBeVisibleImpl(OperationContext* opCtx) const = 0;
 
-    std::string _ns;
+    std::shared_ptr<Ident> _ident;
+
+    boost::optional<UUID> _uuid;
 
     std::shared_ptr<CappedInsertNotifier> _cappedInsertNotifier;
 };
