@@ -34,6 +34,8 @@
 #include <string>
 #include <wiredtiger.h>
 
+#include "mongo/db/catalog/capped_visibility.h"
+#include "mongo/db/storage/collection_markers.h"
 #include "mongo/db/storage/record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_cursor.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
@@ -103,6 +105,7 @@ public:
 
     struct Params {
         NamespaceString nss;
+        boost::optional<UUID> uuid;
         std::string ident;
         std::string engineName;
         bool isCapped;
@@ -122,7 +125,8 @@ public:
 
     virtual ~WiredTigerRecordStore();
 
-    virtual void postConstructorInit(OperationContext* opCtx);
+    // Pass in NamespaceString, it is not possible to resolve the UUID to NamespaceString yet.
+    virtual void postConstructorInit(OperationContext* opCtx, const NamespaceString& ns);
 
     // name of the RecordStore implementation
     virtual const char* name() const;
@@ -166,7 +170,9 @@ public:
                                                const char* damageSource,
                                                const mutablebson::DamageVector& damages) final;
 
-    virtual void printRecordMetadata(OperationContext* opCtx, const RecordId& recordId) const;
+    virtual void printRecordMetadata(OperationContext* opCtx,
+                                     const RecordId& recordId,
+                                     std::set<Timestamp>* recordTimestamps) const;
 
     virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
                                                             bool forward) const = 0;
@@ -174,21 +180,20 @@ public:
     std::unique_ptr<RecordCursor> getRandomCursor(OperationContext* opCtx) const final;
 
     Status doTruncate(OperationContext* opCtx) final;
+    Status doRangeTruncate(OperationContext* opCtx,
+                           const RecordId& minRecordId,
+                           const RecordId& maxRecordId,
+                           int64_t hintDataSizeDiff,
+                           int64_t hintNumRecordsDiff) final;
 
     virtual bool compactSupported() const {
         return !_isEphemeral;
     }
-    virtual bool supportsOnlineCompaction() const {
-        return true;
-    }
-
     virtual Timestamp getPinnedOplog() const final;
 
     Status doCompact(OperationContext* opCtx) final;
 
-    virtual void validate(OperationContext* opCtx,
-                          ValidateResults* results,
-                          BSONObjBuilder* output);
+    virtual void validate(OperationContext* opCtx, bool full, ValidateResults* results);
 
     virtual void appendNumericCustomStats(OperationContext* opCtx,
                                           BSONObjBuilder* result,
@@ -208,7 +213,7 @@ public:
                                         long long dataSize);
 
 
-    Status updateOplogSize(long long newOplogSize) final;
+    Status updateOplogSize(OperationContext* opCtx, long long newOplogSize) final;
 
     const std::string& getURI() const {
         return _uri;
@@ -218,6 +223,8 @@ public:
         return _tableId;
     }
 
+    std::string ns(OperationContext* opCtx) const final;
+
     /*
      * Check the size information for this RecordStore. This function opens a cursor on the
      * RecordStore to determine if it is empty. If it is empty, it will mark the collection as
@@ -243,27 +250,28 @@ public:
 
     bool yieldAndAwaitOplogDeletionRequest(OperationContext* opCtx) override;
 
+    /**
+     * Attempts to truncate oplog entries before the pinned oplog timestamp. Truncation will occur
+     * if the oplog is at capacity and the maximum retention time has elapsed.
+     */
     void reclaimOplog(OperationContext* opCtx) override;
 
     StatusWith<Timestamp> getLatestOplogTimestamp(OperationContext* opCtx) const override;
     StatusWith<Timestamp> getEarliestOplogTimestamp(OperationContext* opCtx) override;
 
-    /**
-     * The `recoveryTimestamp` is when replication recovery would need to replay from for
-     * recoverable rollback, or restart for durable engines. `reclaimOplog` will not
-     * truncate oplog entries in front of this time.
-     */
-    void reclaimOplog(OperationContext* opCtx, Timestamp recoveryTimestamp);
-
-    class OplogStones;
+    class OplogTruncateMarkers;
 
     // Exposed only for testing.
-    OplogStones* oplogStones() {
-        return _oplogStones.get();
+    OplogTruncateMarkers* oplogTruncateMarkers() {
+        return _oplogTruncateMarkers.get();
     };
 
     typedef stdx::variant<int64_t, WiredTigerItem> CursorKey;
 
+    void reserveRecordIds(OperationContext* opCtx,
+                          std::vector<RecordId>* out,
+                          size_t nRecords) final;
+
 protected:
     virtual RecordId getKey(WT_CURSOR* cursor) const = 0;
 
@@ -294,9 +302,9 @@ private:
     void _initNextIdIfNeeded(OperationContext* opCtx);
 
     /**
-     * Adjusts the record count and data size metadata for this record store, respectively. These
-     * functions consult the SizeRecoveryState to determine whether or not to actually change the
-     * size metadata if the server is undergoing recovery.
+     * Adjusts the record count and data size metadata for this record store. The function consults
+     * the SizeRecoveryState to determine whether or not to actually change the size metadata if the
+     * server is undergoing recovery.
      *
      * For most record stores, we will not update the size metadata during recovery, as we trust
      * that the values in the SizeStorer are accurate with respect to the end state of recovery.
@@ -310,8 +318,9 @@ private:
      *      are pending writes to this ident as part of the recovery process, and so we must
      *      always adjust size metadata for these idents.
      */
-    void _changeNumRecords(OperationContext* opCtx, int64_t diff);
-    void _increaseDataSize(OperationContext* opCtx, int64_t amount);
+    void _changeNumRecordsAndDataSize(OperationContext* opCtx,
+                                      int64_t numRecordDiff,
+                                      int64_t dataSizeDiff);
 
     const std::string _uri;
     const uint64_t _tableId;  // not persisted
@@ -338,7 +347,7 @@ private:
     // True if force to update with the full document, and false otherwise.
     const bool _forceUpdateWithFullDocument;
     boost::optional<int64_t> _oplogMaxSize;
-    RecordId _oplogFirstRecord;
+    AtomicWord<Timestamp> _oplogFirstRecordTimestamp{Timestamp()};
 
     // Protects initialization of the _nextIdNum.
     mutable Mutex _initNextIdMutex = MONGO_MAKE_LATCH("WiredTigerRecordStore::_initNextIdMutex");
@@ -350,7 +359,7 @@ private:
     WiredTigerKVEngine* _kvEngine;  // not owned.
 
     // Non-null if this record store is underlying the active oplog.
-    std::shared_ptr<OplogStones> _oplogStones;
+    std::shared_ptr<OplogTruncateMarkers> _oplogTruncateMarkers;
 
     AtomicWord<int64_t>
         _totalTimeTruncating;            // Cumulative amount of time spent truncating the oplog.
@@ -399,6 +408,13 @@ public:
         _saveStorageCursorOnDetachFromOperationContext = saveCursor;
     }
 
+    /**
+     *  Returns the checkpoint ID for checkpoint cursors, otherwise 0.
+     */
+    uint64_t getCheckpointId() const override {
+        return _cursor->getCheckpointId();
+    }
+
 protected:
     virtual RecordId getKey(WT_CURSOR* cursor) const = 0;
 
@@ -409,18 +425,24 @@ protected:
      */
     virtual void initCursorToBeginning() = 0;
 
-    const WiredTigerRecordStore& _rs;
-    OperationContext* _opCtx;
-    const bool _forward;
-    bool _skipNextAdvance = false;
-    boost::optional<WiredTigerCursor> _cursor;
-    bool _eof = false;
+    const uint64_t _tableId;
     RecordId _lastReturnedId;  // If null, need to seek to first/last record.
+    OperationContext* _opCtx;
+    const std::string _uri;
+    const std::string _ident;
+    boost::optional<WiredTigerCursor> _cursor;
+    const KeyFormat _keyFormat;
+    const bool _forward;
+    const bool _isOplog;
+    const bool _isCapped;
+    const boost::optional<UUID> _uuid;
+    bool _skipNextAdvance = false;
+    bool _eof = false;
     bool _hasRestored = true;
 
 private:
     bool isVisible(const RecordId& id);
-    void initOplogVisibility(OperationContext* opCtx);
+    void initCappedVisibility(OperationContext* opCtx);
 
     /**
      * This value is used for visibility calculations on what oplog entries can be returned to a
@@ -428,6 +450,7 @@ private:
      * established.
      */
     boost::optional<std::int64_t> _oplogVisibleTs = boost::none;
+    boost::optional<CappedVisibilitySnapshot> _cappedSnapshot;
 
     /**
      * With WT-8601, WiredTiger no longer maintains commit_timestamp information on writes to logged
