@@ -35,6 +35,7 @@
 #include <memory>
 
 #include "mongo/base/error_codes.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/storage/journal_listener.h"
@@ -52,7 +53,6 @@ namespace mongo {
 
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64_t cursorEpoch)
     : _epoch(epoch),
-      _cursorEpoch(cursorEpoch),
       _session(nullptr),
       _cursorGen(0),
       _cursorsOut(0),
@@ -65,7 +65,6 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
                                      uint64_t epoch,
                                      uint64_t cursorEpoch)
     : _epoch(epoch),
-      _cursorEpoch(cursorEpoch),
       _cache(cache),
       _session(nullptr),
       _cursorGen(0),
@@ -188,20 +187,6 @@ void WiredTigerSession::closeAllCursors(const std::string& uri) {
     }
 }
 
-void WiredTigerSession::closeCursorsForQueuedDrops(WiredTigerKVEngine* engine) {
-    invariant(_session);
-
-    _cursorEpoch = _cache->getCursorEpoch();
-    auto toDrop = engine->filterCursorsWithQueuedDrops(&_cursors);
-
-    for (auto i = toDrop.begin(); i != toDrop.end(); i++) {
-        WT_CURSOR* cursor = i->_cursor;
-        if (cursor) {
-            invariantWTOK(cursor->close(cursor), _session);
-        }
-    }
-}
-
 namespace {
 AtomicWord<unsigned long long> nextTableId(WiredTigerSession::kLastTableId);
 }
@@ -243,6 +228,10 @@ void WiredTigerSessionCache::shuttingDown() {
     closeAll();
 }
 
+void WiredTigerSessionCache::restart() {
+    _shuttingDown.fetchAndBitAnd(~kShuttingDownMask);
+}
+
 bool WiredTigerSessionCache::isShuttingDown() {
     return _shuttingDown.load() & kShuttingDownMask;
 }
@@ -289,32 +278,26 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
     // waiters, as a log flush is much cheaper than a full checkpoint.
     if ((syncType == Fsync::kCheckpointStableTimestamp || syncType == Fsync::kCheckpointAll) &&
         !isEphemeral()) {
-        UniqueWiredTigerSession session = getSession();
-        WT_SESSION* s = session->getSession();
-        {
-            auto journalListener = [&]() -> JournalListener* {
-                // The JournalListener may not be set immediately, so we must check under a mutex so
-                // as not to access the variable while setting a JournalListener. A JournalListener
-                // is only allowed to be set once, so using the pointer outside of a mutex is safe.
-                stdx::unique_lock<Latch> lk(_journalListenerMutex);
-                return _journalListener;
-            }();
-            boost::optional<JournalListener::Token> token;
-            if (journalListener && useListener == UseJournalListener::kUpdate) {
-                // Update a persisted value with the latest write timestamp that is safe across
-                // startup recovery in the repl layer. Then report that timestamp as durable to the
-                // repl layer below after we have flushed in-memory data to disk.
-                // Note: only does a write if primary, otherwise just fetches the timestamp.
-                token = journalListener->getToken(opCtx);
-            }
+        auto journalListener = [&]() -> JournalListener* {
+            // The JournalListener may not be set immediately, so we must check under a mutex so
+            // as not to access the variable while setting a JournalListener. A JournalListener
+            // is only allowed to be set once, so using the pointer outside of a mutex is safe.
+            stdx::unique_lock<Latch> lk(_journalListenerMutex);
+            return _journalListener;
+        }();
+        boost::optional<JournalListener::Token> token;
+        if (journalListener && useListener == UseJournalListener::kUpdate) {
+            // Update a persisted value with the latest write timestamp that is safe across
+            // startup recovery in the repl layer. Then report that timestamp as durable to the
+            // repl layer below after we have flushed in-memory data to disk.
+            // Note: only does a write if primary, otherwise just fetches the timestamp.
+            token = journalListener->getToken(opCtx);
+        }
 
-            auto config = syncType == Fsync::kCheckpointStableTimestamp ? "use_timestamp=true"
-                                                                        : "use_timestamp=false";
-            invariantWTOK(s->checkpoint(s, config), s);
+        getKVEngine()->forceCheckpoint(syncType == Fsync::kCheckpointStableTimestamp);
 
-            if (token) {
-                journalListener->onDurable(token.value());
-            }
+        if (token) {
+            journalListener->onDurable(token.value());
         }
         LOGV2_DEBUG(22418, 4, "created checkpoint (forced)");
         return;
@@ -356,16 +339,10 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
             nullptr);
     }
 
-    // Use the journal when available, or a checkpoint otherwise.
-    if (!isEphemeral()) {
-        invariantWTOK(_waitUntilDurableSession->log_flush(_waitUntilDurableSession, "sync=on"),
-                      _waitUntilDurableSession);
-        LOGV2_DEBUG(22419, 4, "flushed journal");
-    } else {
-        invariantWTOK(_waitUntilDurableSession->checkpoint(_waitUntilDurableSession, nullptr),
-                      _waitUntilDurableSession);
-        LOGV2_DEBUG(22420, 4, "created checkpoint");
-    }
+    // Flush the journal.
+    invariantWTOK(_waitUntilDurableSession->log_flush(_waitUntilDurableSession, "sync=on"),
+                  _waitUntilDurableSession);
+    LOGV2_DEBUG(22419, 4, "flushed journal");
 
     if (token) {
         journalListener->onDurable(token.value());
@@ -375,9 +352,17 @@ void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
 void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(OperationContext* opCtx,
                                                                         std::uint64_t lastCount) {
     invariant(opCtx);
+
+    // It is possible for a prepared transaction to block on bonus eviction inside WiredTiger after
+    // it commits or rolls-back, but this delays it from signalling us to wake up. In the very
+    // worst case that the only evictable page is the one pinned by our cursor, AND there are no
+    // other prepared transactions committing or aborting, we could reach a deadlock. Since the
+    // caller is already expecting spurious wakeups, we impose a large timeout to periodically force
+    // the caller to retry its operation.
+    const auto deadline = Date_t::now() + Seconds(1);
     stdx::unique_lock<Latch> lk(_prepareCommittedOrAbortedMutex);
     if (lastCount == _prepareCommitOrAbortCounter.loadRelaxed()) {
-        opCtx->waitForConditionOrInterrupt(_prepareCommittedOrAbortedCond, lk, [&] {
+        opCtx->waitForConditionOrInterruptUntil(_prepareCommittedOrAbortedCond, lk, deadline, [&] {
             return _prepareCommitOrAbortCounter.loadRelaxed() > lastCount;
         });
     }
@@ -397,16 +382,6 @@ void WiredTigerSessionCache::closeAllCursors(const std::string& uri) {
     }
 }
 
-void WiredTigerSessionCache::closeCursorsForQueuedDrops() {
-    // Increment the cursor epoch so that all cursors from this epoch are closed.
-    _cursorEpoch.fetchAndAdd(1);
-
-    stdx::lock_guard<Latch> lock(_cacheLock);
-    for (SessionCache::iterator i = _sessions.begin(); i != _sessions.end(); i++) {
-        (*i)->closeCursorsForQueuedDrops(_engine);
-    }
-}
-
 size_t WiredTigerSessionCache::getIdleSessionsCount() {
     stdx::lock_guard<Latch> lock(_cacheLock);
     return _sessions.size();
@@ -481,8 +456,7 @@ UniqueWiredTigerSession WiredTigerSessionCache::getSession() {
     }
 
     // Outside of the cache partition lock, but on release will be put back on the cache
-    return UniqueWiredTigerSession(
-        new WiredTigerSession(_conn, this, _epoch.load(), _cursorEpoch.load()));
+    return UniqueWiredTigerSession(new WiredTigerSession(_conn, this, _epoch.load()));
 }
 
 void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
@@ -520,18 +494,10 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
         invariantWTOK(ss->reset(ss), ss);
     }
 
-    // If the cursor epoch has moved on, close all cursors in the session.
-    uint64_t cursorEpoch = _cursorEpoch.load();
-    if (session->_getCursorEpoch() != cursorEpoch)
-        session->closeCursorsForQueuedDrops(_engine);
-
     bool returnedToCache = false;
     uint64_t currentEpoch = _epoch.load();
-    bool dropQueuedIdentsAtSessionEnd = session->isDropQueuedIdentsAtSessionEndAllowed();
 
-    // Reset this session's flag for dropping queued idents to default, before returning it to
-    // session cache. Also set the time this session got idle at.
-    session->dropQueuedIdentsAtSessionEndAllowed(true);
+    // Set the time this session got idle at.
     session->setIdleExpireTime(_clockSource->now());
 
     if (session->_getEpoch() == currentEpoch) {  // check outside of lock to reduce contention
@@ -546,8 +512,9 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
     if (!returnedToCache)
         delete session;
 
-    if (dropQueuedIdentsAtSessionEnd && _engine && _engine->haveDropsQueued())
-        _engine->dropSomeQueuedIdents();
+    if (_engine) {
+        _engine->sizeStorerPeriodicFlush();
+    }
 }
 
 
