@@ -204,10 +204,10 @@ class CreateFromQueryFixture : public mongo::unittest::Test {
 public:
     CreateFromQueryFixture()
         : _opCtx(_serviceContext.makeOperationContext()),
-          _driverOps(new UpdateDriver(
-              new ExpressionContext(_opCtx.get(), nullptr, NamespaceString("foo")))),
-          _driverRepl(new UpdateDriver(
-              new ExpressionContext(_opCtx.get(), nullptr, NamespaceString("foo")))) {
+          _driverOps(new UpdateDriver(new ExpressionContext(
+              _opCtx.get(), nullptr, NamespaceString::createNamespaceString_forTest("foo")))),
+          _driverRepl(new UpdateDriver(new ExpressionContext(
+              _opCtx.get(), nullptr, NamespaceString::createNamespaceString_forTest("foo")))) {
         _driverOps->parse(makeUpdateMod(fromjson("{$set:{'_':1}}")), _arrayFilters);
         _driverRepl->parse(makeUpdateMod(fromjson("{}")), _arrayFilters);
     }
@@ -565,8 +565,7 @@ public:
                    const write_ops::UpdateModification& updateSpec,
                    StringData matchedField = StringData(),
                    std::vector<BSONObj> arrayFilterSpec = {},
-                   bool fromOplog = false,
-                   UpdateIndexData* indexData = nullptr) {
+                   bool fromOplog = false) {
         boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
         _driver = std::make_unique<UpdateDriver>(expCtx);
         std::map<StringData, std::unique_ptr<ExpressionWithPlaceholder>> arrayFilters;
@@ -579,7 +578,6 @@ public:
         }
 
         _driver->setFromOplogApplication(fromOplog);
-        _driver->refreshIndexKeys(indexData);
         _driver->parse(updateSpec, arrayFilters);
 
         const bool validateForStorage = true;
@@ -681,52 +679,5 @@ TEST_F(ModifiedPathsTestFixture, ReplaceFullDocumentAlwaysAffectsIndex) {
     ASSERT_EQ(_modifiedPaths, "{}");
 }
 
-
-TEST_F(ModifiedPathsTestFixture, PipelineUpdatesAlwaysAffectsIndex) {
-    BSONObj spec = fromjson("{$set: {'a.1.b': 1}}");
-    mutablebson::Document doc(fromjson("{a: [{b: 0}]}"));
-    runUpdate(&doc, std::vector<BSONObj>{spec});
-    ASSERT(_driver->modsAffectIndices());
-}
-
-TEST_F(ModifiedPathsTestFixture, DeltaUpdateNotAffectingIndex) {
-    BSONObj spec = fromjson("{d: {a: false}}");
-    mutablebson::Document doc(fromjson("{a: [{b: 0}]}"));
-    runUpdate(&doc,
-              write_ops::UpdateModification::parseFromV2Delta(
-                  spec, write_ops::UpdateModification::DiffOptions{}),
-              ""_sd,
-              {},
-              true /* fromOplog */);
-    ASSERT(!_driver->modsAffectIndices());
-
-    UpdateIndexData indexData;
-    indexData.addPath(FieldRef("p"));
-    runUpdate(&doc,
-              write_ops::UpdateModification::parseFromV2Delta(
-                  spec, write_ops::UpdateModification::DiffOptions{}),
-              ""_sd,
-              {},
-              true /* fromOplog */,
-              &indexData);
-    ASSERT(!_driver->modsAffectIndices());
-}
-
-TEST_F(ModifiedPathsTestFixture, DeltaUpdateAffectingIndex) {
-    BSONObj spec = fromjson("{u: {a: 1}}");
-    mutablebson::Document doc(fromjson("{a: [{b: 0}]}"));
-    UpdateIndexData indexData;
-    indexData.addPath(FieldRef("q"));
-    indexData.addPath(FieldRef("a.p"));
-    runUpdate(&doc,
-              write_ops::UpdateModification::parseFromV2Delta(
-                  spec, write_ops::UpdateModification::DiffOptions{}),
-              ""_sd,
-              {},
-              true /* fromOplog */,
-              &indexData);
-    ASSERT(_driver->modsAffectIndices());
-}
-
 }  // namespace
 }  // namespace mongo
