@@ -101,8 +101,8 @@ protected:
                                      MultiIndexBlock::kNoopOnCommitFn));
             wunit.commit();
             abortOnExit.dismiss();
-        } catch (const DBException& e) {
-            if (ErrorCodes::isInterruption(e.code()))
+        } catch (const DBException&) {
+            if (!_opCtx->checkForInterruptNoAssert().isOK())
                 return true;
 
             throw;
@@ -306,12 +306,11 @@ public:
             int32_t nDocs = 1000;
             OpDebug* const nullOpDebug = nullptr;
             for (int32_t i = 0; i < nDocs; ++i) {
-                ASSERT_OK(collection_internal::insertDocument(
-                    _opCtx,
-                    CollectionPtr(coll, CollectionPtr::NoYieldTag{}),
-                    InsertStatement(BSON("_id" << i)),
-                    nullOpDebug,
-                    true));
+                ASSERT_OK(collection_internal::insertDocument(_opCtx,
+                                                              CollectionPtr(coll),
+                                                              InsertStatement(BSON("_id" << i)),
+                                                              nullOpDebug,
+                                                              true));
             }
             wunit.commit();
             // Request an interrupt.
@@ -553,6 +552,8 @@ protected:
 class IndexCatatalogFixIndexKey : public IndexBuildBase {
 public:
     void run() {
+        AutoGetCollection autoColl(_opCtx, _nss, LockMode::MODE_X);
+
         auto indexCatalog = collection().get()->getIndexCatalog();
 
         ASSERT_BSONOBJ_EQ(BSON("x" << 1), indexCatalog->fixIndexKey(BSON("x" << 1)));
@@ -568,13 +569,13 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
+        client.dropCollection(_nss);
         IndexSpec indexSpec;
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
-        client.createIndex(_ns, indexSpec);
+        client.createIndex(_nss, indexSpec);
 
-        auto response = client.insertAcknowledged(_ns, {BSON("a" << BSONSymbol("mySymbol"))});
+        auto response = client.insertAcknowledged(_nss, {BSON("a" << BSONSymbol("mySymbol"))});
         ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
@@ -585,12 +586,12 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
+        client.dropCollection(_nss);
         IndexSpec indexSpec;
         indexSpec.addKey("a");
-        client.createIndex(_ns, indexSpec);
+        client.createIndex(_nss, indexSpec);
 
-        auto response = client.insertAcknowledged(_ns, {BSON("a" << BSONSymbol("mySymbol"))});
+        auto response = client.insertAcknowledged(_nss, {BSON("a" << BSONSymbol("mySymbol"))});
         ASSERT_OK(getStatusFromWriteCommandReply(response));
         ASSERT_EQUALS(response["n"].Int(), 1);
         ASSERT_EQUALS(client.count(_nss), 1U);
@@ -602,14 +603,14 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
+        client.dropCollection(_nss);
         IndexSpec indexSpec;
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
-        client.createIndex(_ns, indexSpec);
+        client.createIndex(_nss, indexSpec);
 
         auto response = client.insertAcknowledged(
-            _ns, {BSON("a" << BSON("b" << 99 << "c" << BSONSymbol("mySymbol")))});
+            _nss, {BSON("a" << BSON("b" << 99 << "c" << BSONSymbol("mySymbol")))});
         ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
@@ -620,14 +621,14 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
+        client.dropCollection(_nss);
         IndexSpec indexSpec;
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
-        client.createIndex(_ns, indexSpec);
+        client.createIndex(_nss, indexSpec);
 
-        auto response =
-            client.insertAcknowledged(_ns, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
+        auto response = client.insertAcknowledged(
+            _nss, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
         ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
         ASSERT_EQUALS(client.count(_nss), 0U);
     }
@@ -638,13 +639,13 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
-        client.insert(_ns, BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol"))));
+        client.dropCollection(_nss);
+        client.insert(_nss, BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol"))));
         ASSERT_EQUALS(client.count(_nss), 1U);
         IndexSpec indexSpec;
         indexSpec.addKey("a").addOptions(BSON("collation" << BSON("locale"
                                                                   << "fr")));
-        ASSERT_THROWS_CODE(client.createIndex(_ns, indexSpec),
+        ASSERT_THROWS_CODE(client.createIndex(_nss, indexSpec),
                            AssertionException,
                            ErrorCodes::CannotBuildIndexKeys);
     }
@@ -655,9 +656,9 @@ public:
     void run() {
         auto opCtx = cc().makeOperationContext();
         DBDirectClient client(opCtx.get());
-        client.dropCollection(_ns);
+        client.dropCollection(_nss);
         BSONObj cmdResult;
-        ASSERT_TRUE(client.runCommand("unittests",
+        ASSERT_TRUE(client.runCommand({boost::none, "unittests"},
                                       BSON("create"
                                            << "indexupdate"
                                            << "collation"
@@ -666,10 +667,10 @@ public:
                                       cmdResult));
         IndexSpec indexSpec;
         indexSpec.addKey("a");
-        client.createIndex(_ns, indexSpec);
+        client.createIndex(_nss, indexSpec);
 
-        auto response =
-            client.insertAcknowledged(_ns, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
+        auto response = client.insertAcknowledged(
+            _nss, {BSON("a" << BSON_ARRAY(99 << BSONSymbol("mySymbol")))});
         ASSERT_EQUALS(getStatusFromWriteCommandReply(response), ErrorCodes::CannotBuildIndexKeys);
     }
 };
