@@ -170,7 +170,7 @@ FieldCompareResult compareDottedFieldNames(const string& l,
           "compareDottedFieldNames ERROR  l: {l} r: {r}  TOO MANY LOOPS",
           "l"_attr = l,
           "r"_attr = r);
-    verify(0);
+    MONGO_verify(0);
     return SAME;  // will never get here
 }
 }  // namespace mongo
@@ -201,8 +201,8 @@ public:
         BufBuilder b;
         unsigned int written = 0;
         try {
-            for (; written <= 64 * 1024 * 1024 + 1; ++written)
-                // (re)alloc past the buffer 64mb limit
+            for (; written <= mongo::BufferMaxSize + 1; ++written)
+                // (re)alloc past the buffer limit
                 b.appendStr("a");
         } catch (const AssertionException&) {
         }
@@ -290,9 +290,9 @@ public:
         }
         {
             BSONObj k = BSON("x" << 1);
-            verify(k.isFieldNamePrefixOf(BSON("x"
-                                              << "hi")));
-            verify(!k.isFieldNamePrefixOf(BSON("a" << 1)));
+            MONGO_verify(k.isFieldNamePrefixOf(BSON("x"
+                                                    << "hi")));
+            MONGO_verify(!k.isFieldNamePrefixOf(BSON("a" << 1)));
         }
     }
 };
@@ -1556,7 +1556,9 @@ struct BSONArrayBuilderTest {
         BSONObjBuilder objb;
         BSONArrayBuilder arrb;
 
-        auto fieldNameGenerator = [i = 0]() mutable { return std::to_string(i++); };
+        auto fieldNameGenerator = [i = 0]() mutable {
+            return std::to_string(i++);
+        };
 
         objb << fieldNameGenerator() << 100;
         arrb << 100;
