@@ -38,9 +38,30 @@
 #include <limits>
 #include <sstream>
 
+#include <boost/core/swap.hpp>
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/move/utility_core.hpp>
+// IWYU pragma: no_include "boost/multi_index/detail/bidir_node_iterator.hpp"
+#include <boost/operators.hpp>
+// IWYU pragma: no_include "boost/property_tree/detail/exception_implementation.hpp"
+// IWYU pragma: no_include "boost/property_tree/detail/ptree_implementation.hpp"
+#include <boost/property_tree/ptree_fwd.hpp>
+
+#include "mongo/base/status.h"
+#include "mongo/base/string_data.h"
+#include "mongo/bson/bson_validate.h"
+#include "mongo/bson/bsonelement.h"
+#include "mongo/bson/bsonmisc.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
+#include "mongo/bson/json.h"
+#include "mongo/bson/oid.h"
+#include "mongo/bson/timestamp.h"
+#include "mongo/bson/util/bsoncolumnbuilder.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/json.h"
-#include "mongo/dbtests/dbtests.h"
+#include "mongo/dbtests/dbtests.h"  // IWYU pragma: keep
 #include "mongo/logv2/log.h"
 #include "mongo/platform/decimal128.h"
 
@@ -836,12 +857,27 @@ TEST(FromJsonTest, BinDataTypes) {
         {0x05, MD5Type},
         {0x06, Encrypt},
         {0x07, Column},
+        {0x08, Sensitive},
         {0x80, bdtCustom},
     };
     for (const auto& ts : specs) {
-        checkEquivalence(
-            fmt::sprintf(R"({ "a" : { "$binary" : "YWJj", "$type" : "%02x" } })", ts.code),
-            BSONObjBuilder().appendBinData("a", 3, ts.bdt, "abc").obj());
+        if (ts.bdt == Column) {
+            BSONColumnBuilder cb;
+            cb.append(BSON("a"
+                           << "abc")
+                          .getField("a"));
+            BSONBinData columnData = cb.finalize();
+            checkEquivalence(fmt::sprintf(R"({ "a" : { "$binary" : "%s", "$type" : "%02x" } })",
+                                          base64::encode(columnData.data, columnData.length),
+                                          ts.code),
+                             BSONObjBuilder()
+                                 .appendBinData("a", columnData.length, ts.bdt, columnData.data)
+                                 .obj());
+        } else {
+            checkEquivalence(
+                fmt::sprintf(R"({ "a" : { "$binary" : "YWJj", "$type" : "%02x" } })", ts.code),
+                BSONObjBuilder().appendBinData("a", 3, ts.bdt, "abc").obj());
+        }
     }
 }
 
@@ -984,6 +1020,35 @@ TEST(FromJsonTest, TimestampObjectTest) {
     });
 }
 
+TEST(FromJsonTest, JSUUIDTest) {
+    BSONObjBuilder uuidObjBuilder;
+    UUID uuid = assertGet(UUID::parse("5fc51c8b-9a77-49ff-9f94-0a7e96173aa0"));
+    uuid.appendToBuilder(&uuidObjBuilder, "a");
+    checkEquivalence(R"({ "a" : UUID( "5fc51c8b-9a77-49ff-9f94-0a7e96173aa0" ) })",
+                     uuidObjBuilder.obj());
+    checkRejectionEach({
+        R"({ "a" : UUID( 20 ) })",                                       // Not a string
+        R"({ "a" : UUID() })",                                           // NoArgs
+        R"({ "a" : UUID( "a" ) })",                                      // Wrong input size
+        R"({ "a" : UUID( "/5fc51c8b-9a77-49ff-9f94-0a7e96173aa0" ) })",  // Right size, but wrong
+                                                                         // character set
+    });
+}
+
+TEST(FromJsonTest, UUIDObjectTest) {
+    BSONObjBuilder uuidObjBuilder;
+    UUID uuid = assertGet(UUID::parse("5fc51c8b-9a77-49ff-9f94-0a7e96173aa0"));
+    uuid.appendToBuilder(&uuidObjBuilder, "a");
+    checkEquivalence(R"({ "a" : {"$uuid": "5fc51c8b-9a77-49ff-9f94-0a7e96173aa0" } })",
+                     uuidObjBuilder.obj());
+    checkRejectionEach({
+        R"({ "a" : {"$uuid": 20} })",                                       // Not a string
+        R"({ "a" : {"$uuid": "a"} })",                                      // Wrong input size
+        R"({ "a" : {"$uuid": "/5fc51c8b-9a77-49ff-9f94-0a7e96173aa0"} })",  // Right size, but wrong
+                                                                            // character set
+    });
+}
+
 BSONObj re(const std::string& name, const std::string& re, const std::string& options) {
     BSONObjBuilder b;
     b.appendRegex(name, re, options);
@@ -1157,7 +1222,9 @@ TEST(FromJsonTest, NumericTypes) {
 TEST(FromJsonTest, EmbeddedDates) {
     const long long kMin = 1257829200000;
     const long long kMax = 1257829200100;
-    auto makeDate = [](long long ms) { return Date_t::fromMillisSinceEpoch(ms); };
+    auto makeDate = [](long long ms) {
+        return Date_t::fromMillisSinceEpoch(ms);
+    };
     const BSONObj bson =
         B().append("time.valid",
                    B().appendDate("$gt", makeDate(kMin)).appendDate("$lt", makeDate(kMax)).obj())
@@ -1194,6 +1261,71 @@ TEST(FromJsonTest, MinMaxKey) {
     });
 }
 
+/**
+ * Asserts 'inputjson' fails to parse, and that each of the 'expectedContextChars' are shown in a
+ * little snippet of the area we encountered the first parsing error.
+ */
+void assertErrorWithContext(std::string inputjson,
+                            std::initializer_list<char> expectedContextChars) {
+    try {
+        fromjson(inputjson);
+        ASSERT(false) << "Expected to fail to parse";
+    } catch (const DBException& ex) {
+        const auto status = ex.toStatus();
+        const StringData reason = status.reason();
+        LOGV2_DEBUG(7583700, 3, "Indeeded failed to parse", "reason"_attr = status);
+        for (auto&& expectedChar : expectedContextChars) {
+            auto contextStart = reason.find(':', reason.find("Bad character"_sd));
+            ASSERT_NE(contextStart, std::string::npos);
+            auto contextEnd = reason.find("Full input:");
+            ASSERT_NE(contextStart, std::string::npos);
+            auto index = reason.find(expectedChar, contextStart);
+            ASSERT(index < contextEnd)
+                << "Expected to find '" << expectedChar << "' in error message's context snippet: "
+                << reason.substr(contextStart, (contextEnd - contextStart));
+        }
+        // We expect to see this in each error message - showing the character position of the parse
+        // error, like clang error messages:
+        // "{a: 4"
+        //       ^
+        const char positionIndicator = '^';
+        ASSERT_NE(reason.find(positionIndicator), std::string::npos)
+            << "Expected to find the indicator character in the message: " << reason;
+    }
+}
+
+TEST(FromJsonTest, GivesErrorContext) {
+
+    // Missing close brace after 4. This is the easy case, the error is solidly in the middle of the
+    // string:
+    assertErrorWithContext("{$and: [{a: 4, {b: 3}]}", {'4'});
+    // Error right at the beginning:
+    assertErrorWithContext("answer: 4}", {'a'});
+    // Error right at the end:
+    assertErrorWithContext("{answer: 4", {'4'});
+    // Error in the middle of a short string:
+    assertErrorWithContext("{a 4}", {'a', '4'});
+    // Error at the beginning of a short string:
+    assertErrorWithContext("a: 4}", {'a'});
+    // Error at the end of a short string:
+    assertErrorWithContext("{a: 4", {'4'});
+    // Very large input string
+    assertErrorWithContext(R"({
+        a: 4,
+        b: 10,
+        c: [
+            {d: 1, e: 1},
+            {d: 2, e: 2},
+            {d: 3, e: 3}
+        ],
+        f: 5,
+        g: 6
+        z: 7,
+    })",
+                           // Error is missing comma after 6, before 'z'
+                           {'6', 'z'});
+}
+
 }  // namespace FromJsonTests
 }  // namespace
 }  // namespace mongo
