@@ -51,11 +51,13 @@ void dropDatabase(OperationContext* opCtx, const NamespaceString& nss) {
     auto db = databaseHolder->getDb(opCtx, nss.dbName());
 
     if (db) {
+        WriteUnitOfWork wuow(opCtx);
         databaseHolder->dropDb(opCtx, db);
+        wuow.commit();
     }
 }
 
-bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, const std::string& ns) {
+bool collectionExists(OperationContext* opCtx, OldClientContext* ctx, StringData ns) {
     return (bool)CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx,
                                                                             NamespaceString(ns));
 }
@@ -89,7 +91,7 @@ void insertRecord(OperationContext* opCtx, const NamespaceString& nss, const BSO
     auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, nss);
     OpDebug* const nullOpDebug = nullptr;
     ASSERT_OK(collection_internal::insertDocument(
-        opCtx, coll, InsertStatement(data), nullOpDebug, false));
+        opCtx, CollectionPtr(coll), InsertStatement(data), nullOpDebug, false));
 }
 
 void assertOnlyRecord(OperationContext* opCtx, const NamespaceString& nss, const BSONObj& data) {
@@ -148,11 +150,12 @@ size_t getNumIndexEntries(OperationContext* opCtx,
 
 void dropIndex(OperationContext* opCtx, const NamespaceString& nss, const std::string& idxName) {
     CollectionWriter coll(opCtx, nss);
-    auto desc =
-        coll.getWritableCollection(opCtx)->getIndexCatalog()->findIndexByName(opCtx, idxName);
-    ASSERT(desc);
-    ASSERT_OK(coll.getWritableCollection(opCtx)->getIndexCatalog()->dropIndex(
-        opCtx, coll.getWritableCollection(opCtx), desc));
+    auto writableEntry =
+        coll.getWritableCollection(opCtx)->getIndexCatalog()->getWritableEntryByName(opCtx,
+                                                                                     idxName);
+    ASSERT(writableEntry);
+    ASSERT_OK(coll.getWritableCollection(opCtx)->getIndexCatalog()->dropIndexEntry(
+        opCtx, coll.getWritableCollection(opCtx), writableEntry));
 }
 
 }  // namespace
