@@ -30,6 +30,7 @@
 
 #include "mongo/executor/connection_pool.h"
 
+#include "mongo/db/service_context.h"
 #include <fmt/format.h>
 #include <fmt/ostream.h>
 #include <memory>
@@ -97,6 +98,7 @@ void ConnectionPool::ConnectionInterface::indicateUsed() {
     // It is illegal to attempt to use a connection after calling indicateFailure().
     invariant(_status.isOK() || _status == ConnectionPool::kConnectionStateUnknown);
     _lastUsed = now();
+    _timesUsed++;
 }
 
 void ConnectionPool::ConnectionInterface::indicateSuccess() {
@@ -111,6 +113,10 @@ Date_t ConnectionPool::ConnectionInterface::getLastUsed() const {
     return _lastUsed;
 }
 
+size_t ConnectionPool::ConnectionInterface::getTimesUsed() const {
+    return _timesUsed;
+}
+
 const Status& ConnectionPool::ConnectionInterface::getStatus() const {
     return _status;
 }
@@ -140,8 +146,8 @@ std::string ConnectionPool::ConnectionControls::toString() const {
 }
 
 std::string ConnectionPool::HostState::toString() const {
-    return "{{ requests: {}, ready: {}, pending: {}, active: {}, isExpired: {} }}"_format(
-        requests, ready, pending, active, health.isExpired);
+    return "{{ requests: {}, ready: {}, pending: {}, active: {}, leased: {}, isExpired: {} }}"_format(
+        requests, ready, pending, active, leased, health.isExpired);
 }
 
 /**
@@ -165,7 +171,7 @@ public:
         const auto minConns = getPool()->_options.minConnections;
         const auto maxConns = getPool()->_options.maxConnections;
 
-        data.target = stats.requests + stats.active;
+        data.target = stats.requests + stats.active + stats.leased;
         if (data.target < minConns) {
             data.target = minConns;
         } else if (data.target > maxConns) {
@@ -278,7 +284,9 @@ public:
      * Gets a connection from the specific pool. Sinks a unique_lock from the
      * parent to preserve the lock on _mutex
      */
-    Future<ConnectionHandle> getConnection(Milliseconds timeout, ErrorCodes::Error timeoutCode);
+    Future<ConnectionHandle> getConnection(Milliseconds timeout,
+                                           bool lease,
+                                           ErrorCodes::Error timeoutCode);
 
     /**
      * Triggers the shutdown procedure. This function sets isShutdown to true
@@ -300,6 +308,11 @@ public:
      */
     size_t inUseConnections() const;
 
+    /**
+     * Returns the number of leased connections from the pool.
+     */
+    size_t leasedConnections() const;
+
     /**
      * Returns the number of available connections in the pool.
      */
@@ -325,10 +338,20 @@ public:
      */
     size_t neverUsedConnections() const;
 
+    /**
+     * Returns the number of connections that were used only once before being destroyed.
+     */
+    size_t getOnceUsedConnections() const;
+
+    /**
+     * Returns the cumulative amount of time connections were in use by operations.
+     */
+    Milliseconds getTotalConnUsageTime() const;
+
     /**
      * Returns the total number of connections currently open that belong to
      * this pool. This is the sum of refreshingConnections, availableConnections,
-     * and inUseConnections.
+     * inUseConnections, and leasedConnections.
      */
     size_t openConnections() const;
 
@@ -388,6 +411,8 @@ private:
     struct Request {
         Date_t expiration;
         Promise<ConnectionHandle> promise;
+        // Whether or not the requested connection should be "leased".
+        bool lease;
         ErrorCodes::Error timeoutCode;
     };
 
@@ -397,7 +422,7 @@ private:
         }
     };
 
-    ConnectionHandle makeHandle(ConnectionInterface* connection);
+    ConnectionHandle makeHandle(ConnectionInterface* connection, bool isLeased);
 
     /**
      * Given a uniquely-owned OwnedConnection, returns an OwnedConnection
@@ -409,9 +434,13 @@ private:
             ConnWrap(OwnedConnection conn, std::weak_ptr<SpecificPool> owner)
                 : conn{std::move(conn)}, owner{std::move(owner)} {}
             ~ConnWrap() {
-                if (conn->getLastUsed() == Date_t{})
+                if (conn->getTimesUsed() == 0) {
                     if (auto ownerSp = owner.lock())
-                        ++ownerSp->_neverUsed;
+                        ownerSp->_neverUsed.fetchAndAddRelaxed(1);
+                } else if (conn->getTimesUsed() == 1) {
+                    if (auto ownerSp = owner.lock())
+                        ownerSp->_usedOnce.fetchAndAddRelaxed(1);
+                }
             }
             const OwnedConnection conn;
             const std::weak_ptr<SpecificPool> owner;
@@ -431,11 +460,11 @@ private:
 
     void fulfillRequests();
 
-    void returnConnection(ConnectionInterface* connPtr);
+    void returnConnection(ConnectionInterface* connPtr, bool isLeased);
 
     // This internal helper is used both by get and by _fulfillRequests and differs in that it
     // skips some bookkeeping that the other callers do on their own
-    ConnectionHandle tryGetConnection();
+    ConnectionHandle tryGetConnection(bool lease);
 
     template <typename OwnershipPoolType>
     typename OwnershipPoolType::mapped_type takeFromPool(
@@ -464,6 +493,7 @@ private:
     OwnershipPool _processingPool;
     OwnershipPool _droppedProcessingPool;
     OwnershipPool _checkedOutPool;
+    OwnershipPool _leasedPool;
 
     std::vector<Request> _requests;
     Date_t _lastActiveTime;
@@ -487,7 +517,11 @@ private:
 
     size_t _refreshed = 0;
 
-    size_t _neverUsed = 0;
+    AtomicWord<size_t> _neverUsed{0};
+
+    AtomicWord<size_t> _usedOnce{0};
+
+    Milliseconds _totalConnUsageTime{0};
 
     ConnectionWaitTimeHistogram _connAcquisitionWaitTimeStats{};
 
@@ -522,7 +556,7 @@ ConnectionPool::ConnectionPool(std::shared_ptr<DependentTypeFactoryInterface> im
       _factory(std::move(impl)),
       _options(std::move(options)),
       _controller(_options.controllerFactory()),
-      _manager(options.egressTagCloserManager) {
+      _manager(_options.egressTagCloserManager) {
     if (_manager) {
         _manager->add(this);
     }
@@ -602,24 +636,40 @@ void ConnectionPool::mutateTags(
     pool->mutateTags(mutateFunc);
 }
 
-void ConnectionPool::get_forTest(const HostAndPort& hostAndPort,
-                                 Milliseconds timeout,
-                                 ErrorCodes::Error timeoutCode,
-                                 GetConnectionCallback cb) {
+void ConnectionPool::retrieve_forTest(RetrieveConnection retrieve, GetConnectionCallback cb) {
     // We kick ourselves onto the executor queue to prevent us from deadlocking with our own thread
     auto getConnectionFunc =
-        [this, hostAndPort, timeout, timeoutCode, cb = std::move(cb)](Status&&) mutable {
-            get(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode)
-                .thenRunOn(_factory->getExecutor())
-                .getAsync(std::move(cb));
+        [this, retrieve = std::move(retrieve), cb = std::move(cb)](Status&&) mutable {
+            retrieve().thenRunOn(_factory->getExecutor()).getAsync(std::move(cb));
         };
     _factory->getExecutor()->schedule(std::move(getConnectionFunc));
 }
 
-SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::get(const HostAndPort& hostAndPort,
-                                                                 transport::ConnectSSLMode sslMode,
-                                                                 Milliseconds timeout,
-                                                                 ErrorCodes::Error timeoutCode) {
+void ConnectionPool::get_forTest(const HostAndPort& hostAndPort,
+                                 Milliseconds timeout,
+                                 ErrorCodes::Error timeoutCode,
+                                 GetConnectionCallback cb) {
+    auto getConnectionFunc = [this, hostAndPort, timeout, timeoutCode]() mutable {
+        return get(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode);
+    };
+    retrieve_forTest(getConnectionFunc, std::move(cb));
+}
+
+void ConnectionPool::lease_forTest(const HostAndPort& hostAndPort,
+                                   Milliseconds timeout,
+                                   ErrorCodes::Error timeoutCode,
+                                   GetConnectionCallback cb) {
+    auto getConnectionFunc = [this, hostAndPort, timeout, timeoutCode]() mutable {
+        return lease(hostAndPort, transport::kGlobalSSLMode, timeout, timeoutCode);
+    };
+    retrieve_forTest(getConnectionFunc, std::move(cb));
+}
+
+SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::_get(const HostAndPort& hostAndPort,
+                                                                  transport::ConnectSSLMode sslMode,
+                                                                  Milliseconds timeout,
+                                                                  bool lease,
+                                                                  ErrorCodes::Error timeoutCode) {
     auto connRequestedAt = _factory->now();
 
     stdx::lock_guard lk(_mutex);
@@ -633,10 +683,13 @@ SemiFuture<ConnectionPool::ConnectionHandle> ConnectionPool::get(const HostAndPo
 
     invariant(pool);
 
-    auto connFuture = pool->getConnection(timeout, timeoutCode);
+    auto connFuture = pool->getConnection(timeout, lease, timeoutCode);
     pool->updateState();
 
-    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCV()) {
+    // Only count connections being checked-out for ordinary use, not lease, towards cumulative wait
+    // time.
+    // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+    if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe() && !lease) {
         connFuture = std::move(connFuture).tap([connRequestedAt, pool = pool](const auto& conn) {
             pool->recordConnectionWaitTime(connRequestedAt);
         });
@@ -655,12 +708,16 @@ void ConnectionPool::appendConnectionStats(ConnectionPoolStats* stats) const {
         auto& pool = kv.second;
         ConnectionStatsPer hostStats{pool->inUseConnections(),
                                      pool->availableConnections(),
+                                     pool->leasedConnections(),
                                      pool->createdConnections(),
                                      pool->refreshingConnections(),
                                      pool->refreshedConnections(),
-                                     pool->neverUsedConnections()};
+                                     pool->neverUsedConnections(),
+                                     pool->getOnceUsedConnections(),
+                                     pool->getTotalConnUsageTime()};
 
-        if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCV()) {
+        // (Ignore FCV check): This feature flag doesn't have any upgrade/downgrade concerns.
+        if (gFeatureFlagConnHealthMetrics.isEnabledAndIgnoreFCVUnsafe()) {
             hostStats.acquisitionWaitTimes = pool->connectionWaitTimeStats();
         }
         stats->updateStatsForHost(_name, host, hostStats);
@@ -695,6 +752,7 @@ ConnectionPool::SpecificPool::~SpecificPool() {
     if (shouldInvariantOnPoolCorrectness()) {
         invariant(_requests.empty());
         invariant(_checkedOutPool.empty());
+        invariant(_leasedPool.empty());
     }
 }
 
@@ -706,6 +764,10 @@ size_t ConnectionPool::SpecificPool::availableConnections() const {
     return _readyPool.size();
 }
 
+size_t ConnectionPool::SpecificPool::leasedConnections() const {
+    return _leasedPool.size();
+}
+
 size_t ConnectionPool::SpecificPool::refreshingConnections() const {
     return _processingPool.size();
 }
@@ -719,11 +781,19 @@ size_t ConnectionPool::SpecificPool::createdConnections() const {
 }
 
 size_t ConnectionPool::SpecificPool::neverUsedConnections() const {
-    return _neverUsed;
+    return _neverUsed.loadRelaxed();
+}
+
+size_t ConnectionPool::SpecificPool::getOnceUsedConnections() const {
+    return _usedOnce.loadRelaxed();
+}
+
+Milliseconds ConnectionPool::SpecificPool::getTotalConnUsageTime() const {
+    return _totalConnUsageTime;
 }
 
 size_t ConnectionPool::SpecificPool::openConnections() const {
-    return _checkedOutPool.size() + _readyPool.size() + _processingPool.size();
+    return _checkedOutPool.size() + _readyPool.size() + _processingPool.size() + _leasedPool.size();
 }
 
 size_t ConnectionPool::SpecificPool::requestsPending() const {
@@ -731,7 +801,7 @@ size_t ConnectionPool::SpecificPool::requestsPending() const {
 }
 
 Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnection(
-    Milliseconds timeout, ErrorCodes::Error timeoutCode) {
+    Milliseconds timeout, bool lease, ErrorCodes::Error timeoutCode) {
 
     // Reset our activity timestamp
     auto now = _parent->_factory->now();
@@ -765,7 +835,7 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
 
     // If we do not have requests, then we can fulfill immediately
     if (_requests.size() == 0) {
-        auto conn = tryGetConnection();
+        auto conn = tryGetConnection(lease);
 
         if (conn) {
             LOGV2_DEBUG(22559,
@@ -787,23 +857,32 @@ Future<ConnectionPool::ConnectionHandle> ConnectionPool::SpecificPool::getConnec
     const auto expiration = now + timeout;
     auto pf = makePromiseFuture<ConnectionHandle>();
 
-    _requests.push_back({expiration, std::move(pf.promise), timeoutCode});
+    _requests.push_back({expiration, std::move(pf.promise), lease, timeoutCode});
     std::push_heap(begin(_requests), end(_requests), RequestComparator{});
 
     return std::move(pf.future);
 }
 
-auto ConnectionPool::SpecificPool::makeHandle(ConnectionInterface* connection) -> ConnectionHandle {
-    auto deleter = [this, anchor = shared_from_this()](ConnectionInterface* connection) {
+auto ConnectionPool::SpecificPool::makeHandle(ConnectionInterface* connection, bool isLeased)
+    -> ConnectionHandle {
+    auto connUseStartedAt = _parent->_getFastClockSource()->now();
+    auto deleter = [this, anchor = shared_from_this(), connUseStartedAt, isLeased](
+                       ConnectionInterface* connection) {
         stdx::lock_guard lk(_parent->_mutex);
-        returnConnection(connection);
+
+        // Leased connections don't count towards the pool's total connection usage time.
+        if (!isLeased) {
+            _totalConnUsageTime += _parent->_getFastClockSource()->now() - connUseStartedAt;
+        }
+
+        returnConnection(connection, isLeased);
         _lastActiveTime = _parent->_factory->now();
         updateState();
     };
     return ConnectionHandle(connection, std::move(deleter));
 }
 
-ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection() {
+ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection(bool lease) {
     while (_readyPool.size()) {
         // _readyPool is an LRUCache, so its begin() object is the MRU item.
         auto iter = _readyPool.begin();
@@ -825,12 +904,15 @@ ConnectionPool::ConnectionHandle ConnectionPool::SpecificPool::tryGetConnection(
 
         auto connPtr = conn.get();
 
-        // check out the connection
-        _checkedOutPool[connPtr] = std::move(conn);
+        if (lease) {
+            _leasedPool[connPtr] = std::move(conn);
+        } else {
+            _checkedOutPool[connPtr] = std::move(conn);
+        }
 
         // pass it to the user
         connPtr->resetToUnknown();
-        auto handle = makeHandle(connPtr);
+        auto handle = makeHandle(connPtr, lease);
         return handle;
     }
 
@@ -866,15 +948,14 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
         return;
     }
 
-    // Pass a failure on through
-    if (!status.isOK()) {
-        LOGV2_DEBUG(22563,
+    // If the error can be contained to one connection, drop the one connection.
+    if (status.code() == ErrorCodes::ConnectionError) {
+        LOGV2_DEBUG(6832901,
                     kDiagnosticLogLevel,
-                    "Connection failed to {hostAndPort} due to {error}",
-                    "Connection failed",
+                    "Dropping single connection",
                     "hostAndPort"_attr = _hostAndPort,
-                    "error"_attr = redact(status));
-        processFailure(status);
+                    "error"_attr = redact(status),
+                    "numOpenConns"_attr = openConnections());
         return;
     }
 
@@ -888,6 +969,18 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
         return;
     }
 
+    // Pass a failure on through
+    if (!status.isOK()) {
+        LOGV2_DEBUG(22563,
+                    kDiagnosticLogLevel,
+                    "Connection failed to {hostAndPort} due to {error}",
+                    "Connection failed",
+                    "hostAndPort"_attr = _hostAndPort,
+                    "error"_attr = redact(status));
+        processFailure(status);
+        return;
+    }
+
     LOGV2_DEBUG(22565,
                 kDiagnosticLogLevel,
                 "Finishing connection refresh for {hostAndPort}",
@@ -900,10 +993,10 @@ void ConnectionPool::SpecificPool::finishRefresh(ConnectionInterface* connPtr, S
     fulfillRequests();
 }
 
-void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr) {
+void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr, bool isLeased) {
     auto needsRefreshTP = connPtr->getLastUsed() + _parent->_controller->toRefreshTimeout();
 
-    auto conn = takeFromPool(_checkedOutPool, connPtr);
+    auto conn = takeFromPool(isLeased ? _leasedPool : _checkedOutPool, connPtr);
     invariant(conn);
 
     if (_health.isShutdown) {
@@ -917,7 +1010,29 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
     }
 
     if (auto status = conn->getStatus(); !status.isOK()) {
-        // TODO: alert via some callback if the host is bad
+        // Our error handling here is determined by the MongoDB SDAM specification for handling
+        // application errors on established connections. In particular, if a network error occurs,
+        // we must close all idle sockets in the connection pool for the server: "if one socket is
+        // bad, it is likely that all are." However, if the error is just a network _timeout_ error,
+        // we don't drop the connections because the timeout may indicate a slow operation rather
+        // than an unavailable server. Additionally, if we can isolate the error to a single
+        // socket/connection based on it's type, we won't drop other connections/sockets.
+        //
+        // See the spec for additional details:
+        // https://github.com/mongodb/specifications/blob/master/source/server-discovery-and-monitoring/server-discovery-and-monitoring.rst#application-errors
+        bool isSingleConnectionError = status.code() == ErrorCodes::ConnectionError;
+        if (ErrorCodes::isNetworkError(status) && !isSingleConnectionError &&
+            !ErrorCodes::isNetworkTimeoutError(status)) {
+            LOGV2_DEBUG(7719500,
+                        kDiagnosticLogLevel,
+                        "Connection failed to {hostAndPort} due to {error}",
+                        "Connection failed",
+                        "hostAndPort"_attr = _hostAndPort,
+                        "error"_attr = redact(status));
+            processFailure(status);
+            return;
+        }
+        // Otherwise, drop the one connection.
         LOGV2(22566,
               "Ending connection to host {hostAndPort} due to bad connection status: {error}; "
               "{numOpenConns} connections to that host remain open",
@@ -938,8 +1053,7 @@ void ConnectionPool::SpecificPool::returnConnection(ConnectionInterface* connPtr
 
     if (shouldRefreshConnection) {
         auto controls = _parent->_controller->getControls(_id);
-        if (_readyPool.size() + _processingPool.size() + _checkedOutPool.size() >=
-            controls.targetConnections) {
+        if (openConnections() >= controls.targetConnections) {
             // If we already have minConnections, just let the connection lapse
             LOGV2(22567,
                   "Ending idle connection to host {hostAndPort} because the pool meets "
@@ -1006,7 +1120,7 @@ void ConnectionPool::SpecificPool::addToReady(OwnedConnection conn) {
 
         connPtr->indicateSuccess();
 
-        returnConnection(connPtr);
+        returnConnection(connPtr, false);
     });
     connPtr->setTimeout(_parent->_controller->toRefreshTimeout(), std::move(returnConnectionFunc));
 }
@@ -1095,7 +1209,7 @@ void ConnectionPool::SpecificPool::fulfillRequests() {
         // deadlock).
         //
         // None of the heap manipulation code throws, but it's something to keep in mind.
-        auto conn = tryGetConnection();
+        auto conn = tryGetConnection(_requests.front().lease);
 
         if (!conn) {
             break;
@@ -1211,7 +1325,8 @@ void ConnectionPool::SpecificPool::updateHealth() {
     const auto now = _parent->_factory->now();
 
     // We're expired if we have no sign of connection use and are past our expiry
-    _health.isExpired = _requests.empty() && _checkedOutPool.empty() && (_hostExpiration <= now);
+    _health.isExpired = _requests.empty() && _checkedOutPool.empty() && _leasedPool.empty() &&
+        (_hostExpiration <= now);
 
     // We're failed until we get new requests or our timer triggers
     if (_health.isFailed) {
@@ -1229,7 +1344,7 @@ void ConnectionPool::SpecificPool::updateEventTimer() {
     }
 
     // If our expiration comes before our next event, then it is the next event
-    if (_requests.empty() && _checkedOutPool.empty()) {
+    if (_requests.empty() && _checkedOutPool.empty() && _leasedPool.empty()) {
         _hostExpiration = _lastActiveTime + _parent->_controller->hostTimeout();
         if ((_hostExpiration > now) && (_hostExpiration < nextEventTime)) {
             nextEventTime = _hostExpiration;
@@ -1295,6 +1410,7 @@ void ConnectionPool::SpecificPool::updateController() {
         refreshingConnections(),
         availableConnections(),
         inUseConnections(),
+        leasedConnections(),
     };
     LOGV2_DEBUG(22578,
                 kDiagnosticLogLevel,
@@ -1333,6 +1449,7 @@ void ConnectionPool::SpecificPool::updateController() {
             if (shouldInvariantOnPoolCorrectness()) {
                 invariant(pool->_checkedOutPool.empty());
                 invariant(pool->_requests.empty());
+                invariant(pool->_leasedPool.empty());
             }
 
             pool->triggerShutdown(Status(ErrorCodes::ConnectionPoolExpired,
@@ -1381,5 +1498,9 @@ void ConnectionPool::SpecificPool::updateState() {
         });
 }
 
+ClockSource* ConnectionPool::DependentTypeFactoryInterface::getFastClockSource() {
+    return getGlobalServiceContext()->getFastClockSource();
+}
+
 }  // namespace executor
 }  // namespace mongo
