@@ -31,6 +31,7 @@
 
 #include <functional>
 #include <memory>
+#include <mutex>
 #include <queue>
 
 #include "mongo/config.h"
@@ -40,6 +41,7 @@
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/transport/session.h"
 #include "mongo/transport/transport_layer.h"
+#include "mongo/util/clock_source.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/future.h"
 #include "mongo/util/hierarchical_acquisition.h"
@@ -79,6 +81,7 @@ public:
     using ConnectionHandleDeleter = std::function<void(ConnectionInterface* connection)>;
     using ConnectionHandle = std::unique_ptr<ConnectionInterface, ConnectionHandleDeleter>;
 
+    using RetrieveConnection = unique_function<SemiFuture<ConnectionHandle>()>;
     using GetConnectionCallback = unique_function<void(StatusWith<ConnectionHandle>)>;
 
     using PoolId = uint64_t;
@@ -210,6 +213,7 @@ public:
         size_t pending = 0;
         size_t ready = 0;
         size_t active = 0;
+        size_t leased = 0;
 
         std::string toString() const;
     };
@@ -252,16 +256,39 @@ public:
                     const std::function<transport::Session::TagMask(transport::Session::TagMask)>&
                         mutateFunc) override;
 
-    SemiFuture<ConnectionHandle> get(
+    inline SemiFuture<ConnectionHandle> get(
         const HostAndPort& hostAndPort,
         transport::ConnectSSLMode sslMode,
         Milliseconds timeout,
-        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit);
+        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit) {
+        return _get(hostAndPort, sslMode, timeout, false /*lease*/, timeoutCode);
+    }
+
     void get_forTest(const HostAndPort& hostAndPort,
                      Milliseconds timeout,
                      ErrorCodes::Error timeoutCode,
                      GetConnectionCallback cb);
 
+    /**
+     * "Lease" a connection from the pool.
+     *
+     * Connections retrieved via this method are not assumed to be in active use for the duration of
+     * their lease and are reported separately in metrics. Otherwise, this method behaves similarly
+     * to `ConnectionPool::get`.
+     */
+    inline SemiFuture<ConnectionHandle> lease(
+        const HostAndPort& hostAndPort,
+        transport::ConnectSSLMode sslMode,
+        Milliseconds timeout,
+        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit) {
+        return _get(hostAndPort, sslMode, timeout, true /*lease*/, timeoutCode);
+    }
+
+    void lease_forTest(const HostAndPort& hostAndPort,
+                       Milliseconds timeout,
+                       ErrorCodes::Error timeoutCode,
+                       GetConnectionCallback cb);
+
     void appendConnectionStats(ConnectionPoolStats* stats) const;
 
     size_t getNumConnectionsPerHost(const HostAndPort& hostAndPort) const;
@@ -271,6 +298,17 @@ public:
     }
 
 private:
+    ClockSource* _getFastClockSource() const;
+
+    SemiFuture<ConnectionHandle> _get(
+        const HostAndPort& hostAndPort,
+        transport::ConnectSSLMode sslMode,
+        Milliseconds timeout,
+        bool leased,
+        ErrorCodes::Error timeoutCode = ErrorCodes::NetworkInterfaceExceededTimeLimit);
+
+    void retrieve_forTest(RetrieveConnection retrieve, GetConnectionCallback cb);
+
     std::string _name;
 
     const std::shared_ptr<DependentTypeFactoryInterface> _factory;
@@ -285,6 +323,9 @@ private:
     stdx::unordered_map<HostAndPort, std::shared_ptr<SpecificPool>> _pools;
 
     EgressTagCloserManager* _manager;
+
+    mutable ClockSource* _fastClockSource{nullptr};
+    mutable std::once_flag _fastClkSrcInitFlag;
 };
 
 /**
@@ -389,6 +430,11 @@ public:
      */
     Date_t getLastUsed() const;
 
+    /**
+     * Returns the number of times the connection was used by operations.
+     */
+    size_t getTimesUsed() const;
+
     /**
      * Returns the status associated with the connection. If the status is not
      * OK, the connection will not be returned to the pool.
@@ -432,6 +478,7 @@ protected:
 private:
     size_t _generation;
     Date_t _lastUsed;
+    size_t _timesUsed = 0;
     Status _status = ConnectionPool::kConnectionStateUnknown;
 };
 
@@ -499,6 +546,10 @@ public:
         return _pool;
     }
 
+    Options getPoolOptions() const {
+        return _pool->_options;
+    }
+
     virtual void updateConnectionPoolStats([[maybe_unused]] ConnectionPoolStats* cps) const = 0;
 
 protected:
@@ -542,11 +593,25 @@ public:
      */
     virtual Date_t now() = 0;
 
+    /**
+     * Returns the fast clock source.
+     * The default implementation gets it from the global service context.
+     */
+    virtual ClockSource* getFastClockSource();
+
     /**
      * shutdown
      */
     virtual void shutdown() = 0;
 };
 
+inline ClockSource* ConnectionPool::_getFastClockSource() const {
+    if (MONGO_unlikely(!_fastClockSource)) {
+        std::call_once(_fastClkSrcInitFlag,
+                       [&]() { _fastClockSource = _factory->getFastClockSource(); });
+    }
+    return _fastClockSource;
+}
+
 }  // namespace executor
 }  // namespace mongo
