@@ -27,9 +27,15 @@
  *    it in the license file.
  */
 
+#include "mongo/db/cursor_id.h"
+#include "mongo/db/query/cursor_response.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
+#include "mongo/idl/server_parameter_test_util.h"
 #include "mongo/s/catalog/type_database_gen.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/catalog_cache_loader_mock.h"
+#include "mongo/s/shard_cannot_refresh_due_to_locks_held_exception.h"
+#include "mongo/s/shard_version_factory.h"
 #include "mongo/s/sharding_router_test_fixture.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/s/type_collection_common_types_gen.h"
@@ -128,23 +134,29 @@ protected:
         }
     }
 
-    void loadCollection(const ChunkVersion& version) {
+    CollectionType loadCollection(const ShardVersion& version) {
         const auto coll = makeCollectionType(version);
         const auto scopedCollProv = scopedCollectionProvider(coll);
-        const auto scopedChunksProv = scopedChunksProvider(makeChunks(version));
+        const auto scopedChunksProv = scopedChunksProvider(makeChunks(version.placementVersion()));
+        auto future = launchAsync([&] {
+            onCommand([&](const executor::RemoteCommandRequest& request) {
+                return makeCollectionAndIndexesAggregationResponse(coll, std::vector<BSONObj>());
+            });
+        });
 
-        const auto swChunkManager =
+        const auto swCri =
             _catalogCache->getCollectionRoutingInfo(operationContext(), coll.getNss());
-        ASSERT_OK(swChunkManager.getStatus());
+        ASSERT_OK(swCri.getStatus());
+        future.default_timed_get();
+        return coll;
     }
 
     void loadUnshardedCollection(const NamespaceString& nss) {
         const auto scopedCollProvider =
             scopedCollectionProvider(Status(ErrorCodes::NamespaceNotFound, "collection not found"));
 
-        const auto swChunkManager =
-            _catalogCache->getCollectionRoutingInfo(operationContext(), nss);
-        ASSERT_OK(swChunkManager.getStatus());
+        const auto swCri = _catalogCache->getCollectionRoutingInfo(operationContext(), nss);
+        ASSERT_OK(swCri.getStatus());
     }
 
     std::vector<ChunkType> makeChunks(ChunkVersion version) {
@@ -157,22 +169,39 @@ protected:
         return {chunk};
     }
 
-    CollectionType makeCollectionType(const ChunkVersion& collVersion) {
-        return {kNss,
-                collVersion.epoch(),
-                collVersion.getTimestamp(),
-                Date_t::now(),
-                kUUID,
-                kShardKeyPattern.getKeyPattern()};
+    CollectionType makeCollectionType(const ShardVersion& collPlacementVersion) {
+        CollectionType coll{kNss,
+                            collPlacementVersion.placementVersion().epoch(),
+                            collPlacementVersion.placementVersion().getTimestamp(),
+                            Date_t::now(),
+                            kUUID,
+                            kShardKeyPattern.getKeyPattern()};
+        if (collPlacementVersion.indexVersion()) {
+            coll.setIndexVersion(CollectionIndexes(kUUID, *collPlacementVersion.indexVersion()));
+        }
+        return coll;
     }
 
-    const NamespaceString kNss{"catalgoCacheTestDB.foo"};
+    BSONObj makeCollectionAndIndexesAggregationResponse(const CollectionType& coll,
+                                                        const std::vector<BSONObj>& indexes) {
+        BSONObj obj = coll.toBSON();
+        BSONObjBuilder indexField;
+        indexField.append("indexes", indexes);
+        BSONObj newObj = obj.addField(indexField.obj().firstElement());
+        return CursorResponse(CollectionType::ConfigNS, CursorId{0}, {newObj})
+            .toBSON(CursorResponse::ResponseType::InitialResponse);
+    }
+
+    const NamespaceString kNss =
+        NamespaceString::createNamespaceString_forTest("catalgoCacheTestDB.foo");
     const UUID kUUID = UUID::gen();
     const std::string kPattern{"_id"};
     const ShardKeyPattern kShardKeyPattern{BSON(kPattern << 1)};
     const int kDummyPort{12345};
     const HostAndPort kConfigHostAndPort{"DummyConfig", kDummyPort};
     const std::vector<ShardId> kShards{{"0"}, {"1"}};
+    RAIIServerParameterControllerForTest featureFlagController{
+        "featureFlagGlobalIndexesShardingCatalog", true};
 
     std::shared_ptr<CatalogCacheLoaderMock> _catalogCacheLoader;
     std::unique_ptr<CatalogCache> _catalogCache;
@@ -259,7 +288,9 @@ TEST_F(CatalogCacheTest, OnStaleDatabaseVersionNoVersion) {
 
 TEST_F(CatalogCacheTest, OnStaleShardVersionWithSameVersion) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto cachedCollVersion = ChunkVersion({OID::gen(), Timestamp(1, 1)}, {1, 0});
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
 
     loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
@@ -270,7 +301,9 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithSameVersion) {
 
 TEST_F(CatalogCacheTest, OnStaleShardVersionWithNoVersion) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto cachedCollVersion = ChunkVersion({OID::gen(), Timestamp(1, 1)}, {1, 0});
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
 
     loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
@@ -281,11 +314,13 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithNoVersion) {
     ASSERT(status == ErrorCodes::InternalError);
 }
 
-TEST_F(CatalogCacheTest, OnStaleShardVersionWithGraterVersion) {
+TEST_F(CatalogCacheTest, OnStaleShardVersionWithGreaterPlacementVersion) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto cachedCollVersion = ChunkVersion({OID::gen(), Timestamp(1, 1)}, {1, 0});
-    const auto wantedCollVersion =
-        ChunkVersion({cachedCollVersion.epoch(), cachedCollVersion.getTimestamp()}, {2, 0});
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    const auto wantedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
 
     loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
     loadCollection(cachedCollVersion);
@@ -296,31 +331,110 @@ TEST_F(CatalogCacheTest, OnStaleShardVersionWithGraterVersion) {
     ASSERT(status == ErrorCodes::InternalError);
 }
 
+TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksReturnsImmediately) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+
+    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    auto coll = loadCollection(cachedCollVersion);
+
+    const auto swCri = _catalogCache->getCollectionRoutingInfo(
+        operationContext(), coll.getNss(), true /* allowLocks */);
+    ASSERT_OK(swCri.getStatus());
+
+    ASSERT(swCri.getValue().getCollectionVersion() == cachedCollVersion);
+}
+
+TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksNeedsToFetchNewCollInfo) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    const auto wantedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
+
+    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadCollection(cachedCollVersion);
+    _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
+        kNss, wantedCollVersion, kShards[0]);
+
+    {
+        FailPointEnableBlock failPoint("blockCollectionCacheLookup");
+
+        const auto status =
+            _catalogCache->getCollectionRoutingInfo(operationContext(), kNss, true /* allowLocks */)
+                .getStatus();
+
+        ASSERT(status == ErrorCodes::ShardCannotRefreshDueToLocksHeld);
+        auto refreshInfo = status.extraInfo<ShardCannotRefreshDueToLocksHeldInfo>();
+        ASSERT(refreshInfo);
+    }
+    // Cancel ongoing refresh
+    _catalogCache->invalidateCollectionEntry_LINEARIZABLE(kNss);
+}
+
+TEST_F(CatalogCacheTest, GetCollectionRoutingInfoAllowLocksNeedsToFetchNewDBInfo) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    const auto wantedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {2, 0}), boost::optional<CollectionIndexes>(boost::none));
+
+    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    loadCollection(cachedCollVersion);
+    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.db());
+
+    {
+        FailPointEnableBlock failPoint("blockDatabaseCacheLookup");
+
+        const auto status =
+            _catalogCache->getCollectionRoutingInfo(operationContext(), kNss, true /* allowLocks */)
+                .getStatus();
+
+        ASSERT(status == ErrorCodes::ShardCannotRefreshDueToLocksHeld);
+        auto refreshInfo = status.extraInfo<ShardCannotRefreshDueToLocksHeldInfo>();
+        ASSERT(refreshInfo);
+    }
+    // Cancel ongoing refresh
+    _catalogCache->invalidateDatabaseEntry_LINEARIZABLE(kNss.db());
+}
+
 TEST_F(CatalogCacheTest, TimeseriesFieldsAreProperlyPropagatedOnCC) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto epoch = OID::gen();
-    const auto version = ChunkVersion({epoch, Timestamp(42)}, {1, 0});
+    const auto gen = CollectionGeneration(OID::gen(), Timestamp(42));
+    const auto version = ShardVersionFactory::make(ChunkVersion(gen, {1, 0}),
+                                                   boost::optional<CollectionIndexes>(boost::none));
 
     loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
 
     auto coll = makeCollectionType(version);
-    auto chunks = makeChunks(version);
+    auto chunks = makeChunks(version.placementVersion());
     auto timeseriesOptions = TimeseriesOptions("fieldName");
 
-    // 1st refresh: we should find a bucket granularity of seconds (default)
+    // 1st refresh: we should find a bucket granularity of seconds
     {
         TypeCollectionTimeseriesFields tsFields;
+        timeseriesOptions.setGranularity(BucketGranularityEnum::Seconds);
         tsFields.setTimeseriesOptions(timeseriesOptions);
         coll.setTimeseriesFields(tsFields);
 
         const auto scopedCollProv = scopedCollectionProvider(coll);
         const auto scopedChunksProv = scopedChunksProvider(chunks);
+        auto future = launchAsync([&] {
+            onCommand([&](const executor::RemoteCommandRequest& request) {
+                return makeCollectionAndIndexesAggregationResponse(coll, std::vector<BSONObj>());
+            });
+        });
 
-        const auto swChunkManager =
+        const auto swCri =
             _catalogCache->getCollectionRoutingInfoWithRefresh(operationContext(), coll.getNss());
-        ASSERT_OK(swChunkManager.getStatus());
+        ASSERT_OK(swCri.getStatus());
+        future.default_timed_get();
 
-        const auto& chunkManager = swChunkManager.getValue();
+        const auto& chunkManager = swCri.getValue().cm;
         ASSERT(chunkManager.getTimeseriesFields().has_value());
         ASSERT(chunkManager.getTimeseriesFields()->getGranularity() ==
                BucketGranularityEnum::Seconds);
@@ -334,18 +448,24 @@ TEST_F(CatalogCacheTest, TimeseriesFieldsAreProperlyPropagatedOnCC) {
         coll.setTimeseriesFields(tsFields);
 
         auto& lastChunk = chunks.back();
-        ChunkVersion newCollectionVersion = lastChunk.getVersion();
-        newCollectionVersion.incMinor();
-        lastChunk.setVersion(newCollectionVersion);
+        ChunkVersion newCollectionPlacementVersion = lastChunk.getVersion();
+        newCollectionPlacementVersion.incMinor();
+        lastChunk.setVersion(newCollectionPlacementVersion);
 
         const auto scopedCollProv = scopedCollectionProvider(coll);
         const auto scopedChunksProv = scopedChunksProvider(std::vector{lastChunk});
+        auto future = launchAsync([&] {
+            onCommand([&](const executor::RemoteCommandRequest& request) {
+                return makeCollectionAndIndexesAggregationResponse(coll, std::vector<BSONObj>());
+            });
+        });
 
-        const auto swChunkManager =
+        const auto swCri =
             _catalogCache->getCollectionRoutingInfoWithRefresh(operationContext(), coll.getNss());
-        ASSERT_OK(swChunkManager.getStatus());
+        ASSERT_OK(swCri.getStatus());
+        future.default_timed_get();
 
-        const auto& chunkManager = swChunkManager.getValue();
+        const auto& chunkManager = swCri.getValue().cm;
         ASSERT(chunkManager.getTimeseriesFields().has_value());
         ASSERT(chunkManager.getTimeseriesFields()->getGranularity() ==
                BucketGranularityEnum::Hours);
@@ -354,8 +474,9 @@ TEST_F(CatalogCacheTest, TimeseriesFieldsAreProperlyPropagatedOnCC) {
 
 TEST_F(CatalogCacheTest, LookupCollectionWithInvalidOptions) {
     const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
-    const auto epoch = OID::gen();
-    const auto version = ChunkVersion({epoch, Timestamp(42)}, {1, 0});
+    const auto gen = CollectionGeneration(OID::gen(), Timestamp(42));
+    const auto version = ShardVersionFactory::make(ChunkVersion(gen, {1, 0}),
+                                                   boost::optional<CollectionIndexes>(boost::none));
 
     loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
 
@@ -365,11 +486,65 @@ TEST_F(CatalogCacheTest, LookupCollectionWithInvalidOptions) {
     const auto scopedChunksProv = scopedChunksProvider(StatusWith<std::vector<ChunkType>>(
         ErrorCodes::InvalidOptions, "Testing error with invalid options"));
 
-    const auto swChunkManager =
+    const auto swCri =
         _catalogCache->getCollectionRoutingInfoWithRefresh(operationContext(), coll.getNss());
 
-    ASSERT_EQUALS(swChunkManager.getStatus(), ErrorCodes::InvalidOptions);
+    ASSERT_EQUALS(swCri.getStatus(), ErrorCodes::InvalidOptions);
 }
 
+
+TEST_F(CatalogCacheTest, OnStaleShardVersionWithGreaterIndexVersion) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), boost::optional<CollectionIndexes>(boost::none));
+    const auto wantedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), CollectionIndexes(kUUID, Timestamp(1, 0)));
+
+    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    CollectionType coll = loadCollection(cachedCollVersion);
+    _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
+        kNss, wantedCollVersion, kShards[0]);
+
+    auto future = launchAsync([&] {
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            coll.setIndexVersion({coll.getUuid(), Timestamp(1, 0)});
+            return makeCollectionAndIndexesAggregationResponse(
+                coll,
+                {IndexCatalogType("x_1", BSON("x" << 1), BSONObj(), Timestamp(1, 0), coll.getUuid())
+                     .toBSON()});
+        });
+    });
+
+    const auto routingInfo = _catalogCache->getCollectionRoutingInfo(operationContext(), kNss);
+    future.default_timed_get();
+}
+
+TEST_F(CatalogCacheTest, OnStaleShardVersionIndexVersionBumpNotNone) {
+    const auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
+    const CollectionGeneration gen(OID::gen(), Timestamp(1, 1));
+    const auto cachedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), CollectionIndexes(kUUID, Timestamp(1, 0)));
+    const auto wantedCollVersion = ShardVersionFactory::make(
+        ChunkVersion(gen, {1, 0}), CollectionIndexes(kUUID, Timestamp(2, 0)));
+
+    loadDatabases({DatabaseType(kNss.db().toString(), kShards[0], dbVersion)});
+    CollectionType coll = loadCollection(cachedCollVersion);
+    _catalogCache->invalidateShardOrEntireCollectionEntryForShardedCollection(
+        kNss, wantedCollVersion, kShards[0]);
+
+    auto future = launchAsync([&] {
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            coll.setIndexVersion({coll.getUuid(), Timestamp(2, 0)});
+            return makeCollectionAndIndexesAggregationResponse(
+                coll,
+                {IndexCatalogType("x_1", BSON("x" << 1), BSONObj(), Timestamp(2, 0), coll.getUuid())
+                     .toBSON()});
+        });
+    });
+
+    const auto routingInfo = _catalogCache->getCollectionRoutingInfo(operationContext(), kNss);
+    future.default_timed_get();
+}
 }  // namespace
 }  // namespace mongo
