@@ -32,7 +32,9 @@
 #include "mongo/client/remote_command_targeter_factory_mock.h"
 #include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/client.h"
+#include "mongo/db/cursor_id.h"
 #include "mongo/db/query/collation/collator_factory_mock.h"
+#include "mongo/db/query/cursor_response.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database_gen.h"
@@ -40,11 +42,22 @@
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/sharding_feature_flags_gen.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
+BSONObj makeCollectionAndIndexesAggregationResponse(const CollectionType& coll,
+                                                    const std::vector<BSONObj>& indexes) {
+    BSONObj obj = coll.toBSON();
+    BSONObjBuilder indexField;
+    indexField.append("indexes", indexes);
+    BSONObj newObj = obj.addField(indexField.obj().firstElement());
+    return CursorResponse(CollectionType::ConfigNS, CursorId{0}, {newObj})
+        .toBSON(CursorResponse::ResponseType::InitialResponse);
+}
+
 void CatalogCacheTestFixture::setUp() {
     ShardingTestFixture::setUp();
     setRemote(HostAndPort("FakeRemoteClient:34567"));
@@ -53,32 +66,33 @@ void CatalogCacheTestFixture::setUp() {
     CollatorFactoryInterface::set(getServiceContext(), std::make_unique<CollatorFactoryMock>());
 }
 
-executor::NetworkTestEnv::FutureHandle<boost::optional<ChunkManager>>
+executor::NetworkTestEnv::FutureHandle<boost::optional<CollectionRoutingInfo>>
 CatalogCacheTestFixture::scheduleRoutingInfoForcedRefresh(const NamespaceString& nss) {
     return launchAsync([this, nss] {
         auto client = getServiceContext()->makeClient("Test");
         auto const catalogCache = Grid::get(getServiceContext())->catalogCache();
 
-        return boost::make_optional(uassertStatusOK(
-            catalogCache->getCollectionRoutingInfoWithRefresh(operationContext(), nss)));
+        auto cri = uassertStatusOK(
+            catalogCache->getCollectionRoutingInfoWithRefresh(operationContext(), nss));
+        return boost::make_optional(cri);
     });
 }
 
-executor::NetworkTestEnv::FutureHandle<boost::optional<ChunkManager>>
+executor::NetworkTestEnv::FutureHandle<boost::optional<CollectionRoutingInfo>>
 CatalogCacheTestFixture::scheduleRoutingInfoUnforcedRefresh(const NamespaceString& nss) {
     return launchAsync([this, nss] {
         auto client = getServiceContext()->makeClient("Test");
         auto const catalogCache = Grid::get(getServiceContext())->catalogCache();
 
-        return boost::optional<ChunkManager>(
-            uassertStatusOK(catalogCache->getCollectionRoutingInfo(operationContext(), nss)));
+        auto cri = uassertStatusOK(catalogCache->getCollectionRoutingInfo(operationContext(), nss));
+        return boost::make_optional(cri);
     });
 }
 
-executor::NetworkTestEnv::FutureHandle<boost::optional<ChunkManager>>
+executor::NetworkTestEnv::FutureHandle<boost::optional<CollectionRoutingInfo>>
 CatalogCacheTestFixture::scheduleRoutingInfoIncrementalRefresh(const NamespaceString& nss) {
     auto catalogCache = Grid::get(getServiceContext())->catalogCache();
-    const auto cm =
+    const auto [cm, _] =
         uassertStatusOK(catalogCache->getCollectionRoutingInfo(operationContext(), nss));
     ASSERT(cm.isSharded());
 
@@ -117,18 +131,21 @@ std::vector<ShardType> CatalogCacheTestFixture::setupNShards(int numShards) {
     return shards;
 }
 
-ChunkManager CatalogCacheTestFixture::makeChunkManager(
+CollectionRoutingInfo CatalogCacheTestFixture::makeCollectionRoutingInfo(
     const NamespaceString& nss,
     const ShardKeyPattern& shardKeyPattern,
     std::unique_ptr<CollatorInterface> defaultCollator,
     bool unique,
     const std::vector<BSONObj>& splitPoints,
+    const std::vector<BSONObj>& globalIndexes,
     boost::optional<ReshardingFields> reshardingFields) {
     ChunkVersion version({OID::gen(), Timestamp(42)}, {1, 0});
 
     DatabaseType db(nss.db().toString(), {"0"}, DatabaseVersion(UUID::gen(), Timestamp()));
 
     const auto uuid = UUID::gen();
+    boost::optional<Timestamp> indexVersion =
+        globalIndexes.empty() ? boost::none : boost::make_optional(Timestamp(11, 12));
     const BSONObj collectionBSON = [&]() {
         CollectionType coll(nss,
                             version.epoch(),
@@ -146,6 +163,10 @@ ChunkManager CatalogCacheTestFixture::makeChunkManager(
             coll.setReshardingFields(std::move(reshardingFields));
         }
 
+        if (indexVersion) {
+            coll.setIndexVersion({uuid, *indexVersion});
+        }
+
         return coll.toBSON();
     }();
 
@@ -184,6 +205,14 @@ ChunkManager CatalogCacheTestFixture::makeChunkManager(
                        [](const auto& chunk) { return BSON("chunks" << chunk); });
         return aggResult;
     }());
+    if (feature_flags::gGlobalIndexesShardingCatalog.isEnabledAndIgnoreFCVUnsafe()) {
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            ASSERT_EQ(request.target, kConfigHostAndPort);
+            ASSERT_EQ(request.dbname, "config");
+            return makeCollectionAndIndexesAggregationResponse(CollectionType(collectionBSON),
+                                                               globalIndexes);
+        });
+    }
 
     return *future.default_timed_get();
 }
@@ -227,6 +256,28 @@ void CatalogCacheTestFixture::expectCollectionAndChunksAggregation(
     }());
 }
 
+void CatalogCacheTestFixture::expectCollectionAndIndexesAggregation(
+    NamespaceString nss,
+    OID epoch,
+    Timestamp timestamp,
+    UUID uuid,
+    const ShardKeyPattern& shardKeyPattern,
+    boost::optional<Timestamp> indexVersion,
+    const std::vector<BSONObj>& indexes) {
+    if (feature_flags::gGlobalIndexesShardingCatalog.isEnabledAndIgnoreFCVUnsafe()) {
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            ASSERT_EQ(request.target, kConfigHostAndPort);
+            ASSERT_EQ(request.dbname, "config");
+            CollectionType collType(
+                nss, epoch, timestamp, Date_t::now(), uuid, shardKeyPattern.toBSON());
+            if (indexVersion) {
+                collType.setIndexVersion({uuid, *indexVersion});
+            }
+            return makeCollectionAndIndexesAggregationResponse(collType, indexes);
+        });
+    }
+}
+
 ChunkManager CatalogCacheTestFixture::loadRoutingTableWithTwoChunksAndTwoShards(
     NamespaceString nss) {
 
@@ -260,10 +311,8 @@ ChunkManager CatalogCacheTestFixture::loadRoutingTableWithTwoChunksAndTwoShardsI
             expectGetDatabase(nss);
         }
     }
+    CollectionType collType(nss, epoch, timestamp, Date_t::now(), uuid, shardKeyPattern.toBSON());
     expectFindSendBSONObjVector(kConfigHostAndPort, [&]() {
-        CollectionType collType(
-            nss, epoch, timestamp, Date_t::now(), uuid, shardKeyPattern.toBSON());
-
         ChunkVersion version({epoch, timestamp}, {1, 0});
 
         ChunkType chunk1(
@@ -280,8 +329,15 @@ ChunkManager CatalogCacheTestFixture::loadRoutingTableWithTwoChunksAndTwoShardsI
         const auto chunk2Obj = BSON("chunks" << chunk2.toConfigBSON());
         return std::vector<BSONObj>{collType.toBSON(), chunk1Obj, chunk2Obj};
     }());
+    if (feature_flags::gGlobalIndexesShardingCatalog.isEnabledAndIgnoreFCVUnsafe()) {
+        onCommand([&](const executor::RemoteCommandRequest& request) {
+            ASSERT_EQ(request.target, kConfigHostAndPort);
+            ASSERT_EQ(request.dbname, "config");
+            return makeCollectionAndIndexesAggregationResponse(collType, {});
+        });
+    }
 
-    return *future.default_timed_get();
+    return future.default_timed_get()->cm;
 }
 
 }  // namespace mongo
