@@ -27,12 +27,10 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/s/client/shard_registry.h"
 
 #include "mongo/client/replica_set_monitor.h"
+#include "mongo/db/catalog_shard_feature_flag_gen.h"
 #include "mongo/db/client.h"
 #include "mongo/db/vector_clock.h"
 #include "mongo/db/vector_clock_metadata_hook.h"
@@ -44,13 +42,13 @@
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/type_shard.h"
+#include "mongo/s/client/config_shard_wrapper.h"
 #include "mongo/s/grid.h"
 #include "mongo/util/future_util.h"
 #include "mongo/util/str.h"
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
-
 namespace mongo {
 
 namespace {
@@ -65,7 +63,7 @@ using CallbackArgs = executor::TaskExecutor::CallbackArgs;
 
 ShardRegistry::ShardRegistry(ServiceContext* service,
                              std::unique_ptr<ShardFactory> shardFactory,
-                             const ConnectionString& configServerCS,
+                             const boost::optional<ConnectionString>& configServerCS,
                              std::vector<ShardRemovalHook> shardRemovalHooks)
     : _service(service),
       _shardFactory(std::move(shardFactory)),
@@ -88,7 +86,13 @@ ShardRegistry::ShardRegistry(ServiceContext* service,
                  const Time& timeInStore) { return _lookup(opCtx, key, cachedData, timeInStore); },
           1 /* cacheSize */)) {
 
-    invariant(_initConfigServerCS.isValid());
+    if (_initConfigServerCS) {
+        invariant(_initConfigServerCS->isValid());
+    } else {
+        // (Ignore FCV check): This is in mongos so we expect to ignore FCV.
+        invariant(gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe());
+    }
+
     _threadPool.startup();
 }
 
@@ -99,24 +103,24 @@ ShardRegistry::~ShardRegistry() {
 void ShardRegistry::init() {
     invariant(!_isInitialized.load());
 
-    LOGV2_DEBUG(5123000,
-                1,
-                "Initializing ShardRegistry",
-                "configServers"_attr = _initConfigServerCS.toString());
-
     /* The creation of the config shard object will intialize the associated RSM monitor that in
      * turn will call ShardRegistry::updateReplSetHosts(). Hence the config shard object MUST be
      * created after the ShardRegistry is fully constructed. This is why `_configShardData`
      * is initialized here rather than in the ShardRegistry constructor.
      */
-    {
-        stdx::lock_guard<Latch> lk(_mutex);
-        _configShardData = ShardRegistryData::createWithConfigShardOnly(
-            _shardFactory->createShard(ShardId::kConfigServerId, _initConfigServerCS));
-        _latestConnStrings[_initConfigServerCS.getSetName()] = _initConfigServerCS;
-    }
+    if (_initConfigServerCS) {
+        LOGV2_DEBUG(
+            5123000, 1, "Initializing ShardRegistry", "configServers"_attr = _initConfigServerCS);
 
-    _isInitialized.store(true);
+        stdx::lock_guard<Latch> lk(_mutex);
+        _initConfigShard(lk, *_initConfigServerCS);
+        _isInitialized.store(true);
+    } else {
+        LOGV2_DEBUG(
+            7208800,
+            1,
+            "Deferring ShardRegistry initialization until local replica set config is known");
+    }
 }
 
 ShardRegistry::Cache::LookupResult ShardRegistry::_lookup(OperationContext* opCtx,
@@ -126,7 +130,9 @@ ShardRegistry::Cache::LookupResult ShardRegistry::_lookup(OperationContext* opCt
     invariant(key == _kSingleton);
     // This function can potentially block for a long time on network activity, so holding of locks
     // is disallowed.
-    invariant(!opCtx->lockState() || !opCtx->lockState()->isLocked());
+    tassert(7032320,
+            "Can't perform ShardRegistry lookup while holding locks",
+            !opCtx->lockState() || !opCtx->lockState()->isLocked());
 
 
     auto lastForcedReloadIncrement = _forceReloadIncrement.load();
@@ -184,11 +190,13 @@ ShardRegistry::Cache::LookupResult ShardRegistry::_lookup(OperationContext* opCt
         invariant(shard);
 
         auto name = shard->getConnString().getSetName();
-        ReplicaSetMonitor::remove(name);
+        if (shardId != ShardId::kConfigServerId) {
+            // Don't remove the config shard's RSM because it is used to target the config server.
+            ReplicaSetMonitor::remove(name);
+        }
         _removeReplicaSet(name);
         for (auto& callback : _shardRemovalHooks) {
             // Run callbacks asynchronously.
-            // TODO SERVER-50906: Consider running these callbacks synchronously.
             ExecutorFuture<void>(Grid::get(opCtx)->getExecutorPool()->getFixedExecutor())
                 .getAsync([=](const Status&) { callback(shardId); });
         }
@@ -269,7 +277,10 @@ ConnectionString ShardRegistry::getConfigServerConnectionString() const {
 
 std::shared_ptr<Shard> ShardRegistry::getConfigShard() const {
     stdx::lock_guard<Latch> lk(_mutex);
-    return _configShardData.findShard(ShardId::kConfigServerId);
+    auto configShard = _configShardData.findShard(ShardId::kConfigServerId);
+    // Note this should only throw if the local node has not learned its replica set config yet.
+    uassert(ErrorCodes::NotYetInitialized, "Config shard has not been set up yet", configShard);
+    return std::make_shared<ConfigShardWrapper>(configShard);
 }
 
 StatusWith<std::shared_ptr<Shard>> ShardRegistry::getShard(OperationContext* opCtx,
@@ -396,7 +407,6 @@ void ShardRegistry::updateReplSetHosts(const ConnectionString& givenConnString,
             auto newData = ShardRegistryData::createFromExisting(
                 _configShardData, newConnString, _shardFactory.get());
             _configShardData = newData;
-
         } else {
             auto value = _rsmIncrement.addAndFetch(1);
             LOGV2_DEBUG(4620252,
@@ -408,23 +418,20 @@ void ShardRegistry::updateReplSetHosts(const ConnectionString& givenConnString,
     }
 
     // Schedule a lookup, to incorporate the new connection string.
-    _getDataAsync()
-        .thenRunOn(Grid::get(_service)->getExecutorPool()->getFixedExecutor())
-        .ignoreValue()
-        .getAsync([](const Status& status) {
-            if (!status.isOK()) {
-                LOGV2(4620201,
-                      "Error running reload of ShardRegistry for RSM update, caused by {error}",
-                      "Error running reload of ShardRegistry for RSM update",
-                      "error"_attr = redact(status));
-            }
-        });
+    _scheduleLookup();
 }
 
 std::unique_ptr<Shard> ShardRegistry::createConnection(const ConnectionString& connStr) const {
     return _shardFactory->createUniqueShard(ShardId("<unnamed>"), connStr);
 }
 
+std::shared_ptr<Shard> ShardRegistry::createLocalConfigShard() const {
+    invariant(serverGlobalParams.clusterRole.has(ClusterRole::ConfigServer));
+    std::shared_ptr<Shard> configShard =
+        _shardFactory->createShard(ShardId::kConfigServerId, ConnectionString::forLocal());
+    return std::make_shared<ConfigShardWrapper>(configShard);
+}
+
 void ShardRegistry::toBSON(BSONObjBuilder* result) const {
     BSONObjBuilder map;
     BSONObjBuilder hosts;
@@ -462,15 +469,6 @@ void ShardRegistry::updateReplicaSetOnConfigServer(ServiceContext* serviceContex
     auto const grid = Grid::get(opCtx.get());
     auto sr = grid->shardRegistry();
 
-    // First check if this is a config shard lookup.
-    {
-        stdx::lock_guard<Latch> lk(sr->_mutex);
-        if (auto shard = sr->_configShardData.findByRSName(connStr.getSetName())) {
-            // No need to tell the config servers their own connection string.
-            return;
-        }
-    }
-
     auto swRegistryData = sr->_getDataAsync().getNoThrow(opCtx.get());
     if (!swRegistryData.isOK()) {
         LOGV2_DEBUG(
@@ -496,7 +494,7 @@ void ShardRegistry::updateReplicaSetOnConfigServer(ServiceContext* serviceContex
         NamespaceString::kConfigsvrShardsNamespace,
         BSON(ShardType::name(shard->getId().toString())),
         BSON("$set" << BSON(ShardType::host(connStr.toString()))),
-        false,
+        false /* upsert */,
         ShardingCatalogClient::kMajorityWriteConcern);
     auto status = swWasUpdated.getStatus();
     if (!status.isOK()) {
@@ -550,6 +548,41 @@ std::shared_ptr<Shard> ShardRegistry::getShardForHostNoReload(const HostAndPort&
     return data->findByHostAndPort(host);
 }
 
+void ShardRegistry::_scheduleLookup() {
+    _getDataAsync()
+        .thenRunOn(Grid::get(_service)->getExecutorPool()->getFixedExecutor())
+        .ignoreValue()
+        .getAsync([](const Status& status) {
+            if (!status.isOK()) {
+                LOGV2(4620201,
+                      "Error running reload of ShardRegistry for RSM update, caused by {error}",
+                      "Error running reload of ShardRegistry for RSM update",
+                      "error"_attr = redact(status));
+            }
+        });
+}
+
+void ShardRegistry::_initConfigShard(WithLock wl, const ConnectionString& configCS) {
+    _configShardData = ShardRegistryData::createWithConfigShardOnly(
+        _shardFactory->createShard(ShardId::kConfigServerId, configCS));
+    _latestConnStrings[configCS.getSetName()] = configCS;
+}
+
+void ShardRegistry::initConfigShardIfNecessary(const ConnectionString& configCS) {
+    {
+        stdx::lock_guard<Latch> lk(_mutex);
+        if (_isInitialized.load()) {
+            return;
+        }
+
+        _initConfigShard(lk, configCS);
+        _isInitialized.store(true);
+    }
+
+    // Lookup can succeed now that the registry has a real config shard, so schedule one right away.
+    _scheduleLookup();
+}
+
 ////////////// ShardRegistryData //////////////////
 
 ShardRegistryData ShardRegistryData::createWithConfigShardOnly(std::shared_ptr<Shard> configShard) {
@@ -563,7 +596,7 @@ std::pair<ShardRegistryData, Timestamp> ShardRegistryData::createFromCatalogClie
     auto const catalogClient = Grid::get(opCtx)->catalogClient();
 
     auto shardsAndOpTime = uassertStatusOKWithContext(
-        catalogClient->getAllShards(opCtx, repl::ReadConcernLevel::kMajorityReadConcern),
+        catalogClient->getAllShards(opCtx, repl::ReadConcernLevel::kSnapshotReadConcern),
         "could not get updated shard list from config server");
 
     auto shards = std::move(shardsAndOpTime.value);
@@ -605,10 +638,6 @@ std::pair<ShardRegistryData, Timestamp> ShardRegistryData::createFromCatalogClie
 
     ShardRegistryData data;
     for (auto& shardInfo : shardsInfo) {
-        if (std::get<0>(shardInfo) == "config") {
-            continue;
-        }
-
         auto shard = shardFactory->createShard(std::move(std::get<0>(shardInfo)),
                                                std::move(std::get<1>(shardInfo)));
 
