@@ -27,18 +27,13 @@
  *    it in the license file.
  */
 
-
-#include "mongo/platform/basic.h"
-
 #include "mongo/s/mongos_main.h"
 
 #include <boost/optional.hpp>
-#include <memory>
 
 #include "mongo/base/init.h"
 #include "mongo/base/initializer.h"
 #include "mongo/base/status.h"
-#include "mongo/client/connpool.h"
 #include "mongo/client/dbclient_rs.h"
 #include "mongo/client/global_conn_pool.h"
 #include "mongo/client/remote_command_targeter_factory_impl.h"
@@ -51,9 +46,11 @@
 #include "mongo/db/change_stream_options_manager.h"
 #include "mongo/db/client.h"
 #include "mongo/db/client_metadata_propagation_egress_hook.h"
+#include "mongo/db/commands.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/ftdc/ftdc_mongos.h"
 #include "mongo/db/initialize_server_global_state.h"
+#include "mongo/db/keys_collection_client_sharded.h"
 #include "mongo/db/log_process_details.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/operation_context.h"
@@ -89,6 +86,7 @@
 #include "mongo/s/mongos_topology_coordinator.h"
 #include "mongo/s/query/cluster_cursor_cleanup_job.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
+#include "mongo/s/query_analysis_sampler.h"
 #include "mongo/s/read_write_concern_defaults_cache_lookup_mongos.h"
 #include "mongo/s/service_entry_point_mongos.h"
 #include "mongo/s/session_catalog_router.h"
@@ -100,6 +98,7 @@
 #include "mongo/scripting/dbdirectclient_factory.h"
 #include "mongo/scripting/engine.h"
 #include "mongo/stdx/thread.h"
+#include "mongo/transport/ingress_handshake_metrics.h"
 #include "mongo/transport/transport_layer_manager.h"
 #include "mongo/util/admin_access.h"
 #include "mongo/util/cmdline_utils/censor_cmdline.h"
@@ -127,8 +126,11 @@
 #include "mongo/util/text.h"
 #include "mongo/util/version.h"
 
-#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
+#ifdef __linux__
+#include <sys/prctl.h>
+#endif
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 namespace mongo {
 
@@ -246,7 +248,10 @@ void implicitlyAbortAllTransactions(OperationContext* opCtx) {
         OperationContextSession sessionCtx(newOpCtx, std::move(killDetails.killToken));
 
         auto session = OperationContextSession::get(newOpCtx);
-        newOpCtx->setLogicalSessionId(session->getSessionId());
+        {
+            auto lk = stdx::lock_guard(*newOpCtx->getClient());
+            newOpCtx->setLogicalSessionId(session->getSessionId());
+        }
 
         auto txnRouter = TransactionRouter::get(newOpCtx);
         if (txnRouter.isInitialized()) {
@@ -310,9 +315,18 @@ void cleanupTask(const ShutdownTaskArgs& shutdownArgs) {
             lsc->joinOnShutDown();
         }
 
+        if (analyze_shard_key::isFeatureFlagEnabled()) {
+            LOGV2_OPTIONS(
+                6973901, {LogComponent::kDefault}, "Shutting down the QueryAnalysisSampler");
+            analyze_shard_key::QueryAnalysisSampler::get(serviceContext).onShutdown();
+        }
+
         ReplicaSetMonitor::shutdown();
 
-        opCtx->setIsExecutingShutdown();
+        {
+            stdx::lock_guard lg(client);
+            opCtx->setIsExecutingShutdown();
+        }
 
         if (serviceContext) {
             serviceContext->setKillAllOperations();
@@ -341,6 +355,7 @@ void cleanupTask(const ShutdownTaskArgs& shutdownArgs) {
         }
 
         if (auto pool = Grid::get(opCtx)->getExecutorPool()) {
+            LOGV2_OPTIONS(7698300, {LogComponent::kSharding}, "Shutting down the ExecutorPool");
             pool->shutdownAndJoin();
         }
 
@@ -349,6 +364,13 @@ void cleanupTask(const ShutdownTaskArgs& shutdownArgs) {
         }
 
         if (Grid::get(serviceContext)->isShardingInitialized()) {
+            // The CatalogCache must be shuted down before shutting down the CatalogCacheLoader as
+            // the CatalogCache may try to schedule work on CatalogCacheLoader and fail.
+            LOGV2_OPTIONS(7698301, {LogComponent::kSharding}, "Shutting down the CatalogCache");
+            Grid::get(serviceContext)->catalogCache()->shutDownAndJoin();
+
+            LOGV2_OPTIONS(
+                7698302, {LogComponent::kSharding}, "Shutting down the CatalogCacheLoader");
             CatalogCacheLoader::get(serviceContext).shutDown();
         }
 
@@ -436,13 +458,16 @@ Status initializeSharding(OperationContext* opCtx) {
             hookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
             return hookList;
         },
-        boost::none);
+        boost::none,
+        [](ShardingCatalogClient* catalogClient) {
+            return std::make_unique<KeysCollectionClientSharded>(catalogClient);
+        });
 
     if (!status.isOK()) {
         return status;
     }
 
-    status = loadGlobalSettingsFromConfigServer(opCtx);
+    status = loadGlobalSettingsFromConfigServer(opCtx, Grid::get(opCtx)->catalogClient());
     if (!status.isOK()) {
         return status;
     }
@@ -492,7 +517,9 @@ public:
     void onFoundSet(const Key& key) noexcept final {}
 
     void onConfirmedSet(const State& state) noexcept final {
-        auto connStr = state.connStr;
+        const auto& connStr = state.connStr;
+        const auto& setName = connStr.getSetName();
+
         try {
             LOGV2(471693,
                   "Updating the shard registry with confirmed replica set",
@@ -507,16 +534,17 @@ public:
                   "error"_attr = e);
         }
 
-        auto setName = connStr.getSetName();
         bool updateInProgress = false;
         {
             stdx::lock_guard lock(_mutex);
             if (!_hasUpdateState(lock, setName)) {
-                _updateStates.emplace(setName, std::make_shared<ReplSetConfigUpdateState>());
+                _updateStates.emplace(std::piecewise_construct,
+                                      std::forward_as_tuple(setName),
+                                      std::forward_as_tuple());
             }
-            auto updateState = _updateStates.at(setName);
-            updateState->nextUpdateToSend = connStr;
-            updateInProgress = updateState->updateInProgress;
+            auto& updateState = _updateStates.at(setName);
+            updateState.nextUpdateToSend = connStr;
+            updateInProgress = updateState.updateInProgress;
         }
 
         if (!updateInProgress) {
@@ -544,26 +572,28 @@ public:
 private:
     // Schedules updates for replica set 'setName' on the config server. Loosly preserves ordering
     // of update execution. Newer updates will not be overwritten by older updates in config.shards.
-    void _scheduleUpdateConfigServer(std::string setName) {
-        ConnectionString update;
+    void _scheduleUpdateConfigServer(const std::string& setName) {
+        ConnectionString updatedConnectionString;
         {
             stdx::lock_guard lock(_mutex);
             if (!_hasUpdateState(lock, setName)) {
                 return;
             }
-            auto updateState = _updateStates.at(setName);
-            if (updateState->updateInProgress) {
+            auto& updateState = _updateStates.at(setName);
+            if (updateState.updateInProgress) {
                 return;
             }
-            updateState->updateInProgress = true;
-            update = updateState->nextUpdateToSend.value();
-            updateState->nextUpdateToSend = boost::none;
+            updateState.updateInProgress = true;
+            updatedConnectionString = updateState.nextUpdateToSend.value();
+            updateState.nextUpdateToSend = boost::none;
         }
 
         auto executor = Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor();
         auto schedStatus =
             executor
-                ->scheduleWork([self = shared_from_this(), setName, update](auto args) {
+                ->scheduleWork([self = shared_from_this(),
+                                setName,
+                                update = std::move(updatedConnectionString)](const auto& args) {
                     self->_updateConfigServer(args.status, setName, update);
                 })
                 .getStatus();
@@ -579,7 +609,9 @@ private:
         uassertStatusOK(schedStatus);
     }
 
-    void _updateConfigServer(Status status, std::string setName, ConnectionString update) {
+    void _updateConfigServer(const Status& status,
+                             const std::string& setName,
+                             const ConnectionString& update) {
         if (ErrorCodes::isCancellationError(status.code())) {
             stdx::lock_guard lock(_mutex);
             _updateStates.erase(setName);
@@ -607,28 +639,28 @@ private:
         _endUpdateConfigServer(setName, update);
     }
 
-    void _endUpdateConfigServer(std::string setName, ConnectionString update) {
+    void _endUpdateConfigServer(const std::string& setName, const ConnectionString& update) {
         bool moreUpdates = false;
         {
             stdx::lock_guard lock(_mutex);
             invariant(_hasUpdateState(lock, setName));
-            auto updateState = _updateStates.at(setName);
-            updateState->updateInProgress = false;
-            moreUpdates = (updateState->nextUpdateToSend != boost::none);
+            auto& updateState = _updateStates.at(setName);
+            updateState.updateInProgress = false;
+            moreUpdates = (updateState.nextUpdateToSend != boost::none);
             if (!moreUpdates) {
                 _updateStates.erase(setName);
             }
         }
         if (moreUpdates) {
             auto executor = Grid::get(_serviceContext)->getExecutorPool()->getFixedExecutor();
-            executor->schedule([self = shared_from_this(), setName](auto args) {
+            executor->schedule([self = shared_from_this(), setName](const auto& _) {
                 self->_scheduleUpdateConfigServer(setName);
             });
         }
     }
 
     // Returns true if a ReplSetConfigUpdateState exists for replica set setName.
-    bool _hasUpdateState(WithLock, std::string setName) {
+    bool _hasUpdateState(WithLock, const std::string& setName) {
         return (_updateStates.find(setName) != _updateStates.end());
     }
 
@@ -637,11 +669,15 @@ private:
     mutable Mutex _mutex = MONGO_MAKE_LATCH("ShardingReplicaSetChangeListenerMongod::mutex");
 
     struct ReplSetConfigUpdateState {
+        ReplSetConfigUpdateState() = default;
+        ReplSetConfigUpdateState(const ReplSetConfigUpdateState&) = delete;
+        ReplSetConfigUpdateState& operator=(const ReplSetConfigUpdateState&) = delete;
+
         // True when an update to the config.shards is in progress.
         bool updateInProgress = false;
         boost::optional<ConnectionString> nextUpdateToSend;
     };
-    stdx::unordered_map<std::string, std::shared_ptr<ReplSetConfigUpdateState>> _updateStates;
+    stdx::unordered_map<std::string, ReplSetConfigUpdateState> _updateStates;
 };
 
 ExitCode runMongosServer(ServiceContext* serviceContext) {
@@ -738,6 +774,9 @@ ExitCode runMongosServer(ServiceContext* serviceContext) {
                       "error"_attr = redact(ex));
     }
 
+    CommandInvocationHooks::set(serviceContext,
+                                std::make_unique<transport::IngressHandshakeMetricsCommandHooks>());
+
     startMongoSFTDC();
 
     if (mongosGlobalParams.scriptingEnabled) {
@@ -761,7 +800,8 @@ ExitCode runMongosServer(ServiceContext* serviceContext) {
     clusterCursorCleanupJob.go();
 
     UserCacheInvalidator::start(serviceContext, opCtx);
-    if (gFeatureFlagClusterWideConfigM2.isEnabled(serverGlobalParams.featureCompatibility)) {
+    if (gFeatureFlagClusterWideConfigM2.isEnabled(
+            serverGlobalParams.featureCompatibility.acquireFCVSnapshot())) {
         ClusterServerParameterRefresher::start(serviceContext, opCtx);
     }
 
@@ -781,6 +821,8 @@ ExitCode runMongosServer(ServiceContext* serviceContext) {
         return ExitCode::processHealthCheck;
     }
 
+    srand((unsigned)(curTimeMicros64()) ^ (unsigned(uintptr_t(&opCtx))));  // NOLINT
+
     SessionKiller::set(serviceContext,
                        std::make_shared<SessionKiller>(serviceContext, killSessionsRemote));
 
@@ -855,6 +897,18 @@ std::unique_ptr<AuthzManagerExternalState> createAuthzManagerExternalStateMongos
     return std::make_unique<AuthzManagerExternalStateMongos>();
 }
 
+void disableMongosTHPUnderTestingEnvironment() {
+#ifdef __linux__
+    if (TestingProctor::instance().isEnabled()) {
+        if (prctl(PR_SET_THP_DISABLE, 1, 0, 0, 0) == -1) {
+            LOGV2_WARNING(8751802, "Could not disable THP on mongos");
+        } else {
+            LOGV2_INFO(8751803, "Successfully disabled THP on mongos");
+        }
+    }
+#endif
+}
+
 ExitCode main(ServiceContext* serviceContext) {
     serviceContext->setFastClockSource(FastClockSourceFactory::create(Milliseconds{10}));
 
@@ -897,7 +951,7 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersionLatest,
                                      ("EndStartupOptionStorage"))
 // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
 (InitializerContext* context) {
-    serverGlobalParams.mutableFeatureCompatibility.setVersion(multiversion::GenericFCV::kLatest);
+    serverGlobalParams.mutableFCV.setVersion(multiversion::GenericFCV::kLatest);
 }
 
 #ifdef MONGO_CONFIG_SSL
@@ -929,6 +983,8 @@ ExitCode mongos_main(int argc, char* argv[]) {
         return ExitCode::abrupt;
     }
 
+    disableMongosTHPUnderTestingEnvironment();
+
     try {
         auto serviceContextHolder = ServiceContext::make();
         serviceContextHolder->registerClientObserver(
