@@ -104,16 +104,16 @@ AsyncResultsMerger::AsyncResultsMerger(OperationContext* opCtx,
                                        AsyncResultsMergerParams params)
     : _opCtx(opCtx),
       _executor(std::move(executor)),
+      _params(std::move(params)),
       // This strange initialization is to work around the fact that the IDL does not currently
       // support a default value for an enum. The default tailable mode should be 'kNormal', but
       // since that is not supported we treat boost::none (unspecified) to mean 'kNormal'.
-      _tailableMode(params.getTailableMode().value_or(TailableModeEnum::kNormal)),
-      _params(std::move(params)),
+      _tailableMode(_params.getTailableMode().value_or(TailableModeEnum::kNormal)),
       _mergeQueue(MergingComparator(
           _remotes, _params.getSort().value_or(BSONObj()), _params.getCompareWholeSortKey())),
       _promisedMinSortKeys(PromisedMinSortKeyComparator(_params.getSort().value_or(BSONObj()))) {
-    if (params.getTxnNumber()) {
-        invariant(params.getSessionId());
+    if (_params.getTxnNumber()) {
+        invariant(_params.getSessionId());
     }
 
     size_t remoteIndex = 0;
@@ -500,11 +500,24 @@ Status AsyncResultsMerger::_scheduleGetMores(WithLock lk) {
     // Before scheduling more work, check whether the cursor has been invalidated.
     _assertNotInvalidated(lk);
 
+    // Reveal opCtx errors (such as MaxTimeMSExpired) and reflect them in the remote status.
+    invariant(_opCtx, "Cannot schedule a getMore without an OperationContext");
+    const auto interruptStatus = _opCtx->checkForInterruptNoAssert();
+    if (!interruptStatus.isOK()) {
+        for (size_t i = 0; i < _remotes.size(); ++i) {
+            if (!_remotes[i].exhausted()) {
+                _cleanUpFailedBatch(lk, interruptStatus, i);
+            }
+        }
+        return interruptStatus;
+    }
+
     // Schedule remote work on hosts for which we need more results.
     for (size_t i = 0; i < _remotes.size(); ++i) {
         auto& remote = _remotes[i];
 
         if (!remote.status.isOK()) {
+            _cleanUpFailedBatch(lk, remote.status, i);
             return remote.status;
         }
 
@@ -812,7 +825,7 @@ bool AsyncResultsMerger::_addBatchToBuffer(WithLock lk,
             }
         }
 
-        ClusterQueryResult result(obj);
+        ClusterQueryResult result(obj, remote.shardId);
         remote.docBuffer.push(result);
         ++remote.fetchedCount;
     }
@@ -844,16 +857,22 @@ bool AsyncResultsMerger::_haveOutstandingBatchRequests(WithLock) {
     return false;
 }
 
-void AsyncResultsMerger::_scheduleKillCursors(WithLock, OperationContext* opCtx) {
+void AsyncResultsMerger::_scheduleKillCursors(WithLock lk, OperationContext* opCtx) {
     invariant(_killCompleteInfo);
 
     for (const auto& remote : _remotes) {
-        if (remote.status.isOK() && remote.cursorId && !remote.exhausted()) {
+        if (_shouldKillRemote(lk, remote)) {
             BSONObj cmdObj =
                 KillCursorsCommandRequest(_params.getNss(), {remote.cursorId}).toBSON(BSONObj{});
 
-            executor::RemoteCommandRequest request(
-                remote.getTargetHost(), _params.getNss().db().toString(), cmdObj, opCtx);
+            executor::RemoteCommandRequest::Options options;
+            options.fireAndForget = true;
+            executor::RemoteCommandRequest request(remote.getTargetHost(),
+                                                   _params.getNss().db().toString(),
+                                                   cmdObj,
+                                                   rpc::makeEmptyMetadata(),
+                                                   opCtx,
+                                                   options);
             // The 'RemoteCommandRequest' takes the remaining time from the 'opCtx' parameter. If
             // the cursor was killed due to a maxTimeMs timeout, the remaining time will be 0, and
             // the remote request will not be sent. To avoid this, we remove the timeout for the
@@ -866,6 +885,13 @@ void AsyncResultsMerger::_scheduleKillCursors(WithLock, OperationContext* opCtx)
     }
 }
 
+bool AsyncResultsMerger::_shouldKillRemote(WithLock, const RemoteCursorData& remote) {
+    static const std::set<ErrorCodes::Error> kCursorAlreadyDeadCodes = {
+        ErrorCodes::QueryPlanKilled, ErrorCodes::CursorKilled, ErrorCodes::CursorNotFound};
+    return remote.cursorId && !remote.exhausted() &&
+        !kCursorAlreadyDeadCodes.count(remote.status.code());
+}
+
 stdx::shared_future<void> AsyncResultsMerger::kill(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
 
