@@ -40,6 +40,7 @@
 #include "mongo/platform/mutex.h"
 #include "mongo/s/query/async_results_merger_params_gen.h"
 #include "mongo/s/query/cluster_query_result.h"
+#include "mongo/stdx/future.h"
 #include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
@@ -231,22 +232,19 @@ public:
 
     /**
      * Starts shutting down this ARM by canceling all pending requests and scheduling killCursors
-     * on all of the unexhausted remotes. Returns a handle to an event that is signaled when this
-     * ARM is safe to destroy.
+     * on all of the unexhausted remotes. Returns a 'future' that is signaled when this ARM is safe
+     * to destroy.
      *
-     * If there are no pending requests, schedules killCursors and signals the event immediately.
+     * If there are no pending requests, schedules killCursors and signals the future immediately.
      * Otherwise, the last callback that runs after kill() is called signals the event.
      *
-     * Returns an invalid handle if the underlying task executor is shutting down. In this case,
-     * killing is considered complete and the ARM may be destroyed immediately.
-     *
      * May be called multiple times (idempotent).
      *
      * Note that 'opCtx' may or may not be the same as the operation context to which this cursor is
      * currently attached. This is so that a killing thread may call this method with its own
      * operation context.
      */
-    executor::TaskExecutor::EventHandle kill(OperationContext* opCtx);
+    stdx::shared_future<void> kill(OperationContext* opCtx);
 
 private:
     /**
@@ -301,8 +299,9 @@ private:
         // The identity of the shard which the cursor belongs to.
         ShardId shardId;
 
-        // This flag is set if the connection to the remote shard was lost, or never established in
-        // the first place. Only applicable if the 'allowPartialResults' option is enabled.
+        // This flag is set if the connection to the remote shard was lost or never established in
+        // the first place or the connection is interrupted due to MaxTimeMSExpired.
+        // Only applicable if the 'allowPartialResults' option is enabled.
         bool partialResultsReturned = false;
 
         // The buffer of results that have been retrieved but not yet returned to the caller.
@@ -318,6 +317,9 @@ private:
         // Count of fetched docs during ARM processing of the current batch. Used to reduce the
         // batchSize in getMore when mongod returned less docs than the requested batchSize.
         long long fetchedCount = 0;
+
+        // If set to 'true', the cursor on this shard has been invalidated.
+        bool invalidated = false;
     };
 
     class MergingComparator {
@@ -440,6 +442,12 @@ private:
      */
     bool _haveOutstandingBatchRequests(WithLock);
 
+    /**
+     * Called internally when attempting to get a new event for the caller to wait on. Throws if
+     * the shard cursor from which the next result is due has already been invalidated.
+     */
+    void _assertNotInvalidated(WithLock);
+
     /**
      * If a promisedMinSortKey has been received from all remotes, returns the lowest such key.
      * Otherwise, returns boost::none.
@@ -456,6 +464,11 @@ private:
      */
     void _scheduleKillCursors(WithLock, OperationContext* opCtx);
 
+    /**
+     * Checks if we need to schedule a killCursor command for this remote
+     */
+    bool _shouldKillRemote(WithLock, const RemoteCursorData& remote);
+
     /**
      * Updates the given remote's metadata (e.g. the cursor id) based on information in 'response'.
      */
@@ -477,8 +490,8 @@ private:
 
     OperationContext* _opCtx;
     std::shared_ptr<executor::TaskExecutor> _executor;
-    TailableModeEnum _tailableMode;
     AsyncResultsMergerParams _params;
+    TailableModeEnum _tailableMode;
 
     // Must be acquired before accessing any data members (other than _params, which is read-only).
     mutable Mutex _mutex = MONGO_MAKE_LATCH("AsyncResultsMerger::_mutex");
@@ -517,9 +530,28 @@ private:
 
     LifecycleState _lifecycleState = kAlive;
 
-    // Signaled when all outstanding batch request callbacks have run after kill() has been
-    // called. This means that the ARM is safe to delete.
-    executor::TaskExecutor::EventHandle _killCompleteEvent;
+    // Handles the promise/future mechanism used to cleanly shut down the ARM. This avoids race
+    // conditions in cases where the underlying TaskExecutor is simultaneously being torn down.
+    struct KillCompletePromiseFuture {
+        KillCompletePromiseFuture() : _future(_promise.get_future()){};
+
+        // Multiple calls to kill() can be made and each must return a future that will be notified
+        // when the ARM has been cleaned up.
+        stdx::shared_future<void> getFuture() {
+            return _future;
+        }
+
+        // Called by the ARM when all outstanding requests have run. Notifies all threads waiting on
+        // shared futures that the ARM has been cleaned up.
+        void signalFutures() {
+            _promise.set_value();
+        }
+
+    private:
+        stdx::promise<void> _promise;
+        stdx::shared_future<void> _future;
+    };
+    boost::optional<KillCompletePromiseFuture> _killCompleteInfo;
 };
 
 }  // namespace mongo
