@@ -37,6 +37,7 @@
 #include "mongo/db/api_parameters.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/collection_uuid_mismatch_info.h"
+#include "mongo/db/catalog_shard_feature_flag_gen.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/curop.h"
@@ -55,6 +56,10 @@
 #include "mongo/db/query/explain_common.h"
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/query/fle/server_rewrite.h"
+#include "mongo/db/query/query_stats/agg_key.h"
+#include "mongo/db/query/query_stats/key.h"
+#include "mongo/db/query/query_stats/query_stats.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/db/views/view.h"
@@ -97,7 +102,7 @@ namespace {
 // definition. It's okay that this is incorrect, we will repopulate the real namespace map on the
 // mongod. Note that this function must be called before forwarding an aggregation command on an
 // unsharded collection, in order to verify that the involved namespaces are allowed to be sharded.
-auto resolveInvolvedNamespaces(stdx::unordered_set<NamespaceString> involvedNamespaces) {
+auto resolveInvolvedNamespaces(const stdx::unordered_set<NamespaceString>& involvedNamespaces) {
     StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
     for (auto&& nss : involvedNamespaces) {
         resolvedNamespaces.try_emplace(nss.coll(), nss, std::vector<BSONObj>{});
@@ -165,7 +170,7 @@ void appendEmptyResultSetWithStatus(OperationContext* opCtx,
     if (status == ErrorCodes::ShardNotFound) {
         status = {ErrorCodes::NamespaceNotFound, status.reason()};
     }
-    appendEmptyResultSet(opCtx, *result, status, nss.ns());
+    appendEmptyResultSet(opCtx, *result, status, nss);
 }
 
 void updateHostsTargetedMetrics(OperationContext* opCtx,
@@ -193,7 +198,7 @@ void updateHostsTargetedMetrics(OperationContext* opCtx,
             if (nss == executionNss)
                 continue;
 
-            const auto resolvedNsCM =
+            const auto [resolvedNsCM, _] =
                 uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
             if (resolvedNsCM.isSharded()) {
                 std::set<ShardId> shardIdsForNs;
@@ -217,26 +222,39 @@ void updateHostsTargetedMetrics(OperationContext* opCtx,
 }
 
 /**
- * Performs validations related to API versioning and time-series stages.
+ * Performs validations related to API versioning, time-series stages, and general command
+ * validation.
  * Throws UserAssertion if any of the validations fails
  *     - validation of API versioning on each stage on the pipeline
  *     - validation of API versioning on 'AggregateCommandRequest' request
  *     - validation of time-series related stages
+ *     - validation of command parameters
  */
 void performValidationChecks(const OperationContext* opCtx,
                              const AggregateCommandRequest& request,
                              const LiteParsedPipeline& liteParsedPipeline) {
     liteParsedPipeline.validate(opCtx);
     aggregation_request_helper::validateRequestForAPIVersion(opCtx, request);
+    aggregation_request_helper::validateRequestFromClusterQueryWithoutShardKey(request);
 }
 
 /**
  * Rebuilds the pipeline and uses a different granularity value for the 'bucketMaxSpanSeconds' field
  * in the $_internalUnpackBucket stage.
  */
-std::vector<BSONObj> rebuildPipelineWithTimeSeriesGranularity(const std::vector<BSONObj>& pipeline,
-                                                              BucketGranularityEnum granularity) {
-    const auto bucketSpan = timeseries::getMaxSpanSecondsFromGranularity(granularity);
+std::vector<BSONObj> rebuildPipelineWithTimeSeriesGranularity(
+    const std::vector<BSONObj>& pipeline,
+    boost::optional<BucketGranularityEnum> granularity,
+    boost::optional<int32_t> maxSpanSeconds) {
+    int32_t bucketSpan = 0;
+
+    if (maxSpanSeconds) {
+        bucketSpan = *maxSpanSeconds;
+    } else {
+        bucketSpan = timeseries::getMaxSpanSecondsFromGranularity(
+            granularity.get_value_or(BucketGranularityEnum::Seconds));
+    }
+
     std::vector<BSONObj> newPipeline;
     for (auto& stage : pipeline) {
         if (stage.firstElementFieldNameStringData() ==
@@ -262,6 +280,62 @@ std::vector<BSONObj> rebuildPipelineWithTimeSeriesGranularity(const std::vector<
     return newPipeline;
 }
 
+/**
+ * Builds an expCtx with which to parse the request's pipeline, then parses the pipeline and
+ * registers the pre-optimized pipeline with query stats collection.
+ */
+std::unique_ptr<Pipeline, PipelineDeleter> parsePipelineAndRegisterQueryStats(
+    OperationContext* opCtx,
+    const stdx::unordered_set<NamespaceString>& involvedNamespaces,
+    const NamespaceString& executionNss,
+    AggregateCommandRequest& request,
+    boost::optional<CollectionRoutingInfo> cri,
+    bool hasChangeStream,
+    bool shouldDoFLERewrite) {
+    // Populate the collection UUID and the appropriate collation to use.
+    auto [collationObj, uuid] = [&]() -> std::pair<BSONObj, boost::optional<UUID>> {
+        // If this is a change stream, take the user-defined collation if one exists, or an
+        // empty BSONObj otherwise. Change streams never inherit the collection's default
+        // collation, and since collectionless aggregations generally run on the 'admin'
+        // database, the standard logic would attempt to resolve its non-existent UUID and
+        // collation by sending a specious 'listCollections' command to the config servers.
+        if (hasChangeStream) {
+            return {request.getCollation().value_or(BSONObj()), boost::none};
+        }
+
+        return cluster_aggregation_planner::getCollationAndUUID(
+            opCtx,
+            cri ? boost::make_optional(cri->cm) : boost::none,
+            executionNss,
+            request.getCollation().value_or(BSONObj()));
+    }();
+
+    // Build an ExpressionContext for the pipeline. This instantiates an appropriate collator,
+    // includes all involved namespaces, and creates a shared MongoProcessInterface for use by
+    // the pipeline's stages.
+    boost::intrusive_ptr<ExpressionContext> expCtx =
+        makeExpressionContext(opCtx,
+                              request,
+                              collationObj,
+                              uuid,
+                              resolveInvolvedNamespaces(involvedNamespaces),
+                              hasChangeStream);
+
+    auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+    // Skip query stats recording for queryable encryption queries.
+    if (!shouldDoFLERewrite) {
+        query_stats::registerRequest(
+            opCtx,
+            executionNss,
+            [&]() {
+                return std::make_unique<query_stats::AggKey>(
+                    request, *pipeline, expCtx, involvedNamespaces, executionNss);
+            },
+            hasChangeStream);
+    }
+    return pipeline;
+}
+
 }  // namespace
 
 Status ClusterAggregate::runAggregate(OperationContext* opCtx,
@@ -287,7 +361,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                                       AggregateCommandRequest& request,
                                       const LiteParsedPipeline& liteParsedPipeline,
                                       const PrivilegeVector& privileges,
-                                      boost::optional<ChunkManager> cm,
+                                      boost::optional<CollectionRoutingInfo> cri,
                                       BSONObjBuilder* result) {
     // Perform some validations on the LiteParsedPipeline and request before continuing with the
     // aggregation command.
@@ -305,7 +379,8 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
             !request.getNeedsMerge() && !request.getFromMongos());
 
     const auto isSharded = [](OperationContext* opCtx, const NamespaceString& nss) {
-        const auto resolvedNsCM = uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
+        const auto [resolvedNsCM, _] =
+            uassertStatusOK(getCollectionRoutingInfoForTxnCmd(opCtx, nss));
         return resolvedNsCM.isSharded();
     };
 
@@ -317,7 +392,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
     auto startsWithDocuments = liteParsedPipeline.startsWithDocuments();
 
     // If the routing table is not already taken by the higher level, fill it now.
-    if (!cm) {
+    if (!cri) {
         // If the routing table is valid, we obtain a reference to it. If the table is not valid,
         // then either the database does not exist, or there are no shards in the cluster. In the
         // latter case, we always return an empty cursor. In the former case, if the requested
@@ -329,7 +404,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
             sharded_agg_helpers::getExecutionNsRoutingInfo(opCtx, namespaces.executionNss);
 
         if (!executionNsRoutingInfoStatus.isOK()) {
-            uassert(CollectionUUIDMismatchInfo(request.getDbName().toString(),
+            uassert(CollectionUUIDMismatchInfo(request.getDbName(),
                                                *request.getCollectionUUID(),
                                                request.getNamespace().coll().toString(),
                                                boost::none),
@@ -344,7 +419,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
         }
 
         if (executionNsRoutingInfoStatus.isOK()) {
-            cm = std::move(executionNsRoutingInfoStatus.getValue());
+            cri = executionNsRoutingInfoStatus.getValue();
         } else if (!((hasChangeStream || startsWithDocuments) &&
                      executionNsRoutingInfoStatus == ErrorCodes::NamespaceNotFound)) {
             appendEmptyResultSetWithStatus(
@@ -355,46 +430,35 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
 
     boost::intrusive_ptr<ExpressionContext> expCtx;
     const auto pipelineBuilder = [&]() {
-        // Populate the collection UUID and the appropriate collation to use.
-        auto [collationObj, uuid] = [&]() -> std::pair<BSONObj, boost::optional<UUID>> {
-            // If this is a change stream, take the user-defined collation if one exists, or an
-            // empty BSONObj otherwise. Change streams never inherit the collection's default
-            // collation, and since collectionless aggregations generally run on the 'admin'
-            // database, the standard logic would attempt to resolve its non-existent UUID and
-            // collation by sending a specious 'listCollections' command to the config servers.
-            if (hasChangeStream) {
-                return {request.getCollation().value_or(BSONObj()), boost::none};
-            }
-
-            return cluster_aggregation_planner::getCollationAndUUID(
-                opCtx, cm, namespaces.executionNss, request.getCollation().value_or(BSONObj()));
-        }();
-
-        // Build an ExpressionContext for the pipeline. This instantiates an appropriate collator,
-        // resolves all involved namespaces, and creates a shared MongoProcessInterface for use by
-        // the pipeline's stages.
-        expCtx = makeExpressionContext(opCtx,
-                                       request,
-                                       collationObj,
-                                       uuid,
-                                       resolveInvolvedNamespaces(involvedNamespaces),
-                                       hasChangeStream);
-
-        // Parse and optimize the full pipeline.
-        auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+        auto pipeline = parsePipelineAndRegisterQueryStats(opCtx,
+                                                           involvedNamespaces,
+                                                           namespaces.executionNss,
+                                                           request,
+                                                           cri,
+                                                           hasChangeStream,
+                                                           shouldDoFLERewrite);
+        expCtx = pipeline->getContext();
 
         // If the aggregate command supports encrypted collections, do rewrites of the pipeline to
         // support querying against encrypted fields.
         if (shouldDoFLERewrite) {
-            // After this rewriting, the encryption info does not need to be kept around.
-            pipeline = processFLEPipelineS(opCtx,
-                                           namespaces.executionNss,
-                                           request.getEncryptionInformation().value(),
-                                           std::move(pipeline));
-            request.setEncryptionInformation(boost::none);
+            if (!request.getEncryptionInformation()->getCrudProcessed().value_or(false)) {
+                pipeline = processFLEPipelineS(opCtx,
+                                               namespaces.executionNss,
+                                               request.getEncryptionInformation().value(),
+                                               std::move(pipeline));
+                request.getEncryptionInformation()->setCrudProcessed(true);
+            }
+            stdx::lock_guard<Client> lk(*opCtx->getClient());
+            CurOp::get(opCtx)->setShouldOmitDiagnosticInformation_inlock(lk, true);
         }
 
         pipeline->optimizePipeline();
+
+        // Validate the pipeline post-optimization.
+        const bool alreadyOptimized = true;
+        pipeline->validateCommon(alreadyOptimized);
+
         return pipeline;
     };
 
@@ -406,7 +470,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
         opCtx,
         namespaces.executionNss,
         pipelineBuilder,
-        cm,
+        cri,
         involvedNamespaces,
         hasChangeStream,
         startsWithDocuments,
@@ -422,20 +486,53 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                 cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kMongosRequired);
 
     if (!expCtx) {
-        // When the AggregationTargeter chooses a "passthrough" policy, it does not call the
-        // 'pipelineBuilder' function, so we never get an expression context. Because this is a
-        // passthrough, we only need a bare minimum expression context anyway.
+        // When the AggregationTargeter chooses a "passthrough" or "specific shard only"
+        // policy, it does not call the 'pipelineBuilder' function, so we've yet to construct an
+        // expression context or register query stats. Because this is a passthrough, we only need a
+        // bare minimum expression context on mongos.
         invariant(targeter.policy ==
                       cluster_aggregation_planner::AggregationTargeter::kPassthrough ||
                   targeter.policy ==
                       cluster_aggregation_planner::AggregationTargeter::kSpecificShardOnly);
+
         expCtx = make_intrusive<ExpressionContext>(
             opCtx, nullptr, namespaces.executionNss, boost::none, request.getLet());
+        expCtx->addResolvedNamespaces(involvedNamespaces);
+
+
+        // We might need 'inMongos' temporarily set to true for query stats parsing, but we don't
+        // want to modify the value of 'expCtx' for future code execution so we will set it back to
+        // its original value.
+        ON_BLOCK_EXIT([&expCtx, originalInMongosVal = expCtx->inMongos]() {
+            expCtx->inMongos = originalInMongosVal;
+        });
+
+        // In order to parse a change stream request for query stats, 'inMongos' needs
+        // to be set to true.
+        if (hasChangeStream) {
+            expCtx->inMongos = true;
+        }
+
+        // Skip query stats recording for queryable encryption queries.
+        if (!shouldDoFLERewrite) {
+            // We want to hold off parsing the pipeline until it's clear we must. Because of that,
+            // we wait to parse the pipeline until this callback is invoked within
+            // query_stats::registerRequest.
+            query_stats::registerRequest(
+                opCtx,
+                namespaces.executionNss,
+                [&]() {
+                    auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+                    return std::make_unique<query_stats::AggKey>(
+                        request, *pipeline, expCtx, involvedNamespaces, namespaces.executionNss);
+                },
+                hasChangeStream);
+        }
     }
 
     if (request.getExplain()) {
         explain_common::generateServerInfo(result);
-        explain_common::generateServerParameters(result);
+        explain_common::generateServerParameters(opCtx, result);
     }
 
     auto status = [&]() {
@@ -443,13 +540,15 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kPassthrough: {
                 // A pipeline with $changeStream should never be allowed to passthrough.
                 invariant(!hasChangeStream);
+                const bool eligibleForSampling = !request.getExplain();
                 return cluster_aggregation_planner::runPipelineOnPrimaryShard(
                     expCtx,
                     namespaces,
-                    *targeter.cm,
+                    targeter.cri->cm,
                     request.getExplain(),
                     aggregation_request_helper::serializeToCommandDoc(request),
                     privileges,
+                    eligibleForSampling,
                     result);
             }
 
@@ -458,10 +557,11 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                 // If this is an explain write the explain output and return.
                 auto expCtx = targeter.pipeline->getContext();
                 if (expCtx->explain) {
+                    auto opts = SerializationOptions{.verbosity = boost::make_optional(
+                                                         ExplainOptions::Verbosity::kQueryPlanner)};
                     *result << "splitPipeline" << BSONNULL << "mongos"
                             << Document{{"host", getHostNameCachedAndPort()},
-                                        {"stages",
-                                         targeter.pipeline->writeExplainOps(*expCtx->explain)}};
+                                        {"stages", targeter.pipeline->writeExplainOps(opts)}};
                     return Status::OK();
                 }
 
@@ -475,6 +575,7 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
             }
 
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kAnyShard: {
+                const bool eligibleForSampling = !request.getExplain();
                 return cluster_aggregation_planner::dispatchPipelineAndMerge(
                     opCtx,
                     std::move(targeter),
@@ -485,7 +586,8 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                     privileges,
                     result,
                     hasChangeStream,
-                    startsWithDocuments);
+                    startsWithDocuments,
+                    eligibleForSampling);
             }
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::
                 kSpecificShardOnly: {
@@ -505,9 +607,14 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                         "passthrough from mongos",
                         nonChangeStreamLite.allowedToPassthroughFromMongos());
                 ShardId shardId(std::string(request.getPassthroughToShard()->getShard()));
+                // (Ignore FCV check): This is in mongos so we expect to ignore FCV.
                 uassert(6273803,
                         "$_passthroughToShard not supported for queries against config replica set",
-                        shardId != ShardId::kConfigServerId);
+                        shardId != ShardId::kConfigServerId ||
+                            gFeatureFlagCatalogShard.isEnabledAndIgnoreFCVUnsafe());
+                // This is an aggregation pipeline started internally, so it is not eligible for
+                // sampling.
+                const bool eligibleForSampling = false;
 
                 return cluster_aggregation_planner::runPipelineOnSpecificShardOnly(
                     expCtx,
@@ -517,7 +624,8 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
                     aggregation_request_helper::serializeToCommandDoc(request),
                     privileges,
                     shardId,
-                    true,
+                    true /* forPerShardCursor */,
+                    eligibleForSampling,
                     result);
             }
 
@@ -527,14 +635,18 @@ Status ClusterAggregate::runAggregate(OperationContext* opCtx,
     }();
 
     if (status.isOK()) {
-        updateHostsTargetedMetrics(opCtx, namespaces.executionNss, cm, involvedNamespaces);
+        updateHostsTargetedMetrics(opCtx,
+                                   namespaces.executionNss,
+                                   cri ? boost::make_optional(cri->cm) : boost::none,
+                                   involvedNamespaces);
         // Report usage statistics for each stage in the pipeline.
         liteParsedPipeline.tickGlobalStageCounters();
-
         // Add 'command' object to explain output.
         if (expCtx->explain) {
             explain_common::appendIfRoom(
                 aggregation_request_helper::serializeToCommandObj(request), "command", result);
+            collectQueryStatsMongos(opCtx,
+                                    std::move(CurOp::get(opCtx)->debug().queryStatsInfo.key));
         }
     }
     return status;
@@ -569,26 +681,28 @@ Status ClusterAggregate::retryOnViewError(OperationContext* opCtx,
     nsStruct.executionNss = resolvedView.getNamespace();
 
     // For a sharded time-series collection, the routing is based on both routing table and the
-    // granularity value. We need to make sure we use the granularity value of the same version as
-    // the routing table, instead of the one attached in the view error. This way the shard
-    // versioning check can correctly catch stale routing information.
-    boost::optional<ChunkManager> snapshotCm;
+    // bucketMaxSpanSeconds value. We need to make sure we use the bucketMaxSpanSeconds of the same
+    // version as the routing table, instead of the one attached in the view error. This way the
+    // shard versioning check can correctly catch stale routing information.
+    boost::optional<CollectionRoutingInfo> snapshotCri;
     if (nsStruct.executionNss.isTimeseriesBucketsCollection()) {
         auto executionNsRoutingInfoStatus =
             sharded_agg_helpers::getExecutionNsRoutingInfo(opCtx, nsStruct.executionNss);
         if (executionNsRoutingInfoStatus.isOK()) {
-            const auto& cm = executionNsRoutingInfoStatus.getValue();
-            if (cm.isSharded() && cm.getTimeseriesFields()) {
+            const auto& cri = executionNsRoutingInfoStatus.getValue();
+            if (cri.cm.isSharded() && cri.cm.getTimeseriesFields()) {
                 const auto patchedPipeline = rebuildPipelineWithTimeSeriesGranularity(
-                    resolvedAggRequest.getPipeline(), cm.getTimeseriesFields()->getGranularity());
+                    resolvedAggRequest.getPipeline(),
+                    cri.cm.getTimeseriesFields()->getGranularity(),
+                    cri.cm.getTimeseriesFields()->getBucketMaxSpanSeconds());
                 resolvedAggRequest.setPipeline(patchedPipeline);
-                snapshotCm = cm;
+                snapshotCri = cri;
             }
         }
     }
 
     auto status = ClusterAggregate::runAggregate(
-        opCtx, nsStruct, resolvedAggRequest, {resolvedAggRequest}, privileges, snapshotCm, result);
+        opCtx, nsStruct, resolvedAggRequest, {resolvedAggRequest}, privileges, snapshotCri, result);
 
     // If the underlying namespace was changed to a view during retry, then re-run the aggregation
     // on the new resolved namespace.
