@@ -52,7 +52,6 @@ DocumentSourceMergeCursors::DocumentSourceMergeCursors(
     : DocumentSource(kStageName, expCtx),
       _armParamsObj(std::move(ownedParamsSpec)),
       _armParams(std::move(armParams)) {
-    _armParams->setRecordRemoteOpWaitTime(true);
 
     // Populate the shard ids from the 'RemoteCursor'.
     recordRemoteCursorShardIds(_armParams->getRemotes());
@@ -83,7 +82,6 @@ bool DocumentSourceMergeCursors::remotesExhausted() const {
 void DocumentSourceMergeCursors::populateMerger() {
     invariant(!_blockingResultsMerger);
     invariant(_armParams);
-    invariant(_armParams->getRecordRemoteOpWaitTime());
 
     _blockingResultsMerger.emplace(
         pExpCtx->opCtx,
@@ -116,11 +114,10 @@ DocumentSource::GetNextResult DocumentSourceMergeCursors::doGetNext() {
     return Document::fromBsonWithMetaData(*next.getResult());
 }
 
-Value DocumentSourceMergeCursors::serialize(
-    boost::optional<ExplainOptions::Verbosity> explain) const {
+Value DocumentSourceMergeCursors::serialize(const SerializationOptions& opts) const {
     invariant(!_blockingResultsMerger);
     invariant(_armParams);
-    return Value(Document{{kStageName, _armParams->toBSON()}});
+    return Value(Document{{kStageName, _armParams->toBSON(opts)}});
 }
 
 boost::intrusive_ptr<DocumentSource> DocumentSourceMergeCursors::createFromBson(
@@ -129,7 +126,8 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceMergeCursors::createFromBson(
             "$mergeCursors stage expected an object as argument",
             elem.type() == BSONType::Object);
     auto ownedObj = elem.embeddedObject().getOwned();
-    auto armParams = AsyncResultsMergerParams::parse(IDLParserContext(kStageName), ownedObj);
+    auto armParams = AsyncResultsMergerParams::parse(
+        IDLParserContext(kStageName, false /*apiStrict*/, expCtx->ns.tenantId()), ownedObj);
     return new DocumentSourceMergeCursors(expCtx, std::move(armParams), std::move(ownedObj));
 }
 
