@@ -80,7 +80,7 @@ public:
     /**
      * Serializes this stage to be sent to perform the merging on a different host.
      */
-    Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
+    Value serialize(const SerializationOptions& opts = SerializationOptions{}) const final override;
 
     StageConstraints constraints(Pipeline::SplitState pipeState) const final {
         StageConstraints constraints(StreamType::kStreaming,
@@ -155,6 +155,8 @@ protected:
     void doDispose() final;
 
 private:
+    friend class DocumentSourceMergeCursorsMultiTenancyTest;
+
     DocumentSourceMergeCursors(const boost::intrusive_ptr<ExpressionContext>&,
                                AsyncResultsMergerParams,
                                boost::optional<BSONObj> ownedParamsSpec = boost::none);
@@ -170,6 +172,17 @@ private:
      */
     void recordRemoteCursorShardIds(const std::vector<RemoteCursor>& remoteCursors);
 
+    /**
+     * Get the Async Results Merger Params or return boost:none if it is not set. This is used by
+     * DocumentSourceMergeCursorsMultiTenancyTest for unit test purpose.
+     */
+    boost::optional<NamespaceString> getAsyncResultMergerParamsNss_forTest() const {
+        if (_armParams) {
+            return _armParams->getNss();
+        }
+        return boost::none;
+    }
+
     // When we have parsed the params out of a BSONObj, the object needs to stay around while the
     // params are in use. We store them here.
     boost::optional<BSONObj> _armParamsObj;
