@@ -45,6 +45,7 @@
 #include "mongo/db/service_context.h"
 #include "mongo/db/session/session_catalog.h"
 #include "mongo/db/stats/counters.h"
+#include "mongo/db/stats/top.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/check_allowed_op_query_cmd.h"
 #include "mongo/rpc/message.h"
@@ -164,9 +165,23 @@ Future<DbResponse> HandleRequest::handleRequest() {
 
 void HandleRequest::onSuccess(const DbResponse& dbResponse) {
     auto opCtx = rec->getOpCtx();
+    const auto currentOp = CurOp::get(opCtx);
+
     // Mark the op as complete, populate the response length, and log it if appropriate.
-    CurOp::get(opCtx)->completeAndLogOperation(
-        opCtx, logv2::LogComponent::kCommand, dbResponse.response.size(), slowMsOverride);
+    currentOp->completeAndLogOperation(
+        logv2::LogComponent::kCommand,
+        CollectionCatalog::get(opCtx)
+            ->getDatabaseProfileSettings(currentOp->getNSS().dbName())
+            .filter,
+        dbResponse.response.size(),
+        slowMsOverride);
+
+    // Update the source of stats shown in the db.serverStatus().opLatencies section.
+    Top::get(opCtx->getServiceContext())
+        .incrementGlobalLatencyStats(
+            opCtx,
+            durationCount<Microseconds>(currentOp->elapsedTimeExcludingPauses()),
+            currentOp->getReadWriteType());
 }
 
 Future<DbResponse> HandleRequest::run() {
@@ -201,7 +216,7 @@ void ServiceEntryPointMongos::onClientConnect(Client* client) {
     }
 }
 
-void ServiceEntryPointMongos::derivedOnClientDisconnect(Client* client) {
+void ServiceEntryPointMongos::derivedOnClientDisconnect(Client* client) try {
     if (load_balancer_support::isFromLoadBalancer(client)) {
         _loadBalancedConnections.decrement();
 
@@ -241,6 +256,11 @@ void ServiceEntryPointMongos::derivedOnClientDisconnect(Client* client) {
                  "aborting in-progress transaction because load-balanced client disconnected"});
         }
     }
+} catch (const DBException& ex) {
+    LOGV2_DEBUG(8969800,
+                2,
+                "Encountered error while performing client connection cleanup",
+                "error"_attr = ex.toStatus());
 }
 
 void ServiceEntryPointMongos::appendStats(BSONObjBuilder* bob) const {
