@@ -17,6 +17,10 @@ DBCollection.prototype.compact = function() {
     return this._db.getMongo().compact(this._fullName);
 };
 
+DBCollection.prototype.cleanup = function() {
+    return this._db.getMongo().cleanup(this._fullName);
+};
+
 DBCollection.prototype.verify = function() {
     assert(this._fullName, "no fullName");
     assert(this._shortName, "no shortName");
@@ -39,6 +43,9 @@ DBCollection.prototype.help = function() {
     print(
         "\tdb." + shortName +
         ".bulkWrite( operations, <optional params> ) - bulk execute write operations, optional parameters are: w, wtimeout, j");
+    print(
+        "\tdb." + shortName +
+        ".checkMetadataConsistency() - return metadata inconsistency information found in the collection.");
     print(
         "\tdb." + shortName +
         ".count( query = {}, <optional params> ) - count the number of documents that matches the query, optional parameters are: limit, skip, hint, maxTimeMS");
@@ -244,8 +251,7 @@ DBCollection.prototype.find = function(filter, projection, limit, skip, batchSiz
 
         const client = session._getSessionAwareClient();
         const readConcern = client.getReadConcern(session);
-        if (readConcern !== null &&
-            client.canUseReadConcern(session, cursor._convertToCommand(true))) {
+        if (readConcern !== null && client.canUseReadConcern(session, cursor._convertToCommand())) {
             cursor.readConcern(readConcern.level);
         }
     }
@@ -534,8 +540,8 @@ DBCollection.prototype.save = function(obj, opts) {
     if (obj == null)
         throw Error("can't save a null");
 
-    if (typeof (obj) == "number" || typeof (obj) == "string")
-        throw Error("can't save a number or string");
+    if (typeof (obj) == "number" || typeof (obj) == "string" || Array.isArray(obj))
+        throw Error("can't save a number, a string or an array");
 
     if (typeof (obj._id) == "undefined") {
         obj._id = new ObjectId();
@@ -1471,6 +1477,14 @@ DBCollection.prototype.watch = function(pipeline, options) {
     return this.aggregate(pipeline, aggOptions);
 };
 
+DBCollection.prototype.checkMetadataConsistency = function(options = {}) {
+    assert(options instanceof Object,
+           `'options' parameter expected to be type object but found: ${typeof options}`);
+    const res = assert.commandWorked(
+        this._db.runCommand(Object.extend({checkMetadataConsistency: this.getName()}, options)));
+    return new DBCommandCursor(this._db, res);
+};
+
 /**
  * PlanCache
  * Holds a reference to the collection.
