@@ -58,7 +58,6 @@
 #include "mongo/scripting/mozjs/objectwrapper.h"
 #include "mongo/scripting/mozjs/valuereader.h"
 #include "mongo/scripting/mozjs/valuewriter.h"
-#include "mongo/shell/encrypted_shell_options.h"
 #include "mongo/shell/kms.h"
 #include "mongo/shell/kms_gen.h"
 #include "mongo/shell/shell_options.h"
@@ -66,11 +65,10 @@
 
 namespace mongo {
 
-EncryptedShellGlobalParams encryptedShellGlobalParams;
-
 namespace {
 constexpr Duration kCacheInvalidationTime = Minutes(1);
-
+constexpr StringData compactCmdName = "compact"_sd;
+constexpr StringData cleanupCmdName = "cleanup"_sd;
 
 ImplicitEncryptedDBClientCallback* implicitEncryptedDBClientCallback{nullptr};
 
@@ -100,6 +98,7 @@ EncryptedDBClientBase::EncryptedDBClientBase(std::unique_ptr<DBClientBase> conn,
     : _conn(std::move(conn)), _encryptionOptions(std::move(encryptionOptions)), _cx(cx) {
     validateCollection(cx, collection);
     _collection = JS::Heap<JS::Value>(collection);
+    _conn->setAlwaysAppendDollarTenant_forTest();
 };
 
 std::string EncryptedDBClientBase::getServerAddress() const {
@@ -116,7 +115,7 @@ void EncryptedDBClientBase::say(Message& toSend, bool isRetry, std::string* actu
 
 BSONObj EncryptedDBClientBase::encryptDecryptCommand(const BSONObj& object,
                                                      bool encrypt,
-                                                     const StringData databaseName) {
+                                                     const DatabaseName& dbName) {
     std::stack<std::pair<BSONObjIterator, BSONObjBuilder>> frameStack;
 
     // The encryptDecryptCommand frameStack requires a guard because  if encryptMarking or
@@ -167,7 +166,12 @@ BSONObj EncryptedDBClientBase::encryptDecryptCommand(const BSONObj& object,
         }
     }
     invariant(frameStack.size() == 1);
-    frameStack.top().second.append("$db", databaseName);
+    // Append '$db' which shouldn't contain tenantid.
+    frameStack.top().second.append("$db", dbName.toString());
+    // If encrypt request, append '$tenant' which contains tenantid.
+    if (encrypt && dbName.tenantId() && !object.hasField("$tenant")) {
+        dbName.tenantId()->serializeToBSON("$tenant", &frameStack.top().second);
+    }
     return frameStack.top().second.obj();
 }
 
@@ -197,23 +201,19 @@ void EncryptedDBClientBase::decryptPayload(ConstDataRange data,
 }
 
 EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::processResponseFLE1(
-    EncryptedDBClientBase::RunCommandReturn result, const StringData databaseName) {
+    EncryptedDBClientBase::RunCommandReturn result, const DatabaseName& dbName) {
     auto rawReply = result.returnReply->getCommandReply();
-    return prepareReply(
-        std::move(result), databaseName, encryptDecryptCommand(rawReply, false, databaseName));
+    return prepareReply(std::move(result), encryptDecryptCommand(rawReply, false, dbName));
 }
 
 EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::processResponseFLE2(
-    EncryptedDBClientBase::RunCommandReturn result, const StringData databaseName) {
+    EncryptedDBClientBase::RunCommandReturn result) {
     auto rawReply = result.returnReply->getCommandReply();
-    return prepareReply(
-        std::move(result), databaseName, FLEClientCrypto::decryptDocument(rawReply, this));
+    return prepareReply(std::move(result), FLEClientCrypto::decryptDocument(rawReply, this));
 }
 
 EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::prepareReply(
-    EncryptedDBClientBase::RunCommandReturn result,
-    const StringData databaseName,
-    BSONObj decryptedDoc) {
+    EncryptedDBClientBase::RunCommandReturn result, BSONObj decryptedDoc) {
     rpc::OpMsgReplyBuilder replyBuilder;
     replyBuilder.setCommandReply(StatusWith<BSONObj>(decryptedDoc));
     auto msg = replyBuilder.done();
@@ -237,8 +237,11 @@ EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::doRunCommand(
 
 EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::handleEncryptionRequest(
     EncryptedDBClientBase::RunCommandParams params) {
-    auto commandName = params.request.getCommandName().toString();
-    auto databaseName = params.request.getDatabase().toString();
+    auto& request = params.request;
+    auto commandName = request.getCommandName().toString();
+    const DatabaseName dbName = request.body.hasField("$tenant")
+        ? DatabaseName(TenantId(request.body["$tenant"].OID()), request.getDatabase())
+        : DatabaseName(boost::none, request.getDatabase());
 
     if (std::find(kEncryptedCommands.begin(), kEncryptedCommands.end(), StringData(commandName)) ==
         std::end(kEncryptedCommands)) {
@@ -246,7 +249,7 @@ EncryptedDBClientBase::RunCommandReturn EncryptedDBClientBase::handleEncryptionR
     }
 
     EncryptedDBClientBase::RunCommandReturn result(doRunCommand(std::move(params)));
-    return processResponseFLE1(processResponseFLE2(std::move(result), databaseName), databaseName);
+    return processResponseFLE1(processResponseFLE2(std::move(result)), dbName);
 }
 
 std::pair<rpc::UniqueReply, DBClientBase*> EncryptedDBClientBase::runCommandWithTarget(
@@ -543,7 +546,7 @@ void EncryptedDBClientBase::decrypt(mozjs::MozJSImplScope* scope,
 
 boost::optional<EncryptedFieldConfig> EncryptedDBClientBase::getEncryptedFieldConfig(
     const NamespaceString& nss) {
-    auto collsList = _conn->getCollectionInfos(nss.db().toString(), BSON("name" << nss.coll()));
+    auto collsList = _conn->getCollectionInfos(nss.dbName(), BSON("name" << nss.coll()));
     uassert(ErrorCodes::BadValue,
             str::stream() << "Namespace not found: " << nss.toString(),
             !collsList.empty());
@@ -559,30 +562,56 @@ boost::optional<EncryptedFieldConfig> EncryptedDBClientBase::getEncryptedFieldCo
     return EncryptedFieldConfig::parse(IDLParserContext("encryptedFields"), efc.Obj());
 }
 
-void EncryptedDBClientBase::compact(JSContext* cx, JS::CallArgs args) {
+NamespaceString validateStructuredEncryptionParams(JSContext* cx,
+                                                   JS::CallArgs args,
+                                                   StringData cmdName) {
     if (args.length() != 1) {
-        uasserted(ErrorCodes::BadValue, "compact requires 1 arg");
+        uasserted(ErrorCodes::BadValue, str::stream() << cmdName << " requires 1 arg");
     }
     if (!args.get(0).isString()) {
-        uasserted(ErrorCodes::BadValue, "1st param to compact has to be a string");
+        uasserted(ErrorCodes::BadValue,
+                  str::stream() << "1st param to " << cmdName << " has to be a string");
     }
     std::string fullName = mozjs::ValueWriter(cx, args.get(0)).toString();
     NamespaceString nss(fullName);
+
     uassert(
         ErrorCodes::BadValue, str::stream() << "Invalid namespace: " << fullName, nss.isValid());
 
-    auto efc = getEncryptedFieldConfig(nss);
+    return nss;
+}
+
+void EncryptedDBClientBase::compact(JSContext* cx, JS::CallArgs args) {
+    auto nss = validateStructuredEncryptionParams(cx, args, compactCmdName);
+
     BSONObjBuilder builder;
+    auto efc = getEncryptedFieldConfig(nss);
+
     builder.append("compactStructuredEncryptionData", nss.coll());
     builder.append("compactionTokens",
                    efc ? FLEClientCrypto::generateCompactionTokens(*efc, this) : BSONObj());
 
     BSONObj reply;
-    runCommand(nss.db().toString(), builder.obj(), reply, 0);
+    runCommand(nss.dbName(), builder.obj(), reply, 0);
     reply = reply.getOwned();
     mozjs::ValueReader(cx, args.rval()).fromBSON(reply, nullptr, false);
 }
 
+void EncryptedDBClientBase::cleanup(JSContext* cx, JS::CallArgs args) {
+    auto nss = validateStructuredEncryptionParams(cx, args, cleanupCmdName);
+
+    BSONObjBuilder builder;
+    auto efc = getEncryptedFieldConfig(nss);
+
+    builder.append("cleanupStructuredEncryptionData", nss.coll());
+    builder.append("cleanupTokens",
+                   efc ? FLEClientCrypto::generateCompactionTokens(*efc, this) : BSONObj());
+
+    // TODO SERVER-72937: Add call to cleanup function
+    mozjs::ValueReader(cx, args.rval()).fromBSON(BSONObj(), nullptr, false);
+    return;
+}
+
 void EncryptedDBClientBase::trace(JSTracer* trc) {
     JS::TraceEdge(trc, &_collection, "collection object");
 }
@@ -670,7 +699,8 @@ std::vector<uint8_t> EncryptedDBClientBase::getBinDataArg(
             str::stream() << "Incorrect bindata type, expected" << typeName(type) << " but got "
                           << typeName(binType),
             binType == type);
-    auto str = static_cast<std::string*>(JS::GetPrivate(args.get(index).toObjectOrNull()));
+    auto str = JS::GetMaybePtrFromReservedSlot<std::string>(args.get(index).toObjectOrNull(),
+                                                            mozjs::BinDataInfo::BinDataStringSlot);
     uassert(ErrorCodes::BadValue, "Cannot call getter on BinData prototype", str);
     std::string string = base64::decode(*str);
     return std::vector<uint8_t>(string.data(), string.data() + string.length());
@@ -696,7 +726,7 @@ DBClientBase* EncryptedDBClientBase::getRawConnection() {
     return _conn.get();
 }
 
-SecureVector<uint8_t> EncryptedDBClientBase::getKeyMaterialFromDisk(const UUID& uuid) {
+BSONObj EncryptedDBClientBase::getEncryptedKey(const UUID& uuid) {
     NamespaceString fullNameNS = getCollectionNS();
     FindCommandRequest findCmd{fullNameNS};
     findCmd.setFilter(BSON("_id" << uuid));
@@ -723,6 +753,15 @@ SecureVector<uint8_t> EncryptedDBClientBase::getKeyMaterialFromDisk(const UUID&
     auto dataKey = keyStoreRecord.getKeyMaterial();
     uassert(ErrorCodes::BadValue, "Invalid data key.", dataKey.length() != 0);
 
+    return keyStoreRecord.toBSON();
+}
+
+SecureVector<uint8_t> EncryptedDBClientBase::getKeyMaterialFromDisk(const UUID& uuid) {
+    auto rawKey = getEncryptedKey(uuid);
+    auto keyStoreRecord = KeyStoreRecord::parse(IDLParserContext("root"), rawKey);
+
+    auto dataKey = keyStoreRecord.getKeyMaterial();
+
     std::unique_ptr<KMSService> kmsService = KMSServiceController::createFromDisk(
         _encryptionOptions.getKmsProviders().toBSON(), keyStoreRecord.getMasterKey());
     SecureVector<uint8_t> decryptedKey =
@@ -745,6 +784,18 @@ KeyMaterial EncryptedDBClientBase::getKey(const UUID& uuid) {
     return km;
 }
 
+SymmetricKey& EncryptedDBClientBase::getKMSLocalKey() {
+    if (!_localKey.has_value()) {
+        std::unique_ptr<KMSService> kmsService =
+            KMSServiceController::createFromDisk(_encryptionOptions.getKmsProviders().toBSON(),
+                                                 BSON("provider"
+                                                      << "local"));
+        _localKey = std::move(kmsService->getMasterKey());
+    }
+
+    return _localKey.get();
+}
+
 #ifdef MONGO_CONFIG_SSL
 const SSLConfiguration* EncryptedDBClientBase::getSSLConfiguration() {
     return _conn->getSSLConfiguration();
@@ -807,7 +858,7 @@ std::unique_ptr<DBClientBase> createEncryptedDBClientBase(std::unique_ptr<DBClie
 
     static constexpr auto keyVaultClientFieldId = "keyVaultClient";
 
-    if (!arg.isObject() && encryptedShellGlobalParams.awsAccessKeyId.empty()) {
+    if (!arg.isObject()) {
         return conn;
     }
 
@@ -815,32 +866,7 @@ std::unique_ptr<DBClientBase> createEncryptedDBClientBase(std::unique_ptr<DBClie
     JS::RootedValue client(cx);
     JS::RootedValue collection(cx);
 
-    if (!arg.isObject()) {
-        // If arg is not an object, but one of the required encryptedShellGlobalParams
-        // is defined, the user is trying to start an encrypted client with command line
-        // parameters.
-
-        AwsKMS awsKms = AwsKMS(encryptedShellGlobalParams.awsAccessKeyId,
-                               encryptedShellGlobalParams.awsSecretAccessKey);
-
-        awsKms.setUrl(StringData(encryptedShellGlobalParams.awsKmsURL));
-
-        awsKms.setSessionToken(StringData(encryptedShellGlobalParams.awsSessionToken));
-
-        KmsProviders kmsProviders;
-        kmsProviders.setAws(awsKms);
-
-        // The mongoConnection object will never be null.
-        // If the encrypted shell is started through command line parameters, then the user must
-        // default to the implicit connection for the keyvault collection.
-        client.setObjectOrNull(mongoConnection.get());
-
-        // Because we cannot add a schemaMap object through the command line, we set the
-        // schemaMap object in ClientSideFLEOptions to be null so we know to always use
-        // remote schemas.
-        encryptionOptions = ClientSideFLEOptions(encryptedShellGlobalParams.keyVaultNamespace,
-                                                 std::move(kmsProviders));
-    } else {
+    {
         uassert(ErrorCodes::BadValue,
                 "Collection object must be passed to Field Level Encryption Options",
                 arg.isObject());
