@@ -31,9 +31,9 @@
 #include "mongo/platform/basic.h"
 
 #include <boost/optional/optional.hpp>
+#include <cstdlib>
 #include <fstream>
 #include <memory>
-#include <stdlib.h>
 
 #include "mongo/base/checked_cast.h"
 #include "mongo/base/init.h"
@@ -41,6 +41,7 @@
 #include "mongo/base/status_with.h"
 #include "mongo/crypto/sha1_block.h"
 #include "mongo/crypto/sha256_block.h"
+#include "mongo/db/connection_health_metrics_parameter_gen.h"
 #include "mongo/logv2/log.h"
 #include "mongo/platform/random.h"
 #include "mongo/transport/ssl_connection_context.h"
@@ -1371,20 +1372,15 @@ SSLManagerApple::SSLManagerApple(const SSLParams& params, bool isServer)
         }
     }
 
+    // If the user has specified --setParameter tlsUseSystemCA=true, then no params.sslCAFile nor
+    // params.sslClusterCAFile will be defined, and the SSL Manager will fall back to the System CA.
     if (!params.sslCAFile.empty()) {
         auto ca = uassertStatusOK(loadPEM(params.sslCAFile, "", kLoadPEMStripKeys));
         _clientCA = std::move(ca);
-        _sslConfiguration.hasCA = _clientCA && ::CFArrayGetCount(_clientCA.get());
-    }
-
-    if (!params.sslCertificateSelector.empty() || !params.sslClusterCertificateSelector.empty()) {
-        // By using the system keychain, we acknowledge it exists.
-        _sslConfiguration.hasCA = true;
     }
 
     if (!_clientCA) {
-        // No explicit CA was specified, use the Keychain CA explicitly on client connects,
-        // even though we're going to pretend it doesn't exist on server.
+        // No explicit CA was specified, use the Keychain CA explicitly
         ::CFArrayRef certs = nullptr;
         uassertOSStatusOK(SecTrustCopyAnchorCertificates(&certs));
         _clientCA.reset(certs);
@@ -1559,17 +1555,6 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
 
     recordTLSVersion(tlsVersionStatus.getValue(), hostForLogging);
 
-    /* While we always have a system CA via the Keychain,
-     * we'll pretend not to in terms of validation if the server
-     * was started using a PEM file (legacy mode).
-     *
-     * When a certificate selector is used, we'll override hasCA to true
-     * so that the validation path runs anyway.
-     */
-    if (!_sslConfiguration.hasCA && isSSLServer) {
-        return Future<SSLPeerInfo>::makeReady(SSLPeerInfo(sniName));
-    }
-
     const auto badCert = [&](StringData msg, bool warn = false) -> Future<SSLPeerInfo> {
         if (warn) {
             LOGV2_WARNING(23209,
@@ -1594,7 +1579,7 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
             return SSLPeerInfo(sniName);
         } else {
             if (status == ::errSecSuccess) {
-                return badCert(str::stream() << "no SSL certificate provided by peer: "
+                return badCert(str::stream() << "No SSL certificate provided by peer: "
                                              << stringFromOSStatus(status),
                                _weakValidation);
             } else {
@@ -1675,11 +1660,12 @@ Future<SSLPeerInfo> SSLManagerApple::parseAndValidatePeerCertificate(
     // The cipher will be presented as a number.
     ::SSLCipherSuite cipher;
     uassertOSStatusOK(::SSLGetNegotiatedCipher(ssl, &cipher));
-
-    LOGV2_INFO(6723803,
-               "Accepted TLS connection from peer",
-               "peerSubjectName"_attr = peerSubjectName,
-               "cipher"_attr = cipher);
+    if (!serverGlobalParams.quiet.load() && gEnableDetailedConnectionHealthMetricLogLines) {
+        LOGV2_INFO(6723803,
+                   "Accepted TLS connection from peer",
+                   "peerSubjectName"_attr = peerSubjectName,
+                   "cipher"_attr = cipher);
+    }
 
     // Server side.
     if (remoteHost.empty()) {
