@@ -33,8 +33,8 @@
 #include "mongo/config.h"
 #include "mongo/platform/basic.h"
 
+#include "mongo/transport/asio/asio_transport_layer.h"
 #include "mongo/transport/service_entry_point.h"
-#include "mongo/transport/transport_layer_asio.h"
 #include "mongo/transport/transport_layer_manager.h"
 #include "mongo/util/net/ssl/context.hpp"
 #include "mongo/util/net/ssl_manager.h"
@@ -42,6 +42,7 @@
 
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/net/ssl_types.h"
 
 #if MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
 #include "mongo/util/net/dh_openssl.h"
@@ -57,7 +58,7 @@ namespace {
 // Test implementation needed by ASIO transport.
 class ServiceEntryPointUtil : public ServiceEntryPoint {
 public:
-    void startSession(transport::SessionHandle session) override {
+    void startSession(std::shared_ptr<transport::Session> session) override {
         stdx::unique_lock<Latch> lk(_mutex);
         _sessions.push_back(std::move(session));
         LOGV2(2303202, "started session");
@@ -66,7 +67,7 @@ public:
 
     void endAllSessions(transport::Session::TagMask tags) override {
         LOGV2(2303302, "end all sessions");
-        std::vector<transport::SessionHandle> old_sessions;
+        std::vector<std::shared_ptr<transport::Session>> old_sessions;
         {
             stdx::unique_lock<Latch> lock(_mutex);
             old_sessions.swap(_sessions);
@@ -94,6 +95,10 @@ public:
         MONGO_UNREACHABLE;
     }
 
+    logv2::LogSeverity slowSessionWorkflowLogSeverity() override {
+        MONGO_UNIMPLEMENTED;
+    }
+
     void setTransportLayer(transport::TransportLayer* tl) {
         _transport = tl;
     }
@@ -106,7 +111,7 @@ public:
 private:
     mutable Mutex _mutex = MONGO_MAKE_LATCH("::_mutex");
     stdx::condition_variable _cv;
-    std::vector<transport::SessionHandle> _sessions;
+    std::vector<std::shared_ptr<transport::Session>> _sessions;
     transport::TransportLayer* _transport = nullptr;
 };
 
@@ -420,6 +425,105 @@ FlattenedX509Name flattenX509Name(const SSLX509Name& name) {
     return ret;
 }
 
+TEST(SSLManager, FilterClusterDN) {
+    static const stdx::unordered_set<std::string> defaultMatchingAttributes = {
+        "0.9.2342.19200300.100.1.25",  // DC
+        "2.5.4.10",                    // O
+        "2.5.4.11",                    // OU
+    };
+    std::vector<std::pair<std::string, std::string>> tests = {
+        // Single-valued RDNs.
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,L=New York City,ST=New York,C=US",
+         "OU=Kernel,O=MongoDB,DC=example"},
+        // Multi-valued RDN.
+        {"CN=server+OU=Kernel,O=MongoDB,L=New York City,ST=New York,C=US", "OU=Kernel,O=MongoDB"},
+        // Multiple DC attributes.
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,DC=net,L=New York City,ST=New York,C=US",
+         "OU=Kernel,O=MongoDB,DC=example,DC=net"},
+    };
+
+    for (const auto& test : tests) {
+        LOGV2(7498900, "Testing DN: ", "test_first"_attr = test.first);
+        auto swUnfilteredDN = parseDN(test.first);
+        auto swExpectedFilteredDN = parseDN(test.second);
+
+        ASSERT_OK(swUnfilteredDN.getStatus());
+        ASSERT_OK(swExpectedFilteredDN.getStatus());
+        ASSERT_OK(swUnfilteredDN.getValue().normalizeStrings());
+        ASSERT_OK(swExpectedFilteredDN.getValue().normalizeStrings());
+
+        auto actualFilteredDN =
+            filterClusterDN(swUnfilteredDN.getValue(), defaultMatchingAttributes);
+        ASSERT_TRUE(actualFilteredDN == swExpectedFilteredDN.getValue());
+    }
+};
+
+TEST(SSLManager, DNContains) {
+    // Checks if the second RDN is contained by the first (order does not matter).
+    // The bool is the expected value.
+    std::vector<std::tuple<std::string, std::string, bool>> tests = {
+        // Single-valued RDNs positive case.
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,L=New York City,ST=New York,C=US",
+         "CN=server,L=New York City,ST=New York,C=US",
+         true},
+        // Single-valued RDNs mismatched value.
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,L=New York City,ST=New York,C=US",
+         "CN=server,L=Yonkers,ST=New York,C=US",
+         false},
+        // Single-valued RDNs missing attribute.
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,ST=New York,C=US",
+         "CN=server,L=Yonkers,ST=New York,C=US",
+         false},
+        // Multi-valued RDN negative case (attribute value mismatch).
+        {"CN=server,OU=Kernel,O=MongoDB,L=New York City+ST=New York,C=US",
+         "CN=server,L=Yonkers+ST=New York",
+         false},
+        // Multi-valued RDN negative case (matching attributes in single-value RDNs, first RDN needs
+        // to be filtered beforehand).
+        {"CN=server,OU=Kernel,O=MongoDB,L=New York City+ST=New York,C=US",
+         "CN=server,L=New York City",
+         false},
+        // Multi-valued RDN negative case (input DN has attributes in single-value RDNs while match
+        // expects multi-valued RDN).
+        {"CN=server,OU=Kernel,O=MongoDB,L=New York City,ST=New York,C=US",
+         "CN=server,L=New York City+ST=New York",
+         false},
+        // Multi-valued RDN positive case (full multi-valued RDN present in second).
+        {"CN=server,OU=Kernel,O=MongoDB,L=New York City+ST=New York,C=US",
+         "CN=server,L=New York City+ST=New York",
+         true},
+        // Multiple attributes positive case (order should not matter).
+        {"CN=server,OU=Kernel,O=MongoDB,DC=net,DC=example,L=New York City,ST=New York,C=US",
+         "OU=Kernel,O=MongoDB,DC=example,DC=net",
+         true},
+        // Multiple attributes positive case (missing in second, but should not matter).
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,DC=net,L=New York City,ST=New York,C=US",
+         "OU=Kernel,O=MongoDB,DC=example",
+         true},
+        // Multiple attributes negative case (missing in first).
+        {"CN=server,OU=Kernel,O=MongoDB,DC=example,L=New York City,ST=New York,C=US",
+         "OU=Kernel,O=MongoDB,DC=example,DC=net",
+         false},
+    };
+
+    for (const auto& test : tests) {
+        LOGV2(7498901, "Testing DN: ", "test_first"_attr = std::get<0>(test));
+        auto swExternalDN = parseDN(std::get<0>(test));
+        auto swMatchPatternDN = parseDN(std::get<1>(test));
+
+        ASSERT_OK(swExternalDN.getStatus());
+        ASSERT_OK(swMatchPatternDN.getStatus());
+
+        auto externalDN = swExternalDN.getValue();
+        auto matchPatternDN = swMatchPatternDN.getValue();
+
+        ASSERT_OK(externalDN.normalizeStrings());
+        ASSERT_OK(matchPatternDN.normalizeStrings());
+
+        ASSERT_EQ(externalDN.contains(matchPatternDN), std::get<2>(test));
+    }
+};
+
 TEST(SSLManager, DNParsingAndNormalization) {
     std::vector<std::pair<std::string, FlattenedX509Name>> tests = {
         // Basic DN parsing
@@ -503,10 +607,10 @@ TEST(SSLManager, RotateCertificatesFromFile) {
     auto options = [] {
         ServerGlobalParams params;
         params.noUnixSocket = true;
-        transport::TransportLayerASIO::Options opts(&params);
+        transport::AsioTransportLayer::Options opts(&params);
         return opts;
     }();
-    transport::TransportLayerASIO tla(options, &sepu);
+    transport::AsioTransportLayer tla(options, &sepu);
     uassertStatusOK(tla.rotateCertificates(manager, false /* asyncOCSPStaple */));
 }
 
@@ -518,9 +622,12 @@ TEST(SSLManager, InitContextFromFileShouldFail) {
     params.sslCAFile = "jstests/libs/ca.pem";
     params.sslClusterFile = "jstests/libs/client.pem";
 #if MONGO_CONFIG_SSL_PROVIDER == MONGO_CONFIG_SSL_PROVIDER_OPENSSL
-    ASSERT_THROWS_CODE([&params] { SSLManagerInterface::create(params, true /* isSSLServer */); }(),
-                       DBException,
-                       ErrorCodes::InvalidSSLConfiguration);
+    ASSERT_THROWS_CODE(
+        [&params] {
+            SSLManagerInterface::create(params, true /* isSSLServer */);
+        }(),
+        DBException,
+        ErrorCodes::InvalidSSLConfiguration);
 #endif
 }
 
@@ -539,10 +646,10 @@ TEST(SSLManager, RotateClusterCertificatesFromFile) {
     auto options = [] {
         ServerGlobalParams params;
         params.noUnixSocket = true;
-        transport::TransportLayerASIO::Options opts(&params);
+        transport::AsioTransportLayer::Options opts(&params);
         return opts;
     }();
-    transport::TransportLayerASIO tla(options, &sepu);
+    transport::AsioTransportLayer tla(options, &sepu);
     uassertStatusOK(tla.rotateCertificates(manager, false /* asyncOCSPStaple */));
 }
 
@@ -607,10 +714,10 @@ TEST(SSLManager, TransientSSLParams) {
     auto options = [] {
         ServerGlobalParams params;
         params.noUnixSocket = true;
-        transport::TransportLayerASIO::Options opts(&params);
+        transport::AsioTransportLayer::Options opts(&params);
         return opts;
     }();
-    transport::TransportLayerASIO tla(options, &sepu);
+    transport::AsioTransportLayer tla(options, &sepu);
 
     TransientSSLParams transientSSLParams;
     transientSSLParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
@@ -640,10 +747,10 @@ TEST(SSLManager, TransientSSLParamsStressTestWithTransport) {
     auto options = [] {
         ServerGlobalParams params;
         params.noUnixSocket = true;
-        transport::TransportLayerASIO::Options opts(&params);
+        transport::AsioTransportLayer::Options opts(&params);
         return opts;
     }();
-    transport::TransportLayerASIO tla(options, &sepu);
+    transport::AsioTransportLayer tla(options, &sepu);
 
     TransientSSLParams transientSSLParams;
     transientSSLParams.sslClusterPEMPayload = loadFile("jstests/libs/client.pem");
@@ -781,6 +888,5 @@ TEST(SSLManager, InitContextNoSanWarning) {
     ASSERT_FALSE(isSanWarningWritten(getCapturedTextFormatLogMessages()));
 }
 
-
 }  // namespace
 }  // namespace mongo
